import { utils as utils$1 } from '@qubejs/core';
import 'core-js/modules/es.object.assign.js';
import 'core-js/modules/es.regexp.to-string.js';
import 'core-js/modules/es.promise.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.search.js';
import 'core-js/modules/es.string.match.js';
import 'core-js/modules/es.regexp.constructor.js';
import 'core-js/modules/es.string.replace.js';
import { __rest, __awaiter as __awaiter$1 } from 'tslib';
import 'core-js/modules/es.string.trim.js';
import filter$1 from 'lodash/filter';
import 'core-js/modules/es.array.sort.js';
import moment from 'moment-timezone';
import 'core-js/modules/es.string.starts-with.js';
import 'core-js/modules/es.array.iterator.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import 'core-js/modules/es.promise.finally.js';
import { createSlice, createAsyncThunk } from '@reduxjs/toolkit';
import 'core-js/modules/es.array.includes.js';
import 'core-js/modules/es.string.includes.js';
import 'core-js/modules/es.array.reduce.js';
import uniq from 'lodash/uniq';
import groupBy from 'lodash/groupBy';
import React$2, { useState, useEffect, createContext, memo, useLayoutEffect, useCallback, useMemo, isValidElement, cloneElement, useContext, Component as Component$1 } from 'react';
export { ThemeProvider } from '@emotion/react';
import { jsxs, jsx, Fragment } from 'react/jsx-runtime';
import { connect } from 'react-redux';
import { useParams, useLocation, Routes, Route } from 'react-router-dom';
import 'core-js/modules/es.symbol.description.js';
import TreeView from '@mui/lab/TreeView';
import ExpandMoreIcon from '@mui/icons-material/ExpandMore';
import ChevronRightIcon from '@mui/icons-material/ChevronRight';
import TreeItem from '@mui/lab/TreeItem';
import '@mui/material/List';
import '@mui/material/ListItem';
import '@mui/material/ListItemButton';
import '@mui/material/ListItemText';
import update from 'immutability-helper';
import sortBy from 'lodash/sortBy';
import Tooltip from '@mui/material/Tooltip';

const {
  QueryString
} = utils$1;
const location = window.location;
const query$1 = {
  get: search => {
    let browserSearch = '';
    if (location.href.indexOf('?') > -1) {
      browserSearch = location.href.substr(location.href.indexOf('?'));
    }
    return new QueryString(search || location.search || browserSearch).toObject();
  }
};

var queryString$1 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  query: query$1,
  QueryString: QueryString
});

let languages = {
  en: {
    messages: {}
  }
};
// window.getLangs = () => {
//   return languages;
// }
const localStorage$1 = window.localStorage;
let currentLanguage = localStorage$1.getItem('lang') || 'en';
const translate$5 = message => {
  // if ((languages[currentLanguage] &&
  //   !languages[currentLanguage].messages[message])) {
  //   languages[currentLanguage].messages[message] = message;
  // }
  return languages[currentLanguage] && languages[currentLanguage].messages[message] || languages.en.messages[message] || message;
};
const set$1 = lang => {
  if (languages[lang]) {
    currentLanguage = lang;
    localStorage$1 && localStorage$1.setItem('lang', lang);
  }
};
const get$2 = () => {
  return currentLanguage;
};
const loadLanguages = all => {
  languages = Object.assign(Object.assign({}, languages), all);
};
const updateLanguage = (lang, langName) => {
  languages[langName] = Object.assign(Object.assign({}, languages[langName]), {
    messages: Object.assign(Object.assign({}, languages[langName].messages), lang)
  });
};

var translate$6 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  translate: translate$5,
  set: set$1,
  get: get$2,
  loadLanguages: loadLanguages,
  updateLanguage: updateLanguage
});

let MESSAGES = {
  INVALID_CREDENTIALS: "Username or Password doesn't match.",
  INSUFFICIENT_PERMISSION: "You don't have sufficent permission to access this",
  NOT_LOGGED_IN: "You aren't logged in please login to access the application",
  DATA_INEGRITY: 'Unable to delete record, depent data found.',
  INVALID_LICENCE: 'Missing valid licence key',
  UNEXPECTED_ERROR: 'Oops something went wrong.',
  EMAIL_NOT_VERIFIED: 'Opps looks like your email is not verified. Please check your inbox and verify your email.'
};
const addMessages = newMessages => {
  MESSAGES = Object.assign(Object.assign({}, MESSAGES), newMessages);
};
const messages = {
  get: key => {
    return translate$5(MESSAGES[key]);
  }
};

var errorMessages = /*#__PURE__*/Object.freeze({
  __proto__: null,
  messages: messages,
  addMessages: addMessages
});

class EventManager {
  constructor() {
    this.events = {};
  }
  subscribe(evtName, fn) {
    if (!this.events[evtName]) {
      this.events[evtName] = [];
    }
    this.events[evtName].push(fn);
  }
  subscribeOnce(evtName, fn) {
    if (!this.events[evtName]) {
      this.events[evtName] = [];
    }
    if (this.events[evtName].length === 0) {
      this.events[evtName].push(fn);
    }
  }
  unsubscribe(evtName, fn) {
    if (this.events[evtName] && fn) {
      const idx = this.events[evtName].indexOf(fn);
      if (idx > -1) {
        this.events[evtName].splice(idx, 1);
      }
    } else if (this.events[evtName]) {
      delete this.events[evtName];
    }
  }
  emit(evtName, ...params) {
    let output;
    if (this.events[evtName]) {
      this.events[evtName].forEach(fn => {
        if (output !== false) {
          output = fn.apply(this, params);
        }
      });
    }
    return output;
  }
}

const GlobalOptions = utils$1.GlobalOptions;
const CONSTANTS = {
  STATUS: {
    UNKNOWN: undefined,
    SUCCESS: 'success',
    ERROR: 'error',
    OK: 'ok'
  },
  contentType: {
    PAGE: 'PAGE',
    SITE_MAP: 'SITE_MAP'
  }
};
const GLOBAL_OPTIONS = {
  inputVariant: new GlobalOptions({
    outlined: 'Outlined',
    standard: 'Standard',
    filled: 'Filled',
    default: 'Default'
  }),
  contentType: new GlobalOptions({
    PAGE: {
      text: 'Page',
      size: 'small'
    },
    SITE_MAP: {
      text: 'Site Map',
      size: 'small'
    }
  }),
  pricingTemplates: new GlobalOptions({
    default: 'default',
    'box-style': 'box-style'
  }),
  featureContentTemplates: new GlobalOptions({
    default: 'Default',
    'with-bg': 'With Background'
  }),
  editorStyle: new GlobalOptions({
    full: 'full',
    compact: 'compact',
    default: 'default',
    basic: 'basic'
  }),
  datePickerInstanceType: new GlobalOptions({
    DatePicker: 'DatePicker',
    TimePicker: 'TimePicker',
    DateTimePicker: 'DateTimePicker',
    DesktopDatePicker: 'DesktopDatePicker',
    MobileDatePicker: 'MobileDatePicker'
  }),
  userTabs: new GlobalOptions({
    ALL_USERS: 'All Users',
    ACTIVE_USERS: 'Active Users',
    INACTIVE_USERS: 'Inactive Users'
  }),
  mediaStatus: new GlobalOptions({
    DRAFT: 'Draft',
    PUBLISHED: 'Published'
  }),
  dislcaimerStyles: new GlobalOptions({
    'sq-disclaimer--bg-gray': 'sq-disclaimer--bg-gray',
    'sq-disclaimer--bg-black': 'sq-disclaimer--bg-black',
    'sq-disclaimer--top-border': 'sq-disclaimer--top-border'
  }),
  imageStyles: new GlobalOptions({
    'sq-image--inline-auto': 'sq-image--inline-auto',
    'sq-image--medium': 'sq-image--medium',
    'sq-image--auto-full': 'sq-image--auto-full'
  }),
  emailStatus: new GlobalOptions({
    DRAFT: 'Draft',
    PUBLISHED: 'Published',
    DEPRICATED: 'Depricated',
    ARCHIVED: 'Archived'
  }),
  mediaGalleryTabs: new GlobalOptions({
    ALL_MEDIA: 'All Media',
    UPLOAD_MEDIA: 'Upload Media'
  }),
  roleTabs: new GlobalOptions({
    ALL_ROLES: 'All Roles'
  }),
  roleType: new GlobalOptions({
    INTERNAL: 'Internal',
    EXTERNAL: 'External'
  }),
  alerTypes: new GlobalOptions({
    info: 'Info',
    error: 'Error',
    success: 'Success',
    warning: 'Warning'
  }),
  permissionTypes: new GlobalOptions({
    PERMISSIONS: {
      text: 'Permissions',
      key: 'permission'
    },
    LEFT_NAVIGATION: {
      text: 'Left Navigation',
      key: 'navigation'
    },
    ASSIGN_USERS: {
      text: 'Assign Users'
    }
  }),
  noOfResultsDropdown: new GlobalOptions({
    25: 25,
    50: 50,
    100: 100
  }),
  progressTypes: new GlobalOptions({
    default: 'default',
    circle: 'circle',
    cube: 'cube',
    round: 'round',
    ripple: 'ripple'
  }),
  progressStyles: new GlobalOptions({
    'full-screen': 'full-screen'
  }),
  maskTypes: new GlobalOptions({
    phone: 'phone',
    number: 'number',
    currency: 'currency',
    percentage: 'percentage',
    appendText: 'appendText',
    titleCase: 'titleCase',
    upperCase: 'upperCase',
    lowerCase: 'lowerCase',
    appendViewOnlyText: 'appendViewOnlyText'
  }),
  compareOperators: new GlobalOptions({
    bool: 'bool type equals',
    '=': '= equals',
    '!=': '!= not equals'
  }),
  headerTags: new GlobalOptions({
    h1: 'H1',
    h2: 'H2',
    h3: 'H3',
    h4: 'H4',
    h5: 'H5',
    h6: 'H6'
  }),
  bodyContainers: new GlobalOptions({
    'body-container-small': 'body-container-small',
    'body-container-med': 'body-container-med',
    'body-container-lg': 'body-container-lg',
    'body-container-xl': 'body-container-xl'
  }),
  linkTypes: new GlobalOptions({
    Button: 'Button',
    LinkButton: 'LinkButton'
  }),
  variants: new GlobalOptions({
    contained: 'Contained',
    outlined: 'Outlined',
    default: 'Default'
  }),
  pageWrapperClasses: new GlobalOptions({
    'sq-content-page--full-width': 'sq-content-page--full-width',
    'sq-content-page--h-100': 'sq-content-page--h-100',
    'sq-content-page--header-footer-body': 'sq-content-page--header-footer-body'
  }),
  buttonSize: new GlobalOptions({
    small: 'small',
    medium: 'medium',
    large: 'large'
  }),
  colorTypes: new GlobalOptions({
    inherit: 'inherit',
    primary: 'primary',
    secondary: 'secondary',
    success: 'success',
    info: 'info',
    error: 'error',
    warning: 'warning'
  }),
  iconColorTypes: new GlobalOptions({
    inherit: 'inherit',
    primary: 'primary',
    secondary: 'secondary',
    success: 'success',
    info: 'info',
    error: 'error',
    warning: 'warning',
    blue: 'blue',
    'blue-dark': 'blue-dark',
    pink: 'pink',
    'pink-dark': 'pink-dark',
    green: 'green',
    'green-dark': 'green-dark',
    yellow: 'yellow',
    'yellow-dark': 'yellow-dark',
    purple: 'purple',
    'purple-dark': 'purple-dark',
    red: 'red',
    'red-dark': 'red-dark'
  }),
  radioDisplay: new GlobalOptions({
    Column: 'Column',
    Inline: 'Inline'
  }),
  headerStyles: new GlobalOptions({
    'sq-header--center': 'sq-header--center',
    'sq-header--left': 'sq-header--left',
    'mb-narrow': 'Margin bottom narrow',
    'mb-wide': 'Margin bottom wide',
    'mb-none': 'Margin bottom none'
  }),
  heroStyles: new GlobalOptions({
    'sq-hero-content--left-aligned': 'sq-hero-content--left-aligned',
    'sq-hero-content--right-aligned': 'sq-hero-content--right-aligned',
    'sq-hero-content--center-aligned': 'sq-hero-content--center-aligned',
    'sq-hero-content--large-background-theme-dark': 'sq-hero-content--large-background-theme-dark',
    'sq-hero-content--bg-black': 'sq-hero-content--bg-black',
    'sq-hero-content--bg-gray': 'sq-hero-content--bg-gray',
    'sq-hero-content--dark': 'sq-hero-content--dark',
    'sq-hero-content--full-width': 'sq-hero-content--full-width',
    'sq-hero-content--dark-secondary': 'sq-hero-content--dark-secondary'
  }),
  imageBlockWithTextStyles: new GlobalOptions({
    'bg-black': 'bg-black',
    'bg-gray': 'bg-gray'
  }),
  imageBlockWithTextCss: new GlobalOptions({
    'sq-image-block-with-text--left-aligned': 'sq-image-block-with-text--left-aligned'
  }),
  imageWithHeaderBody: new GlobalOptions({
    bordered: 'bordered',
    shadow: 'shadow'
  }),
  imageAlign: new GlobalOptions({
    left: 'left',
    right: 'right'
  }),
  heroTemplates: new GlobalOptions({
    'without-image': 'without-image',
    default: 'default',
    'with-background': 'with-background',
    'large-background': 'large-background'
  }),
  formStyles: new GlobalOptions({
    'sq-form--2-cols': '2 Columns',
    'sq-form--3-cols': '3 Columns',
    'sq-form--inline-auto': 'Inline Auto',
    'sq-form--narrow-space': 'Narrow Space',
    'pb-0': 'No Padding bottom'
  }),
  genericStyles: new GlobalOptions({
    'mb-wide': 'Margin bottom wide',
    'mt-wide': 'Margin top wide',
    'mr-wide': 'Margin right wide',
    'ml-wide': 'Margin left wide',
    'mb-jumbo': 'Margin bottom jumbo',
    'mt-jumbo': 'Margin top jumbo',
    'mb-none': 'Margin bottom none',
    'mt-none': 'Margin top none',
    'pb-wide': 'Padding bottom wide',
    'pt-wide': 'Padding top wide',
    'pl-wide': 'Padding left wide',
    'pr-wide': 'Padding right wide',
    'pb-0': 'Padding bottom none',
    'm-0': 'Margin none',
    'p-0': 'Padding none',
    'pt-none': 'Padding top none',
    'pb-jumbo': 'Padding bottom jumbo',
    'pt-jumbo': 'Padding top jumbo',
    'text-center': 'text-center'
  }),
  htmlTags: new GlobalOptions({
    span: 'span',
    div: 'div',
    h1: 'h1',
    h2: 'h2',
    h3: 'h3',
    h4: 'h4',
    h5: 'h5',
    p: 'p'
  }),
  actionTypes: new GlobalOptions({
    submit: 'submit',
    api: 'api',
    module: 'module',
    redirect: 'redirect'
  }),
  methodTypes: new GlobalOptions({
    get: 'GET',
    post: 'POST',
    patch: 'PATCH',
    update: 'UPDATE',
    delete: 'DELETE'
  }),
  salutation: new GlobalOptions({
    'Mr.': 'Mr.',
    'Mrs.': 'Mrs.',
    'Miss.': 'Miss'
  })
};

var globals = /*#__PURE__*/Object.freeze({
  __proto__: null,
  CONSTANTS: CONSTANTS,
  GLOBAL_OPTIONS: GLOBAL_OPTIONS,
  GlobalOptions: GlobalOptions
});

var __awaiter = (undefined && undefined.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
// --------- HELPERS
function getElementOffset(el) {
    let top = 0;
    let left = 0;
    let element = el;
    // Loop through the DOM tree
    // and add it's parent's offset to get page offset
    do {
        top += element.offsetTop || 0;
        left += element.offsetLeft || 0;
        element = element.offsetParent;
    } while (element);
    return {
        top,
        left,
    };
}
// --------- SCROLL INTERFACES
// ScrollDomElement and ScrollWindow have identical interfaces
class ScrollDomElement {
    constructor(element) {
        this.element = element;
    }
    getHorizontalScroll() {
        return this.element.scrollLeft;
    }
    getVerticalScroll() {
        return this.element.scrollTop;
    }
    getMaxHorizontalScroll() {
        return this.element.scrollWidth - this.element.clientWidth;
    }
    getMaxVerticalScroll() {
        return this.element.scrollHeight - this.element.clientHeight;
    }
    getHorizontalElementScrollOffset(elementToScrollTo, elementToScroll) {
        return (getElementOffset(elementToScrollTo).left -
            getElementOffset(elementToScroll).left);
    }
    getVerticalElementScrollOffset(elementToScrollTo, elementToScroll) {
        return (getElementOffset(elementToScrollTo).top -
            getElementOffset(elementToScroll).top);
    }
    scrollTo(x, y) {
        this.element.scrollLeft = x;
        this.element.scrollTop = y;
    }
}
class ScrollWindow {
    constructor() {
        this.element = window;
    }
    getHorizontalScroll() {
        return window.scrollX || document.documentElement.scrollLeft;
    }
    getVerticalScroll() {
        return window.scrollY || document.documentElement.scrollTop;
    }
    getMaxHorizontalScroll() {
        return (Math.max(document.body.scrollWidth, document.documentElement.scrollWidth, document.body.offsetWidth, document.documentElement.offsetWidth, document.body.clientWidth, document.documentElement.clientWidth) - window.innerWidth);
    }
    getMaxVerticalScroll() {
        return (Math.max(document.body.scrollHeight, document.documentElement.scrollHeight, document.body.offsetHeight, document.documentElement.offsetHeight, document.body.clientHeight, document.documentElement.clientHeight) - window.innerHeight);
    }
    getHorizontalElementScrollOffset(elementToScrollTo) {
        const scrollLeft = window.scrollX || document.documentElement.scrollLeft;
        return scrollLeft + elementToScrollTo.getBoundingClientRect().left;
    }
    getVerticalElementScrollOffset(elementToScrollTo) {
        const scrollTop = window.scrollY || document.documentElement.scrollTop;
        return scrollTop + elementToScrollTo.getBoundingClientRect().top;
    }
    scrollTo(x, y) {
        window.scrollTo(x, y);
    }
}
const activeAnimations = {
    elements: [],
    cancelMethods: [],
    add: (element, cancelAnimation) => {
        activeAnimations.elements.push(element);
        activeAnimations.cancelMethods.push(cancelAnimation);
    },
    remove: (element, shouldStop) => {
        const index = activeAnimations.elements.indexOf(element);
        if (index > -1) {
            // Stop animation
            if (shouldStop) {
                activeAnimations.cancelMethods[index]();
            }
            // Remove it
            activeAnimations.elements.splice(index, 1);
            activeAnimations.cancelMethods.splice(index, 1);
        }
    },
};
// --------- CHECK IF CODE IS RUNNING IN A BROWSER
const WINDOW_EXISTS = typeof window !== 'undefined';
// --------- ANIMATE SCROLL TO
const defaultOptions = {
    cancelOnUserAction: true,
    easing: (t) => --t * t * t + 1,
    elementToScroll: WINDOW_EXISTS ? window : null,
    horizontalOffset: 0,
    maxDuration: 3000,
    minDuration: 250,
    speed: 500,
    verticalOffset: 0,
};
function animateScrollTo(numberOrCoordsOrElement, userOptions = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        // Check for server rendering
        if (!WINDOW_EXISTS) {
            // @ts-ignore
            // If it still gets called on server, return Promise for API consistency
            return new Promise((resolve) => {
                resolve(false); // Returning false on server
            });
        }
        else if (!window.Promise) {
            throw "Browser doesn't support Promises, and animated-scroll-to depends on it, please provide a polyfill.";
        }
        let x;
        let y;
        let scrollToElement;
        let options = Object.assign(Object.assign({}, defaultOptions), userOptions);
        const isWindow = options.elementToScroll === window;
        const isElement = !!options.elementToScroll.nodeName;
        if (!isWindow && !isElement) {
            throw 'Element to scroll needs to be either window or DOM element.';
        }
        // Check for "scroll-behavior: smooth" as it can break the animation
        // https://github.com/Stanko/animated-scroll-to/issues/55
        const scrollBehaviorElement = isWindow
            ? document.documentElement
            : options.elementToScroll;
        const scrollBehavior = getComputedStyle(scrollBehaviorElement).getPropertyValue('scroll-behavior');
        if (scrollBehavior === 'smooth') {
            console.warn(`${scrollBehaviorElement.tagName} has "scroll-behavior: smooth" which can mess up with animated-scroll-to's animations`);
        }
        // Select the correct scrolling interface
        const elementToScroll = isWindow
            ? new ScrollWindow()
            : new ScrollDomElement(options.elementToScroll);
        if (numberOrCoordsOrElement instanceof Element) {
            scrollToElement = numberOrCoordsOrElement;
            // If "elementToScroll" is not a parent of "scrollToElement"
            if (isElement &&
                (!options.elementToScroll.contains(scrollToElement) ||
                    options.elementToScroll.isSameNode(scrollToElement))) {
                throw 'options.elementToScroll has to be a parent of scrollToElement';
            }
            x = elementToScroll.getHorizontalElementScrollOffset(scrollToElement, options.elementToScroll);
            y = elementToScroll.getVerticalElementScrollOffset(scrollToElement, options.elementToScroll);
        }
        else if (typeof numberOrCoordsOrElement === 'number') {
            x = elementToScroll.getHorizontalScroll();
            y = numberOrCoordsOrElement;
        }
        else if (Array.isArray(numberOrCoordsOrElement) &&
            numberOrCoordsOrElement.length === 2) {
            x =
                numberOrCoordsOrElement[0] === null
                    ? elementToScroll.getHorizontalScroll()
                    : numberOrCoordsOrElement[0];
            y =
                numberOrCoordsOrElement[1] === null
                    ? elementToScroll.getVerticalScroll()
                    : numberOrCoordsOrElement[1];
        }
        else {
            // ERROR
            throw ('Wrong function signature. Check documentation.\n' +
                'Available method signatures are:\n' +
                '  animateScrollTo(y:number, options)\n' +
                '  animateScrollTo([x:number | null, y:number | null], options)\n' +
                '  animateScrollTo(scrollToElement:Element, options)');
        }
        // Add offsets
        x += options.horizontalOffset;
        y += options.verticalOffset;
        // Horizontal scroll distance
        const maxHorizontalScroll = elementToScroll.getMaxHorizontalScroll();
        const initialHorizontalScroll = elementToScroll.getHorizontalScroll();
        // If user specified scroll position is greater than maximum available scroll
        if (x > maxHorizontalScroll) {
            x = maxHorizontalScroll;
        }
        // Calculate distance to scroll
        const horizontalDistanceToScroll = x - initialHorizontalScroll;
        // Vertical scroll distance distance
        const maxVerticalScroll = elementToScroll.getMaxVerticalScroll();
        const initialVerticalScroll = elementToScroll.getVerticalScroll();
        // If user specified scroll position is greater than maximum available scroll
        if (y > maxVerticalScroll) {
            y = maxVerticalScroll;
        }
        // Calculate distance to scroll
        const verticalDistanceToScroll = y - initialVerticalScroll;
        // Calculate duration of the scroll
        const horizontalDuration = Math.abs(Math.round((horizontalDistanceToScroll / 1000) * options.speed));
        const verticalDuration = Math.abs(Math.round((verticalDistanceToScroll / 1000) * options.speed));
        let duration = horizontalDuration > verticalDuration
            ? horizontalDuration
            : verticalDuration;
        // Set minimum and maximum duration
        if (duration < options.minDuration) {
            duration = options.minDuration;
        }
        else if (duration > options.maxDuration) {
            duration = options.maxDuration;
        }
        // @ts-ignore
        return new Promise((resolve, reject) => {
            // Scroll is already in place, nothing to do
            if (horizontalDistanceToScroll === 0 && verticalDistanceToScroll === 0) {
                // Resolve promise with a boolean hasScrolledToPosition set to true
                resolve(true);
            }
            // Cancel existing animation if it is already running on the same element
            activeAnimations.remove(elementToScroll.element, true);
            // To cancel animation we have to store request animation frame ID
            let requestID;
            // Cancel animation handler
            const cancelAnimation = () => {
                removeListeners();
                cancelAnimationFrame(requestID);
                // Resolve promise with a boolean hasScrolledToPosition set to false
                resolve(false);
            };
            // Registering animation so it can be canceled if function
            // gets called again on the same element
            activeAnimations.add(elementToScroll.element, cancelAnimation);
            // Prevent user actions handler
            const preventDefaultHandler = (e) => e.preventDefault();
            const handler = options.cancelOnUserAction
                ? cancelAnimation
                : preventDefaultHandler;
            // If animation is not cancelable by the user, we can't use passive events
            const eventOptions = options.cancelOnUserAction
                ? { passive: true }
                : { passive: false };
            const events = ['wheel', 'touchstart', 'keydown', 'mousedown'];
            // Function to remove listeners after animation is finished
            const removeListeners = () => {
                events.forEach((eventName) => {
                    elementToScroll.element.removeEventListener(eventName, handler, eventOptions);
                });
            };
            // Add listeners
            events.forEach((eventName) => {
                elementToScroll.element.addEventListener(eventName, handler, eventOptions);
            });
            // Animation
            const startingTime = Date.now();
            const step = () => {
                var timeDiff = Date.now() - startingTime;
                var t = timeDiff / duration;
                const horizontalScrollPosition = Math.round(initialHorizontalScroll +
                    horizontalDistanceToScroll * options.easing(t));
                const verticalScrollPosition = Math.round(initialVerticalScroll + verticalDistanceToScroll * options.easing(t));
                if (timeDiff < duration &&
                    (horizontalScrollPosition !== x || verticalScrollPosition !== y)) {
                    // If scroll didn't reach desired position or time is not elapsed
                    // Scroll to a new position
                    elementToScroll.scrollTo(horizontalScrollPosition, verticalScrollPosition);
                    // And request a new step
                    requestID = requestAnimationFrame(step);
                }
                else {
                    // If the time elapsed or we reached the desired offset
                    // Set scroll to the desired offset (when rounding made it to be off a pixel or two)
                    // Clear animation frame to be sure
                    elementToScroll.scrollTo(x, y);
                    cancelAnimationFrame(requestID);
                    // Remove listeners
                    removeListeners();
                    // Remove animation from the active animations coordinator
                    activeAnimations.remove(elementToScroll.element, false);
                    // Resolve promise with a boolean hasScrolledToPosition set to true
                    resolve(true);
                }
            };
            // Start animating scroll
            requestID = requestAnimationFrame(step);
        });
    });
}

const events = new EventManager();

var appEvents = /*#__PURE__*/Object.freeze({
  __proto__: null,
  events: events
});

let history;
let navigate;
let urlMapping = {};
let customHandlers = {};
const setUrlMapping = newMap => {
  urlMapping = Object.assign(Object.assign({}, urlMapping), newMap);
};
const setHistory = newHistory => {
  history = newHistory;
};
const setNavigate = _navigate => {
  navigate = _navigate;
};
const setCustomHandlers = newHandlers => {
  customHandlers = Object.assign(Object.assign({}, customHandlers), newHandlers);
};
const redirectTo = (screen, params = {}, _a = {}) => {
  var {
      target = ''
    } = _a,
    options = __rest(_a, ["target"]);
  if (!screen) {
    return;
  }
  if (screen === 'self') {
    screen = window.location.pathname;
  }
  if (params.handler === 'custom') {
    events.emit('beforeRedirect', screen, params, Object.assign({
      target
    }, options));
    customHandlers[params.handlerType] && customHandlers[params.handlerType](screen, params, Object.assign({
      target
    }, options));
  } else {
    const url = urlMapping[screen];
    const query = new QueryString(params).toString();
    let matchedRegex;
    Object.keys(urlMapping).forEach(key => {
      if (typeof urlMapping[key] === 'object') {
        if (urlMapping[key].type === 'regex') {
          if (screen.match(new RegExp(key, 'g'))) {
            matchedRegex = key;
          }
        }
      }
    });
    if (url) {
      if (target === '_top') {
        window.top.location.href = url + query;
      } else if (target === '_blank') {
        window.open(url + query);
      } else {
        events.emit('beforeRedirect', screen, params, Object.assign({
          target
        }, options));
        animateScrollTo(0);
        setTimeout(() => {
          if (navigate) {
            navigate(url + query);
          } else {
            history.push(url + query);
          }
        });
      }
    } else if (matchedRegex) {
      const processEdUrl = screen.replace(new RegExp(matchedRegex, 'g'), urlMapping[matchedRegex].target);
      if (target === '_top') {
        window.top.location.href = processEdUrl + query;
      } else if (target === '_blank') {
        window.open(processEdUrl + query);
      } else {
        setTimeout(() => {
          events.emit('beforeRedirect', screen, params, Object.assign({
            target
          }, options));
          animateScrollTo(0);
          if (navigate) {
            navigate(processEdUrl + query);
          } else {
            history.push(processEdUrl + query);
          }
        });
      }
    } else {
      if (target === '_top') {
        window.top.location.href = screen + query;
      } else if (target === '_blank') {
        window.open(screen + query);
      } else {
        window.location.href = screen + query;
      }
    }
  }
  setTimeout(() => {
    events.emit('afterRedirect', screen, params, Object.assign({
      target
    }, options));
  }, 120);
};

var redirect = /*#__PURE__*/Object.freeze({
  __proto__: null,
  setUrlMapping: setUrlMapping,
  setHistory: setHistory,
  setNavigate: setNavigate,
  setCustomHandlers: setCustomHandlers,
  redirectTo: redirectTo
});

const _window$3 = window;
let CODES = {
  UNAUTHORIZE_CODE: 403,
  LOGIN_FAILED: 401,
  BAD_REQUEST: 400,
  GENERIC_ERROR: 500,
  NOT_FOUND: 404
};
let TIMEOUT = 1000 * 60 * 5;
const setErrorCodes = newCodes => {
  CODES = Object.assign(Object.assign({}, CODES), newCodes);
};
let responseParsers = {};
const addParsers = newParsers => {
  responseParsers = Object.assign(Object.assign({}, responseParsers), newParsers);
};
const getParsers = () => responseParsers;
const defaultHeaders = {
  'Content-Type': 'application/json'
};
class ApiBridge {
  constructor() {
    this.reset();
  }
  reset() {
    this.events = new EventManager();
    this.headers = {};
  }
  addHeader(name, value) {
    this.headers[name] = value;
  }
  removeHeader(name) {
    delete this.headers[name];
  }
  getCustomHeaders() {
    const url = _window$3.location !== _window$3.parent.location ? document.referrer : document.location.href;
    const accesstore = document.location.ancestorOrigins && document.location.ancestorOrigins[0];
    return Object.assign({
      'x-referer': accesstore || url
    }, this.headers);
  }
  getPrefix(data) {
    const result = this.events.emit('onPrefix', data);
    return result || _window$3.API_SERVER || '';
  }
  handleCatch(ex) {
    var _a;
    const response = {
      status: 'error',
      error: {
        message: ex === null || ex === void 0 ? void 0 : ex.toString(),
        stack: (_a = ex.stack) === null || _a === void 0 ? void 0 : _a.toString()
      }
    };
    this.events.emit('onUnRecognizedError', response);
    return response;
  }
  get(url, params = {}, headers = {}, query = {}, {
    plain = false
  } = {}) {
    const promisObj = fetch(this.getPrefix({
      url,
      body: params
    }) + encodeURI(url) + new QueryString(Object.assign(Object.assign({}, params), query)).toString(), {
      method: 'GET',
      headers: Object.assign(Object.assign(Object.assign({}, defaultHeaders), this.getCustomHeaders()), headers),
      signal: AbortSignal.timeout(TIMEOUT)
    });
    if (plain) {
      return promisObj;
    }
    return promisObj.then(this.checkStatus.bind(this)).then(this.parseJSON).then(this.processCustomParser.bind(this)).then(this.messageParser.bind(this)).then(this.responseReader.bind(this)).catch(this.handleCatch.bind(this));
  }
  rawPost(url, body, headers = {}, query = {}, {
    method = 'POST',
    plain = false
  } = {}) {
    const promisObj = fetch(this.getPrefix({
      url,
      body
    }) + encodeURI(url) + new QueryString(query).toString(), {
      method: method,
      headers: Object.assign(Object.assign({}, this.getCustomHeaders()), headers),
      body,
      signal: AbortSignal.timeout(TIMEOUT)
    });
    if (plain) {
      return promisObj;
    }
    return promisObj.then(this.checkStatus.bind(this)).then(this.parseJSON).then(this.processCustomParser.bind(this)).then(this.messageParser.bind(this)).then(this.responseReader.bind(this)).catch(this.handleCatch.bind(this));
  }
  post(url, body, headers = {}, query = {}, {
    plain = false
  } = {}) {
    const promisObj = fetch(this.getPrefix({
      url,
      body
    }) + encodeURI(url) + new QueryString(query).toString(), {
      method: 'POST',
      headers: Object.assign(Object.assign(Object.assign({}, defaultHeaders), this.getCustomHeaders()), headers),
      body: JSON.stringify(body),
      signal: AbortSignal.timeout(TIMEOUT)
    });
    if (plain) {
      return promisObj;
    }
    return promisObj.then(this.checkStatus.bind(this)).then(this.parseJSON).then(this.processCustomParser.bind(this)).then(this.messageParser.bind(this)).then(this.responseReader.bind(this)).catch(this.handleCatch.bind(this));
  }
  update(url, body, headers = {}, query = {}, {
    plain = false
  } = {}) {
    const promisObj = fetch(this.getPrefix({
      url,
      body
    }) + encodeURI(url) + new QueryString(query).toString(), {
      method: 'PUT',
      headers: Object.assign(Object.assign(Object.assign({}, defaultHeaders), this.getCustomHeaders()), headers),
      body: JSON.stringify(body),
      signal: AbortSignal.timeout(TIMEOUT)
    });
    if (plain) {
      return promisObj;
    }
    return promisObj.then(this.checkStatus.bind(this)).then(this.parseJSON).then(this.processCustomParser.bind(this)).then(this.messageParser.bind(this)).then(this.responseReader.bind(this)).catch(this.handleCatch.bind(this));
  }
  patch(url, body = {}, headers = {}, query = {}, {
    plain = false
  } = {}) {
    const promisObj = fetch(this.getPrefix({
      url,
      body
    }) + encodeURI(url) + new QueryString(query).toString(), {
      method: 'PATCH',
      headers: Object.assign(Object.assign(Object.assign({}, defaultHeaders), this.getCustomHeaders()), headers),
      body: JSON.stringify(body),
      signal: AbortSignal.timeout(TIMEOUT)
    });
    if (plain) {
      return promisObj;
    }
    return promisObj.then(this.checkStatus.bind(this)).then(this.parseJSON).then(this.processCustomParser.bind(this)).then(this.messageParser.bind(this)).then(this.responseReader.bind(this)).catch(this.handleCatch.bind(this));
  }
  delete(url, body = {}, headers = {}, query, {
    plain = false
  } = {}) {
    const promisObj = fetch(this.getPrefix({
      url,
      body
    }) + encodeURI(url) + new QueryString(query).toString(), {
      method: 'DELETE',
      headers: Object.assign(Object.assign(Object.assign({}, defaultHeaders), this.getCustomHeaders()), headers),
      body: JSON.stringify(body),
      signal: AbortSignal.timeout(TIMEOUT)
    });
    if (plain) {
      return promisObj;
    }
    return promisObj.then(this.checkStatus.bind(this)).then(this.parseJSON).then(this.processCustomParser.bind(this)).then(this.messageParser.bind(this)).then(this.responseReader.bind(this)).catch(this.handleCatch.bind(this));
  }
  checkStatus(response) {
    if (response.status >= 200 && response.status < 300) {
      return response;
    } else if (response.status === CODES.LOGIN_FAILED) {
      return response;
    } else if (response.status === CODES.UNAUTHORIZE_CODE) {
      this.events.emit('onUnauthroized', response);
      return response;
    } else if (response.status === CODES.BAD_REQUEST) {
      return response;
    } else if (response.status === CODES.GENERIC_ERROR) {
      return {
        code: response.status,
        status: CONSTANTS.STATUS.UNKNOWN,
        error: {
          message: 'Unexpected error',
          key: 'UNEXPECTED_ERROR'
        }
      };
    } else if (response.status === CODES.NOT_FOUND) {
      return {
        code: response.status,
        status: CONSTANTS.STATUS.UNKNOWN,
        error: {
          message: 'Page not found',
          key: 'NOT_FOUND'
        }
      };
    } else {
      return new Promise(function (resolve) {
        resolve(Object.assign({
          code: response.status,
          status: CONSTANTS.STATUS.SUCCESS
        }, (response === null || response === void 0 ? void 0 : response.data) || {}));
      });
    }
  }
  parseJSON(response) {
    if (response && !response.error) {
      let resp = {};
      try {
        resp = response.json();
      } catch (ex) {
        resp = {
          error: response || {}
        };
      }
      return resp;
    } else {
      return response;
    }
  }
  responseReader(response) {
    var _a;
    switch (response.status) {
      case CONSTANTS.STATUS.SUCCESS:
        return response;
      case CONSTANTS.STATUS.ERROR:
        switch ((_a = response.error) === null || _a === void 0 ? void 0 : _a.handler) {
          case 'REDIRECT':
            response.error.redirectUrl && redirectTo(response.error.redirectUrl);
            break;
          case 'POPUP':
            this.events.emit('onErrorPopup', response);
            break;
          case 'CUSTOM':
            this.events.emit('onCustomError', response);
            break;
        }
        return response;
      case CONSTANTS.STATUS.UNKNOWN:
        this.events.emit('onUnRecognizedError', response);
        return response;
      default:
        return response;
    }
  }
  processCustomParser(response) {
    const parser = response.parser || 'default';
    let newResult;
    if (responseParsers[parser]) {
      newResult = responseParsers[parser].call(this, response);
    }
    return newResult || response;
  }
  messageParser(response) {
    var _a, _b;
    if (((_a = response === null || response === void 0 ? void 0 : response.error) === null || _a === void 0 ? void 0 : _a.key) && messages.get(response.error.key)) {
      response.error.message = messages.get(response.error.key);
      response.error.errorMessage = messages.get(response.error.key);
    }
    if (response === null || response === void 0 ? void 0 : response.errors) {
      Object.keys(response.errors).forEach(errorField => {
        if (response.errors[errorField].errors) {
          this.messageParser(response.errors[errorField]);
        }
        if (response.errors[errorField].key && messages.get(response.errors[errorField].key)) {
          response.errors[errorField].message = messages.get(response.errors[errorField].key);
          response.errors[errorField].errorMessage = messages.get(response.errors[errorField].key);
        }
      });
    }
    if ((_b = response === null || response === void 0 ? void 0 : response.error) === null || _b === void 0 ? void 0 : _b.errors) {
      Object.keys(response.error.errors).forEach(errorField => {
        if (response.error.errors[errorField].errors) {
          this.messageParser(response.error.errors[errorField]);
        }
        if (response.error.errors[errorField].key && messages.get(response.error.errors[errorField].key)) {
          response.error.errors[errorField].message = messages.get(response.error.errors[errorField].key);
          response.error.errors[errorField].errorMessage = messages.get(response.error.errors[errorField].key);
        }
      });
    }
    return response;
  }
}
var apiBridge$1 = new ApiBridge();

const _body = document.body;
const _window$2 = window;
const order = ['xs', 'sm', 'md', 'lg', 'xlg'];
const breakpoints = {
  current: () => {
    const styles = _window$2.getComputedStyle(_body, ':after');
    const content = styles['content'];
    return (content || 'xs').replace(/"/g, '').replace(/'/g, '');
  },
  down: breakpoint => {
    const current = breakpoints.current();
    const currentIdx = order.indexOf(current);
    const breakIdx = order.indexOf(breakpoint);
    if (currentIdx <= breakIdx) {
      return true;
    }
    return false;
  },
  up: breakpoint => {
    const current = breakpoints.current();
    const currentIdx = order.indexOf(current);
    const breakIdx = order.indexOf(breakpoint);
    if (currentIdx >= breakIdx) {
      return true;
    }
    return false;
  }
};
const scriptManager = {
  insertDynamicScript: (dScript, target = 'head') => {
    const str = dScript;
    // Create an element outside the document to parse the string with
    const head = document.createElement('head');
    // Parse the string
    head.innerHTML = str;
    // Copy those nodes to the real `head`, duplicating script elements so
    // they get processed
    let node = head.firstChild;
    while (node) {
      const next = node.nextSibling;
      if (node.tagName === 'SCRIPT') {
        // Just appending this element wouldn't run it, we have to make a fresh copy
        const newNode = document.createElement('script');
        if (node.src) {
          newNode.src = node.src;
        }
        while (node.firstChild) {
          // Note we have to clone these nodes
          newNode.appendChild(node.firstChild.cloneNode(true));
          node.removeChild(node.firstChild);
        }
        node = newNode;
      }
      document[target].appendChild(node);
      node = next;
    }
  }
};
var browser = {
  breakpoints,
  scriptManager
};

const common$3 = utils$1.common;

const {
  currency,
  signs
} = utils$1.currency;
let currentCurrency = (localStorage === null || localStorage === void 0 ? void 0 : localStorage.getItem('currency')) || 'USD';
const set = newCurr => {
  if (currency[newCurr]) {
    currentCurrency = newCurr;
    localStorage && localStorage.setItem('currency', newCurr);
  }
};
const get$1 = () => {
  return currentCurrency;
};
const getSign = curr => {
  return signs[curr] || signs[currentCurrency] || '';
};

var currency$1 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  'default': currency,
  currency: currency,
  signs: signs,
  set: set,
  get: get$1,
  getSign: getSign
});

class Storage {
  constructor(components = {}) {
    this.components = components;
    this.helpers = {};
  }
  setHelpers(helpers) {
    this.helpers = Object.assign(Object.assign({}, this.helpers), helpers);
  }
  add(name, data) {
    this.components[name] = data;
  }
  get() {
    return this.components;
  }
  remove(key) {
    delete this.components[key];
  }
  set(newComps) {
    this.components = Object.assign(Object.assign({}, this.components), newComps);
  }
}
class GroupStorage {
  constructor(components = {}) {
    this.components = {
      default: components
    };
    this.fns = {};
  }
  setHelpers(helpers) {
    this.helpers = Object.assign(Object.assign({}, this.helpers), helpers);
  }
  add(name, data, group = 'default') {
    this.components[group][name] = data;
  }
  get(group) {
    return this.components[group] || this.components.default;
  }
  remove(group) {
    delete this.components[group];
  }
  getAll() {
    let newObj = {};
    Object.keys(this.components).forEach(a => {
      newObj = Object.assign(Object.assign({}, newObj), this.components[a]);
    });
    return newObj;
  }
  set(newComps, group = 'default') {
    this.components = Object.assign(Object.assign({}, this.components), {
      [group]: Object.assign(Object.assign({}, this.components[group] || {}), newComps)
    });
  }
}
const getParseJSON$1 = (obj, isNull) => {
  try {
    return typeof obj == 'string' ? JSON.parse(obj) : obj || (isNull ? undefined : {});
  } catch (e) {
    return obj || (isNull ? undefined : {});
  }
};
class PreferenceStorage {
  constructor() {
    this.helpers = {};
    this.defaultPrefData = null;
    this.namedPrefData = {};
    this._win = window;
    this.events = new EventManager();
  }
  setData(data) {
    this.defaultPrefData = data;
  }
  setNamedData(data) {
    this.namedPrefData = data;
  }
  ensureNoSlashStart(url) {
    if ((url === null || url === void 0 ? void 0 : url.substr(0, 1)) === '/') {
      return url.substr(1);
    } else {
      return url;
    }
  }
  getAllNames() {
    return Object.keys(this.namedPrefData[this.preferenceKey()] || {});
  }
  preferenceKey() {
    return this._win.location.pathname;
  }
  nameString(name) {
    return (name || '').split(' ').join('_').split('/').join('_');
  }
  getKey(prefix) {
    return `${prefix || 'default'}_${this.nameString(this.ensureNoSlashStart(this.preferenceKey()))}`;
  }
  read(key, isNull) {
    if (this.defaultPrefData) {
      return this.defaultPrefData[this.getKey(key)] || (isNull ? undefined : {});
    }
    return getParseJSON$1(this._win.localStorage.getItem(this.getKey(key)), isNull);
  }
  readString(key, isNull) {
    let value;
    if (this.defaultPrefData) {
      value = this.defaultPrefData[this.getKey(key)] || (isNull ? undefined : {});
    } else {
      value = getParseJSON$1(this._win.localStorage.getItem(this.getKey(key)), isNull);
    }
    return typeof value === 'string' ? value : null;
  }
  write(key, data) {
    if (this.defaultPrefData) {
      this.defaultPrefData[this.getKey(key)] = data;
      this.events.emit('onWrite', this.getKey(key), data, this.preferenceKey(), 'default');
      return;
    }
    data && this._win.localStorage.setItem(this.getKey(key), JSON.stringify(data));
  }
  writeAll(obj) {
    const objToSend = {};
    Object.keys(obj).forEach(itemKey => {
      objToSend[this.getKey(itemKey)] = obj[itemKey] || '';
    });
    if (this.defaultPrefData) {
      Object.keys(objToSend).forEach(itemKey => {
        this.defaultPrefData[itemKey] = objToSend[itemKey];
      });
      this.events.emit('onWriteAll', objToSend, this.preferenceKey(), 'default');
      return;
    }
    Object.keys(objToSend).forEach(itemKey => {
      const data = obj[itemKey] || {};
      this._win.localStorage.setItem(itemKey, JSON.stringify(data));
    });
  }
  readNamed(key, name, isNull) {
    if (this.namedPrefData[this.preferenceKey()] && this.namedPrefData[this.preferenceKey()][name]) {
      return this.namedPrefData[this.preferenceKey()][name][this.getKey(key)] || (isNull ? undefined : {});
    }
    return isNull ? undefined : {};
  }
  writeNamed(key, data, name) {
    if (!this.namedPrefData[this.preferenceKey()]) {
      this.namedPrefData[this.preferenceKey()] = {};
    }
    if (!this.namedPrefData[this.preferenceKey()][name]) {
      this.namedPrefData[this.preferenceKey()][name] = {};
    }
    this.namedPrefData[this.preferenceKey()][name][this.getKey(key)] = data;
    this.events.emit('onWriteNamed', this.getKey(key), data, this.preferenceKey(), name);
  }
  writeAllNamed(obj, name) {
    const objToSend = {};
    Object.keys(obj).forEach(itemKey => {
      objToSend[this.getKey(itemKey)] = obj[itemKey] || {};
    });
    if (!this.namedPrefData[this.preferenceKey()]) {
      this.namedPrefData[this.preferenceKey()] = {};
    }
    if (!this.namedPrefData[this.preferenceKey()][name]) {
      this.namedPrefData[this.preferenceKey()][name] = {};
    }
    Object.keys(objToSend).forEach(itemKey => {
      this.namedPrefData[this.preferenceKey()][name][itemKey] = objToSend[itemKey];
    });
    this.events.emit('onWriteAllNamed', objToSend, this.preferenceKey(), name);
    return;
  }
  setHelpers(helpers) {
    this.helpers = Object.assign(Object.assign({}, this.helpers), helpers);
  }
  clearData() {
    if (this.defaultPrefData) {
      this.defaultPrefData = null;
    }
  }
}

const icons = new GroupStorage();
const components$1 = new Storage();
const containers$1 = new Storage();
const dynamicComponents = new Storage();
const charts = new Storage();
const pageBuilder = new Storage();
const richText = new Storage();
const preference$2 = new PreferenceStorage();

var storage = /*#__PURE__*/Object.freeze({
  __proto__: null,
  icons: icons,
  components: components$1,
  containers: containers$1,
  dynamicComponents: dynamicComponents,
  pageBuilder: pageBuilder,
  preference: preference$2,
  richText: richText,
  charts: charts
});

const datetime = utils$1.datetime;
const {
  DateTime: DateTime$1
} = utils$1.datetime;

var datetime$1 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  datetime: datetime,
  DateTime: DateTime$1
});

var colors = ['accent-1', 'accent-2', 'accent-3', 'accent-4', 'accent-5'];
const setColors = _colors => {
  colors = _colors;
};
const getColorByIndex = idx => {
  return colors[idx % colors.length];
};
const getColorByChar = chars => {
  var str = (new String(chars).charAt(0) || 'a').toLowerCase();
  return colors[str.charCodeAt(0) % colors.length];
};
var accent = {
  getColorByIndex,
  getColorByChar,
  setColors
};

const string = {
  titleCase: val => {
    return !common$3.isNullOrUndefined(val === null || val === void 0 ? void 0 : val.trim()) ? val.trim()[0].toUpperCase() + val.trim().substr(1).toLowerCase() : '';
  },
  getFirstChar: val => {
    return !common$3.isNullOrUndefined(val) ? val.trim()[0] : '';
  },
  getTwoChars: val => {
    if (!common$3.isNullOrUndefined(val)) {
      const array = val.split(' ').filter(i => !!i);
      return (array[0] && array[0][0] || '') + (array[1] && array[1][0] || '');
    }
    return '';
  }
};

const object$2 = utils$1.object;
const common$2 = utils$1.common;
class CustomProcessor {
  constructor() {
    this.processor = {
      compare: {
        equals: (value, {
          matchValue: _matchValue = ''
        } = {}) => {
          return value === _matchValue;
        },
        notEquals: (value, {
          matchValue: _matchValue2 = ''
        } = {}) => {
          return value !== _matchValue2;
        },
        exists: value => {
          return !!value;
        }
      },
      dataType: {
        string: value => {
          return !common$2.isNullOrUndefined(value) ? value.toString() : value;
        },
        code: value => {
          return !common$2.isNullOrUndefined(value === null || value === void 0 ? void 0 : value.code) ? value.code.toString() : '';
        },
        name: value => {
          return !common$2.isNullOrUndefined(value === null || value === void 0 ? void 0 : value.name) ? value.name.toString() : '';
        },
        number: value => {
          return !common$2.isNullOrUndefined(value) ? value * 1 : value;
        },
        forceArray: value => {
          if (value && Array.isArray(value)) {
            return value;
          }
          return [];
        }
      },
      common: {
        accentByChar: value => {
          return accent.getColorByChar(string.getTwoChars(value));
        },
        getTwoChars: value => {
          return string.getTwoChars(value);
        },
        addClassName: (value, {
          oldValue: _oldValue = ''
        } = {}) => {
          return _oldValue + ' ' + value;
        },
        join: (value, {
          fields
        }, {
          userData
        }) => {
          const arr = fields === null || fields === void 0 ? void 0 : fields.split('+');
          const values = arr === null || arr === void 0 ? void 0 : arr.map(val => {
            val = val.trim().replace(new RegExp('&nbsp;', 'g'), ' ');
            if (val.charAt(0) === '.') {
              return object$2.getDataFromKey(userData, val.substr(1));
            }
            return val;
          }).join('');
          return values;
        }
      },
      array: {
        join: value => {
          return Array.isArray(value) ? value.join(',') : value;
        },
        extractByKey: (value, {
          key: _key = 'code'
        } = {}) => {
          return Array.isArray(value) ? value.map(i => i[_key]).join(',') : value;
        },
        extractDataArray: result => {
          if (result.status === CONSTANTS.STATUS.SUCCESS) {
            return {
              pages: result.data.data
            };
          }
        }
      },
      globals: {
        options: value => {
          var _a;
          return (_a = this.globalOptions[value]) === null || _a === void 0 ? void 0 : _a.toArray();
        },
        optionsText: (value, {
          name
        } = {}) => {
          var _a;
          const option = (_a = this.globalOptions[name]) === null || _a === void 0 ? void 0 : _a.getText(value);
          return option || '';
        },
        optionsKey: (value, {
          name,
          keyName
        } = {}) => {
          const option = this.globalOptions[name].get(value);
          return option && option[keyName] || '';
        },
        filterOptions: (value, _a) => {
          var {
              optionsName
            } = _a,
            params = __rest(_a, ["optionsName"]);
          if (this.globalOptions[optionsName]) {
            return this.globalOptions[optionsName].fromData(value, params);
          }
          return value;
        },
        getOptionArray: value => {
          if (this.globalOptions[value]) {
            return this.globalOptions[value].toArray();
          }
          return [];
        },
        getOption: (value, {
          optionsName,
          options,
          textField: _textField = 'text',
          valueField: _valueField = 'value'
        } = {}) => {
          if (options) {
            const filterOpt = options.filter(i => i[_valueField] === value);
            if (filterOpt.length > 0) {
              return {
                text: filterOpt[0][_textField],
                value: filterOpt[0][_valueField]
              };
            }
          }
          if (this.globalOptions[optionsName]) {
            return this.globalOptions[optionsName].get(value);
          }
          return value;
        }
      }
    };
    this.globalOptions = {};
  }
  registerOptions(options) {
    this.globalOptions = options;
  }
  add(moduleName, actions) {
    if (!this.processor[moduleName]) {
      this.processor[moduleName] = {};
    }
    this.processor[moduleName] = Object.assign(Object.assign({}, this.processor[moduleName]), actions);
  }
  remove(moduleName) {
    delete this.processor[moduleName];
  }
  execute(moduleAction, ...params) {
    const actions = moduleAction.split('.');
    const moduleName = actions[0];
    const action = actions[1];
    let output;
    if (this.processor[moduleName]) {
      if (this.processor[moduleName][action]) {
        output = this.processor[moduleName][action].apply(this, params);
      } else {
        throw `${moduleName}.${action} action not found`;
      }
    } else {
      throw `${moduleName} module not found`;
    }
    return output;
  }
}

class CustomModuleManager {
  constructor() {
    this.modules = {};
  }
  add(moduleName, actions) {
    if (!this.modules[moduleName]) {
      this.modules[moduleName] = {};
    }
    this.modules[moduleName] = Object.assign(Object.assign({}, this.modules[moduleName]), actions);
  }
  remove(moduleName) {
    delete this.modules[moduleName];
  }
  execute(moduleAction, ...params) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const actions = moduleAction.split('.');
      const moduleName = actions[0];
      const action = actions[1];
      let output;
      if (this.modules[moduleName]) {
        if (this.modules[moduleName][action]) {
          output = this.modules[moduleName][action].apply(this, params);
        } else {
          throw `${moduleName}.${action} action not found`;
        }
      } else {
        throw `${moduleName} module not found`;
      }
      return output;
    });
  }
}

const {
  setDefaults,
  getDefaults,
  getFormatters: _getFormatters,
  setFormatters: _setFormatters
} = utils$1.format;
const {
  DateTime
} = utils$1.datetime;
setDefaults({
  date: {},
  currency: {
    decimals: 0
  }
});
const oldFromatters = _getFormatters();
const formatters$1 = Object.assign(Object.assign({}, oldFromatters), {
  currency: (value, options = {}) => {
    const {
        currency = get$1()
      } = options,
      rest = __rest(options, ["currency"]);
    const sign = getSign(currency);
    return oldFromatters.currency(value, Object.assign({
      sign,
      currency
    }, rest));
  },
  currencyAbr: (value, options = {}) => {
    const {
        currency = get$1()
      } = options,
      rest = __rest(options, ["currency"]);
    const sign = getSign(currency);
    return oldFromatters.currencyAbr(value, Object.assign({
      sign,
      currency
    }, rest));
  },
  number: (value, options = {}) => {
    const rest = __rest(options, []);
    return oldFromatters.currency(value, Object.assign({
      sign: '',
      currency: 'none'
    }, rest));
  },
  numberAbr: (value, options = {}) => {
    const rest = __rest(options, []);
    return oldFromatters.currencyAbr(value, Object.assign({
      sign: '',
      currency: 'none'
    }, rest));
  },
  dateFull: (value, options = {}) => {
    var _a;
    const {
      format = 'MMM, DD YYYY'
    } = Object.assign(Object.assign({}, ((_a = getDefaults().date) === null || _a === void 0 ? void 0 : _a.dateFull) || {}), options);
    return value ? new DateTime(value).toString(format) : '';
  },
  dateForTimezone: (value, options = {}) => {
    var _a;
    const {
      timezone = 'America/New_York',
      format = 'MMM, DD YYYY hh:mm A zz'
    } = Object.assign(Object.assign({}, ((_a = getDefaults().date) === null || _a === void 0 ? void 0 : _a.dateForTimezone) || {}), options);
    return value ? new DateTime(value)._date.tz(timezone).format(format) : '';
  },
  shortDate: (value, options = {}) => {
    var _a;
    const {
      format = 'MM/DD/YY (ddd)'
    } = Object.assign(Object.assign({}, ((_a = getDefaults().date) === null || _a === void 0 ? void 0 : _a.shortDate) || {}), options);
    return value ? new DateTime(value).toString(format) : '';
  },
  dateOnly: (value, options = {}) => {
    var _a;
    const {
      format = 'DD/MM'
    } = Object.assign(Object.assign({}, ((_a = getDefaults().date) === null || _a === void 0 ? void 0 : _a.dateOnly) || {}), options);
    return value ? new DateTime(value).toString(format) : '';
  },
  monthYear: (value, options = {}) => {
    var _a;
    const {
      format = 'MMM YYYY'
    } = Object.assign(Object.assign({}, ((_a = getDefaults().date) === null || _a === void 0 ? void 0 : _a.monthYear) || {}), options);
    return value ? new DateTime(value).toString(format) : '';
  },
  dateFullTime: (value, options = {}) => {
    var _a;
    const {
      format = 'MMM, DD YYYY hh:mm A'
    } = Object.assign(Object.assign({}, ((_a = getDefaults().date) === null || _a === void 0 ? void 0 : _a.dateFullTime) || {}), options);
    return value ? new DateTime(value).toString(format) : '';
  },
  time: (value, options = {}) => {
    var _a;
    const {
      format = 'MMM, DD YYYY hh:mm A'
    } = Object.assign(Object.assign({}, ((_a = getDefaults().date) === null || _a === void 0 ? void 0 : _a.time) || {}), options);
    return value ? new DateTime(value).toString(format) : '';
  }
});
_setFormatters(formatters$1);
const addFormatter = (name, formatter) => {
  _setFormatters({
    [name]: formatter
  });
};
const getFormatters$1 = () => {
  return Object.assign(Object.assign({}, _getFormatters()), formatters$1);
};
const setFormatters = newFormatters => {
  return _setFormatters(newFormatters);
};

var format = /*#__PURE__*/Object.freeze({
  __proto__: null,
  getFormatters: getFormatters$1,
  setFormatters: setFormatters,
  formatters: formatters$1,
  setDefaults: setDefaults,
  addFormatter: addFormatter
});

const {
  getFormatters
} = utils$1.format;
const {
  isNullOrUndefined
} = utils$1.common;
const masks = {
  currency: {
    mask: (value, options = {}) => {
      return getFormatters().currency(value, Object.assign({
        sign: getSign(),
        currency: get$1(),
        input: true,
        decimals: 2
      }, options));
    },
    unmask: (value, options = {}) => {
      return value && value.toString().replace(/[^0-9.-]/g, '');
    }
  },
  appendViewOnlyText: {
    mask: (value, {
      appendAt: _appendAt = 'start',
      text: _text = ''
    } = {}) => {
      return value.indexOf(_text) === -1 ? (_appendAt === 'start' ? _text : '') + `${value || ''}` + (_appendAt === 'end' ? _text : '') : value;
    },
    unmask: (value, {
      text: _text2 = ''
    } = {}) => {
      return value.replace(_text2, '');
    }
  },
  titleCase: {
    mask: (value, {
      input: _input = false
    }) => {
      var _a, _b;
      const allArr = value.split(' ');
      for (let i = 0; i < allArr.length; i++) {
        allArr[i] = ((_a = allArr[i][0] || '') === null || _a === void 0 ? void 0 : _a.toUpperCase()) + ((_b = allArr[i] || '') === null || _b === void 0 ? void 0 : _b.substr(1));
      }
      return allArr.join(' ');
    },
    unmask: value => {
      return value;
    }
  },
  upperCase: {
    mask: (value, {
      input: _input2 = false
    }) => {
      return value === null || value === void 0 ? void 0 : value.toString().toUpperCase();
    },
    unmask: value => {
      return value;
    }
  },
  lowerCase: {
    mask: (value, {
      input: _input3 = false
    }) => {
      return value === null || value === void 0 ? void 0 : value.toString().toLowerCase();
    },
    unmask: value => {
      return value;
    }
  },
  appendText: {
    mask: (value, {
      appendAt: _appendAt2 = 'start',
      text: _text3 = ''
    } = {}) => {
      return value.indexOf(_text3) === -1 ? (_appendAt2 === 'start' ? _text3 : '') + `${value || ''}` + (_appendAt2 === 'end' ? _text3 : '') : value;
    },
    unmask: (value, {
      text: _text4 = ''
    } = {}) => {
      return value;
    }
  },
  number: {
    mask: (value, {
      pattern: _pattern = '',
      param: _param = 'D'
    } = {}) => {
      let out = '';
      if (_pattern) {
        const allValues = _pattern.split('');
        const valueArra = value.toString().split('');
        let idx = 0;
        allValues.forEach(digit => {
          if (digit === _param) {
            out += valueArra[idx] || '';
            idx++;
          } else if (idx < valueArra.length) {
            out += digit;
          }
        });
      }
      return out;
    },
    unmask: value => {
      return value && value.toString().replace(/[^0-9.]/g, '');
    }
  },
  phone: {
    mask: value => {
      value = isNullOrUndefined(value) ? '' : value.toString();
      const x = value.replace(/\D/g, '').match(/(\d{0,3})(\d{0,3})(\d{0,4})/);
      if (!x) {
        return value;
      }
      return !x[2] ? x[1] : '(' + x[1] + ') ' + x[2] + (x[3] ? '-' + x[3] : '');
    },
    unmask: value => {
      return value && value.toString().replace(/[^0-9]/g, '').substr(0, 10);
    }
  },
  INphone: {
    mask: value => {
      value = isNullOrUndefined(value) ? '' : value.toString();
      const x = value.replace(/\D/g, '').match(/(\d{0,4})(\d{0,6})/);
      if (!x) {
        return value;
      }
      return `${!x[2] ? x[1] : +x[1] + ' - ' + x[2]}`;
    },
    unmask: value => {
      return value && value.toString().replace(/[^0-9]/g, '').substr(0, 10);
    }
  },
  percentage: {
    mask: (value, {
      input: _input4 = false
    } = {}) => {
      if (!_input4) {
        return value + '%';
      } else {
        return value;
      }
    },
    unmask: value => {
      return value && value.toString().replace(/[^0-9.-]/g, '');
    }
  }
};
const getMasks = () => masks;
const addMask = (name, obj) => {
  masks[name] = obj;
};

var mask = /*#__PURE__*/Object.freeze({
  __proto__: null,
  getMasks: getMasks,
  addMask: addMask,
  masks: masks
});

const {
  getRandomS4,
  getRandomS8,
  getRandomS6,
  preFix
} = utils$1.number;

var number = /*#__PURE__*/Object.freeze({
  __proto__: null,
  getRandomS4: getRandomS4,
  getRandomS8: getRandomS8,
  getRandomS6: getRandomS6,
  preFix: preFix
});

const {
  getValue,
  idFromLabel
} = utils$1.common;

var properties$1 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  getValue: getValue,
  idFromLabel: idFromLabel
});

const {
  Timeunit: Timeunit$1
} = utils$1;
class CookieManager {
  constructor() {
    this.cookieInfo = {};
    this.timeUnit = new Timeunit$1();
  }
  add(cname, cvalue, period = '1d', path = '/') {
    let expires = '';
    if (period) {
      const d = new Date();
      d.setTime(d.getTime() + this.timeUnit.getTimeunit(period));
      expires = d.toUTCString();
    }
    if (expires) {
      expires = `expires=${expires};`;
    }
    document.cookie = `${cname}=${cvalue};${expires}path=${path};`;
  }
  remove(cname, path = '/') {
    document.cookie = `${cname}=;expires=Thu, 01 Jan 1970 00:00:00 UTC;path=${path};`;
  }
  get(name) {
    return this.getAll()[name];
  }
  getAll() {
    const obj = {};
    const decodedCookie = decodeURIComponent(document.cookie);
    const ca = decodedCookie.split(';');
    for (let i = 0; i < ca.length; i++) {
      let c = ca[i];
      while (c.charAt(0) == ' ') {
        c = c.substring(1);
      }
      const splitV = c.split('=');
      if (splitV[0]) {
        obj[splitV[0]] = splitV[1] || '';
      }
    }
    return obj;
  }
}
var cookieMgr = new CookieManager();

const {
  Validator: Validator$1,
  addValidator,
  getValidators,
  addMessage,
  getMessages,
  setObject
} = utils$1.validator;
Validator$1.parseMessage = message => {
  return translate$5(message);
};
addValidator('hasCookieValue', (value, {
  cookieName
}) => {
  return !!cookieMgr.getAll()[cookieName];
});
addValidator('cookieValue', (value, {
  cookieName
}) => {
  return cookieMgr.getAll()[cookieName] === value;
});

var validator = /*#__PURE__*/Object.freeze({
  __proto__: null,
  Validator: Validator$1,
  addValidator: addValidator,
  getValidators: getValidators,
  addMessage: addMessage,
  getMessages: getMessages,
  setObject: setObject
});

const _window$1 = window;
const getWindow = () => _window$1;

var window$1 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  getWindow: getWindow
});

const {
  path,
  object: object$1,
  timer,
  Timeunit
} = utils$1;
setObject(object$1);
const processor$1 = new CustomProcessor();

var utils = /*#__PURE__*/Object.freeze({
  __proto__: null,
  apiBridge: apiBridge$1,
  appEvents: appEvents,
  browser: browser,
  common: common$3,
  currency: currency$1,
  datetime: datetime$1,
  object: object$1,
  path: path,
  errorMessages: errorMessages,
  cookie: cookieMgr,
  CustomModule: CustomModuleManager,
  EventManager: EventManager,
  CustomProcessor: CustomProcessor,
  format: format,
  mask: mask,
  Timeunit: Timeunit,
  number: number,
  properties: properties$1,
  queryString: queryString$1,
  accentColors: accent,
  redirect: redirect,
  timer: timer,
  translate: translate$6,
  validator: validator,
  processor: processor$1,
  storage: storage,
  string: string,
  setErrorCodes: setErrorCodes,
  getParsers: getParsers,
  addParsers: addParsers,
  win: window$1
});

class DefaultE$1 {}
class Filterable {
  constructor(data = [], {
    entityType = data => {
      return data;
    },
    Queryable = DefaultE$1
  }) {
    this.rData = data;
    this.entityType = entityType;
    this.Queryable = Queryable;
  }
  filterBy(options) {
    return new this.Queryable(filter$1(this.rData, options), {
      entityType: this.entityType
    });
  }
}

class DefaultE {}
class Sortable {
  constructor(data = [], {
    entityType = DefaultE,
    Queryable = DefaultE
  } = {}) {
    this.rData = data;
    this.entityType = entityType;
    this._sortOrder = [];
    this.Queryable = Queryable;
  }
  sortOrder(field, order = 'asc') {
    this._sortOrder.push({
      field,
      order
    });
    return this;
  }
  sortBy(field, order = 'asc') {
    this.sortOrder(field, order);
    const data = this.rData.sort((a, b) => {
      let idx;
      for (let count = 0; count < this._sortOrder.length; count++) {
        const order = this._sortOrder[count].order;
        const field = this._sortOrder[count].field;
        let data1 = a[field];
        let data2 = b[field];
        if (typeof data1 === 'string') {
          data1 = data1.toLowerCase();
        }
        if (typeof data2 === 'string') {
          data2 = data2.toLowerCase();
        }
        if (data1 > data2) {
          idx = order === 'asc' ? 1 : -1;
          break;
        } else if (data1 < data2) {
          idx = order === 'asc' ? -1 : 1;
          break;
        } else {
          idx = 0;
        }
      }
      return idx;
    });
    this._sortOrder = [];
    return new this.Queryable(data, {
      entityType: this.entityType
    });
  }
}

class Queryable {
  constructor(data = [], {
    entityType = data => data
  } = {}) {
    this.rData = data;
    this.entityType = entityType;
  }
  filterBy(options) {
    return new Filterable(this.rData, {
      entityType: this.entityType,
      Queryable
    }).filterBy(options);
  }
  sortBy(field, asc) {
    return new Sortable(this.rData, {
      entityType: this.entityType,
      Queryable
    }).sortBy(field, asc);
  }
  sortOrder(field, asc) {
    return new Sortable(this.rData, {
      entityType: this.entityType,
      Queryable
    }).sortOrder(field, asc);
  }
  toArray(derived = true) {
    return this.rData.map(item => {
      const fix = new this.entityType(item);
      return fix.toJson ? fix.toJson(derived) : fix;
    });
  }
  toObject(key, value, defaultValue = '') {
    const obj = {};
    this.rData.forEach(item => {
      obj[item[key]] = item[value];
    });
    return obj;
  }
}

var collection = /*#__PURE__*/Object.freeze({
  __proto__: null,
  Filterable: Filterable,
  Sortable: Sortable,
  Queryable: Queryable
});

class Entity {
  constructor({
    raw = {},
    entityType = Entity
  }, options = {}) {
    this.hasChanged = false;
    this.hasValidated = false;
    const {
      ref,
      repo,
      snapshot,
      validators = {}
    } = options;
    this.entityType = entityType;
    this.repo = repo;
    this.uid = raw.uid;
    this.rawData = raw;
    this.setDefaults();
    this.setData(raw);
    this.isSystem = !!raw.isSystem;
    this.setDerivedFields();
    this.validators = new Validator$1(Object.assign({}, validators));
  }
  onUpdateFields(options) {}
  onBeforeJSON(options) {}
  updateField(name, value) {
    this[name] = this.entityType.fields[name] ? this.entityType.fields[name](value) : value;
    this.onUpdateFields && this.onUpdateFields({
      [name]: value
    });
  }
  updateFields(fields) {
    Object.keys(fields).forEach(fieldKey => {
      if (this.entityType.fields[fieldKey] != undefined) {
        this[fieldKey] = this.entityType.fields[fieldKey](fields[fieldKey]);
      }
    });
    this.onUpdateFields && this.onUpdateFields(fields);
  }
  setDefaults() {
    this.entityType.fields = Object.assign({}, this.entityType.fields);
  }
  validateAll() {
    this.validators.setValues(this.toJson());
    this._isValid = this.validators.validateAll();
    return this._isValid;
  }
  validate(field) {
    this.validators.setValues(this.toJson());
    return this.validators.validate(field);
  }
  setData(raw) {
    if (Object.keys(this.entityType.fields).length > 0) {
      Object.keys(this.entityType.fields).forEach(fieldKey => {
        this[fieldKey] = this.entityType.fields[fieldKey](raw[fieldKey]);
      });
    } else {
      Object.keys(raw).forEach(fieldKey => {
        this[fieldKey] = raw[fieldKey];
      });
    }
    this.entityType.infoFields && Object.keys(this.entityType.infoFields).forEach(fieldKey => {
      this[fieldKey] = this.entityType.infoFields[fieldKey](raw[fieldKey]);
    });
  }
  setDerivedFields() {
    if (this.entityType.derivedFields) {
      Object.keys(this.entityType.derivedFields).forEach(fieldKey => {
        this[fieldKey] = this.entityType.derivedFields[fieldKey](this);
      });
    }
  }
  updateRaw() {
    Object.keys(this.entityType.fields).forEach(fieldKey => {
      this.rawData[fieldKey] = this.entityType.fields[fieldKey](this[fieldKey]);
    });
    return this;
  }
  toJson(setDerived) {
    const objToReturn = {};
    if (this.uid) {
      objToReturn.uid = this.uid;
    }
    Object.keys(this.entityType.fields).forEach(fieldKey => {
      objToReturn[fieldKey] = this[fieldKey];
    });
    if (setDerived && this.entityType.derivedFields) {
      Object.keys(this.entityType.derivedFields).forEach(fieldKey => {
        objToReturn[fieldKey] = this.entityType.derivedFields[fieldKey](this);
      });
    }
    let overrides;
    if (typeof this.onBeforeJSON === 'function') {
      overrides = this.onBeforeJSON(objToReturn);
    }
    return Object.assign(Object.assign({}, objToReturn), overrides);
  }
}
Entity.fields = {};

var entity = /*#__PURE__*/Object.freeze({
  __proto__: null,
  Entity: Entity
});

class DataTypes {
  isNullOrUndefined(value) {
    return value === null || value === undefined;
  }
  isObject(value) {
    return value && typeof value === 'object';
  }
  bool(defaultValue) {
    return value => {
      return Boolean(value) || defaultValue;
    };
  }
  string(defaultValue) {
    return value => {
      return !this.isNullOrUndefined(value) && value.toString() || defaultValue;
    };
  }
  number(defaultValue) {
    return value => {
      const numRegex = /^\s*[+-]?(\d+|\.\d+|\d+\.\d+|\d+\.)(e[+-]?\d+)?\s*$/;
      const finalValue = !this.isNullOrUndefined(value) ? value.toString() : '';
      if (!finalValue.match(numRegex) && !this.isObject(value)) {
        return value;
      }
      if (!this.isNullOrUndefined(value) && !isNaN(value)) {
        return value * 1;
      } else if (this.isNullOrUndefined(value) && !isNaN(defaultValue)) {
        return defaultValue * 1;
      } else if (typeof value === 'string') {
        return value;
      }
      return this.isObject(value) ? undefined : value && value.toString();
    };
  }
  any(defaultValue) {
    return value => {
      return this.isNullOrUndefined(value) ? defaultValue : value;
    };
  }
  array(defaultValue) {
    return value => {
      return this.isNullOrUndefined(value) ? defaultValue : value;
    };
  }
  object(defaultValue) {
    return value => {
      return value === undefined ? defaultValue : value;
    };
  }
  date(defaultValue) {
    return value => {
      return value && new Date(value) || defaultValue;
    };
  }
  dateISOString(defaultValue) {
    return value => {
      return this.isNullOrUndefined(value) ? defaultValue && new Date(defaultValue).toISOString() : value && moment(value).isValid() ? new Date(value).toISOString() : value;
    };
  }
}
var dataTypes = new DataTypes();

var helpers$1 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  DataTypes: dataTypes
});

class Repository {
  constructor({
    type,
    apiUrl,
    collection
  }) {
    this.typeOf = type;
    this.apiUrl = apiUrl;
    this.collection = collection;
    this.getEntity = this.getEntity.bind(this);
    this.getQueryable = this.getQueryable.bind(this);
  }
  createNew(raw) {
    return new this.typeOf(raw);
  }
  createCollection(data) {
    return new Queryable(data, {
      entityType: this.typeOf
    });
  }
  get(params = {}, {
    type = 'get',
    url
  } = {}) {
    return __awaiter$1(this, void 0, void 0, function* () {
      return yield apiBridge$1[type](url || this.apiUrl, params);
    });
  }
  getOne(params, options = {}) {
    return __awaiter$1(this, void 0, void 0, function* () {
      return yield this.get(params, Object.assign(Object.assign({}, options), {
        type: 'post'
      })).then(this.getEntity.bind(this));
    });
  }
  getMany(params, options = {}) {
    return __awaiter$1(this, void 0, void 0, function* () {
      return yield this.get(params, Object.assign(Object.assign({}, options), {
        type: 'post'
      })).then(this.getQueryable);
    });
  }
  create(params = {}, {
    url
  } = {}) {
    return __awaiter$1(this, void 0, void 0, function* () {
      return yield apiBridge$1.post(url || this.apiUrl, params).then(this.getEntity);
    });
  }
  search(params, {
    url
  } = {}) {
    return __awaiter$1(this, void 0, void 0, function* () {
      return yield apiBridge$1.post(url || this.apiUrl, params).then(this.getQueryable);
    });
  }
  getEntity(response) {
    if (response.status === CONSTANTS.STATUS.SUCCESS) {
      return Object.assign(Object.assign({}, response), {
        data: new this.typeOf(response.data)
      });
    }
    return response;
  }
  toQueryable(data) {
    return new Queryable(data, {
      entityType: this.typeOf
    });
  }
  getQueryable(response, dataKey) {
    if (response.status === CONSTANTS.STATUS.SUCCESS) {
      return Object.assign(Object.assign({}, response), {
        data: Object.assign(Object.assign({}, response.data), {
          [dataKey]: response.data[dataKey] ? new Queryable(response.data[dataKey], {
            entityType: this.typeOf
          }) : {}
        })
      });
    }
    return response;
  }
  fetch(params, {
    url,
    dataKey = this.collection
  } = {}) {
    return __awaiter$1(this, void 0, void 0, function* () {
      return yield apiBridge$1.post(url || this.apiUrl, params).then(res => this.getQueryable(res, dataKey));
    });
  }
  update(params, {
    url
  } = {}) {
    return __awaiter$1(this, void 0, void 0, function* () {
      return yield apiBridge$1.update(url || this.apiUrl, params).then(this.getEntity.bind(this));
    });
  }
  updateWithPost(params, {
    url
  } = {}) {
    return __awaiter$1(this, void 0, void 0, function* () {
      return yield apiBridge$1.post(url || this.apiUrl, params).then(this.getEntity.bind(this));
    });
  }
  delete(params, {
    url
  } = {}) {
    return __awaiter$1(this, void 0, void 0, function* () {
      return yield apiBridge$1.delete(url || this.apiUrl, params);
    });
  }
  deleteByUid(params, {
    url
  } = {}) {
    return __awaiter$1(this, void 0, void 0, function* () {
      return yield apiBridge$1.delete(url || this.apiUrl, {
        uid: params.uid
      });
    });
  }
  post(url, params) {
    return __awaiter$1(this, void 0, void 0, function* () {
      return yield apiBridge$1.post(url, params);
    });
  }
  postOne(url, params) {
    return __awaiter$1(this, void 0, void 0, function* () {
      return yield apiBridge$1.post(url, params).then(this.getEntity.bind(this));
    });
  }
}

var repository = /*#__PURE__*/Object.freeze({
  __proto__: null,
  Repository: Repository
});

const domain = {
  collection,
  entity,
  helpers: helpers$1,
  repository
};

var index$n = /*#__PURE__*/Object.freeze({
  __proto__: null,
  'default': domain
});

const customHooks = new CustomModuleManager();
customHooks.add('data', {
  extractResult: (result, {
    dataKey: _dataKey = 'data',
    rootKey: _rootKey = 'data',
    rootErrorKey: _rootErrorKey = 'error',
    index: _index = 0
  } = {}) => {
    if (result.status === CONSTANTS.STATUS.SUCCESS) {
      if (result[_rootKey] && result[_rootKey][_dataKey] && result[_rootKey][_dataKey].length > _index) {
        return result[_rootKey][_dataKey][_index];
      }
    } else if (result.status === CONSTANTS.STATUS.ERROR) {
      if (result[_rootErrorKey] && result[_rootErrorKey]) {
        return result[_rootErrorKey];
      }
    }
    return null;
  }
});
customHooks.add('contentPages', {
  getAppsData: (result, data) => __awaiter$1(void 0, void 0, void 0, function* () {
    var _a;
    let navigation = [];
    const promises = [];
    if ((_a = data.payload) === null || _a === void 0 ? void 0 : _a.apps) {
      const siteMapPath = data.payload.dataUrl || '/env/sitemap.json';
      data.payload.apps.forEach(appName => {
        promises.push(apiBridge$1.get(`${appName === '/' ? '' : `/${appName}`}${siteMapPath}`));
      });
    }
    yield Promise.all(promises).then(result => {
      result.forEach(dataItem => {
        var _a, _b;
        if ((_a = dataItem === null || dataItem === void 0 ? void 0 : dataItem.data) === null || _a === void 0 ? void 0 : _a.navigation) {
          navigation = [...navigation, ...((_b = dataItem === null || dataItem === void 0 ? void 0 : dataItem.data) === null || _b === void 0 ? void 0 : _b.navigation)];
        }
      });
    });
    if (result.status === CONSTANTS.STATUS.SUCCESS) {
      return Object.assign(Object.assign({}, result), {
        data: {
          navigation
        }
      });
    }
  }),
  extractDataArray: result => {
    if (result.status === CONSTANTS.STATUS.SUCCESS) {
      return Object.assign(Object.assign({}, result.data), {
        pages: result.data.data
      });
    }
  }
});

const initialState$7 = {
  appLoaded: false,
  isLoading: false,
  error: {},
  notification: {},
  popup: {},
  popupScreen: {}
};
const common$1 = createSlice({
  name: 'common',
  initialState: initialState$7,
  reducers: {
    startLoading: state => {
      state.isLoading = true;
    },
    stopLoading: state => {
      state.isLoading = false;
    },
    setError: (state, action) => {
      state.error.hasError = true;
      state.error.errorMessage = action.payload.message;
    },
    clearError: (state, action) => {
      state.error = {};
    },
    setNotification: (state, action) => {
      state.notification.message = action.payload.message;
      state.notification.type = action.payload.type;
      state.notification.show = true;
    },
    setNotificationTimeout: (state, action) => {
      state.notification.prevTimeout = action.payload;
    },
    setPopup: (state, action) => {
      state.popup.message = action.payload.message;
      state.popup.title = action.payload.title;
      state.popup.type = action.payload.type;
      state.popup.severity = action.payload.severity;
      state.popup.show = true;
    },
    closePopup: state => {
      state.popup = {};
      state.popup.show = false;
    },
    setPopupScreen: (state, action) => {
      const _a = action.payload,
        {
          name,
          title,
          dialogProps = {}
        } = _a,
        props = __rest(_a, ["name", "title", "dialogProps"]);
      state.popupScreen.name = name;
      state.popupScreen.title = title;
      state.popupScreen.dialogProps = dialogProps;
      state.popupScreen.props = props;
      state.popupScreen.show = true;
    },
    closePopupScreen: state => {
      state.popupScreen = {};
      state.popupScreen.show = false;
    },
    setCloseNotification: state => {
      state.notification.show = false;
    }
  }
});
const {
  setNotification,
  setNotificationTimeout,
  startLoading,
  stopLoading,
  setError,
  closePopup,
  clearError,
  closePopupScreen
} = common$1.actions;
const selectCurrentNotification = state => state.common.notification;
const showNotificationMessage = payload => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  const {
    timeout = 3000
  } = payload;
  const currentNotification = selectCurrentNotification(getState());
  currentNotification.prevTimeout && clearTimeout(currentNotification.prevTimeout);
  yield dispatch(setNotificationTimeout(null));
  yield dispatch(setNotification(payload));
  const nextTimeout = setTimeout(() => {
    dispatch(closeNotification());
    payload.callback && payload.callback();
  }, timeout);
  yield dispatch(setNotificationTimeout(nextTimeout));
});
const closeNotification = () => (dispatch, getState) => {
  const currentNotification = selectCurrentNotification(getState());
  currentNotification.prevTimeout && clearTimeout(currentNotification.prevTimeout);
  dispatch(common$1.actions.setCloseNotification());
};
const showPopup = payload => dispatch => __awaiter$1(void 0, void 0, void 0, function* () {
  yield dispatch(common$1.actions.setPopup(payload));
});
const showPopupScreen = payload => dispatch => __awaiter$1(void 0, void 0, void 0, function* () {
  yield dispatch(common$1.actions.setPopupScreen(payload));
});
const getParseJSON = (obj, isNull) => {
  try {
    return typeof obj == 'string' ? JSON.parse(obj) : obj || (isNull ? undefined : {});
  } catch (e) {
    return obj || (isNull ? undefined : {});
  }
};
const getFilterKey = prefix => `${prefix || 'currentFilter'}_${(window.location.pathname || "").split("/").join("_")}`;
const getSortKey = prefix => `${prefix || 'currentSort'}_${(window.location.pathname || "").split("/").join("_")}`;
const getCurrentFilter = () => getParseJSON(window.localStorage.getItem(getFilterKey()));
const setCurrentFilter = data => {
  data && window.localStorage.setItem(getFilterKey(), JSON.stringify(data));
};
const getCustomKeyData = (key, isArray) => getParseJSON(window.localStorage.getItem(getFilterKey(key)), isArray);
const setCustomKeyData = (key, data) => {
  data && window.localStorage.setItem(getFilterKey(key), JSON.stringify(data));
};
const getCurrentSort = () => getParseJSON(window.localStorage.getItem(getSortKey()));
const setCurrentSort = data => {
  data && window.localStorage.setItem(getSortKey(), JSON.stringify(data));
};
const clearCurrentFilter = () => window.localStorage.removeItem(getFilterKey());
const clearCurrentSort = () => window.localStorage.removeItem(getSortKey());
const clearAllFilters = () => () => __awaiter$1(void 0, void 0, void 0, function* () {
  Object.keys(window.localStorage).forEach(key => {
    if (key.includes("currentFilter_") || key.includes("currentSort_")) {
      window.localStorage.removeItem(key);
    }
  });
});
var index$l = common$1.reducer;

var index$m = /*#__PURE__*/Object.freeze({
  __proto__: null,
  setNotification: setNotification,
  setNotificationTimeout: setNotificationTimeout,
  startLoading: startLoading,
  stopLoading: stopLoading,
  setError: setError,
  closePopup: closePopup,
  clearError: clearError,
  closePopupScreen: closePopupScreen,
  selectCurrentNotification: selectCurrentNotification,
  showNotificationMessage: showNotificationMessage,
  closeNotification: closeNotification,
  showPopup: showPopup,
  showPopupScreen: showPopupScreen,
  getParseJSON: getParseJSON,
  getCurrentFilter: getCurrentFilter,
  setCurrentFilter: setCurrentFilter,
  getCustomKeyData: getCustomKeyData,
  setCustomKeyData: setCustomKeyData,
  getCurrentSort: getCurrentSort,
  setCurrentSort: setCurrentSort,
  clearCurrentFilter: clearCurrentFilter,
  clearCurrentSort: clearCurrentSort,
  clearAllFilters: clearAllFilters,
  'default': index$l
});

const {
  queryString,
  apiBridge,
  object,
  common,
  processor
} = utils;
const {
  query
} = queryString;
const getSystem = () => {
  return {
    query: query.get(),
    currentUrl: getWindow().location.pathname
  };
};
const initialState$6 = {
  pageData: {},
  userData: Object.assign({}, getSystem()),
  metaData: {},
  protectedData: {},
  isContentLoading: false
};
const parseCustomModule = text => {
  const moduleName = text.indexOf('(') > -1 ? text.substr(0, text.indexOf('(')) : text;
  const params = {};
  const fnMatch = text.match(/[(].*[)]/);
  if (fnMatch) {
    const str = fnMatch[0].substr(1, fnMatch[0].length - 2);
    const arrayParams = str.split(',');
    arrayParams.forEach(itemParam => {
      var _a;
      const askVal = itemParam.trim();
      const arr = [askVal.substr(0, askVal.indexOf(':')), askVal.substr(askVal.indexOf(':') + 1).trim()];
      if (arr[0]) {
        params[arr[0]] = ((_a = arr[1]) === null || _a === void 0 ? void 0 : _a.trim()) || '';
      }
    });
  }
  return {
    module: moduleName,
    params
  };
};
const processEachParam = (userData, key, defaultValue, state) => {
  let value;
  if (key && key.toString().substr(0, 2) === '::') {
    const moduleName = key.toString().split('::');
    const passedKey = key.substr(key.toString().lastIndexOf('::') + 2, key.length - 4).trim();
    const parsedModule = parseCustomModule(moduleName[1]);
    if (passedKey.substr(0, 1) === '.') {
      value = object.getDataFromKey(userData, passedKey.substr(1), defaultValue);
    } else {
      value = passedKey;
    }
    if (parsedModule) {
      value = processor.execute(parsedModule.module, value, parsedModule.params, {
        state,
        userData
      });
    }
  } else if (key && key.toString().substr(0, 1) === '.') {
    value = object.getDataFromKey(userData, key.substr(1), defaultValue);
  } else if (!common.isNullOrUndefined(key)) {
    value = key;
  }
  return value;
};
const processParams = (userData, params = {}, defaultValue = null, state) => {
  let newObj = {};
  Object.keys(params).forEach(key => {
    let value;
    if (typeof params[key] === 'object' && params[key] !== null && params[key].match) {
      const validator = new Validator$1(params[key].match);
      validator.setValues(userData);
      value = validator.validateAll();
    } else if (typeof params[key] === 'object' && params[key] !== null && !Array.isArray(params[key])) {
      value = processParams(userData, params[key], defaultValue, state);
    } else {
      value = processEachParam(userData, params[key], defaultValue, state);
    }
    if (!common.isNullOrUndefined(value)) {
      if (key.startsWith('...') && typeof value === 'object' && !Array.isArray(value)) {
        newObj = Object.assign(Object.assign({}, newObj), value);
      } else {
        newObj[key] = value;
      }
    }
  });
  return newObj;
};
const extractUrlInfo = (url, config) => {
  if (config) {
    let isFound = false;
    let output;
    Object.keys(config).forEach(urlKey => {
      if (!isFound && url.match(new RegExp(urlKey))) {
        isFound = true;
        output = {};
        const foundCOnfig = config[urlKey];
        if (foundCOnfig.url) {
          output.url = foundCOnfig.url;
        }
        if (foundCOnfig.params) {
          output.params = processParams({
            url
          }, foundCOnfig.params);
        }
        if (foundCOnfig.method) {
          output.method = foundCOnfig.method;
        }
      }
    });
    if (output) {
      return output;
    }
  }
  return {};
};
const clearUrl = url => {
  if (url.indexOf('/index.html') > -1) {
    url = url.replace('/index.html', '');
  } else if (url.indexOf('.html') > -1) {
    url = url.replace('.html', '');
  }
  return path.ensureNoSlashAtEnd(url);
};
const fetchJsonPath = ({
  url,
  params,
  headers
}) => {
  var _a, _b, _c, _d, _e;
  const mode = ((_a = getWindow().APP_CONFIG) === null || _a === void 0 ? void 0 : _a.siteMode) === 'static' ? 'get' : 'post';
  url = clearUrl(url);
  const {
    url: overrideUrl,
    params: overrideParams,
    method: overrideMethod
  } = extractUrlInfo(url, (_d = (_c = (_b = getWindow().APP_CONFIG) === null || _b === void 0 ? void 0 : _b.siteMap) === null || _c === void 0 ? void 0 : _c.siteMap) === null || _d === void 0 ? void 0 : _d.dynamicContentConfig);
  const postFix = !overrideUrl && mode === 'get' ? '/get.json' : '';
  const cb = mode === 'get' ? {
    _cb: (_e = getWindow().APP_CONFIG) === null || _e === void 0 ? void 0 : _e.appVersion
  } : {};
  return apiBridge[overrideMethod || mode](`${overrideUrl || url}${postFix}`, Object.assign(Object.assign({}, cb), overrideParams || params), headers);
};
const getIfMatchPagesOnRoute = (path, errorCode) => {
  var _a, _b;
  let filePath = '';
  const siteConfig = (_a = getWindow().APP_CONFIG) === null || _a === void 0 ? void 0 : _a.siteMap;
  siteConfig && ((_b = siteConfig === null || siteConfig === void 0 ? void 0 : siteConfig.siteMap) === null || _b === void 0 ? void 0 : _b.errorRedirects) && Object.keys(siteConfig.siteMap.errorRedirects).forEach(key => {
    if (typeof siteConfig.siteMap.errorRedirects[key] === 'object') {
      if (path.match(key) && siteConfig.siteMap.errorRedirects[key][errorCode]) {
        filePath = siteConfig.siteMap.errorRedirects[key][errorCode];
      }
    }
  });
  return filePath;
};
const fetchContentPage = createAsyncThunk('content/fetchContentPage', url => __awaiter$1(void 0, void 0, void 0, function* () {
  var _a, _b, _c, _d;
  const fullUrl = url || getWindow().location.href;
  let response = yield fetchJsonPath({
    url: fullUrl
  });
  if (response.error) {
    const specificUrl = getIfMatchPagesOnRoute(fullUrl, response.code);
    if (specificUrl) {
      response = yield fetchJsonPath({
        url: specificUrl
      });
    } else if ((_b = (_a = getWindow().APP_CONFIG) === null || _a === void 0 ? void 0 : _a.siteMap.siteMap) === null || _b === void 0 ? void 0 : _b.errorRedirects[response.code]) {
      response = yield fetchJsonPath({
        url: (_d = (_c = getWindow().APP_CONFIG) === null || _c === void 0 ? void 0 : _c.siteMap.siteMap) === null || _d === void 0 ? void 0 : _d.errorRedirects[response.code]
      });
    }
  }
  // The value we return becomes the `fulfilled` action payload
  return {
    data: response.data,
    pageId: url
  };
}));
const sendContact = createAsyncThunk('content/sendContact', data => __awaiter$1(void 0, void 0, void 0, function* () {
  const response = yield apiBridge.post(data.url || '/api/v1/contact/message', data.payload);
  // The value we return becomes the `fulfilled` action payload
  return response;
}));
const updateErrorData = data => dispatch => {
  const errors = {
    lastError: {}
  };
  Object.keys(data).forEach(errorKey => {
    var _a, _b, _c;
    if ((_a = data[errorKey]) === null || _a === void 0 ? void 0 : _a.errors) {
      errors[`${errorKey}_errors`] = (_b = data[errorKey]) === null || _b === void 0 ? void 0 : _b.errors;
    } else if ((_c = data[errorKey]) === null || _c === void 0 ? void 0 : _c.error) {
      errors.lastError[errorKey] = data[errorKey];
    }
  });
  if (data.error === true) {
    errors.lastError = Object.assign(Object.assign({}, errors.lastError), data);
  }
  dispatch(updateUserData(errors));
};
const checkAndPostApi = (data, pageData, prevAction) => dispatch => __awaiter$1(void 0, void 0, void 0, function* () {
  const result = [];
  if (data.bulk && Array.isArray(data.bulk)) {
    data.bulk.forEach(item => {
      result.push(dispatch(postApi(Object.assign(Object.assign({}, item), {
        currentData: prevAction === null || prevAction === void 0 ? void 0 : prevAction.currentData
      }), pageData)));
    });
  } else {
    if (Array.isArray(data)) {
      data.forEach(item => {
        result.push(dispatch(postApi(Object.assign(Object.assign({}, item), {
          currentData: prevAction === null || prevAction === void 0 ? void 0 : prevAction.currentData
        }), pageData)));
      });
    } else {
      if (prevAction === null || prevAction === void 0 ? void 0 : prevAction.currentData) {
        data.currentData = prevAction === null || prevAction === void 0 ? void 0 : prevAction.currentData;
      }
      result.push(dispatch(postApi(data, pageData)));
    }
  }
  return Promise.all(result);
});
const initApplication = data => dispatch => __awaiter$1(void 0, void 0, void 0, function* () {
  var _e, _f;
  const response = {};
  if (data.globals && data.globals.path) {
    const result = yield fetchJsonPath({
      url: data.globals.path
    });
    const pageData = result.data.pageData;
    yield dispatch(updateUserData({
      isSubmitting: false
    }));
    yield dispatch(updateProtectedUserData({
      root: Object.assign(Object.assign({}, data), pageData)
    }));
    if (pageData.init) {
      yield dispatch(mergeUserData(pageData.init));
    }
    if (pageData.merge) {
      yield dispatch(mergeUserData(pageData.merge));
    }
    if ((_e = pageData.hook) === null || _e === void 0 ? void 0 : _e.load) {
      yield dispatch(checkAndPostApi(pageData.hook.load, pageData));
    }
    if ((_f = pageData.hook) === null || _f === void 0 ? void 0 : _f.afterLoad) {
      yield dispatch(checkAndPostApi(pageData.hook.afterLoad, pageData));
    }
  } else {
    yield dispatch(updateProtectedUserData({
      root: Object.assign({}, data)
    }));
  }
  events.emit('application.config.onLoad', data);
  return response;
});
const executeHook = (payload, pageData) => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  let response = {};
  if (payload.preCall) {
    yield dispatch(updateUserData(Object.assign({}, payload.preCall)));
  }
  if (payload.hook) {
    response = yield customHooks.execute(payload.hook, response, {
      state: getState(),
      payload,
      data: {
        params: processParams(getState().content.userData, payload.params, undefined, getState()),
        headers: processParams(getState().content.userData, payload.headers, undefined, getState()),
        query: processParams(getState().content.userData, payload.query, undefined, getState())
      },
      userData: getState().content.userData,
      dispatch,
      getState
    });
  }
  if (payload.postCall) {
    yield dispatch(updateUserData(Object.assign({}, payload.postCall)));
  }
  if (response.status === 'success') {
    const data = payload.dataKey ? {
      [payload.dataKey]: response.data
    } : response.data;
    yield dispatch(updateUserData(Object.assign(Object.assign({}, data), {
      lastError: {}
    })));
  } else if (response.status === 'error') {
    yield dispatch(updateErrorData(response.error));
  }
  if (payload.after) {
    dispatch(checkAndPostApi(payload.after, pageData, payload));
  }
  return response;
});
const postApi = (payload, pageResponse) => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  var _g, _h, _j, _k, _l, _m, _o, _p, _q;
  const currentData = payload.currentData || {};
  if (payload.match) {
    const validator = new Validator$1(payload.match);
    validator.setValues(selectUserData(getState()));
    if (!validator.validateAll()) {
      return;
    }
  }
  if (payload.preCall) {
    yield dispatch(mergeUserData(Object.assign({}, payload.preCall)));
  }
  let response = {
    status: 'success',
    data: {}
  };
  if (payload.preHook) {
    response = yield customHooks.execute(payload.preHook, response, {
      state: getState(),
      payload,
      data: {
        params: processParams(Object.assign(Object.assign({}, getState().content.userData), currentData), payload.params, undefined, getState()),
        headers: processParams(Object.assign(Object.assign({}, getState().content.userData), currentData), payload.headers, undefined, getState()),
        query: processParams(Object.assign(Object.assign({}, getState().content.userData), currentData), payload.query, undefined, getState())
      },
      userData: Object.assign(Object.assign({}, getState().content.userData), currentData),
      dispatch,
      getState
    });
  }
  if (payload.method && payload.url) {
    let paramToProcess = {
      method: payload.method,
      url: payload.url
    };
    paramToProcess = processParams(Object.assign(Object.assign({}, getState().content.userData), currentData), paramToProcess, undefined, getState());
    response = yield apiBridge[paramToProcess.method.toLowerCase()](paramToProcess.url, processParams(Object.assign(Object.assign({}, getState().content.userData), currentData), payload.params, undefined, getState()), processParams(Object.assign(Object.assign({}, getState().content.userData), currentData), payload.headers, undefined, getState()), processParams(Object.assign(Object.assign({}, getState().content.userData), currentData), payload.query, undefined, getState()));
  }
  if (payload.postHook) {
    response = yield customHooks.execute(payload.postHook, response, {
      state: getState(),
      payload,
      data: {
        params: processParams(Object.assign(Object.assign({}, getState().content.userData), currentData), payload.params, undefined, getState()),
        headers: processParams(Object.assign(Object.assign({}, getState().content.userData), currentData), payload.headers, undefined, getState()),
        query: processParams(Object.assign(Object.assign({}, getState().content.userData), currentData), payload.query, undefined, getState())
      },
      userData: Object.assign(Object.assign({}, getState().content.userData), currentData),
      dispatch,
      getState
    });
  }
  let finalObj = {};
  if (payload.defaultResponse && payload.defaultResponse[response.status]) {
    if (Array.isArray(payload.defaultResponse[response.status])) {
      let isValid = false;
      payload.defaultResponse[response.status].forEach(item => {
        if (item.match && !isValid) {
          const valid = new Validator$1(item.match);
          valid.setValues(Object.assign(Object.assign({}, getState().content.userData), currentData));
          if (valid.validateAll()) {
            finalObj = item;
            isValid = true;
          }
        }
      });
    } else {
      finalObj = payload.defaultResponse && (payload.defaultResponse[response.status] || payload.defaultResponse.error);
    }
  }
  response = object.extendData(finalObj, response);
  const {
    notification
  } = response || {};
  if (notification) {
    yield dispatch(showNotificationMessage(notification));
  }
  if (payload.postCall) {
    yield dispatch(mergeUserData(Object.assign({}, payload.postCall)));
  }
  if (response.status === 'success') {
    const data = payload.dataKey ? {
      [payload.dataKey]: response.data
    } : response.data;
    if (payload.saveType === 'protected') {
      yield dispatch(updateProtectedUserData(Object.assign({}, data)));
    } else {
      yield dispatch(updateUserData(Object.assign(Object.assign({}, data), {
        lastError: {}
      })));
    }
    if ((_g = payload === null || payload === void 0 ? void 0 : payload.finally) === null || _g === void 0 ? void 0 : _g.successAction) {
      if (Array.isArray((_h = payload === null || payload === void 0 ? void 0 : payload.finally) === null || _h === void 0 ? void 0 : _h.successAction)) {
        (_j = payload === null || payload === void 0 ? void 0 : payload.finally) === null || _j === void 0 ? void 0 : _j.successAction.forEach(actionItem => {
          let isValid = true;
          if (actionItem.match) {
            const vldtr = new Validator$1(actionItem.match);
            vldtr.setValues(Object.assign(Object.assign({}, getState().content.userData), currentData));
            isValid = vldtr.validateAll();
          }
          isValid && events.emit('dynamicContent.onAction', {}, Object.assign(Object.assign({}, actionItem), {
            currentData
          }), {});
        });
      } else {
        events.emit('dynamicContent.onAction', {}, Object.assign(Object.assign({}, (_k = payload === null || payload === void 0 ? void 0 : payload.finally) === null || _k === void 0 ? void 0 : _k.successAction), {
          currentData
        }), {});
      }
    }
    if (payload.successAfterScript) {
      browser.scriptManager.insertDynamicScript(payload.successAfterScript, 'body');
    }
  } else if (response.status === 'error') {
    yield dispatch(updateErrorData(response.error));
    if ((_l = payload.finally) === null || _l === void 0 ? void 0 : _l.errorAction) {
      if (Array.isArray(payload.finally.errorAction)) {
        (_m = payload === null || payload === void 0 ? void 0 : payload.finally) === null || _m === void 0 ? void 0 : _m.errorAction.forEach(actionItem => {
          let isValid = true;
          if (actionItem.match) {
            const vldtr = new Validator$1(actionItem.match);
            vldtr.setValues(Object.assign(Object.assign({}, getState().content.userData), currentData));
            isValid = vldtr.validateAll();
          }
          isValid && events.emit('dynamicContent.onAction', {}, Object.assign(Object.assign({}, actionItem), {
            currentData
          }), {});
        });
      } else {
        events.emit('dynamicContent.onAction', {}, Object.assign(Object.assign({}, (_o = payload === null || payload === void 0 ? void 0 : payload.finally) === null || _o === void 0 ? void 0 : _o.errorAction), {
          currentData
        }), {});
      }
    }
    if (payload.errorAfterScript) {
      browser.scriptManager.insertDynamicScript(payload.errorAfterScript, 'body');
    }
  }
  if (payload.runInit) {
    dispatch(mergeUserData((_p = pageResponse === null || pageResponse === void 0 ? void 0 : pageResponse.pageData) === null || _p === void 0 ? void 0 : _p.init));
  }
  if (payload.runMerge) {
    dispatch(mergeUserData((_q = pageResponse === null || pageResponse === void 0 ? void 0 : pageResponse.pageData) === null || _q === void 0 ? void 0 : _q.merge));
  }
  if (payload.after) {
    dispatch(checkAndPostApi(payload.after, pageResponse, payload));
  }
  return response;
});
const uploadApi = (payload, pageResponse) => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  var _r, _s;
  if (payload.match) {
    const validator = new Validator$1(payload.match);
    validator.setValues(selectUserData(getState()));
    if (!validator.validateAll()) {
      return;
    }
  }
  if (payload.preCall) {
    yield dispatch(mergeUserData(Object.assign({}, payload.preCall)));
  }
  let response = {
    status: 'success',
    data: {}
  };
  if (payload.preHook) {
    response = yield customHooks.execute(payload.preHook, response, {
      state: getState(),
      payload,
      data: {
        params: processParams(getState().content.userData, payload.params, undefined, getState()),
        headers: processParams(getState().content.userData, payload.headers, undefined, getState()),
        query: processParams(getState().content.userData, payload.query, undefined, getState())
      },
      userData: getState().content.userData,
      dispatch,
      getState
    });
  }
  if (payload.url) {
    let paramToProcess = {
      method: payload.method,
      url: payload.url
    };
    paramToProcess = processParams(getState().content.userData, paramToProcess, undefined, getState());
    const formData = new FormData();
    payload.data.files.forEach(file => {
      formData.append('file', file);
    });
    response = yield apiBridge$1.rawPost(paramToProcess.url, formData, processParams(getState().content.userData, payload.headers, undefined, getState()), Object.assign({
      fileName: (payload.data.fileNames || payload.data.files).map(i => i.name)
    }, processParams(getState().content.userData, payload.params, undefined, getState())));
  }
  if (payload.postHook) {
    response = yield customHooks.execute(payload.postHook, response, {
      state: getState(),
      payload,
      data: {
        params: processParams(getState().content.userData, payload.params, undefined, getState()),
        headers: processParams(getState().content.userData, payload.headers, undefined, getState()),
        query: processParams(getState().content.userData, payload.query, undefined, getState())
      },
      userData: getState().content.userData,
      dispatch,
      getState
    });
  }
  let finalObj = {};
  if (payload.defaultResponse && payload.defaultResponse[response.status]) {
    if (Array.isArray(payload.defaultResponse[response.status])) {
      console.log(payload.defaultResponse);
      let isValid = false;
      payload.defaultResponse[response.status].forEach(item => {
        if (item.match && !isValid) {
          const valid = new Validator$1(item.match);
          valid.setValues(getState().content.userData);
          if (valid.validateAll()) {
            finalObj = item;
            isValid = true;
          }
        }
      });
    } else {
      finalObj = payload.defaultResponse && (payload.defaultResponse[response.status] || payload.defaultResponse.error);
    }
  }
  response = object.extendData(finalObj, response);
  const {
    notification
  } = response || {};
  if (notification) {
    yield dispatch(showNotificationMessage(notification));
  }
  if (payload.postCall) {
    yield dispatch(mergeUserData(Object.assign({}, payload.postCall)));
  }
  if (response.status === 'success') {
    const data = payload.dataKey ? {
      [payload.dataKey]: response.data
    } : response.data;
    payload.success && payload.success(response.data);
    if (payload.saveType === 'protected') {
      yield dispatch(updateProtectedUserData(Object.assign({}, data)));
    } else {
      yield dispatch(updateUserData(Object.assign(Object.assign({}, data), {
        lastError: {}
      })));
    }
  } else if (response.status === 'error') {
    payload.failed && payload.failed(response.error);
    yield dispatch(updateErrorData(response.error));
  }
  if (payload.runInit) {
    dispatch(mergeUserData((_r = pageResponse === null || pageResponse === void 0 ? void 0 : pageResponse.pageData) === null || _r === void 0 ? void 0 : _r.init));
  }
  if (payload.runMerge) {
    dispatch(mergeUserData((_s = pageResponse === null || pageResponse === void 0 ? void 0 : pageResponse.pageData) === null || _s === void 0 ? void 0 : _s.merge));
  }
  if (payload.after) {
    dispatch(checkAndPostApi(payload.after, pageResponse, payload));
  }
  return response;
});
const downloadApi = (payload, pageResponse) => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  var _t, _u;
  if (payload.match) {
    const validator = new Validator$1(payload.match);
    validator.setValues(selectUserData(getState()));
    if (!validator.validateAll()) {
      return;
    }
  }
  if (payload.preCall) {
    yield dispatch(mergeUserData(Object.assign({}, payload.preCall)));
  }
  let response = {
    status: 'success',
    data: {}
  };
  if (payload.preHook) {
    response = yield customHooks.execute(payload.preHook, response, {
      state: getState(),
      payload,
      data: {
        params: processParams(getState().content.userData, payload.params, undefined, getState()),
        headers: processParams(getState().content.userData, payload.headers, undefined, getState()),
        query: processParams(getState().content.userData, payload.query, undefined, getState())
      },
      userData: getState().content.userData,
      dispatch,
      getState
    });
  }
  if (payload.href || payload.url) {
    let responseStatus;
    let paramToProcess = {
      method: payload.method,
      url: payload.url,
      href: payload.href
    };
    paramToProcess = processParams(getState().content.userData, paramToProcess, undefined, getState());
    const method = paramToProcess.method || 'get';
    yield apiBridge[method](paramToProcess.href || paramToProcess.url, processParams(getState().content.userData, payload.params, undefined, getState()), processParams(getState().content.userData, payload.headers, undefined, getState()), processParams(getState().content.userData, payload.query, undefined, getState()), {
      plain: true
    }).then(res => {
      responseStatus = res.status;
      return res.blob();
    }).then(data => {
      if (responseStatus === 200) {
        const blob = new Blob([data], {
          type: 'application/octetstream'
        });
        const url = getWindow().URL || getWindow().webkitURL;
        const link = url.createObjectURL(blob);
        const a = document.createElement('a');
        a.href = link;
        const newurl = payload.href || payload.url;
        const strip = newurl.indexOf('?') > -1 ? newurl.indexOf('?') : newurl.length;
        const exactUrl = newurl.substr(0, strip);
        const fileName = exactUrl.substr(exactUrl.lastIndexOf('/') + 1) || 'no-name';
        a.download = payload.fileName || fileName; // this should be the file name with the required extension
        document.body.appendChild(a);
        a.click();
        a.remove();
        getWindow().URL.revokeObjectURL(link);
      } else {
        response.status = 'error';
        response.error = {};
      }
    });
  }
  if (payload.postHook) {
    response = yield customHooks.execute(payload.postHook, response, {
      state: getState(),
      payload,
      data: {
        params: processParams(getState().content.userData, payload.params, undefined, getState()),
        headers: processParams(getState().content.userData, payload.headers, undefined, getState()),
        query: processParams(getState().content.userData, payload.query, undefined, getState())
      },
      userData: getState().content.userData,
      dispatch,
      getState
    });
  }
  let finalObj = {};
  if (payload.defaultResponse && payload.defaultResponse[response.status]) {
    if (Array.isArray(payload.defaultResponse[response.status])) {
      console.log(payload.defaultResponse);
      let isValid = false;
      payload.defaultResponse[response.status].forEach(item => {
        if (item.match && !isValid) {
          const valid = new Validator$1(item.match);
          valid.setValues(getState().content.userData);
          if (valid.validateAll()) {
            finalObj = item;
            isValid = true;
          }
        }
      });
    } else {
      finalObj = payload.defaultResponse && (payload.defaultResponse[response.status] || payload.defaultResponse.error);
    }
  }
  response = object.extendData(finalObj, response);
  const {
    notification
  } = response || {};
  if (notification) {
    yield dispatch(showNotificationMessage(notification));
  }
  if (payload.postCall) {
    yield dispatch(mergeUserData(Object.assign({}, payload.postCall)));
  }
  if (response.status === 'success') {
    const data = payload.dataKey ? {
      [payload.dataKey]: response.data
    } : response.data;
    yield dispatch(updateUserData(Object.assign(Object.assign({}, data), {
      lastError: {}
    })));
  } else if (response.status === 'error') {
    yield dispatch(updateErrorData(response.error));
  }
  if (payload.runInit) {
    dispatch(mergeUserData((_t = pageResponse === null || pageResponse === void 0 ? void 0 : pageResponse.pageData) === null || _t === void 0 ? void 0 : _t.init));
  }
  if (payload.runMerge) {
    dispatch(mergeUserData((_u = pageResponse === null || pageResponse === void 0 ? void 0 : pageResponse.pageData) === null || _u === void 0 ? void 0 : _u.merge));
  }
  if (payload.after) {
    dispatch(checkAndPostApi(payload.after, pageResponse, payload));
  }
  return response;
});
const content = createSlice({
  name: 'content',
  initialState: initialState$6,
  reducers: {
    updateMetaData: (state, action) => {
      var _a;
      state.metaData = {
        prev: (_a = state.metaData) === null || _a === void 0 ? void 0 : _a.latest,
        latest: Object.assign({
          url: action.payload.url
        }, action.payload.data)
      };
    },
    updateProtectedUserData: (state, action) => {
      state.protectedData = Object.assign(Object.assign({}, state.protectedData), action.payload);
      state.userData = Object.assign(Object.assign(Object.assign({}, state.userData), action.payload), getSystem());
    },
    clearAllUserData: state => {
      state.userData = Object.assign(Object.assign({}, state.protectedData), getSystem());
    },
    clearWithFilter: (state, action) => {
      const data = {};
      if (action.payload.keys) {
        action.payload.keys.forEach(key => {
          data[key] = undefined;
        });
      }
      state.userData = Object.assign(Object.assign(Object.assign(Object.assign({}, state.userData), data), state.protectedData), getSystem());
    },
    updateUserData: (state, action) => {
      state.userData = Object.assign(Object.assign(Object.assign({}, state.userData), action.payload), getSystem());
    }
  }
});
const resetLastError = () => dispatch => {
  dispatch(updateUserData({
    lastError: {}
  }));
};
const selectRootData = state => {
  return state.content.protectedData.root || {};
};
const selectUserData = state => {
  return state.content.userData;
};
const mergeUserData = payload => (dispatch, getState) => {
  const root = selectRootData(getState());
  const data = processParams(getState().content.userData, Object.assign(Object.assign({}, root.merge), payload), '', getState());
  const merged = object.extendData(getState().content.userData, data);
  dispatch(updateUserData(merged));
};
const resetUserData = payload => (dispatch, getState) => {
  switch (payload.type) {
    case 'clearAll':
      dispatch(content.actions.clearAllUserData());
      break;
    case 'keys':
      dispatch(content.actions.clearWithFilter({
        keys: payload.keys
      }));
      break;
    case 'except':
      dispatch(content.actions.clearWithFilter({
        except: payload.except
      }));
      break;
  }
};
const {
  updateProtectedUserData,
  updateUserData,
  clearAllUserData,
  clearWithFilter,
  updateMetaData
} = content.actions;
var index$j = content.reducer;

var index$k = /*#__PURE__*/Object.freeze({
  __proto__: null,
  parseCustomModule: parseCustomModule,
  processEachParam: processEachParam,
  processParams: processParams,
  fetchJsonPath: fetchJsonPath,
  fetchContentPage: fetchContentPage,
  sendContact: sendContact,
  updateErrorData: updateErrorData,
  checkAndPostApi: checkAndPostApi,
  initApplication: initApplication,
  executeHook: executeHook,
  postApi: postApi,
  uploadApi: uploadApi,
  downloadApi: downloadApi,
  resetLastError: resetLastError,
  selectRootData: selectRootData,
  selectUserData: selectUserData,
  mergeUserData: mergeUserData,
  resetUserData: resetUserData,
  updateProtectedUserData: updateProtectedUserData,
  updateUserData: updateUserData,
  clearAllUserData: clearAllUserData,
  clearWithFilter: clearWithFilter,
  updateMetaData: updateMetaData,
  customHooks: customHooks,
  'default': index$j
});

var config = {
  globals: {
    path: '/content/admin/root'
  },
  urls: {
    protected: ['content/admin/(?!(login|register|forgotpassword)$).*$']
  },
  apis: {
    user: '/api/v1/admin/user',
    uploadMedia: '/api/v1/admin/media',
    getPropertyList: '/api/v1/admin/media',
    getUsersByRole: '/api/v1/admin/media',
    getAllPermissions: '/api/v1/admin/permission',
    savePermissionByRole: '/api/v1/admin/permission/save',
    searchPermissionsMapping: '/api/v1/admin/permission/map/search',
    getMedia: '/api/v1/admin/media',
    updateMedia: '/api/v1/admin/media',
    searchMedia: '/api/v1/admin/media/search',
    deleteMedia: '/api/v1/admin/media',
    userInfo: '/api/v1/admin/user/info',
    userRoles: '/api/v1/admin/role',
    searchRole: '/api/v1/admin/role/search',
    roleMapping: '/api/v1/admin/role/mapping',
    getUsers: '/api/v1/admin/users/all',
    searchUsers: '/api/v1/admin/users/search',
    searchEmailTemplates: '/api/v1/admin/emailtemplate/search',
    getPage: '/api/v1/admin/content/page/get',
    getFieldsMeta: '/api/v1/admin/content/page/getfieldsmeta',
    getPageByPath: '/api/v1/admin/content/page/getbypath',
    getContentTree: '/api/v1/admin/content/page/gettree',
    contentPage: '/api/v1/admin/content/page',
    clonePage: '/api/v1/admin/content/page/clone'
  },
  urlMapping: {
    login: '/content/admin/login',
    register: '/content/admin/register',
    adminDashboard: '/content/admin/portal/dashboard',
    createNewEmailTemplate: '/content/admin/portal/emailtemplates/new',
    usersList: '/content/admin/portal/users',
    createNewUser: '/content/admin/portal/users/create',
    editEmailTemplate: '/content/admin/portal/emailtemplates/edit',
    editUser: '/content/admin/portal/users/edit',
    emailTemplatelList: '/content/admin/portal/emailtemplates',
    editPage: '/content/admin/portal/pages/edit',
    editSiteMap: '/content/admin/portal/pages/editmap',
    addNewPage: '/content/admin/portal/pages/create',
    previewDraftPage: '/content/admin/portal/pages/previewdraft',
    '/content/admin(.*)': {
      type: 'regex',
      target: '/content/admin/$1'
    }
  }
};

const initialState$5 = {
  contentData: {
    pageData: {
      title: 'Untitled Page'
    }
  }
};
const admin = createSlice({
  name: 'admin',
  initialState: initialState$5,
  reducers: {
    setMedia: (state, action) => {
      state.media = action.payload;
    },
    setMediaPage: (state, action) => {
      state.mediaPage = action.payload;
    },
    setContentPage: (state, action) => {
      state.contentData = action.payload;
    },
    setContentPages: (state, action) => {
      state.contentPages = action.payload;
    },
    setContentPagination: (state, action) => {
      state.contentPagesPagination = action.payload;
    },
    setFieldsMeta: (state, action) => {
      state.fieldsMeta = action.payload;
    },
    setContentTree: (state, action) => {
      state.contentTree = action.payload;
    },
    setRoot: (state, action) => {
      state.root = action.payload;
    }
  }
});
const getPage = (payload, {
  url,
  method: _method = 'post',
  params,
  hook,
  query
} = {}) => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  const result = yield apiBridge$1[_method](url || config.apis.getPage, params ? processParams(Object.assign(Object.assign({}, selectContentData(getState())), selectUserData(getState())), params, undefined, getState()) : payload, undefined, processParams(Object.assign(Object.assign({}, selectContentData(getState())), selectUserData(getState())), query, undefined, getState()));
  if (result.status === CONSTANTS.STATUS.SUCCESS) {
    if (hook) {
      result.data = yield customHooks.execute(hook, result);
    }
    yield dispatch(setContentPage(result.data));
  }
});
const createPage = () => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  yield dispatch(setContentPage(initialState$5.contentData));
});
const getFieldsMeta = (payload, {
  url,
  method: _method2 = 'post',
  params,
  query,
  hook
} = {}) => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  if (url && _method2) {
    const result = yield apiBridge$1[_method2](url, params ? processParams(Object.assign(Object.assign({}, selectContentData(getState())), selectUserData(getState())), params, undefined, getState()) : payload, undefined, processParams(Object.assign(Object.assign({}, selectContentData(getState())), selectUserData(getState())), query, undefined, getState()));
    if (result.status === CONSTANTS.STATUS.SUCCESS) {
      if (hook) {
        result.data = yield customHooks.execute(hook, result);
      }
      yield dispatch(setFieldsMeta(result.data));
    }
  }
});
const loadPageTree = (payload, {
  url,
  method: _method3 = 'post',
  params,
  query
} = {}) => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  const result = yield apiBridge$1[_method3](url || config.apis.getContentTree, params ? processParams(Object.assign(Object.assign(Object.assign({}, selectContentData(getState())), selectUserData(getState())), payload), params, undefined, getState()) : payload, undefined, processParams(Object.assign(Object.assign(Object.assign({}, selectContentData(getState())), selectUserData(getState())), payload), query, undefined, getState()));
  if (result.status === CONSTANTS.STATUS.SUCCESS) {
    yield dispatch(setContentTree(result.data));
  }
});
const loadPagesByPath = (payload, {
  url,
  method: _method4 = 'post',
  params,
  hook,
  query
} = {}, query2) => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  var _a, _b;
  const result = yield apiBridge$1[_method4](url || config.apis.getPageByPath, params ? processParams(Object.assign(Object.assign(Object.assign({}, selectContentData(getState())), selectUserData(getState())), payload), params, undefined, getState()) : payload, undefined, Object.assign(Object.assign({}, processParams(Object.assign(Object.assign(Object.assign({}, selectContentData(getState())), selectUserData(getState())), payload), query, undefined, getState())), query2));
  if (result.status === CONSTANTS.STATUS.SUCCESS) {
    if (hook) {
      result.data = yield customHooks.execute(hook, result);
    }
    const pageData = {
      total: ((_a = result.data) === null || _a === void 0 ? void 0 : _a.total) || ((_b = result.data) === null || _b === void 0 ? void 0 : _b.totalItems),
      pageSize: result.data.pageSize,
      currentPage: result.data.currentPage,
      isLast: result.data.last,
      totalPages: result.data.totalPages
    };
    yield dispatch(setContentPages(result.data.pages));
    yield dispatch(setContentPagination(pageData));
  }
});
const copyMediaLink = payload => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  navigator.clipboard.writeText(payload);
  dispatch(showNotificationMessage({
    message: 'Media link has been copied.',
    type: 'success'
  }));
});
const deleteLink = (payload, {
  url,
  params,
  method: _method5 = 'delete',
  hook
} = {}) => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  const result = yield apiBridge$1[_method5](url || config.apis.deleteMedia, params ? processParams(Object.assign(Object.assign(Object.assign({}, selectContentData(getState())), selectUserData(getState())), payload), params, undefined, getState()) : payload);
  if (result.status === CONSTANTS.STATUS.SUCCESS) {
    if (hook) {
      result.data = yield customHooks.execute(hook, result);
    }
    yield dispatch(showNotificationMessage({
      message: 'Media link has been deleted.',
      type: 'success'
    }));
    return;
  }
});
const updateMedia = (payload, {
  url,
  params,
  method: _method6 = 'patch',
  hook
} = {}) => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  const result = yield apiBridge$1[_method6](url || config.apis.updateMedia, params ? processParams(Object.assign(Object.assign(Object.assign({}, selectContentData(getState())), selectUserData(getState())), payload), params, undefined, getState()) : payload);
  if (result.status === CONSTANTS.STATUS.SUCCESS) {
    if (hook) {
      result.data = yield customHooks.execute(hook, result);
    }
    yield dispatch(showNotificationMessage({
      message: 'Media updated successfully.',
      type: 'success'
    }));
    return;
  }
});
const uploadMedia = (payload, {
  url,
  params,
  hook
} = {}) => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  if (payload.data) {
    const formData = new FormData();
    payload.data.files.forEach(file => {
      formData.append('file', file);
    });
    const fileName = (payload.data.fileNames || payload.data.files).map(i => i.name);
    const contentType = {
      contentType: 'VIDEO',
      status: 'DRAFT'
    };
    if (!fileName.includes('.mp4')) {
      delete contentType['contentType'];
    }
    const result = yield apiBridge$1.rawPost(url || config.apis.uploadMedia, formData, {}, Object.assign(Object.assign({
      fileName: fileName
    }, params ? processParams(Object.assign(Object.assign({}, selectContentData(getState())), selectUserData(getState())), params, undefined, getState()) : {}), contentType));
    if (result.status === CONSTANTS.STATUS.SUCCESS) {
      if (hook) {
        result.data = yield customHooks.execute(hook, result);
      }
      dispatch(showNotificationMessage({
        message: 'Uploaded media successfully'
      }));
      payload.success(result.data.files);
    } else {
      dispatch(showNotificationMessage({
        message: 'Upload media failed',
        type: 'error'
      }));
    }
  }
});
const loadMedia = ({
  body,
  query
} = {}, {
  url,
  method: _method7 = 'post',
  params,
  hook
} = {}) => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  const result = yield apiBridge$1[_method7](url || config.apis.searchMedia, params ? processParams(Object.assign(Object.assign(Object.assign({}, selectContentData(getState())), selectUserData(getState())), body), params, undefined, getState()) : body, undefined, processParams(Object.assign(Object.assign({}, selectContentData(getState())), selectUserData(getState())), query, undefined, getState()));
  if (result.status === CONSTANTS.STATUS.SUCCESS) {
    if (hook) {
      result.data = yield customHooks.execute(hook, result);
    }
    const pageData = {
      pageSize: result.data.pageSize,
      currentPage: result.data.currentPage,
      isFirst: result.data.first,
      isLast: result.data.last,
      totalPages: result.data.totalPages
    };
    yield dispatch(setMedia(result.data.data));
    yield dispatch(setMediaPage(pageData));
  }
});
const savePageDraft = (payload, {
  url,
  method: _method8 = 'patch',
  params,
  hook,
  autoSave,
  query
} = {}) => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  const result = yield apiBridge$1[_method8](url || config.apis.contentPage, params ? processParams(Object.assign(Object.assign(Object.assign({}, selectContentData(getState())), selectUserData(getState())), payload), params, undefined, getState()) : payload, undefined, processParams(Object.assign(Object.assign({}, selectContentData(getState())), selectUserData(getState())), query, undefined, getState()));
  if (result.status === CONSTANTS.STATUS.SUCCESS) {
    !autoSave && dispatch(showNotificationMessage({
      message: 'Page saved successfully'
    }));
    if (hook) {
      result.data = yield customHooks.execute(hook, result);
    }
    yield dispatch(setContentPage(result.data));
  }
});
const deletePage = (payload, {
  url,
  method: _method9 = 'delete',
  params,
  hook,
  query
} = {}) => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  const result = yield apiBridge$1[_method9](url || config.apis.contentPage, params ? processParams(Object.assign(Object.assign(Object.assign({}, selectContentData(getState())), selectUserData(getState())), payload), params, undefined, getState()) : {
    uid: payload.uid
  }, undefined, processParams(Object.assign(Object.assign({}, selectContentData(getState())), selectUserData(getState())), query, undefined, getState()));
  if (result.status === CONSTANTS.STATUS.SUCCESS) {
    dispatch(showNotificationMessage({
      message: 'Page deleted successfully'
    }));
    if (hook) {
      result.data = yield customHooks.execute(hook, result);
    }
    yield dispatch(setContentPage(result.data));
  }
});
const clonePage = (payload, {
  url,
  method: _method10 = 'post',
  params,
  hook,
  query
} = {}, {
  success,
  error
} = {}) => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  const result = yield apiBridge$1[_method10](url || config.apis.clonePage, params ? processParams(Object.assign(Object.assign(Object.assign({}, selectContentData(getState())), selectUserData(getState())), payload), params, undefined, getState()) : {
    from: payload.from,
    to: payload.to,
    type: payload.type
  }, undefined, processParams(Object.assign(Object.assign({}, selectContentData(getState())), selectUserData(getState())), query, undefined, getState()));
  if (result.status === CONSTANTS.STATUS.SUCCESS) {
    dispatch(showNotificationMessage({
      message: 'Page cloned successfully'
    }));
    if (hook) {
      result.data = yield customHooks.execute(hook, result);
    }
    yield dispatch(setContentPage(result.data));
    success && success(result);
  } else {
    error && error(result);
  }
});
const selectContentData = state => {
  return state.admin.contentData;
};
customHooks.add('admin', {
  afterPageCreate: (result, {
    dispatch
  }) => {
    if (result.status === CONSTANTS.STATUS.SUCCESS) {
      switch (result.data.type) {
        case CONSTANTS.contentType.PAGE:
          redirectTo('editPage', {
            path: result.data.path
          });
          break;
        case CONSTANTS.contentType.SITE_MAP:
          redirectTo('editSiteMap', {
            path: result.data.path
          });
          break;
      }
    }
    return result;
  }
});
const {
  setRoot,
  setContentPage,
  setContentTree,
  setContentPages,
  setContentPagination,
  setFieldsMeta,
  setMedia,
  setMediaPage
} = admin.actions;
var index$h = admin.reducer;

var index$i = /*#__PURE__*/Object.freeze({
  __proto__: null,
  getPage: getPage,
  createPage: createPage,
  getFieldsMeta: getFieldsMeta,
  loadPageTree: loadPageTree,
  loadPagesByPath: loadPagesByPath,
  copyMediaLink: copyMediaLink,
  deleteLink: deleteLink,
  updateMedia: updateMedia,
  uploadMedia: uploadMedia,
  loadMedia: loadMedia,
  savePageDraft: savePageDraft,
  deletePage: deletePage,
  clonePage: clonePage,
  selectContentData: selectContentData,
  setRoot: setRoot,
  setContentPage: setContentPage,
  setContentTree: setContentTree,
  setContentPages: setContentPages,
  setContentPagination: setContentPagination,
  setFieldsMeta: setFieldsMeta,
  setMedia: setMedia,
  setMediaPage: setMediaPage,
  'default': index$h
});

const initialState$4 = {
  currentUser: null
};
const authentication = createSlice({
  name: 'authentication',
  initialState: initialState$4,
  reducers: {
    setUser: (state, action) => {
      state.currentUser = action.payload;
    },
    setUsers: (state, action) => {
      state.users = action.payload;
    },
    setAllRoles: (state, action) => {
      state.allRoles = action.payload;
    },
    setAllRolesPages: (state, action) => {
      state.allRolesPages = action.payload;
    },
    setRoles: (state, action) => {
      state.roles = action.payload;
    },
    setPermissions: (state, action) => {
      state.permissions = action.payload;
    },
    setCurrentRole: (state, action) => {
      state.currentRole = action.payload;
    },
    clearAddUserError: state => {
      state.currentUserError = {};
    },
    updateCurrentRolePermissions: (state, action) => {
      state.currentRole.allPermissions = Object.assign(Object.assign({}, state.currentRole.allPermissions), {
        [action.payload.key]: action.payload.data
      });
    },
    setRoleUsers: (state, action) => {
      state.currentRoleUsers = action.payload;
    },
    setAddUserError: (state, action) => {
      state.currentUserError = Object.assign({
        error: true
      }, action.payload);
    },
    setUserPagination: (state, action) => {
      state.usersPagination = action.payload;
    },
    setPreference: (state, action) => {
      state.currentUserPreference = action.payload;
    }
  }
});
customHooks.add('authentication', {
  login: (response, {
    payload,
    dispatch,
    state
  }) => __awaiter$1(void 0, void 0, void 0, function* () {
    if (response.status === CONSTANTS.STATUS.SUCCESS) {
      if (response.data.token) {
        cookieMgr.add('qjs-token', response.data.token);
        apiBridge$1.addHeader('qjs-token', `${response.data.token}`);
      }
      yield dispatch(setUser(response.data));
      yield dispatch(updateProtectedUserData({
        currentUser: response.data
      }));
      yield dispatch(loadUserProfile());
      const returnUrl = query$1.get().returnUrl;
      if (returnUrl) {
        redirectTo(returnUrl);
        return response;
      }
      redirectTo('adminDashboard');
    }
    return response;
  }),
  loadUserProfile: (response, {
    dispatch
  }) => __awaiter$1(void 0, void 0, void 0, function* () {
    dispatch(loadUserProfile());
  })
});
const loadUserProfile = userProfile => dispatch => __awaiter$1(void 0, void 0, void 0, function* () {
  // console.log(getState().content.userData);
  const response = yield apiBridge$1[(userProfile === null || userProfile === void 0 ? void 0 : userProfile.method) || 'get']((userProfile === null || userProfile === void 0 ? void 0 : userProfile.url) || config.apis.userInfo, {});
  if (response.status !== CONSTANTS.STATUS.SUCCESS && cookieMgr.get('qjs-token')) {
    cookieMgr.remove('qjs-token');
    yield dispatch(showNotificationMessage({
      message: response.error.errorMessage || 'Unauthorized',
      type: 'error'
    }));
    yield dispatch(clearUser());
    yield dispatch(updateProtectedUserData({
      currentUser: undefined,
      currentUserPreference: undefined
    }));
  } else if (response.status === CONSTANTS.STATUS.SUCCESS) {
    response.data.userInfo.roles = response.data.permissions ? uniq(response.data.permissions.map(a => a.roleCode)) : [];
    response.data.userInfo.allPermissions = response.data.permissions ? response.data.permissions.reduce((a, b) => {
      return a.concat(b.permission);
    }, []).map(a => a.code) : [];
    yield dispatch(setUser(response.data.userInfo));
    yield dispatch(setPreference(response.data.preference));
    yield dispatch(updateProtectedUserData({
      currentUser: response.data.userInfo,
      currentUserPreference: response.data.preference
    }));
  }
});
const clearUser = () => dispatch => {
  cookieMgr.remove('qjs-token');
  apiBridge$1.removeHeader('qjs-token');
  setTimeout(() => {
    dispatch(setUser(null));
    dispatch(updateProtectedUserData({
      currentUser: null,
      currentUserPreference: null
    }));
  }, 200);
};
const loadRoles = (payload = {}, {
  url
} = {}) => dispatch => __awaiter$1(void 0, void 0, void 0, function* () {
  const response = yield apiBridge$1.get(url || config.apis.userRoles, payload);
  if (response.status === CONSTANTS.STATUS.SUCCESS) {
    dispatch(setRoles({
      [GLOBAL_OPTIONS.roleTabs.keys.ALL_ROLES]: response.data
    }));
  }
});
const showAllRoles = (payload, {
  url
}) => dispatch => __awaiter$1(void 0, void 0, void 0, function* () {
  var _a;
  const response = yield apiBridge$1.post(url || config.apis.searchRole, Object.assign({}, payload.body), {}, payload.query);
  if (response.status === CONSTANTS.STATUS.SUCCESS) {
    const data = {
      [GLOBAL_OPTIONS.roleTabs.keys.ALL_ROLES]: response.data
    };
    const pageData = {
      total: (_a = response.data) === null || _a === void 0 ? void 0 : _a.totalItems,
      pageSize: response.data.pageSize,
      currentPage: response.data.currentPage,
      isLast: response.data.last,
      totalPages: response.data.totalPages
    };
    yield dispatch(setAllRoles(data));
    yield dispatch(setAllRolesPages(pageData));
  }
});
const removeUserToRole = (payload, {
  url
} = {}) => dispatch => __awaiter$1(void 0, void 0, void 0, function* () {
  const response = yield apiBridge$1.delete(url || config.apis.roleMapping, payload);
  if (response.status === CONSTANTS.STATUS.SUCCESS) {
    yield dispatch(showNotificationMessage({
      message: 'User removed successfully',
      type: 'success'
    }));
  }
});
const removeRole = (payload, {
  url
} = {}) => dispatch => __awaiter$1(void 0, void 0, void 0, function* () {
  const response = yield apiBridge$1.delete(url || config.apis.userRoles, payload);
  if (response.status === CONSTANTS.STATUS.SUCCESS) {
    yield dispatch(showNotificationMessage({
      message: 'Role removed successfully',
      type: 'success'
    }));
  }
});
const hasPermission = (permission, state) => {
  var _a, _b, _c;
  return ((_c = (_b = (_a = state === null || state === void 0 ? void 0 : state.authentication) === null || _a === void 0 ? void 0 : _a.currentUser) === null || _b === void 0 ? void 0 : _b.allPermissions) === null || _c === void 0 ? void 0 : _c.indexOf(permission)) > -1;
};
const populateFullName = data => {
  return data.map(i => {
    return Object.assign(Object.assign({}, i), {
      fullName: `${i.firstName} ${i.lastName}`,
      displayName: `${i.firstName} ${i.lastName} (${i.emailId})`
    });
  });
};
const loadUsers$1 = (payload, {
  url
} = {}) => dispatch => __awaiter$1(void 0, void 0, void 0, function* () {
  var _b;
  const response = yield apiBridge$1.post(url || config.apis.searchUsers, Object.assign({}, payload.body), {}, payload.query);
  if (response.status === CONSTANTS.STATUS.SUCCESS) {
    const pagination = {
      total: (_b = response.data) === null || _b === void 0 ? void 0 : _b.totalItems,
      pageSize: response.data.pageSize,
      currentPage: response.data.currentPage,
      isLast: response.data.last,
      totalPages: response.data.totalPages
    };
    response.data.data = populateFullName(response.data.data);
    yield dispatch(setUsers$1(response.data.data));
    yield dispatch(setUserPagination(pagination));
  }
});
const loadUsersByRole = (payload, {
  url
} = {}) => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  const response = yield apiBridge$1.get(url || config.apis.getUsersByRole, {
    roleCode: payload.roleCode
  });
  dispatch(setRoleUsers(response.data));
});
const loadAllUsers = payload => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  const response = yield apiBridge$1.get(config.apis.getUsersByRole, Object.assign({}, payload.body), {}, payload.query);
  dispatch(setRoleUsers(response.data));
});
const getProperty = payload => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  const response = yield apiBridge$1.post(config.apis.getPropertyList, Object.assign({}, payload.body), {}, payload.query);
  dispatch(setUsers$1(response.data));
});
const addUserToRole = (payload, {
  url
} = {}) => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  var _c;
  const response = yield apiBridge$1.post(url || config.apis.roleMapping, payload);
  if (response.status === CONSTANTS.STATUS.SUCCESS) {
    yield dispatch(showNotificationMessage({
      message: 'User added saved successfully',
      type: 'success'
    }));
  } else if (((_c = response.validationError) === null || _c === void 0 ? void 0 : _c.key) === 'CODE_IS_DUPLICATE') {
    yield dispatch(setAddUserError({
      errorMessage: 'User already exists'
    }));
  }
});
const savePermissionMapping = (payload, {
  url,
  userProfile
} = {}) => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  const response = yield apiBridge$1.post(url || config.apis.savePermissionByRole, payload);
  if (response.status === CONSTANTS.STATUS.SUCCESS) {
    yield dispatch(showNotificationMessage({
      message: 'Permssions saved successfully',
      type: 'success'
    }));
    yield dispatch(loadUserProfile(userProfile));
    redirectTo('viewRoles');
  } else {
    yield dispatch(showNotificationMessage({
      message: 'Something went wrong',
      type: 'error'
    }));
  }
});
const loadPermissions = (payload, {
  url
} = {}) => dispatch => __awaiter$1(void 0, void 0, void 0, function* () {
  const response = yield apiBridge$1.get(url || config.apis.getAllPermissions, {});
  const filterable = new domain.collection.Filterable(response.data);
  const obj = {
    [GLOBAL_OPTIONS.permissionTypes.keys.PERMISSIONS]: groupBy(filterable.filterBy({
      groupTypeCode: GLOBAL_OPTIONS.permissionTypes.keys.PERMISSIONS
    }).toArray(), 'groupName'),
    [GLOBAL_OPTIONS.permissionTypes.keys.LEFT_NAVIGATION]: groupBy(filterable.filterBy({
      groupTypeCode: GLOBAL_OPTIONS.permissionTypes.keys.LEFT_NAVIGATION
    }).toArray(), 'groupName')
  };
  dispatch(setPermissions(obj));
});
const getPermissionsByRole = (roleCode, {
  url
} = {}) => dispatch => __awaiter$1(void 0, void 0, void 0, function* () {
  var _d;
  const response = yield apiBridge$1.post(url || config.apis.searchPermissionsMapping, {
    roleCode
  });
  if (response.status === CONSTANTS.STATUS.SUCCESS) {
    const dataObj = Object.assign({}, response.data);
    dataObj.allPermissions = {};
    ((_d = response.data) === null || _d === void 0 ? void 0 : _d.permission) && response.data.permission.forEach(perm => {
      const uniQKey = `${perm.groupTypeCode}_${perm.groupName}`;
      if (dataObj.allPermissions[uniQKey]) {
        dataObj.allPermissions[uniQKey].push(perm.code);
      } else {
        dataObj.allPermissions[uniQKey] = [perm.code];
      }
    });
    dispatch(setCurrentRole(dataObj));
  } else {
    dispatch(showNotificationMessage({
      message: 'Failed to get permissions',
      type: 'error'
    }));
  }
});
const deactivateUser = (payload, config$1 = {}) => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  const {
    url
  } = processParams(Object.assign(Object.assign({}, selectUserData(getState())), payload), config$1);
  const response = yield apiBridge$1.patch(url || `${config.apis.user}/${payload.emailId}/deactivate`);
  if (response.status === CONSTANTS.STATUS.SUCCESS) {
    yield dispatch(showNotificationMessage({
      message: 'User deactivate successfully',
      type: 'success'
    }));
  }
});
const deleteUser = (payload, config$1 = {}) => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  const {
    url
  } = processParams(Object.assign(Object.assign({}, selectUserData(getState())), payload), config$1);
  const response = yield apiBridge$1.delete(url || `${config.apis.user}/${payload.emailId}/delete`);
  if (response.status === CONSTANTS.STATUS.SUCCESS) {
    yield dispatch(showNotificationMessage({
      message: 'User deleted successfully',
      type: 'success'
    }));
  }
});
const reactivateUser = (payload, config$1 = {}) => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  const {
    url
  } = processParams(Object.assign(Object.assign({}, selectUserData(getState())), payload), config$1);
  const response = yield apiBridge$1.patch(url || `${config.apis.user}/${payload.emailId}/activate`);
  if (response.status === CONSTANTS.STATUS.SUCCESS) {
    yield dispatch(showNotificationMessage({
      message: 'User activate successfully',
      type: 'success'
    }));
  }
});
const resetPassword = (payload, config$1 = {}) => (dispatch, getState) => __awaiter$1(void 0, void 0, void 0, function* () {
  const {
    url
  } = processParams(Object.assign(Object.assign({}, selectUserData(getState())), payload), config$1);
  const response = yield apiBridge$1.patch(url || `${config.apis.user}/${payload.emailId}/resetPassword`);
  if (response.status === CONSTANTS.STATUS.SUCCESS) {
    yield dispatch(showNotificationMessage({
      message: 'Password reset completed successfully.',
      type: 'success'
    }));
  }
});
const {
  setUser,
  setRoles,
  setUsers: setUsers$1,
  setUserPagination,
  setPreference,
  setAllRoles,
  setAllRolesPages,
  setAddUserError,
  setCurrentRole,
  setPermissions,
  setRoleUsers,
  updateCurrentRolePermissions,
  clearAddUserError
} = authentication.actions;
var index$f = authentication.reducer;

var index$g = /*#__PURE__*/Object.freeze({
  __proto__: null,
  loadUserProfile: loadUserProfile,
  clearUser: clearUser,
  loadRoles: loadRoles,
  showAllRoles: showAllRoles,
  removeUserToRole: removeUserToRole,
  removeRole: removeRole,
  hasPermission: hasPermission,
  loadUsers: loadUsers$1,
  loadUsersByRole: loadUsersByRole,
  loadAllUsers: loadAllUsers,
  getProperty: getProperty,
  addUserToRole: addUserToRole,
  savePermissionMapping: savePermissionMapping,
  loadPermissions: loadPermissions,
  getPermissionsByRole: getPermissionsByRole,
  deactivateUser: deactivateUser,
  deleteUser: deleteUser,
  reactivateUser: reactivateUser,
  resetPassword: resetPassword,
  setUser: setUser,
  setRoles: setRoles,
  setUsers: setUsers$1,
  setUserPagination: setUserPagination,
  setPreference: setPreference,
  setAllRoles: setAllRoles,
  setAllRolesPages: setAllRolesPages,
  setAddUserError: setAddUserError,
  setCurrentRole: setCurrentRole,
  setPermissions: setPermissions,
  setRoleUsers: setRoleUsers,
  updateCurrentRolePermissions: updateCurrentRolePermissions,
  clearAddUserError: clearAddUserError,
  'default': index$f
});

const initialState$3 = {
  users: null
};
const user = createSlice({
  name: 'user',
  initialState: initialState$3,
  reducers: {
    setUsers: (state, action) => {
      state.users = action.payload;
    }
  }
});
// customHooks.add('user', {});
const loadUsers = () => dispatch => __awaiter$1(void 0, void 0, void 0, function* () {
  const result = yield apiBridge$1.post(config.apis.getUsers, {});
  if (result.status === CONSTANTS.STATUS.SUCCESS) {
    dispatch(setUsers(result.data.users));
  }
});
const {
  setUsers
} = user.actions;
var index$d = user.reducer;

var index$e = /*#__PURE__*/Object.freeze({
  __proto__: null,
  loadUsers: loadUsers,
  setUsers: setUsers,
  'default': index$d
});

const initialState$2 = {
  currentUser: null
};
const emailtemplate = createSlice({
  name: 'emailtemplate',
  initialState: initialState$2,
  reducers: {
    setTemplates: (state, action) => {
      state.templates = action.payload;
    },
    setTemplatesPagination: (state, action) => {
      state.templatesPagination = action.payload;
    }
  }
});
customHooks.add('emailtemplate', {
  // loadUserProfile: async (response, { dispatch }) => {
  //   dispatch(loadUserProfile());
  // },
});
const loadTemplates = (payload, {
  url
} = {}) => dispatch => __awaiter$1(void 0, void 0, void 0, function* () {
  var _a, _b, _c, _d, _e;
  const response = yield apiBridge$1.post(url || config.apis.searchEmailTemplates, Object.assign({}, payload.body), {}, payload.query);
  if (response.status === CONSTANTS.STATUS.SUCCESS) {
    const pagination = {
      total: (_a = response.data) === null || _a === void 0 ? void 0 : _a.totalItems,
      pageSize: (_b = response.data) === null || _b === void 0 ? void 0 : _b.pageSize,
      currentPage: (_c = response.data) === null || _c === void 0 ? void 0 : _c.currentPage,
      isLast: (_d = response.data) === null || _d === void 0 ? void 0 : _d.last,
      totalPages: (_e = response.data) === null || _e === void 0 ? void 0 : _e.totalPages
    };
    yield dispatch(setTemplates(response.data.data));
    yield dispatch(setTemplatesPagination(pagination));
  }
});
const {
  setTemplates,
  setTemplatesPagination
} = emailtemplate.actions;
var index$b = emailtemplate.reducer;

var index$c = /*#__PURE__*/Object.freeze({
  __proto__: null,
  loadTemplates: loadTemplates,
  setTemplates: setTemplates,
  setTemplatesPagination: setTemplatesPagination,
  'default': index$b
});

const application = createSlice({
  name: 'application',
  initialState: {
    appLoaded: false,
    theme: 'main'
  },
  reducers: {
    setAppLoaded: state => {
      state.appLoaded = true;
    }
  }
});
const {
  setAppLoaded
} = application.actions;
var index$9 = application.reducer;

var index$a = /*#__PURE__*/Object.freeze({
  __proto__: null,
  setAppLoaded: setAppLoaded,
  'default': index$9
});

var index$7 = {
  content: index$j,
  common: index$l,
  admin: index$h,
  authentication: index$f,
  user: index$d,
  emailtemplate: index$b,
  application: index$9
};

var index$8 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  content: index$k,
  common: index$m,
  admin: index$i,
  authentication: index$g,
  user: index$e,
  emailtemplate: index$c,
  application: index$a,
  'default': index$7
});

const register = (plugin = {}) => {
  const {
    components = {},
    templates = {},
    containers = {},
    hooks = {},
    formatters = {},
    icons: icons$1 = {},
    dynamicComponents: dynamicComponents$1 = {}
  } = plugin;
  components$1.set(components);
  containers$1.set(templates);
  containers$1.set(containers);
  Object.keys(hooks).forEach(key => {
    customHooks.add(key, hooks[key]);
  });
  formatters && setFormatters(formatters);
  icons$1 && icons.set(icons$1);
  dynamicComponents$1 && dynamicComponents.set(dynamicComponents$1);
};
// export { storage, components };

var plugins = /*#__PURE__*/Object.freeze({
  __proto__: null,
  register: register
});

const useId = () => {
  const [id, setData] = useState('');
  useEffect(() => {
    if (!id) {
      setData(getRandomS8());
    }
  }, []);
  return id;
};

var index$6 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  useId: useId
});

var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

var propTypes = {exports: {}};

var reactIs = {exports: {}};

var reactIs_production_min = {};

/** @license React v16.13.1
 * react-is.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
var b="function"===typeof Symbol&&Symbol.for,c=b?Symbol.for("react.element"):60103,d=b?Symbol.for("react.portal"):60106,e=b?Symbol.for("react.fragment"):60107,f=b?Symbol.for("react.strict_mode"):60108,g=b?Symbol.for("react.profiler"):60114,h=b?Symbol.for("react.provider"):60109,k=b?Symbol.for("react.context"):60110,l=b?Symbol.for("react.async_mode"):60111,m=b?Symbol.for("react.concurrent_mode"):60111,n=b?Symbol.for("react.forward_ref"):60112,p=b?Symbol.for("react.suspense"):60113,q=b?
Symbol.for("react.suspense_list"):60120,r=b?Symbol.for("react.memo"):60115,t=b?Symbol.for("react.lazy"):60116,v=b?Symbol.for("react.block"):60121,w=b?Symbol.for("react.fundamental"):60117,x=b?Symbol.for("react.responder"):60118,y=b?Symbol.for("react.scope"):60119;
function z(a){if("object"===typeof a&&null!==a){var u=a.$$typeof;switch(u){case c:switch(a=a.type,a){case l:case m:case e:case g:case f:case p:return a;default:switch(a=a&&a.$$typeof,a){case k:case n:case t:case r:case h:return a;default:return u}}case d:return u}}}function A(a){return z(a)===m}reactIs_production_min.AsyncMode=l;reactIs_production_min.ConcurrentMode=m;reactIs_production_min.ContextConsumer=k;reactIs_production_min.ContextProvider=h;reactIs_production_min.Element=c;reactIs_production_min.ForwardRef=n;reactIs_production_min.Fragment=e;reactIs_production_min.Lazy=t;reactIs_production_min.Memo=r;reactIs_production_min.Portal=d;
reactIs_production_min.Profiler=g;reactIs_production_min.StrictMode=f;reactIs_production_min.Suspense=p;reactIs_production_min.isAsyncMode=function(a){return A(a)||z(a)===l};reactIs_production_min.isConcurrentMode=A;reactIs_production_min.isContextConsumer=function(a){return z(a)===k};reactIs_production_min.isContextProvider=function(a){return z(a)===h};reactIs_production_min.isElement=function(a){return "object"===typeof a&&null!==a&&a.$$typeof===c};reactIs_production_min.isForwardRef=function(a){return z(a)===n};reactIs_production_min.isFragment=function(a){return z(a)===e};reactIs_production_min.isLazy=function(a){return z(a)===t};
reactIs_production_min.isMemo=function(a){return z(a)===r};reactIs_production_min.isPortal=function(a){return z(a)===d};reactIs_production_min.isProfiler=function(a){return z(a)===g};reactIs_production_min.isStrictMode=function(a){return z(a)===f};reactIs_production_min.isSuspense=function(a){return z(a)===p};
reactIs_production_min.isValidElementType=function(a){return "string"===typeof a||"function"===typeof a||a===e||a===m||a===g||a===f||a===p||a===q||"object"===typeof a&&null!==a&&(a.$$typeof===t||a.$$typeof===r||a.$$typeof===h||a.$$typeof===k||a.$$typeof===n||a.$$typeof===w||a.$$typeof===x||a.$$typeof===y||a.$$typeof===v)};reactIs_production_min.typeOf=z;

var reactIs_development = {};

/** @license React v16.13.1
 * react-is.development.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */



if (process.env.NODE_ENV !== "production") {
  (function() {

// The Symbol used to tag the ReactElement-like types. If there is no native Symbol
// nor polyfill, then a plain number is used for performance.
var hasSymbol = typeof Symbol === 'function' && Symbol.for;
var REACT_ELEMENT_TYPE = hasSymbol ? Symbol.for('react.element') : 0xeac7;
var REACT_PORTAL_TYPE = hasSymbol ? Symbol.for('react.portal') : 0xeaca;
var REACT_FRAGMENT_TYPE = hasSymbol ? Symbol.for('react.fragment') : 0xeacb;
var REACT_STRICT_MODE_TYPE = hasSymbol ? Symbol.for('react.strict_mode') : 0xeacc;
var REACT_PROFILER_TYPE = hasSymbol ? Symbol.for('react.profiler') : 0xead2;
var REACT_PROVIDER_TYPE = hasSymbol ? Symbol.for('react.provider') : 0xeacd;
var REACT_CONTEXT_TYPE = hasSymbol ? Symbol.for('react.context') : 0xeace; // TODO: We don't use AsyncMode or ConcurrentMode anymore. They were temporary
// (unstable) APIs that have been removed. Can we remove the symbols?

var REACT_ASYNC_MODE_TYPE = hasSymbol ? Symbol.for('react.async_mode') : 0xeacf;
var REACT_CONCURRENT_MODE_TYPE = hasSymbol ? Symbol.for('react.concurrent_mode') : 0xeacf;
var REACT_FORWARD_REF_TYPE = hasSymbol ? Symbol.for('react.forward_ref') : 0xead0;
var REACT_SUSPENSE_TYPE = hasSymbol ? Symbol.for('react.suspense') : 0xead1;
var REACT_SUSPENSE_LIST_TYPE = hasSymbol ? Symbol.for('react.suspense_list') : 0xead8;
var REACT_MEMO_TYPE = hasSymbol ? Symbol.for('react.memo') : 0xead3;
var REACT_LAZY_TYPE = hasSymbol ? Symbol.for('react.lazy') : 0xead4;
var REACT_BLOCK_TYPE = hasSymbol ? Symbol.for('react.block') : 0xead9;
var REACT_FUNDAMENTAL_TYPE = hasSymbol ? Symbol.for('react.fundamental') : 0xead5;
var REACT_RESPONDER_TYPE = hasSymbol ? Symbol.for('react.responder') : 0xead6;
var REACT_SCOPE_TYPE = hasSymbol ? Symbol.for('react.scope') : 0xead7;

function isValidElementType(type) {
  return typeof type === 'string' || typeof type === 'function' || // Note: its typeof might be other than 'symbol' or 'number' if it's a polyfill.
  type === REACT_FRAGMENT_TYPE || type === REACT_CONCURRENT_MODE_TYPE || type === REACT_PROFILER_TYPE || type === REACT_STRICT_MODE_TYPE || type === REACT_SUSPENSE_TYPE || type === REACT_SUSPENSE_LIST_TYPE || typeof type === 'object' && type !== null && (type.$$typeof === REACT_LAZY_TYPE || type.$$typeof === REACT_MEMO_TYPE || type.$$typeof === REACT_PROVIDER_TYPE || type.$$typeof === REACT_CONTEXT_TYPE || type.$$typeof === REACT_FORWARD_REF_TYPE || type.$$typeof === REACT_FUNDAMENTAL_TYPE || type.$$typeof === REACT_RESPONDER_TYPE || type.$$typeof === REACT_SCOPE_TYPE || type.$$typeof === REACT_BLOCK_TYPE);
}

function typeOf(object) {
  if (typeof object === 'object' && object !== null) {
    var $$typeof = object.$$typeof;

    switch ($$typeof) {
      case REACT_ELEMENT_TYPE:
        var type = object.type;

        switch (type) {
          case REACT_ASYNC_MODE_TYPE:
          case REACT_CONCURRENT_MODE_TYPE:
          case REACT_FRAGMENT_TYPE:
          case REACT_PROFILER_TYPE:
          case REACT_STRICT_MODE_TYPE:
          case REACT_SUSPENSE_TYPE:
            return type;

          default:
            var $$typeofType = type && type.$$typeof;

            switch ($$typeofType) {
              case REACT_CONTEXT_TYPE:
              case REACT_FORWARD_REF_TYPE:
              case REACT_LAZY_TYPE:
              case REACT_MEMO_TYPE:
              case REACT_PROVIDER_TYPE:
                return $$typeofType;

              default:
                return $$typeof;
            }

        }

      case REACT_PORTAL_TYPE:
        return $$typeof;
    }
  }

  return undefined;
} // AsyncMode is deprecated along with isAsyncMode

var AsyncMode = REACT_ASYNC_MODE_TYPE;
var ConcurrentMode = REACT_CONCURRENT_MODE_TYPE;
var ContextConsumer = REACT_CONTEXT_TYPE;
var ContextProvider = REACT_PROVIDER_TYPE;
var Element = REACT_ELEMENT_TYPE;
var ForwardRef = REACT_FORWARD_REF_TYPE;
var Fragment = REACT_FRAGMENT_TYPE;
var Lazy = REACT_LAZY_TYPE;
var Memo = REACT_MEMO_TYPE;
var Portal = REACT_PORTAL_TYPE;
var Profiler = REACT_PROFILER_TYPE;
var StrictMode = REACT_STRICT_MODE_TYPE;
var Suspense = REACT_SUSPENSE_TYPE;
var hasWarnedAboutDeprecatedIsAsyncMode = false; // AsyncMode should be deprecated

function isAsyncMode(object) {
  {
    if (!hasWarnedAboutDeprecatedIsAsyncMode) {
      hasWarnedAboutDeprecatedIsAsyncMode = true; // Using console['warn'] to evade Babel and ESLint

      console['warn']('The ReactIs.isAsyncMode() alias has been deprecated, ' + 'and will be removed in React 17+. Update your code to use ' + 'ReactIs.isConcurrentMode() instead. It has the exact same API.');
    }
  }

  return isConcurrentMode(object) || typeOf(object) === REACT_ASYNC_MODE_TYPE;
}
function isConcurrentMode(object) {
  return typeOf(object) === REACT_CONCURRENT_MODE_TYPE;
}
function isContextConsumer(object) {
  return typeOf(object) === REACT_CONTEXT_TYPE;
}
function isContextProvider(object) {
  return typeOf(object) === REACT_PROVIDER_TYPE;
}
function isElement(object) {
  return typeof object === 'object' && object !== null && object.$$typeof === REACT_ELEMENT_TYPE;
}
function isForwardRef(object) {
  return typeOf(object) === REACT_FORWARD_REF_TYPE;
}
function isFragment(object) {
  return typeOf(object) === REACT_FRAGMENT_TYPE;
}
function isLazy(object) {
  return typeOf(object) === REACT_LAZY_TYPE;
}
function isMemo(object) {
  return typeOf(object) === REACT_MEMO_TYPE;
}
function isPortal(object) {
  return typeOf(object) === REACT_PORTAL_TYPE;
}
function isProfiler(object) {
  return typeOf(object) === REACT_PROFILER_TYPE;
}
function isStrictMode(object) {
  return typeOf(object) === REACT_STRICT_MODE_TYPE;
}
function isSuspense(object) {
  return typeOf(object) === REACT_SUSPENSE_TYPE;
}

reactIs_development.AsyncMode = AsyncMode;
reactIs_development.ConcurrentMode = ConcurrentMode;
reactIs_development.ContextConsumer = ContextConsumer;
reactIs_development.ContextProvider = ContextProvider;
reactIs_development.Element = Element;
reactIs_development.ForwardRef = ForwardRef;
reactIs_development.Fragment = Fragment;
reactIs_development.Lazy = Lazy;
reactIs_development.Memo = Memo;
reactIs_development.Portal = Portal;
reactIs_development.Profiler = Profiler;
reactIs_development.StrictMode = StrictMode;
reactIs_development.Suspense = Suspense;
reactIs_development.isAsyncMode = isAsyncMode;
reactIs_development.isConcurrentMode = isConcurrentMode;
reactIs_development.isContextConsumer = isContextConsumer;
reactIs_development.isContextProvider = isContextProvider;
reactIs_development.isElement = isElement;
reactIs_development.isForwardRef = isForwardRef;
reactIs_development.isFragment = isFragment;
reactIs_development.isLazy = isLazy;
reactIs_development.isMemo = isMemo;
reactIs_development.isPortal = isPortal;
reactIs_development.isProfiler = isProfiler;
reactIs_development.isStrictMode = isStrictMode;
reactIs_development.isSuspense = isSuspense;
reactIs_development.isValidElementType = isValidElementType;
reactIs_development.typeOf = typeOf;
  })();
}

if (process.env.NODE_ENV === 'production') {
  reactIs.exports = reactIs_production_min;
} else {
  reactIs.exports = reactIs_development;
}

/*
object-assign
(c) Sindre Sorhus
@license MIT
*/
/* eslint-disable no-unused-vars */
var getOwnPropertySymbols = Object.getOwnPropertySymbols;
var hasOwnProperty = Object.prototype.hasOwnProperty;
var propIsEnumerable = Object.prototype.propertyIsEnumerable;

function toObject(val) {
	if (val === null || val === undefined) {
		throw new TypeError('Object.assign cannot be called with null or undefined');
	}

	return Object(val);
}

function shouldUseNative() {
	try {
		if (!Object.assign) {
			return false;
		}

		// Detect buggy property enumeration order in older V8 versions.

		// https://bugs.chromium.org/p/v8/issues/detail?id=4118
		var test1 = new String('abc');  // eslint-disable-line no-new-wrappers
		test1[5] = 'de';
		if (Object.getOwnPropertyNames(test1)[0] === '5') {
			return false;
		}

		// https://bugs.chromium.org/p/v8/issues/detail?id=3056
		var test2 = {};
		for (var i = 0; i < 10; i++) {
			test2['_' + String.fromCharCode(i)] = i;
		}
		var order2 = Object.getOwnPropertyNames(test2).map(function (n) {
			return test2[n];
		});
		if (order2.join('') !== '0123456789') {
			return false;
		}

		// https://bugs.chromium.org/p/v8/issues/detail?id=3056
		var test3 = {};
		'abcdefghijklmnopqrst'.split('').forEach(function (letter) {
			test3[letter] = letter;
		});
		if (Object.keys(Object.assign({}, test3)).join('') !==
				'abcdefghijklmnopqrst') {
			return false;
		}

		return true;
	} catch (err) {
		// We don't expect any of the above to throw, but better to be safe.
		return false;
	}
}

var objectAssign = shouldUseNative() ? Object.assign : function (target, source) {
	var from;
	var to = toObject(target);
	var symbols;

	for (var s = 1; s < arguments.length; s++) {
		from = Object(arguments[s]);

		for (var key in from) {
			if (hasOwnProperty.call(from, key)) {
				to[key] = from[key];
			}
		}

		if (getOwnPropertySymbols) {
			symbols = getOwnPropertySymbols(from);
			for (var i = 0; i < symbols.length; i++) {
				if (propIsEnumerable.call(from, symbols[i])) {
					to[symbols[i]] = from[symbols[i]];
				}
			}
		}
	}

	return to;
};

/**
 * Copyright (c) 2013-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

var ReactPropTypesSecret$3 = 'SECRET_DO_NOT_PASS_THIS_OR_YOU_WILL_BE_FIRED';

var ReactPropTypesSecret_1 = ReactPropTypesSecret$3;

var has$2 = Function.call.bind(Object.prototype.hasOwnProperty);

/**
 * Copyright (c) 2013-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

var printWarning$1 = function() {};

if (process.env.NODE_ENV !== 'production') {
  var ReactPropTypesSecret$2 = ReactPropTypesSecret_1;
  var loggedTypeFailures = {};
  var has$1 = has$2;

  printWarning$1 = function(text) {
    var message = 'Warning: ' + text;
    if (typeof console !== 'undefined') {
      console.error(message);
    }
    try {
      // --- Welcome to debugging React ---
      // This error was thrown as a convenience so that you can use this stack
      // to find the callsite that caused this warning to fire.
      throw new Error(message);
    } catch (x) { /**/ }
  };
}

/**
 * Assert that the values match with the type specs.
 * Error messages are memorized and will only be shown once.
 *
 * @param {object} typeSpecs Map of name to a ReactPropType
 * @param {object} values Runtime values that need to be type-checked
 * @param {string} location e.g. "prop", "context", "child context"
 * @param {string} componentName Name of the component for error messages.
 * @param {?Function} getStack Returns the component stack.
 * @private
 */
function checkPropTypes$1(typeSpecs, values, location, componentName, getStack) {
  if (process.env.NODE_ENV !== 'production') {
    for (var typeSpecName in typeSpecs) {
      if (has$1(typeSpecs, typeSpecName)) {
        var error;
        // Prop type validation may throw. In case they do, we don't want to
        // fail the render phase where it didn't fail before. So we log it.
        // After these have been cleaned up, we'll let them throw.
        try {
          // This is intentionally an invariant that gets caught. It's the same
          // behavior as without this statement except with a better message.
          if (typeof typeSpecs[typeSpecName] !== 'function') {
            var err = Error(
              (componentName || 'React class') + ': ' + location + ' type `' + typeSpecName + '` is invalid; ' +
              'it must be a function, usually from the `prop-types` package, but received `' + typeof typeSpecs[typeSpecName] + '`.' +
              'This often happens because of typos such as `PropTypes.function` instead of `PropTypes.func`.'
            );
            err.name = 'Invariant Violation';
            throw err;
          }
          error = typeSpecs[typeSpecName](values, typeSpecName, componentName, location, null, ReactPropTypesSecret$2);
        } catch (ex) {
          error = ex;
        }
        if (error && !(error instanceof Error)) {
          printWarning$1(
            (componentName || 'React class') + ': type specification of ' +
            location + ' `' + typeSpecName + '` is invalid; the type checker ' +
            'function must return `null` or an `Error` but returned a ' + typeof error + '. ' +
            'You may have forgotten to pass an argument to the type checker ' +
            'creator (arrayOf, instanceOf, objectOf, oneOf, oneOfType, and ' +
            'shape all require an argument).'
          );
        }
        if (error instanceof Error && !(error.message in loggedTypeFailures)) {
          // Only monitor this failure once because there tends to be a lot of the
          // same error.
          loggedTypeFailures[error.message] = true;

          var stack = getStack ? getStack() : '';

          printWarning$1(
            'Failed ' + location + ' type: ' + error.message + (stack != null ? stack : '')
          );
        }
      }
    }
  }
}

/**
 * Resets warning cache when testing.
 *
 * @private
 */
checkPropTypes$1.resetWarningCache = function() {
  if (process.env.NODE_ENV !== 'production') {
    loggedTypeFailures = {};
  }
};

var checkPropTypes_1 = checkPropTypes$1;

/**
 * Copyright (c) 2013-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

var ReactIs$1 = reactIs.exports;
var assign = objectAssign;

var ReactPropTypesSecret$1 = ReactPropTypesSecret_1;
var has = has$2;
var checkPropTypes = checkPropTypes_1;

var printWarning = function() {};

if (process.env.NODE_ENV !== 'production') {
  printWarning = function(text) {
    var message = 'Warning: ' + text;
    if (typeof console !== 'undefined') {
      console.error(message);
    }
    try {
      // --- Welcome to debugging React ---
      // This error was thrown as a convenience so that you can use this stack
      // to find the callsite that caused this warning to fire.
      throw new Error(message);
    } catch (x) {}
  };
}

function emptyFunctionThatReturnsNull() {
  return null;
}

var factoryWithTypeCheckers = function(isValidElement, throwOnDirectAccess) {
  /* global Symbol */
  var ITERATOR_SYMBOL = typeof Symbol === 'function' && Symbol.iterator;
  var FAUX_ITERATOR_SYMBOL = '@@iterator'; // Before Symbol spec.

  /**
   * Returns the iterator method function contained on the iterable object.
   *
   * Be sure to invoke the function with the iterable as context:
   *
   *     var iteratorFn = getIteratorFn(myIterable);
   *     if (iteratorFn) {
   *       var iterator = iteratorFn.call(myIterable);
   *       ...
   *     }
   *
   * @param {?object} maybeIterable
   * @return {?function}
   */
  function getIteratorFn(maybeIterable) {
    var iteratorFn = maybeIterable && (ITERATOR_SYMBOL && maybeIterable[ITERATOR_SYMBOL] || maybeIterable[FAUX_ITERATOR_SYMBOL]);
    if (typeof iteratorFn === 'function') {
      return iteratorFn;
    }
  }

  /**
   * Collection of methods that allow declaration and validation of props that are
   * supplied to React components. Example usage:
   *
   *   var Props = require('ReactPropTypes');
   *   var MyArticle = React.createClass({
   *     propTypes: {
   *       // An optional string prop named "description".
   *       description: Props.string,
   *
   *       // A required enum prop named "category".
   *       category: Props.oneOf(['News','Photos']).isRequired,
   *
   *       // A prop named "dialog" that requires an instance of Dialog.
   *       dialog: Props.instanceOf(Dialog).isRequired
   *     },
   *     render: function() { ... }
   *   });
   *
   * A more formal specification of how these methods are used:
   *
   *   type := array|bool|func|object|number|string|oneOf([...])|instanceOf(...)
   *   decl := ReactPropTypes.{type}(.isRequired)?
   *
   * Each and every declaration produces a function with the same signature. This
   * allows the creation of custom validation functions. For example:
   *
   *  var MyLink = React.createClass({
   *    propTypes: {
   *      // An optional string or URI prop named "href".
   *      href: function(props, propName, componentName) {
   *        var propValue = props[propName];
   *        if (propValue != null && typeof propValue !== 'string' &&
   *            !(propValue instanceof URI)) {
   *          return new Error(
   *            'Expected a string or an URI for ' + propName + ' in ' +
   *            componentName
   *          );
   *        }
   *      }
   *    },
   *    render: function() {...}
   *  });
   *
   * @internal
   */

  var ANONYMOUS = '<<anonymous>>';

  // Important!
  // Keep this list in sync with production version in `./factoryWithThrowingShims.js`.
  var ReactPropTypes = {
    array: createPrimitiveTypeChecker('array'),
    bigint: createPrimitiveTypeChecker('bigint'),
    bool: createPrimitiveTypeChecker('boolean'),
    func: createPrimitiveTypeChecker('function'),
    number: createPrimitiveTypeChecker('number'),
    object: createPrimitiveTypeChecker('object'),
    string: createPrimitiveTypeChecker('string'),
    symbol: createPrimitiveTypeChecker('symbol'),

    any: createAnyTypeChecker(),
    arrayOf: createArrayOfTypeChecker,
    element: createElementTypeChecker(),
    elementType: createElementTypeTypeChecker(),
    instanceOf: createInstanceTypeChecker,
    node: createNodeChecker(),
    objectOf: createObjectOfTypeChecker,
    oneOf: createEnumTypeChecker,
    oneOfType: createUnionTypeChecker,
    shape: createShapeTypeChecker,
    exact: createStrictShapeTypeChecker,
  };

  /**
   * inlined Object.is polyfill to avoid requiring consumers ship their own
   * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/is
   */
  /*eslint-disable no-self-compare*/
  function is(x, y) {
    // SameValue algorithm
    if (x === y) {
      // Steps 1-5, 7-10
      // Steps 6.b-6.e: +0 != -0
      return x !== 0 || 1 / x === 1 / y;
    } else {
      // Step 6.a: NaN == NaN
      return x !== x && y !== y;
    }
  }
  /*eslint-enable no-self-compare*/

  /**
   * We use an Error-like object for backward compatibility as people may call
   * PropTypes directly and inspect their output. However, we don't use real
   * Errors anymore. We don't inspect their stack anyway, and creating them
   * is prohibitively expensive if they are created too often, such as what
   * happens in oneOfType() for any type before the one that matched.
   */
  function PropTypeError(message, data) {
    this.message = message;
    this.data = data && typeof data === 'object' ? data: {};
    this.stack = '';
  }
  // Make `instanceof Error` still work for returned errors.
  PropTypeError.prototype = Error.prototype;

  function createChainableTypeChecker(validate) {
    if (process.env.NODE_ENV !== 'production') {
      var manualPropTypeCallCache = {};
      var manualPropTypeWarningCount = 0;
    }
    function checkType(isRequired, props, propName, componentName, location, propFullName, secret) {
      componentName = componentName || ANONYMOUS;
      propFullName = propFullName || propName;

      if (secret !== ReactPropTypesSecret$1) {
        if (throwOnDirectAccess) {
          // New behavior only for users of `prop-types` package
          var err = new Error(
            'Calling PropTypes validators directly is not supported by the `prop-types` package. ' +
            'Use `PropTypes.checkPropTypes()` to call them. ' +
            'Read more at http://fb.me/use-check-prop-types'
          );
          err.name = 'Invariant Violation';
          throw err;
        } else if (process.env.NODE_ENV !== 'production' && typeof console !== 'undefined') {
          // Old behavior for people using React.PropTypes
          var cacheKey = componentName + ':' + propName;
          if (
            !manualPropTypeCallCache[cacheKey] &&
            // Avoid spamming the console because they are often not actionable except for lib authors
            manualPropTypeWarningCount < 3
          ) {
            printWarning(
              'You are manually calling a React.PropTypes validation ' +
              'function for the `' + propFullName + '` prop on `' + componentName + '`. This is deprecated ' +
              'and will throw in the standalone `prop-types` package. ' +
              'You may be seeing this warning due to a third-party PropTypes ' +
              'library. See https://fb.me/react-warning-dont-call-proptypes ' + 'for details.'
            );
            manualPropTypeCallCache[cacheKey] = true;
            manualPropTypeWarningCount++;
          }
        }
      }
      if (props[propName] == null) {
        if (isRequired) {
          if (props[propName] === null) {
            return new PropTypeError('The ' + location + ' `' + propFullName + '` is marked as required ' + ('in `' + componentName + '`, but its value is `null`.'));
          }
          return new PropTypeError('The ' + location + ' `' + propFullName + '` is marked as required in ' + ('`' + componentName + '`, but its value is `undefined`.'));
        }
        return null;
      } else {
        return validate(props, propName, componentName, location, propFullName);
      }
    }

    var chainedCheckType = checkType.bind(null, false);
    chainedCheckType.isRequired = checkType.bind(null, true);

    return chainedCheckType;
  }

  function createPrimitiveTypeChecker(expectedType) {
    function validate(props, propName, componentName, location, propFullName, secret) {
      var propValue = props[propName];
      var propType = getPropType(propValue);
      if (propType !== expectedType) {
        // `propValue` being instance of, say, date/regexp, pass the 'object'
        // check, but we can offer a more precise error message here rather than
        // 'of type `object`'.
        var preciseType = getPreciseType(propValue);

        return new PropTypeError(
          'Invalid ' + location + ' `' + propFullName + '` of type ' + ('`' + preciseType + '` supplied to `' + componentName + '`, expected ') + ('`' + expectedType + '`.'),
          {expectedType: expectedType}
        );
      }
      return null;
    }
    return createChainableTypeChecker(validate);
  }

  function createAnyTypeChecker() {
    return createChainableTypeChecker(emptyFunctionThatReturnsNull);
  }

  function createArrayOfTypeChecker(typeChecker) {
    function validate(props, propName, componentName, location, propFullName) {
      if (typeof typeChecker !== 'function') {
        return new PropTypeError('Property `' + propFullName + '` of component `' + componentName + '` has invalid PropType notation inside arrayOf.');
      }
      var propValue = props[propName];
      if (!Array.isArray(propValue)) {
        var propType = getPropType(propValue);
        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type ' + ('`' + propType + '` supplied to `' + componentName + '`, expected an array.'));
      }
      for (var i = 0; i < propValue.length; i++) {
        var error = typeChecker(propValue, i, componentName, location, propFullName + '[' + i + ']', ReactPropTypesSecret$1);
        if (error instanceof Error) {
          return error;
        }
      }
      return null;
    }
    return createChainableTypeChecker(validate);
  }

  function createElementTypeChecker() {
    function validate(props, propName, componentName, location, propFullName) {
      var propValue = props[propName];
      if (!isValidElement(propValue)) {
        var propType = getPropType(propValue);
        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type ' + ('`' + propType + '` supplied to `' + componentName + '`, expected a single ReactElement.'));
      }
      return null;
    }
    return createChainableTypeChecker(validate);
  }

  function createElementTypeTypeChecker() {
    function validate(props, propName, componentName, location, propFullName) {
      var propValue = props[propName];
      if (!ReactIs$1.isValidElementType(propValue)) {
        var propType = getPropType(propValue);
        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type ' + ('`' + propType + '` supplied to `' + componentName + '`, expected a single ReactElement type.'));
      }
      return null;
    }
    return createChainableTypeChecker(validate);
  }

  function createInstanceTypeChecker(expectedClass) {
    function validate(props, propName, componentName, location, propFullName) {
      if (!(props[propName] instanceof expectedClass)) {
        var expectedClassName = expectedClass.name || ANONYMOUS;
        var actualClassName = getClassName(props[propName]);
        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type ' + ('`' + actualClassName + '` supplied to `' + componentName + '`, expected ') + ('instance of `' + expectedClassName + '`.'));
      }
      return null;
    }
    return createChainableTypeChecker(validate);
  }

  function createEnumTypeChecker(expectedValues) {
    if (!Array.isArray(expectedValues)) {
      if (process.env.NODE_ENV !== 'production') {
        if (arguments.length > 1) {
          printWarning(
            'Invalid arguments supplied to oneOf, expected an array, got ' + arguments.length + ' arguments. ' +
            'A common mistake is to write oneOf(x, y, z) instead of oneOf([x, y, z]).'
          );
        } else {
          printWarning('Invalid argument supplied to oneOf, expected an array.');
        }
      }
      return emptyFunctionThatReturnsNull;
    }

    function validate(props, propName, componentName, location, propFullName) {
      var propValue = props[propName];
      for (var i = 0; i < expectedValues.length; i++) {
        if (is(propValue, expectedValues[i])) {
          return null;
        }
      }

      var valuesString = JSON.stringify(expectedValues, function replacer(key, value) {
        var type = getPreciseType(value);
        if (type === 'symbol') {
          return String(value);
        }
        return value;
      });
      return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of value `' + String(propValue) + '` ' + ('supplied to `' + componentName + '`, expected one of ' + valuesString + '.'));
    }
    return createChainableTypeChecker(validate);
  }

  function createObjectOfTypeChecker(typeChecker) {
    function validate(props, propName, componentName, location, propFullName) {
      if (typeof typeChecker !== 'function') {
        return new PropTypeError('Property `' + propFullName + '` of component `' + componentName + '` has invalid PropType notation inside objectOf.');
      }
      var propValue = props[propName];
      var propType = getPropType(propValue);
      if (propType !== 'object') {
        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type ' + ('`' + propType + '` supplied to `' + componentName + '`, expected an object.'));
      }
      for (var key in propValue) {
        if (has(propValue, key)) {
          var error = typeChecker(propValue, key, componentName, location, propFullName + '.' + key, ReactPropTypesSecret$1);
          if (error instanceof Error) {
            return error;
          }
        }
      }
      return null;
    }
    return createChainableTypeChecker(validate);
  }

  function createUnionTypeChecker(arrayOfTypeCheckers) {
    if (!Array.isArray(arrayOfTypeCheckers)) {
      process.env.NODE_ENV !== 'production' ? printWarning('Invalid argument supplied to oneOfType, expected an instance of array.') : void 0;
      return emptyFunctionThatReturnsNull;
    }

    for (var i = 0; i < arrayOfTypeCheckers.length; i++) {
      var checker = arrayOfTypeCheckers[i];
      if (typeof checker !== 'function') {
        printWarning(
          'Invalid argument supplied to oneOfType. Expected an array of check functions, but ' +
          'received ' + getPostfixForTypeWarning(checker) + ' at index ' + i + '.'
        );
        return emptyFunctionThatReturnsNull;
      }
    }

    function validate(props, propName, componentName, location, propFullName) {
      var expectedTypes = [];
      for (var i = 0; i < arrayOfTypeCheckers.length; i++) {
        var checker = arrayOfTypeCheckers[i];
        var checkerResult = checker(props, propName, componentName, location, propFullName, ReactPropTypesSecret$1);
        if (checkerResult == null) {
          return null;
        }
        if (checkerResult.data && has(checkerResult.data, 'expectedType')) {
          expectedTypes.push(checkerResult.data.expectedType);
        }
      }
      var expectedTypesMessage = (expectedTypes.length > 0) ? ', expected one of type [' + expectedTypes.join(', ') + ']': '';
      return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` supplied to ' + ('`' + componentName + '`' + expectedTypesMessage + '.'));
    }
    return createChainableTypeChecker(validate);
  }

  function createNodeChecker() {
    function validate(props, propName, componentName, location, propFullName) {
      if (!isNode(props[propName])) {
        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` supplied to ' + ('`' + componentName + '`, expected a ReactNode.'));
      }
      return null;
    }
    return createChainableTypeChecker(validate);
  }

  function invalidValidatorError(componentName, location, propFullName, key, type) {
    return new PropTypeError(
      (componentName || 'React class') + ': ' + location + ' type `' + propFullName + '.' + key + '` is invalid; ' +
      'it must be a function, usually from the `prop-types` package, but received `' + type + '`.'
    );
  }

  function createShapeTypeChecker(shapeTypes) {
    function validate(props, propName, componentName, location, propFullName) {
      var propValue = props[propName];
      var propType = getPropType(propValue);
      if (propType !== 'object') {
        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type `' + propType + '` ' + ('supplied to `' + componentName + '`, expected `object`.'));
      }
      for (var key in shapeTypes) {
        var checker = shapeTypes[key];
        if (typeof checker !== 'function') {
          return invalidValidatorError(componentName, location, propFullName, key, getPreciseType(checker));
        }
        var error = checker(propValue, key, componentName, location, propFullName + '.' + key, ReactPropTypesSecret$1);
        if (error) {
          return error;
        }
      }
      return null;
    }
    return createChainableTypeChecker(validate);
  }

  function createStrictShapeTypeChecker(shapeTypes) {
    function validate(props, propName, componentName, location, propFullName) {
      var propValue = props[propName];
      var propType = getPropType(propValue);
      if (propType !== 'object') {
        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type `' + propType + '` ' + ('supplied to `' + componentName + '`, expected `object`.'));
      }
      // We need to check all keys in case some are required but missing from props.
      var allKeys = assign({}, props[propName], shapeTypes);
      for (var key in allKeys) {
        var checker = shapeTypes[key];
        if (has(shapeTypes, key) && typeof checker !== 'function') {
          return invalidValidatorError(componentName, location, propFullName, key, getPreciseType(checker));
        }
        if (!checker) {
          return new PropTypeError(
            'Invalid ' + location + ' `' + propFullName + '` key `' + key + '` supplied to `' + componentName + '`.' +
            '\nBad object: ' + JSON.stringify(props[propName], null, '  ') +
            '\nValid keys: ' + JSON.stringify(Object.keys(shapeTypes), null, '  ')
          );
        }
        var error = checker(propValue, key, componentName, location, propFullName + '.' + key, ReactPropTypesSecret$1);
        if (error) {
          return error;
        }
      }
      return null;
    }

    return createChainableTypeChecker(validate);
  }

  function isNode(propValue) {
    switch (typeof propValue) {
      case 'number':
      case 'string':
      case 'undefined':
        return true;
      case 'boolean':
        return !propValue;
      case 'object':
        if (Array.isArray(propValue)) {
          return propValue.every(isNode);
        }
        if (propValue === null || isValidElement(propValue)) {
          return true;
        }

        var iteratorFn = getIteratorFn(propValue);
        if (iteratorFn) {
          var iterator = iteratorFn.call(propValue);
          var step;
          if (iteratorFn !== propValue.entries) {
            while (!(step = iterator.next()).done) {
              if (!isNode(step.value)) {
                return false;
              }
            }
          } else {
            // Iterator will provide entry [k,v] tuples rather than values.
            while (!(step = iterator.next()).done) {
              var entry = step.value;
              if (entry) {
                if (!isNode(entry[1])) {
                  return false;
                }
              }
            }
          }
        } else {
          return false;
        }

        return true;
      default:
        return false;
    }
  }

  function isSymbol(propType, propValue) {
    // Native Symbol.
    if (propType === 'symbol') {
      return true;
    }

    // falsy value can't be a Symbol
    if (!propValue) {
      return false;
    }

    // 19.4.3.5 Symbol.prototype[@@toStringTag] === 'Symbol'
    if (propValue['@@toStringTag'] === 'Symbol') {
      return true;
    }

    // Fallback for non-spec compliant Symbols which are polyfilled.
    if (typeof Symbol === 'function' && propValue instanceof Symbol) {
      return true;
    }

    return false;
  }

  // Equivalent of `typeof` but with special handling for array and regexp.
  function getPropType(propValue) {
    var propType = typeof propValue;
    if (Array.isArray(propValue)) {
      return 'array';
    }
    if (propValue instanceof RegExp) {
      // Old webkits (at least until Android 4.0) return 'function' rather than
      // 'object' for typeof a RegExp. We'll normalize this here so that /bla/
      // passes PropTypes.object.
      return 'object';
    }
    if (isSymbol(propType, propValue)) {
      return 'symbol';
    }
    return propType;
  }

  // This handles more types than `getPropType`. Only used for error messages.
  // See `createPrimitiveTypeChecker`.
  function getPreciseType(propValue) {
    if (typeof propValue === 'undefined' || propValue === null) {
      return '' + propValue;
    }
    var propType = getPropType(propValue);
    if (propType === 'object') {
      if (propValue instanceof Date) {
        return 'date';
      } else if (propValue instanceof RegExp) {
        return 'regexp';
      }
    }
    return propType;
  }

  // Returns a string that is postfixed to a warning about an invalid type.
  // For example, "undefined" or "of type array"
  function getPostfixForTypeWarning(value) {
    var type = getPreciseType(value);
    switch (type) {
      case 'array':
      case 'object':
        return 'an ' + type;
      case 'boolean':
      case 'date':
      case 'regexp':
        return 'a ' + type;
      default:
        return type;
    }
  }

  // Returns class name of the object, if any.
  function getClassName(propValue) {
    if (!propValue.constructor || !propValue.constructor.name) {
      return ANONYMOUS;
    }
    return propValue.constructor.name;
  }

  ReactPropTypes.checkPropTypes = checkPropTypes;
  ReactPropTypes.resetWarningCache = checkPropTypes.resetWarningCache;
  ReactPropTypes.PropTypes = ReactPropTypes;

  return ReactPropTypes;
};

/**
 * Copyright (c) 2013-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

var ReactPropTypesSecret = ReactPropTypesSecret_1;

function emptyFunction() {}
function emptyFunctionWithReset() {}
emptyFunctionWithReset.resetWarningCache = emptyFunction;

var factoryWithThrowingShims = function() {
  function shim(props, propName, componentName, location, propFullName, secret) {
    if (secret === ReactPropTypesSecret) {
      // It is still safe when called from React.
      return;
    }
    var err = new Error(
      'Calling PropTypes validators directly is not supported by the `prop-types` package. ' +
      'Use PropTypes.checkPropTypes() to call them. ' +
      'Read more at http://fb.me/use-check-prop-types'
    );
    err.name = 'Invariant Violation';
    throw err;
  }  shim.isRequired = shim;
  function getShim() {
    return shim;
  }  // Important!
  // Keep this list in sync with production version in `./factoryWithTypeCheckers.js`.
  var ReactPropTypes = {
    array: shim,
    bigint: shim,
    bool: shim,
    func: shim,
    number: shim,
    object: shim,
    string: shim,
    symbol: shim,

    any: shim,
    arrayOf: getShim,
    element: shim,
    elementType: shim,
    instanceOf: getShim,
    node: shim,
    objectOf: getShim,
    oneOf: getShim,
    oneOfType: getShim,
    shape: getShim,
    exact: getShim,

    checkPropTypes: emptyFunctionWithReset,
    resetWarningCache: emptyFunction
  };

  ReactPropTypes.PropTypes = ReactPropTypes;

  return ReactPropTypes;
};

/**
 * Copyright (c) 2013-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

if (process.env.NODE_ENV !== 'production') {
  var ReactIs = reactIs.exports;

  // By explicitly using `prop-types` you are opting into new development behavior.
  // http://fb.me/prop-types-in-prod
  var throwOnDirectAccess = true;
  propTypes.exports = factoryWithTypeCheckers(ReactIs.isElement, throwOnDirectAccess);
} else {
  // By explicitly using `prop-types` you are opting into new production behavior.
  // http://fb.me/prop-types-in-prod
  propTypes.exports = factoryWithThrowingShims();
}

var PropTypes = propTypes.exports;

class ErrorBoundry$1 extends React$2.Component {
  constructor(props) {
    super(props);
    this.state = {
      hasError: false
    };
  }
  componentDidCatch(error) {
    console.log('@@@@@', error);
    // You can also log the error to an error reporting service
    this.setState({
      hasError: true
    });
  }
  render() {
    const {
      Alert
    } = components$1.get();
    if (this.state.hasError) {
      // You can render any custom fallback UI
      return jsxs("div", {
        className: "error-boundry mt-wide mb-wide",
        children: [Alert && jsx(Alert, {
          type: "error",
          message: "Some error has occured"
        }), !Alert && jsx("div", {
          className: 'sq-error',
          children: "Some error has occured"
        })]
      });
    }
    return this.props.children;
  }
}
ErrorBoundry$1.propTypes = {
  children: PropTypes.any
};

var components = /*#__PURE__*/Object.freeze({
  __proto__: null,
  ErrorBoundry: ErrorBoundry$1
});

/**
 * Create the React Context
 */ const DndContext = createContext({
    dragDropManager: undefined
});

/**
 * Adapted from React: https://github.com/facebook/react/blob/master/packages/shared/formatProdErrorMessage.js
 *
 * Do not require this module directly! Use normal throw error calls. These messages will be replaced with error codes
 * during build.
 * @param {number} code
 */
function formatProdErrorMessage(code) {
  return "Minified Redux error #" + code + "; visit https://redux.js.org/Errors?code=" + code + " for the full message or " + 'use the non-minified dev environment for full errors. ';
}

// Inlined version of the `symbol-observable` polyfill
var $$observable = (function () {
  return typeof Symbol === 'function' && Symbol.observable || '@@observable';
})();

/**
 * These are private action types reserved by Redux.
 * For any unknown actions, you must return the current state.
 * If the current state is undefined, you must return the initial state.
 * Do not reference these action types directly in your code.
 */
var randomString = function randomString() {
  return Math.random().toString(36).substring(7).split('').join('.');
};

var ActionTypes = {
  INIT: "@@redux/INIT" + randomString(),
  REPLACE: "@@redux/REPLACE" + randomString(),
  PROBE_UNKNOWN_ACTION: function PROBE_UNKNOWN_ACTION() {
    return "@@redux/PROBE_UNKNOWN_ACTION" + randomString();
  }
};

/**
 * @param {any} obj The object to inspect.
 * @returns {boolean} True if the argument appears to be a plain object.
 */
function isPlainObject(obj) {
  if (typeof obj !== 'object' || obj === null) return false;
  var proto = obj;

  while (Object.getPrototypeOf(proto) !== null) {
    proto = Object.getPrototypeOf(proto);
  }

  return Object.getPrototypeOf(obj) === proto;
}

// Inlined / shortened version of `kindOf` from https://github.com/jonschlinkert/kind-of
function miniKindOf(val) {
  if (val === void 0) return 'undefined';
  if (val === null) return 'null';
  var type = typeof val;

  switch (type) {
    case 'boolean':
    case 'string':
    case 'number':
    case 'symbol':
    case 'function':
      {
        return type;
      }
  }

  if (Array.isArray(val)) return 'array';
  if (isDate(val)) return 'date';
  if (isError(val)) return 'error';
  var constructorName = ctorName(val);

  switch (constructorName) {
    case 'Symbol':
    case 'Promise':
    case 'WeakMap':
    case 'WeakSet':
    case 'Map':
    case 'Set':
      return constructorName;
  } // other


  return type.slice(8, -1).toLowerCase().replace(/\s/g, '');
}

function ctorName(val) {
  return typeof val.constructor === 'function' ? val.constructor.name : null;
}

function isError(val) {
  return val instanceof Error || typeof val.message === 'string' && val.constructor && typeof val.constructor.stackTraceLimit === 'number';
}

function isDate(val) {
  if (val instanceof Date) return true;
  return typeof val.toDateString === 'function' && typeof val.getDate === 'function' && typeof val.setDate === 'function';
}

function kindOf(val) {
  var typeOfVal = typeof val;

  if (process.env.NODE_ENV !== 'production') {
    typeOfVal = miniKindOf(val);
  }

  return typeOfVal;
}

/**
 * @deprecated
 *
 * **We recommend using the `configureStore` method
 * of the `@reduxjs/toolkit` package**, which replaces `createStore`.
 *
 * Redux Toolkit is our recommended approach for writing Redux logic today,
 * including store setup, reducers, data fetching, and more.
 *
 * **For more details, please read this Redux docs page:**
 * **https://redux.js.org/introduction/why-rtk-is-redux-today**
 *
 * `configureStore` from Redux Toolkit is an improved version of `createStore` that
 * simplifies setup and helps avoid common bugs.
 *
 * You should not be using the `redux` core package by itself today, except for learning purposes.
 * The `createStore` method from the core `redux` package will not be removed, but we encourage
 * all users to migrate to using Redux Toolkit for all Redux code.
 *
 * If you want to use `createStore` without this visual deprecation warning, use
 * the `legacy_createStore` import instead:
 *
 * `import { legacy_createStore as createStore} from 'redux'`
 *
 */

function createStore(reducer, preloadedState, enhancer) {
  var _ref2;

  if (typeof preloadedState === 'function' && typeof enhancer === 'function' || typeof enhancer === 'function' && typeof arguments[3] === 'function') {
    throw new Error(process.env.NODE_ENV === "production" ? formatProdErrorMessage(0) : 'It looks like you are passing several store enhancers to ' + 'createStore(). This is not supported. Instead, compose them ' + 'together to a single function. See https://redux.js.org/tutorials/fundamentals/part-4-store#creating-a-store-with-enhancers for an example.');
  }

  if (typeof preloadedState === 'function' && typeof enhancer === 'undefined') {
    enhancer = preloadedState;
    preloadedState = undefined;
  }

  if (typeof enhancer !== 'undefined') {
    if (typeof enhancer !== 'function') {
      throw new Error(process.env.NODE_ENV === "production" ? formatProdErrorMessage(1) : "Expected the enhancer to be a function. Instead, received: '" + kindOf(enhancer) + "'");
    }

    return enhancer(createStore)(reducer, preloadedState);
  }

  if (typeof reducer !== 'function') {
    throw new Error(process.env.NODE_ENV === "production" ? formatProdErrorMessage(2) : "Expected the root reducer to be a function. Instead, received: '" + kindOf(reducer) + "'");
  }

  var currentReducer = reducer;
  var currentState = preloadedState;
  var currentListeners = [];
  var nextListeners = currentListeners;
  var isDispatching = false;
  /**
   * This makes a shallow copy of currentListeners so we can use
   * nextListeners as a temporary list while dispatching.
   *
   * This prevents any bugs around consumers calling
   * subscribe/unsubscribe in the middle of a dispatch.
   */

  function ensureCanMutateNextListeners() {
    if (nextListeners === currentListeners) {
      nextListeners = currentListeners.slice();
    }
  }
  /**
   * Reads the state tree managed by the store.
   *
   * @returns {any} The current state tree of your application.
   */


  function getState() {
    if (isDispatching) {
      throw new Error(process.env.NODE_ENV === "production" ? formatProdErrorMessage(3) : 'You may not call store.getState() while the reducer is executing. ' + 'The reducer has already received the state as an argument. ' + 'Pass it down from the top reducer instead of reading it from the store.');
    }

    return currentState;
  }
  /**
   * Adds a change listener. It will be called any time an action is dispatched,
   * and some part of the state tree may potentially have changed. You may then
   * call `getState()` to read the current state tree inside the callback.
   *
   * You may call `dispatch()` from a change listener, with the following
   * caveats:
   *
   * 1. The subscriptions are snapshotted just before every `dispatch()` call.
   * If you subscribe or unsubscribe while the listeners are being invoked, this
   * will not have any effect on the `dispatch()` that is currently in progress.
   * However, the next `dispatch()` call, whether nested or not, will use a more
   * recent snapshot of the subscription list.
   *
   * 2. The listener should not expect to see all state changes, as the state
   * might have been updated multiple times during a nested `dispatch()` before
   * the listener is called. It is, however, guaranteed that all subscribers
   * registered before the `dispatch()` started will be called with the latest
   * state by the time it exits.
   *
   * @param {Function} listener A callback to be invoked on every dispatch.
   * @returns {Function} A function to remove this change listener.
   */


  function subscribe(listener) {
    if (typeof listener !== 'function') {
      throw new Error(process.env.NODE_ENV === "production" ? formatProdErrorMessage(4) : "Expected the listener to be a function. Instead, received: '" + kindOf(listener) + "'");
    }

    if (isDispatching) {
      throw new Error(process.env.NODE_ENV === "production" ? formatProdErrorMessage(5) : 'You may not call store.subscribe() while the reducer is executing. ' + 'If you would like to be notified after the store has been updated, subscribe from a ' + 'component and invoke store.getState() in the callback to access the latest state. ' + 'See https://redux.js.org/api/store#subscribelistener for more details.');
    }

    var isSubscribed = true;
    ensureCanMutateNextListeners();
    nextListeners.push(listener);
    return function unsubscribe() {
      if (!isSubscribed) {
        return;
      }

      if (isDispatching) {
        throw new Error(process.env.NODE_ENV === "production" ? formatProdErrorMessage(6) : 'You may not unsubscribe from a store listener while the reducer is executing. ' + 'See https://redux.js.org/api/store#subscribelistener for more details.');
      }

      isSubscribed = false;
      ensureCanMutateNextListeners();
      var index = nextListeners.indexOf(listener);
      nextListeners.splice(index, 1);
      currentListeners = null;
    };
  }
  /**
   * Dispatches an action. It is the only way to trigger a state change.
   *
   * The `reducer` function, used to create the store, will be called with the
   * current state tree and the given `action`. Its return value will
   * be considered the **next** state of the tree, and the change listeners
   * will be notified.
   *
   * The base implementation only supports plain object actions. If you want to
   * dispatch a Promise, an Observable, a thunk, or something else, you need to
   * wrap your store creating function into the corresponding middleware. For
   * example, see the documentation for the `redux-thunk` package. Even the
   * middleware will eventually dispatch plain object actions using this method.
   *
   * @param {Object} action A plain object representing “what changed”. It is
   * a good idea to keep actions serializable so you can record and replay user
   * sessions, or use the time travelling `redux-devtools`. An action must have
   * a `type` property which may not be `undefined`. It is a good idea to use
   * string constants for action types.
   *
   * @returns {Object} For convenience, the same action object you dispatched.
   *
   * Note that, if you use a custom middleware, it may wrap `dispatch()` to
   * return something else (for example, a Promise you can await).
   */


  function dispatch(action) {
    if (!isPlainObject(action)) {
      throw new Error(process.env.NODE_ENV === "production" ? formatProdErrorMessage(7) : "Actions must be plain objects. Instead, the actual type was: '" + kindOf(action) + "'. You may need to add middleware to your store setup to handle dispatching other values, such as 'redux-thunk' to handle dispatching functions. See https://redux.js.org/tutorials/fundamentals/part-4-store#middleware and https://redux.js.org/tutorials/fundamentals/part-6-async-logic#using-the-redux-thunk-middleware for examples.");
    }

    if (typeof action.type === 'undefined') {
      throw new Error(process.env.NODE_ENV === "production" ? formatProdErrorMessage(8) : 'Actions may not have an undefined "type" property. You may have misspelled an action type string constant.');
    }

    if (isDispatching) {
      throw new Error(process.env.NODE_ENV === "production" ? formatProdErrorMessage(9) : 'Reducers may not dispatch actions.');
    }

    try {
      isDispatching = true;
      currentState = currentReducer(currentState, action);
    } finally {
      isDispatching = false;
    }

    var listeners = currentListeners = nextListeners;

    for (var i = 0; i < listeners.length; i++) {
      var listener = listeners[i];
      listener();
    }

    return action;
  }
  /**
   * Replaces the reducer currently used by the store to calculate the state.
   *
   * You might need this if your app implements code splitting and you want to
   * load some of the reducers dynamically. You might also need this if you
   * implement a hot reloading mechanism for Redux.
   *
   * @param {Function} nextReducer The reducer for the store to use instead.
   * @returns {void}
   */


  function replaceReducer(nextReducer) {
    if (typeof nextReducer !== 'function') {
      throw new Error(process.env.NODE_ENV === "production" ? formatProdErrorMessage(10) : "Expected the nextReducer to be a function. Instead, received: '" + kindOf(nextReducer));
    }

    currentReducer = nextReducer; // This action has a similiar effect to ActionTypes.INIT.
    // Any reducers that existed in both the new and old rootReducer
    // will receive the previous state. This effectively populates
    // the new state tree with any relevant data from the old one.

    dispatch({
      type: ActionTypes.REPLACE
    });
  }
  /**
   * Interoperability point for observable/reactive libraries.
   * @returns {observable} A minimal observable of state changes.
   * For more information, see the observable proposal:
   * https://github.com/tc39/proposal-observable
   */


  function observable() {
    var _ref;

    var outerSubscribe = subscribe;
    return _ref = {
      /**
       * The minimal observable subscription method.
       * @param {Object} observer Any object that can be used as an observer.
       * The observer object should have a `next` method.
       * @returns {subscription} An object with an `unsubscribe` method that can
       * be used to unsubscribe the observable from the store, and prevent further
       * emission of values from the observable.
       */
      subscribe: function subscribe(observer) {
        if (typeof observer !== 'object' || observer === null) {
          throw new Error(process.env.NODE_ENV === "production" ? formatProdErrorMessage(11) : "Expected the observer to be an object. Instead, received: '" + kindOf(observer) + "'");
        }

        function observeState() {
          if (observer.next) {
            observer.next(getState());
          }
        }

        observeState();
        var unsubscribe = outerSubscribe(observeState);
        return {
          unsubscribe: unsubscribe
        };
      }
    }, _ref[$$observable] = function () {
      return this;
    }, _ref;
  } // When a store is created, an "INIT" action is dispatched so that every
  // reducer returns their initial state. This effectively populates
  // the initial state tree.


  dispatch({
    type: ActionTypes.INIT
  });
  return _ref2 = {
    dispatch: dispatch,
    subscribe: subscribe,
    getState: getState,
    replaceReducer: replaceReducer
  }, _ref2[$$observable] = observable, _ref2;
}

/**
 * Use invariant() to assert state which your program assumes to be true.
 *
 * Provide sprintf-style format (only %s is supported) and arguments
 * to provide information about what broke and what you were
 * expecting.
 *
 * The invariant message will be stripped in production, but the invariant
 * will remain to ensure logic does not differ in production.
 */ function invariant(condition, format, ...args) {
    if (isProduction()) {
        if (format === undefined) {
            throw new Error('invariant requires an error message argument');
        }
    }
    if (!condition) {
        let error;
        if (format === undefined) {
            error = new Error('Minified exception occurred; use the non-minified dev environment ' + 'for the full error message and additional helpful warnings.');
        } else {
            let argIndex = 0;
            error = new Error(format.replace(/%s/g, function() {
                return args[argIndex++];
            }));
            error.name = 'Invariant Violation';
        }
        error.framesToPop = 1 // we don't care about invariant's own frame
        ;
        throw error;
    }
}
function isProduction() {
    return typeof process !== 'undefined' && process.env['NODE_ENV'] === 'production';
}

// cheap lodash replacements
/**
 * drop-in replacement for _.get
 * @param obj
 * @param path
 * @param defaultValue
 */ function get(obj, path, defaultValue) {
    return path.split('.').reduce((a, c)=>a && a[c] ? a[c] : defaultValue || null
    , obj);
}
/**
 * drop-in replacement for _.without
 */ function without$1(items, item) {
    return items.filter((i)=>i !== item
    );
}
/**
 * drop-in replacement for _.isString
 * @param input
 */ function isObject(input) {
    return typeof input === 'object';
}
/**
 * replacement for _.xor
 * @param itemsA
 * @param itemsB
 */ function xor(itemsA, itemsB) {
    const map = new Map();
    const insertItem = (item)=>{
        map.set(item, map.has(item) ? map.get(item) + 1 : 1);
    };
    itemsA.forEach(insertItem);
    itemsB.forEach(insertItem);
    const result = [];
    map.forEach((count, key)=>{
        if (count === 1) {
            result.push(key);
        }
    });
    return result;
}
/**
 * replacement for _.intersection
 * @param itemsA
 * @param itemsB
 */ function intersection(itemsA, itemsB) {
    return itemsA.filter((t)=>itemsB.indexOf(t) > -1
    );
}

const INIT_COORDS = 'dnd-core/INIT_COORDS';
const BEGIN_DRAG = 'dnd-core/BEGIN_DRAG';
const PUBLISH_DRAG_SOURCE = 'dnd-core/PUBLISH_DRAG_SOURCE';
const HOVER = 'dnd-core/HOVER';
const DROP = 'dnd-core/DROP';
const END_DRAG = 'dnd-core/END_DRAG';

function setClientOffset(clientOffset, sourceClientOffset) {
    return {
        type: INIT_COORDS,
        payload: {
            sourceClientOffset: sourceClientOffset || null,
            clientOffset: clientOffset || null
        }
    };
}

const ResetCoordinatesAction = {
    type: INIT_COORDS,
    payload: {
        clientOffset: null,
        sourceClientOffset: null
    }
};
function createBeginDrag(manager) {
    return function beginDrag(sourceIds = [], options = {
        publishSource: true
    }) {
        const { publishSource =true , clientOffset , getSourceClientOffset ,  } = options;
        const monitor = manager.getMonitor();
        const registry = manager.getRegistry();
        // Initialize the coordinates using the client offset
        manager.dispatch(setClientOffset(clientOffset));
        verifyInvariants$1(sourceIds, monitor, registry);
        // Get the draggable source
        const sourceId = getDraggableSource(sourceIds, monitor);
        if (sourceId == null) {
            manager.dispatch(ResetCoordinatesAction);
            return;
        }
        // Get the source client offset
        let sourceClientOffset = null;
        if (clientOffset) {
            if (!getSourceClientOffset) {
                throw new Error('getSourceClientOffset must be defined');
            }
            verifyGetSourceClientOffsetIsFunction(getSourceClientOffset);
            sourceClientOffset = getSourceClientOffset(sourceId);
        }
        // Initialize the full coordinates
        manager.dispatch(setClientOffset(clientOffset, sourceClientOffset));
        const source = registry.getSource(sourceId);
        const item = source.beginDrag(monitor, sourceId);
        // If source.beginDrag returns null, this is an indicator to cancel the drag
        if (item == null) {
            return undefined;
        }
        verifyItemIsObject(item);
        registry.pinSource(sourceId);
        const itemType = registry.getSourceType(sourceId);
        return {
            type: BEGIN_DRAG,
            payload: {
                itemType,
                item,
                sourceId,
                clientOffset: clientOffset || null,
                sourceClientOffset: sourceClientOffset || null,
                isSourcePublic: !!publishSource
            }
        };
    };
}
function verifyInvariants$1(sourceIds, monitor, registry) {
    invariant(!monitor.isDragging(), 'Cannot call beginDrag while dragging.');
    sourceIds.forEach(function(sourceId) {
        invariant(registry.getSource(sourceId), 'Expected sourceIds to be registered.');
    });
}
function verifyGetSourceClientOffsetIsFunction(getSourceClientOffset) {
    invariant(typeof getSourceClientOffset === 'function', 'When clientOffset is provided, getSourceClientOffset must be a function.');
}
function verifyItemIsObject(item) {
    invariant(isObject(item), 'Item must be an object.');
}
function getDraggableSource(sourceIds, monitor) {
    let sourceId = null;
    for(let i = sourceIds.length - 1; i >= 0; i--){
        if (monitor.canDragSource(sourceIds[i])) {
            sourceId = sourceIds[i];
            break;
        }
    }
    return sourceId;
}

function _defineProperty$4(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _objectSpread$4(target) {
    for(var i = 1; i < arguments.length; i++){
        var source = arguments[i] != null ? arguments[i] : {};
        var ownKeys = Object.keys(source);
        if (typeof Object.getOwnPropertySymbols === 'function') {
            ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function(sym) {
                return Object.getOwnPropertyDescriptor(source, sym).enumerable;
            }));
        }
        ownKeys.forEach(function(key) {
            _defineProperty$4(target, key, source[key]);
        });
    }
    return target;
}
function createDrop(manager) {
    return function drop(options = {}) {
        const monitor = manager.getMonitor();
        const registry = manager.getRegistry();
        verifyInvariants(monitor);
        const targetIds = getDroppableTargets(monitor);
        // Multiple actions are dispatched here, which is why this doesn't return an action
        targetIds.forEach((targetId, index)=>{
            const dropResult = determineDropResult(targetId, index, registry, monitor);
            const action = {
                type: DROP,
                payload: {
                    dropResult: _objectSpread$4({}, options, dropResult)
                }
            };
            manager.dispatch(action);
        });
    };
}
function verifyInvariants(monitor) {
    invariant(monitor.isDragging(), 'Cannot call drop while not dragging.');
    invariant(!monitor.didDrop(), 'Cannot call drop twice during one drag operation.');
}
function determineDropResult(targetId, index, registry, monitor) {
    const target = registry.getTarget(targetId);
    let dropResult = target ? target.drop(monitor, targetId) : undefined;
    verifyDropResultType(dropResult);
    if (typeof dropResult === 'undefined') {
        dropResult = index === 0 ? {} : monitor.getDropResult();
    }
    return dropResult;
}
function verifyDropResultType(dropResult) {
    invariant(typeof dropResult === 'undefined' || isObject(dropResult), 'Drop result must either be an object or undefined.');
}
function getDroppableTargets(monitor) {
    const targetIds = monitor.getTargetIds().filter(monitor.canDropOnTarget, monitor);
    targetIds.reverse();
    return targetIds;
}

function createEndDrag(manager) {
    return function endDrag() {
        const monitor = manager.getMonitor();
        const registry = manager.getRegistry();
        verifyIsDragging(monitor);
        const sourceId = monitor.getSourceId();
        if (sourceId != null) {
            const source = registry.getSource(sourceId, true);
            source.endDrag(monitor, sourceId);
            registry.unpinSource();
        }
        return {
            type: END_DRAG
        };
    };
}
function verifyIsDragging(monitor) {
    invariant(monitor.isDragging(), 'Cannot call endDrag while not dragging.');
}

function matchesType(targetType, draggedItemType) {
    if (draggedItemType === null) {
        return targetType === null;
    }
    return Array.isArray(targetType) ? targetType.some((t)=>t === draggedItemType
    ) : targetType === draggedItemType;
}

function createHover(manager) {
    return function hover(targetIdsArg, { clientOffset  } = {}) {
        verifyTargetIdsIsArray(targetIdsArg);
        const targetIds = targetIdsArg.slice(0);
        const monitor = manager.getMonitor();
        const registry = manager.getRegistry();
        const draggedItemType = monitor.getItemType();
        removeNonMatchingTargetIds(targetIds, registry, draggedItemType);
        checkInvariants(targetIds, monitor, registry);
        hoverAllTargets(targetIds, monitor, registry);
        return {
            type: HOVER,
            payload: {
                targetIds,
                clientOffset: clientOffset || null
            }
        };
    };
}
function verifyTargetIdsIsArray(targetIdsArg) {
    invariant(Array.isArray(targetIdsArg), 'Expected targetIds to be an array.');
}
function checkInvariants(targetIds, monitor, registry) {
    invariant(monitor.isDragging(), 'Cannot call hover while not dragging.');
    invariant(!monitor.didDrop(), 'Cannot call hover after drop.');
    for(let i = 0; i < targetIds.length; i++){
        const targetId = targetIds[i];
        invariant(targetIds.lastIndexOf(targetId) === i, 'Expected targetIds to be unique in the passed array.');
        const target = registry.getTarget(targetId);
        invariant(target, 'Expected targetIds to be registered.');
    }
}
function removeNonMatchingTargetIds(targetIds, registry, draggedItemType) {
    // Remove those targetIds that don't match the targetType.  This
    // fixes shallow isOver which would only be non-shallow because of
    // non-matching targets.
    for(let i = targetIds.length - 1; i >= 0; i--){
        const targetId = targetIds[i];
        const targetType = registry.getTargetType(targetId);
        if (!matchesType(targetType, draggedItemType)) {
            targetIds.splice(i, 1);
        }
    }
}
function hoverAllTargets(targetIds, monitor, registry) {
    // Finally call hover on all matching targets.
    targetIds.forEach(function(targetId) {
        const target = registry.getTarget(targetId);
        target.hover(monitor, targetId);
    });
}

function createPublishDragSource(manager) {
    return function publishDragSource() {
        const monitor = manager.getMonitor();
        if (monitor.isDragging()) {
            return {
                type: PUBLISH_DRAG_SOURCE
            };
        }
        return;
    };
}

function createDragDropActions(manager) {
    return {
        beginDrag: createBeginDrag(manager),
        publishDragSource: createPublishDragSource(manager),
        hover: createHover(manager),
        drop: createDrop(manager),
        endDrag: createEndDrag(manager)
    };
}

class DragDropManagerImpl {
    receiveBackend(backend) {
        this.backend = backend;
    }
    getMonitor() {
        return this.monitor;
    }
    getBackend() {
        return this.backend;
    }
    getRegistry() {
        return this.monitor.registry;
    }
    getActions() {
        /* eslint-disable-next-line @typescript-eslint/no-this-alias */ const manager = this;
        const { dispatch  } = this.store;
        function bindActionCreator(actionCreator) {
            return (...args)=>{
                const action = actionCreator.apply(manager, args);
                if (typeof action !== 'undefined') {
                    dispatch(action);
                }
            };
        }
        const actions = createDragDropActions(this);
        return Object.keys(actions).reduce((boundActions, key)=>{
            const action = actions[key];
            boundActions[key] = bindActionCreator(action);
            return boundActions;
        }, {});
    }
    dispatch(action) {
        this.store.dispatch(action);
    }
    constructor(store, monitor){
        this.isSetUp = false;
        this.handleRefCountChange = ()=>{
            const shouldSetUp = this.store.getState().refCount > 0;
            if (this.backend) {
                if (shouldSetUp && !this.isSetUp) {
                    this.backend.setup();
                    this.isSetUp = true;
                } else if (!shouldSetUp && this.isSetUp) {
                    this.backend.teardown();
                    this.isSetUp = false;
                }
            }
        };
        this.store = store;
        this.monitor = monitor;
        store.subscribe(this.handleRefCountChange);
    }
}

/**
 * Coordinate addition
 * @param a The first coordinate
 * @param b The second coordinate
 */ function add(a, b) {
    return {
        x: a.x + b.x,
        y: a.y + b.y
    };
}
/**
 * Coordinate subtraction
 * @param a The first coordinate
 * @param b The second coordinate
 */ function subtract(a, b) {
    return {
        x: a.x - b.x,
        y: a.y - b.y
    };
}
/**
 * Returns the cartesian distance of the drag source component's position, based on its position
 * at the time when the current drag operation has started, and the movement difference.
 *
 * Returns null if no item is being dragged.
 *
 * @param state The offset state to compute from
 */ function getSourceClientOffset(state) {
    const { clientOffset , initialClientOffset , initialSourceClientOffset  } = state;
    if (!clientOffset || !initialClientOffset || !initialSourceClientOffset) {
        return null;
    }
    return subtract(add(clientOffset, initialSourceClientOffset), initialClientOffset);
}
/**
 * Determines the x,y offset between the client offset and the initial client offset
 *
 * @param state The offset state to compute from
 */ function getDifferenceFromInitialOffset(state) {
    const { clientOffset , initialClientOffset  } = state;
    if (!clientOffset || !initialClientOffset) {
        return null;
    }
    return subtract(clientOffset, initialClientOffset);
}

const NONE = [];
const ALL = [];
NONE.__IS_NONE__ = true;
ALL.__IS_ALL__ = true;
/**
 * Determines if the given handler IDs are dirty or not.
 *
 * @param dirtyIds The set of dirty handler ids
 * @param handlerIds The set of handler ids to check
 */ function areDirty(dirtyIds, handlerIds) {
    if (dirtyIds === NONE) {
        return false;
    }
    if (dirtyIds === ALL || typeof handlerIds === 'undefined') {
        return true;
    }
    const commonIds = intersection(handlerIds, dirtyIds);
    return commonIds.length > 0;
}

class DragDropMonitorImpl {
    subscribeToStateChange(listener, options = {}) {
        const { handlerIds  } = options;
        invariant(typeof listener === 'function', 'listener must be a function.');
        invariant(typeof handlerIds === 'undefined' || Array.isArray(handlerIds), 'handlerIds, when specified, must be an array of strings.');
        let prevStateId = this.store.getState().stateId;
        const handleChange = ()=>{
            const state = this.store.getState();
            const currentStateId = state.stateId;
            try {
                const canSkipListener = currentStateId === prevStateId || currentStateId === prevStateId + 1 && !areDirty(state.dirtyHandlerIds, handlerIds);
                if (!canSkipListener) {
                    listener();
                }
            } finally{
                prevStateId = currentStateId;
            }
        };
        return this.store.subscribe(handleChange);
    }
    subscribeToOffsetChange(listener) {
        invariant(typeof listener === 'function', 'listener must be a function.');
        let previousState = this.store.getState().dragOffset;
        const handleChange = ()=>{
            const nextState = this.store.getState().dragOffset;
            if (nextState === previousState) {
                return;
            }
            previousState = nextState;
            listener();
        };
        return this.store.subscribe(handleChange);
    }
    canDragSource(sourceId) {
        if (!sourceId) {
            return false;
        }
        const source = this.registry.getSource(sourceId);
        invariant(source, `Expected to find a valid source. sourceId=${sourceId}`);
        if (this.isDragging()) {
            return false;
        }
        return source.canDrag(this, sourceId);
    }
    canDropOnTarget(targetId) {
        // undefined on initial render
        if (!targetId) {
            return false;
        }
        const target = this.registry.getTarget(targetId);
        invariant(target, `Expected to find a valid target. targetId=${targetId}`);
        if (!this.isDragging() || this.didDrop()) {
            return false;
        }
        const targetType = this.registry.getTargetType(targetId);
        const draggedItemType = this.getItemType();
        return matchesType(targetType, draggedItemType) && target.canDrop(this, targetId);
    }
    isDragging() {
        return Boolean(this.getItemType());
    }
    isDraggingSource(sourceId) {
        // undefined on initial render
        if (!sourceId) {
            return false;
        }
        const source = this.registry.getSource(sourceId, true);
        invariant(source, `Expected to find a valid source. sourceId=${sourceId}`);
        if (!this.isDragging() || !this.isSourcePublic()) {
            return false;
        }
        const sourceType = this.registry.getSourceType(sourceId);
        const draggedItemType = this.getItemType();
        if (sourceType !== draggedItemType) {
            return false;
        }
        return source.isDragging(this, sourceId);
    }
    isOverTarget(targetId, options = {
        shallow: false
    }) {
        // undefined on initial render
        if (!targetId) {
            return false;
        }
        const { shallow  } = options;
        if (!this.isDragging()) {
            return false;
        }
        const targetType = this.registry.getTargetType(targetId);
        const draggedItemType = this.getItemType();
        if (draggedItemType && !matchesType(targetType, draggedItemType)) {
            return false;
        }
        const targetIds = this.getTargetIds();
        if (!targetIds.length) {
            return false;
        }
        const index = targetIds.indexOf(targetId);
        if (shallow) {
            return index === targetIds.length - 1;
        } else {
            return index > -1;
        }
    }
    getItemType() {
        return this.store.getState().dragOperation.itemType;
    }
    getItem() {
        return this.store.getState().dragOperation.item;
    }
    getSourceId() {
        return this.store.getState().dragOperation.sourceId;
    }
    getTargetIds() {
        return this.store.getState().dragOperation.targetIds;
    }
    getDropResult() {
        return this.store.getState().dragOperation.dropResult;
    }
    didDrop() {
        return this.store.getState().dragOperation.didDrop;
    }
    isSourcePublic() {
        return Boolean(this.store.getState().dragOperation.isSourcePublic);
    }
    getInitialClientOffset() {
        return this.store.getState().dragOffset.initialClientOffset;
    }
    getInitialSourceClientOffset() {
        return this.store.getState().dragOffset.initialSourceClientOffset;
    }
    getClientOffset() {
        return this.store.getState().dragOffset.clientOffset;
    }
    getSourceClientOffset() {
        return getSourceClientOffset(this.store.getState().dragOffset);
    }
    getDifferenceFromInitialOffset() {
        return getDifferenceFromInitialOffset(this.store.getState().dragOffset);
    }
    constructor(store, registry){
        this.store = store;
        this.registry = registry;
    }
}

// Safari 6 and 6.1 for desktop, iPad, and iPhone are the only browsers that
// have WebKitMutationObserver but not un-prefixed MutationObserver.
// Must use `global` or `self` instead of `window` to work in both frames and web
// workers. `global` is a provision of Browserify, Mr, Mrs, or Mop.
/* globals self */ const scope = typeof global !== 'undefined' ? global : self;
const BrowserMutationObserver = scope.MutationObserver || scope.WebKitMutationObserver;
function makeRequestCallFromTimer(callback) {
    return function requestCall() {
        // We dispatch a timeout with a specified delay of 0 for engines that
        // can reliably accommodate that request. This will usually be snapped
        // to a 4 milisecond delay, but once we're flushing, there's no delay
        // between events.
        const timeoutHandle = setTimeout(handleTimer, 0);
        // However, since this timer gets frequently dropped in Firefox
        // workers, we enlist an interval handle that will try to fire
        // an event 20 times per second until it succeeds.
        const intervalHandle = setInterval(handleTimer, 50);
        function handleTimer() {
            // Whichever timer succeeds will cancel both timers and
            // execute the callback.
            clearTimeout(timeoutHandle);
            clearInterval(intervalHandle);
            callback();
        }
    };
}
// To request a high priority event, we induce a mutation observer by toggling
// the text of a text node between "1" and "-1".
function makeRequestCallFromMutationObserver(callback) {
    let toggle = 1;
    const observer = new BrowserMutationObserver(callback);
    const node = document.createTextNode('');
    observer.observe(node, {
        characterData: true
    });
    return function requestCall() {
        toggle = -toggle;
        node.data = toggle;
    };
}
const makeRequestCall = typeof BrowserMutationObserver === 'function' ? // reliably everywhere they are implemented.
// They are implemented in all modern browsers.
//
// - Android 4-4.3
// - Chrome 26-34
// - Firefox 14-29
// - Internet Explorer 11
// - iPad Safari 6-7.1
// - iPhone Safari 7-7.1
// - Safari 6-7
makeRequestCallFromMutationObserver : // task queue, are implemented in Internet Explorer 10, Safari 5.0-1, and Opera
// 11-12, and in web workers in many engines.
// Although message channels yield to any queued rendering and IO tasks, they
// would be better than imposing the 4ms delay of timers.
// However, they do not work reliably in Internet Explorer or Safari.
// Internet Explorer 10 is the only browser that has setImmediate but does
// not have MutationObservers.
// Although setImmediate yields to the browser's renderer, it would be
// preferrable to falling back to setTimeout since it does not have
// the minimum 4ms penalty.
// Unfortunately there appears to be a bug in Internet Explorer 10 Mobile (and
// Desktop to a lesser extent) that renders both setImmediate and
// MessageChannel useless for the purposes of ASAP.
// https://github.com/kriskowal/q/issues/396
// Timers are implemented universally.
// We fall back to timers in workers in most engines, and in foreground
// contexts in the following browsers.
// However, note that even this simple case requires nuances to operate in a
// broad spectrum of browsers.
//
// - Firefox 3-13
// - Internet Explorer 6-9
// - iPad Safari 4.3
// - Lynx 2.8.7
makeRequestCallFromTimer;

class AsapQueue {
    // Use the fastest means possible to execute a task in its own turn, with
    // priority over other events including IO, animation, reflow, and redraw
    // events in browsers.
    //
    // An exception thrown by a task will permanently interrupt the processing of
    // subsequent tasks. The higher level `asap` function ensures that if an
    // exception is thrown by a task, that the task queue will continue flushing as
    // soon as possible, but if you use `rawAsap` directly, you are responsible to
    // either ensure that no exceptions are thrown from your task, or to manually
    // call `rawAsap.requestFlush` if an exception is thrown.
    enqueueTask(task) {
        const { queue: q , requestFlush  } = this;
        if (!q.length) {
            requestFlush();
            this.flushing = true;
        }
        // Equivalent to push, but avoids a function call.
        q[q.length] = task;
    }
    constructor(){
        this.queue = [];
        // We queue errors to ensure they are thrown in right order (FIFO).
        // Array-as-queue is good enough here, since we are just dealing with exceptions.
        this.pendingErrors = [];
        // Once a flush has been requested, no further calls to `requestFlush` are
        // necessary until the next `flush` completes.
        // @ts-ignore
        this.flushing = false;
        // The position of the next task to execute in the task queue. This is
        // preserved between calls to `flush` so that it can be resumed if
        // a task throws an exception.
        this.index = 0;
        // If a task schedules additional tasks recursively, the task queue can grow
        // unbounded. To prevent memory exhaustion, the task queue will periodically
        // truncate already-completed tasks.
        this.capacity = 1024;
        // The flush function processes all tasks that have been scheduled with
        // `rawAsap` unless and until one of those tasks throws an exception.
        // If a task throws an exception, `flush` ensures that its state will remain
        // consistent and will resume where it left off when called again.
        // However, `flush` does not make any arrangements to be called again if an
        // exception is thrown.
        this.flush = ()=>{
            const { queue: q  } = this;
            while(this.index < q.length){
                const currentIndex = this.index;
                // Advance the index before calling the task. This ensures that we will
                // begin flushing on the next task the task throws an error.
                this.index++;
                q[currentIndex].call();
                // Prevent leaking memory for long chains of recursive calls to `asap`.
                // If we call `asap` within tasks scheduled by `asap`, the queue will
                // grow, but to avoid an O(n) walk for every task we execute, we don't
                // shift tasks off the queue after they have been executed.
                // Instead, we periodically shift 1024 tasks off the queue.
                if (this.index > this.capacity) {
                    // Manually shift all values starting at the index back to the
                    // beginning of the queue.
                    for(let scan = 0, newLength = q.length - this.index; scan < newLength; scan++){
                        q[scan] = q[scan + this.index];
                    }
                    q.length -= this.index;
                    this.index = 0;
                }
            }
            q.length = 0;
            this.index = 0;
            this.flushing = false;
        };
        // In a web browser, exceptions are not fatal. However, to avoid
        // slowing down the queue of pending tasks, we rethrow the error in a
        // lower priority turn.
        this.registerPendingError = (err)=>{
            this.pendingErrors.push(err);
            this.requestErrorThrow();
        };
        // `requestFlush` requests that the high priority event queue be flushed as
        // soon as possible.
        // This is useful to prevent an error thrown in a task from stalling the event
        // queue if the exception handled by Node.js’s
        // `process.on("uncaughtException")` or by a domain.
        // `requestFlush` is implemented using a strategy based on data collected from
        // every available SauceLabs Selenium web driver worker at time of writing.
        // https://docs.google.com/spreadsheets/d/1mG-5UYGup5qxGdEMWkhP6BWCz053NUb2E1QoUTU16uA/edit#gid=783724593
        this.requestFlush = makeRequestCall(this.flush);
        this.requestErrorThrow = makeRequestCallFromTimer(()=>{
            // Throw first error
            if (this.pendingErrors.length) {
                throw this.pendingErrors.shift();
            }
        });
    }
} // The message channel technique was discovered by Malte Ubl and was the
 // original foundation for this library.
 // http://www.nonblocking.io/2011/06/windownexttick.html
 // Safari 6.0.5 (at least) intermittently fails to create message ports on a
 // page's first load. Thankfully, this version of Safari supports
 // MutationObservers, so we don't need to fall back in that case.
 // function makeRequestCallFromMessageChannel(callback) {
 //     var channel = new MessageChannel();
 //     channel.port1.onmessage = callback;
 //     return function requestCall() {
 //         channel.port2.postMessage(0);
 //     };
 // }
 // For reasons explained above, we are also unable to use `setImmediate`
 // under any circumstances.
 // Even if we were, there is another bug in Internet Explorer 10.
 // It is not sufficient to assign `setImmediate` to `requestFlush` because
 // `setImmediate` must be called *by name* and therefore must be wrapped in a
 // closure.
 // Never forget.
 // function makeRequestCallFromSetImmediate(callback) {
 //     return function requestCall() {
 //         setImmediate(callback);
 //     };
 // }
 // Safari 6.0 has a problem where timers will get lost while the user is
 // scrolling. This problem does not impact ASAP because Safari 6.0 supports
 // mutation observers, so that implementation is used instead.
 // However, if we ever elect to use timers in Safari, the prevalent work-around
 // is to add a scroll event listener that calls for a flush.
 // `setTimeout` does not call the passed callback if the delay is less than
 // approximately 7 in web workers in Firefox 8 through 18, and sometimes not
 // even then.
 // This is for `asap.js` only.
 // Its name will be periodically randomized to break any code that depends on
 // // its existence.
 // rawAsap.makeRequestCallFromTimer = makeRequestCallFromTimer
 // ASAP was originally a nextTick shim included in Q. This was factored out
 // into this ASAP package. It was later adapted to RSVP which made further
 // amendments. These decisions, particularly to marginalize MessageChannel and
 // to capture the MutationObserver implementation in a closure, were integrated
 // back into ASAP proper.
 // https://github.com/tildeio/rsvp.js/blob/cddf7232546a9cf858524b75cde6f9edf72620a7/lib/rsvp/asap.js

// `call`, just like a function.
class RawTask {
    call() {
        try {
            this.task && this.task();
        } catch (error) {
            this.onError(error);
        } finally{
            this.task = null;
            this.release(this);
        }
    }
    constructor(onError, release){
        this.onError = onError;
        this.release = release;
        this.task = null;
    }
}

class TaskFactory {
    create(task) {
        const tasks = this.freeTasks;
        const t1 = tasks.length ? tasks.pop() : new RawTask(this.onError, (t)=>tasks[tasks.length] = t
        );
        t1.task = task;
        return t1;
    }
    constructor(onError){
        this.onError = onError;
        this.freeTasks = [];
    }
}

const asapQueue = new AsapQueue();
const taskFactory = new TaskFactory(asapQueue.registerPendingError);
/**
 * Calls a task as soon as possible after returning, in its own event, with priority
 * over other events like animation, reflow, and repaint. An error thrown from an
 * event will not interrupt, nor even substantially slow down the processing of
 * other events, but will be rather postponed to a lower priority event.
 * @param {{call}} task A callable object, typically a function that takes no
 * arguments.
 */ function asap(task) {
    asapQueue.enqueueTask(taskFactory.create(task));
}

const ADD_SOURCE = 'dnd-core/ADD_SOURCE';
const ADD_TARGET = 'dnd-core/ADD_TARGET';
const REMOVE_SOURCE = 'dnd-core/REMOVE_SOURCE';
const REMOVE_TARGET = 'dnd-core/REMOVE_TARGET';
function addSource(sourceId) {
    return {
        type: ADD_SOURCE,
        payload: {
            sourceId
        }
    };
}
function addTarget(targetId) {
    return {
        type: ADD_TARGET,
        payload: {
            targetId
        }
    };
}
function removeSource(sourceId) {
    return {
        type: REMOVE_SOURCE,
        payload: {
            sourceId
        }
    };
}
function removeTarget(targetId) {
    return {
        type: REMOVE_TARGET,
        payload: {
            targetId
        }
    };
}

function validateSourceContract(source) {
    invariant(typeof source.canDrag === 'function', 'Expected canDrag to be a function.');
    invariant(typeof source.beginDrag === 'function', 'Expected beginDrag to be a function.');
    invariant(typeof source.endDrag === 'function', 'Expected endDrag to be a function.');
}
function validateTargetContract(target) {
    invariant(typeof target.canDrop === 'function', 'Expected canDrop to be a function.');
    invariant(typeof target.hover === 'function', 'Expected hover to be a function.');
    invariant(typeof target.drop === 'function', 'Expected beginDrag to be a function.');
}
function validateType(type, allowArray) {
    if (allowArray && Array.isArray(type)) {
        type.forEach((t)=>validateType(t, false)
        );
        return;
    }
    invariant(typeof type === 'string' || typeof type === 'symbol', allowArray ? 'Type can only be a string, a symbol, or an array of either.' : 'Type can only be a string or a symbol.');
}

var HandlerRole;
(function(HandlerRole) {
    HandlerRole["SOURCE"] = "SOURCE";
    HandlerRole["TARGET"] = "TARGET";
})(HandlerRole || (HandlerRole = {}));

let nextUniqueId = 0;
function getNextUniqueId() {
    return nextUniqueId++;
}

function getNextHandlerId(role) {
    const id = getNextUniqueId().toString();
    switch(role){
        case HandlerRole.SOURCE:
            return `S${id}`;
        case HandlerRole.TARGET:
            return `T${id}`;
        default:
            throw new Error(`Unknown Handler Role: ${role}`);
    }
}
function parseRoleFromHandlerId(handlerId) {
    switch(handlerId[0]){
        case 'S':
            return HandlerRole.SOURCE;
        case 'T':
            return HandlerRole.TARGET;
        default:
            throw new Error(`Cannot parse handler ID: ${handlerId}`);
    }
}
function mapContainsValue(map, searchValue) {
    const entries = map.entries();
    let isDone = false;
    do {
        const { done , value: [, value] ,  } = entries.next();
        if (value === searchValue) {
            return true;
        }
        isDone = !!done;
    }while (!isDone)
    return false;
}
class HandlerRegistryImpl {
    addSource(type, source) {
        validateType(type);
        validateSourceContract(source);
        const sourceId = this.addHandler(HandlerRole.SOURCE, type, source);
        this.store.dispatch(addSource(sourceId));
        return sourceId;
    }
    addTarget(type, target) {
        validateType(type, true);
        validateTargetContract(target);
        const targetId = this.addHandler(HandlerRole.TARGET, type, target);
        this.store.dispatch(addTarget(targetId));
        return targetId;
    }
    containsHandler(handler) {
        return mapContainsValue(this.dragSources, handler) || mapContainsValue(this.dropTargets, handler);
    }
    getSource(sourceId, includePinned = false) {
        invariant(this.isSourceId(sourceId), 'Expected a valid source ID.');
        const isPinned = includePinned && sourceId === this.pinnedSourceId;
        const source = isPinned ? this.pinnedSource : this.dragSources.get(sourceId);
        return source;
    }
    getTarget(targetId) {
        invariant(this.isTargetId(targetId), 'Expected a valid target ID.');
        return this.dropTargets.get(targetId);
    }
    getSourceType(sourceId) {
        invariant(this.isSourceId(sourceId), 'Expected a valid source ID.');
        return this.types.get(sourceId);
    }
    getTargetType(targetId) {
        invariant(this.isTargetId(targetId), 'Expected a valid target ID.');
        return this.types.get(targetId);
    }
    isSourceId(handlerId) {
        const role = parseRoleFromHandlerId(handlerId);
        return role === HandlerRole.SOURCE;
    }
    isTargetId(handlerId) {
        const role = parseRoleFromHandlerId(handlerId);
        return role === HandlerRole.TARGET;
    }
    removeSource(sourceId) {
        invariant(this.getSource(sourceId), 'Expected an existing source.');
        this.store.dispatch(removeSource(sourceId));
        asap(()=>{
            this.dragSources.delete(sourceId);
            this.types.delete(sourceId);
        });
    }
    removeTarget(targetId) {
        invariant(this.getTarget(targetId), 'Expected an existing target.');
        this.store.dispatch(removeTarget(targetId));
        this.dropTargets.delete(targetId);
        this.types.delete(targetId);
    }
    pinSource(sourceId) {
        const source = this.getSource(sourceId);
        invariant(source, 'Expected an existing source.');
        this.pinnedSourceId = sourceId;
        this.pinnedSource = source;
    }
    unpinSource() {
        invariant(this.pinnedSource, 'No source is pinned at the time.');
        this.pinnedSourceId = null;
        this.pinnedSource = null;
    }
    addHandler(role, type, handler) {
        const id = getNextHandlerId(role);
        this.types.set(id, type);
        if (role === HandlerRole.SOURCE) {
            this.dragSources.set(id, handler);
        } else if (role === HandlerRole.TARGET) {
            this.dropTargets.set(id, handler);
        }
        return id;
    }
    constructor(store){
        this.types = new Map();
        this.dragSources = new Map();
        this.dropTargets = new Map();
        this.pinnedSourceId = null;
        this.pinnedSource = null;
        this.store = store;
    }
}

const strictEquality = (a, b)=>a === b
;
/**
 * Determine if two cartesian coordinate offsets are equal
 * @param offsetA
 * @param offsetB
 */ function areCoordsEqual(offsetA, offsetB) {
    if (!offsetA && !offsetB) {
        return true;
    } else if (!offsetA || !offsetB) {
        return false;
    } else {
        return offsetA.x === offsetB.x && offsetA.y === offsetB.y;
    }
}
/**
 * Determines if two arrays of items are equal
 * @param a The first array of items
 * @param b The second array of items
 */ function areArraysEqual(a, b, isEqual = strictEquality) {
    if (a.length !== b.length) {
        return false;
    }
    for(let i = 0; i < a.length; ++i){
        if (!isEqual(a[i], b[i])) {
            return false;
        }
    }
    return true;
}

function reduce$5(// eslint-disable-next-line @typescript-eslint/no-unused-vars
_state = NONE, action) {
    switch(action.type){
        case HOVER:
            break;
        case ADD_SOURCE:
        case ADD_TARGET:
        case REMOVE_TARGET:
        case REMOVE_SOURCE:
            return NONE;
        case BEGIN_DRAG:
        case PUBLISH_DRAG_SOURCE:
        case END_DRAG:
        case DROP:
        default:
            return ALL;
    }
    const { targetIds =[] , prevTargetIds =[]  } = action.payload;
    const result = xor(targetIds, prevTargetIds);
    const didChange = result.length > 0 || !areArraysEqual(targetIds, prevTargetIds);
    if (!didChange) {
        return NONE;
    }
    // Check the target ids at the innermost position. If they are valid, add them
    // to the result
    const prevInnermostTargetId = prevTargetIds[prevTargetIds.length - 1];
    const innermostTargetId = targetIds[targetIds.length - 1];
    if (prevInnermostTargetId !== innermostTargetId) {
        if (prevInnermostTargetId) {
            result.push(prevInnermostTargetId);
        }
        if (innermostTargetId) {
            result.push(innermostTargetId);
        }
    }
    return result;
}

function _defineProperty$3(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _objectSpread$3(target) {
    for(var i = 1; i < arguments.length; i++){
        var source = arguments[i] != null ? arguments[i] : {};
        var ownKeys = Object.keys(source);
        if (typeof Object.getOwnPropertySymbols === 'function') {
            ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function(sym) {
                return Object.getOwnPropertyDescriptor(source, sym).enumerable;
            }));
        }
        ownKeys.forEach(function(key) {
            _defineProperty$3(target, key, source[key]);
        });
    }
    return target;
}
const initialState$1 = {
    initialSourceClientOffset: null,
    initialClientOffset: null,
    clientOffset: null
};
function reduce$4(state = initialState$1, action) {
    const { payload  } = action;
    switch(action.type){
        case INIT_COORDS:
        case BEGIN_DRAG:
            return {
                initialSourceClientOffset: payload.sourceClientOffset,
                initialClientOffset: payload.clientOffset,
                clientOffset: payload.clientOffset
            };
        case HOVER:
            if (areCoordsEqual(state.clientOffset, payload.clientOffset)) {
                return state;
            }
            return _objectSpread$3({}, state, {
                clientOffset: payload.clientOffset
            });
        case END_DRAG:
        case DROP:
            return initialState$1;
        default:
            return state;
    }
}

function _defineProperty$2(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _objectSpread$2(target) {
    for(var i = 1; i < arguments.length; i++){
        var source = arguments[i] != null ? arguments[i] : {};
        var ownKeys = Object.keys(source);
        if (typeof Object.getOwnPropertySymbols === 'function') {
            ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function(sym) {
                return Object.getOwnPropertyDescriptor(source, sym).enumerable;
            }));
        }
        ownKeys.forEach(function(key) {
            _defineProperty$2(target, key, source[key]);
        });
    }
    return target;
}
const initialState = {
    itemType: null,
    item: null,
    sourceId: null,
    targetIds: [],
    dropResult: null,
    didDrop: false,
    isSourcePublic: null
};
function reduce$3(state = initialState, action) {
    const { payload  } = action;
    switch(action.type){
        case BEGIN_DRAG:
            return _objectSpread$2({}, state, {
                itemType: payload.itemType,
                item: payload.item,
                sourceId: payload.sourceId,
                isSourcePublic: payload.isSourcePublic,
                dropResult: null,
                didDrop: false
            });
        case PUBLISH_DRAG_SOURCE:
            return _objectSpread$2({}, state, {
                isSourcePublic: true
            });
        case HOVER:
            return _objectSpread$2({}, state, {
                targetIds: payload.targetIds
            });
        case REMOVE_TARGET:
            if (state.targetIds.indexOf(payload.targetId) === -1) {
                return state;
            }
            return _objectSpread$2({}, state, {
                targetIds: without$1(state.targetIds, payload.targetId)
            });
        case DROP:
            return _objectSpread$2({}, state, {
                dropResult: payload.dropResult,
                didDrop: true,
                targetIds: []
            });
        case END_DRAG:
            return _objectSpread$2({}, state, {
                itemType: null,
                item: null,
                sourceId: null,
                dropResult: null,
                didDrop: false,
                isSourcePublic: null,
                targetIds: []
            });
        default:
            return state;
    }
}

function reduce$2(state = 0, action) {
    switch(action.type){
        case ADD_SOURCE:
        case ADD_TARGET:
            return state + 1;
        case REMOVE_SOURCE:
        case REMOVE_TARGET:
            return state - 1;
        default:
            return state;
    }
}

function reduce$1(state = 0) {
    return state + 1;
}

function _defineProperty$1(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _objectSpread$1(target) {
    for(var i = 1; i < arguments.length; i++){
        var source = arguments[i] != null ? arguments[i] : {};
        var ownKeys = Object.keys(source);
        if (typeof Object.getOwnPropertySymbols === 'function') {
            ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function(sym) {
                return Object.getOwnPropertyDescriptor(source, sym).enumerable;
            }));
        }
        ownKeys.forEach(function(key) {
            _defineProperty$1(target, key, source[key]);
        });
    }
    return target;
}
function reduce(state = {}, action) {
    return {
        dirtyHandlerIds: reduce$5(state.dirtyHandlerIds, {
            type: action.type,
            payload: _objectSpread$1({}, action.payload, {
                prevTargetIds: get(state, 'dragOperation.targetIds', [])
            })
        }),
        dragOffset: reduce$4(state.dragOffset, action),
        refCount: reduce$2(state.refCount, action),
        dragOperation: reduce$3(state.dragOperation, action),
        stateId: reduce$1(state.stateId)
    };
}

function createDragDropManager(backendFactory, globalContext = undefined, backendOptions = {}, debugMode = false) {
    const store = makeStoreInstance(debugMode);
    const monitor = new DragDropMonitorImpl(store, new HandlerRegistryImpl(store));
    const manager = new DragDropManagerImpl(store, monitor);
    const backend = backendFactory(manager, globalContext, backendOptions);
    manager.receiveBackend(backend);
    return manager;
}
function makeStoreInstance(debugMode) {
    // TODO: if we ever make a react-native version of this,
    // we'll need to consider how to pull off dev-tooling
    const reduxDevTools = typeof window !== 'undefined' && window.__REDUX_DEVTOOLS_EXTENSION__;
    return createStore(reduce, debugMode && reduxDevTools && reduxDevTools({
        name: 'dnd-core',
        instanceId: 'dnd-core'
    }));
}

function _objectWithoutProperties(source, excluded) {
    if (source == null) return {};
    var target = _objectWithoutPropertiesLoose(source, excluded);
    var key, i;
    if (Object.getOwnPropertySymbols) {
        var sourceSymbolKeys = Object.getOwnPropertySymbols(source);
        for(i = 0; i < sourceSymbolKeys.length; i++){
            key = sourceSymbolKeys[i];
            if (excluded.indexOf(key) >= 0) continue;
            if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
            target[key] = source[key];
        }
    }
    return target;
}
function _objectWithoutPropertiesLoose(source, excluded) {
    if (source == null) return {};
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;
    for(i = 0; i < sourceKeys.length; i++){
        key = sourceKeys[i];
        if (excluded.indexOf(key) >= 0) continue;
        target[key] = source[key];
    }
    return target;
}
let refCount = 0;
const INSTANCE_SYM = Symbol.for('__REACT_DND_CONTEXT_INSTANCE__');
var DndProvider = /*#__PURE__*/ memo(function DndProvider(_param) {
    var { children  } = _param, props = _objectWithoutProperties(_param, [
        "children"
    ]);
    const [manager, isGlobalInstance] = getDndContextValue(props) // memoized from props
    ;
    /**
		 * If the global context was used to store the DND context
		 * then where theres no more references to it we should
		 * clean it up to avoid memory leaks
		 */ useEffect(()=>{
        if (isGlobalInstance) {
            const context = getGlobalContext();
            ++refCount;
            return ()=>{
                if (--refCount === 0) {
                    context[INSTANCE_SYM] = null;
                }
            };
        }
        return;
    }, []);
    return /*#__PURE__*/ jsx(DndContext.Provider, {
        value: manager,
        children: children
    });
});
function getDndContextValue(props) {
    if ('manager' in props) {
        const manager = {
            dragDropManager: props.manager
        };
        return [
            manager,
            false
        ];
    }
    const manager = createSingletonDndContext(props.backend, props.context, props.options, props.debugMode);
    const isGlobalInstance = !props.context;
    return [
        manager,
        isGlobalInstance
    ];
}
function createSingletonDndContext(backend, context = getGlobalContext(), options, debugMode) {
    const ctx = context;
    if (!ctx[INSTANCE_SYM]) {
        ctx[INSTANCE_SYM] = {
            dragDropManager: createDragDropManager(backend, context, options, debugMode)
        };
    }
    return ctx[INSTANCE_SYM];
}
function getGlobalContext() {
    return typeof global !== 'undefined' ? global : window;
}

// do not edit .js files directly - edit src/index.jst



var fastDeepEqual = function equal(a, b) {
  if (a === b) return true;

  if (a && b && typeof a == 'object' && typeof b == 'object') {
    if (a.constructor !== b.constructor) return false;

    var length, i, keys;
    if (Array.isArray(a)) {
      length = a.length;
      if (length != b.length) return false;
      for (i = length; i-- !== 0;)
        if (!equal(a[i], b[i])) return false;
      return true;
    }



    if (a.constructor === RegExp) return a.source === b.source && a.flags === b.flags;
    if (a.valueOf !== Object.prototype.valueOf) return a.valueOf() === b.valueOf();
    if (a.toString !== Object.prototype.toString) return a.toString() === b.toString();

    keys = Object.keys(a);
    length = keys.length;
    if (length !== Object.keys(b).length) return false;

    for (i = length; i-- !== 0;)
      if (!Object.prototype.hasOwnProperty.call(b, keys[i])) return false;

    for (i = length; i-- !== 0;) {
      var key = keys[i];

      if (!equal(a[key], b[key])) return false;
    }

    return true;
  }

  // true if both NaN, false otherwise
  return a!==a && b!==b;
};

// suppress the useLayoutEffect warning on server side.
const useIsomorphicLayoutEffect = typeof window !== 'undefined' ? useLayoutEffect : useEffect;

/**
 *
 * @param monitor The monitor to collect state from
 * @param collect The collecting function
 * @param onUpdate A method to invoke when updates occur
 */ function useCollector(monitor, collect, onUpdate) {
    const [collected, setCollected] = useState(()=>collect(monitor)
    );
    const updateCollected = useCallback(()=>{
        const nextValue = collect(monitor);
        // This needs to be a deep-equality check because some monitor-collected values
        // include XYCoord objects that may be equivalent, but do not have instance equality.
        if (!fastDeepEqual(collected, nextValue)) {
            setCollected(nextValue);
            if (onUpdate) {
                onUpdate();
            }
        }
    }, [
        collected,
        monitor,
        onUpdate
    ]);
    // update the collected properties after react renders.
    // Note that the "Dustbin Stress Test" fails if this is not
    // done when the component updates
    useIsomorphicLayoutEffect(updateCollected);
    return [
        collected,
        updateCollected
    ];
}

function useMonitorOutput(monitor, collect, onCollect) {
    const [collected, updateCollected] = useCollector(monitor, collect, onCollect);
    useIsomorphicLayoutEffect(function subscribeToMonitorStateChange() {
        const handlerId = monitor.getHandlerId();
        if (handlerId == null) {
            return;
        }
        return monitor.subscribeToStateChange(updateCollected, {
            handlerIds: [
                handlerId
            ]
        });
    }, [
        monitor,
        updateCollected
    ]);
    return collected;
}

function useCollectedProps(collector, monitor, connector) {
    return useMonitorOutput(monitor, collector || (()=>({})
    ), ()=>connector.reconnect()
    );
}

function useOptionalFactory(arg, deps) {
    const memoDeps = [
        ...deps || []
    ];
    if (deps == null && typeof arg !== 'function') {
        memoDeps.push(arg);
    }
    return useMemo(()=>{
        return typeof arg === 'function' ? arg() : arg;
    }, memoDeps);
}

function useConnectDragSource(connector) {
    return useMemo(()=>connector.hooks.dragSource()
    , [
        connector
    ]);
}
function useConnectDragPreview(connector) {
    return useMemo(()=>connector.hooks.dragPreview()
    , [
        connector
    ]);
}

let isCallingCanDrag = false;
let isCallingIsDragging = false;
class DragSourceMonitorImpl {
    receiveHandlerId(sourceId) {
        this.sourceId = sourceId;
    }
    getHandlerId() {
        return this.sourceId;
    }
    canDrag() {
        invariant(!isCallingCanDrag, 'You may not call monitor.canDrag() inside your canDrag() implementation. ' + 'Read more: http://react-dnd.github.io/react-dnd/docs/api/drag-source-monitor');
        try {
            isCallingCanDrag = true;
            return this.internalMonitor.canDragSource(this.sourceId);
        } finally{
            isCallingCanDrag = false;
        }
    }
    isDragging() {
        if (!this.sourceId) {
            return false;
        }
        invariant(!isCallingIsDragging, 'You may not call monitor.isDragging() inside your isDragging() implementation. ' + 'Read more: http://react-dnd.github.io/react-dnd/docs/api/drag-source-monitor');
        try {
            isCallingIsDragging = true;
            return this.internalMonitor.isDraggingSource(this.sourceId);
        } finally{
            isCallingIsDragging = false;
        }
    }
    subscribeToStateChange(listener, options) {
        return this.internalMonitor.subscribeToStateChange(listener, options);
    }
    isDraggingSource(sourceId) {
        return this.internalMonitor.isDraggingSource(sourceId);
    }
    isOverTarget(targetId, options) {
        return this.internalMonitor.isOverTarget(targetId, options);
    }
    getTargetIds() {
        return this.internalMonitor.getTargetIds();
    }
    isSourcePublic() {
        return this.internalMonitor.isSourcePublic();
    }
    getSourceId() {
        return this.internalMonitor.getSourceId();
    }
    subscribeToOffsetChange(listener) {
        return this.internalMonitor.subscribeToOffsetChange(listener);
    }
    canDragSource(sourceId) {
        return this.internalMonitor.canDragSource(sourceId);
    }
    canDropOnTarget(targetId) {
        return this.internalMonitor.canDropOnTarget(targetId);
    }
    getItemType() {
        return this.internalMonitor.getItemType();
    }
    getItem() {
        return this.internalMonitor.getItem();
    }
    getDropResult() {
        return this.internalMonitor.getDropResult();
    }
    didDrop() {
        return this.internalMonitor.didDrop();
    }
    getInitialClientOffset() {
        return this.internalMonitor.getInitialClientOffset();
    }
    getInitialSourceClientOffset() {
        return this.internalMonitor.getInitialSourceClientOffset();
    }
    getSourceClientOffset() {
        return this.internalMonitor.getSourceClientOffset();
    }
    getClientOffset() {
        return this.internalMonitor.getClientOffset();
    }
    getDifferenceFromInitialOffset() {
        return this.internalMonitor.getDifferenceFromInitialOffset();
    }
    constructor(manager){
        this.sourceId = null;
        this.internalMonitor = manager.getMonitor();
    }
}

let isCallingCanDrop = false;
class DropTargetMonitorImpl {
    receiveHandlerId(targetId) {
        this.targetId = targetId;
    }
    getHandlerId() {
        return this.targetId;
    }
    subscribeToStateChange(listener, options) {
        return this.internalMonitor.subscribeToStateChange(listener, options);
    }
    canDrop() {
        // Cut out early if the target id has not been set. This should prevent errors
        // where the user has an older version of dnd-core like in
        // https://github.com/react-dnd/react-dnd/issues/1310
        if (!this.targetId) {
            return false;
        }
        invariant(!isCallingCanDrop, 'You may not call monitor.canDrop() inside your canDrop() implementation. ' + 'Read more: http://react-dnd.github.io/react-dnd/docs/api/drop-target-monitor');
        try {
            isCallingCanDrop = true;
            return this.internalMonitor.canDropOnTarget(this.targetId);
        } finally{
            isCallingCanDrop = false;
        }
    }
    isOver(options) {
        if (!this.targetId) {
            return false;
        }
        return this.internalMonitor.isOverTarget(this.targetId, options);
    }
    getItemType() {
        return this.internalMonitor.getItemType();
    }
    getItem() {
        return this.internalMonitor.getItem();
    }
    getDropResult() {
        return this.internalMonitor.getDropResult();
    }
    didDrop() {
        return this.internalMonitor.didDrop();
    }
    getInitialClientOffset() {
        return this.internalMonitor.getInitialClientOffset();
    }
    getInitialSourceClientOffset() {
        return this.internalMonitor.getInitialSourceClientOffset();
    }
    getSourceClientOffset() {
        return this.internalMonitor.getSourceClientOffset();
    }
    getClientOffset() {
        return this.internalMonitor.getClientOffset();
    }
    getDifferenceFromInitialOffset() {
        return this.internalMonitor.getDifferenceFromInitialOffset();
    }
    constructor(manager){
        this.targetId = null;
        this.internalMonitor = manager.getMonitor();
    }
}

function registerTarget(type, target, manager) {
    const registry = manager.getRegistry();
    const targetId = registry.addTarget(type, target);
    return [
        targetId,
        ()=>registry.removeTarget(targetId)
    ];
}
function registerSource(type, source, manager) {
    const registry = manager.getRegistry();
    const sourceId = registry.addSource(type, source);
    return [
        sourceId,
        ()=>registry.removeSource(sourceId)
    ];
}

function shallowEqual(objA, objB, compare, compareContext) {
    let compareResult = compare ? compare.call(compareContext, objA, objB) : void 0;
    if (compareResult !== void 0) {
        return !!compareResult;
    }
    if (objA === objB) {
        return true;
    }
    if (typeof objA !== 'object' || !objA || typeof objB !== 'object' || !objB) {
        return false;
    }
    const keysA = Object.keys(objA);
    const keysB = Object.keys(objB);
    if (keysA.length !== keysB.length) {
        return false;
    }
    const bHasOwnProperty = Object.prototype.hasOwnProperty.bind(objB);
    // Test for A's keys different from B.
    for(let idx = 0; idx < keysA.length; idx++){
        const key = keysA[idx];
        if (!bHasOwnProperty(key)) {
            return false;
        }
        const valueA = objA[key];
        const valueB = objB[key];
        compareResult = compare ? compare.call(compareContext, valueA, valueB, key) : void 0;
        if (compareResult === false || compareResult === void 0 && valueA !== valueB) {
            return false;
        }
    }
    return true;
}

function isRef(obj) {
    return(// eslint-disable-next-line no-prototype-builtins
    obj !== null && typeof obj === 'object' && Object.prototype.hasOwnProperty.call(obj, 'current'));
}

function throwIfCompositeComponentElement(element) {
    // Custom components can no longer be wrapped directly in React DnD 2.0
    // so that we don't need to depend on findDOMNode() from react-dom.
    if (typeof element.type === 'string') {
        return;
    }
    const displayName = element.type.displayName || element.type.name || 'the component';
    throw new Error('Only native element nodes can now be passed to React DnD connectors.' + `You can either wrap ${displayName} into a <div>, or turn it into a ` + 'drag source or a drop target itself.');
}
function wrapHookToRecognizeElement(hook) {
    return (elementOrNode = null, options = null)=>{
        // When passed a node, call the hook straight away.
        if (!isValidElement(elementOrNode)) {
            const node = elementOrNode;
            hook(node, options);
            // return the node so it can be chained (e.g. when within callback refs
            // <div ref={node => connectDragSource(connectDropTarget(node))}/>
            return node;
        }
        // If passed a ReactElement, clone it and attach this function as a ref.
        // This helps us achieve a neat API where user doesn't even know that refs
        // are being used under the hood.
        const element = elementOrNode;
        throwIfCompositeComponentElement(element);
        // When no options are passed, use the hook directly
        const ref = options ? (node)=>hook(node, options)
         : hook;
        return cloneWithRef(element, ref);
    };
}
function wrapConnectorHooks(hooks) {
    const wrappedHooks = {};
    Object.keys(hooks).forEach((key)=>{
        const hook = hooks[key];
        // ref objects should be passed straight through without wrapping
        if (key.endsWith('Ref')) {
            wrappedHooks[key] = hooks[key];
        } else {
            const wrappedHook = wrapHookToRecognizeElement(hook);
            wrappedHooks[key] = ()=>wrappedHook
            ;
        }
    });
    return wrappedHooks;
}
function setRef(ref, node) {
    if (typeof ref === 'function') {
        ref(node);
    } else {
        ref.current = node;
    }
}
function cloneWithRef(element, newRef) {
    const previousRef = element.ref;
    invariant(typeof previousRef !== 'string', 'Cannot connect React DnD to an element with an existing string ref. ' + 'Please convert it to use a callback ref instead, or wrap it into a <span> or <div>. ' + 'Read more: https://reactjs.org/docs/refs-and-the-dom.html#callback-refs');
    if (!previousRef) {
        // When there is no ref on the element, use the new ref directly
        return cloneElement(element, {
            ref: newRef
        });
    } else {
        return cloneElement(element, {
            ref: (node)=>{
                setRef(previousRef, node);
                setRef(newRef, node);
            }
        });
    }
}

class SourceConnector {
    receiveHandlerId(newHandlerId) {
        if (this.handlerId === newHandlerId) {
            return;
        }
        this.handlerId = newHandlerId;
        this.reconnect();
    }
    get connectTarget() {
        return this.dragSource;
    }
    get dragSourceOptions() {
        return this.dragSourceOptionsInternal;
    }
    set dragSourceOptions(options) {
        this.dragSourceOptionsInternal = options;
    }
    get dragPreviewOptions() {
        return this.dragPreviewOptionsInternal;
    }
    set dragPreviewOptions(options) {
        this.dragPreviewOptionsInternal = options;
    }
    reconnect() {
        const didChange = this.reconnectDragSource();
        this.reconnectDragPreview(didChange);
    }
    reconnectDragSource() {
        const dragSource = this.dragSource;
        // if nothing has changed then don't resubscribe
        const didChange = this.didHandlerIdChange() || this.didConnectedDragSourceChange() || this.didDragSourceOptionsChange();
        if (didChange) {
            this.disconnectDragSource();
        }
        if (!this.handlerId) {
            return didChange;
        }
        if (!dragSource) {
            this.lastConnectedDragSource = dragSource;
            return didChange;
        }
        if (didChange) {
            this.lastConnectedHandlerId = this.handlerId;
            this.lastConnectedDragSource = dragSource;
            this.lastConnectedDragSourceOptions = this.dragSourceOptions;
            this.dragSourceUnsubscribe = this.backend.connectDragSource(this.handlerId, dragSource, this.dragSourceOptions);
        }
        return didChange;
    }
    reconnectDragPreview(forceDidChange = false) {
        const dragPreview = this.dragPreview;
        // if nothing has changed then don't resubscribe
        const didChange = forceDidChange || this.didHandlerIdChange() || this.didConnectedDragPreviewChange() || this.didDragPreviewOptionsChange();
        if (didChange) {
            this.disconnectDragPreview();
        }
        if (!this.handlerId) {
            return;
        }
        if (!dragPreview) {
            this.lastConnectedDragPreview = dragPreview;
            return;
        }
        if (didChange) {
            this.lastConnectedHandlerId = this.handlerId;
            this.lastConnectedDragPreview = dragPreview;
            this.lastConnectedDragPreviewOptions = this.dragPreviewOptions;
            this.dragPreviewUnsubscribe = this.backend.connectDragPreview(this.handlerId, dragPreview, this.dragPreviewOptions);
        }
    }
    didHandlerIdChange() {
        return this.lastConnectedHandlerId !== this.handlerId;
    }
    didConnectedDragSourceChange() {
        return this.lastConnectedDragSource !== this.dragSource;
    }
    didConnectedDragPreviewChange() {
        return this.lastConnectedDragPreview !== this.dragPreview;
    }
    didDragSourceOptionsChange() {
        return !shallowEqual(this.lastConnectedDragSourceOptions, this.dragSourceOptions);
    }
    didDragPreviewOptionsChange() {
        return !shallowEqual(this.lastConnectedDragPreviewOptions, this.dragPreviewOptions);
    }
    disconnectDragSource() {
        if (this.dragSourceUnsubscribe) {
            this.dragSourceUnsubscribe();
            this.dragSourceUnsubscribe = undefined;
        }
    }
    disconnectDragPreview() {
        if (this.dragPreviewUnsubscribe) {
            this.dragPreviewUnsubscribe();
            this.dragPreviewUnsubscribe = undefined;
            this.dragPreviewNode = null;
            this.dragPreviewRef = null;
        }
    }
    get dragSource() {
        return this.dragSourceNode || this.dragSourceRef && this.dragSourceRef.current;
    }
    get dragPreview() {
        return this.dragPreviewNode || this.dragPreviewRef && this.dragPreviewRef.current;
    }
    clearDragSource() {
        this.dragSourceNode = null;
        this.dragSourceRef = null;
    }
    clearDragPreview() {
        this.dragPreviewNode = null;
        this.dragPreviewRef = null;
    }
    constructor(backend){
        this.hooks = wrapConnectorHooks({
            dragSource: (node, options)=>{
                this.clearDragSource();
                this.dragSourceOptions = options || null;
                if (isRef(node)) {
                    this.dragSourceRef = node;
                } else {
                    this.dragSourceNode = node;
                }
                this.reconnectDragSource();
            },
            dragPreview: (node, options)=>{
                this.clearDragPreview();
                this.dragPreviewOptions = options || null;
                if (isRef(node)) {
                    this.dragPreviewRef = node;
                } else {
                    this.dragPreviewNode = node;
                }
                this.reconnectDragPreview();
            }
        });
        this.handlerId = null;
        // The drop target may either be attached via ref or connect function
        this.dragSourceRef = null;
        this.dragSourceOptionsInternal = null;
        // The drag preview may either be attached via ref or connect function
        this.dragPreviewRef = null;
        this.dragPreviewOptionsInternal = null;
        this.lastConnectedHandlerId = null;
        this.lastConnectedDragSource = null;
        this.lastConnectedDragSourceOptions = null;
        this.lastConnectedDragPreview = null;
        this.lastConnectedDragPreviewOptions = null;
        this.backend = backend;
    }
}

class TargetConnector {
    get connectTarget() {
        return this.dropTarget;
    }
    reconnect() {
        // if nothing has changed then don't resubscribe
        const didChange = this.didHandlerIdChange() || this.didDropTargetChange() || this.didOptionsChange();
        if (didChange) {
            this.disconnectDropTarget();
        }
        const dropTarget = this.dropTarget;
        if (!this.handlerId) {
            return;
        }
        if (!dropTarget) {
            this.lastConnectedDropTarget = dropTarget;
            return;
        }
        if (didChange) {
            this.lastConnectedHandlerId = this.handlerId;
            this.lastConnectedDropTarget = dropTarget;
            this.lastConnectedDropTargetOptions = this.dropTargetOptions;
            this.unsubscribeDropTarget = this.backend.connectDropTarget(this.handlerId, dropTarget, this.dropTargetOptions);
        }
    }
    receiveHandlerId(newHandlerId) {
        if (newHandlerId === this.handlerId) {
            return;
        }
        this.handlerId = newHandlerId;
        this.reconnect();
    }
    get dropTargetOptions() {
        return this.dropTargetOptionsInternal;
    }
    set dropTargetOptions(options) {
        this.dropTargetOptionsInternal = options;
    }
    didHandlerIdChange() {
        return this.lastConnectedHandlerId !== this.handlerId;
    }
    didDropTargetChange() {
        return this.lastConnectedDropTarget !== this.dropTarget;
    }
    didOptionsChange() {
        return !shallowEqual(this.lastConnectedDropTargetOptions, this.dropTargetOptions);
    }
    disconnectDropTarget() {
        if (this.unsubscribeDropTarget) {
            this.unsubscribeDropTarget();
            this.unsubscribeDropTarget = undefined;
        }
    }
    get dropTarget() {
        return this.dropTargetNode || this.dropTargetRef && this.dropTargetRef.current;
    }
    clearDropTarget() {
        this.dropTargetRef = null;
        this.dropTargetNode = null;
    }
    constructor(backend){
        this.hooks = wrapConnectorHooks({
            dropTarget: (node, options)=>{
                this.clearDropTarget();
                this.dropTargetOptions = options;
                if (isRef(node)) {
                    this.dropTargetRef = node;
                } else {
                    this.dropTargetNode = node;
                }
                this.reconnect();
            }
        });
        this.handlerId = null;
        // The drop target may either be attached via ref or connect function
        this.dropTargetRef = null;
        this.dropTargetOptionsInternal = null;
        this.lastConnectedHandlerId = null;
        this.lastConnectedDropTarget = null;
        this.lastConnectedDropTargetOptions = null;
        this.backend = backend;
    }
}

/**
 * A hook to retrieve the DragDropManager from Context
 */ function useDragDropManager() {
    const { dragDropManager  } = useContext(DndContext);
    invariant(dragDropManager != null, 'Expected drag drop context');
    return dragDropManager;
}

function useDragSourceConnector(dragSourceOptions, dragPreviewOptions) {
    const manager = useDragDropManager();
    const connector = useMemo(()=>new SourceConnector(manager.getBackend())
    , [
        manager
    ]);
    useIsomorphicLayoutEffect(()=>{
        connector.dragSourceOptions = dragSourceOptions || null;
        connector.reconnect();
        return ()=>connector.disconnectDragSource()
        ;
    }, [
        connector,
        dragSourceOptions
    ]);
    useIsomorphicLayoutEffect(()=>{
        connector.dragPreviewOptions = dragPreviewOptions || null;
        connector.reconnect();
        return ()=>connector.disconnectDragPreview()
        ;
    }, [
        connector,
        dragPreviewOptions
    ]);
    return connector;
}

function useDragSourceMonitor() {
    const manager = useDragDropManager();
    return useMemo(()=>new DragSourceMonitorImpl(manager)
    , [
        manager
    ]);
}

class DragSourceImpl {
    beginDrag() {
        const spec = this.spec;
        const monitor = this.monitor;
        let result = null;
        if (typeof spec.item === 'object') {
            result = spec.item;
        } else if (typeof spec.item === 'function') {
            result = spec.item(monitor);
        } else {
            result = {};
        }
        return result !== null && result !== void 0 ? result : null;
    }
    canDrag() {
        const spec = this.spec;
        const monitor = this.monitor;
        if (typeof spec.canDrag === 'boolean') {
            return spec.canDrag;
        } else if (typeof spec.canDrag === 'function') {
            return spec.canDrag(monitor);
        } else {
            return true;
        }
    }
    isDragging(globalMonitor, target) {
        const spec = this.spec;
        const monitor = this.monitor;
        const { isDragging  } = spec;
        return isDragging ? isDragging(monitor) : target === globalMonitor.getSourceId();
    }
    endDrag() {
        const spec = this.spec;
        const monitor = this.monitor;
        const connector = this.connector;
        const { end  } = spec;
        if (end) {
            end(monitor.getItem(), monitor);
        }
        connector.reconnect();
    }
    constructor(spec, monitor, connector){
        this.spec = spec;
        this.monitor = monitor;
        this.connector = connector;
    }
}

function useDragSource(spec, monitor, connector) {
    const handler = useMemo(()=>new DragSourceImpl(spec, monitor, connector)
    , [
        monitor,
        connector
    ]);
    useEffect(()=>{
        handler.spec = spec;
    }, [
        spec
    ]);
    return handler;
}

function useDragType(spec) {
    return useMemo(()=>{
        const result = spec.type;
        invariant(result != null, 'spec.type must be defined');
        return result;
    }, [
        spec
    ]);
}

function useRegisteredDragSource(spec, monitor, connector) {
    const manager = useDragDropManager();
    const handler = useDragSource(spec, monitor, connector);
    const itemType = useDragType(spec);
    useIsomorphicLayoutEffect(function registerDragSource() {
        if (itemType != null) {
            const [handlerId, unregister] = registerSource(itemType, handler, manager);
            monitor.receiveHandlerId(handlerId);
            connector.receiveHandlerId(handlerId);
            return unregister;
        }
        return;
    }, [
        manager,
        monitor,
        connector,
        handler,
        itemType
    ]);
}

/**
 * useDragSource hook
 * @param sourceSpec The drag source specification (object or function, function preferred)
 * @param deps The memoization deps array to use when evaluating spec changes
 */ function useDrag(specArg, deps) {
    const spec = useOptionalFactory(specArg, deps);
    invariant(!spec.begin, `useDrag::spec.begin was deprecated in v14. Replace spec.begin() with spec.item(). (see more here - https://react-dnd.github.io/react-dnd/docs/api/use-drag)`);
    const monitor = useDragSourceMonitor();
    const connector = useDragSourceConnector(spec.options, spec.previewOptions);
    useRegisteredDragSource(spec, monitor, connector);
    return [
        useCollectedProps(spec.collect, monitor, connector),
        useConnectDragSource(connector),
        useConnectDragPreview(connector), 
    ];
}

function useConnectDropTarget(connector) {
    return useMemo(()=>connector.hooks.dropTarget()
    , [
        connector
    ]);
}

function useDropTargetConnector(options) {
    const manager = useDragDropManager();
    const connector = useMemo(()=>new TargetConnector(manager.getBackend())
    , [
        manager
    ]);
    useIsomorphicLayoutEffect(()=>{
        connector.dropTargetOptions = options || null;
        connector.reconnect();
        return ()=>connector.disconnectDropTarget()
        ;
    }, [
        options
    ]);
    return connector;
}

function useDropTargetMonitor() {
    const manager = useDragDropManager();
    return useMemo(()=>new DropTargetMonitorImpl(manager)
    , [
        manager
    ]);
}

/**
 * Internal utility hook to get an array-version of spec.accept.
 * The main utility here is that we aren't creating a new array on every render if a non-array spec.accept is passed in.
 * @param spec
 */ function useAccept(spec) {
    const { accept  } = spec;
    return useMemo(()=>{
        invariant(spec.accept != null, 'accept must be defined');
        return Array.isArray(accept) ? accept : [
            accept
        ];
    }, [
        accept
    ]);
}

class DropTargetImpl {
    canDrop() {
        const spec = this.spec;
        const monitor = this.monitor;
        return spec.canDrop ? spec.canDrop(monitor.getItem(), monitor) : true;
    }
    hover() {
        const spec = this.spec;
        const monitor = this.monitor;
        if (spec.hover) {
            spec.hover(monitor.getItem(), monitor);
        }
    }
    drop() {
        const spec = this.spec;
        const monitor = this.monitor;
        if (spec.drop) {
            return spec.drop(monitor.getItem(), monitor);
        }
        return;
    }
    constructor(spec, monitor){
        this.spec = spec;
        this.monitor = monitor;
    }
}

function useDropTarget(spec, monitor) {
    const dropTarget = useMemo(()=>new DropTargetImpl(spec, monitor)
    , [
        monitor
    ]);
    useEffect(()=>{
        dropTarget.spec = spec;
    }, [
        spec
    ]);
    return dropTarget;
}

function useRegisteredDropTarget(spec, monitor, connector) {
    const manager = useDragDropManager();
    const dropTarget = useDropTarget(spec, monitor);
    const accept = useAccept(spec);
    useIsomorphicLayoutEffect(function registerDropTarget() {
        const [handlerId, unregister] = registerTarget(accept, dropTarget, manager);
        monitor.receiveHandlerId(handlerId);
        connector.receiveHandlerId(handlerId);
        return unregister;
    }, [
        manager,
        monitor,
        dropTarget,
        connector,
        accept.map((a)=>a.toString()
        ).join('|'), 
    ]);
}

/**
 * useDropTarget Hook
 * @param spec The drop target specification (object or function, function preferred)
 * @param deps The memoization deps array to use when evaluating spec changes
 */ function useDrop(specArg, deps) {
    const spec = useOptionalFactory(specArg, deps);
    const monitor = useDropTargetMonitor();
    const connector = useDropTargetConnector(spec.options);
    useRegisteredDropTarget(spec, monitor, connector);
    return [
        useCollectedProps(spec.collect, monitor, connector),
        useConnectDropTarget(connector), 
    ];
}

// cheap lodash replacements
function memoize(fn) {
    let result = null;
    const memoized = ()=>{
        if (result == null) {
            result = fn();
        }
        return result;
    };
    return memoized;
}
/**
 * drop-in replacement for _.without
 */ function without(items, item) {
    return items.filter((i)=>i !== item
    );
}
function union(itemsA, itemsB) {
    const set = new Set();
    const insertItem = (item)=>set.add(item)
    ;
    itemsA.forEach(insertItem);
    itemsB.forEach(insertItem);
    const result = [];
    set.forEach((key)=>result.push(key)
    );
    return result;
}

class EnterLeaveCounter {
    enter(enteringNode) {
        const previousLength = this.entered.length;
        const isNodeEntered = (node)=>this.isNodeInDocument(node) && (!node.contains || node.contains(enteringNode))
        ;
        this.entered = union(this.entered.filter(isNodeEntered), [
            enteringNode
        ]);
        return previousLength === 0 && this.entered.length > 0;
    }
    leave(leavingNode) {
        const previousLength = this.entered.length;
        this.entered = without(this.entered.filter(this.isNodeInDocument), leavingNode);
        return previousLength > 0 && this.entered.length === 0;
    }
    reset() {
        this.entered = [];
    }
    constructor(isNodeInDocument){
        this.entered = [];
        this.isNodeInDocument = isNodeInDocument;
    }
}

class NativeDragSource {
    initializeExposedProperties() {
        Object.keys(this.config.exposeProperties).forEach((property)=>{
            Object.defineProperty(this.item, property, {
                configurable: true,
                enumerable: true,
                get () {
                    // eslint-disable-next-line no-console
                    console.warn(`Browser doesn't allow reading "${property}" until the drop event.`);
                    return null;
                }
            });
        });
    }
    loadDataTransfer(dataTransfer) {
        if (dataTransfer) {
            const newProperties = {};
            Object.keys(this.config.exposeProperties).forEach((property)=>{
                const propertyFn = this.config.exposeProperties[property];
                if (propertyFn != null) {
                    newProperties[property] = {
                        value: propertyFn(dataTransfer, this.config.matchesTypes),
                        configurable: true,
                        enumerable: true
                    };
                }
            });
            Object.defineProperties(this.item, newProperties);
        }
    }
    canDrag() {
        return true;
    }
    beginDrag() {
        return this.item;
    }
    isDragging(monitor, handle) {
        return handle === monitor.getSourceId();
    }
    endDrag() {
    // empty
    }
    constructor(config){
        this.config = config;
        this.item = {};
        this.initializeExposedProperties();
    }
}

const FILE = '__NATIVE_FILE__';
const URL = '__NATIVE_URL__';
const TEXT = '__NATIVE_TEXT__';
const HTML = '__NATIVE_HTML__';

var NativeTypes = /*#__PURE__*/Object.freeze({
  __proto__: null,
  FILE: FILE,
  URL: URL,
  TEXT: TEXT,
  HTML: HTML
});

function getDataFromDataTransfer(dataTransfer, typesToTry, defaultValue) {
    const result = typesToTry.reduce((resultSoFar, typeToTry)=>resultSoFar || dataTransfer.getData(typeToTry)
    , '');
    return result != null ? result : defaultValue;
}

const nativeTypesConfig = {
    [FILE]: {
        exposeProperties: {
            files: (dataTransfer)=>Array.prototype.slice.call(dataTransfer.files)
            ,
            items: (dataTransfer)=>dataTransfer.items
            ,
            dataTransfer: (dataTransfer)=>dataTransfer
        },
        matchesTypes: [
            'Files'
        ]
    },
    [HTML]: {
        exposeProperties: {
            html: (dataTransfer, matchesTypes)=>getDataFromDataTransfer(dataTransfer, matchesTypes, '')
            ,
            dataTransfer: (dataTransfer)=>dataTransfer
        },
        matchesTypes: [
            'Html',
            'text/html'
        ]
    },
    [URL]: {
        exposeProperties: {
            urls: (dataTransfer, matchesTypes)=>getDataFromDataTransfer(dataTransfer, matchesTypes, '').split('\n')
            ,
            dataTransfer: (dataTransfer)=>dataTransfer
        },
        matchesTypes: [
            'Url',
            'text/uri-list'
        ]
    },
    [TEXT]: {
        exposeProperties: {
            text: (dataTransfer, matchesTypes)=>getDataFromDataTransfer(dataTransfer, matchesTypes, '')
            ,
            dataTransfer: (dataTransfer)=>dataTransfer
        },
        matchesTypes: [
            'Text',
            'text/plain'
        ]
    }
};

function createNativeDragSource(type, dataTransfer) {
    const config = nativeTypesConfig[type];
    if (!config) {
        throw new Error(`native type ${type} has no configuration`);
    }
    const result = new NativeDragSource(config);
    result.loadDataTransfer(dataTransfer);
    return result;
}
function matchNativeItemType(dataTransfer) {
    if (!dataTransfer) {
        return null;
    }
    const dataTransferTypes = Array.prototype.slice.call(dataTransfer.types || []);
    return Object.keys(nativeTypesConfig).filter((nativeItemType)=>{
        const typeConfig = nativeTypesConfig[nativeItemType];
        if (!(typeConfig === null || typeConfig === void 0 ? void 0 : typeConfig.matchesTypes)) {
            return false;
        }
        return typeConfig.matchesTypes.some((t)=>dataTransferTypes.indexOf(t) > -1
        );
    })[0] || null;
}

const isFirefox = memoize(()=>/firefox/i.test(navigator.userAgent)
);
const isSafari = memoize(()=>Boolean(window.safari)
);

class MonotonicInterpolant {
    interpolate(x) {
        const { xs , ys , c1s , c2s , c3s  } = this;
        // The rightmost point in the dataset should give an exact result
        let i = xs.length - 1;
        if (x === xs[i]) {
            return ys[i];
        }
        // Search for the interval x is in, returning the corresponding y if x is one of the original xs
        let low = 0;
        let high = c3s.length - 1;
        let mid;
        while(low <= high){
            mid = Math.floor(0.5 * (low + high));
            const xHere = xs[mid];
            if (xHere < x) {
                low = mid + 1;
            } else if (xHere > x) {
                high = mid - 1;
            } else {
                return ys[mid];
            }
        }
        i = Math.max(0, high);
        // Interpolate
        const diff = x - xs[i];
        const diffSq = diff * diff;
        return ys[i] + c1s[i] * diff + c2s[i] * diffSq + c3s[i] * diff * diffSq;
    }
    constructor(xs, ys){
        const { length  } = xs;
        // Rearrange xs and ys so that xs is sorted
        const indexes = [];
        for(let i = 0; i < length; i++){
            indexes.push(i);
        }
        indexes.sort((a, b)=>xs[a] < xs[b] ? -1 : 1
        );
        const dxs = [];
        const ms = [];
        let dx;
        let dy;
        for(let i1 = 0; i1 < length - 1; i1++){
            dx = xs[i1 + 1] - xs[i1];
            dy = ys[i1 + 1] - ys[i1];
            dxs.push(dx);
            ms.push(dy / dx);
        }
        // Get degree-1 coefficients
        const c1s = [
            ms[0]
        ];
        for(let i2 = 0; i2 < dxs.length - 1; i2++){
            const m2 = ms[i2];
            const mNext = ms[i2 + 1];
            if (m2 * mNext <= 0) {
                c1s.push(0);
            } else {
                dx = dxs[i2];
                const dxNext = dxs[i2 + 1];
                const common = dx + dxNext;
                c1s.push(3 * common / ((common + dxNext) / m2 + (common + dx) / mNext));
            }
        }
        c1s.push(ms[ms.length - 1]);
        // Get degree-2 and degree-3 coefficients
        const c2s = [];
        const c3s = [];
        let m;
        for(let i3 = 0; i3 < c1s.length - 1; i3++){
            m = ms[i3];
            const c1 = c1s[i3];
            const invDx = 1 / dxs[i3];
            const common = c1 + c1s[i3 + 1] - m - m;
            c2s.push((m - c1 - common) * invDx);
            c3s.push(common * invDx * invDx);
        }
        this.xs = xs;
        this.ys = ys;
        this.c1s = c1s;
        this.c2s = c2s;
        this.c3s = c3s;
    }
}

const ELEMENT_NODE = 1;
function getNodeClientOffset(node) {
    const el = node.nodeType === ELEMENT_NODE ? node : node.parentElement;
    if (!el) {
        return null;
    }
    const { top , left  } = el.getBoundingClientRect();
    return {
        x: left,
        y: top
    };
}
function getEventClientOffset(e) {
    return {
        x: e.clientX,
        y: e.clientY
    };
}
function isImageNode(node) {
    var ref;
    return node.nodeName === 'IMG' && (isFirefox() || !((ref = document.documentElement) === null || ref === void 0 ? void 0 : ref.contains(node)));
}
function getDragPreviewSize(isImage, dragPreview, sourceWidth, sourceHeight) {
    let dragPreviewWidth = isImage ? dragPreview.width : sourceWidth;
    let dragPreviewHeight = isImage ? dragPreview.height : sourceHeight;
    // Work around @2x coordinate discrepancies in browsers
    if (isSafari() && isImage) {
        dragPreviewHeight /= window.devicePixelRatio;
        dragPreviewWidth /= window.devicePixelRatio;
    }
    return {
        dragPreviewWidth,
        dragPreviewHeight
    };
}
function getDragPreviewOffset(sourceNode, dragPreview, clientOffset, anchorPoint, offsetPoint) {
    // The browsers will use the image intrinsic size under different conditions.
    // Firefox only cares if it's an image, but WebKit also wants it to be detached.
    const isImage = isImageNode(dragPreview);
    const dragPreviewNode = isImage ? sourceNode : dragPreview;
    const dragPreviewNodeOffsetFromClient = getNodeClientOffset(dragPreviewNode);
    const offsetFromDragPreview = {
        x: clientOffset.x - dragPreviewNodeOffsetFromClient.x,
        y: clientOffset.y - dragPreviewNodeOffsetFromClient.y
    };
    const { offsetWidth: sourceWidth , offsetHeight: sourceHeight  } = sourceNode;
    const { anchorX , anchorY  } = anchorPoint;
    const { dragPreviewWidth , dragPreviewHeight  } = getDragPreviewSize(isImage, dragPreview, sourceWidth, sourceHeight);
    const calculateYOffset = ()=>{
        const interpolantY = new MonotonicInterpolant([
            0,
            0.5,
            1
        ], [
            // Dock to the top
            offsetFromDragPreview.y,
            // Align at the center
            (offsetFromDragPreview.y / sourceHeight) * dragPreviewHeight,
            // Dock to the bottom
            offsetFromDragPreview.y + dragPreviewHeight - sourceHeight, 
        ]);
        let y = interpolantY.interpolate(anchorY);
        // Work around Safari 8 positioning bug
        if (isSafari() && isImage) {
            // We'll have to wait for @3x to see if this is entirely correct
            y += (window.devicePixelRatio - 1) * dragPreviewHeight;
        }
        return y;
    };
    const calculateXOffset = ()=>{
        // Interpolate coordinates depending on anchor point
        // If you know a simpler way to do this, let me know
        const interpolantX = new MonotonicInterpolant([
            0,
            0.5,
            1
        ], [
            // Dock to the left
            offsetFromDragPreview.x,
            // Align at the center
            (offsetFromDragPreview.x / sourceWidth) * dragPreviewWidth,
            // Dock to the right
            offsetFromDragPreview.x + dragPreviewWidth - sourceWidth, 
        ]);
        return interpolantX.interpolate(anchorX);
    };
    // Force offsets if specified in the options.
    const { offsetX , offsetY  } = offsetPoint;
    const isManualOffsetX = offsetX === 0 || offsetX;
    const isManualOffsetY = offsetY === 0 || offsetY;
    return {
        x: isManualOffsetX ? offsetX : calculateXOffset(),
        y: isManualOffsetY ? offsetY : calculateYOffset()
    };
}

class OptionsReader {
    get window() {
        if (this.globalContext) {
            return this.globalContext;
        } else if (typeof window !== 'undefined') {
            return window;
        }
        return undefined;
    }
    get document() {
        var ref;
        if ((ref = this.globalContext) === null || ref === void 0 ? void 0 : ref.document) {
            return this.globalContext.document;
        } else if (this.window) {
            return this.window.document;
        } else {
            return undefined;
        }
    }
    get rootElement() {
        var ref;
        return ((ref = this.optionsArgs) === null || ref === void 0 ? void 0 : ref.rootElement) || this.window;
    }
    constructor(globalContext, options){
        this.ownerDocument = null;
        this.globalContext = globalContext;
        this.optionsArgs = options;
    }
}

function _defineProperty(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _objectSpread(target) {
    for(var i = 1; i < arguments.length; i++){
        var source = arguments[i] != null ? arguments[i] : {};
        var ownKeys = Object.keys(source);
        if (typeof Object.getOwnPropertySymbols === 'function') {
            ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function(sym) {
                return Object.getOwnPropertyDescriptor(source, sym).enumerable;
            }));
        }
        ownKeys.forEach(function(key) {
            _defineProperty(target, key, source[key]);
        });
    }
    return target;
}
class HTML5BackendImpl {
    /**
	 * Generate profiling statistics for the HTML5Backend.
	 */ profile() {
        var ref, ref1;
        return {
            sourcePreviewNodes: this.sourcePreviewNodes.size,
            sourcePreviewNodeOptions: this.sourcePreviewNodeOptions.size,
            sourceNodeOptions: this.sourceNodeOptions.size,
            sourceNodes: this.sourceNodes.size,
            dragStartSourceIds: ((ref = this.dragStartSourceIds) === null || ref === void 0 ? void 0 : ref.length) || 0,
            dropTargetIds: this.dropTargetIds.length,
            dragEnterTargetIds: this.dragEnterTargetIds.length,
            dragOverTargetIds: ((ref1 = this.dragOverTargetIds) === null || ref1 === void 0 ? void 0 : ref1.length) || 0
        };
    }
    // public for test
    get window() {
        return this.options.window;
    }
    get document() {
        return this.options.document;
    }
    /**
	 * Get the root element to use for event subscriptions
	 */ get rootElement() {
        return this.options.rootElement;
    }
    setup() {
        const root = this.rootElement;
        if (root === undefined) {
            return;
        }
        if (root.__isReactDndBackendSetUp) {
            throw new Error('Cannot have two HTML5 backends at the same time.');
        }
        root.__isReactDndBackendSetUp = true;
        this.addEventListeners(root);
    }
    teardown() {
        const root = this.rootElement;
        if (root === undefined) {
            return;
        }
        root.__isReactDndBackendSetUp = false;
        this.removeEventListeners(this.rootElement);
        this.clearCurrentDragSourceNode();
        if (this.asyncEndDragFrameId) {
            var ref;
            (ref = this.window) === null || ref === void 0 ? void 0 : ref.cancelAnimationFrame(this.asyncEndDragFrameId);
        }
    }
    connectDragPreview(sourceId, node, options) {
        this.sourcePreviewNodeOptions.set(sourceId, options);
        this.sourcePreviewNodes.set(sourceId, node);
        return ()=>{
            this.sourcePreviewNodes.delete(sourceId);
            this.sourcePreviewNodeOptions.delete(sourceId);
        };
    }
    connectDragSource(sourceId, node, options) {
        this.sourceNodes.set(sourceId, node);
        this.sourceNodeOptions.set(sourceId, options);
        const handleDragStart = (e)=>this.handleDragStart(e, sourceId)
        ;
        const handleSelectStart = (e)=>this.handleSelectStart(e)
        ;
        node.setAttribute('draggable', 'true');
        node.addEventListener('dragstart', handleDragStart);
        node.addEventListener('selectstart', handleSelectStart);
        return ()=>{
            this.sourceNodes.delete(sourceId);
            this.sourceNodeOptions.delete(sourceId);
            node.removeEventListener('dragstart', handleDragStart);
            node.removeEventListener('selectstart', handleSelectStart);
            node.setAttribute('draggable', 'false');
        };
    }
    connectDropTarget(targetId, node) {
        const handleDragEnter = (e)=>this.handleDragEnter(e, targetId)
        ;
        const handleDragOver = (e)=>this.handleDragOver(e, targetId)
        ;
        const handleDrop = (e)=>this.handleDrop(e, targetId)
        ;
        node.addEventListener('dragenter', handleDragEnter);
        node.addEventListener('dragover', handleDragOver);
        node.addEventListener('drop', handleDrop);
        return ()=>{
            node.removeEventListener('dragenter', handleDragEnter);
            node.removeEventListener('dragover', handleDragOver);
            node.removeEventListener('drop', handleDrop);
        };
    }
    addEventListeners(target) {
        // SSR Fix (https://github.com/react-dnd/react-dnd/pull/813
        if (!target.addEventListener) {
            return;
        }
        target.addEventListener('dragstart', this.handleTopDragStart);
        target.addEventListener('dragstart', this.handleTopDragStartCapture, true);
        target.addEventListener('dragend', this.handleTopDragEndCapture, true);
        target.addEventListener('dragenter', this.handleTopDragEnter);
        target.addEventListener('dragenter', this.handleTopDragEnterCapture, true);
        target.addEventListener('dragleave', this.handleTopDragLeaveCapture, true);
        target.addEventListener('dragover', this.handleTopDragOver);
        target.addEventListener('dragover', this.handleTopDragOverCapture, true);
        target.addEventListener('drop', this.handleTopDrop);
        target.addEventListener('drop', this.handleTopDropCapture, true);
    }
    removeEventListeners(target) {
        // SSR Fix (https://github.com/react-dnd/react-dnd/pull/813
        if (!target.removeEventListener) {
            return;
        }
        target.removeEventListener('dragstart', this.handleTopDragStart);
        target.removeEventListener('dragstart', this.handleTopDragStartCapture, true);
        target.removeEventListener('dragend', this.handleTopDragEndCapture, true);
        target.removeEventListener('dragenter', this.handleTopDragEnter);
        target.removeEventListener('dragenter', this.handleTopDragEnterCapture, true);
        target.removeEventListener('dragleave', this.handleTopDragLeaveCapture, true);
        target.removeEventListener('dragover', this.handleTopDragOver);
        target.removeEventListener('dragover', this.handleTopDragOverCapture, true);
        target.removeEventListener('drop', this.handleTopDrop);
        target.removeEventListener('drop', this.handleTopDropCapture, true);
    }
    getCurrentSourceNodeOptions() {
        const sourceId = this.monitor.getSourceId();
        const sourceNodeOptions = this.sourceNodeOptions.get(sourceId);
        return _objectSpread({
            dropEffect: this.altKeyPressed ? 'copy' : 'move'
        }, sourceNodeOptions || {});
    }
    getCurrentDropEffect() {
        if (this.isDraggingNativeItem()) {
            // It makes more sense to default to 'copy' for native resources
            return 'copy';
        }
        return this.getCurrentSourceNodeOptions().dropEffect;
    }
    getCurrentSourcePreviewNodeOptions() {
        const sourceId = this.monitor.getSourceId();
        const sourcePreviewNodeOptions = this.sourcePreviewNodeOptions.get(sourceId);
        return _objectSpread({
            anchorX: 0.5,
            anchorY: 0.5,
            captureDraggingState: false
        }, sourcePreviewNodeOptions || {});
    }
    isDraggingNativeItem() {
        const itemType = this.monitor.getItemType();
        return Object.keys(NativeTypes).some((key)=>NativeTypes[key] === itemType
        );
    }
    beginDragNativeItem(type, dataTransfer) {
        this.clearCurrentDragSourceNode();
        this.currentNativeSource = createNativeDragSource(type, dataTransfer);
        this.currentNativeHandle = this.registry.addSource(type, this.currentNativeSource);
        this.actions.beginDrag([
            this.currentNativeHandle
        ]);
    }
    setCurrentDragSourceNode(node) {
        this.clearCurrentDragSourceNode();
        this.currentDragSourceNode = node;
        // A timeout of > 0 is necessary to resolve Firefox issue referenced
        // See:
        //   * https://github.com/react-dnd/react-dnd/pull/928
        //   * https://github.com/react-dnd/react-dnd/issues/869
        const MOUSE_MOVE_TIMEOUT = 1000;
        // Receiving a mouse event in the middle of a dragging operation
        // means it has ended and the drag source node disappeared from DOM,
        // so the browser didn't dispatch the dragend event.
        //
        // We need to wait before we start listening for mousemove events.
        // This is needed because the drag preview needs to be drawn or else it fires an 'mousemove' event
        // immediately in some browsers.
        //
        // See:
        //   * https://github.com/react-dnd/react-dnd/pull/928
        //   * https://github.com/react-dnd/react-dnd/issues/869
        //
        this.mouseMoveTimeoutTimer = setTimeout(()=>{
            var ref;
            return (ref = this.rootElement) === null || ref === void 0 ? void 0 : ref.addEventListener('mousemove', this.endDragIfSourceWasRemovedFromDOM, true);
        }, MOUSE_MOVE_TIMEOUT);
    }
    clearCurrentDragSourceNode() {
        if (this.currentDragSourceNode) {
            this.currentDragSourceNode = null;
            if (this.rootElement) {
                var ref;
                (ref = this.window) === null || ref === void 0 ? void 0 : ref.clearTimeout(this.mouseMoveTimeoutTimer || undefined);
                this.rootElement.removeEventListener('mousemove', this.endDragIfSourceWasRemovedFromDOM, true);
            }
            this.mouseMoveTimeoutTimer = null;
            return true;
        }
        return false;
    }
    handleDragStart(e, sourceId) {
        if (e.defaultPrevented) {
            return;
        }
        if (!this.dragStartSourceIds) {
            this.dragStartSourceIds = [];
        }
        this.dragStartSourceIds.unshift(sourceId);
    }
    handleDragEnter(_e, targetId) {
        this.dragEnterTargetIds.unshift(targetId);
    }
    handleDragOver(_e, targetId) {
        if (this.dragOverTargetIds === null) {
            this.dragOverTargetIds = [];
        }
        this.dragOverTargetIds.unshift(targetId);
    }
    handleDrop(_e, targetId) {
        this.dropTargetIds.unshift(targetId);
    }
    constructor(manager, globalContext, options){
        this.sourcePreviewNodes = new Map();
        this.sourcePreviewNodeOptions = new Map();
        this.sourceNodes = new Map();
        this.sourceNodeOptions = new Map();
        this.dragStartSourceIds = null;
        this.dropTargetIds = [];
        this.dragEnterTargetIds = [];
        this.currentNativeSource = null;
        this.currentNativeHandle = null;
        this.currentDragSourceNode = null;
        this.altKeyPressed = false;
        this.mouseMoveTimeoutTimer = null;
        this.asyncEndDragFrameId = null;
        this.dragOverTargetIds = null;
        this.lastClientOffset = null;
        this.hoverRafId = null;
        this.getSourceClientOffset = (sourceId)=>{
            const source = this.sourceNodes.get(sourceId);
            return source && getNodeClientOffset(source) || null;
        };
        this.endDragNativeItem = ()=>{
            if (!this.isDraggingNativeItem()) {
                return;
            }
            this.actions.endDrag();
            if (this.currentNativeHandle) {
                this.registry.removeSource(this.currentNativeHandle);
            }
            this.currentNativeHandle = null;
            this.currentNativeSource = null;
        };
        this.isNodeInDocument = (node)=>{
            // Check the node either in the main document or in the current context
            return Boolean(node && this.document && this.document.body && this.document.body.contains(node));
        };
        this.endDragIfSourceWasRemovedFromDOM = ()=>{
            const node = this.currentDragSourceNode;
            if (node == null || this.isNodeInDocument(node)) {
                return;
            }
            if (this.clearCurrentDragSourceNode() && this.monitor.isDragging()) {
                this.actions.endDrag();
            }
            this.cancelHover();
        };
        this.scheduleHover = (dragOverTargetIds)=>{
            if (this.hoverRafId === null && typeof requestAnimationFrame !== 'undefined') {
                this.hoverRafId = requestAnimationFrame(()=>{
                    if (this.monitor.isDragging()) {
                        this.actions.hover(dragOverTargetIds || [], {
                            clientOffset: this.lastClientOffset
                        });
                    }
                    this.hoverRafId = null;
                });
            }
        };
        this.cancelHover = ()=>{
            if (this.hoverRafId !== null && typeof cancelAnimationFrame !== 'undefined') {
                cancelAnimationFrame(this.hoverRafId);
                this.hoverRafId = null;
            }
        };
        this.handleTopDragStartCapture = ()=>{
            this.clearCurrentDragSourceNode();
            this.dragStartSourceIds = [];
        };
        this.handleTopDragStart = (e)=>{
            if (e.defaultPrevented) {
                return;
            }
            const { dragStartSourceIds  } = this;
            this.dragStartSourceIds = null;
            const clientOffset = getEventClientOffset(e);
            // Avoid crashing if we missed a drop event or our previous drag died
            if (this.monitor.isDragging()) {
                this.actions.endDrag();
                this.cancelHover();
            }
            // Don't publish the source just yet (see why below)
            this.actions.beginDrag(dragStartSourceIds || [], {
                publishSource: false,
                getSourceClientOffset: this.getSourceClientOffset,
                clientOffset
            });
            const { dataTransfer  } = e;
            const nativeType = matchNativeItemType(dataTransfer);
            if (this.monitor.isDragging()) {
                if (dataTransfer && typeof dataTransfer.setDragImage === 'function') {
                    // Use custom drag image if user specifies it.
                    // If child drag source refuses drag but parent agrees,
                    // use parent's node as drag image. Neither works in IE though.
                    const sourceId = this.monitor.getSourceId();
                    const sourceNode = this.sourceNodes.get(sourceId);
                    const dragPreview = this.sourcePreviewNodes.get(sourceId) || sourceNode;
                    if (dragPreview) {
                        const { anchorX , anchorY , offsetX , offsetY  } = this.getCurrentSourcePreviewNodeOptions();
                        const anchorPoint = {
                            anchorX,
                            anchorY
                        };
                        const offsetPoint = {
                            offsetX,
                            offsetY
                        };
                        const dragPreviewOffset = getDragPreviewOffset(sourceNode, dragPreview, clientOffset, anchorPoint, offsetPoint);
                        dataTransfer.setDragImage(dragPreview, dragPreviewOffset.x, dragPreviewOffset.y);
                    }
                }
                try {
                    // Firefox won't drag without setting data
                    dataTransfer === null || dataTransfer === void 0 ? void 0 : dataTransfer.setData('application/json', {});
                } catch (err) {
                // IE doesn't support MIME types in setData
                }
                // Store drag source node so we can check whether
                // it is removed from DOM and trigger endDrag manually.
                this.setCurrentDragSourceNode(e.target);
                // Now we are ready to publish the drag source.. or are we not?
                const { captureDraggingState  } = this.getCurrentSourcePreviewNodeOptions();
                if (!captureDraggingState) {
                    // Usually we want to publish it in the next tick so that browser
                    // is able to screenshot the current (not yet dragging) state.
                    //
                    // It also neatly avoids a situation where render() returns null
                    // in the same tick for the source element, and browser freaks out.
                    setTimeout(()=>this.actions.publishDragSource()
                    , 0);
                } else {
                    // In some cases the user may want to override this behavior, e.g.
                    // to work around IE not supporting custom drag previews.
                    //
                    // When using a custom drag layer, the only way to prevent
                    // the default drag preview from drawing in IE is to screenshot
                    // the dragging state in which the node itself has zero opacity
                    // and height. In this case, though, returning null from render()
                    // will abruptly end the dragging, which is not obvious.
                    //
                    // This is the reason such behavior is strictly opt-in.
                    this.actions.publishDragSource();
                }
            } else if (nativeType) {
                // A native item (such as URL) dragged from inside the document
                this.beginDragNativeItem(nativeType);
            } else if (dataTransfer && !dataTransfer.types && (e.target && !e.target.hasAttribute || !e.target.hasAttribute('draggable'))) {
                // Looks like a Safari bug: dataTransfer.types is null, but there was no draggable.
                // Just let it drag. It's a native type (URL or text) and will be picked up in
                // dragenter handler.
                return;
            } else {
                // If by this time no drag source reacted, tell browser not to drag.
                e.preventDefault();
            }
        };
        this.handleTopDragEndCapture = ()=>{
            if (this.clearCurrentDragSourceNode() && this.monitor.isDragging()) {
                // Firefox can dispatch this event in an infinite loop
                // if dragend handler does something like showing an alert.
                // Only proceed if we have not handled it already.
                this.actions.endDrag();
            }
            this.cancelHover();
        };
        this.handleTopDragEnterCapture = (e)=>{
            this.dragEnterTargetIds = [];
            if (this.isDraggingNativeItem()) {
                var ref;
                (ref = this.currentNativeSource) === null || ref === void 0 ? void 0 : ref.loadDataTransfer(e.dataTransfer);
            }
            const isFirstEnter = this.enterLeaveCounter.enter(e.target);
            if (!isFirstEnter || this.monitor.isDragging()) {
                return;
            }
            const { dataTransfer  } = e;
            const nativeType = matchNativeItemType(dataTransfer);
            if (nativeType) {
                // A native item (such as file or URL) dragged from outside the document
                this.beginDragNativeItem(nativeType, dataTransfer);
            }
        };
        this.handleTopDragEnter = (e)=>{
            const { dragEnterTargetIds  } = this;
            this.dragEnterTargetIds = [];
            if (!this.monitor.isDragging()) {
                // This is probably a native item type we don't understand.
                return;
            }
            this.altKeyPressed = e.altKey;
            // If the target changes position as the result of `dragenter`, `dragover` might still
            // get dispatched despite target being no longer there. The easy solution is to check
            // whether there actually is a target before firing `hover`.
            if (dragEnterTargetIds.length > 0) {
                this.actions.hover(dragEnterTargetIds, {
                    clientOffset: getEventClientOffset(e)
                });
            }
            const canDrop = dragEnterTargetIds.some((targetId)=>this.monitor.canDropOnTarget(targetId)
            );
            if (canDrop) {
                // IE requires this to fire dragover events
                e.preventDefault();
                if (e.dataTransfer) {
                    e.dataTransfer.dropEffect = this.getCurrentDropEffect();
                }
            }
        };
        this.handleTopDragOverCapture = (e)=>{
            this.dragOverTargetIds = [];
            if (this.isDraggingNativeItem()) {
                var ref;
                (ref = this.currentNativeSource) === null || ref === void 0 ? void 0 : ref.loadDataTransfer(e.dataTransfer);
            }
        };
        this.handleTopDragOver = (e)=>{
            const { dragOverTargetIds  } = this;
            this.dragOverTargetIds = [];
            if (!this.monitor.isDragging()) {
                // This is probably a native item type we don't understand.
                // Prevent default "drop and blow away the whole document" action.
                e.preventDefault();
                if (e.dataTransfer) {
                    e.dataTransfer.dropEffect = 'none';
                }
                return;
            }
            this.altKeyPressed = e.altKey;
            this.lastClientOffset = getEventClientOffset(e);
            this.scheduleHover(dragOverTargetIds);
            const canDrop = (dragOverTargetIds || []).some((targetId)=>this.monitor.canDropOnTarget(targetId)
            );
            if (canDrop) {
                // Show user-specified drop effect.
                e.preventDefault();
                if (e.dataTransfer) {
                    e.dataTransfer.dropEffect = this.getCurrentDropEffect();
                }
            } else if (this.isDraggingNativeItem()) {
                // Don't show a nice cursor but still prevent default
                // "drop and blow away the whole document" action.
                e.preventDefault();
            } else {
                e.preventDefault();
                if (e.dataTransfer) {
                    e.dataTransfer.dropEffect = 'none';
                }
            }
        };
        this.handleTopDragLeaveCapture = (e)=>{
            if (this.isDraggingNativeItem()) {
                e.preventDefault();
            }
            const isLastLeave = this.enterLeaveCounter.leave(e.target);
            if (!isLastLeave) {
                return;
            }
            if (this.isDraggingNativeItem()) {
                setTimeout(()=>this.endDragNativeItem()
                , 0);
            }
            this.cancelHover();
        };
        this.handleTopDropCapture = (e)=>{
            this.dropTargetIds = [];
            if (this.isDraggingNativeItem()) {
                var ref;
                e.preventDefault();
                (ref = this.currentNativeSource) === null || ref === void 0 ? void 0 : ref.loadDataTransfer(e.dataTransfer);
            } else if (matchNativeItemType(e.dataTransfer)) {
                // Dragging some elements, like <a> and <img> may still behave like a native drag event,
                // even if the current drag event matches a user-defined type.
                // Stop the default behavior when we're not expecting a native item to be dropped.
                e.preventDefault();
            }
            this.enterLeaveCounter.reset();
        };
        this.handleTopDrop = (e)=>{
            const { dropTargetIds  } = this;
            this.dropTargetIds = [];
            this.actions.hover(dropTargetIds, {
                clientOffset: getEventClientOffset(e)
            });
            this.actions.drop({
                dropEffect: this.getCurrentDropEffect()
            });
            if (this.isDraggingNativeItem()) {
                this.endDragNativeItem();
            } else if (this.monitor.isDragging()) {
                this.actions.endDrag();
            }
            this.cancelHover();
        };
        this.handleSelectStart = (e)=>{
            const target = e.target;
            // Only IE requires us to explicitly say
            // we want drag drop operation to start
            if (typeof target.dragDrop !== 'function') {
                return;
            }
            // Inputs and textareas should be selectable
            if (target.tagName === 'INPUT' || target.tagName === 'SELECT' || target.tagName === 'TEXTAREA' || target.isContentEditable) {
                return;
            }
            // For other targets, ask IE
            // to enable drag and drop
            e.preventDefault();
            target.dragDrop();
        };
        this.options = new OptionsReader(globalContext, options);
        this.actions = manager.getActions();
        this.monitor = manager.getMonitor();
        this.registry = manager.getRegistry();
        this.enterLeaveCounter = new EnterLeaveCounter(this.isNodeInDocument);
    }
}

const HTML5Backend = function createBackend(manager, context, options) {
    return new HTML5BackendImpl(manager, context, options);
};

class Content extends Component$1 {
  constructor(props) {
    super(props);
    this.state = {};
    this.onChange = this.onChange.bind(this);
    this.onClick = this.onClick.bind(this);
    this.onDialogAction = this.onDialogAction.bind(this);
  }
  onChange(value, field, block) {
    const {
      onChange
    } = this.props;
    const {
      checkForAction = true
    } = value || {};
    const allForms = this.getForms(this.props.pageData.items);
    onChange && onChange(value, field, Object.assign(Object.assign({}, block), {
      forms: allForms
    }));
    // console.log(checkForAction);
    // console.log(block);
    // console.log(field);
    if (checkForAction === true) {
      setTimeout(() => {
        this.checkForAction(value, Object.assign(Object.assign({}, block), {
          forms: allForms
        }), field);
      }, 100);
    }
  }
  onAction(value, action, block) {
    if (action.confirm) {
      this.setState({
        confirmAction: true,
        action: action,
        actionValue: value
      });
      return;
    }
    const {
      onAction
    } = this.props;
    const allForms = this.getForms(this.props.pageData.items);
    onAction && onAction(value, action, Object.assign(Object.assign({}, block), {
      forms: allForms
    }));
  }
  onDialogAction(dialgAction, action, block) {
    if (dialgAction.action === 'ok') {
      const {
        onAction
      } = this.props;
      const allForms = this.getForms(this.props.pageData.items);
      onAction && onAction(this.state.actionValue, action, Object.assign(Object.assign({}, block), {
        forms: allForms
      }));
      setTimeout(() => {
        this.setState({
          confirmAction: false,
          action: null,
          actionValue: null
        });
      }, 200);
      return;
    }
    this.setState({
      confirmAction: false,
      action: null,
      actionValue: null
    });
  }
  getForms(items = []) {
    let allForms = [];
    items.forEach(item => {
      let forms = [];
      if (item.items) {
        forms = this.getForms(item.items);
      }
      allForms = allForms.concat(forms);
      if (item.component === 'Form') {
        allForms.push(item);
      }
    });
    return allForms;
  }
  checkForAction(e, block, field) {
    const target = field || block;
    const actionType = target.actionType;
    if (actionType) {
      const allForms = this.getForms(this.props.pageData.items);
      this.onAction({}, Object.assign({}, target), Object.assign(Object.assign({}, block), {
        forms: allForms
      }));
    }
  }
  onClick(e, block, field) {
    this.checkForAction(e, block, field);
  }
  render() {
    const _a = this.props,
      {
        pageData = {},
        location,
        keyPressChange = true
      } = _a,
      rest = __rest(_a, ["pageData", "location", "keyPressChange"]);
    const {
      Dialog
    } = components$1.get();
    const {
      userData = {}
    } = rest;
    const {
      className = ''
    } = pageData;
    const compMap = components$1.get();
    return jsxs("div", {
      className: `sq-content-page__body ${className}`,
      children: [pageData.items && pageData.items.map((block, idx) => {
        let validator;
        let isValid = true;
        if (block.match) {
          validator = new Validator$1(Object.assign({}, block.match));
          validator.setValues(userData);
          isValid = validator.validateAll();
        }
        const {
            component
          } = block;
          __rest(block, ["component"]);
        const Comp = compMap[component];
        block = object$1.processBlock(block, {
          userData
        });
        if (!Comp) {
          console.warn(`${component} component not found`);
        }
        return Comp && isValid ? jsx(ErrorBoundry$1, {
          children: jsx(Comp, Object.assign({}, rest, {
            value: userData[block.name]
          }, block, {
            errors: userData[block.name + '_errors'],
            onClick: (e, field) => {
              this.onClick(e, block, field);
            },
            onChange: (value, field) => {
              this.onChange(value, field, block);
            },
            onAction: (value, action, overrideBlock) => {
              this.onAction(value, action, overrideBlock || block);
            },
            onFieldKeyPress: (value, field, data) => {
              keyPressChange && this.onChange({
                value: Object.assign(Object.assign({}, data), {
                  [field.name]: value.value
                })
              }, field, block);
            }
          }))
        }, idx) : undefined;
      }), this.state.action && this.state.action.confirm && jsx(Dialog, {
        title: this.state.action.confirm.title,
        content: this.state.action.confirm.content,
        classes: {
          body: 'sq-dialog__content-body--confirm'
        },
        closeButton: false,
        open: this.state.confirmAction,
        onAction: (data, dialgAction) => this.onDialogAction(dialgAction, this.state.action),
        actions: this.state.action.confirm.actions || [{
          buttonText: 'Yes',
          action: 'ok'
        }, {
          buttonText: 'Cancel',
          variant: 'outlined',
          action: 'cancel'
        }]
      })]
    });
  }
}
Content.propTypes = {
  commonStore: PropTypes.object,
  contentStore: PropTypes.object
};

const Default = ({
  children,
  pageData
}) => {
  return jsx("div", {
    className: `sq-content-page ${pageData.wrapperClassName || ''}`,
    children: children
  });
};

const _window = window;
containers$1.set({
  Default
});
class DynamicContent extends Component$1 {
  constructor(props) {
    super(props);
    this.state = {
      activeTheme: 'main',
      pageData: {},
      page: {
        pageYOffset: _window.pageYOffset,
        innerHeight: _window.innerHeight
      }
    };
    this.onChange = this.onChange.bind(this);
    this.onAction = this.onAction.bind(this);
    this.onRefresh = this.onRefresh.bind(this);
    this.onAfterRedirect = this.onAfterRedirect.bind(this);
  }
  componentDidMount() {
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        location = {},
        url: overrideUrl,
        pageConfig
      } = this.props;
      this.setState({
        url: overrideUrl || location.pathname
      }, () => {
        this.fetchPage(true, pageConfig);
      });
      events.subscribe('refreshPage', this.onRefresh);
      events.subscribe('afterRedirect', this.onAfterRedirect);
      events.subscribeOnce('dynamicContent.onAction', this.onAction);
    });
  }
  componentWillUnmount() {
    return __awaiter$1(this, void 0, void 0, function* () {
      events.unsubscribe('afterRedirect', this.onAfterRedirect);
      events.unsubscribe('refreshPage', this.onRefresh);
      events.unsubscribe('dynamicContent.onAction', this.onAction);
    });
  }
  onAfterRedirect() {
    this.props.raiseAction(updateUserData({}));
  }
  onRefresh() {
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        refresh
      } = this.props.pageData || {};
      let isValid = true;
      if (refresh) {
        if (refresh.match) {
          const validations = new Validator$1(Object.assign({}, refresh.match));
          validations.setValues(Object.assign({}, this.getUpdatedUserData()));
          isValid = validations.validateAll();
        }
      }
      if (isValid) {
        this.refreshPage(refresh === null || refresh === void 0 ? void 0 : refresh.forceLoad);
      }
    });
  }
  processHook(hook, pageData) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const arr = [];
      if (hook) {
        if (Array.isArray(hook)) {
          hook.forEach(item => {
            arr.push(this.postApi(item, pageData));
          });
        } else {
          arr.push(this.postApi(hook, pageData));
        }
      }
      return Promise.all(arr);
    });
  }
  postApi(data, pageData) {
    return this.props.contentActions.postApi(data, pageData || this.state.pageData);
  }
  refreshPage(forceLoad) {
    return __awaiter$1(this, void 0, void 0, function* () {
      if (forceLoad) {
        this.fetchPage();
      } else {
        this.fetchPage(undefined, this.state.pageData);
      }
    });
  }
  fetchPage(firstTime, dataForPage) {
    var _a, _b, _c;
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        onAnalytics,
        transitionSpeed = 300,
        initialData,
        mergePageData = {}
      } = this.props;
      if (initialData) {
        yield this.props.contentActions.updateUserData(initialData);
      }
      if (!firstTime) {
        this.setState({
          isOut: true
        });
        yield timer.delay(transitionSpeed);
      }
      const interval = window.setTimeout(() => {
        this.setState({
          isLoading: true
        });
      }, 300);
      let resp;
      let pageResponse;
      if (!dataForPage) {
        resp = yield this.props.contentActions.fetchContentPage(this.state.url).unwrap();
        pageResponse = resp.data;
        if (pageResponse.pageData) {
          pageResponse.pageData = Object.assign(Object.assign({}, pageResponse.pageData), mergePageData);
        }
      } else {
        pageResponse = dataForPage;
      }
      if (pageResponse.pageData.updatePageTitle) {
        window.document.title = pageResponse.pageData.title;
      }
      if (pageResponse.pageData.headScript) {
        browser.scriptManager.insertDynamicScript(pageResponse.pageData.headScript);
      }
      if (pageResponse.pageData.reset) {
        yield this.props.contentActions.resetUserData(pageResponse.pageData.reset);
      }
      if (initialData) {
        yield this.props.contentActions.updateUserData(initialData);
      }
      yield this.props.contentActions.updateMetaData({
        url: this.state.url,
        data: {
          metaData: pageResponse.metaData,
          siteMap: pageResponse.siteMap
        }
      });
      yield this.props.contentActions.updateUserData((_a = pageResponse.metaData) === null || _a === void 0 ? void 0 : _a.userData);
      yield this.props.contentActions.mergeUserData(pageResponse.pageData.merge);
      yield this.processHook((_b = pageResponse.pageData.hook) === null || _b === void 0 ? void 0 : _b.load, pageResponse);
      yield this.props.contentActions.mergeUserData(pageResponse.pageData.init);
      yield this.props.contentActions.mergeUserData(pageResponse.pageData.merge);
      window.clearTimeout(interval);
      const {
        analytics = {}
      } = pageResponse.pageData;
      this.setState({
        isLoading: false,
        pageData: pageResponse
      });
      onAnalytics && onAnalytics(Object.assign({
        type: 'pageview',
        page_title: pageResponse.pageData.title
      }, analytics.load));
      this.setState({
        isOut: false,
        isIn: true
      });
      yield timer.delay(transitionSpeed);
      this.setState({
        isOut: false,
        isIn: false
      });
      yield this.processHook((_c = pageResponse.pageData.hook) === null || _c === void 0 ? void 0 : _c.afterLoad, pageResponse);
    });
  }
  componentDidUpdate() {
    var _a, _b, _c, _d, _e;
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        location = {},
        url: overrideUrl,
        onThemeChange
      } = this.props;
      const activeTheme = this.state.activeTheme;
      const pageDataTheme = ((_b = (_a = this.state.pageData) === null || _a === void 0 ? void 0 : _a.pageData) === null || _b === void 0 ? void 0 : _b.theme) || ((_e = (_d = (_c = this.state.pageData) === null || _c === void 0 ? void 0 : _c.siteMap) === null || _d === void 0 ? void 0 : _d.siteMap) === null || _e === void 0 ? void 0 : _e.theme);
      if (activeTheme && pageDataTheme && activeTheme !== pageDataTheme) {
        onThemeChange && onThemeChange(pageDataTheme);
      }
      if (this.state.url != (overrideUrl || location.pathname)) {
        this.setState({
          url: overrideUrl || location.pathname
        }, () => {
          this.fetchPage();
        });
      }
    });
  }
  getUpdatedUserData() {
    const {
      userData
    } = this.props.store.content;
    return Object.assign(Object.assign({}, userData), {
      query: Object.assign({}, query$1.get())
    });
  }
  getValidationResult(field, block, value) {
    var _a;
    if (field === null || field === void 0 ? void 0 : field.validators) {
      const allValidators = {};
      (_a = block.fields) === null || _a === void 0 ? void 0 : _a.forEach(field => {
        if (field.validators) {
          allValidators[field.name] = {
            validators: field.validators,
            impactOn: field.impactOn
          };
        }
      });
      const validations = new Validator$1(Object.assign(Object.assign({}, allValidators), {
        [field.name]: {
          validators: field.validators,
          impactOn: field.impactOn
        }
      }));
      validations.setValues(Object.assign(Object.assign({}, this.getUpdatedUserData()), value));
      validations.validate(field.name);
      return validations;
    }
  }
  onChange(value, field, block) {
    var _a, _b, _c;
    return __awaiter$1(this, void 0, void 0, function* () {
      if ((_a = field || block) === null || _a === void 0 ? void 0 : _a.beforeChangeAction) {
        yield this.onAction({}, field.beforeChangeAction, block);
      }
      const {
        onContentChange,
        contentParams
      } = this.props;
      let obj = {};
      block = (field === null || field === void 0 ? void 0 : field.block) ? field.block : block;
      if (block && block.name) {
        obj[block.name] = value.value;
      } else {
        obj = value.value;
      }
      let nestErrors;
      let currentError;
      let currentValue;
      let currentForm;
      let lastError;
      let existingErrors = this.props.store.content.userData[block.name + '_errors'] || {};
      if (((_b = value.formTrace) === null || _b === void 0 ? void 0 : _b.length) > 0) {
        currentValue = value.value;
        if ((_c = value.orignField) === null || _c === void 0 ? void 0 : _c.validators) {
          nestErrors = {};
          currentError = nestErrors;
          value.formTrace.forEach(tForm => {
            currentValue = currentValue[tForm.name];
            existingErrors = existingErrors[tForm.name] || {};
            lastError = currentError;
            currentError[tForm.name] = {
              errors: Object.assign({}, existingErrors.errors)
            };
            currentError = currentError[tForm.name].errors;
            currentForm = tForm;
          });
          const _validations = this.getValidationResult(value.orignField, currentForm, currentValue);
          if (_validations) {
            lastError[currentForm.name].errors = Object.assign(Object.assign({}, lastError[currentForm.name].errors), _validations.errors);
          }
        }
      }
      const validations = this.getValidationResult(field, block, value.value);
      if (validations || nestErrors) {
        this.props.contentActions.updateUserData({
          [block.name + '_errors']: Object.assign(Object.assign(Object.assign({}, this.props.store.content.userData[block.name + '_errors']), validations === null || validations === void 0 ? void 0 : validations.errors), nestErrors)
        });
      }
      this.props.contentActions.updateUserData(obj);
      this.props.contentActions.mergeUserData(this.state.pageData.pageData.merge);
      window.setTimeout(() => {
        var _a;
        onContentChange && onContentChange(processParams(this.props.store.content.userData, contentParams));
        if ((_a = field || block) === null || _a === void 0 ? void 0 : _a.changeAction) {
          this.onAction({
            value: currentValue
          }, field.changeAction, block);
        }
      }, 300);
    });
  }
  hasMatchingGroup(form, group) {
    if (!group) {
      return true;
    }
    if (typeof form.group === 'string') {
      return form.group === group;
    } else if (Array.isArray(form.group)) {
      return form.group.indexOf(group) > -1;
    }
    return false;
  }
  validateForms(forms = [], group) {
    this.startValidating = true;
    let isValid = true;
    forms.forEach(form => {
      if (this.hasMatchingGroup(form, group) && !this.validateForm(form, isValid)) {
        isValid = false;
      }
    });
    this.startValidating = false;
    return isValid;
  }
  getValidateFormFull(block, value = {}, prevValid) {
    var _a;
    let fullValid = prevValid !== undefined ? prevValid : true;
    const allErros = {};
    const validators = {};
    if (block.match) {
      let blockMatch = true;
      const valid = new Validator$1(block.match);
      valid.setValues(Object.assign(Object.assign({}, this.getUpdatedUserData()), value[block.name]));
      blockMatch = valid.validateAll();
      if (!blockMatch && !block.forceValidate) {
        return {
          isValid: true,
          errors: {}
        };
      }
    }
    (_a = block.fields) === null || _a === void 0 ? void 0 : _a.forEach(item => {
      let hasMatch = true;
      if (item.match) {
        const valid = new Validator$1(item.match);
        valid.setValues(Object.assign(Object.assign({}, this.getUpdatedUserData()), value[block.name]));
        hasMatch = valid.validateAll();
      }
      if (hasMatch && item.validators) {
        validators[item.name] = {
          validators: item.validators
        };
      }
      if (item.cmpType === 'Form') {
        const {
          isValid: inernvalValid,
          errors: newErrors
        } = this.getValidateFormFull(item, value[item.name], prevValid);
        if (!inernvalValid) {
          fullValid = inernvalValid;
          allErros[item.name] = {
            errors: newErrors
          };
        }
      }
    });
    const onFirstFail = (failed, name) => {
      events.emit('onFirstFieldFailed', failed, name, block);
    };
    const evets = {};
    if (fullValid) {
      evets.onFirstFail = onFirstFail;
    }
    const validObj = new Validator$1(validators, evets);
    validObj.setValues(Object.assign(Object.assign({}, this.getUpdatedUserData()), value[block.name]));
    const isValid = validObj.validateAll() && fullValid;
    return {
      isValid,
      errors: Object.assign(Object.assign({}, validObj.errors), allErros)
    };
  }
  validateForm(block, prevValid) {
    const {
      isValid,
      errors
    } = this.getValidateFormFull(block, this.getUpdatedUserData(), prevValid);
    this.props.contentActions.updateUserData({
      [block.name + '_errors']: Object.assign(Object.assign({}, this.props.store.content.userData[block.name + '_errors']), errors)
    });
    return isValid;
  }
  onAction(value, action, block) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        onSubmit,
        onCancel
      } = this.props;
      let result;
      let isValid = true;
      const newUserData = Object.assign(Object.assign({}, this.props.store.content.userData), action.currentData || {});
      // action.nextAction = (action) => this.onAction(value, action, block);
      switch (action.actionType) {
        case 'file-upload':
          yield this.props.contentActions.updateUserData({
            isUploadingFile: true
          });
          result = yield this.props.contentActions.uploadApi(action, this.state.pageData);
          yield this.props.contentActions.mergeUserData(this.state.pageData.pageData.merge);
          yield this.props.contentActions.updateUserData({
            isUploadingFile: false
          });
          if (result.status === 'success') {
            const data = action.dataKey ? {
              [action.dataKey]: result.data
            } : result.data;
            yield this.props.contentActions.updateUserData(Object.assign(Object.assign({}, data), {
              lastError: {}
            }));
          }
          this.checkForInlineErrors(result, action);
          this.validateResults(result);
          break;
        case 'reset':
          yield this.props.contentActions.resetUserData(action);
          break;
        case 'download-doc':
          yield this.props.contentActions.updateUserData({
            isSubmitting: true,
            isDownloadingFile: true
          });
          result = yield this.props.contentActions.downloadApi(action, this.state.pageData);
          yield this.props.contentActions.mergeUserData(this.state.pageData.pageData.merge);
          yield this.props.contentActions.updateUserData({
            isSubmitting: false,
            isDownloadingFile: false
          });
          if (result.status === 'success') {
            const data = action.dataKey ? {
              [action.dataKey]: result.data
            } : result.data;
            yield this.props.contentActions.updateUserData(Object.assign(Object.assign({}, data), {
              lastError: {}
            }));
          }
          this.checkForInlineErrors(result, action);
          this.validateResults(result);
          break;
        case 'api':
          yield this.props.contentActions.updateUserData({
            isSubmitting: true
          });
          result = yield this.props.contentActions.postApi(action, this.state.pageData);
          yield this.props.contentActions.mergeUserData(this.state.pageData.pageData.merge);
          yield this.props.contentActions.updateUserData({
            isSubmitting: false
          });
          if (result.status === 'success') {
            const data = action.dataKey ? {
              [action.dataKey]: result.data
            } : result.data;
            yield this.props.contentActions.updateUserData(Object.assign(Object.assign({}, data), {
              lastError: {}
            }));
          }
          this.checkForInlineErrors(result, action);
          this.validateResults(result);
          break;
        case 'module':
          if (action.validate) {
            isValid = this.validateForms(block.forms, action.validateGroup);
          }
          if (isValid) {
            yield this.props.contentActions.updateUserData({
              isSubmitting: true
            });
            result = yield this.props.contentActions.executeHook(action, this.state.pageData);
            yield this.props.contentActions.mergeUserData(this.state.pageData.pageData.merge);
            yield this.props.contentActions.updateUserData({
              isSubmitting: false
            });
            if (result.status === 'success') {
              const data = action.dataKey ? {
                [action.dataKey]: result.data
              } : result.data;
              yield this.props.contentActions.updateUserData(Object.assign(Object.assign({}, data), {
                lastError: {}
              }));
            }
            this.checkForInlineErrors(result, action);
            this.validateResults(result);
          }
          break;
        case 'submit-form':
          isValid = this.validateForm(block);
          if (isValid) {
            yield this.props.contentActions.updateUserData({
              isSubmitting: true
            });
            result = yield this.props.contentActions.postApi(action, this.state.pageData);
            yield this.props.contentActions.mergeUserData(this.state.pageData.pageData.merge);
            yield this.props.contentActions.updateUserData({
              isSubmitting: false
            });
            this.checkForInlineErrors(result, action);
            this.validateResults(result);
          }
          break;
        case 'submit':
          isValid = this.validateForms(block.forms, action.validateGroup);
          if (isValid) {
            yield this.props.contentActions.updateUserData({
              isSubmitting: true
            });
            result = yield this.props.contentActions.postApi(action, this.state.pageData);
            yield this.props.contentActions.mergeUserData(this.state.pageData.pageData.merge);
            yield this.props.contentActions.updateUserData({
              isSubmitting: false
            });
            this.checkForInlineErrors(result, action);
            this.validateResults(result);
          }
          break;
        case 'submit-event':
          isValid = this.validateForms(block.forms, action.validateGroup);
          if (isValid) {
            onSubmit && onSubmit(processParams(newUserData, action.params));
          }
          break;
        case 'cancel-event':
          onCancel && onCancel(processParams(newUserData, action.params));
          break;
        case 'user-store':
          if (Array.isArray(action.bulk)) {
            const actions = [];
            action.bulk.forEach(actionItem => {
              let validated = true;
              if (actionItem.executeIf) {
                const storeItemValidator = new Validator$1(actionItem.executeIf);
                storeItemValidator.setValues(newUserData);
                validated = storeItemValidator.validateAll();
              }
              if (validated) {
                actions.push(this.props.contentActions.mergeUserData(Object.assign({}, processParams(newUserData, actionItem.params))));
              }
            });
            yield Promise.all(actions);
          } else {
            let validated = true;
            if (action.executeIf) {
              const storeItemValidator = new Validator$1(action.executeIf);
              storeItemValidator.setValues(newUserData);
              validated = storeItemValidator.validateAll();
            }
            if (validated) {
              yield this.props.contentActions.mergeUserData(Object.assign({}, processParams(newUserData, action.params)));
            }
          }
          break;
        case 'notify-message':
          yield this.props.commonActions.showNotificationMessage(Object.assign({}, processParams(newUserData, action.params)));
          break;
        case 'popup':
          yield this.props.commonActions.showPopup(Object.assign({}, processParams(newUserData, action.params)));
          break;
        case 'close-popup':
          yield this.props.commonActions.closePopup();
          action.eventEmit !== false && events.emit('onPopupCloseAction', action);
          break;
        case 'popup-screen':
          yield this.props.commonActions.showPopupScreen(Object.assign({}, processParams(newUserData, action.params)));
          break;
        case 'close-popup-screen':
          yield this.props.commonActions.closePopupScreen();
          action.eventEmit !== false && events.emit('onPopupScreenCloseAction', action);
          break;
        case 'redirect':
          redirectTo(action.to, processParams(newUserData, action.urlParams || action.params), action.options);
          break;
        case 'self-redirect':
          redirectTo(window.location.pathname, processParams(newUserData, action.urlParams || action.params), action.options);
          break;
      }
    });
  }
  checkForInlineErrors(result, action) {
    var _a, _b, _c;
    if (action.rootErrorKey && ((_a = result.error) === null || _a === void 0 ? void 0 : _a.errors)) {
      result.error.errors = {
        [action.rootErrorKey]: {
          errors: (_b = result.error) === null || _b === void 0 ? void 0 : _b.errors
        }
      };
    }
    if ((_c = result.error) === null || _c === void 0 ? void 0 : _c.errors) {
      Object.keys(result.error.errors).forEach(errorKey => {
        if (result.error.errors[errorKey].errors) {
          this.props.contentActions.updateUserData({
            [`${errorKey}_errors`]: result.error.errors[errorKey].errors
          });
        } else if (result.error.errors[errorKey].error) {
          this.props.contentActions.updateUserData({
            [`${errorKey}_errors`]: result.error.errors[errorKey]
          });
        }
      });
    }
  }
  validateResults(result) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r;
    if (result && typeof ((_a = result.data) === null || _a === void 0 ? void 0 : _a.redirect) === 'string') {
      redirectTo((_b = result.data) === null || _b === void 0 ? void 0 : _b.redirect);
    } else if (result && ((_d = (_c = result.data) === null || _c === void 0 ? void 0 : _c.redirect) === null || _d === void 0 ? void 0 : _d.to)) {
      redirectTo((_e = result.data) === null || _e === void 0 ? void 0 : _e.redirect.to, processParams(this.props.store.content.userData, (_f = result.data) === null || _f === void 0 ? void 0 : _f.redirect.urlParams), (_h = (_g = result.data) === null || _g === void 0 ? void 0 : _g.redirect) === null || _h === void 0 ? void 0 : _h.options);
    }
    if (result && typeof ((_j = result.error) === null || _j === void 0 ? void 0 : _j.redirect) === 'string') {
      redirectTo((_k = result.error) === null || _k === void 0 ? void 0 : _k.redirect);
    } else if (result && ((_m = (_l = result.error) === null || _l === void 0 ? void 0 : _l.redirect) === null || _m === void 0 ? void 0 : _m.to)) {
      redirectTo((_o = result.error) === null || _o === void 0 ? void 0 : _o.redirect.to, processParams(this.props.store.content.userData, (_p = result.error) === null || _p === void 0 ? void 0 : _p.redirect.urlParams), (_r = (_q = result.error) === null || _q === void 0 ? void 0 : _q.redirect) === null || _r === void 0 ? void 0 : _r.options);
    }
  }
  render() {
    var _a, _b;
    const {
      Progress,
      Snackbar
    } = components$1.get();
    const _c = this.props,
      {
        containerTemplate: overrideContainerTemplate,
        mode = 'actual',
        rootClass = 'row',
        transition: rootTransition
      } = _c,
      allProps = __rest(_c, ["containerTemplate", "mode", "rootClass", "transition"]);
    const {
      dataPacket,
      store
    } = allProps;
    const userData = Object.assign(Object.assign(Object.assign(Object.assign({}, this.props.store.content.userData), {
      query: Object.assign({}, query$1.get())
    }), dataPacket), {
      contentPage: true
    });
    const {
      root = {}
    } = userData;
    const {
      pageData = {},
      metaData
    } = this.state.pageData;
    const dynamicParams = processParams(userData, Object.assign(Object.assign({}, root.merge), pageData.inject) || {});
    const {
        classes = {}
      } = dynamicParams,
      restDynamic = __rest(dynamicParams, ["classes"]);
    const updatedPageData = Object.assign(Object.assign({}, pageData), restDynamic);
    const {
      container,
      containerTemplate,
      contentBodyClass = '',
      rootClassName = '',
      transition = {}
    } = updatedPageData;
    const ContentTemplateContainer = mode === 'actual' ? containers$1.get()[overrideContainerTemplate || containerTemplate] || containers$1.get().Default : containers$1.get().Default;
    const ContentContainer = containers$1.get()[container] || Content;
    const {
      out: tranOut = 'out-up',
      in: tranIn = 'out-in',
      loading = 'loading',
      loadingColor = 'primary'
    } = rootTransition || transition;
    const classState = this.state.isOut ? `transition transition-page--${tranOut}` : this.state.isIn ? `transition transition-page--${tranIn}` : '';
    const loadingState = this.state.isLoading ? `transition transition-page--${loading}` : '';
    return jsxs("div", {
      className: `dynamic-content ${rootClass} ${rootClassName} ${loadingState} ${classes.root || ''}`,
      children: [jsx(Snackbar, {
        open: store.common.notification.show,
        message: store.common.notification.message,
        autoHideDuration: null,
        onClose: (evt, reason) => {
          if (reason !== 'clickaway') {
            this.props.commonActions.closeNotification();
            return;
          }
        },
        severity: store.common.notification.type
      }), jsxs(DndProvider, {
        backend: HTML5Backend,
        children: [jsx(ContentTemplateContainer, Object.assign({}, allProps, {
          pageData: updatedPageData,
          metaData: metaData,
          data: this.state.pageData,
          pageState: this.state.page,
          userData: userData,
          children: jsx("div", {
            className: `${contentBodyClass} dynamic-content__body ${classState} ${classes.body || ''}`,
            children: jsx(ContentContainer, Object.assign({}, allProps, {
              pageData: updatedPageData,
              metaData: metaData,
              data: this.state.pageData,
              userData: userData,
              pageState: this.state.page,
              onChange: this.onChange,
              onAction: this.onAction
            }))
          })
        })), !((_a = store.common.popupScreen) === null || _a === void 0 ? void 0 : _a.show) && this.state.isLoading && jsx(Progress, {
          color: loadingColor,
          className: `${classes.progress || ''}`,
          style: 'fixed',
          overlayStyle: "full"
        }), ((_b = store.common.popupScreen) === null || _b === void 0 ? void 0 : _b.show) && this.state.isLoading && jsx(Progress, {
          color: loadingColor,
          className: `${classes.progress || ''}`,
          style: 'screen'
        })]
      })]
    });
  }
}
DynamicContent.propTypes = {
  contentActions: PropTypes.object,
  className: PropTypes.string,
  commonActions: PropTypes.object,
  raiseAction: PropTypes.func
};
const mapStateToProps$6 = state => {
  return {
    store: Object.assign({}, state)
  };
};
const mapDispatchToProps$6 = dispatch => {
  return {
    contentActions: {
      postApi: (data, pageData) => dispatch(postApi(data, pageData)),
      checkAndPostApi: (data, pageData) => dispatch(checkAndPostApi(data, pageData)),
      downloadApi: (data, pageData) => dispatch(downloadApi(data, pageData)),
      uploadApi: (data, pageData) => dispatch(uploadApi(data, pageData)),
      updateMetaData: data => dispatch(updateMetaData(data)),
      executeHook: (data, pageData) => dispatch(executeHook(data, pageData)),
      fetchContentPage: data => dispatch(fetchContentPage(data)),
      resetUserData: data => dispatch(resetUserData(data)),
      updateUserData: data => dispatch(updateUserData(data)),
      mergeUserData: data => dispatch(mergeUserData(data)),
      sendContact: data => dispatch(sendContact(data)),
      updateErrorData: data => dispatch(updateErrorData(data))
    },
    commonActions: {
      showNotificationMessage: data => dispatch(showNotificationMessage(data)),
      closeNotification: () => dispatch(closeNotification()),
      startLoading: data => dispatch(startLoading(data)),
      showPopup: data => dispatch(showPopup(data)),
      stopLoading: data => dispatch(stopLoading(data)),
      setError: data => dispatch(setError(data)),
      clearError: data => dispatch(clearError(data)),
      showPopupScreen: data => dispatch(showPopupScreen(data)),
      closePopup: () => dispatch(closePopup()),
      closePopupScreen: () => dispatch(closePopupScreen())
    },
    raiseAction: dispatch
  };
};
var DynamicContentImpl = connect(mapStateToProps$6, mapDispatchToProps$6)(DynamicContent);

const DynamicContentRoot = props => {
  const params = useParams();
  const location = useLocation();
  return jsx(DynamicContentImpl, Object.assign({}, props, {
    location: location,
    params: params
  }));
};

class ErrorBoundry extends React$2.Component {
  constructor(props) {
    super(props);
    this.state = {
      hasError: false
    };
  }
  componentDidCatch(error) {
    console.log('@@root:error', error);
    // You can also log the error to an error reporting service
    this.setState({
      hasError: true
    });
  }
  render() {
    const {
      LinkButton
    } = components$1.get();
    const {
      header = 'Unexpected error occured',
      body = 'There is some error on this page please contact administrator.',
      redirectUrl = '/',
      buttonText = 'Refresh',
      buttonProps = {}
    } = this.props;
    if (this.state.hasError) {
      // You can render any custom fallback UI
      return jsx("div", {
        className: "error-boundry container",
        children: jsxs("div", {
          className: "container-form",
          children: [jsx("h1", {
            children: header
          }), jsx("p", {
            children: body
          }), jsx(LinkButton, Object.assign({
            buttonText: buttonText,
            to: redirectUrl,
            type: "Button",
            variant: "outlined"
          }, buttonProps))]
        })
      });
    }
    return this.props.children;
  }
}
ErrorBoundry.propTypes = {
  children: PropTypes.any,
  header: PropTypes.string,
  body: PropTypes.string,
  redirectUrl: PropTypes.string,
  buttonProps: PropTypes.object,
  buttonText: PropTypes.string
};

const Application = _a => {
  var _b;
  var {
      children,
      progressProps = {}
    } = _a,
    rest = __rest(_a, ["children", "progressProps"]);
  const {
    store,
    userData,
    raiseAction
  } = rest;
  const containers = containers$1.get();
  const {
    Dialog,
    Progress
  } = components$1.get();
  const {
    popupScreen
  } = store.common;
  const RouterComponent = containers[popupScreen.name] ? containers[popupScreen.name] : ((_b = popupScreen === null || popupScreen === void 0 ? void 0 : popupScreen.props) === null || _b === void 0 ? void 0 : _b.url) ? DynamicContentRoot : undefined;
  const Routerprops = __rest(popupScreen.props || {}, []);
  const handleClosePopup = () => {
    raiseAction(closePopup());
  };
  const handleClosePopupScreen = () => {
    raiseAction(closePopupScreen());
  };
  return jsxs(Fragment, {
    children: [!RouterComponent && store.common.isLoading && jsx(Progress, Object.assign({
      style: "fixed",
      color: "secondary"
    }, progressProps)), jsx(Dialog, {
      open: store.common.popup.show,
      content: store.common.popup.message,
      title: store.common.popup.title,
      actions: store.common.popup.actions || [{
        buttonText: 'Ok',
        actionType: 'close'
      }],
      onClose: handleClosePopup,
      onAction: handleClosePopup,
      severity: store.common.popup.type
    }), RouterComponent && store.common.popupScreen.show && jsx(Dialog, Object.assign({
      open: store.common.popupScreen.show,
      isLoading: store.common.isLoading,
      fullScreen: browser.breakpoints.down('sm'),
      content: jsxs(ErrorBoundry, {
        children: [store.common.isLoading && jsx(Progress, Object.assign({
          color: "secondary"
        }, progressProps, {
          style: "screen"
        })), jsx(RouterComponent, Object.assign({}, rest, {
          dataPacket: Object.assign(Object.assign({}, userData), {
            hasDialog: true
          })
        }, Routerprops, store.common.popupScreen.props))]
      })
    }, store.common.popupScreen.dialogProps, {
      title: store.common.popupScreen.title,
      onClose: () => {
        handleClosePopupScreen();
        apiBridge$1.events.emit('onPopupScreenClose');
      },
      onAction: () => {
        handleClosePopupScreen();
        apiBridge$1.events.emit('onPopupScreenAction');
      }
    })), jsx("div", {
      className: "sq-app__main",
      children: children
    })]
  });
};
// export default Application;
const mapStateToProps$5 = state => {
  return {
    store: Object.assign({}, state),
    userData: state.content.userData
  };
};
const mapDispatchToProps$5 = dispatch => {
  return {
    commonActions: {
      closePopup: () => dispatch(closePopup()),
      closePopupScreen: () => dispatch(closePopupScreen())
    },
    raiseAction: dispatch
  };
};
var index$5 = connect(mapStateToProps$5, mapDispatchToProps$5)(Application);

var lib$6 = {};

var lib$5 = {};

(function (exports) {
Object.defineProperty(exports, "__esModule", { value: true });
exports.Doctype = exports.CDATA = exports.Tag = exports.Style = exports.Script = exports.Comment = exports.Directive = exports.Text = exports.Root = exports.isTag = exports.ElementType = void 0;
/** Types of elements found in htmlparser2's DOM */
var ElementType;
(function (ElementType) {
    /** Type for the root element of a document */
    ElementType["Root"] = "root";
    /** Type for Text */
    ElementType["Text"] = "text";
    /** Type for <? ... ?> */
    ElementType["Directive"] = "directive";
    /** Type for <!-- ... --> */
    ElementType["Comment"] = "comment";
    /** Type for <script> tags */
    ElementType["Script"] = "script";
    /** Type for <style> tags */
    ElementType["Style"] = "style";
    /** Type for Any tag */
    ElementType["Tag"] = "tag";
    /** Type for <![CDATA[ ... ]]> */
    ElementType["CDATA"] = "cdata";
    /** Type for <!doctype ...> */
    ElementType["Doctype"] = "doctype";
})(ElementType = exports.ElementType || (exports.ElementType = {}));
/**
 * Tests whether an element is a tag or not.
 *
 * @param elem Element to test
 */
function isTag(elem) {
    return (elem.type === ElementType.Tag ||
        elem.type === ElementType.Script ||
        elem.type === ElementType.Style);
}
exports.isTag = isTag;
// Exports for backwards compatibility
/** Type for the root element of a document */
exports.Root = ElementType.Root;
/** Type for Text */
exports.Text = ElementType.Text;
/** Type for <? ... ?> */
exports.Directive = ElementType.Directive;
/** Type for <!-- ... --> */
exports.Comment = ElementType.Comment;
/** Type for <script> tags */
exports.Script = ElementType.Script;
/** Type for <style> tags */
exports.Style = ElementType.Style;
/** Type for Any tag */
exports.Tag = ElementType.Tag;
/** Type for <![CDATA[ ... ]]> */
exports.CDATA = ElementType.CDATA;
/** Type for <!doctype ...> */
exports.Doctype = ElementType.Doctype;
}(lib$5));

var node = {};

var __extends = (commonjsGlobal && commonjsGlobal.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign$1 = (commonjsGlobal && commonjsGlobal.__assign) || function () {
    __assign$1 = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign$1.apply(this, arguments);
};
Object.defineProperty(node, "__esModule", { value: true });
node.cloneNode = node.hasChildren = node.isDocument = node.isDirective = node.isComment = node.isText = node.isCDATA = node.isTag = node.Element = node.Document = node.CDATA = node.NodeWithChildren = node.ProcessingInstruction = node.Comment = node.Text = node.DataNode = node.Node = void 0;
var domelementtype_1$1 = lib$5;
/**
 * This object will be used as the prototype for Nodes when creating a
 * DOM-Level-1-compliant structure.
 */
var Node = /** @class */ (function () {
    function Node() {
        /** Parent of the node */
        this.parent = null;
        /** Previous sibling */
        this.prev = null;
        /** Next sibling */
        this.next = null;
        /** The start index of the node. Requires `withStartIndices` on the handler to be `true. */
        this.startIndex = null;
        /** The end index of the node. Requires `withEndIndices` on the handler to be `true. */
        this.endIndex = null;
    }
    Object.defineProperty(Node.prototype, "parentNode", {
        // Read-write aliases for properties
        /**
         * Same as {@link parent}.
         * [DOM spec](https://dom.spec.whatwg.org)-compatible alias.
         */
        get: function () {
            return this.parent;
        },
        set: function (parent) {
            this.parent = parent;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Node.prototype, "previousSibling", {
        /**
         * Same as {@link prev}.
         * [DOM spec](https://dom.spec.whatwg.org)-compatible alias.
         */
        get: function () {
            return this.prev;
        },
        set: function (prev) {
            this.prev = prev;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Node.prototype, "nextSibling", {
        /**
         * Same as {@link next}.
         * [DOM spec](https://dom.spec.whatwg.org)-compatible alias.
         */
        get: function () {
            return this.next;
        },
        set: function (next) {
            this.next = next;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Clone this node, and optionally its children.
     *
     * @param recursive Clone child nodes as well.
     * @returns A clone of the node.
     */
    Node.prototype.cloneNode = function (recursive) {
        if (recursive === void 0) { recursive = false; }
        return cloneNode(this, recursive);
    };
    return Node;
}());
node.Node = Node;
/**
 * A node that contains some data.
 */
var DataNode = /** @class */ (function (_super) {
    __extends(DataNode, _super);
    /**
     * @param data The content of the data node
     */
    function DataNode(data) {
        var _this = _super.call(this) || this;
        _this.data = data;
        return _this;
    }
    Object.defineProperty(DataNode.prototype, "nodeValue", {
        /**
         * Same as {@link data}.
         * [DOM spec](https://dom.spec.whatwg.org)-compatible alias.
         */
        get: function () {
            return this.data;
        },
        set: function (data) {
            this.data = data;
        },
        enumerable: false,
        configurable: true
    });
    return DataNode;
}(Node));
node.DataNode = DataNode;
/**
 * Text within the document.
 */
var Text = /** @class */ (function (_super) {
    __extends(Text, _super);
    function Text() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.type = domelementtype_1$1.ElementType.Text;
        return _this;
    }
    Object.defineProperty(Text.prototype, "nodeType", {
        get: function () {
            return 3;
        },
        enumerable: false,
        configurable: true
    });
    return Text;
}(DataNode));
node.Text = Text;
/**
 * Comments within the document.
 */
var Comment = /** @class */ (function (_super) {
    __extends(Comment, _super);
    function Comment() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.type = domelementtype_1$1.ElementType.Comment;
        return _this;
    }
    Object.defineProperty(Comment.prototype, "nodeType", {
        get: function () {
            return 8;
        },
        enumerable: false,
        configurable: true
    });
    return Comment;
}(DataNode));
node.Comment = Comment;
/**
 * Processing instructions, including doc types.
 */
var ProcessingInstruction = /** @class */ (function (_super) {
    __extends(ProcessingInstruction, _super);
    function ProcessingInstruction(name, data) {
        var _this = _super.call(this, data) || this;
        _this.name = name;
        _this.type = domelementtype_1$1.ElementType.Directive;
        return _this;
    }
    Object.defineProperty(ProcessingInstruction.prototype, "nodeType", {
        get: function () {
            return 1;
        },
        enumerable: false,
        configurable: true
    });
    return ProcessingInstruction;
}(DataNode));
node.ProcessingInstruction = ProcessingInstruction;
/**
 * A `Node` that can have children.
 */
var NodeWithChildren = /** @class */ (function (_super) {
    __extends(NodeWithChildren, _super);
    /**
     * @param children Children of the node. Only certain node types can have children.
     */
    function NodeWithChildren(children) {
        var _this = _super.call(this) || this;
        _this.children = children;
        return _this;
    }
    Object.defineProperty(NodeWithChildren.prototype, "firstChild", {
        // Aliases
        /** First child of the node. */
        get: function () {
            var _a;
            return (_a = this.children[0]) !== null && _a !== void 0 ? _a : null;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(NodeWithChildren.prototype, "lastChild", {
        /** Last child of the node. */
        get: function () {
            return this.children.length > 0
                ? this.children[this.children.length - 1]
                : null;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(NodeWithChildren.prototype, "childNodes", {
        /**
         * Same as {@link children}.
         * [DOM spec](https://dom.spec.whatwg.org)-compatible alias.
         */
        get: function () {
            return this.children;
        },
        set: function (children) {
            this.children = children;
        },
        enumerable: false,
        configurable: true
    });
    return NodeWithChildren;
}(Node));
node.NodeWithChildren = NodeWithChildren;
var CDATA = /** @class */ (function (_super) {
    __extends(CDATA, _super);
    function CDATA() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.type = domelementtype_1$1.ElementType.CDATA;
        return _this;
    }
    Object.defineProperty(CDATA.prototype, "nodeType", {
        get: function () {
            return 4;
        },
        enumerable: false,
        configurable: true
    });
    return CDATA;
}(NodeWithChildren));
node.CDATA = CDATA;
/**
 * The root node of the document.
 */
var Document = /** @class */ (function (_super) {
    __extends(Document, _super);
    function Document() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.type = domelementtype_1$1.ElementType.Root;
        return _this;
    }
    Object.defineProperty(Document.prototype, "nodeType", {
        get: function () {
            return 9;
        },
        enumerable: false,
        configurable: true
    });
    return Document;
}(NodeWithChildren));
node.Document = Document;
/**
 * An element within the DOM.
 */
var Element$1 = /** @class */ (function (_super) {
    __extends(Element, _super);
    /**
     * @param name Name of the tag, eg. `div`, `span`.
     * @param attribs Object mapping attribute names to attribute values.
     * @param children Children of the node.
     */
    function Element(name, attribs, children, type) {
        if (children === void 0) { children = []; }
        if (type === void 0) { type = name === "script"
            ? domelementtype_1$1.ElementType.Script
            : name === "style"
                ? domelementtype_1$1.ElementType.Style
                : domelementtype_1$1.ElementType.Tag; }
        var _this = _super.call(this, children) || this;
        _this.name = name;
        _this.attribs = attribs;
        _this.type = type;
        return _this;
    }
    Object.defineProperty(Element.prototype, "nodeType", {
        get: function () {
            return 1;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Element.prototype, "tagName", {
        // DOM Level 1 aliases
        /**
         * Same as {@link name}.
         * [DOM spec](https://dom.spec.whatwg.org)-compatible alias.
         */
        get: function () {
            return this.name;
        },
        set: function (name) {
            this.name = name;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Element.prototype, "attributes", {
        get: function () {
            var _this = this;
            return Object.keys(this.attribs).map(function (name) {
                var _a, _b;
                return ({
                    name: name,
                    value: _this.attribs[name],
                    namespace: (_a = _this["x-attribsNamespace"]) === null || _a === void 0 ? void 0 : _a[name],
                    prefix: (_b = _this["x-attribsPrefix"]) === null || _b === void 0 ? void 0 : _b[name],
                });
            });
        },
        enumerable: false,
        configurable: true
    });
    return Element;
}(NodeWithChildren));
node.Element = Element$1;
/**
 * @param node Node to check.
 * @returns `true` if the node is a `Element`, `false` otherwise.
 */
function isTag(node) {
    return (0, domelementtype_1$1.isTag)(node);
}
node.isTag = isTag;
/**
 * @param node Node to check.
 * @returns `true` if the node has the type `CDATA`, `false` otherwise.
 */
function isCDATA(node) {
    return node.type === domelementtype_1$1.ElementType.CDATA;
}
node.isCDATA = isCDATA;
/**
 * @param node Node to check.
 * @returns `true` if the node has the type `Text`, `false` otherwise.
 */
function isText(node) {
    return node.type === domelementtype_1$1.ElementType.Text;
}
node.isText = isText;
/**
 * @param node Node to check.
 * @returns `true` if the node has the type `Comment`, `false` otherwise.
 */
function isComment(node) {
    return node.type === domelementtype_1$1.ElementType.Comment;
}
node.isComment = isComment;
/**
 * @param node Node to check.
 * @returns `true` if the node has the type `ProcessingInstruction`, `false` otherwise.
 */
function isDirective(node) {
    return node.type === domelementtype_1$1.ElementType.Directive;
}
node.isDirective = isDirective;
/**
 * @param node Node to check.
 * @returns `true` if the node has the type `ProcessingInstruction`, `false` otherwise.
 */
function isDocument(node) {
    return node.type === domelementtype_1$1.ElementType.Root;
}
node.isDocument = isDocument;
/**
 * @param node Node to check.
 * @returns `true` if the node has children, `false` otherwise.
 */
function hasChildren(node) {
    return Object.prototype.hasOwnProperty.call(node, "children");
}
node.hasChildren = hasChildren;
/**
 * Clone a node, and optionally its children.
 *
 * @param recursive Clone child nodes as well.
 * @returns A clone of the node.
 */
function cloneNode(node, recursive) {
    if (recursive === void 0) { recursive = false; }
    var result;
    if (isText(node)) {
        result = new Text(node.data);
    }
    else if (isComment(node)) {
        result = new Comment(node.data);
    }
    else if (isTag(node)) {
        var children = recursive ? cloneChildren(node.children) : [];
        var clone_1 = new Element$1(node.name, __assign$1({}, node.attribs), children);
        children.forEach(function (child) { return (child.parent = clone_1); });
        if (node.namespace != null) {
            clone_1.namespace = node.namespace;
        }
        if (node["x-attribsNamespace"]) {
            clone_1["x-attribsNamespace"] = __assign$1({}, node["x-attribsNamespace"]);
        }
        if (node["x-attribsPrefix"]) {
            clone_1["x-attribsPrefix"] = __assign$1({}, node["x-attribsPrefix"]);
        }
        result = clone_1;
    }
    else if (isCDATA(node)) {
        var children = recursive ? cloneChildren(node.children) : [];
        var clone_2 = new CDATA(children);
        children.forEach(function (child) { return (child.parent = clone_2); });
        result = clone_2;
    }
    else if (isDocument(node)) {
        var children = recursive ? cloneChildren(node.children) : [];
        var clone_3 = new Document(children);
        children.forEach(function (child) { return (child.parent = clone_3); });
        if (node["x-mode"]) {
            clone_3["x-mode"] = node["x-mode"];
        }
        result = clone_3;
    }
    else if (isDirective(node)) {
        var instruction = new ProcessingInstruction(node.name, node.data);
        if (node["x-name"] != null) {
            instruction["x-name"] = node["x-name"];
            instruction["x-publicId"] = node["x-publicId"];
            instruction["x-systemId"] = node["x-systemId"];
        }
        result = instruction;
    }
    else {
        throw new Error("Not implemented yet: ".concat(node.type));
    }
    result.startIndex = node.startIndex;
    result.endIndex = node.endIndex;
    if (node.sourceCodeLocation != null) {
        result.sourceCodeLocation = node.sourceCodeLocation;
    }
    return result;
}
node.cloneNode = cloneNode;
function cloneChildren(childs) {
    var children = childs.map(function (child) { return cloneNode(child, true); });
    for (var i = 1; i < children.length; i++) {
        children[i].prev = children[i - 1];
        children[i - 1].next = children[i];
    }
    return children;
}

(function (exports) {
var __createBinding = (commonjsGlobal && commonjsGlobal.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (commonjsGlobal && commonjsGlobal.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DomHandler = void 0;
var domelementtype_1 = lib$5;
var node_js_1 = node;
__exportStar(node, exports);
// Default options
var defaultOpts = {
    withStartIndices: false,
    withEndIndices: false,
    xmlMode: false,
};
var DomHandler = /** @class */ (function () {
    /**
     * @param callback Called once parsing has completed.
     * @param options Settings for the handler.
     * @param elementCB Callback whenever a tag is closed.
     */
    function DomHandler(callback, options, elementCB) {
        /** The elements of the DOM */
        this.dom = [];
        /** The root element for the DOM */
        this.root = new node_js_1.Document(this.dom);
        /** Indicated whether parsing has been completed. */
        this.done = false;
        /** Stack of open tags. */
        this.tagStack = [this.root];
        /** A data node that is still being written to. */
        this.lastNode = null;
        /** Reference to the parser instance. Used for location information. */
        this.parser = null;
        // Make it possible to skip arguments, for backwards-compatibility
        if (typeof options === "function") {
            elementCB = options;
            options = defaultOpts;
        }
        if (typeof callback === "object") {
            options = callback;
            callback = undefined;
        }
        this.callback = callback !== null && callback !== void 0 ? callback : null;
        this.options = options !== null && options !== void 0 ? options : defaultOpts;
        this.elementCB = elementCB !== null && elementCB !== void 0 ? elementCB : null;
    }
    DomHandler.prototype.onparserinit = function (parser) {
        this.parser = parser;
    };
    // Resets the handler back to starting state
    DomHandler.prototype.onreset = function () {
        this.dom = [];
        this.root = new node_js_1.Document(this.dom);
        this.done = false;
        this.tagStack = [this.root];
        this.lastNode = null;
        this.parser = null;
    };
    // Signals the handler that parsing is done
    DomHandler.prototype.onend = function () {
        if (this.done)
            return;
        this.done = true;
        this.parser = null;
        this.handleCallback(null);
    };
    DomHandler.prototype.onerror = function (error) {
        this.handleCallback(error);
    };
    DomHandler.prototype.onclosetag = function () {
        this.lastNode = null;
        var elem = this.tagStack.pop();
        if (this.options.withEndIndices) {
            elem.endIndex = this.parser.endIndex;
        }
        if (this.elementCB)
            this.elementCB(elem);
    };
    DomHandler.prototype.onopentag = function (name, attribs) {
        var type = this.options.xmlMode ? domelementtype_1.ElementType.Tag : undefined;
        var element = new node_js_1.Element(name, attribs, undefined, type);
        this.addNode(element);
        this.tagStack.push(element);
    };
    DomHandler.prototype.ontext = function (data) {
        var lastNode = this.lastNode;
        if (lastNode && lastNode.type === domelementtype_1.ElementType.Text) {
            lastNode.data += data;
            if (this.options.withEndIndices) {
                lastNode.endIndex = this.parser.endIndex;
            }
        }
        else {
            var node = new node_js_1.Text(data);
            this.addNode(node);
            this.lastNode = node;
        }
    };
    DomHandler.prototype.oncomment = function (data) {
        if (this.lastNode && this.lastNode.type === domelementtype_1.ElementType.Comment) {
            this.lastNode.data += data;
            return;
        }
        var node = new node_js_1.Comment(data);
        this.addNode(node);
        this.lastNode = node;
    };
    DomHandler.prototype.oncommentend = function () {
        this.lastNode = null;
    };
    DomHandler.prototype.oncdatastart = function () {
        var text = new node_js_1.Text("");
        var node = new node_js_1.CDATA([text]);
        this.addNode(node);
        text.parent = node;
        this.lastNode = text;
    };
    DomHandler.prototype.oncdataend = function () {
        this.lastNode = null;
    };
    DomHandler.prototype.onprocessinginstruction = function (name, data) {
        var node = new node_js_1.ProcessingInstruction(name, data);
        this.addNode(node);
    };
    DomHandler.prototype.handleCallback = function (error) {
        if (typeof this.callback === "function") {
            this.callback(error, this.dom);
        }
        else if (error) {
            throw error;
        }
    };
    DomHandler.prototype.addNode = function (node) {
        var parent = this.tagStack[this.tagStack.length - 1];
        var previousSibling = parent.children[parent.children.length - 1];
        if (this.options.withStartIndices) {
            node.startIndex = this.parser.startIndex;
        }
        if (this.options.withEndIndices) {
            node.endIndex = this.parser.endIndex;
        }
        parent.children.push(node);
        if (previousSibling) {
            node.prev = previousSibling;
            previousSibling.next = node;
        }
        node.parent = parent;
        this.lastNode = null;
    };
    return DomHandler;
}());
exports.DomHandler = DomHandler;
exports.default = DomHandler;
}(lib$6));

var htmlDomParser = {exports: {}};

var lib$4 = {};

var Parser$2 = {};

var Tokenizer = {};

var decode = {};

var decodeDataHtml = {};

// Generated using scripts/write-decode-map.ts
Object.defineProperty(decodeDataHtml, "__esModule", { value: true });
decodeDataHtml.default = new Uint16Array(
// prettier-ignore
"\u1d41<\xd5\u0131\u028a\u049d\u057b\u05d0\u0675\u06de\u07a2\u07d6\u080f\u0a4a\u0a91\u0da1\u0e6d\u0f09\u0f26\u10ca\u1228\u12e1\u1415\u149d\u14c3\u14df\u1525\0\0\0\0\0\0\u156b\u16cd\u198d\u1c12\u1ddd\u1f7e\u2060\u21b0\u228d\u23c0\u23fb\u2442\u2824\u2912\u2d08\u2e48\u2fce\u3016\u32ba\u3639\u37ac\u38fe\u3a28\u3a71\u3ae0\u3b2e\u0800EMabcfglmnoprstu\\bfms\x7f\x84\x8b\x90\x95\x98\xa6\xb3\xb9\xc8\xcflig\u803b\xc6\u40c6P\u803b&\u4026cute\u803b\xc1\u40c1reve;\u4102\u0100iyx}rc\u803b\xc2\u40c2;\u4410r;\uc000\ud835\udd04rave\u803b\xc0\u40c0pha;\u4391acr;\u4100d;\u6a53\u0100gp\x9d\xa1on;\u4104f;\uc000\ud835\udd38plyFunction;\u6061ing\u803b\xc5\u40c5\u0100cs\xbe\xc3r;\uc000\ud835\udc9cign;\u6254ilde\u803b\xc3\u40c3ml\u803b\xc4\u40c4\u0400aceforsu\xe5\xfb\xfe\u0117\u011c\u0122\u0127\u012a\u0100cr\xea\xf2kslash;\u6216\u0176\xf6\xf8;\u6ae7ed;\u6306y;\u4411\u0180crt\u0105\u010b\u0114ause;\u6235noullis;\u612ca;\u4392r;\uc000\ud835\udd05pf;\uc000\ud835\udd39eve;\u42d8c\xf2\u0113mpeq;\u624e\u0700HOacdefhilorsu\u014d\u0151\u0156\u0180\u019e\u01a2\u01b5\u01b7\u01ba\u01dc\u0215\u0273\u0278\u027ecy;\u4427PY\u803b\xa9\u40a9\u0180cpy\u015d\u0162\u017aute;\u4106\u0100;i\u0167\u0168\u62d2talDifferentialD;\u6145leys;\u612d\u0200aeio\u0189\u018e\u0194\u0198ron;\u410cdil\u803b\xc7\u40c7rc;\u4108nint;\u6230ot;\u410a\u0100dn\u01a7\u01adilla;\u40b8terDot;\u40b7\xf2\u017fi;\u43a7rcle\u0200DMPT\u01c7\u01cb\u01d1\u01d6ot;\u6299inus;\u6296lus;\u6295imes;\u6297o\u0100cs\u01e2\u01f8kwiseContourIntegral;\u6232eCurly\u0100DQ\u0203\u020foubleQuote;\u601duote;\u6019\u0200lnpu\u021e\u0228\u0247\u0255on\u0100;e\u0225\u0226\u6237;\u6a74\u0180git\u022f\u0236\u023aruent;\u6261nt;\u622fourIntegral;\u622e\u0100fr\u024c\u024e;\u6102oduct;\u6210nterClockwiseContourIntegral;\u6233oss;\u6a2fcr;\uc000\ud835\udc9ep\u0100;C\u0284\u0285\u62d3ap;\u624d\u0580DJSZacefios\u02a0\u02ac\u02b0\u02b4\u02b8\u02cb\u02d7\u02e1\u02e6\u0333\u048d\u0100;o\u0179\u02a5trahd;\u6911cy;\u4402cy;\u4405cy;\u440f\u0180grs\u02bf\u02c4\u02c7ger;\u6021r;\u61a1hv;\u6ae4\u0100ay\u02d0\u02d5ron;\u410e;\u4414l\u0100;t\u02dd\u02de\u6207a;\u4394r;\uc000\ud835\udd07\u0100af\u02eb\u0327\u0100cm\u02f0\u0322ritical\u0200ADGT\u0300\u0306\u0316\u031ccute;\u40b4o\u0174\u030b\u030d;\u42d9bleAcute;\u42ddrave;\u4060ilde;\u42dcond;\u62c4ferentialD;\u6146\u0470\u033d\0\0\0\u0342\u0354\0\u0405f;\uc000\ud835\udd3b\u0180;DE\u0348\u0349\u034d\u40a8ot;\u60dcqual;\u6250ble\u0300CDLRUV\u0363\u0372\u0382\u03cf\u03e2\u03f8ontourIntegra\xec\u0239o\u0274\u0379\0\0\u037b\xbb\u0349nArrow;\u61d3\u0100eo\u0387\u03a4ft\u0180ART\u0390\u0396\u03a1rrow;\u61d0ightArrow;\u61d4e\xe5\u02cang\u0100LR\u03ab\u03c4eft\u0100AR\u03b3\u03b9rrow;\u67f8ightArrow;\u67faightArrow;\u67f9ight\u0100AT\u03d8\u03derrow;\u61d2ee;\u62a8p\u0241\u03e9\0\0\u03efrrow;\u61d1ownArrow;\u61d5erticalBar;\u6225n\u0300ABLRTa\u0412\u042a\u0430\u045e\u047f\u037crrow\u0180;BU\u041d\u041e\u0422\u6193ar;\u6913pArrow;\u61f5reve;\u4311eft\u02d2\u043a\0\u0446\0\u0450ightVector;\u6950eeVector;\u695eector\u0100;B\u0459\u045a\u61bdar;\u6956ight\u01d4\u0467\0\u0471eeVector;\u695fector\u0100;B\u047a\u047b\u61c1ar;\u6957ee\u0100;A\u0486\u0487\u62a4rrow;\u61a7\u0100ct\u0492\u0497r;\uc000\ud835\udc9frok;\u4110\u0800NTacdfglmopqstux\u04bd\u04c0\u04c4\u04cb\u04de\u04e2\u04e7\u04ee\u04f5\u0521\u052f\u0536\u0552\u055d\u0560\u0565G;\u414aH\u803b\xd0\u40d0cute\u803b\xc9\u40c9\u0180aiy\u04d2\u04d7\u04dcron;\u411arc\u803b\xca\u40ca;\u442dot;\u4116r;\uc000\ud835\udd08rave\u803b\xc8\u40c8ement;\u6208\u0100ap\u04fa\u04fecr;\u4112ty\u0253\u0506\0\0\u0512mallSquare;\u65fberySmallSquare;\u65ab\u0100gp\u0526\u052aon;\u4118f;\uc000\ud835\udd3csilon;\u4395u\u0100ai\u053c\u0549l\u0100;T\u0542\u0543\u6a75ilde;\u6242librium;\u61cc\u0100ci\u0557\u055ar;\u6130m;\u6a73a;\u4397ml\u803b\xcb\u40cb\u0100ip\u056a\u056fsts;\u6203onentialE;\u6147\u0280cfios\u0585\u0588\u058d\u05b2\u05ccy;\u4424r;\uc000\ud835\udd09lled\u0253\u0597\0\0\u05a3mallSquare;\u65fcerySmallSquare;\u65aa\u0370\u05ba\0\u05bf\0\0\u05c4f;\uc000\ud835\udd3dAll;\u6200riertrf;\u6131c\xf2\u05cb\u0600JTabcdfgorst\u05e8\u05ec\u05ef\u05fa\u0600\u0612\u0616\u061b\u061d\u0623\u066c\u0672cy;\u4403\u803b>\u403emma\u0100;d\u05f7\u05f8\u4393;\u43dcreve;\u411e\u0180eiy\u0607\u060c\u0610dil;\u4122rc;\u411c;\u4413ot;\u4120r;\uc000\ud835\udd0a;\u62d9pf;\uc000\ud835\udd3eeater\u0300EFGLST\u0635\u0644\u064e\u0656\u065b\u0666qual\u0100;L\u063e\u063f\u6265ess;\u62dbullEqual;\u6267reater;\u6aa2ess;\u6277lantEqual;\u6a7eilde;\u6273cr;\uc000\ud835\udca2;\u626b\u0400Aacfiosu\u0685\u068b\u0696\u069b\u069e\u06aa\u06be\u06caRDcy;\u442a\u0100ct\u0690\u0694ek;\u42c7;\u405eirc;\u4124r;\u610clbertSpace;\u610b\u01f0\u06af\0\u06b2f;\u610dizontalLine;\u6500\u0100ct\u06c3\u06c5\xf2\u06a9rok;\u4126mp\u0144\u06d0\u06d8ownHum\xf0\u012fqual;\u624f\u0700EJOacdfgmnostu\u06fa\u06fe\u0703\u0707\u070e\u071a\u071e\u0721\u0728\u0744\u0778\u078b\u078f\u0795cy;\u4415lig;\u4132cy;\u4401cute\u803b\xcd\u40cd\u0100iy\u0713\u0718rc\u803b\xce\u40ce;\u4418ot;\u4130r;\u6111rave\u803b\xcc\u40cc\u0180;ap\u0720\u072f\u073f\u0100cg\u0734\u0737r;\u412ainaryI;\u6148lie\xf3\u03dd\u01f4\u0749\0\u0762\u0100;e\u074d\u074e\u622c\u0100gr\u0753\u0758ral;\u622bsection;\u62c2isible\u0100CT\u076c\u0772omma;\u6063imes;\u6062\u0180gpt\u077f\u0783\u0788on;\u412ef;\uc000\ud835\udd40a;\u4399cr;\u6110ilde;\u4128\u01eb\u079a\0\u079ecy;\u4406l\u803b\xcf\u40cf\u0280cfosu\u07ac\u07b7\u07bc\u07c2\u07d0\u0100iy\u07b1\u07b5rc;\u4134;\u4419r;\uc000\ud835\udd0dpf;\uc000\ud835\udd41\u01e3\u07c7\0\u07ccr;\uc000\ud835\udca5rcy;\u4408kcy;\u4404\u0380HJacfos\u07e4\u07e8\u07ec\u07f1\u07fd\u0802\u0808cy;\u4425cy;\u440cppa;\u439a\u0100ey\u07f6\u07fbdil;\u4136;\u441ar;\uc000\ud835\udd0epf;\uc000\ud835\udd42cr;\uc000\ud835\udca6\u0580JTaceflmost\u0825\u0829\u082c\u0850\u0863\u09b3\u09b8\u09c7\u09cd\u0a37\u0a47cy;\u4409\u803b<\u403c\u0280cmnpr\u0837\u083c\u0841\u0844\u084dute;\u4139bda;\u439bg;\u67ealacetrf;\u6112r;\u619e\u0180aey\u0857\u085c\u0861ron;\u413ddil;\u413b;\u441b\u0100fs\u0868\u0970t\u0500ACDFRTUVar\u087e\u08a9\u08b1\u08e0\u08e6\u08fc\u092f\u095b\u0390\u096a\u0100nr\u0883\u088fgleBracket;\u67e8row\u0180;BR\u0899\u089a\u089e\u6190ar;\u61e4ightArrow;\u61c6eiling;\u6308o\u01f5\u08b7\0\u08c3bleBracket;\u67e6n\u01d4\u08c8\0\u08d2eeVector;\u6961ector\u0100;B\u08db\u08dc\u61c3ar;\u6959loor;\u630aight\u0100AV\u08ef\u08f5rrow;\u6194ector;\u694e\u0100er\u0901\u0917e\u0180;AV\u0909\u090a\u0910\u62a3rrow;\u61a4ector;\u695aiangle\u0180;BE\u0924\u0925\u0929\u62b2ar;\u69cfqual;\u62b4p\u0180DTV\u0937\u0942\u094cownVector;\u6951eeVector;\u6960ector\u0100;B\u0956\u0957\u61bfar;\u6958ector\u0100;B\u0965\u0966\u61bcar;\u6952ight\xe1\u039cs\u0300EFGLST\u097e\u098b\u0995\u099d\u09a2\u09adqualGreater;\u62daullEqual;\u6266reater;\u6276ess;\u6aa1lantEqual;\u6a7dilde;\u6272r;\uc000\ud835\udd0f\u0100;e\u09bd\u09be\u62d8ftarrow;\u61daidot;\u413f\u0180npw\u09d4\u0a16\u0a1bg\u0200LRlr\u09de\u09f7\u0a02\u0a10eft\u0100AR\u09e6\u09ecrrow;\u67f5ightArrow;\u67f7ightArrow;\u67f6eft\u0100ar\u03b3\u0a0aight\xe1\u03bfight\xe1\u03caf;\uc000\ud835\udd43er\u0100LR\u0a22\u0a2ceftArrow;\u6199ightArrow;\u6198\u0180cht\u0a3e\u0a40\u0a42\xf2\u084c;\u61b0rok;\u4141;\u626a\u0400acefiosu\u0a5a\u0a5d\u0a60\u0a77\u0a7c\u0a85\u0a8b\u0a8ep;\u6905y;\u441c\u0100dl\u0a65\u0a6fiumSpace;\u605flintrf;\u6133r;\uc000\ud835\udd10nusPlus;\u6213pf;\uc000\ud835\udd44c\xf2\u0a76;\u439c\u0480Jacefostu\u0aa3\u0aa7\u0aad\u0ac0\u0b14\u0b19\u0d91\u0d97\u0d9ecy;\u440acute;\u4143\u0180aey\u0ab4\u0ab9\u0aberon;\u4147dil;\u4145;\u441d\u0180gsw\u0ac7\u0af0\u0b0eative\u0180MTV\u0ad3\u0adf\u0ae8ediumSpace;\u600bhi\u0100cn\u0ae6\u0ad8\xeb\u0ad9eryThi\xee\u0ad9ted\u0100GL\u0af8\u0b06reaterGreate\xf2\u0673essLes\xf3\u0a48Line;\u400ar;\uc000\ud835\udd11\u0200Bnpt\u0b22\u0b28\u0b37\u0b3areak;\u6060BreakingSpace;\u40a0f;\u6115\u0680;CDEGHLNPRSTV\u0b55\u0b56\u0b6a\u0b7c\u0ba1\u0beb\u0c04\u0c5e\u0c84\u0ca6\u0cd8\u0d61\u0d85\u6aec\u0100ou\u0b5b\u0b64ngruent;\u6262pCap;\u626doubleVerticalBar;\u6226\u0180lqx\u0b83\u0b8a\u0b9bement;\u6209ual\u0100;T\u0b92\u0b93\u6260ilde;\uc000\u2242\u0338ists;\u6204reater\u0380;EFGLST\u0bb6\u0bb7\u0bbd\u0bc9\u0bd3\u0bd8\u0be5\u626fqual;\u6271ullEqual;\uc000\u2267\u0338reater;\uc000\u226b\u0338ess;\u6279lantEqual;\uc000\u2a7e\u0338ilde;\u6275ump\u0144\u0bf2\u0bfdownHump;\uc000\u224e\u0338qual;\uc000\u224f\u0338e\u0100fs\u0c0a\u0c27tTriangle\u0180;BE\u0c1a\u0c1b\u0c21\u62eaar;\uc000\u29cf\u0338qual;\u62ecs\u0300;EGLST\u0c35\u0c36\u0c3c\u0c44\u0c4b\u0c58\u626equal;\u6270reater;\u6278ess;\uc000\u226a\u0338lantEqual;\uc000\u2a7d\u0338ilde;\u6274ested\u0100GL\u0c68\u0c79reaterGreater;\uc000\u2aa2\u0338essLess;\uc000\u2aa1\u0338recedes\u0180;ES\u0c92\u0c93\u0c9b\u6280qual;\uc000\u2aaf\u0338lantEqual;\u62e0\u0100ei\u0cab\u0cb9verseElement;\u620cghtTriangle\u0180;BE\u0ccb\u0ccc\u0cd2\u62ebar;\uc000\u29d0\u0338qual;\u62ed\u0100qu\u0cdd\u0d0cuareSu\u0100bp\u0ce8\u0cf9set\u0100;E\u0cf0\u0cf3\uc000\u228f\u0338qual;\u62e2erset\u0100;E\u0d03\u0d06\uc000\u2290\u0338qual;\u62e3\u0180bcp\u0d13\u0d24\u0d4eset\u0100;E\u0d1b\u0d1e\uc000\u2282\u20d2qual;\u6288ceeds\u0200;EST\u0d32\u0d33\u0d3b\u0d46\u6281qual;\uc000\u2ab0\u0338lantEqual;\u62e1ilde;\uc000\u227f\u0338erset\u0100;E\u0d58\u0d5b\uc000\u2283\u20d2qual;\u6289ilde\u0200;EFT\u0d6e\u0d6f\u0d75\u0d7f\u6241qual;\u6244ullEqual;\u6247ilde;\u6249erticalBar;\u6224cr;\uc000\ud835\udca9ilde\u803b\xd1\u40d1;\u439d\u0700Eacdfgmoprstuv\u0dbd\u0dc2\u0dc9\u0dd5\u0ddb\u0de0\u0de7\u0dfc\u0e02\u0e20\u0e22\u0e32\u0e3f\u0e44lig;\u4152cute\u803b\xd3\u40d3\u0100iy\u0dce\u0dd3rc\u803b\xd4\u40d4;\u441eblac;\u4150r;\uc000\ud835\udd12rave\u803b\xd2\u40d2\u0180aei\u0dee\u0df2\u0df6cr;\u414cga;\u43a9cron;\u439fpf;\uc000\ud835\udd46enCurly\u0100DQ\u0e0e\u0e1aoubleQuote;\u601cuote;\u6018;\u6a54\u0100cl\u0e27\u0e2cr;\uc000\ud835\udcaaash\u803b\xd8\u40d8i\u016c\u0e37\u0e3cde\u803b\xd5\u40d5es;\u6a37ml\u803b\xd6\u40d6er\u0100BP\u0e4b\u0e60\u0100ar\u0e50\u0e53r;\u603eac\u0100ek\u0e5a\u0e5c;\u63deet;\u63b4arenthesis;\u63dc\u0480acfhilors\u0e7f\u0e87\u0e8a\u0e8f\u0e92\u0e94\u0e9d\u0eb0\u0efcrtialD;\u6202y;\u441fr;\uc000\ud835\udd13i;\u43a6;\u43a0usMinus;\u40b1\u0100ip\u0ea2\u0eadncareplan\xe5\u069df;\u6119\u0200;eio\u0eb9\u0eba\u0ee0\u0ee4\u6abbcedes\u0200;EST\u0ec8\u0ec9\u0ecf\u0eda\u627aqual;\u6aaflantEqual;\u627cilde;\u627eme;\u6033\u0100dp\u0ee9\u0eeeuct;\u620fortion\u0100;a\u0225\u0ef9l;\u621d\u0100ci\u0f01\u0f06r;\uc000\ud835\udcab;\u43a8\u0200Ufos\u0f11\u0f16\u0f1b\u0f1fOT\u803b\"\u4022r;\uc000\ud835\udd14pf;\u611acr;\uc000\ud835\udcac\u0600BEacefhiorsu\u0f3e\u0f43\u0f47\u0f60\u0f73\u0fa7\u0faa\u0fad\u1096\u10a9\u10b4\u10bearr;\u6910G\u803b\xae\u40ae\u0180cnr\u0f4e\u0f53\u0f56ute;\u4154g;\u67ebr\u0100;t\u0f5c\u0f5d\u61a0l;\u6916\u0180aey\u0f67\u0f6c\u0f71ron;\u4158dil;\u4156;\u4420\u0100;v\u0f78\u0f79\u611cerse\u0100EU\u0f82\u0f99\u0100lq\u0f87\u0f8eement;\u620builibrium;\u61cbpEquilibrium;\u696fr\xbb\u0f79o;\u43a1ght\u0400ACDFTUVa\u0fc1\u0feb\u0ff3\u1022\u1028\u105b\u1087\u03d8\u0100nr\u0fc6\u0fd2gleBracket;\u67e9row\u0180;BL\u0fdc\u0fdd\u0fe1\u6192ar;\u61e5eftArrow;\u61c4eiling;\u6309o\u01f5\u0ff9\0\u1005bleBracket;\u67e7n\u01d4\u100a\0\u1014eeVector;\u695dector\u0100;B\u101d\u101e\u61c2ar;\u6955loor;\u630b\u0100er\u102d\u1043e\u0180;AV\u1035\u1036\u103c\u62a2rrow;\u61a6ector;\u695biangle\u0180;BE\u1050\u1051\u1055\u62b3ar;\u69d0qual;\u62b5p\u0180DTV\u1063\u106e\u1078ownVector;\u694feeVector;\u695cector\u0100;B\u1082\u1083\u61bear;\u6954ector\u0100;B\u1091\u1092\u61c0ar;\u6953\u0100pu\u109b\u109ef;\u611dndImplies;\u6970ightarrow;\u61db\u0100ch\u10b9\u10bcr;\u611b;\u61b1leDelayed;\u69f4\u0680HOacfhimoqstu\u10e4\u10f1\u10f7\u10fd\u1119\u111e\u1151\u1156\u1161\u1167\u11b5\u11bb\u11bf\u0100Cc\u10e9\u10eeHcy;\u4429y;\u4428FTcy;\u442ccute;\u415a\u0280;aeiy\u1108\u1109\u110e\u1113\u1117\u6abcron;\u4160dil;\u415erc;\u415c;\u4421r;\uc000\ud835\udd16ort\u0200DLRU\u112a\u1134\u113e\u1149ownArrow\xbb\u041eeftArrow\xbb\u089aightArrow\xbb\u0fddpArrow;\u6191gma;\u43a3allCircle;\u6218pf;\uc000\ud835\udd4a\u0272\u116d\0\0\u1170t;\u621aare\u0200;ISU\u117b\u117c\u1189\u11af\u65a1ntersection;\u6293u\u0100bp\u118f\u119eset\u0100;E\u1197\u1198\u628fqual;\u6291erset\u0100;E\u11a8\u11a9\u6290qual;\u6292nion;\u6294cr;\uc000\ud835\udcaear;\u62c6\u0200bcmp\u11c8\u11db\u1209\u120b\u0100;s\u11cd\u11ce\u62d0et\u0100;E\u11cd\u11d5qual;\u6286\u0100ch\u11e0\u1205eeds\u0200;EST\u11ed\u11ee\u11f4\u11ff\u627bqual;\u6ab0lantEqual;\u627dilde;\u627fTh\xe1\u0f8c;\u6211\u0180;es\u1212\u1213\u1223\u62d1rset\u0100;E\u121c\u121d\u6283qual;\u6287et\xbb\u1213\u0580HRSacfhiors\u123e\u1244\u1249\u1255\u125e\u1271\u1276\u129f\u12c2\u12c8\u12d1ORN\u803b\xde\u40deADE;\u6122\u0100Hc\u124e\u1252cy;\u440by;\u4426\u0100bu\u125a\u125c;\u4009;\u43a4\u0180aey\u1265\u126a\u126fron;\u4164dil;\u4162;\u4422r;\uc000\ud835\udd17\u0100ei\u127b\u1289\u01f2\u1280\0\u1287efore;\u6234a;\u4398\u0100cn\u128e\u1298kSpace;\uc000\u205f\u200aSpace;\u6009lde\u0200;EFT\u12ab\u12ac\u12b2\u12bc\u623cqual;\u6243ullEqual;\u6245ilde;\u6248pf;\uc000\ud835\udd4bipleDot;\u60db\u0100ct\u12d6\u12dbr;\uc000\ud835\udcafrok;\u4166\u0ae1\u12f7\u130e\u131a\u1326\0\u132c\u1331\0\0\0\0\0\u1338\u133d\u1377\u1385\0\u13ff\u1404\u140a\u1410\u0100cr\u12fb\u1301ute\u803b\xda\u40dar\u0100;o\u1307\u1308\u619fcir;\u6949r\u01e3\u1313\0\u1316y;\u440eve;\u416c\u0100iy\u131e\u1323rc\u803b\xdb\u40db;\u4423blac;\u4170r;\uc000\ud835\udd18rave\u803b\xd9\u40d9acr;\u416a\u0100di\u1341\u1369er\u0100BP\u1348\u135d\u0100ar\u134d\u1350r;\u405fac\u0100ek\u1357\u1359;\u63dfet;\u63b5arenthesis;\u63ddon\u0100;P\u1370\u1371\u62c3lus;\u628e\u0100gp\u137b\u137fon;\u4172f;\uc000\ud835\udd4c\u0400ADETadps\u1395\u13ae\u13b8\u13c4\u03e8\u13d2\u13d7\u13f3rrow\u0180;BD\u1150\u13a0\u13a4ar;\u6912ownArrow;\u61c5ownArrow;\u6195quilibrium;\u696eee\u0100;A\u13cb\u13cc\u62a5rrow;\u61a5own\xe1\u03f3er\u0100LR\u13de\u13e8eftArrow;\u6196ightArrow;\u6197i\u0100;l\u13f9\u13fa\u43d2on;\u43a5ing;\u416ecr;\uc000\ud835\udcb0ilde;\u4168ml\u803b\xdc\u40dc\u0480Dbcdefosv\u1427\u142c\u1430\u1433\u143e\u1485\u148a\u1490\u1496ash;\u62abar;\u6aeby;\u4412ash\u0100;l\u143b\u143c\u62a9;\u6ae6\u0100er\u1443\u1445;\u62c1\u0180bty\u144c\u1450\u147aar;\u6016\u0100;i\u144f\u1455cal\u0200BLST\u1461\u1465\u146a\u1474ar;\u6223ine;\u407ceparator;\u6758ilde;\u6240ThinSpace;\u600ar;\uc000\ud835\udd19pf;\uc000\ud835\udd4dcr;\uc000\ud835\udcb1dash;\u62aa\u0280cefos\u14a7\u14ac\u14b1\u14b6\u14bcirc;\u4174dge;\u62c0r;\uc000\ud835\udd1apf;\uc000\ud835\udd4ecr;\uc000\ud835\udcb2\u0200fios\u14cb\u14d0\u14d2\u14d8r;\uc000\ud835\udd1b;\u439epf;\uc000\ud835\udd4fcr;\uc000\ud835\udcb3\u0480AIUacfosu\u14f1\u14f5\u14f9\u14fd\u1504\u150f\u1514\u151a\u1520cy;\u442fcy;\u4407cy;\u442ecute\u803b\xdd\u40dd\u0100iy\u1509\u150drc;\u4176;\u442br;\uc000\ud835\udd1cpf;\uc000\ud835\udd50cr;\uc000\ud835\udcb4ml;\u4178\u0400Hacdefos\u1535\u1539\u153f\u154b\u154f\u155d\u1560\u1564cy;\u4416cute;\u4179\u0100ay\u1544\u1549ron;\u417d;\u4417ot;\u417b\u01f2\u1554\0\u155boWidt\xe8\u0ad9a;\u4396r;\u6128pf;\u6124cr;\uc000\ud835\udcb5\u0be1\u1583\u158a\u1590\0\u15b0\u15b6\u15bf\0\0\0\0\u15c6\u15db\u15eb\u165f\u166d\0\u1695\u169b\u16b2\u16b9\0\u16becute\u803b\xe1\u40e1reve;\u4103\u0300;Ediuy\u159c\u159d\u15a1\u15a3\u15a8\u15ad\u623e;\uc000\u223e\u0333;\u623frc\u803b\xe2\u40e2te\u80bb\xb4\u0306;\u4430lig\u803b\xe6\u40e6\u0100;r\xb2\u15ba;\uc000\ud835\udd1erave\u803b\xe0\u40e0\u0100ep\u15ca\u15d6\u0100fp\u15cf\u15d4sym;\u6135\xe8\u15d3ha;\u43b1\u0100ap\u15dfc\u0100cl\u15e4\u15e7r;\u4101g;\u6a3f\u0264\u15f0\0\0\u160a\u0280;adsv\u15fa\u15fb\u15ff\u1601\u1607\u6227nd;\u6a55;\u6a5clope;\u6a58;\u6a5a\u0380;elmrsz\u1618\u1619\u161b\u161e\u163f\u164f\u1659\u6220;\u69a4e\xbb\u1619sd\u0100;a\u1625\u1626\u6221\u0461\u1630\u1632\u1634\u1636\u1638\u163a\u163c\u163e;\u69a8;\u69a9;\u69aa;\u69ab;\u69ac;\u69ad;\u69ae;\u69aft\u0100;v\u1645\u1646\u621fb\u0100;d\u164c\u164d\u62be;\u699d\u0100pt\u1654\u1657h;\u6222\xbb\xb9arr;\u637c\u0100gp\u1663\u1667on;\u4105f;\uc000\ud835\udd52\u0380;Eaeiop\u12c1\u167b\u167d\u1682\u1684\u1687\u168a;\u6a70cir;\u6a6f;\u624ad;\u624bs;\u4027rox\u0100;e\u12c1\u1692\xf1\u1683ing\u803b\xe5\u40e5\u0180cty\u16a1\u16a6\u16a8r;\uc000\ud835\udcb6;\u402amp\u0100;e\u12c1\u16af\xf1\u0288ilde\u803b\xe3\u40e3ml\u803b\xe4\u40e4\u0100ci\u16c2\u16c8onin\xf4\u0272nt;\u6a11\u0800Nabcdefiklnoprsu\u16ed\u16f1\u1730\u173c\u1743\u1748\u1778\u177d\u17e0\u17e6\u1839\u1850\u170d\u193d\u1948\u1970ot;\u6aed\u0100cr\u16f6\u171ek\u0200ceps\u1700\u1705\u170d\u1713ong;\u624cpsilon;\u43f6rime;\u6035im\u0100;e\u171a\u171b\u623dq;\u62cd\u0176\u1722\u1726ee;\u62bded\u0100;g\u172c\u172d\u6305e\xbb\u172drk\u0100;t\u135c\u1737brk;\u63b6\u0100oy\u1701\u1741;\u4431quo;\u601e\u0280cmprt\u1753\u175b\u1761\u1764\u1768aus\u0100;e\u010a\u0109ptyv;\u69b0s\xe9\u170cno\xf5\u0113\u0180ahw\u176f\u1771\u1773;\u43b2;\u6136een;\u626cr;\uc000\ud835\udd1fg\u0380costuvw\u178d\u179d\u17b3\u17c1\u17d5\u17db\u17de\u0180aiu\u1794\u1796\u179a\xf0\u0760rc;\u65efp\xbb\u1371\u0180dpt\u17a4\u17a8\u17adot;\u6a00lus;\u6a01imes;\u6a02\u0271\u17b9\0\0\u17becup;\u6a06ar;\u6605riangle\u0100du\u17cd\u17d2own;\u65bdp;\u65b3plus;\u6a04e\xe5\u1444\xe5\u14adarow;\u690d\u0180ako\u17ed\u1826\u1835\u0100cn\u17f2\u1823k\u0180lst\u17fa\u05ab\u1802ozenge;\u69ebriangle\u0200;dlr\u1812\u1813\u1818\u181d\u65b4own;\u65beeft;\u65c2ight;\u65b8k;\u6423\u01b1\u182b\0\u1833\u01b2\u182f\0\u1831;\u6592;\u65914;\u6593ck;\u6588\u0100eo\u183e\u184d\u0100;q\u1843\u1846\uc000=\u20e5uiv;\uc000\u2261\u20e5t;\u6310\u0200ptwx\u1859\u185e\u1867\u186cf;\uc000\ud835\udd53\u0100;t\u13cb\u1863om\xbb\u13cctie;\u62c8\u0600DHUVbdhmptuv\u1885\u1896\u18aa\u18bb\u18d7\u18db\u18ec\u18ff\u1905\u190a\u1910\u1921\u0200LRlr\u188e\u1890\u1892\u1894;\u6557;\u6554;\u6556;\u6553\u0280;DUdu\u18a1\u18a2\u18a4\u18a6\u18a8\u6550;\u6566;\u6569;\u6564;\u6567\u0200LRlr\u18b3\u18b5\u18b7\u18b9;\u655d;\u655a;\u655c;\u6559\u0380;HLRhlr\u18ca\u18cb\u18cd\u18cf\u18d1\u18d3\u18d5\u6551;\u656c;\u6563;\u6560;\u656b;\u6562;\u655fox;\u69c9\u0200LRlr\u18e4\u18e6\u18e8\u18ea;\u6555;\u6552;\u6510;\u650c\u0280;DUdu\u06bd\u18f7\u18f9\u18fb\u18fd;\u6565;\u6568;\u652c;\u6534inus;\u629flus;\u629eimes;\u62a0\u0200LRlr\u1919\u191b\u191d\u191f;\u655b;\u6558;\u6518;\u6514\u0380;HLRhlr\u1930\u1931\u1933\u1935\u1937\u1939\u193b\u6502;\u656a;\u6561;\u655e;\u653c;\u6524;\u651c\u0100ev\u0123\u1942bar\u803b\xa6\u40a6\u0200ceio\u1951\u1956\u195a\u1960r;\uc000\ud835\udcb7mi;\u604fm\u0100;e\u171a\u171cl\u0180;bh\u1968\u1969\u196b\u405c;\u69c5sub;\u67c8\u016c\u1974\u197el\u0100;e\u1979\u197a\u6022t\xbb\u197ap\u0180;Ee\u012f\u1985\u1987;\u6aae\u0100;q\u06dc\u06db\u0ce1\u19a7\0\u19e8\u1a11\u1a15\u1a32\0\u1a37\u1a50\0\0\u1ab4\0\0\u1ac1\0\0\u1b21\u1b2e\u1b4d\u1b52\0\u1bfd\0\u1c0c\u0180cpr\u19ad\u19b2\u19ddute;\u4107\u0300;abcds\u19bf\u19c0\u19c4\u19ca\u19d5\u19d9\u6229nd;\u6a44rcup;\u6a49\u0100au\u19cf\u19d2p;\u6a4bp;\u6a47ot;\u6a40;\uc000\u2229\ufe00\u0100eo\u19e2\u19e5t;\u6041\xee\u0693\u0200aeiu\u19f0\u19fb\u1a01\u1a05\u01f0\u19f5\0\u19f8s;\u6a4don;\u410ddil\u803b\xe7\u40e7rc;\u4109ps\u0100;s\u1a0c\u1a0d\u6a4cm;\u6a50ot;\u410b\u0180dmn\u1a1b\u1a20\u1a26il\u80bb\xb8\u01adptyv;\u69b2t\u8100\xa2;e\u1a2d\u1a2e\u40a2r\xe4\u01b2r;\uc000\ud835\udd20\u0180cei\u1a3d\u1a40\u1a4dy;\u4447ck\u0100;m\u1a47\u1a48\u6713ark\xbb\u1a48;\u43c7r\u0380;Ecefms\u1a5f\u1a60\u1a62\u1a6b\u1aa4\u1aaa\u1aae\u65cb;\u69c3\u0180;el\u1a69\u1a6a\u1a6d\u42c6q;\u6257e\u0261\u1a74\0\0\u1a88rrow\u0100lr\u1a7c\u1a81eft;\u61baight;\u61bb\u0280RSacd\u1a92\u1a94\u1a96\u1a9a\u1a9f\xbb\u0f47;\u64c8st;\u629birc;\u629aash;\u629dnint;\u6a10id;\u6aefcir;\u69c2ubs\u0100;u\u1abb\u1abc\u6663it\xbb\u1abc\u02ec\u1ac7\u1ad4\u1afa\0\u1b0aon\u0100;e\u1acd\u1ace\u403a\u0100;q\xc7\xc6\u026d\u1ad9\0\0\u1ae2a\u0100;t\u1ade\u1adf\u402c;\u4040\u0180;fl\u1ae8\u1ae9\u1aeb\u6201\xee\u1160e\u0100mx\u1af1\u1af6ent\xbb\u1ae9e\xf3\u024d\u01e7\u1afe\0\u1b07\u0100;d\u12bb\u1b02ot;\u6a6dn\xf4\u0246\u0180fry\u1b10\u1b14\u1b17;\uc000\ud835\udd54o\xe4\u0254\u8100\xa9;s\u0155\u1b1dr;\u6117\u0100ao\u1b25\u1b29rr;\u61b5ss;\u6717\u0100cu\u1b32\u1b37r;\uc000\ud835\udcb8\u0100bp\u1b3c\u1b44\u0100;e\u1b41\u1b42\u6acf;\u6ad1\u0100;e\u1b49\u1b4a\u6ad0;\u6ad2dot;\u62ef\u0380delprvw\u1b60\u1b6c\u1b77\u1b82\u1bac\u1bd4\u1bf9arr\u0100lr\u1b68\u1b6a;\u6938;\u6935\u0270\u1b72\0\0\u1b75r;\u62dec;\u62dfarr\u0100;p\u1b7f\u1b80\u61b6;\u693d\u0300;bcdos\u1b8f\u1b90\u1b96\u1ba1\u1ba5\u1ba8\u622arcap;\u6a48\u0100au\u1b9b\u1b9ep;\u6a46p;\u6a4aot;\u628dr;\u6a45;\uc000\u222a\ufe00\u0200alrv\u1bb5\u1bbf\u1bde\u1be3rr\u0100;m\u1bbc\u1bbd\u61b7;\u693cy\u0180evw\u1bc7\u1bd4\u1bd8q\u0270\u1bce\0\0\u1bd2re\xe3\u1b73u\xe3\u1b75ee;\u62ceedge;\u62cfen\u803b\xa4\u40a4earrow\u0100lr\u1bee\u1bf3eft\xbb\u1b80ight\xbb\u1bbde\xe4\u1bdd\u0100ci\u1c01\u1c07onin\xf4\u01f7nt;\u6231lcty;\u632d\u0980AHabcdefhijlorstuwz\u1c38\u1c3b\u1c3f\u1c5d\u1c69\u1c75\u1c8a\u1c9e\u1cac\u1cb7\u1cfb\u1cff\u1d0d\u1d7b\u1d91\u1dab\u1dbb\u1dc6\u1dcdr\xf2\u0381ar;\u6965\u0200glrs\u1c48\u1c4d\u1c52\u1c54ger;\u6020eth;\u6138\xf2\u1133h\u0100;v\u1c5a\u1c5b\u6010\xbb\u090a\u016b\u1c61\u1c67arow;\u690fa\xe3\u0315\u0100ay\u1c6e\u1c73ron;\u410f;\u4434\u0180;ao\u0332\u1c7c\u1c84\u0100gr\u02bf\u1c81r;\u61catseq;\u6a77\u0180glm\u1c91\u1c94\u1c98\u803b\xb0\u40b0ta;\u43b4ptyv;\u69b1\u0100ir\u1ca3\u1ca8sht;\u697f;\uc000\ud835\udd21ar\u0100lr\u1cb3\u1cb5\xbb\u08dc\xbb\u101e\u0280aegsv\u1cc2\u0378\u1cd6\u1cdc\u1ce0m\u0180;os\u0326\u1cca\u1cd4nd\u0100;s\u0326\u1cd1uit;\u6666amma;\u43ddin;\u62f2\u0180;io\u1ce7\u1ce8\u1cf8\u40f7de\u8100\xf7;o\u1ce7\u1cf0ntimes;\u62c7n\xf8\u1cf7cy;\u4452c\u026f\u1d06\0\0\u1d0arn;\u631eop;\u630d\u0280lptuw\u1d18\u1d1d\u1d22\u1d49\u1d55lar;\u4024f;\uc000\ud835\udd55\u0280;emps\u030b\u1d2d\u1d37\u1d3d\u1d42q\u0100;d\u0352\u1d33ot;\u6251inus;\u6238lus;\u6214quare;\u62a1blebarwedg\xe5\xfan\u0180adh\u112e\u1d5d\u1d67ownarrow\xf3\u1c83arpoon\u0100lr\u1d72\u1d76ef\xf4\u1cb4igh\xf4\u1cb6\u0162\u1d7f\u1d85karo\xf7\u0f42\u026f\u1d8a\0\0\u1d8ern;\u631fop;\u630c\u0180cot\u1d98\u1da3\u1da6\u0100ry\u1d9d\u1da1;\uc000\ud835\udcb9;\u4455l;\u69f6rok;\u4111\u0100dr\u1db0\u1db4ot;\u62f1i\u0100;f\u1dba\u1816\u65bf\u0100ah\u1dc0\u1dc3r\xf2\u0429a\xf2\u0fa6angle;\u69a6\u0100ci\u1dd2\u1dd5y;\u445fgrarr;\u67ff\u0900Dacdefglmnopqrstux\u1e01\u1e09\u1e19\u1e38\u0578\u1e3c\u1e49\u1e61\u1e7e\u1ea5\u1eaf\u1ebd\u1ee1\u1f2a\u1f37\u1f44\u1f4e\u1f5a\u0100Do\u1e06\u1d34o\xf4\u1c89\u0100cs\u1e0e\u1e14ute\u803b\xe9\u40e9ter;\u6a6e\u0200aioy\u1e22\u1e27\u1e31\u1e36ron;\u411br\u0100;c\u1e2d\u1e2e\u6256\u803b\xea\u40ealon;\u6255;\u444dot;\u4117\u0100Dr\u1e41\u1e45ot;\u6252;\uc000\ud835\udd22\u0180;rs\u1e50\u1e51\u1e57\u6a9aave\u803b\xe8\u40e8\u0100;d\u1e5c\u1e5d\u6a96ot;\u6a98\u0200;ils\u1e6a\u1e6b\u1e72\u1e74\u6a99nters;\u63e7;\u6113\u0100;d\u1e79\u1e7a\u6a95ot;\u6a97\u0180aps\u1e85\u1e89\u1e97cr;\u4113ty\u0180;sv\u1e92\u1e93\u1e95\u6205et\xbb\u1e93p\u01001;\u1e9d\u1ea4\u0133\u1ea1\u1ea3;\u6004;\u6005\u6003\u0100gs\u1eaa\u1eac;\u414bp;\u6002\u0100gp\u1eb4\u1eb8on;\u4119f;\uc000\ud835\udd56\u0180als\u1ec4\u1ece\u1ed2r\u0100;s\u1eca\u1ecb\u62d5l;\u69e3us;\u6a71i\u0180;lv\u1eda\u1edb\u1edf\u43b5on\xbb\u1edb;\u43f5\u0200csuv\u1eea\u1ef3\u1f0b\u1f23\u0100io\u1eef\u1e31rc\xbb\u1e2e\u0269\u1ef9\0\0\u1efb\xed\u0548ant\u0100gl\u1f02\u1f06tr\xbb\u1e5dess\xbb\u1e7a\u0180aei\u1f12\u1f16\u1f1als;\u403dst;\u625fv\u0100;D\u0235\u1f20D;\u6a78parsl;\u69e5\u0100Da\u1f2f\u1f33ot;\u6253rr;\u6971\u0180cdi\u1f3e\u1f41\u1ef8r;\u612fo\xf4\u0352\u0100ah\u1f49\u1f4b;\u43b7\u803b\xf0\u40f0\u0100mr\u1f53\u1f57l\u803b\xeb\u40ebo;\u60ac\u0180cip\u1f61\u1f64\u1f67l;\u4021s\xf4\u056e\u0100eo\u1f6c\u1f74ctatio\xee\u0559nential\xe5\u0579\u09e1\u1f92\0\u1f9e\0\u1fa1\u1fa7\0\0\u1fc6\u1fcc\0\u1fd3\0\u1fe6\u1fea\u2000\0\u2008\u205allingdotse\xf1\u1e44y;\u4444male;\u6640\u0180ilr\u1fad\u1fb3\u1fc1lig;\u8000\ufb03\u0269\u1fb9\0\0\u1fbdg;\u8000\ufb00ig;\u8000\ufb04;\uc000\ud835\udd23lig;\u8000\ufb01lig;\uc000fj\u0180alt\u1fd9\u1fdc\u1fe1t;\u666dig;\u8000\ufb02ns;\u65b1of;\u4192\u01f0\u1fee\0\u1ff3f;\uc000\ud835\udd57\u0100ak\u05bf\u1ff7\u0100;v\u1ffc\u1ffd\u62d4;\u6ad9artint;\u6a0d\u0100ao\u200c\u2055\u0100cs\u2011\u2052\u03b1\u201a\u2030\u2038\u2045\u2048\0\u2050\u03b2\u2022\u2025\u2027\u202a\u202c\0\u202e\u803b\xbd\u40bd;\u6153\u803b\xbc\u40bc;\u6155;\u6159;\u615b\u01b3\u2034\0\u2036;\u6154;\u6156\u02b4\u203e\u2041\0\0\u2043\u803b\xbe\u40be;\u6157;\u615c5;\u6158\u01b6\u204c\0\u204e;\u615a;\u615d8;\u615el;\u6044wn;\u6322cr;\uc000\ud835\udcbb\u0880Eabcdefgijlnorstv\u2082\u2089\u209f\u20a5\u20b0\u20b4\u20f0\u20f5\u20fa\u20ff\u2103\u2112\u2138\u0317\u213e\u2152\u219e\u0100;l\u064d\u2087;\u6a8c\u0180cmp\u2090\u2095\u209dute;\u41f5ma\u0100;d\u209c\u1cda\u43b3;\u6a86reve;\u411f\u0100iy\u20aa\u20aerc;\u411d;\u4433ot;\u4121\u0200;lqs\u063e\u0642\u20bd\u20c9\u0180;qs\u063e\u064c\u20c4lan\xf4\u0665\u0200;cdl\u0665\u20d2\u20d5\u20e5c;\u6aa9ot\u0100;o\u20dc\u20dd\u6a80\u0100;l\u20e2\u20e3\u6a82;\u6a84\u0100;e\u20ea\u20ed\uc000\u22db\ufe00s;\u6a94r;\uc000\ud835\udd24\u0100;g\u0673\u061bmel;\u6137cy;\u4453\u0200;Eaj\u065a\u210c\u210e\u2110;\u6a92;\u6aa5;\u6aa4\u0200Eaes\u211b\u211d\u2129\u2134;\u6269p\u0100;p\u2123\u2124\u6a8arox\xbb\u2124\u0100;q\u212e\u212f\u6a88\u0100;q\u212e\u211bim;\u62e7pf;\uc000\ud835\udd58\u0100ci\u2143\u2146r;\u610am\u0180;el\u066b\u214e\u2150;\u6a8e;\u6a90\u8300>;cdlqr\u05ee\u2160\u216a\u216e\u2173\u2179\u0100ci\u2165\u2167;\u6aa7r;\u6a7aot;\u62d7Par;\u6995uest;\u6a7c\u0280adels\u2184\u216a\u2190\u0656\u219b\u01f0\u2189\0\u218epro\xf8\u209er;\u6978q\u0100lq\u063f\u2196les\xf3\u2088i\xed\u066b\u0100en\u21a3\u21adrtneqq;\uc000\u2269\ufe00\xc5\u21aa\u0500Aabcefkosy\u21c4\u21c7\u21f1\u21f5\u21fa\u2218\u221d\u222f\u2268\u227dr\xf2\u03a0\u0200ilmr\u21d0\u21d4\u21d7\u21dbrs\xf0\u1484f\xbb\u2024il\xf4\u06a9\u0100dr\u21e0\u21e4cy;\u444a\u0180;cw\u08f4\u21eb\u21efir;\u6948;\u61adar;\u610firc;\u4125\u0180alr\u2201\u220e\u2213rts\u0100;u\u2209\u220a\u6665it\xbb\u220alip;\u6026con;\u62b9r;\uc000\ud835\udd25s\u0100ew\u2223\u2229arow;\u6925arow;\u6926\u0280amopr\u223a\u223e\u2243\u225e\u2263rr;\u61fftht;\u623bk\u0100lr\u2249\u2253eftarrow;\u61a9ightarrow;\u61aaf;\uc000\ud835\udd59bar;\u6015\u0180clt\u226f\u2274\u2278r;\uc000\ud835\udcbdas\xe8\u21f4rok;\u4127\u0100bp\u2282\u2287ull;\u6043hen\xbb\u1c5b\u0ae1\u22a3\0\u22aa\0\u22b8\u22c5\u22ce\0\u22d5\u22f3\0\0\u22f8\u2322\u2367\u2362\u237f\0\u2386\u23aa\u23b4cute\u803b\xed\u40ed\u0180;iy\u0771\u22b0\u22b5rc\u803b\xee\u40ee;\u4438\u0100cx\u22bc\u22bfy;\u4435cl\u803b\xa1\u40a1\u0100fr\u039f\u22c9;\uc000\ud835\udd26rave\u803b\xec\u40ec\u0200;ino\u073e\u22dd\u22e9\u22ee\u0100in\u22e2\u22e6nt;\u6a0ct;\u622dfin;\u69dcta;\u6129lig;\u4133\u0180aop\u22fe\u231a\u231d\u0180cgt\u2305\u2308\u2317r;\u412b\u0180elp\u071f\u230f\u2313in\xe5\u078ear\xf4\u0720h;\u4131f;\u62b7ed;\u41b5\u0280;cfot\u04f4\u232c\u2331\u233d\u2341are;\u6105in\u0100;t\u2338\u2339\u621eie;\u69dddo\xf4\u2319\u0280;celp\u0757\u234c\u2350\u235b\u2361al;\u62ba\u0100gr\u2355\u2359er\xf3\u1563\xe3\u234darhk;\u6a17rod;\u6a3c\u0200cgpt\u236f\u2372\u2376\u237by;\u4451on;\u412ff;\uc000\ud835\udd5aa;\u43b9uest\u803b\xbf\u40bf\u0100ci\u238a\u238fr;\uc000\ud835\udcben\u0280;Edsv\u04f4\u239b\u239d\u23a1\u04f3;\u62f9ot;\u62f5\u0100;v\u23a6\u23a7\u62f4;\u62f3\u0100;i\u0777\u23aelde;\u4129\u01eb\u23b8\0\u23bccy;\u4456l\u803b\xef\u40ef\u0300cfmosu\u23cc\u23d7\u23dc\u23e1\u23e7\u23f5\u0100iy\u23d1\u23d5rc;\u4135;\u4439r;\uc000\ud835\udd27ath;\u4237pf;\uc000\ud835\udd5b\u01e3\u23ec\0\u23f1r;\uc000\ud835\udcbfrcy;\u4458kcy;\u4454\u0400acfghjos\u240b\u2416\u2422\u2427\u242d\u2431\u2435\u243bppa\u0100;v\u2413\u2414\u43ba;\u43f0\u0100ey\u241b\u2420dil;\u4137;\u443ar;\uc000\ud835\udd28reen;\u4138cy;\u4445cy;\u445cpf;\uc000\ud835\udd5ccr;\uc000\ud835\udcc0\u0b80ABEHabcdefghjlmnoprstuv\u2470\u2481\u2486\u248d\u2491\u250e\u253d\u255a\u2580\u264e\u265e\u2665\u2679\u267d\u269a\u26b2\u26d8\u275d\u2768\u278b\u27c0\u2801\u2812\u0180art\u2477\u247a\u247cr\xf2\u09c6\xf2\u0395ail;\u691barr;\u690e\u0100;g\u0994\u248b;\u6a8bar;\u6962\u0963\u24a5\0\u24aa\0\u24b1\0\0\0\0\0\u24b5\u24ba\0\u24c6\u24c8\u24cd\0\u24f9ute;\u413amptyv;\u69b4ra\xee\u084cbda;\u43bbg\u0180;dl\u088e\u24c1\u24c3;\u6991\xe5\u088e;\u6a85uo\u803b\xab\u40abr\u0400;bfhlpst\u0899\u24de\u24e6\u24e9\u24eb\u24ee\u24f1\u24f5\u0100;f\u089d\u24e3s;\u691fs;\u691d\xeb\u2252p;\u61abl;\u6939im;\u6973l;\u61a2\u0180;ae\u24ff\u2500\u2504\u6aabil;\u6919\u0100;s\u2509\u250a\u6aad;\uc000\u2aad\ufe00\u0180abr\u2515\u2519\u251drr;\u690crk;\u6772\u0100ak\u2522\u252cc\u0100ek\u2528\u252a;\u407b;\u405b\u0100es\u2531\u2533;\u698bl\u0100du\u2539\u253b;\u698f;\u698d\u0200aeuy\u2546\u254b\u2556\u2558ron;\u413e\u0100di\u2550\u2554il;\u413c\xec\u08b0\xe2\u2529;\u443b\u0200cqrs\u2563\u2566\u256d\u257da;\u6936uo\u0100;r\u0e19\u1746\u0100du\u2572\u2577har;\u6967shar;\u694bh;\u61b2\u0280;fgqs\u258b\u258c\u0989\u25f3\u25ff\u6264t\u0280ahlrt\u2598\u25a4\u25b7\u25c2\u25e8rrow\u0100;t\u0899\u25a1a\xe9\u24f6arpoon\u0100du\u25af\u25b4own\xbb\u045ap\xbb\u0966eftarrows;\u61c7ight\u0180ahs\u25cd\u25d6\u25derrow\u0100;s\u08f4\u08a7arpoon\xf3\u0f98quigarro\xf7\u21f0hreetimes;\u62cb\u0180;qs\u258b\u0993\u25falan\xf4\u09ac\u0280;cdgs\u09ac\u260a\u260d\u261d\u2628c;\u6aa8ot\u0100;o\u2614\u2615\u6a7f\u0100;r\u261a\u261b\u6a81;\u6a83\u0100;e\u2622\u2625\uc000\u22da\ufe00s;\u6a93\u0280adegs\u2633\u2639\u263d\u2649\u264bppro\xf8\u24c6ot;\u62d6q\u0100gq\u2643\u2645\xf4\u0989gt\xf2\u248c\xf4\u099bi\xed\u09b2\u0180ilr\u2655\u08e1\u265asht;\u697c;\uc000\ud835\udd29\u0100;E\u099c\u2663;\u6a91\u0161\u2669\u2676r\u0100du\u25b2\u266e\u0100;l\u0965\u2673;\u696alk;\u6584cy;\u4459\u0280;acht\u0a48\u2688\u268b\u2691\u2696r\xf2\u25c1orne\xf2\u1d08ard;\u696bri;\u65fa\u0100io\u269f\u26a4dot;\u4140ust\u0100;a\u26ac\u26ad\u63b0che\xbb\u26ad\u0200Eaes\u26bb\u26bd\u26c9\u26d4;\u6268p\u0100;p\u26c3\u26c4\u6a89rox\xbb\u26c4\u0100;q\u26ce\u26cf\u6a87\u0100;q\u26ce\u26bbim;\u62e6\u0400abnoptwz\u26e9\u26f4\u26f7\u271a\u272f\u2741\u2747\u2750\u0100nr\u26ee\u26f1g;\u67ecr;\u61fdr\xeb\u08c1g\u0180lmr\u26ff\u270d\u2714eft\u0100ar\u09e6\u2707ight\xe1\u09f2apsto;\u67fcight\xe1\u09fdparrow\u0100lr\u2725\u2729ef\xf4\u24edight;\u61ac\u0180afl\u2736\u2739\u273dr;\u6985;\uc000\ud835\udd5dus;\u6a2dimes;\u6a34\u0161\u274b\u274fst;\u6217\xe1\u134e\u0180;ef\u2757\u2758\u1800\u65cange\xbb\u2758ar\u0100;l\u2764\u2765\u4028t;\u6993\u0280achmt\u2773\u2776\u277c\u2785\u2787r\xf2\u08a8orne\xf2\u1d8car\u0100;d\u0f98\u2783;\u696d;\u600eri;\u62bf\u0300achiqt\u2798\u279d\u0a40\u27a2\u27ae\u27bbquo;\u6039r;\uc000\ud835\udcc1m\u0180;eg\u09b2\u27aa\u27ac;\u6a8d;\u6a8f\u0100bu\u252a\u27b3o\u0100;r\u0e1f\u27b9;\u601arok;\u4142\u8400<;cdhilqr\u082b\u27d2\u2639\u27dc\u27e0\u27e5\u27ea\u27f0\u0100ci\u27d7\u27d9;\u6aa6r;\u6a79re\xe5\u25f2mes;\u62c9arr;\u6976uest;\u6a7b\u0100Pi\u27f5\u27f9ar;\u6996\u0180;ef\u2800\u092d\u181b\u65c3r\u0100du\u2807\u280dshar;\u694ahar;\u6966\u0100en\u2817\u2821rtneqq;\uc000\u2268\ufe00\xc5\u281e\u0700Dacdefhilnopsu\u2840\u2845\u2882\u288e\u2893\u28a0\u28a5\u28a8\u28da\u28e2\u28e4\u0a83\u28f3\u2902Dot;\u623a\u0200clpr\u284e\u2852\u2863\u287dr\u803b\xaf\u40af\u0100et\u2857\u2859;\u6642\u0100;e\u285e\u285f\u6720se\xbb\u285f\u0100;s\u103b\u2868to\u0200;dlu\u103b\u2873\u2877\u287bow\xee\u048cef\xf4\u090f\xf0\u13d1ker;\u65ae\u0100oy\u2887\u288cmma;\u6a29;\u443cash;\u6014asuredangle\xbb\u1626r;\uc000\ud835\udd2ao;\u6127\u0180cdn\u28af\u28b4\u28c9ro\u803b\xb5\u40b5\u0200;acd\u1464\u28bd\u28c0\u28c4s\xf4\u16a7ir;\u6af0ot\u80bb\xb7\u01b5us\u0180;bd\u28d2\u1903\u28d3\u6212\u0100;u\u1d3c\u28d8;\u6a2a\u0163\u28de\u28e1p;\u6adb\xf2\u2212\xf0\u0a81\u0100dp\u28e9\u28eeels;\u62a7f;\uc000\ud835\udd5e\u0100ct\u28f8\u28fdr;\uc000\ud835\udcc2pos\xbb\u159d\u0180;lm\u2909\u290a\u290d\u43bctimap;\u62b8\u0c00GLRVabcdefghijlmoprstuvw\u2942\u2953\u297e\u2989\u2998\u29da\u29e9\u2a15\u2a1a\u2a58\u2a5d\u2a83\u2a95\u2aa4\u2aa8\u2b04\u2b07\u2b44\u2b7f\u2bae\u2c34\u2c67\u2c7c\u2ce9\u0100gt\u2947\u294b;\uc000\u22d9\u0338\u0100;v\u2950\u0bcf\uc000\u226b\u20d2\u0180elt\u295a\u2972\u2976ft\u0100ar\u2961\u2967rrow;\u61cdightarrow;\u61ce;\uc000\u22d8\u0338\u0100;v\u297b\u0c47\uc000\u226a\u20d2ightarrow;\u61cf\u0100Dd\u298e\u2993ash;\u62afash;\u62ae\u0280bcnpt\u29a3\u29a7\u29ac\u29b1\u29ccla\xbb\u02deute;\u4144g;\uc000\u2220\u20d2\u0280;Eiop\u0d84\u29bc\u29c0\u29c5\u29c8;\uc000\u2a70\u0338d;\uc000\u224b\u0338s;\u4149ro\xf8\u0d84ur\u0100;a\u29d3\u29d4\u666el\u0100;s\u29d3\u0b38\u01f3\u29df\0\u29e3p\u80bb\xa0\u0b37mp\u0100;e\u0bf9\u0c00\u0280aeouy\u29f4\u29fe\u2a03\u2a10\u2a13\u01f0\u29f9\0\u29fb;\u6a43on;\u4148dil;\u4146ng\u0100;d\u0d7e\u2a0aot;\uc000\u2a6d\u0338p;\u6a42;\u443dash;\u6013\u0380;Aadqsx\u0b92\u2a29\u2a2d\u2a3b\u2a41\u2a45\u2a50rr;\u61d7r\u0100hr\u2a33\u2a36k;\u6924\u0100;o\u13f2\u13f0ot;\uc000\u2250\u0338ui\xf6\u0b63\u0100ei\u2a4a\u2a4ear;\u6928\xed\u0b98ist\u0100;s\u0ba0\u0b9fr;\uc000\ud835\udd2b\u0200Eest\u0bc5\u2a66\u2a79\u2a7c\u0180;qs\u0bbc\u2a6d\u0be1\u0180;qs\u0bbc\u0bc5\u2a74lan\xf4\u0be2i\xed\u0bea\u0100;r\u0bb6\u2a81\xbb\u0bb7\u0180Aap\u2a8a\u2a8d\u2a91r\xf2\u2971rr;\u61aear;\u6af2\u0180;sv\u0f8d\u2a9c\u0f8c\u0100;d\u2aa1\u2aa2\u62fc;\u62facy;\u445a\u0380AEadest\u2ab7\u2aba\u2abe\u2ac2\u2ac5\u2af6\u2af9r\xf2\u2966;\uc000\u2266\u0338rr;\u619ar;\u6025\u0200;fqs\u0c3b\u2ace\u2ae3\u2aeft\u0100ar\u2ad4\u2ad9rro\xf7\u2ac1ightarro\xf7\u2a90\u0180;qs\u0c3b\u2aba\u2aealan\xf4\u0c55\u0100;s\u0c55\u2af4\xbb\u0c36i\xed\u0c5d\u0100;r\u0c35\u2afei\u0100;e\u0c1a\u0c25i\xe4\u0d90\u0100pt\u2b0c\u2b11f;\uc000\ud835\udd5f\u8180\xac;in\u2b19\u2b1a\u2b36\u40acn\u0200;Edv\u0b89\u2b24\u2b28\u2b2e;\uc000\u22f9\u0338ot;\uc000\u22f5\u0338\u01e1\u0b89\u2b33\u2b35;\u62f7;\u62f6i\u0100;v\u0cb8\u2b3c\u01e1\u0cb8\u2b41\u2b43;\u62fe;\u62fd\u0180aor\u2b4b\u2b63\u2b69r\u0200;ast\u0b7b\u2b55\u2b5a\u2b5flle\xec\u0b7bl;\uc000\u2afd\u20e5;\uc000\u2202\u0338lint;\u6a14\u0180;ce\u0c92\u2b70\u2b73u\xe5\u0ca5\u0100;c\u0c98\u2b78\u0100;e\u0c92\u2b7d\xf1\u0c98\u0200Aait\u2b88\u2b8b\u2b9d\u2ba7r\xf2\u2988rr\u0180;cw\u2b94\u2b95\u2b99\u619b;\uc000\u2933\u0338;\uc000\u219d\u0338ghtarrow\xbb\u2b95ri\u0100;e\u0ccb\u0cd6\u0380chimpqu\u2bbd\u2bcd\u2bd9\u2b04\u0b78\u2be4\u2bef\u0200;cer\u0d32\u2bc6\u0d37\u2bc9u\xe5\u0d45;\uc000\ud835\udcc3ort\u026d\u2b05\0\0\u2bd6ar\xe1\u2b56m\u0100;e\u0d6e\u2bdf\u0100;q\u0d74\u0d73su\u0100bp\u2beb\u2bed\xe5\u0cf8\xe5\u0d0b\u0180bcp\u2bf6\u2c11\u2c19\u0200;Ees\u2bff\u2c00\u0d22\u2c04\u6284;\uc000\u2ac5\u0338et\u0100;e\u0d1b\u2c0bq\u0100;q\u0d23\u2c00c\u0100;e\u0d32\u2c17\xf1\u0d38\u0200;Ees\u2c22\u2c23\u0d5f\u2c27\u6285;\uc000\u2ac6\u0338et\u0100;e\u0d58\u2c2eq\u0100;q\u0d60\u2c23\u0200gilr\u2c3d\u2c3f\u2c45\u2c47\xec\u0bd7lde\u803b\xf1\u40f1\xe7\u0c43iangle\u0100lr\u2c52\u2c5ceft\u0100;e\u0c1a\u2c5a\xf1\u0c26ight\u0100;e\u0ccb\u2c65\xf1\u0cd7\u0100;m\u2c6c\u2c6d\u43bd\u0180;es\u2c74\u2c75\u2c79\u4023ro;\u6116p;\u6007\u0480DHadgilrs\u2c8f\u2c94\u2c99\u2c9e\u2ca3\u2cb0\u2cb6\u2cd3\u2ce3ash;\u62adarr;\u6904p;\uc000\u224d\u20d2ash;\u62ac\u0100et\u2ca8\u2cac;\uc000\u2265\u20d2;\uc000>\u20d2nfin;\u69de\u0180Aet\u2cbd\u2cc1\u2cc5rr;\u6902;\uc000\u2264\u20d2\u0100;r\u2cca\u2ccd\uc000<\u20d2ie;\uc000\u22b4\u20d2\u0100At\u2cd8\u2cdcrr;\u6903rie;\uc000\u22b5\u20d2im;\uc000\u223c\u20d2\u0180Aan\u2cf0\u2cf4\u2d02rr;\u61d6r\u0100hr\u2cfa\u2cfdk;\u6923\u0100;o\u13e7\u13e5ear;\u6927\u1253\u1a95\0\0\0\0\0\0\0\0\0\0\0\0\0\u2d2d\0\u2d38\u2d48\u2d60\u2d65\u2d72\u2d84\u1b07\0\0\u2d8d\u2dab\0\u2dc8\u2dce\0\u2ddc\u2e19\u2e2b\u2e3e\u2e43\u0100cs\u2d31\u1a97ute\u803b\xf3\u40f3\u0100iy\u2d3c\u2d45r\u0100;c\u1a9e\u2d42\u803b\xf4\u40f4;\u443e\u0280abios\u1aa0\u2d52\u2d57\u01c8\u2d5alac;\u4151v;\u6a38old;\u69bclig;\u4153\u0100cr\u2d69\u2d6dir;\u69bf;\uc000\ud835\udd2c\u036f\u2d79\0\0\u2d7c\0\u2d82n;\u42dbave\u803b\xf2\u40f2;\u69c1\u0100bm\u2d88\u0df4ar;\u69b5\u0200acit\u2d95\u2d98\u2da5\u2da8r\xf2\u1a80\u0100ir\u2d9d\u2da0r;\u69beoss;\u69bbn\xe5\u0e52;\u69c0\u0180aei\u2db1\u2db5\u2db9cr;\u414dga;\u43c9\u0180cdn\u2dc0\u2dc5\u01cdron;\u43bf;\u69b6pf;\uc000\ud835\udd60\u0180ael\u2dd4\u2dd7\u01d2r;\u69b7rp;\u69b9\u0380;adiosv\u2dea\u2deb\u2dee\u2e08\u2e0d\u2e10\u2e16\u6228r\xf2\u1a86\u0200;efm\u2df7\u2df8\u2e02\u2e05\u6a5dr\u0100;o\u2dfe\u2dff\u6134f\xbb\u2dff\u803b\xaa\u40aa\u803b\xba\u40bagof;\u62b6r;\u6a56lope;\u6a57;\u6a5b\u0180clo\u2e1f\u2e21\u2e27\xf2\u2e01ash\u803b\xf8\u40f8l;\u6298i\u016c\u2e2f\u2e34de\u803b\xf5\u40f5es\u0100;a\u01db\u2e3as;\u6a36ml\u803b\xf6\u40f6bar;\u633d\u0ae1\u2e5e\0\u2e7d\0\u2e80\u2e9d\0\u2ea2\u2eb9\0\0\u2ecb\u0e9c\0\u2f13\0\0\u2f2b\u2fbc\0\u2fc8r\u0200;ast\u0403\u2e67\u2e72\u0e85\u8100\xb6;l\u2e6d\u2e6e\u40b6le\xec\u0403\u0269\u2e78\0\0\u2e7bm;\u6af3;\u6afdy;\u443fr\u0280cimpt\u2e8b\u2e8f\u2e93\u1865\u2e97nt;\u4025od;\u402eil;\u6030enk;\u6031r;\uc000\ud835\udd2d\u0180imo\u2ea8\u2eb0\u2eb4\u0100;v\u2ead\u2eae\u43c6;\u43d5ma\xf4\u0a76ne;\u660e\u0180;tv\u2ebf\u2ec0\u2ec8\u43c0chfork\xbb\u1ffd;\u43d6\u0100au\u2ecf\u2edfn\u0100ck\u2ed5\u2eddk\u0100;h\u21f4\u2edb;\u610e\xf6\u21f4s\u0480;abcdemst\u2ef3\u2ef4\u1908\u2ef9\u2efd\u2f04\u2f06\u2f0a\u2f0e\u402bcir;\u6a23ir;\u6a22\u0100ou\u1d40\u2f02;\u6a25;\u6a72n\u80bb\xb1\u0e9dim;\u6a26wo;\u6a27\u0180ipu\u2f19\u2f20\u2f25ntint;\u6a15f;\uc000\ud835\udd61nd\u803b\xa3\u40a3\u0500;Eaceinosu\u0ec8\u2f3f\u2f41\u2f44\u2f47\u2f81\u2f89\u2f92\u2f7e\u2fb6;\u6ab3p;\u6ab7u\xe5\u0ed9\u0100;c\u0ece\u2f4c\u0300;acens\u0ec8\u2f59\u2f5f\u2f66\u2f68\u2f7eppro\xf8\u2f43urlye\xf1\u0ed9\xf1\u0ece\u0180aes\u2f6f\u2f76\u2f7approx;\u6ab9qq;\u6ab5im;\u62e8i\xed\u0edfme\u0100;s\u2f88\u0eae\u6032\u0180Eas\u2f78\u2f90\u2f7a\xf0\u2f75\u0180dfp\u0eec\u2f99\u2faf\u0180als\u2fa0\u2fa5\u2faalar;\u632eine;\u6312urf;\u6313\u0100;t\u0efb\u2fb4\xef\u0efbrel;\u62b0\u0100ci\u2fc0\u2fc5r;\uc000\ud835\udcc5;\u43c8ncsp;\u6008\u0300fiopsu\u2fda\u22e2\u2fdf\u2fe5\u2feb\u2ff1r;\uc000\ud835\udd2epf;\uc000\ud835\udd62rime;\u6057cr;\uc000\ud835\udcc6\u0180aeo\u2ff8\u3009\u3013t\u0100ei\u2ffe\u3005rnion\xf3\u06b0nt;\u6a16st\u0100;e\u3010\u3011\u403f\xf1\u1f19\xf4\u0f14\u0a80ABHabcdefhilmnoprstux\u3040\u3051\u3055\u3059\u30e0\u310e\u312b\u3147\u3162\u3172\u318e\u3206\u3215\u3224\u3229\u3258\u326e\u3272\u3290\u32b0\u32b7\u0180art\u3047\u304a\u304cr\xf2\u10b3\xf2\u03ddail;\u691car\xf2\u1c65ar;\u6964\u0380cdenqrt\u3068\u3075\u3078\u307f\u308f\u3094\u30cc\u0100eu\u306d\u3071;\uc000\u223d\u0331te;\u4155i\xe3\u116emptyv;\u69b3g\u0200;del\u0fd1\u3089\u308b\u308d;\u6992;\u69a5\xe5\u0fd1uo\u803b\xbb\u40bbr\u0580;abcfhlpstw\u0fdc\u30ac\u30af\u30b7\u30b9\u30bc\u30be\u30c0\u30c3\u30c7\u30cap;\u6975\u0100;f\u0fe0\u30b4s;\u6920;\u6933s;\u691e\xeb\u225d\xf0\u272el;\u6945im;\u6974l;\u61a3;\u619d\u0100ai\u30d1\u30d5il;\u691ao\u0100;n\u30db\u30dc\u6236al\xf3\u0f1e\u0180abr\u30e7\u30ea\u30eer\xf2\u17e5rk;\u6773\u0100ak\u30f3\u30fdc\u0100ek\u30f9\u30fb;\u407d;\u405d\u0100es\u3102\u3104;\u698cl\u0100du\u310a\u310c;\u698e;\u6990\u0200aeuy\u3117\u311c\u3127\u3129ron;\u4159\u0100di\u3121\u3125il;\u4157\xec\u0ff2\xe2\u30fa;\u4440\u0200clqs\u3134\u3137\u313d\u3144a;\u6937dhar;\u6969uo\u0100;r\u020e\u020dh;\u61b3\u0180acg\u314e\u315f\u0f44l\u0200;ips\u0f78\u3158\u315b\u109cn\xe5\u10bbar\xf4\u0fa9t;\u65ad\u0180ilr\u3169\u1023\u316esht;\u697d;\uc000\ud835\udd2f\u0100ao\u3177\u3186r\u0100du\u317d\u317f\xbb\u047b\u0100;l\u1091\u3184;\u696c\u0100;v\u318b\u318c\u43c1;\u43f1\u0180gns\u3195\u31f9\u31fcht\u0300ahlrst\u31a4\u31b0\u31c2\u31d8\u31e4\u31eerrow\u0100;t\u0fdc\u31ada\xe9\u30c8arpoon\u0100du\u31bb\u31bfow\xee\u317ep\xbb\u1092eft\u0100ah\u31ca\u31d0rrow\xf3\u0feaarpoon\xf3\u0551ightarrows;\u61c9quigarro\xf7\u30cbhreetimes;\u62ccg;\u42daingdotse\xf1\u1f32\u0180ahm\u320d\u3210\u3213r\xf2\u0feaa\xf2\u0551;\u600foust\u0100;a\u321e\u321f\u63b1che\xbb\u321fmid;\u6aee\u0200abpt\u3232\u323d\u3240\u3252\u0100nr\u3237\u323ag;\u67edr;\u61fer\xeb\u1003\u0180afl\u3247\u324a\u324er;\u6986;\uc000\ud835\udd63us;\u6a2eimes;\u6a35\u0100ap\u325d\u3267r\u0100;g\u3263\u3264\u4029t;\u6994olint;\u6a12ar\xf2\u31e3\u0200achq\u327b\u3280\u10bc\u3285quo;\u603ar;\uc000\ud835\udcc7\u0100bu\u30fb\u328ao\u0100;r\u0214\u0213\u0180hir\u3297\u329b\u32a0re\xe5\u31f8mes;\u62cai\u0200;efl\u32aa\u1059\u1821\u32ab\u65b9tri;\u69celuhar;\u6968;\u611e\u0d61\u32d5\u32db\u32df\u332c\u3338\u3371\0\u337a\u33a4\0\0\u33ec\u33f0\0\u3428\u3448\u345a\u34ad\u34b1\u34ca\u34f1\0\u3616\0\0\u3633cute;\u415bqu\xef\u27ba\u0500;Eaceinpsy\u11ed\u32f3\u32f5\u32ff\u3302\u330b\u330f\u331f\u3326\u3329;\u6ab4\u01f0\u32fa\0\u32fc;\u6ab8on;\u4161u\xe5\u11fe\u0100;d\u11f3\u3307il;\u415frc;\u415d\u0180Eas\u3316\u3318\u331b;\u6ab6p;\u6abaim;\u62e9olint;\u6a13i\xed\u1204;\u4441ot\u0180;be\u3334\u1d47\u3335\u62c5;\u6a66\u0380Aacmstx\u3346\u334a\u3357\u335b\u335e\u3363\u336drr;\u61d8r\u0100hr\u3350\u3352\xeb\u2228\u0100;o\u0a36\u0a34t\u803b\xa7\u40a7i;\u403bwar;\u6929m\u0100in\u3369\xf0nu\xf3\xf1t;\u6736r\u0100;o\u3376\u2055\uc000\ud835\udd30\u0200acoy\u3382\u3386\u3391\u33a0rp;\u666f\u0100hy\u338b\u338fcy;\u4449;\u4448rt\u026d\u3399\0\0\u339ci\xe4\u1464ara\xec\u2e6f\u803b\xad\u40ad\u0100gm\u33a8\u33b4ma\u0180;fv\u33b1\u33b2\u33b2\u43c3;\u43c2\u0400;deglnpr\u12ab\u33c5\u33c9\u33ce\u33d6\u33de\u33e1\u33e6ot;\u6a6a\u0100;q\u12b1\u12b0\u0100;E\u33d3\u33d4\u6a9e;\u6aa0\u0100;E\u33db\u33dc\u6a9d;\u6a9fe;\u6246lus;\u6a24arr;\u6972ar\xf2\u113d\u0200aeit\u33f8\u3408\u340f\u3417\u0100ls\u33fd\u3404lsetm\xe9\u336ahp;\u6a33parsl;\u69e4\u0100dl\u1463\u3414e;\u6323\u0100;e\u341c\u341d\u6aaa\u0100;s\u3422\u3423\u6aac;\uc000\u2aac\ufe00\u0180flp\u342e\u3433\u3442tcy;\u444c\u0100;b\u3438\u3439\u402f\u0100;a\u343e\u343f\u69c4r;\u633ff;\uc000\ud835\udd64a\u0100dr\u344d\u0402es\u0100;u\u3454\u3455\u6660it\xbb\u3455\u0180csu\u3460\u3479\u349f\u0100au\u3465\u346fp\u0100;s\u1188\u346b;\uc000\u2293\ufe00p\u0100;s\u11b4\u3475;\uc000\u2294\ufe00u\u0100bp\u347f\u348f\u0180;es\u1197\u119c\u3486et\u0100;e\u1197\u348d\xf1\u119d\u0180;es\u11a8\u11ad\u3496et\u0100;e\u11a8\u349d\xf1\u11ae\u0180;af\u117b\u34a6\u05b0r\u0165\u34ab\u05b1\xbb\u117car\xf2\u1148\u0200cemt\u34b9\u34be\u34c2\u34c5r;\uc000\ud835\udcc8tm\xee\xf1i\xec\u3415ar\xe6\u11be\u0100ar\u34ce\u34d5r\u0100;f\u34d4\u17bf\u6606\u0100an\u34da\u34edight\u0100ep\u34e3\u34eapsilo\xee\u1ee0h\xe9\u2eafs\xbb\u2852\u0280bcmnp\u34fb\u355e\u1209\u358b\u358e\u0480;Edemnprs\u350e\u350f\u3511\u3515\u351e\u3523\u352c\u3531\u3536\u6282;\u6ac5ot;\u6abd\u0100;d\u11da\u351aot;\u6ac3ult;\u6ac1\u0100Ee\u3528\u352a;\u6acb;\u628alus;\u6abfarr;\u6979\u0180eiu\u353d\u3552\u3555t\u0180;en\u350e\u3545\u354bq\u0100;q\u11da\u350feq\u0100;q\u352b\u3528m;\u6ac7\u0100bp\u355a\u355c;\u6ad5;\u6ad3c\u0300;acens\u11ed\u356c\u3572\u3579\u357b\u3326ppro\xf8\u32faurlye\xf1\u11fe\xf1\u11f3\u0180aes\u3582\u3588\u331bppro\xf8\u331aq\xf1\u3317g;\u666a\u0680123;Edehlmnps\u35a9\u35ac\u35af\u121c\u35b2\u35b4\u35c0\u35c9\u35d5\u35da\u35df\u35e8\u35ed\u803b\xb9\u40b9\u803b\xb2\u40b2\u803b\xb3\u40b3;\u6ac6\u0100os\u35b9\u35bct;\u6abeub;\u6ad8\u0100;d\u1222\u35c5ot;\u6ac4s\u0100ou\u35cf\u35d2l;\u67c9b;\u6ad7arr;\u697bult;\u6ac2\u0100Ee\u35e4\u35e6;\u6acc;\u628blus;\u6ac0\u0180eiu\u35f4\u3609\u360ct\u0180;en\u121c\u35fc\u3602q\u0100;q\u1222\u35b2eq\u0100;q\u35e7\u35e4m;\u6ac8\u0100bp\u3611\u3613;\u6ad4;\u6ad6\u0180Aan\u361c\u3620\u362drr;\u61d9r\u0100hr\u3626\u3628\xeb\u222e\u0100;o\u0a2b\u0a29war;\u692alig\u803b\xdf\u40df\u0be1\u3651\u365d\u3660\u12ce\u3673\u3679\0\u367e\u36c2\0\0\0\0\0\u36db\u3703\0\u3709\u376c\0\0\0\u3787\u0272\u3656\0\0\u365bget;\u6316;\u43c4r\xeb\u0e5f\u0180aey\u3666\u366b\u3670ron;\u4165dil;\u4163;\u4442lrec;\u6315r;\uc000\ud835\udd31\u0200eiko\u3686\u369d\u36b5\u36bc\u01f2\u368b\0\u3691e\u01004f\u1284\u1281a\u0180;sv\u3698\u3699\u369b\u43b8ym;\u43d1\u0100cn\u36a2\u36b2k\u0100as\u36a8\u36aeppro\xf8\u12c1im\xbb\u12acs\xf0\u129e\u0100as\u36ba\u36ae\xf0\u12c1rn\u803b\xfe\u40fe\u01ec\u031f\u36c6\u22e7es\u8180\xd7;bd\u36cf\u36d0\u36d8\u40d7\u0100;a\u190f\u36d5r;\u6a31;\u6a30\u0180eps\u36e1\u36e3\u3700\xe1\u2a4d\u0200;bcf\u0486\u36ec\u36f0\u36f4ot;\u6336ir;\u6af1\u0100;o\u36f9\u36fc\uc000\ud835\udd65rk;\u6ada\xe1\u3362rime;\u6034\u0180aip\u370f\u3712\u3764d\xe5\u1248\u0380adempst\u3721\u374d\u3740\u3751\u3757\u375c\u375fngle\u0280;dlqr\u3730\u3731\u3736\u3740\u3742\u65b5own\xbb\u1dbbeft\u0100;e\u2800\u373e\xf1\u092e;\u625cight\u0100;e\u32aa\u374b\xf1\u105aot;\u65ecinus;\u6a3alus;\u6a39b;\u69cdime;\u6a3bezium;\u63e2\u0180cht\u3772\u377d\u3781\u0100ry\u3777\u377b;\uc000\ud835\udcc9;\u4446cy;\u445brok;\u4167\u0100io\u378b\u378ex\xf4\u1777head\u0100lr\u3797\u37a0eftarro\xf7\u084fightarrow\xbb\u0f5d\u0900AHabcdfghlmoprstuw\u37d0\u37d3\u37d7\u37e4\u37f0\u37fc\u380e\u381c\u3823\u3834\u3851\u385d\u386b\u38a9\u38cc\u38d2\u38ea\u38f6r\xf2\u03edar;\u6963\u0100cr\u37dc\u37e2ute\u803b\xfa\u40fa\xf2\u1150r\u01e3\u37ea\0\u37edy;\u445eve;\u416d\u0100iy\u37f5\u37farc\u803b\xfb\u40fb;\u4443\u0180abh\u3803\u3806\u380br\xf2\u13adlac;\u4171a\xf2\u13c3\u0100ir\u3813\u3818sht;\u697e;\uc000\ud835\udd32rave\u803b\xf9\u40f9\u0161\u3827\u3831r\u0100lr\u382c\u382e\xbb\u0957\xbb\u1083lk;\u6580\u0100ct\u3839\u384d\u026f\u383f\0\0\u384arn\u0100;e\u3845\u3846\u631cr\xbb\u3846op;\u630fri;\u65f8\u0100al\u3856\u385acr;\u416b\u80bb\xa8\u0349\u0100gp\u3862\u3866on;\u4173f;\uc000\ud835\udd66\u0300adhlsu\u114b\u3878\u387d\u1372\u3891\u38a0own\xe1\u13b3arpoon\u0100lr\u3888\u388cef\xf4\u382digh\xf4\u382fi\u0180;hl\u3899\u389a\u389c\u43c5\xbb\u13faon\xbb\u389aparrows;\u61c8\u0180cit\u38b0\u38c4\u38c8\u026f\u38b6\0\0\u38c1rn\u0100;e\u38bc\u38bd\u631dr\xbb\u38bdop;\u630eng;\u416fri;\u65f9cr;\uc000\ud835\udcca\u0180dir\u38d9\u38dd\u38e2ot;\u62f0lde;\u4169i\u0100;f\u3730\u38e8\xbb\u1813\u0100am\u38ef\u38f2r\xf2\u38a8l\u803b\xfc\u40fcangle;\u69a7\u0780ABDacdeflnoprsz\u391c\u391f\u3929\u392d\u39b5\u39b8\u39bd\u39df\u39e4\u39e8\u39f3\u39f9\u39fd\u3a01\u3a20r\xf2\u03f7ar\u0100;v\u3926\u3927\u6ae8;\u6ae9as\xe8\u03e1\u0100nr\u3932\u3937grt;\u699c\u0380eknprst\u34e3\u3946\u394b\u3952\u395d\u3964\u3996app\xe1\u2415othin\xe7\u1e96\u0180hir\u34eb\u2ec8\u3959op\xf4\u2fb5\u0100;h\u13b7\u3962\xef\u318d\u0100iu\u3969\u396dgm\xe1\u33b3\u0100bp\u3972\u3984setneq\u0100;q\u397d\u3980\uc000\u228a\ufe00;\uc000\u2acb\ufe00setneq\u0100;q\u398f\u3992\uc000\u228b\ufe00;\uc000\u2acc\ufe00\u0100hr\u399b\u399fet\xe1\u369ciangle\u0100lr\u39aa\u39afeft\xbb\u0925ight\xbb\u1051y;\u4432ash\xbb\u1036\u0180elr\u39c4\u39d2\u39d7\u0180;be\u2dea\u39cb\u39cfar;\u62bbq;\u625alip;\u62ee\u0100bt\u39dc\u1468a\xf2\u1469r;\uc000\ud835\udd33tr\xe9\u39aesu\u0100bp\u39ef\u39f1\xbb\u0d1c\xbb\u0d59pf;\uc000\ud835\udd67ro\xf0\u0efbtr\xe9\u39b4\u0100cu\u3a06\u3a0br;\uc000\ud835\udccb\u0100bp\u3a10\u3a18n\u0100Ee\u3980\u3a16\xbb\u397en\u0100Ee\u3992\u3a1e\xbb\u3990igzag;\u699a\u0380cefoprs\u3a36\u3a3b\u3a56\u3a5b\u3a54\u3a61\u3a6airc;\u4175\u0100di\u3a40\u3a51\u0100bg\u3a45\u3a49ar;\u6a5fe\u0100;q\u15fa\u3a4f;\u6259erp;\u6118r;\uc000\ud835\udd34pf;\uc000\ud835\udd68\u0100;e\u1479\u3a66at\xe8\u1479cr;\uc000\ud835\udccc\u0ae3\u178e\u3a87\0\u3a8b\0\u3a90\u3a9b\0\0\u3a9d\u3aa8\u3aab\u3aaf\0\0\u3ac3\u3ace\0\u3ad8\u17dc\u17dftr\xe9\u17d1r;\uc000\ud835\udd35\u0100Aa\u3a94\u3a97r\xf2\u03c3r\xf2\u09f6;\u43be\u0100Aa\u3aa1\u3aa4r\xf2\u03b8r\xf2\u09eba\xf0\u2713is;\u62fb\u0180dpt\u17a4\u3ab5\u3abe\u0100fl\u3aba\u17a9;\uc000\ud835\udd69im\xe5\u17b2\u0100Aa\u3ac7\u3acar\xf2\u03cer\xf2\u0a01\u0100cq\u3ad2\u17b8r;\uc000\ud835\udccd\u0100pt\u17d6\u3adcr\xe9\u17d4\u0400acefiosu\u3af0\u3afd\u3b08\u3b0c\u3b11\u3b15\u3b1b\u3b21c\u0100uy\u3af6\u3afbte\u803b\xfd\u40fd;\u444f\u0100iy\u3b02\u3b06rc;\u4177;\u444bn\u803b\xa5\u40a5r;\uc000\ud835\udd36cy;\u4457pf;\uc000\ud835\udd6acr;\uc000\ud835\udcce\u0100cm\u3b26\u3b29y;\u444el\u803b\xff\u40ff\u0500acdefhiosw\u3b42\u3b48\u3b54\u3b58\u3b64\u3b69\u3b6d\u3b74\u3b7a\u3b80cute;\u417a\u0100ay\u3b4d\u3b52ron;\u417e;\u4437ot;\u417c\u0100et\u3b5d\u3b61tr\xe6\u155fa;\u43b6r;\uc000\ud835\udd37cy;\u4436grarr;\u61ddpf;\uc000\ud835\udd6bcr;\uc000\ud835\udccf\u0100jn\u3b85\u3b87;\u600dj;\u600c"
    .split("")
    .map(function (c) { return c.charCodeAt(0); }));

var decodeDataXml = {};

// Generated using scripts/write-decode-map.ts
Object.defineProperty(decodeDataXml, "__esModule", { value: true });
decodeDataXml.default = new Uint16Array(
// prettier-ignore
"\u0200aglq\t\x15\x18\x1b\u026d\x0f\0\0\x12p;\u4026os;\u4027t;\u403et;\u403cuot;\u4022"
    .split("")
    .map(function (c) { return c.charCodeAt(0); }));

var decode_codepoint = {};

(function (exports) {
// Adapted from https://github.com/mathiasbynens/he/blob/36afe179392226cf1b6ccdb16ebbb7a5a844d93a/src/he.js#L106-L134
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.replaceCodePoint = exports.fromCodePoint = void 0;
var decodeMap = new Map([
    [0, 65533],
    // C1 Unicode control character reference replacements
    [128, 8364],
    [130, 8218],
    [131, 402],
    [132, 8222],
    [133, 8230],
    [134, 8224],
    [135, 8225],
    [136, 710],
    [137, 8240],
    [138, 352],
    [139, 8249],
    [140, 338],
    [142, 381],
    [145, 8216],
    [146, 8217],
    [147, 8220],
    [148, 8221],
    [149, 8226],
    [150, 8211],
    [151, 8212],
    [152, 732],
    [153, 8482],
    [154, 353],
    [155, 8250],
    [156, 339],
    [158, 382],
    [159, 376],
]);
/**
 * Polyfill for `String.fromCodePoint`. It is used to create a string from a Unicode code point.
 */
exports.fromCodePoint = 
// eslint-disable-next-line @typescript-eslint/no-unnecessary-condition, node/no-unsupported-features/es-builtins
(_a = String.fromCodePoint) !== null && _a !== void 0 ? _a : function (codePoint) {
    var output = "";
    if (codePoint > 0xffff) {
        codePoint -= 0x10000;
        output += String.fromCharCode(((codePoint >>> 10) & 0x3ff) | 0xd800);
        codePoint = 0xdc00 | (codePoint & 0x3ff);
    }
    output += String.fromCharCode(codePoint);
    return output;
};
/**
 * Replace the given code point with a replacement character if it is a
 * surrogate or is outside the valid range. Otherwise return the code
 * point unchanged.
 */
function replaceCodePoint(codePoint) {
    var _a;
    if ((codePoint >= 0xd800 && codePoint <= 0xdfff) || codePoint > 0x10ffff) {
        return 0xfffd;
    }
    return (_a = decodeMap.get(codePoint)) !== null && _a !== void 0 ? _a : codePoint;
}
exports.replaceCodePoint = replaceCodePoint;
/**
 * Replace the code point if relevant, then convert it to a string.
 *
 * @deprecated Use `fromCodePoint(replaceCodePoint(codePoint))` instead.
 * @param codePoint The code point to decode.
 * @returns The decoded code point.
 */
function decodeCodePoint(codePoint) {
    return (0, exports.fromCodePoint)(replaceCodePoint(codePoint));
}
exports.default = decodeCodePoint;

}(decode_codepoint));

(function (exports) {
var __createBinding = (commonjsGlobal && commonjsGlobal.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (commonjsGlobal && commonjsGlobal.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (commonjsGlobal && commonjsGlobal.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (commonjsGlobal && commonjsGlobal.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.decodeXML = exports.decodeHTMLStrict = exports.decodeHTMLAttribute = exports.decodeHTML = exports.determineBranch = exports.EntityDecoder = exports.DecodingMode = exports.BinTrieFlags = exports.fromCodePoint = exports.replaceCodePoint = exports.decodeCodePoint = exports.xmlDecodeTree = exports.htmlDecodeTree = void 0;
var decode_data_html_js_1 = __importDefault(decodeDataHtml);
exports.htmlDecodeTree = decode_data_html_js_1.default;
var decode_data_xml_js_1 = __importDefault(decodeDataXml);
exports.xmlDecodeTree = decode_data_xml_js_1.default;
var decode_codepoint_js_1 = __importStar(decode_codepoint);
exports.decodeCodePoint = decode_codepoint_js_1.default;
var decode_codepoint_js_2 = decode_codepoint;
Object.defineProperty(exports, "replaceCodePoint", { enumerable: true, get: function () { return decode_codepoint_js_2.replaceCodePoint; } });
Object.defineProperty(exports, "fromCodePoint", { enumerable: true, get: function () { return decode_codepoint_js_2.fromCodePoint; } });
var CharCodes;
(function (CharCodes) {
    CharCodes[CharCodes["NUM"] = 35] = "NUM";
    CharCodes[CharCodes["SEMI"] = 59] = "SEMI";
    CharCodes[CharCodes["EQUALS"] = 61] = "EQUALS";
    CharCodes[CharCodes["ZERO"] = 48] = "ZERO";
    CharCodes[CharCodes["NINE"] = 57] = "NINE";
    CharCodes[CharCodes["LOWER_A"] = 97] = "LOWER_A";
    CharCodes[CharCodes["LOWER_F"] = 102] = "LOWER_F";
    CharCodes[CharCodes["LOWER_X"] = 120] = "LOWER_X";
    CharCodes[CharCodes["LOWER_Z"] = 122] = "LOWER_Z";
    CharCodes[CharCodes["UPPER_A"] = 65] = "UPPER_A";
    CharCodes[CharCodes["UPPER_F"] = 70] = "UPPER_F";
    CharCodes[CharCodes["UPPER_Z"] = 90] = "UPPER_Z";
})(CharCodes || (CharCodes = {}));
/** Bit that needs to be set to convert an upper case ASCII character to lower case */
var TO_LOWER_BIT = 32;
var BinTrieFlags;
(function (BinTrieFlags) {
    BinTrieFlags[BinTrieFlags["VALUE_LENGTH"] = 49152] = "VALUE_LENGTH";
    BinTrieFlags[BinTrieFlags["BRANCH_LENGTH"] = 16256] = "BRANCH_LENGTH";
    BinTrieFlags[BinTrieFlags["JUMP_TABLE"] = 127] = "JUMP_TABLE";
})(BinTrieFlags = exports.BinTrieFlags || (exports.BinTrieFlags = {}));
function isNumber(code) {
    return code >= CharCodes.ZERO && code <= CharCodes.NINE;
}
function isHexadecimalCharacter(code) {
    return ((code >= CharCodes.UPPER_A && code <= CharCodes.UPPER_F) ||
        (code >= CharCodes.LOWER_A && code <= CharCodes.LOWER_F));
}
function isAsciiAlphaNumeric(code) {
    return ((code >= CharCodes.UPPER_A && code <= CharCodes.UPPER_Z) ||
        (code >= CharCodes.LOWER_A && code <= CharCodes.LOWER_Z) ||
        isNumber(code));
}
/**
 * Checks if the given character is a valid end character for an entity in an attribute.
 *
 * Attribute values that aren't terminated properly aren't parsed, and shouldn't lead to a parser error.
 * See the example in https://html.spec.whatwg.org/multipage/parsing.html#named-character-reference-state
 */
function isEntityInAttributeInvalidEnd(code) {
    return code === CharCodes.EQUALS || isAsciiAlphaNumeric(code);
}
var EntityDecoderState;
(function (EntityDecoderState) {
    EntityDecoderState[EntityDecoderState["EntityStart"] = 0] = "EntityStart";
    EntityDecoderState[EntityDecoderState["NumericStart"] = 1] = "NumericStart";
    EntityDecoderState[EntityDecoderState["NumericDecimal"] = 2] = "NumericDecimal";
    EntityDecoderState[EntityDecoderState["NumericHex"] = 3] = "NumericHex";
    EntityDecoderState[EntityDecoderState["NamedEntity"] = 4] = "NamedEntity";
})(EntityDecoderState || (EntityDecoderState = {}));
var DecodingMode;
(function (DecodingMode) {
    /** Entities in text nodes that can end with any character. */
    DecodingMode[DecodingMode["Legacy"] = 0] = "Legacy";
    /** Only allow entities terminated with a semicolon. */
    DecodingMode[DecodingMode["Strict"] = 1] = "Strict";
    /** Entities in attributes have limitations on ending characters. */
    DecodingMode[DecodingMode["Attribute"] = 2] = "Attribute";
})(DecodingMode = exports.DecodingMode || (exports.DecodingMode = {}));
/**
 * Token decoder with support of writing partial entities.
 */
var EntityDecoder = /** @class */ (function () {
    function EntityDecoder(
    /** The tree used to decode entities. */
    decodeTree, 
    /**
     * The function that is called when a codepoint is decoded.
     *
     * For multi-byte named entities, this will be called multiple times,
     * with the second codepoint, and the same `consumed` value.
     *
     * @param codepoint The decoded codepoint.
     * @param consumed The number of bytes consumed by the decoder.
     */
    emitCodePoint, 
    /** An object that is used to produce errors. */
    errors) {
        this.decodeTree = decodeTree;
        this.emitCodePoint = emitCodePoint;
        this.errors = errors;
        /** The current state of the decoder. */
        this.state = EntityDecoderState.EntityStart;
        /** Characters that were consumed while parsing an entity. */
        this.consumed = 1;
        /**
         * The result of the entity.
         *
         * Either the result index of a numeric entity, or the codepoint of a
         * numeric entity.
         */
        this.result = 0;
        /** The current index in the decode tree. */
        this.treeIndex = 0;
        /** The number of characters that were consumed in excess. */
        this.excess = 1;
        /** The mode in which the decoder is operating. */
        this.decodeMode = DecodingMode.Strict;
    }
    /** Resets the instance to make it reusable. */
    EntityDecoder.prototype.startEntity = function (decodeMode) {
        this.decodeMode = decodeMode;
        this.state = EntityDecoderState.EntityStart;
        this.result = 0;
        this.treeIndex = 0;
        this.excess = 1;
        this.consumed = 1;
    };
    /**
     * Write an entity to the decoder. This can be called multiple times with partial entities.
     * If the entity is incomplete, the decoder will return -1.
     *
     * Mirrors the implementation of `getDecoder`, but with the ability to stop decoding if the
     * entity is incomplete, and resume when the next string is written.
     *
     * @param string The string containing the entity (or a continuation of the entity).
     * @param offset The offset at which the entity begins. Should be 0 if this is not the first call.
     * @returns The number of characters that were consumed, or -1 if the entity is incomplete.
     */
    EntityDecoder.prototype.write = function (str, offset) {
        switch (this.state) {
            case EntityDecoderState.EntityStart: {
                if (str.charCodeAt(offset) === CharCodes.NUM) {
                    this.state = EntityDecoderState.NumericStart;
                    this.consumed += 1;
                    return this.stateNumericStart(str, offset + 1);
                }
                this.state = EntityDecoderState.NamedEntity;
                return this.stateNamedEntity(str, offset);
            }
            case EntityDecoderState.NumericStart: {
                return this.stateNumericStart(str, offset);
            }
            case EntityDecoderState.NumericDecimal: {
                return this.stateNumericDecimal(str, offset);
            }
            case EntityDecoderState.NumericHex: {
                return this.stateNumericHex(str, offset);
            }
            case EntityDecoderState.NamedEntity: {
                return this.stateNamedEntity(str, offset);
            }
        }
    };
    /**
     * Switches between the numeric decimal and hexadecimal states.
     *
     * Equivalent to the `Numeric character reference state` in the HTML spec.
     *
     * @param str The string containing the entity (or a continuation of the entity).
     * @param offset The current offset.
     * @returns The number of characters that were consumed, or -1 if the entity is incomplete.
     */
    EntityDecoder.prototype.stateNumericStart = function (str, offset) {
        if (offset >= str.length) {
            return -1;
        }
        if ((str.charCodeAt(offset) | TO_LOWER_BIT) === CharCodes.LOWER_X) {
            this.state = EntityDecoderState.NumericHex;
            this.consumed += 1;
            return this.stateNumericHex(str, offset + 1);
        }
        this.state = EntityDecoderState.NumericDecimal;
        return this.stateNumericDecimal(str, offset);
    };
    EntityDecoder.prototype.addToNumericResult = function (str, start, end, base) {
        if (start !== end) {
            var digitCount = end - start;
            this.result =
                this.result * Math.pow(base, digitCount) +
                    parseInt(str.substr(start, digitCount), base);
            this.consumed += digitCount;
        }
    };
    /**
     * Parses a hexadecimal numeric entity.
     *
     * Equivalent to the `Hexademical character reference state` in the HTML spec.
     *
     * @param str The string containing the entity (or a continuation of the entity).
     * @param offset The current offset.
     * @returns The number of characters that were consumed, or -1 if the entity is incomplete.
     */
    EntityDecoder.prototype.stateNumericHex = function (str, offset) {
        var startIdx = offset;
        while (offset < str.length) {
            var char = str.charCodeAt(offset);
            if (isNumber(char) || isHexadecimalCharacter(char)) {
                offset += 1;
            }
            else {
                this.addToNumericResult(str, startIdx, offset, 16);
                return this.emitNumericEntity(char, 3);
            }
        }
        this.addToNumericResult(str, startIdx, offset, 16);
        return -1;
    };
    /**
     * Parses a decimal numeric entity.
     *
     * Equivalent to the `Decimal character reference state` in the HTML spec.
     *
     * @param str The string containing the entity (or a continuation of the entity).
     * @param offset The current offset.
     * @returns The number of characters that were consumed, or -1 if the entity is incomplete.
     */
    EntityDecoder.prototype.stateNumericDecimal = function (str, offset) {
        var startIdx = offset;
        while (offset < str.length) {
            var char = str.charCodeAt(offset);
            if (isNumber(char)) {
                offset += 1;
            }
            else {
                this.addToNumericResult(str, startIdx, offset, 10);
                return this.emitNumericEntity(char, 2);
            }
        }
        this.addToNumericResult(str, startIdx, offset, 10);
        return -1;
    };
    /**
     * Validate and emit a numeric entity.
     *
     * Implements the logic from the `Hexademical character reference start
     * state` and `Numeric character reference end state` in the HTML spec.
     *
     * @param lastCp The last code point of the entity. Used to see if the
     *               entity was terminated with a semicolon.
     * @param expectedLength The minimum number of characters that should be
     *                       consumed. Used to validate that at least one digit
     *                       was consumed.
     * @returns The number of characters that were consumed.
     */
    EntityDecoder.prototype.emitNumericEntity = function (lastCp, expectedLength) {
        var _a;
        // Ensure we consumed at least one digit.
        if (this.consumed <= expectedLength) {
            (_a = this.errors) === null || _a === void 0 ? void 0 : _a.absenceOfDigitsInNumericCharacterReference(this.consumed);
            return 0;
        }
        // Figure out if this is a legit end of the entity
        if (lastCp === CharCodes.SEMI) {
            this.consumed += 1;
        }
        else if (this.decodeMode === DecodingMode.Strict) {
            return 0;
        }
        this.emitCodePoint((0, decode_codepoint_js_1.replaceCodePoint)(this.result), this.consumed);
        if (this.errors) {
            if (lastCp !== CharCodes.SEMI) {
                this.errors.missingSemicolonAfterCharacterReference();
            }
            this.errors.validateNumericCharacterReference(this.result);
        }
        return this.consumed;
    };
    /**
     * Parses a named entity.
     *
     * Equivalent to the `Named character reference state` in the HTML spec.
     *
     * @param str The string containing the entity (or a continuation of the entity).
     * @param offset The current offset.
     * @returns The number of characters that were consumed, or -1 if the entity is incomplete.
     */
    EntityDecoder.prototype.stateNamedEntity = function (str, offset) {
        var decodeTree = this.decodeTree;
        var current = decodeTree[this.treeIndex];
        // The mask is the number of bytes of the value, including the current byte.
        var valueLength = (current & BinTrieFlags.VALUE_LENGTH) >> 14;
        for (; offset < str.length; offset++, this.excess++) {
            var char = str.charCodeAt(offset);
            this.treeIndex = determineBranch(decodeTree, current, this.treeIndex + Math.max(1, valueLength), char);
            if (this.treeIndex < 0) {
                return this.result === 0 ||
                    // If we are parsing an attribute
                    (this.decodeMode === DecodingMode.Attribute &&
                        // We shouldn't have consumed any characters after the entity,
                        (valueLength === 0 ||
                            // And there should be no invalid characters.
                            isEntityInAttributeInvalidEnd(char)))
                    ? 0
                    : this.emitNotTerminatedNamedEntity();
            }
            current = decodeTree[this.treeIndex];
            valueLength = (current & BinTrieFlags.VALUE_LENGTH) >> 14;
            // If the branch is a value, store it and continue
            if (valueLength !== 0) {
                // If the entity is terminated by a semicolon, we are done.
                if (char === CharCodes.SEMI) {
                    return this.emitNamedEntityData(this.treeIndex, valueLength, this.consumed + this.excess);
                }
                // If we encounter a non-terminated (legacy) entity while parsing strictly, then ignore it.
                if (this.decodeMode !== DecodingMode.Strict) {
                    this.result = this.treeIndex;
                    this.consumed += this.excess;
                    this.excess = 0;
                }
            }
        }
        return -1;
    };
    /**
     * Emit a named entity that was not terminated with a semicolon.
     *
     * @returns The number of characters consumed.
     */
    EntityDecoder.prototype.emitNotTerminatedNamedEntity = function () {
        var _a;
        var _b = this, result = _b.result, decodeTree = _b.decodeTree;
        var valueLength = (decodeTree[result] & BinTrieFlags.VALUE_LENGTH) >> 14;
        this.emitNamedEntityData(result, valueLength, this.consumed);
        (_a = this.errors) === null || _a === void 0 ? void 0 : _a.missingSemicolonAfterCharacterReference();
        return this.consumed;
    };
    /**
     * Emit a named entity.
     *
     * @param result The index of the entity in the decode tree.
     * @param valueLength The number of bytes in the entity.
     * @param consumed The number of characters consumed.
     *
     * @returns The number of characters consumed.
     */
    EntityDecoder.prototype.emitNamedEntityData = function (result, valueLength, consumed) {
        var decodeTree = this.decodeTree;
        this.emitCodePoint(valueLength === 1
            ? decodeTree[result] & ~BinTrieFlags.VALUE_LENGTH
            : decodeTree[result + 1], consumed);
        if (valueLength === 3) {
            // For multi-byte values, we need to emit the second byte.
            this.emitCodePoint(decodeTree[result + 2], consumed);
        }
        return consumed;
    };
    /**
     * Signal to the parser that the end of the input was reached.
     *
     * Remaining data will be emitted and relevant errors will be produced.
     *
     * @returns The number of characters consumed.
     */
    EntityDecoder.prototype.end = function () {
        var _a;
        switch (this.state) {
            case EntityDecoderState.NamedEntity: {
                // Emit a named entity if we have one.
                return this.result !== 0 &&
                    (this.decodeMode !== DecodingMode.Attribute ||
                        this.result === this.treeIndex)
                    ? this.emitNotTerminatedNamedEntity()
                    : 0;
            }
            // Otherwise, emit a numeric entity if we have one.
            case EntityDecoderState.NumericDecimal: {
                return this.emitNumericEntity(0, 2);
            }
            case EntityDecoderState.NumericHex: {
                return this.emitNumericEntity(0, 3);
            }
            case EntityDecoderState.NumericStart: {
                (_a = this.errors) === null || _a === void 0 ? void 0 : _a.absenceOfDigitsInNumericCharacterReference(this.consumed);
                return 0;
            }
            case EntityDecoderState.EntityStart: {
                // Return 0 if we have no entity.
                return 0;
            }
        }
    };
    return EntityDecoder;
}());
exports.EntityDecoder = EntityDecoder;
/**
 * Creates a function that decodes entities in a string.
 *
 * @param decodeTree The decode tree.
 * @returns A function that decodes entities in a string.
 */
function getDecoder(decodeTree) {
    var ret = "";
    var decoder = new EntityDecoder(decodeTree, function (str) { return (ret += (0, decode_codepoint_js_1.fromCodePoint)(str)); });
    return function decodeWithTrie(str, decodeMode) {
        var lastIndex = 0;
        var offset = 0;
        while ((offset = str.indexOf("&", offset)) >= 0) {
            ret += str.slice(lastIndex, offset);
            decoder.startEntity(decodeMode);
            var len = decoder.write(str, 
            // Skip the "&"
            offset + 1);
            if (len < 0) {
                lastIndex = offset + decoder.end();
                break;
            }
            lastIndex = offset + len;
            // If `len` is 0, skip the current `&` and continue.
            offset = len === 0 ? lastIndex + 1 : lastIndex;
        }
        var result = ret + str.slice(lastIndex);
        // Make sure we don't keep a reference to the final string.
        ret = "";
        return result;
    };
}
/**
 * Determines the branch of the current node that is taken given the current
 * character. This function is used to traverse the trie.
 *
 * @param decodeTree The trie.
 * @param current The current node.
 * @param nodeIdx The index right after the current node and its value.
 * @param char The current character.
 * @returns The index of the next node, or -1 if no branch is taken.
 */
function determineBranch(decodeTree, current, nodeIdx, char) {
    var branchCount = (current & BinTrieFlags.BRANCH_LENGTH) >> 7;
    var jumpOffset = current & BinTrieFlags.JUMP_TABLE;
    // Case 1: Single branch encoded in jump offset
    if (branchCount === 0) {
        return jumpOffset !== 0 && char === jumpOffset ? nodeIdx : -1;
    }
    // Case 2: Multiple branches encoded in jump table
    if (jumpOffset) {
        var value = char - jumpOffset;
        return value < 0 || value >= branchCount
            ? -1
            : decodeTree[nodeIdx + value] - 1;
    }
    // Case 3: Multiple branches encoded in dictionary
    // Binary search for the character.
    var lo = nodeIdx;
    var hi = lo + branchCount - 1;
    while (lo <= hi) {
        var mid = (lo + hi) >>> 1;
        var midVal = decodeTree[mid];
        if (midVal < char) {
            lo = mid + 1;
        }
        else if (midVal > char) {
            hi = mid - 1;
        }
        else {
            return decodeTree[mid + branchCount];
        }
    }
    return -1;
}
exports.determineBranch = determineBranch;
var htmlDecoder = getDecoder(decode_data_html_js_1.default);
var xmlDecoder = getDecoder(decode_data_xml_js_1.default);
/**
 * Decodes an HTML string.
 *
 * @param str The string to decode.
 * @param mode The decoding mode.
 * @returns The decoded string.
 */
function decodeHTML(str, mode) {
    if (mode === void 0) { mode = DecodingMode.Legacy; }
    return htmlDecoder(str, mode);
}
exports.decodeHTML = decodeHTML;
/**
 * Decodes an HTML string in an attribute.
 *
 * @param str The string to decode.
 * @returns The decoded string.
 */
function decodeHTMLAttribute(str) {
    return htmlDecoder(str, DecodingMode.Attribute);
}
exports.decodeHTMLAttribute = decodeHTMLAttribute;
/**
 * Decodes an HTML string, requiring all entities to be terminated by a semicolon.
 *
 * @param str The string to decode.
 * @returns The decoded string.
 */
function decodeHTMLStrict(str) {
    return htmlDecoder(str, DecodingMode.Strict);
}
exports.decodeHTMLStrict = decodeHTMLStrict;
/**
 * Decodes an XML string, requiring all entities to be terminated by a semicolon.
 *
 * @param str The string to decode.
 * @returns The decoded string.
 */
function decodeXML(str) {
    return xmlDecoder(str, DecodingMode.Strict);
}
exports.decodeXML = decodeXML;

}(decode));

(function (exports) {
Object.defineProperty(exports, "__esModule", { value: true });
exports.QuoteType = void 0;
var decode_js_1 = decode;
var CharCodes;
(function (CharCodes) {
    CharCodes[CharCodes["Tab"] = 9] = "Tab";
    CharCodes[CharCodes["NewLine"] = 10] = "NewLine";
    CharCodes[CharCodes["FormFeed"] = 12] = "FormFeed";
    CharCodes[CharCodes["CarriageReturn"] = 13] = "CarriageReturn";
    CharCodes[CharCodes["Space"] = 32] = "Space";
    CharCodes[CharCodes["ExclamationMark"] = 33] = "ExclamationMark";
    CharCodes[CharCodes["Number"] = 35] = "Number";
    CharCodes[CharCodes["Amp"] = 38] = "Amp";
    CharCodes[CharCodes["SingleQuote"] = 39] = "SingleQuote";
    CharCodes[CharCodes["DoubleQuote"] = 34] = "DoubleQuote";
    CharCodes[CharCodes["Dash"] = 45] = "Dash";
    CharCodes[CharCodes["Slash"] = 47] = "Slash";
    CharCodes[CharCodes["Zero"] = 48] = "Zero";
    CharCodes[CharCodes["Nine"] = 57] = "Nine";
    CharCodes[CharCodes["Semi"] = 59] = "Semi";
    CharCodes[CharCodes["Lt"] = 60] = "Lt";
    CharCodes[CharCodes["Eq"] = 61] = "Eq";
    CharCodes[CharCodes["Gt"] = 62] = "Gt";
    CharCodes[CharCodes["Questionmark"] = 63] = "Questionmark";
    CharCodes[CharCodes["UpperA"] = 65] = "UpperA";
    CharCodes[CharCodes["LowerA"] = 97] = "LowerA";
    CharCodes[CharCodes["UpperF"] = 70] = "UpperF";
    CharCodes[CharCodes["LowerF"] = 102] = "LowerF";
    CharCodes[CharCodes["UpperZ"] = 90] = "UpperZ";
    CharCodes[CharCodes["LowerZ"] = 122] = "LowerZ";
    CharCodes[CharCodes["LowerX"] = 120] = "LowerX";
    CharCodes[CharCodes["OpeningSquareBracket"] = 91] = "OpeningSquareBracket";
})(CharCodes || (CharCodes = {}));
/** All the states the tokenizer can be in. */
var State;
(function (State) {
    State[State["Text"] = 1] = "Text";
    State[State["BeforeTagName"] = 2] = "BeforeTagName";
    State[State["InTagName"] = 3] = "InTagName";
    State[State["InSelfClosingTag"] = 4] = "InSelfClosingTag";
    State[State["BeforeClosingTagName"] = 5] = "BeforeClosingTagName";
    State[State["InClosingTagName"] = 6] = "InClosingTagName";
    State[State["AfterClosingTagName"] = 7] = "AfterClosingTagName";
    // Attributes
    State[State["BeforeAttributeName"] = 8] = "BeforeAttributeName";
    State[State["InAttributeName"] = 9] = "InAttributeName";
    State[State["AfterAttributeName"] = 10] = "AfterAttributeName";
    State[State["BeforeAttributeValue"] = 11] = "BeforeAttributeValue";
    State[State["InAttributeValueDq"] = 12] = "InAttributeValueDq";
    State[State["InAttributeValueSq"] = 13] = "InAttributeValueSq";
    State[State["InAttributeValueNq"] = 14] = "InAttributeValueNq";
    // Declarations
    State[State["BeforeDeclaration"] = 15] = "BeforeDeclaration";
    State[State["InDeclaration"] = 16] = "InDeclaration";
    // Processing instructions
    State[State["InProcessingInstruction"] = 17] = "InProcessingInstruction";
    // Comments & CDATA
    State[State["BeforeComment"] = 18] = "BeforeComment";
    State[State["CDATASequence"] = 19] = "CDATASequence";
    State[State["InSpecialComment"] = 20] = "InSpecialComment";
    State[State["InCommentLike"] = 21] = "InCommentLike";
    // Special tags
    State[State["BeforeSpecialS"] = 22] = "BeforeSpecialS";
    State[State["SpecialStartSequence"] = 23] = "SpecialStartSequence";
    State[State["InSpecialTag"] = 24] = "InSpecialTag";
    State[State["InEntity"] = 25] = "InEntity";
})(State || (State = {}));
function isWhitespace(c) {
    return (c === CharCodes.Space ||
        c === CharCodes.NewLine ||
        c === CharCodes.Tab ||
        c === CharCodes.FormFeed ||
        c === CharCodes.CarriageReturn);
}
function isEndOfTagSection(c) {
    return c === CharCodes.Slash || c === CharCodes.Gt || isWhitespace(c);
}
function isASCIIAlpha(c) {
    return ((c >= CharCodes.LowerA && c <= CharCodes.LowerZ) ||
        (c >= CharCodes.UpperA && c <= CharCodes.UpperZ));
}
var QuoteType;
(function (QuoteType) {
    QuoteType[QuoteType["NoValue"] = 0] = "NoValue";
    QuoteType[QuoteType["Unquoted"] = 1] = "Unquoted";
    QuoteType[QuoteType["Single"] = 2] = "Single";
    QuoteType[QuoteType["Double"] = 3] = "Double";
})(QuoteType = exports.QuoteType || (exports.QuoteType = {}));
/**
 * Sequences used to match longer strings.
 *
 * We don't have `Script`, `Style`, or `Title` here. Instead, we re-use the *End
 * sequences with an increased offset.
 */
var Sequences = {
    Cdata: new Uint8Array([0x43, 0x44, 0x41, 0x54, 0x41, 0x5b]),
    CdataEnd: new Uint8Array([0x5d, 0x5d, 0x3e]),
    CommentEnd: new Uint8Array([0x2d, 0x2d, 0x3e]),
    ScriptEnd: new Uint8Array([0x3c, 0x2f, 0x73, 0x63, 0x72, 0x69, 0x70, 0x74]),
    StyleEnd: new Uint8Array([0x3c, 0x2f, 0x73, 0x74, 0x79, 0x6c, 0x65]),
    TitleEnd: new Uint8Array([0x3c, 0x2f, 0x74, 0x69, 0x74, 0x6c, 0x65]), // `</title`
};
var Tokenizer = /** @class */ (function () {
    function Tokenizer(_a, cbs) {
        var _b = _a.xmlMode, xmlMode = _b === void 0 ? false : _b, _c = _a.decodeEntities, decodeEntities = _c === void 0 ? true : _c;
        var _this = this;
        this.cbs = cbs;
        /** The current state the tokenizer is in. */
        this.state = State.Text;
        /** The read buffer. */
        this.buffer = "";
        /** The beginning of the section that is currently being read. */
        this.sectionStart = 0;
        /** The index within the buffer that we are currently looking at. */
        this.index = 0;
        /** The start of the last entity. */
        this.entityStart = 0;
        /** Some behavior, eg. when decoding entities, is done while we are in another state. This keeps track of the other state type. */
        this.baseState = State.Text;
        /** For special parsing behavior inside of script and style tags. */
        this.isSpecial = false;
        /** Indicates whether the tokenizer has been paused. */
        this.running = true;
        /** The offset of the current buffer. */
        this.offset = 0;
        this.currentSequence = undefined;
        this.sequenceIndex = 0;
        this.xmlMode = xmlMode;
        this.decodeEntities = decodeEntities;
        this.entityDecoder = new decode_js_1.EntityDecoder(xmlMode ? decode_js_1.xmlDecodeTree : decode_js_1.htmlDecodeTree, function (cp, consumed) { return _this.emitCodePoint(cp, consumed); });
    }
    Tokenizer.prototype.reset = function () {
        this.state = State.Text;
        this.buffer = "";
        this.sectionStart = 0;
        this.index = 0;
        this.baseState = State.Text;
        this.currentSequence = undefined;
        this.running = true;
        this.offset = 0;
    };
    Tokenizer.prototype.write = function (chunk) {
        this.offset += this.buffer.length;
        this.buffer = chunk;
        this.parse();
    };
    Tokenizer.prototype.end = function () {
        if (this.running)
            this.finish();
    };
    Tokenizer.prototype.pause = function () {
        this.running = false;
    };
    Tokenizer.prototype.resume = function () {
        this.running = true;
        if (this.index < this.buffer.length + this.offset) {
            this.parse();
        }
    };
    Tokenizer.prototype.stateText = function (c) {
        if (c === CharCodes.Lt ||
            (!this.decodeEntities && this.fastForwardTo(CharCodes.Lt))) {
            if (this.index > this.sectionStart) {
                this.cbs.ontext(this.sectionStart, this.index);
            }
            this.state = State.BeforeTagName;
            this.sectionStart = this.index;
        }
        else if (this.decodeEntities && c === CharCodes.Amp) {
            this.startEntity();
        }
    };
    Tokenizer.prototype.stateSpecialStartSequence = function (c) {
        var isEnd = this.sequenceIndex === this.currentSequence.length;
        var isMatch = isEnd
            ? // If we are at the end of the sequence, make sure the tag name has ended
                isEndOfTagSection(c)
            : // Otherwise, do a case-insensitive comparison
                (c | 0x20) === this.currentSequence[this.sequenceIndex];
        if (!isMatch) {
            this.isSpecial = false;
        }
        else if (!isEnd) {
            this.sequenceIndex++;
            return;
        }
        this.sequenceIndex = 0;
        this.state = State.InTagName;
        this.stateInTagName(c);
    };
    /** Look for an end tag. For <title> tags, also decode entities. */
    Tokenizer.prototype.stateInSpecialTag = function (c) {
        if (this.sequenceIndex === this.currentSequence.length) {
            if (c === CharCodes.Gt || isWhitespace(c)) {
                var endOfText = this.index - this.currentSequence.length;
                if (this.sectionStart < endOfText) {
                    // Spoof the index so that reported locations match up.
                    var actualIndex = this.index;
                    this.index = endOfText;
                    this.cbs.ontext(this.sectionStart, endOfText);
                    this.index = actualIndex;
                }
                this.isSpecial = false;
                this.sectionStart = endOfText + 2; // Skip over the `</`
                this.stateInClosingTagName(c);
                return; // We are done; skip the rest of the function.
            }
            this.sequenceIndex = 0;
        }
        if ((c | 0x20) === this.currentSequence[this.sequenceIndex]) {
            this.sequenceIndex += 1;
        }
        else if (this.sequenceIndex === 0) {
            if (this.currentSequence === Sequences.TitleEnd) {
                // We have to parse entities in <title> tags.
                if (this.decodeEntities && c === CharCodes.Amp) {
                    this.startEntity();
                }
            }
            else if (this.fastForwardTo(CharCodes.Lt)) {
                // Outside of <title> tags, we can fast-forward.
                this.sequenceIndex = 1;
            }
        }
        else {
            // If we see a `<`, set the sequence index to 1; useful for eg. `<</script>`.
            this.sequenceIndex = Number(c === CharCodes.Lt);
        }
    };
    Tokenizer.prototype.stateCDATASequence = function (c) {
        if (c === Sequences.Cdata[this.sequenceIndex]) {
            if (++this.sequenceIndex === Sequences.Cdata.length) {
                this.state = State.InCommentLike;
                this.currentSequence = Sequences.CdataEnd;
                this.sequenceIndex = 0;
                this.sectionStart = this.index + 1;
            }
        }
        else {
            this.sequenceIndex = 0;
            this.state = State.InDeclaration;
            this.stateInDeclaration(c); // Reconsume the character
        }
    };
    /**
     * When we wait for one specific character, we can speed things up
     * by skipping through the buffer until we find it.
     *
     * @returns Whether the character was found.
     */
    Tokenizer.prototype.fastForwardTo = function (c) {
        while (++this.index < this.buffer.length + this.offset) {
            if (this.buffer.charCodeAt(this.index - this.offset) === c) {
                return true;
            }
        }
        /*
         * We increment the index at the end of the `parse` loop,
         * so set it to `buffer.length - 1` here.
         *
         * TODO: Refactor `parse` to increment index before calling states.
         */
        this.index = this.buffer.length + this.offset - 1;
        return false;
    };
    /**
     * Comments and CDATA end with `-->` and `]]>`.
     *
     * Their common qualities are:
     * - Their end sequences have a distinct character they start with.
     * - That character is then repeated, so we have to check multiple repeats.
     * - All characters but the start character of the sequence can be skipped.
     */
    Tokenizer.prototype.stateInCommentLike = function (c) {
        if (c === this.currentSequence[this.sequenceIndex]) {
            if (++this.sequenceIndex === this.currentSequence.length) {
                if (this.currentSequence === Sequences.CdataEnd) {
                    this.cbs.oncdata(this.sectionStart, this.index, 2);
                }
                else {
                    this.cbs.oncomment(this.sectionStart, this.index, 2);
                }
                this.sequenceIndex = 0;
                this.sectionStart = this.index + 1;
                this.state = State.Text;
            }
        }
        else if (this.sequenceIndex === 0) {
            // Fast-forward to the first character of the sequence
            if (this.fastForwardTo(this.currentSequence[0])) {
                this.sequenceIndex = 1;
            }
        }
        else if (c !== this.currentSequence[this.sequenceIndex - 1]) {
            // Allow long sequences, eg. --->, ]]]>
            this.sequenceIndex = 0;
        }
    };
    /**
     * HTML only allows ASCII alpha characters (a-z and A-Z) at the beginning of a tag name.
     *
     * XML allows a lot more characters here (@see https://www.w3.org/TR/REC-xml/#NT-NameStartChar).
     * We allow anything that wouldn't end the tag.
     */
    Tokenizer.prototype.isTagStartChar = function (c) {
        return this.xmlMode ? !isEndOfTagSection(c) : isASCIIAlpha(c);
    };
    Tokenizer.prototype.startSpecial = function (sequence, offset) {
        this.isSpecial = true;
        this.currentSequence = sequence;
        this.sequenceIndex = offset;
        this.state = State.SpecialStartSequence;
    };
    Tokenizer.prototype.stateBeforeTagName = function (c) {
        if (c === CharCodes.ExclamationMark) {
            this.state = State.BeforeDeclaration;
            this.sectionStart = this.index + 1;
        }
        else if (c === CharCodes.Questionmark) {
            this.state = State.InProcessingInstruction;
            this.sectionStart = this.index + 1;
        }
        else if (this.isTagStartChar(c)) {
            var lower = c | 0x20;
            this.sectionStart = this.index;
            if (!this.xmlMode && lower === Sequences.TitleEnd[2]) {
                this.startSpecial(Sequences.TitleEnd, 3);
            }
            else {
                this.state =
                    !this.xmlMode && lower === Sequences.ScriptEnd[2]
                        ? State.BeforeSpecialS
                        : State.InTagName;
            }
        }
        else if (c === CharCodes.Slash) {
            this.state = State.BeforeClosingTagName;
        }
        else {
            this.state = State.Text;
            this.stateText(c);
        }
    };
    Tokenizer.prototype.stateInTagName = function (c) {
        if (isEndOfTagSection(c)) {
            this.cbs.onopentagname(this.sectionStart, this.index);
            this.sectionStart = -1;
            this.state = State.BeforeAttributeName;
            this.stateBeforeAttributeName(c);
        }
    };
    Tokenizer.prototype.stateBeforeClosingTagName = function (c) {
        if (isWhitespace(c)) ;
        else if (c === CharCodes.Gt) {
            this.state = State.Text;
        }
        else {
            this.state = this.isTagStartChar(c)
                ? State.InClosingTagName
                : State.InSpecialComment;
            this.sectionStart = this.index;
        }
    };
    Tokenizer.prototype.stateInClosingTagName = function (c) {
        if (c === CharCodes.Gt || isWhitespace(c)) {
            this.cbs.onclosetag(this.sectionStart, this.index);
            this.sectionStart = -1;
            this.state = State.AfterClosingTagName;
            this.stateAfterClosingTagName(c);
        }
    };
    Tokenizer.prototype.stateAfterClosingTagName = function (c) {
        // Skip everything until ">"
        if (c === CharCodes.Gt || this.fastForwardTo(CharCodes.Gt)) {
            this.state = State.Text;
            this.sectionStart = this.index + 1;
        }
    };
    Tokenizer.prototype.stateBeforeAttributeName = function (c) {
        if (c === CharCodes.Gt) {
            this.cbs.onopentagend(this.index);
            if (this.isSpecial) {
                this.state = State.InSpecialTag;
                this.sequenceIndex = 0;
            }
            else {
                this.state = State.Text;
            }
            this.sectionStart = this.index + 1;
        }
        else if (c === CharCodes.Slash) {
            this.state = State.InSelfClosingTag;
        }
        else if (!isWhitespace(c)) {
            this.state = State.InAttributeName;
            this.sectionStart = this.index;
        }
    };
    Tokenizer.prototype.stateInSelfClosingTag = function (c) {
        if (c === CharCodes.Gt) {
            this.cbs.onselfclosingtag(this.index);
            this.state = State.Text;
            this.sectionStart = this.index + 1;
            this.isSpecial = false; // Reset special state, in case of self-closing special tags
        }
        else if (!isWhitespace(c)) {
            this.state = State.BeforeAttributeName;
            this.stateBeforeAttributeName(c);
        }
    };
    Tokenizer.prototype.stateInAttributeName = function (c) {
        if (c === CharCodes.Eq || isEndOfTagSection(c)) {
            this.cbs.onattribname(this.sectionStart, this.index);
            this.sectionStart = -1;
            this.state = State.AfterAttributeName;
            this.stateAfterAttributeName(c);
        }
    };
    Tokenizer.prototype.stateAfterAttributeName = function (c) {
        if (c === CharCodes.Eq) {
            this.state = State.BeforeAttributeValue;
        }
        else if (c === CharCodes.Slash || c === CharCodes.Gt) {
            this.cbs.onattribend(QuoteType.NoValue, this.index);
            this.state = State.BeforeAttributeName;
            this.stateBeforeAttributeName(c);
        }
        else if (!isWhitespace(c)) {
            this.cbs.onattribend(QuoteType.NoValue, this.index);
            this.state = State.InAttributeName;
            this.sectionStart = this.index;
        }
    };
    Tokenizer.prototype.stateBeforeAttributeValue = function (c) {
        if (c === CharCodes.DoubleQuote) {
            this.state = State.InAttributeValueDq;
            this.sectionStart = this.index + 1;
        }
        else if (c === CharCodes.SingleQuote) {
            this.state = State.InAttributeValueSq;
            this.sectionStart = this.index + 1;
        }
        else if (!isWhitespace(c)) {
            this.sectionStart = this.index;
            this.state = State.InAttributeValueNq;
            this.stateInAttributeValueNoQuotes(c); // Reconsume token
        }
    };
    Tokenizer.prototype.handleInAttributeValue = function (c, quote) {
        if (c === quote ||
            (!this.decodeEntities && this.fastForwardTo(quote))) {
            this.cbs.onattribdata(this.sectionStart, this.index);
            this.sectionStart = -1;
            this.cbs.onattribend(quote === CharCodes.DoubleQuote
                ? QuoteType.Double
                : QuoteType.Single, this.index);
            this.state = State.BeforeAttributeName;
        }
        else if (this.decodeEntities && c === CharCodes.Amp) {
            this.startEntity();
        }
    };
    Tokenizer.prototype.stateInAttributeValueDoubleQuotes = function (c) {
        this.handleInAttributeValue(c, CharCodes.DoubleQuote);
    };
    Tokenizer.prototype.stateInAttributeValueSingleQuotes = function (c) {
        this.handleInAttributeValue(c, CharCodes.SingleQuote);
    };
    Tokenizer.prototype.stateInAttributeValueNoQuotes = function (c) {
        if (isWhitespace(c) || c === CharCodes.Gt) {
            this.cbs.onattribdata(this.sectionStart, this.index);
            this.sectionStart = -1;
            this.cbs.onattribend(QuoteType.Unquoted, this.index);
            this.state = State.BeforeAttributeName;
            this.stateBeforeAttributeName(c);
        }
        else if (this.decodeEntities && c === CharCodes.Amp) {
            this.startEntity();
        }
    };
    Tokenizer.prototype.stateBeforeDeclaration = function (c) {
        if (c === CharCodes.OpeningSquareBracket) {
            this.state = State.CDATASequence;
            this.sequenceIndex = 0;
        }
        else {
            this.state =
                c === CharCodes.Dash
                    ? State.BeforeComment
                    : State.InDeclaration;
        }
    };
    Tokenizer.prototype.stateInDeclaration = function (c) {
        if (c === CharCodes.Gt || this.fastForwardTo(CharCodes.Gt)) {
            this.cbs.ondeclaration(this.sectionStart, this.index);
            this.state = State.Text;
            this.sectionStart = this.index + 1;
        }
    };
    Tokenizer.prototype.stateInProcessingInstruction = function (c) {
        if (c === CharCodes.Gt || this.fastForwardTo(CharCodes.Gt)) {
            this.cbs.onprocessinginstruction(this.sectionStart, this.index);
            this.state = State.Text;
            this.sectionStart = this.index + 1;
        }
    };
    Tokenizer.prototype.stateBeforeComment = function (c) {
        if (c === CharCodes.Dash) {
            this.state = State.InCommentLike;
            this.currentSequence = Sequences.CommentEnd;
            // Allow short comments (eg. <!-->)
            this.sequenceIndex = 2;
            this.sectionStart = this.index + 1;
        }
        else {
            this.state = State.InDeclaration;
        }
    };
    Tokenizer.prototype.stateInSpecialComment = function (c) {
        if (c === CharCodes.Gt || this.fastForwardTo(CharCodes.Gt)) {
            this.cbs.oncomment(this.sectionStart, this.index, 0);
            this.state = State.Text;
            this.sectionStart = this.index + 1;
        }
    };
    Tokenizer.prototype.stateBeforeSpecialS = function (c) {
        var lower = c | 0x20;
        if (lower === Sequences.ScriptEnd[3]) {
            this.startSpecial(Sequences.ScriptEnd, 4);
        }
        else if (lower === Sequences.StyleEnd[3]) {
            this.startSpecial(Sequences.StyleEnd, 4);
        }
        else {
            this.state = State.InTagName;
            this.stateInTagName(c); // Consume the token again
        }
    };
    Tokenizer.prototype.startEntity = function () {
        this.baseState = this.state;
        this.state = State.InEntity;
        this.entityStart = this.index;
        this.entityDecoder.startEntity(this.xmlMode
            ? decode_js_1.DecodingMode.Strict
            : this.baseState === State.Text ||
                this.baseState === State.InSpecialTag
                ? decode_js_1.DecodingMode.Legacy
                : decode_js_1.DecodingMode.Attribute);
    };
    Tokenizer.prototype.stateInEntity = function () {
        var length = this.entityDecoder.write(this.buffer, this.index - this.offset);
        // If `length` is positive, we are done with the entity.
        if (length >= 0) {
            this.state = this.baseState;
            if (length === 0) {
                this.index = this.entityStart;
            }
        }
        else {
            // Mark buffer as consumed.
            this.index = this.offset + this.buffer.length - 1;
        }
    };
    /**
     * Remove data that has already been consumed from the buffer.
     */
    Tokenizer.prototype.cleanup = function () {
        // If we are inside of text or attributes, emit what we already have.
        if (this.running && this.sectionStart !== this.index) {
            if (this.state === State.Text ||
                (this.state === State.InSpecialTag && this.sequenceIndex === 0)) {
                this.cbs.ontext(this.sectionStart, this.index);
                this.sectionStart = this.index;
            }
            else if (this.state === State.InAttributeValueDq ||
                this.state === State.InAttributeValueSq ||
                this.state === State.InAttributeValueNq) {
                this.cbs.onattribdata(this.sectionStart, this.index);
                this.sectionStart = this.index;
            }
        }
    };
    Tokenizer.prototype.shouldContinue = function () {
        return this.index < this.buffer.length + this.offset && this.running;
    };
    /**
     * Iterates through the buffer, calling the function corresponding to the current state.
     *
     * States that are more likely to be hit are higher up, as a performance improvement.
     */
    Tokenizer.prototype.parse = function () {
        while (this.shouldContinue()) {
            var c = this.buffer.charCodeAt(this.index - this.offset);
            switch (this.state) {
                case State.Text: {
                    this.stateText(c);
                    break;
                }
                case State.SpecialStartSequence: {
                    this.stateSpecialStartSequence(c);
                    break;
                }
                case State.InSpecialTag: {
                    this.stateInSpecialTag(c);
                    break;
                }
                case State.CDATASequence: {
                    this.stateCDATASequence(c);
                    break;
                }
                case State.InAttributeValueDq: {
                    this.stateInAttributeValueDoubleQuotes(c);
                    break;
                }
                case State.InAttributeName: {
                    this.stateInAttributeName(c);
                    break;
                }
                case State.InCommentLike: {
                    this.stateInCommentLike(c);
                    break;
                }
                case State.InSpecialComment: {
                    this.stateInSpecialComment(c);
                    break;
                }
                case State.BeforeAttributeName: {
                    this.stateBeforeAttributeName(c);
                    break;
                }
                case State.InTagName: {
                    this.stateInTagName(c);
                    break;
                }
                case State.InClosingTagName: {
                    this.stateInClosingTagName(c);
                    break;
                }
                case State.BeforeTagName: {
                    this.stateBeforeTagName(c);
                    break;
                }
                case State.AfterAttributeName: {
                    this.stateAfterAttributeName(c);
                    break;
                }
                case State.InAttributeValueSq: {
                    this.stateInAttributeValueSingleQuotes(c);
                    break;
                }
                case State.BeforeAttributeValue: {
                    this.stateBeforeAttributeValue(c);
                    break;
                }
                case State.BeforeClosingTagName: {
                    this.stateBeforeClosingTagName(c);
                    break;
                }
                case State.AfterClosingTagName: {
                    this.stateAfterClosingTagName(c);
                    break;
                }
                case State.BeforeSpecialS: {
                    this.stateBeforeSpecialS(c);
                    break;
                }
                case State.InAttributeValueNq: {
                    this.stateInAttributeValueNoQuotes(c);
                    break;
                }
                case State.InSelfClosingTag: {
                    this.stateInSelfClosingTag(c);
                    break;
                }
                case State.InDeclaration: {
                    this.stateInDeclaration(c);
                    break;
                }
                case State.BeforeDeclaration: {
                    this.stateBeforeDeclaration(c);
                    break;
                }
                case State.BeforeComment: {
                    this.stateBeforeComment(c);
                    break;
                }
                case State.InProcessingInstruction: {
                    this.stateInProcessingInstruction(c);
                    break;
                }
                case State.InEntity: {
                    this.stateInEntity();
                    break;
                }
            }
            this.index++;
        }
        this.cleanup();
    };
    Tokenizer.prototype.finish = function () {
        if (this.state === State.InEntity) {
            this.entityDecoder.end();
            this.state = this.baseState;
        }
        this.handleTrailingData();
        this.cbs.onend();
    };
    /** Handle any trailing data. */
    Tokenizer.prototype.handleTrailingData = function () {
        var endIndex = this.buffer.length + this.offset;
        // If there is no remaining data, we are done.
        if (this.sectionStart >= endIndex) {
            return;
        }
        if (this.state === State.InCommentLike) {
            if (this.currentSequence === Sequences.CdataEnd) {
                this.cbs.oncdata(this.sectionStart, endIndex, 0);
            }
            else {
                this.cbs.oncomment(this.sectionStart, endIndex, 0);
            }
        }
        else if (this.state === State.InTagName ||
            this.state === State.BeforeAttributeName ||
            this.state === State.BeforeAttributeValue ||
            this.state === State.AfterAttributeName ||
            this.state === State.InAttributeName ||
            this.state === State.InAttributeValueSq ||
            this.state === State.InAttributeValueDq ||
            this.state === State.InAttributeValueNq ||
            this.state === State.InClosingTagName) ;
        else {
            this.cbs.ontext(this.sectionStart, endIndex);
        }
    };
    Tokenizer.prototype.emitCodePoint = function (cp, consumed) {
        if (this.baseState !== State.Text &&
            this.baseState !== State.InSpecialTag) {
            if (this.sectionStart < this.entityStart) {
                this.cbs.onattribdata(this.sectionStart, this.entityStart);
            }
            this.sectionStart = this.entityStart + consumed;
            this.index = this.sectionStart - 1;
            this.cbs.onattribentity(cp);
        }
        else {
            if (this.sectionStart < this.entityStart) {
                this.cbs.ontext(this.sectionStart, this.entityStart);
            }
            this.sectionStart = this.entityStart + consumed;
            this.index = this.sectionStart - 1;
            this.cbs.ontextentity(cp, this.sectionStart);
        }
    };
    return Tokenizer;
}());
exports.default = Tokenizer;

}(Tokenizer));

var __createBinding$1 = (commonjsGlobal && commonjsGlobal.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault$1 = (commonjsGlobal && commonjsGlobal.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar$1 = (commonjsGlobal && commonjsGlobal.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding$1(result, mod, k);
    __setModuleDefault$1(result, mod);
    return result;
};
Object.defineProperty(Parser$2, "__esModule", { value: true });
Parser$2.Parser = void 0;
var Tokenizer_js_1 = __importStar$1(Tokenizer);
var decode_js_1 = decode;
var formTags = new Set([
    "input",
    "option",
    "optgroup",
    "select",
    "button",
    "datalist",
    "textarea",
]);
var pTag = new Set(["p"]);
var tableSectionTags = new Set(["thead", "tbody"]);
var ddtTags = new Set(["dd", "dt"]);
var rtpTags = new Set(["rt", "rp"]);
var openImpliesClose = new Map([
    ["tr", new Set(["tr", "th", "td"])],
    ["th", new Set(["th"])],
    ["td", new Set(["thead", "th", "td"])],
    ["body", new Set(["head", "link", "script"])],
    ["li", new Set(["li"])],
    ["p", pTag],
    ["h1", pTag],
    ["h2", pTag],
    ["h3", pTag],
    ["h4", pTag],
    ["h5", pTag],
    ["h6", pTag],
    ["select", formTags],
    ["input", formTags],
    ["output", formTags],
    ["button", formTags],
    ["datalist", formTags],
    ["textarea", formTags],
    ["option", new Set(["option"])],
    ["optgroup", new Set(["optgroup", "option"])],
    ["dd", ddtTags],
    ["dt", ddtTags],
    ["address", pTag],
    ["article", pTag],
    ["aside", pTag],
    ["blockquote", pTag],
    ["details", pTag],
    ["div", pTag],
    ["dl", pTag],
    ["fieldset", pTag],
    ["figcaption", pTag],
    ["figure", pTag],
    ["footer", pTag],
    ["form", pTag],
    ["header", pTag],
    ["hr", pTag],
    ["main", pTag],
    ["nav", pTag],
    ["ol", pTag],
    ["pre", pTag],
    ["section", pTag],
    ["table", pTag],
    ["ul", pTag],
    ["rt", rtpTags],
    ["rp", rtpTags],
    ["tbody", tableSectionTags],
    ["tfoot", tableSectionTags],
]);
var voidElements = new Set([
    "area",
    "base",
    "basefont",
    "br",
    "col",
    "command",
    "embed",
    "frame",
    "hr",
    "img",
    "input",
    "isindex",
    "keygen",
    "link",
    "meta",
    "param",
    "source",
    "track",
    "wbr",
]);
var foreignContextElements = new Set(["math", "svg"]);
var htmlIntegrationElements = new Set([
    "mi",
    "mo",
    "mn",
    "ms",
    "mtext",
    "annotation-xml",
    "foreignobject",
    "desc",
    "title",
]);
var reNameEnd = /\s|\//;
var Parser$1 = /** @class */ (function () {
    function Parser(cbs, options) {
        if (options === void 0) { options = {}; }
        var _a, _b, _c, _d, _e;
        this.options = options;
        /** The start index of the last event. */
        this.startIndex = 0;
        /** The end index of the last event. */
        this.endIndex = 0;
        /**
         * Store the start index of the current open tag,
         * so we can update the start index for attributes.
         */
        this.openTagStart = 0;
        this.tagname = "";
        this.attribname = "";
        this.attribvalue = "";
        this.attribs = null;
        this.stack = [];
        this.buffers = [];
        this.bufferOffset = 0;
        /** The index of the last written buffer. Used when resuming after a `pause()`. */
        this.writeIndex = 0;
        /** Indicates whether the parser has finished running / `.end` has been called. */
        this.ended = false;
        this.cbs = cbs !== null && cbs !== void 0 ? cbs : {};
        this.htmlMode = !this.options.xmlMode;
        this.lowerCaseTagNames = (_a = options.lowerCaseTags) !== null && _a !== void 0 ? _a : this.htmlMode;
        this.lowerCaseAttributeNames =
            (_b = options.lowerCaseAttributeNames) !== null && _b !== void 0 ? _b : this.htmlMode;
        this.tokenizer = new ((_c = options.Tokenizer) !== null && _c !== void 0 ? _c : Tokenizer_js_1.default)(this.options, this);
        this.foreignContext = [!this.htmlMode];
        (_e = (_d = this.cbs).onparserinit) === null || _e === void 0 ? void 0 : _e.call(_d, this);
    }
    // Tokenizer event handlers
    /** @internal */
    Parser.prototype.ontext = function (start, endIndex) {
        var _a, _b;
        var data = this.getSlice(start, endIndex);
        this.endIndex = endIndex - 1;
        (_b = (_a = this.cbs).ontext) === null || _b === void 0 ? void 0 : _b.call(_a, data);
        this.startIndex = endIndex;
    };
    /** @internal */
    Parser.prototype.ontextentity = function (cp, endIndex) {
        var _a, _b;
        this.endIndex = endIndex - 1;
        (_b = (_a = this.cbs).ontext) === null || _b === void 0 ? void 0 : _b.call(_a, (0, decode_js_1.fromCodePoint)(cp));
        this.startIndex = endIndex;
    };
    /**
     * Checks if the current tag is a void element. Override this if you want
     * to specify your own additional void elements.
     */
    Parser.prototype.isVoidElement = function (name) {
        return this.htmlMode && voidElements.has(name);
    };
    /** @internal */
    Parser.prototype.onopentagname = function (start, endIndex) {
        this.endIndex = endIndex;
        var name = this.getSlice(start, endIndex);
        if (this.lowerCaseTagNames) {
            name = name.toLowerCase();
        }
        this.emitOpenTag(name);
    };
    Parser.prototype.emitOpenTag = function (name) {
        var _a, _b, _c, _d;
        this.openTagStart = this.startIndex;
        this.tagname = name;
        var impliesClose = this.htmlMode && openImpliesClose.get(name);
        if (impliesClose) {
            while (this.stack.length > 0 && impliesClose.has(this.stack[0])) {
                var element = this.stack.shift();
                (_b = (_a = this.cbs).onclosetag) === null || _b === void 0 ? void 0 : _b.call(_a, element, true);
            }
        }
        if (!this.isVoidElement(name)) {
            this.stack.unshift(name);
            if (this.htmlMode) {
                if (foreignContextElements.has(name)) {
                    this.foreignContext.unshift(true);
                }
                else if (htmlIntegrationElements.has(name)) {
                    this.foreignContext.unshift(false);
                }
            }
        }
        (_d = (_c = this.cbs).onopentagname) === null || _d === void 0 ? void 0 : _d.call(_c, name);
        if (this.cbs.onopentag)
            this.attribs = {};
    };
    Parser.prototype.endOpenTag = function (isImplied) {
        var _a, _b;
        this.startIndex = this.openTagStart;
        if (this.attribs) {
            (_b = (_a = this.cbs).onopentag) === null || _b === void 0 ? void 0 : _b.call(_a, this.tagname, this.attribs, isImplied);
            this.attribs = null;
        }
        if (this.cbs.onclosetag && this.isVoidElement(this.tagname)) {
            this.cbs.onclosetag(this.tagname, true);
        }
        this.tagname = "";
    };
    /** @internal */
    Parser.prototype.onopentagend = function (endIndex) {
        this.endIndex = endIndex;
        this.endOpenTag(false);
        // Set `startIndex` for next node
        this.startIndex = endIndex + 1;
    };
    /** @internal */
    Parser.prototype.onclosetag = function (start, endIndex) {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        this.endIndex = endIndex;
        var name = this.getSlice(start, endIndex);
        if (this.lowerCaseTagNames) {
            name = name.toLowerCase();
        }
        if (this.htmlMode &&
            (foreignContextElements.has(name) ||
                htmlIntegrationElements.has(name))) {
            this.foreignContext.shift();
        }
        if (!this.isVoidElement(name)) {
            var pos = this.stack.indexOf(name);
            if (pos !== -1) {
                for (var index = 0; index <= pos; index++) {
                    var element = this.stack.shift();
                    // We know the stack has sufficient elements.
                    (_b = (_a = this.cbs).onclosetag) === null || _b === void 0 ? void 0 : _b.call(_a, element, index !== pos);
                }
            }
            else if (this.htmlMode && name === "p") {
                // Implicit open before close
                this.emitOpenTag("p");
                this.closeCurrentTag(true);
            }
        }
        else if (this.htmlMode && name === "br") {
            // We can't use `emitOpenTag` for implicit open, as `br` would be implicitly closed.
            (_d = (_c = this.cbs).onopentagname) === null || _d === void 0 ? void 0 : _d.call(_c, "br");
            (_f = (_e = this.cbs).onopentag) === null || _f === void 0 ? void 0 : _f.call(_e, "br", {}, true);
            (_h = (_g = this.cbs).onclosetag) === null || _h === void 0 ? void 0 : _h.call(_g, "br", false);
        }
        // Set `startIndex` for next node
        this.startIndex = endIndex + 1;
    };
    /** @internal */
    Parser.prototype.onselfclosingtag = function (endIndex) {
        this.endIndex = endIndex;
        if (this.options.recognizeSelfClosing || this.foreignContext[0]) {
            this.closeCurrentTag(false);
            // Set `startIndex` for next node
            this.startIndex = endIndex + 1;
        }
        else {
            // Ignore the fact that the tag is self-closing.
            this.onopentagend(endIndex);
        }
    };
    Parser.prototype.closeCurrentTag = function (isOpenImplied) {
        var _a, _b;
        var name = this.tagname;
        this.endOpenTag(isOpenImplied);
        // Self-closing tags will be on the top of the stack
        if (this.stack[0] === name) {
            // If the opening tag isn't implied, the closing tag has to be implied.
            (_b = (_a = this.cbs).onclosetag) === null || _b === void 0 ? void 0 : _b.call(_a, name, !isOpenImplied);
            this.stack.shift();
        }
    };
    /** @internal */
    Parser.prototype.onattribname = function (start, endIndex) {
        this.startIndex = start;
        var name = this.getSlice(start, endIndex);
        this.attribname = this.lowerCaseAttributeNames
            ? name.toLowerCase()
            : name;
    };
    /** @internal */
    Parser.prototype.onattribdata = function (start, endIndex) {
        this.attribvalue += this.getSlice(start, endIndex);
    };
    /** @internal */
    Parser.prototype.onattribentity = function (cp) {
        this.attribvalue += (0, decode_js_1.fromCodePoint)(cp);
    };
    /** @internal */
    Parser.prototype.onattribend = function (quote, endIndex) {
        var _a, _b;
        this.endIndex = endIndex;
        (_b = (_a = this.cbs).onattribute) === null || _b === void 0 ? void 0 : _b.call(_a, this.attribname, this.attribvalue, quote === Tokenizer_js_1.QuoteType.Double
            ? '"'
            : quote === Tokenizer_js_1.QuoteType.Single
                ? "'"
                : quote === Tokenizer_js_1.QuoteType.NoValue
                    ? undefined
                    : null);
        if (this.attribs &&
            !Object.prototype.hasOwnProperty.call(this.attribs, this.attribname)) {
            this.attribs[this.attribname] = this.attribvalue;
        }
        this.attribvalue = "";
    };
    Parser.prototype.getInstructionName = function (value) {
        var index = value.search(reNameEnd);
        var name = index < 0 ? value : value.substr(0, index);
        if (this.lowerCaseTagNames) {
            name = name.toLowerCase();
        }
        return name;
    };
    /** @internal */
    Parser.prototype.ondeclaration = function (start, endIndex) {
        this.endIndex = endIndex;
        var value = this.getSlice(start, endIndex);
        if (this.cbs.onprocessinginstruction) {
            var name = this.getInstructionName(value);
            this.cbs.onprocessinginstruction("!".concat(name), "!".concat(value));
        }
        // Set `startIndex` for next node
        this.startIndex = endIndex + 1;
    };
    /** @internal */
    Parser.prototype.onprocessinginstruction = function (start, endIndex) {
        this.endIndex = endIndex;
        var value = this.getSlice(start, endIndex);
        if (this.cbs.onprocessinginstruction) {
            var name = this.getInstructionName(value);
            this.cbs.onprocessinginstruction("?".concat(name), "?".concat(value));
        }
        // Set `startIndex` for next node
        this.startIndex = endIndex + 1;
    };
    /** @internal */
    Parser.prototype.oncomment = function (start, endIndex, offset) {
        var _a, _b, _c, _d;
        this.endIndex = endIndex;
        (_b = (_a = this.cbs).oncomment) === null || _b === void 0 ? void 0 : _b.call(_a, this.getSlice(start, endIndex - offset));
        (_d = (_c = this.cbs).oncommentend) === null || _d === void 0 ? void 0 : _d.call(_c);
        // Set `startIndex` for next node
        this.startIndex = endIndex + 1;
    };
    /** @internal */
    Parser.prototype.oncdata = function (start, endIndex, offset) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        this.endIndex = endIndex;
        var value = this.getSlice(start, endIndex - offset);
        if (!this.htmlMode || this.options.recognizeCDATA) {
            (_b = (_a = this.cbs).oncdatastart) === null || _b === void 0 ? void 0 : _b.call(_a);
            (_d = (_c = this.cbs).ontext) === null || _d === void 0 ? void 0 : _d.call(_c, value);
            (_f = (_e = this.cbs).oncdataend) === null || _f === void 0 ? void 0 : _f.call(_e);
        }
        else {
            (_h = (_g = this.cbs).oncomment) === null || _h === void 0 ? void 0 : _h.call(_g, "[CDATA[".concat(value, "]]"));
            (_k = (_j = this.cbs).oncommentend) === null || _k === void 0 ? void 0 : _k.call(_j);
        }
        // Set `startIndex` for next node
        this.startIndex = endIndex + 1;
    };
    /** @internal */
    Parser.prototype.onend = function () {
        var _a, _b;
        if (this.cbs.onclosetag) {
            // Set the end index for all remaining tags
            this.endIndex = this.startIndex;
            for (var index = 0; index < this.stack.length; index++) {
                this.cbs.onclosetag(this.stack[index], true);
            }
        }
        (_b = (_a = this.cbs).onend) === null || _b === void 0 ? void 0 : _b.call(_a);
    };
    /**
     * Resets the parser to a blank state, ready to parse a new HTML document
     */
    Parser.prototype.reset = function () {
        var _a, _b, _c, _d;
        (_b = (_a = this.cbs).onreset) === null || _b === void 0 ? void 0 : _b.call(_a);
        this.tokenizer.reset();
        this.tagname = "";
        this.attribname = "";
        this.attribs = null;
        this.stack.length = 0;
        this.startIndex = 0;
        this.endIndex = 0;
        (_d = (_c = this.cbs).onparserinit) === null || _d === void 0 ? void 0 : _d.call(_c, this);
        this.buffers.length = 0;
        this.foreignContext.length = 0;
        this.foreignContext.unshift(!this.htmlMode);
        this.bufferOffset = 0;
        this.writeIndex = 0;
        this.ended = false;
    };
    /**
     * Resets the parser, then parses a complete document and
     * pushes it to the handler.
     *
     * @param data Document to parse.
     */
    Parser.prototype.parseComplete = function (data) {
        this.reset();
        this.end(data);
    };
    Parser.prototype.getSlice = function (start, end) {
        while (start - this.bufferOffset >= this.buffers[0].length) {
            this.shiftBuffer();
        }
        var slice = this.buffers[0].slice(start - this.bufferOffset, end - this.bufferOffset);
        while (end - this.bufferOffset > this.buffers[0].length) {
            this.shiftBuffer();
            slice += this.buffers[0].slice(0, end - this.bufferOffset);
        }
        return slice;
    };
    Parser.prototype.shiftBuffer = function () {
        this.bufferOffset += this.buffers[0].length;
        this.writeIndex--;
        this.buffers.shift();
    };
    /**
     * Parses a chunk of data and calls the corresponding callbacks.
     *
     * @param chunk Chunk to parse.
     */
    Parser.prototype.write = function (chunk) {
        var _a, _b;
        if (this.ended) {
            (_b = (_a = this.cbs).onerror) === null || _b === void 0 ? void 0 : _b.call(_a, new Error(".write() after done!"));
            return;
        }
        this.buffers.push(chunk);
        if (this.tokenizer.running) {
            this.tokenizer.write(chunk);
            this.writeIndex++;
        }
    };
    /**
     * Parses the end of the buffer and clears the stack, calls onend.
     *
     * @param chunk Optional final chunk to parse.
     */
    Parser.prototype.end = function (chunk) {
        var _a, _b;
        if (this.ended) {
            (_b = (_a = this.cbs).onerror) === null || _b === void 0 ? void 0 : _b.call(_a, new Error(".end() after done!"));
            return;
        }
        if (chunk)
            this.write(chunk);
        this.ended = true;
        this.tokenizer.end();
    };
    /**
     * Pauses parsing. The parser won't emit events until `resume` is called.
     */
    Parser.prototype.pause = function () {
        this.tokenizer.pause();
    };
    /**
     * Resumes parsing after `pause` was called.
     */
    Parser.prototype.resume = function () {
        this.tokenizer.resume();
        while (this.tokenizer.running &&
            this.writeIndex < this.buffers.length) {
            this.tokenizer.write(this.buffers[this.writeIndex++]);
        }
        if (this.ended)
            this.tokenizer.end();
    };
    /**
     * Alias of `write`, for backwards compatibility.
     *
     * @param chunk Chunk to parse.
     * @deprecated
     */
    Parser.prototype.parseChunk = function (chunk) {
        this.write(chunk);
    };
    /**
     * Alias of `end`, for backwards compatibility.
     *
     * @param chunk Optional final chunk to parse.
     * @deprecated
     */
    Parser.prototype.done = function (chunk) {
        this.end(chunk);
    };
    return Parser;
}());
Parser$2.Parser = Parser$1;

var lib$3 = {};

var stringify = {};

var lib$2 = {};

var lib$1 = {};

var encode = {};

var encodeHtml = {};

// Generated using scripts/write-encode-map.ts
Object.defineProperty(encodeHtml, "__esModule", { value: true });
function restoreDiff(arr) {
    for (var i = 1; i < arr.length; i++) {
        arr[i][0] += arr[i - 1][0] + 1;
    }
    return arr;
}
// prettier-ignore
encodeHtml.default = new Map(/* #__PURE__ */ restoreDiff([[9, "&Tab;"], [0, "&NewLine;"], [22, "&excl;"], [0, "&quot;"], [0, "&num;"], [0, "&dollar;"], [0, "&percnt;"], [0, "&amp;"], [0, "&apos;"], [0, "&lpar;"], [0, "&rpar;"], [0, "&ast;"], [0, "&plus;"], [0, "&comma;"], [1, "&period;"], [0, "&sol;"], [10, "&colon;"], [0, "&semi;"], [0, { v: "&lt;", n: 8402, o: "&nvlt;" }], [0, { v: "&equals;", n: 8421, o: "&bne;" }], [0, { v: "&gt;", n: 8402, o: "&nvgt;" }], [0, "&quest;"], [0, "&commat;"], [26, "&lbrack;"], [0, "&bsol;"], [0, "&rbrack;"], [0, "&Hat;"], [0, "&lowbar;"], [0, "&DiacriticalGrave;"], [5, { n: 106, o: "&fjlig;" }], [20, "&lbrace;"], [0, "&verbar;"], [0, "&rbrace;"], [34, "&nbsp;"], [0, "&iexcl;"], [0, "&cent;"], [0, "&pound;"], [0, "&curren;"], [0, "&yen;"], [0, "&brvbar;"], [0, "&sect;"], [0, "&die;"], [0, "&copy;"], [0, "&ordf;"], [0, "&laquo;"], [0, "&not;"], [0, "&shy;"], [0, "&circledR;"], [0, "&macr;"], [0, "&deg;"], [0, "&PlusMinus;"], [0, "&sup2;"], [0, "&sup3;"], [0, "&acute;"], [0, "&micro;"], [0, "&para;"], [0, "&centerdot;"], [0, "&cedil;"], [0, "&sup1;"], [0, "&ordm;"], [0, "&raquo;"], [0, "&frac14;"], [0, "&frac12;"], [0, "&frac34;"], [0, "&iquest;"], [0, "&Agrave;"], [0, "&Aacute;"], [0, "&Acirc;"], [0, "&Atilde;"], [0, "&Auml;"], [0, "&angst;"], [0, "&AElig;"], [0, "&Ccedil;"], [0, "&Egrave;"], [0, "&Eacute;"], [0, "&Ecirc;"], [0, "&Euml;"], [0, "&Igrave;"], [0, "&Iacute;"], [0, "&Icirc;"], [0, "&Iuml;"], [0, "&ETH;"], [0, "&Ntilde;"], [0, "&Ograve;"], [0, "&Oacute;"], [0, "&Ocirc;"], [0, "&Otilde;"], [0, "&Ouml;"], [0, "&times;"], [0, "&Oslash;"], [0, "&Ugrave;"], [0, "&Uacute;"], [0, "&Ucirc;"], [0, "&Uuml;"], [0, "&Yacute;"], [0, "&THORN;"], [0, "&szlig;"], [0, "&agrave;"], [0, "&aacute;"], [0, "&acirc;"], [0, "&atilde;"], [0, "&auml;"], [0, "&aring;"], [0, "&aelig;"], [0, "&ccedil;"], [0, "&egrave;"], [0, "&eacute;"], [0, "&ecirc;"], [0, "&euml;"], [0, "&igrave;"], [0, "&iacute;"], [0, "&icirc;"], [0, "&iuml;"], [0, "&eth;"], [0, "&ntilde;"], [0, "&ograve;"], [0, "&oacute;"], [0, "&ocirc;"], [0, "&otilde;"], [0, "&ouml;"], [0, "&div;"], [0, "&oslash;"], [0, "&ugrave;"], [0, "&uacute;"], [0, "&ucirc;"], [0, "&uuml;"], [0, "&yacute;"], [0, "&thorn;"], [0, "&yuml;"], [0, "&Amacr;"], [0, "&amacr;"], [0, "&Abreve;"], [0, "&abreve;"], [0, "&Aogon;"], [0, "&aogon;"], [0, "&Cacute;"], [0, "&cacute;"], [0, "&Ccirc;"], [0, "&ccirc;"], [0, "&Cdot;"], [0, "&cdot;"], [0, "&Ccaron;"], [0, "&ccaron;"], [0, "&Dcaron;"], [0, "&dcaron;"], [0, "&Dstrok;"], [0, "&dstrok;"], [0, "&Emacr;"], [0, "&emacr;"], [2, "&Edot;"], [0, "&edot;"], [0, "&Eogon;"], [0, "&eogon;"], [0, "&Ecaron;"], [0, "&ecaron;"], [0, "&Gcirc;"], [0, "&gcirc;"], [0, "&Gbreve;"], [0, "&gbreve;"], [0, "&Gdot;"], [0, "&gdot;"], [0, "&Gcedil;"], [1, "&Hcirc;"], [0, "&hcirc;"], [0, "&Hstrok;"], [0, "&hstrok;"], [0, "&Itilde;"], [0, "&itilde;"], [0, "&Imacr;"], [0, "&imacr;"], [2, "&Iogon;"], [0, "&iogon;"], [0, "&Idot;"], [0, "&imath;"], [0, "&IJlig;"], [0, "&ijlig;"], [0, "&Jcirc;"], [0, "&jcirc;"], [0, "&Kcedil;"], [0, "&kcedil;"], [0, "&kgreen;"], [0, "&Lacute;"], [0, "&lacute;"], [0, "&Lcedil;"], [0, "&lcedil;"], [0, "&Lcaron;"], [0, "&lcaron;"], [0, "&Lmidot;"], [0, "&lmidot;"], [0, "&Lstrok;"], [0, "&lstrok;"], [0, "&Nacute;"], [0, "&nacute;"], [0, "&Ncedil;"], [0, "&ncedil;"], [0, "&Ncaron;"], [0, "&ncaron;"], [0, "&napos;"], [0, "&ENG;"], [0, "&eng;"], [0, "&Omacr;"], [0, "&omacr;"], [2, "&Odblac;"], [0, "&odblac;"], [0, "&OElig;"], [0, "&oelig;"], [0, "&Racute;"], [0, "&racute;"], [0, "&Rcedil;"], [0, "&rcedil;"], [0, "&Rcaron;"], [0, "&rcaron;"], [0, "&Sacute;"], [0, "&sacute;"], [0, "&Scirc;"], [0, "&scirc;"], [0, "&Scedil;"], [0, "&scedil;"], [0, "&Scaron;"], [0, "&scaron;"], [0, "&Tcedil;"], [0, "&tcedil;"], [0, "&Tcaron;"], [0, "&tcaron;"], [0, "&Tstrok;"], [0, "&tstrok;"], [0, "&Utilde;"], [0, "&utilde;"], [0, "&Umacr;"], [0, "&umacr;"], [0, "&Ubreve;"], [0, "&ubreve;"], [0, "&Uring;"], [0, "&uring;"], [0, "&Udblac;"], [0, "&udblac;"], [0, "&Uogon;"], [0, "&uogon;"], [0, "&Wcirc;"], [0, "&wcirc;"], [0, "&Ycirc;"], [0, "&ycirc;"], [0, "&Yuml;"], [0, "&Zacute;"], [0, "&zacute;"], [0, "&Zdot;"], [0, "&zdot;"], [0, "&Zcaron;"], [0, "&zcaron;"], [19, "&fnof;"], [34, "&imped;"], [63, "&gacute;"], [65, "&jmath;"], [142, "&circ;"], [0, "&caron;"], [16, "&breve;"], [0, "&DiacriticalDot;"], [0, "&ring;"], [0, "&ogon;"], [0, "&DiacriticalTilde;"], [0, "&dblac;"], [51, "&DownBreve;"], [127, "&Alpha;"], [0, "&Beta;"], [0, "&Gamma;"], [0, "&Delta;"], [0, "&Epsilon;"], [0, "&Zeta;"], [0, "&Eta;"], [0, "&Theta;"], [0, "&Iota;"], [0, "&Kappa;"], [0, "&Lambda;"], [0, "&Mu;"], [0, "&Nu;"], [0, "&Xi;"], [0, "&Omicron;"], [0, "&Pi;"], [0, "&Rho;"], [1, "&Sigma;"], [0, "&Tau;"], [0, "&Upsilon;"], [0, "&Phi;"], [0, "&Chi;"], [0, "&Psi;"], [0, "&ohm;"], [7, "&alpha;"], [0, "&beta;"], [0, "&gamma;"], [0, "&delta;"], [0, "&epsi;"], [0, "&zeta;"], [0, "&eta;"], [0, "&theta;"], [0, "&iota;"], [0, "&kappa;"], [0, "&lambda;"], [0, "&mu;"], [0, "&nu;"], [0, "&xi;"], [0, "&omicron;"], [0, "&pi;"], [0, "&rho;"], [0, "&sigmaf;"], [0, "&sigma;"], [0, "&tau;"], [0, "&upsi;"], [0, "&phi;"], [0, "&chi;"], [0, "&psi;"], [0, "&omega;"], [7, "&thetasym;"], [0, "&Upsi;"], [2, "&phiv;"], [0, "&piv;"], [5, "&Gammad;"], [0, "&digamma;"], [18, "&kappav;"], [0, "&rhov;"], [3, "&epsiv;"], [0, "&backepsilon;"], [10, "&IOcy;"], [0, "&DJcy;"], [0, "&GJcy;"], [0, "&Jukcy;"], [0, "&DScy;"], [0, "&Iukcy;"], [0, "&YIcy;"], [0, "&Jsercy;"], [0, "&LJcy;"], [0, "&NJcy;"], [0, "&TSHcy;"], [0, "&KJcy;"], [1, "&Ubrcy;"], [0, "&DZcy;"], [0, "&Acy;"], [0, "&Bcy;"], [0, "&Vcy;"], [0, "&Gcy;"], [0, "&Dcy;"], [0, "&IEcy;"], [0, "&ZHcy;"], [0, "&Zcy;"], [0, "&Icy;"], [0, "&Jcy;"], [0, "&Kcy;"], [0, "&Lcy;"], [0, "&Mcy;"], [0, "&Ncy;"], [0, "&Ocy;"], [0, "&Pcy;"], [0, "&Rcy;"], [0, "&Scy;"], [0, "&Tcy;"], [0, "&Ucy;"], [0, "&Fcy;"], [0, "&KHcy;"], [0, "&TScy;"], [0, "&CHcy;"], [0, "&SHcy;"], [0, "&SHCHcy;"], [0, "&HARDcy;"], [0, "&Ycy;"], [0, "&SOFTcy;"], [0, "&Ecy;"], [0, "&YUcy;"], [0, "&YAcy;"], [0, "&acy;"], [0, "&bcy;"], [0, "&vcy;"], [0, "&gcy;"], [0, "&dcy;"], [0, "&iecy;"], [0, "&zhcy;"], [0, "&zcy;"], [0, "&icy;"], [0, "&jcy;"], [0, "&kcy;"], [0, "&lcy;"], [0, "&mcy;"], [0, "&ncy;"], [0, "&ocy;"], [0, "&pcy;"], [0, "&rcy;"], [0, "&scy;"], [0, "&tcy;"], [0, "&ucy;"], [0, "&fcy;"], [0, "&khcy;"], [0, "&tscy;"], [0, "&chcy;"], [0, "&shcy;"], [0, "&shchcy;"], [0, "&hardcy;"], [0, "&ycy;"], [0, "&softcy;"], [0, "&ecy;"], [0, "&yucy;"], [0, "&yacy;"], [1, "&iocy;"], [0, "&djcy;"], [0, "&gjcy;"], [0, "&jukcy;"], [0, "&dscy;"], [0, "&iukcy;"], [0, "&yicy;"], [0, "&jsercy;"], [0, "&ljcy;"], [0, "&njcy;"], [0, "&tshcy;"], [0, "&kjcy;"], [1, "&ubrcy;"], [0, "&dzcy;"], [7074, "&ensp;"], [0, "&emsp;"], [0, "&emsp13;"], [0, "&emsp14;"], [1, "&numsp;"], [0, "&puncsp;"], [0, "&ThinSpace;"], [0, "&hairsp;"], [0, "&NegativeMediumSpace;"], [0, "&zwnj;"], [0, "&zwj;"], [0, "&lrm;"], [0, "&rlm;"], [0, "&dash;"], [2, "&ndash;"], [0, "&mdash;"], [0, "&horbar;"], [0, "&Verbar;"], [1, "&lsquo;"], [0, "&CloseCurlyQuote;"], [0, "&lsquor;"], [1, "&ldquo;"], [0, "&CloseCurlyDoubleQuote;"], [0, "&bdquo;"], [1, "&dagger;"], [0, "&Dagger;"], [0, "&bull;"], [2, "&nldr;"], [0, "&hellip;"], [9, "&permil;"], [0, "&pertenk;"], [0, "&prime;"], [0, "&Prime;"], [0, "&tprime;"], [0, "&backprime;"], [3, "&lsaquo;"], [0, "&rsaquo;"], [3, "&oline;"], [2, "&caret;"], [1, "&hybull;"], [0, "&frasl;"], [10, "&bsemi;"], [7, "&qprime;"], [7, { v: "&MediumSpace;", n: 8202, o: "&ThickSpace;" }], [0, "&NoBreak;"], [0, "&af;"], [0, "&InvisibleTimes;"], [0, "&ic;"], [72, "&euro;"], [46, "&tdot;"], [0, "&DotDot;"], [37, "&complexes;"], [2, "&incare;"], [4, "&gscr;"], [0, "&hamilt;"], [0, "&Hfr;"], [0, "&Hopf;"], [0, "&planckh;"], [0, "&hbar;"], [0, "&imagline;"], [0, "&Ifr;"], [0, "&lagran;"], [0, "&ell;"], [1, "&naturals;"], [0, "&numero;"], [0, "&copysr;"], [0, "&weierp;"], [0, "&Popf;"], [0, "&Qopf;"], [0, "&realine;"], [0, "&real;"], [0, "&reals;"], [0, "&rx;"], [3, "&trade;"], [1, "&integers;"], [2, "&mho;"], [0, "&zeetrf;"], [0, "&iiota;"], [2, "&bernou;"], [0, "&Cayleys;"], [1, "&escr;"], [0, "&Escr;"], [0, "&Fouriertrf;"], [1, "&Mellintrf;"], [0, "&order;"], [0, "&alefsym;"], [0, "&beth;"], [0, "&gimel;"], [0, "&daleth;"], [12, "&CapitalDifferentialD;"], [0, "&dd;"], [0, "&ee;"], [0, "&ii;"], [10, "&frac13;"], [0, "&frac23;"], [0, "&frac15;"], [0, "&frac25;"], [0, "&frac35;"], [0, "&frac45;"], [0, "&frac16;"], [0, "&frac56;"], [0, "&frac18;"], [0, "&frac38;"], [0, "&frac58;"], [0, "&frac78;"], [49, "&larr;"], [0, "&ShortUpArrow;"], [0, "&rarr;"], [0, "&darr;"], [0, "&harr;"], [0, "&updownarrow;"], [0, "&nwarr;"], [0, "&nearr;"], [0, "&LowerRightArrow;"], [0, "&LowerLeftArrow;"], [0, "&nlarr;"], [0, "&nrarr;"], [1, { v: "&rarrw;", n: 824, o: "&nrarrw;" }], [0, "&Larr;"], [0, "&Uarr;"], [0, "&Rarr;"], [0, "&Darr;"], [0, "&larrtl;"], [0, "&rarrtl;"], [0, "&LeftTeeArrow;"], [0, "&mapstoup;"], [0, "&map;"], [0, "&DownTeeArrow;"], [1, "&hookleftarrow;"], [0, "&hookrightarrow;"], [0, "&larrlp;"], [0, "&looparrowright;"], [0, "&harrw;"], [0, "&nharr;"], [1, "&lsh;"], [0, "&rsh;"], [0, "&ldsh;"], [0, "&rdsh;"], [1, "&crarr;"], [0, "&cularr;"], [0, "&curarr;"], [2, "&circlearrowleft;"], [0, "&circlearrowright;"], [0, "&leftharpoonup;"], [0, "&DownLeftVector;"], [0, "&RightUpVector;"], [0, "&LeftUpVector;"], [0, "&rharu;"], [0, "&DownRightVector;"], [0, "&dharr;"], [0, "&dharl;"], [0, "&RightArrowLeftArrow;"], [0, "&udarr;"], [0, "&LeftArrowRightArrow;"], [0, "&leftleftarrows;"], [0, "&upuparrows;"], [0, "&rightrightarrows;"], [0, "&ddarr;"], [0, "&leftrightharpoons;"], [0, "&Equilibrium;"], [0, "&nlArr;"], [0, "&nhArr;"], [0, "&nrArr;"], [0, "&DoubleLeftArrow;"], [0, "&DoubleUpArrow;"], [0, "&DoubleRightArrow;"], [0, "&dArr;"], [0, "&DoubleLeftRightArrow;"], [0, "&DoubleUpDownArrow;"], [0, "&nwArr;"], [0, "&neArr;"], [0, "&seArr;"], [0, "&swArr;"], [0, "&lAarr;"], [0, "&rAarr;"], [1, "&zigrarr;"], [6, "&larrb;"], [0, "&rarrb;"], [15, "&DownArrowUpArrow;"], [7, "&loarr;"], [0, "&roarr;"], [0, "&hoarr;"], [0, "&forall;"], [0, "&comp;"], [0, { v: "&part;", n: 824, o: "&npart;" }], [0, "&exist;"], [0, "&nexist;"], [0, "&empty;"], [1, "&Del;"], [0, "&Element;"], [0, "&NotElement;"], [1, "&ni;"], [0, "&notni;"], [2, "&prod;"], [0, "&coprod;"], [0, "&sum;"], [0, "&minus;"], [0, "&MinusPlus;"], [0, "&dotplus;"], [1, "&Backslash;"], [0, "&lowast;"], [0, "&compfn;"], [1, "&radic;"], [2, "&prop;"], [0, "&infin;"], [0, "&angrt;"], [0, { v: "&ang;", n: 8402, o: "&nang;" }], [0, "&angmsd;"], [0, "&angsph;"], [0, "&mid;"], [0, "&nmid;"], [0, "&DoubleVerticalBar;"], [0, "&NotDoubleVerticalBar;"], [0, "&and;"], [0, "&or;"], [0, { v: "&cap;", n: 65024, o: "&caps;" }], [0, { v: "&cup;", n: 65024, o: "&cups;" }], [0, "&int;"], [0, "&Int;"], [0, "&iiint;"], [0, "&conint;"], [0, "&Conint;"], [0, "&Cconint;"], [0, "&cwint;"], [0, "&ClockwiseContourIntegral;"], [0, "&awconint;"], [0, "&there4;"], [0, "&becaus;"], [0, "&ratio;"], [0, "&Colon;"], [0, "&dotminus;"], [1, "&mDDot;"], [0, "&homtht;"], [0, { v: "&sim;", n: 8402, o: "&nvsim;" }], [0, { v: "&backsim;", n: 817, o: "&race;" }], [0, { v: "&ac;", n: 819, o: "&acE;" }], [0, "&acd;"], [0, "&VerticalTilde;"], [0, "&NotTilde;"], [0, { v: "&eqsim;", n: 824, o: "&nesim;" }], [0, "&sime;"], [0, "&NotTildeEqual;"], [0, "&cong;"], [0, "&simne;"], [0, "&ncong;"], [0, "&ap;"], [0, "&nap;"], [0, "&ape;"], [0, { v: "&apid;", n: 824, o: "&napid;" }], [0, "&backcong;"], [0, { v: "&asympeq;", n: 8402, o: "&nvap;" }], [0, { v: "&bump;", n: 824, o: "&nbump;" }], [0, { v: "&bumpe;", n: 824, o: "&nbumpe;" }], [0, { v: "&doteq;", n: 824, o: "&nedot;" }], [0, "&doteqdot;"], [0, "&efDot;"], [0, "&erDot;"], [0, "&Assign;"], [0, "&ecolon;"], [0, "&ecir;"], [0, "&circeq;"], [1, "&wedgeq;"], [0, "&veeeq;"], [1, "&triangleq;"], [2, "&equest;"], [0, "&ne;"], [0, { v: "&Congruent;", n: 8421, o: "&bnequiv;" }], [0, "&nequiv;"], [1, { v: "&le;", n: 8402, o: "&nvle;" }], [0, { v: "&ge;", n: 8402, o: "&nvge;" }], [0, { v: "&lE;", n: 824, o: "&nlE;" }], [0, { v: "&gE;", n: 824, o: "&ngE;" }], [0, { v: "&lnE;", n: 65024, o: "&lvertneqq;" }], [0, { v: "&gnE;", n: 65024, o: "&gvertneqq;" }], [0, { v: "&ll;", n: new Map(/* #__PURE__ */ restoreDiff([[824, "&nLtv;"], [7577, "&nLt;"]])) }], [0, { v: "&gg;", n: new Map(/* #__PURE__ */ restoreDiff([[824, "&nGtv;"], [7577, "&nGt;"]])) }], [0, "&between;"], [0, "&NotCupCap;"], [0, "&nless;"], [0, "&ngt;"], [0, "&nle;"], [0, "&nge;"], [0, "&lesssim;"], [0, "&GreaterTilde;"], [0, "&nlsim;"], [0, "&ngsim;"], [0, "&LessGreater;"], [0, "&gl;"], [0, "&NotLessGreater;"], [0, "&NotGreaterLess;"], [0, "&pr;"], [0, "&sc;"], [0, "&prcue;"], [0, "&sccue;"], [0, "&PrecedesTilde;"], [0, { v: "&scsim;", n: 824, o: "&NotSucceedsTilde;" }], [0, "&NotPrecedes;"], [0, "&NotSucceeds;"], [0, { v: "&sub;", n: 8402, o: "&NotSubset;" }], [0, { v: "&sup;", n: 8402, o: "&NotSuperset;" }], [0, "&nsub;"], [0, "&nsup;"], [0, "&sube;"], [0, "&supe;"], [0, "&NotSubsetEqual;"], [0, "&NotSupersetEqual;"], [0, { v: "&subne;", n: 65024, o: "&varsubsetneq;" }], [0, { v: "&supne;", n: 65024, o: "&varsupsetneq;" }], [1, "&cupdot;"], [0, "&UnionPlus;"], [0, { v: "&sqsub;", n: 824, o: "&NotSquareSubset;" }], [0, { v: "&sqsup;", n: 824, o: "&NotSquareSuperset;" }], [0, "&sqsube;"], [0, "&sqsupe;"], [0, { v: "&sqcap;", n: 65024, o: "&sqcaps;" }], [0, { v: "&sqcup;", n: 65024, o: "&sqcups;" }], [0, "&CirclePlus;"], [0, "&CircleMinus;"], [0, "&CircleTimes;"], [0, "&osol;"], [0, "&CircleDot;"], [0, "&circledcirc;"], [0, "&circledast;"], [1, "&circleddash;"], [0, "&boxplus;"], [0, "&boxminus;"], [0, "&boxtimes;"], [0, "&dotsquare;"], [0, "&RightTee;"], [0, "&dashv;"], [0, "&DownTee;"], [0, "&bot;"], [1, "&models;"], [0, "&DoubleRightTee;"], [0, "&Vdash;"], [0, "&Vvdash;"], [0, "&VDash;"], [0, "&nvdash;"], [0, "&nvDash;"], [0, "&nVdash;"], [0, "&nVDash;"], [0, "&prurel;"], [1, "&LeftTriangle;"], [0, "&RightTriangle;"], [0, { v: "&LeftTriangleEqual;", n: 8402, o: "&nvltrie;" }], [0, { v: "&RightTriangleEqual;", n: 8402, o: "&nvrtrie;" }], [0, "&origof;"], [0, "&imof;"], [0, "&multimap;"], [0, "&hercon;"], [0, "&intcal;"], [0, "&veebar;"], [1, "&barvee;"], [0, "&angrtvb;"], [0, "&lrtri;"], [0, "&bigwedge;"], [0, "&bigvee;"], [0, "&bigcap;"], [0, "&bigcup;"], [0, "&diam;"], [0, "&sdot;"], [0, "&sstarf;"], [0, "&divideontimes;"], [0, "&bowtie;"], [0, "&ltimes;"], [0, "&rtimes;"], [0, "&leftthreetimes;"], [0, "&rightthreetimes;"], [0, "&backsimeq;"], [0, "&curlyvee;"], [0, "&curlywedge;"], [0, "&Sub;"], [0, "&Sup;"], [0, "&Cap;"], [0, "&Cup;"], [0, "&fork;"], [0, "&epar;"], [0, "&lessdot;"], [0, "&gtdot;"], [0, { v: "&Ll;", n: 824, o: "&nLl;" }], [0, { v: "&Gg;", n: 824, o: "&nGg;" }], [0, { v: "&leg;", n: 65024, o: "&lesg;" }], [0, { v: "&gel;", n: 65024, o: "&gesl;" }], [2, "&cuepr;"], [0, "&cuesc;"], [0, "&NotPrecedesSlantEqual;"], [0, "&NotSucceedsSlantEqual;"], [0, "&NotSquareSubsetEqual;"], [0, "&NotSquareSupersetEqual;"], [2, "&lnsim;"], [0, "&gnsim;"], [0, "&precnsim;"], [0, "&scnsim;"], [0, "&nltri;"], [0, "&NotRightTriangle;"], [0, "&nltrie;"], [0, "&NotRightTriangleEqual;"], [0, "&vellip;"], [0, "&ctdot;"], [0, "&utdot;"], [0, "&dtdot;"], [0, "&disin;"], [0, "&isinsv;"], [0, "&isins;"], [0, { v: "&isindot;", n: 824, o: "&notindot;" }], [0, "&notinvc;"], [0, "&notinvb;"], [1, { v: "&isinE;", n: 824, o: "&notinE;" }], [0, "&nisd;"], [0, "&xnis;"], [0, "&nis;"], [0, "&notnivc;"], [0, "&notnivb;"], [6, "&barwed;"], [0, "&Barwed;"], [1, "&lceil;"], [0, "&rceil;"], [0, "&LeftFloor;"], [0, "&rfloor;"], [0, "&drcrop;"], [0, "&dlcrop;"], [0, "&urcrop;"], [0, "&ulcrop;"], [0, "&bnot;"], [1, "&profline;"], [0, "&profsurf;"], [1, "&telrec;"], [0, "&target;"], [5, "&ulcorn;"], [0, "&urcorn;"], [0, "&dlcorn;"], [0, "&drcorn;"], [2, "&frown;"], [0, "&smile;"], [9, "&cylcty;"], [0, "&profalar;"], [7, "&topbot;"], [6, "&ovbar;"], [1, "&solbar;"], [60, "&angzarr;"], [51, "&lmoustache;"], [0, "&rmoustache;"], [2, "&OverBracket;"], [0, "&bbrk;"], [0, "&bbrktbrk;"], [37, "&OverParenthesis;"], [0, "&UnderParenthesis;"], [0, "&OverBrace;"], [0, "&UnderBrace;"], [2, "&trpezium;"], [4, "&elinters;"], [59, "&blank;"], [164, "&circledS;"], [55, "&boxh;"], [1, "&boxv;"], [9, "&boxdr;"], [3, "&boxdl;"], [3, "&boxur;"], [3, "&boxul;"], [3, "&boxvr;"], [7, "&boxvl;"], [7, "&boxhd;"], [7, "&boxhu;"], [7, "&boxvh;"], [19, "&boxH;"], [0, "&boxV;"], [0, "&boxdR;"], [0, "&boxDr;"], [0, "&boxDR;"], [0, "&boxdL;"], [0, "&boxDl;"], [0, "&boxDL;"], [0, "&boxuR;"], [0, "&boxUr;"], [0, "&boxUR;"], [0, "&boxuL;"], [0, "&boxUl;"], [0, "&boxUL;"], [0, "&boxvR;"], [0, "&boxVr;"], [0, "&boxVR;"], [0, "&boxvL;"], [0, "&boxVl;"], [0, "&boxVL;"], [0, "&boxHd;"], [0, "&boxhD;"], [0, "&boxHD;"], [0, "&boxHu;"], [0, "&boxhU;"], [0, "&boxHU;"], [0, "&boxvH;"], [0, "&boxVh;"], [0, "&boxVH;"], [19, "&uhblk;"], [3, "&lhblk;"], [3, "&block;"], [8, "&blk14;"], [0, "&blk12;"], [0, "&blk34;"], [13, "&square;"], [8, "&blacksquare;"], [0, "&EmptyVerySmallSquare;"], [1, "&rect;"], [0, "&marker;"], [2, "&fltns;"], [1, "&bigtriangleup;"], [0, "&blacktriangle;"], [0, "&triangle;"], [2, "&blacktriangleright;"], [0, "&rtri;"], [3, "&bigtriangledown;"], [0, "&blacktriangledown;"], [0, "&dtri;"], [2, "&blacktriangleleft;"], [0, "&ltri;"], [6, "&loz;"], [0, "&cir;"], [32, "&tridot;"], [2, "&bigcirc;"], [8, "&ultri;"], [0, "&urtri;"], [0, "&lltri;"], [0, "&EmptySmallSquare;"], [0, "&FilledSmallSquare;"], [8, "&bigstar;"], [0, "&star;"], [7, "&phone;"], [49, "&female;"], [1, "&male;"], [29, "&spades;"], [2, "&clubs;"], [1, "&hearts;"], [0, "&diamondsuit;"], [3, "&sung;"], [2, "&flat;"], [0, "&natural;"], [0, "&sharp;"], [163, "&check;"], [3, "&cross;"], [8, "&malt;"], [21, "&sext;"], [33, "&VerticalSeparator;"], [25, "&lbbrk;"], [0, "&rbbrk;"], [84, "&bsolhsub;"], [0, "&suphsol;"], [28, "&LeftDoubleBracket;"], [0, "&RightDoubleBracket;"], [0, "&lang;"], [0, "&rang;"], [0, "&Lang;"], [0, "&Rang;"], [0, "&loang;"], [0, "&roang;"], [7, "&longleftarrow;"], [0, "&longrightarrow;"], [0, "&longleftrightarrow;"], [0, "&DoubleLongLeftArrow;"], [0, "&DoubleLongRightArrow;"], [0, "&DoubleLongLeftRightArrow;"], [1, "&longmapsto;"], [2, "&dzigrarr;"], [258, "&nvlArr;"], [0, "&nvrArr;"], [0, "&nvHarr;"], [0, "&Map;"], [6, "&lbarr;"], [0, "&bkarow;"], [0, "&lBarr;"], [0, "&dbkarow;"], [0, "&drbkarow;"], [0, "&DDotrahd;"], [0, "&UpArrowBar;"], [0, "&DownArrowBar;"], [2, "&Rarrtl;"], [2, "&latail;"], [0, "&ratail;"], [0, "&lAtail;"], [0, "&rAtail;"], [0, "&larrfs;"], [0, "&rarrfs;"], [0, "&larrbfs;"], [0, "&rarrbfs;"], [2, "&nwarhk;"], [0, "&nearhk;"], [0, "&hksearow;"], [0, "&hkswarow;"], [0, "&nwnear;"], [0, "&nesear;"], [0, "&seswar;"], [0, "&swnwar;"], [8, { v: "&rarrc;", n: 824, o: "&nrarrc;" }], [1, "&cudarrr;"], [0, "&ldca;"], [0, "&rdca;"], [0, "&cudarrl;"], [0, "&larrpl;"], [2, "&curarrm;"], [0, "&cularrp;"], [7, "&rarrpl;"], [2, "&harrcir;"], [0, "&Uarrocir;"], [0, "&lurdshar;"], [0, "&ldrushar;"], [2, "&LeftRightVector;"], [0, "&RightUpDownVector;"], [0, "&DownLeftRightVector;"], [0, "&LeftUpDownVector;"], [0, "&LeftVectorBar;"], [0, "&RightVectorBar;"], [0, "&RightUpVectorBar;"], [0, "&RightDownVectorBar;"], [0, "&DownLeftVectorBar;"], [0, "&DownRightVectorBar;"], [0, "&LeftUpVectorBar;"], [0, "&LeftDownVectorBar;"], [0, "&LeftTeeVector;"], [0, "&RightTeeVector;"], [0, "&RightUpTeeVector;"], [0, "&RightDownTeeVector;"], [0, "&DownLeftTeeVector;"], [0, "&DownRightTeeVector;"], [0, "&LeftUpTeeVector;"], [0, "&LeftDownTeeVector;"], [0, "&lHar;"], [0, "&uHar;"], [0, "&rHar;"], [0, "&dHar;"], [0, "&luruhar;"], [0, "&ldrdhar;"], [0, "&ruluhar;"], [0, "&rdldhar;"], [0, "&lharul;"], [0, "&llhard;"], [0, "&rharul;"], [0, "&lrhard;"], [0, "&udhar;"], [0, "&duhar;"], [0, "&RoundImplies;"], [0, "&erarr;"], [0, "&simrarr;"], [0, "&larrsim;"], [0, "&rarrsim;"], [0, "&rarrap;"], [0, "&ltlarr;"], [1, "&gtrarr;"], [0, "&subrarr;"], [1, "&suplarr;"], [0, "&lfisht;"], [0, "&rfisht;"], [0, "&ufisht;"], [0, "&dfisht;"], [5, "&lopar;"], [0, "&ropar;"], [4, "&lbrke;"], [0, "&rbrke;"], [0, "&lbrkslu;"], [0, "&rbrksld;"], [0, "&lbrksld;"], [0, "&rbrkslu;"], [0, "&langd;"], [0, "&rangd;"], [0, "&lparlt;"], [0, "&rpargt;"], [0, "&gtlPar;"], [0, "&ltrPar;"], [3, "&vzigzag;"], [1, "&vangrt;"], [0, "&angrtvbd;"], [6, "&ange;"], [0, "&range;"], [0, "&dwangle;"], [0, "&uwangle;"], [0, "&angmsdaa;"], [0, "&angmsdab;"], [0, "&angmsdac;"], [0, "&angmsdad;"], [0, "&angmsdae;"], [0, "&angmsdaf;"], [0, "&angmsdag;"], [0, "&angmsdah;"], [0, "&bemptyv;"], [0, "&demptyv;"], [0, "&cemptyv;"], [0, "&raemptyv;"], [0, "&laemptyv;"], [0, "&ohbar;"], [0, "&omid;"], [0, "&opar;"], [1, "&operp;"], [1, "&olcross;"], [0, "&odsold;"], [1, "&olcir;"], [0, "&ofcir;"], [0, "&olt;"], [0, "&ogt;"], [0, "&cirscir;"], [0, "&cirE;"], [0, "&solb;"], [0, "&bsolb;"], [3, "&boxbox;"], [3, "&trisb;"], [0, "&rtriltri;"], [0, { v: "&LeftTriangleBar;", n: 824, o: "&NotLeftTriangleBar;" }], [0, { v: "&RightTriangleBar;", n: 824, o: "&NotRightTriangleBar;" }], [11, "&iinfin;"], [0, "&infintie;"], [0, "&nvinfin;"], [4, "&eparsl;"], [0, "&smeparsl;"], [0, "&eqvparsl;"], [5, "&blacklozenge;"], [8, "&RuleDelayed;"], [1, "&dsol;"], [9, "&bigodot;"], [0, "&bigoplus;"], [0, "&bigotimes;"], [1, "&biguplus;"], [1, "&bigsqcup;"], [5, "&iiiint;"], [0, "&fpartint;"], [2, "&cirfnint;"], [0, "&awint;"], [0, "&rppolint;"], [0, "&scpolint;"], [0, "&npolint;"], [0, "&pointint;"], [0, "&quatint;"], [0, "&intlarhk;"], [10, "&pluscir;"], [0, "&plusacir;"], [0, "&simplus;"], [0, "&plusdu;"], [0, "&plussim;"], [0, "&plustwo;"], [1, "&mcomma;"], [0, "&minusdu;"], [2, "&loplus;"], [0, "&roplus;"], [0, "&Cross;"], [0, "&timesd;"], [0, "&timesbar;"], [1, "&smashp;"], [0, "&lotimes;"], [0, "&rotimes;"], [0, "&otimesas;"], [0, "&Otimes;"], [0, "&odiv;"], [0, "&triplus;"], [0, "&triminus;"], [0, "&tritime;"], [0, "&intprod;"], [2, "&amalg;"], [0, "&capdot;"], [1, "&ncup;"], [0, "&ncap;"], [0, "&capand;"], [0, "&cupor;"], [0, "&cupcap;"], [0, "&capcup;"], [0, "&cupbrcap;"], [0, "&capbrcup;"], [0, "&cupcup;"], [0, "&capcap;"], [0, "&ccups;"], [0, "&ccaps;"], [2, "&ccupssm;"], [2, "&And;"], [0, "&Or;"], [0, "&andand;"], [0, "&oror;"], [0, "&orslope;"], [0, "&andslope;"], [1, "&andv;"], [0, "&orv;"], [0, "&andd;"], [0, "&ord;"], [1, "&wedbar;"], [6, "&sdote;"], [3, "&simdot;"], [2, { v: "&congdot;", n: 824, o: "&ncongdot;" }], [0, "&easter;"], [0, "&apacir;"], [0, { v: "&apE;", n: 824, o: "&napE;" }], [0, "&eplus;"], [0, "&pluse;"], [0, "&Esim;"], [0, "&Colone;"], [0, "&Equal;"], [1, "&ddotseq;"], [0, "&equivDD;"], [0, "&ltcir;"], [0, "&gtcir;"], [0, "&ltquest;"], [0, "&gtquest;"], [0, { v: "&leqslant;", n: 824, o: "&nleqslant;" }], [0, { v: "&geqslant;", n: 824, o: "&ngeqslant;" }], [0, "&lesdot;"], [0, "&gesdot;"], [0, "&lesdoto;"], [0, "&gesdoto;"], [0, "&lesdotor;"], [0, "&gesdotol;"], [0, "&lap;"], [0, "&gap;"], [0, "&lne;"], [0, "&gne;"], [0, "&lnap;"], [0, "&gnap;"], [0, "&lEg;"], [0, "&gEl;"], [0, "&lsime;"], [0, "&gsime;"], [0, "&lsimg;"], [0, "&gsiml;"], [0, "&lgE;"], [0, "&glE;"], [0, "&lesges;"], [0, "&gesles;"], [0, "&els;"], [0, "&egs;"], [0, "&elsdot;"], [0, "&egsdot;"], [0, "&el;"], [0, "&eg;"], [2, "&siml;"], [0, "&simg;"], [0, "&simlE;"], [0, "&simgE;"], [0, { v: "&LessLess;", n: 824, o: "&NotNestedLessLess;" }], [0, { v: "&GreaterGreater;", n: 824, o: "&NotNestedGreaterGreater;" }], [1, "&glj;"], [0, "&gla;"], [0, "&ltcc;"], [0, "&gtcc;"], [0, "&lescc;"], [0, "&gescc;"], [0, "&smt;"], [0, "&lat;"], [0, { v: "&smte;", n: 65024, o: "&smtes;" }], [0, { v: "&late;", n: 65024, o: "&lates;" }], [0, "&bumpE;"], [0, { v: "&PrecedesEqual;", n: 824, o: "&NotPrecedesEqual;" }], [0, { v: "&sce;", n: 824, o: "&NotSucceedsEqual;" }], [2, "&prE;"], [0, "&scE;"], [0, "&precneqq;"], [0, "&scnE;"], [0, "&prap;"], [0, "&scap;"], [0, "&precnapprox;"], [0, "&scnap;"], [0, "&Pr;"], [0, "&Sc;"], [0, "&subdot;"], [0, "&supdot;"], [0, "&subplus;"], [0, "&supplus;"], [0, "&submult;"], [0, "&supmult;"], [0, "&subedot;"], [0, "&supedot;"], [0, { v: "&subE;", n: 824, o: "&nsubE;" }], [0, { v: "&supE;", n: 824, o: "&nsupE;" }], [0, "&subsim;"], [0, "&supsim;"], [2, { v: "&subnE;", n: 65024, o: "&varsubsetneqq;" }], [0, { v: "&supnE;", n: 65024, o: "&varsupsetneqq;" }], [2, "&csub;"], [0, "&csup;"], [0, "&csube;"], [0, "&csupe;"], [0, "&subsup;"], [0, "&supsub;"], [0, "&subsub;"], [0, "&supsup;"], [0, "&suphsub;"], [0, "&supdsub;"], [0, "&forkv;"], [0, "&topfork;"], [0, "&mlcp;"], [8, "&Dashv;"], [1, "&Vdashl;"], [0, "&Barv;"], [0, "&vBar;"], [0, "&vBarv;"], [1, "&Vbar;"], [0, "&Not;"], [0, "&bNot;"], [0, "&rnmid;"], [0, "&cirmid;"], [0, "&midcir;"], [0, "&topcir;"], [0, "&nhpar;"], [0, "&parsim;"], [9, { v: "&parsl;", n: 8421, o: "&nparsl;" }], [44343, { n: new Map(/* #__PURE__ */ restoreDiff([[56476, "&Ascr;"], [1, "&Cscr;"], [0, "&Dscr;"], [2, "&Gscr;"], [2, "&Jscr;"], [0, "&Kscr;"], [2, "&Nscr;"], [0, "&Oscr;"], [0, "&Pscr;"], [0, "&Qscr;"], [1, "&Sscr;"], [0, "&Tscr;"], [0, "&Uscr;"], [0, "&Vscr;"], [0, "&Wscr;"], [0, "&Xscr;"], [0, "&Yscr;"], [0, "&Zscr;"], [0, "&ascr;"], [0, "&bscr;"], [0, "&cscr;"], [0, "&dscr;"], [1, "&fscr;"], [1, "&hscr;"], [0, "&iscr;"], [0, "&jscr;"], [0, "&kscr;"], [0, "&lscr;"], [0, "&mscr;"], [0, "&nscr;"], [1, "&pscr;"], [0, "&qscr;"], [0, "&rscr;"], [0, "&sscr;"], [0, "&tscr;"], [0, "&uscr;"], [0, "&vscr;"], [0, "&wscr;"], [0, "&xscr;"], [0, "&yscr;"], [0, "&zscr;"], [52, "&Afr;"], [0, "&Bfr;"], [1, "&Dfr;"], [0, "&Efr;"], [0, "&Ffr;"], [0, "&Gfr;"], [2, "&Jfr;"], [0, "&Kfr;"], [0, "&Lfr;"], [0, "&Mfr;"], [0, "&Nfr;"], [0, "&Ofr;"], [0, "&Pfr;"], [0, "&Qfr;"], [1, "&Sfr;"], [0, "&Tfr;"], [0, "&Ufr;"], [0, "&Vfr;"], [0, "&Wfr;"], [0, "&Xfr;"], [0, "&Yfr;"], [1, "&afr;"], [0, "&bfr;"], [0, "&cfr;"], [0, "&dfr;"], [0, "&efr;"], [0, "&ffr;"], [0, "&gfr;"], [0, "&hfr;"], [0, "&ifr;"], [0, "&jfr;"], [0, "&kfr;"], [0, "&lfr;"], [0, "&mfr;"], [0, "&nfr;"], [0, "&ofr;"], [0, "&pfr;"], [0, "&qfr;"], [0, "&rfr;"], [0, "&sfr;"], [0, "&tfr;"], [0, "&ufr;"], [0, "&vfr;"], [0, "&wfr;"], [0, "&xfr;"], [0, "&yfr;"], [0, "&zfr;"], [0, "&Aopf;"], [0, "&Bopf;"], [1, "&Dopf;"], [0, "&Eopf;"], [0, "&Fopf;"], [0, "&Gopf;"], [1, "&Iopf;"], [0, "&Jopf;"], [0, "&Kopf;"], [0, "&Lopf;"], [0, "&Mopf;"], [1, "&Oopf;"], [3, "&Sopf;"], [0, "&Topf;"], [0, "&Uopf;"], [0, "&Vopf;"], [0, "&Wopf;"], [0, "&Xopf;"], [0, "&Yopf;"], [1, "&aopf;"], [0, "&bopf;"], [0, "&copf;"], [0, "&dopf;"], [0, "&eopf;"], [0, "&fopf;"], [0, "&gopf;"], [0, "&hopf;"], [0, "&iopf;"], [0, "&jopf;"], [0, "&kopf;"], [0, "&lopf;"], [0, "&mopf;"], [0, "&nopf;"], [0, "&oopf;"], [0, "&popf;"], [0, "&qopf;"], [0, "&ropf;"], [0, "&sopf;"], [0, "&topf;"], [0, "&uopf;"], [0, "&vopf;"], [0, "&wopf;"], [0, "&xopf;"], [0, "&yopf;"], [0, "&zopf;"]])) }], [8906, "&fflig;"], [0, "&filig;"], [0, "&fllig;"], [0, "&ffilig;"], [0, "&ffllig;"]]));

var _escape = {};

(function (exports) {
Object.defineProperty(exports, "__esModule", { value: true });
exports.escapeText = exports.escapeAttribute = exports.escapeUTF8 = exports.escape = exports.encodeXML = exports.getCodePoint = exports.xmlReplacer = void 0;
exports.xmlReplacer = /["&'<>$\x80-\uFFFF]/g;
var xmlCodeMap = new Map([
    [34, "&quot;"],
    [38, "&amp;"],
    [39, "&apos;"],
    [60, "&lt;"],
    [62, "&gt;"],
]);
// For compatibility with node < 4, we wrap `codePointAt`
exports.getCodePoint = 
// eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
String.prototype.codePointAt != null
    ? function (str, index) { return str.codePointAt(index); }
    : // http://mathiasbynens.be/notes/javascript-encoding#surrogate-formulae
        function (c, index) {
            return (c.charCodeAt(index) & 0xfc00) === 0xd800
                ? (c.charCodeAt(index) - 0xd800) * 0x400 +
                    c.charCodeAt(index + 1) -
                    0xdc00 +
                    0x10000
                : c.charCodeAt(index);
        };
/**
 * Encodes all non-ASCII characters, as well as characters not valid in XML
 * documents using XML entities.
 *
 * If a character has no equivalent entity, a
 * numeric hexadecimal reference (eg. `&#xfc;`) will be used.
 */
function encodeXML(str) {
    var ret = "";
    var lastIdx = 0;
    var match;
    while ((match = exports.xmlReplacer.exec(str)) !== null) {
        var i = match.index;
        var char = str.charCodeAt(i);
        var next = xmlCodeMap.get(char);
        if (next !== undefined) {
            ret += str.substring(lastIdx, i) + next;
            lastIdx = i + 1;
        }
        else {
            ret += "".concat(str.substring(lastIdx, i), "&#x").concat((0, exports.getCodePoint)(str, i).toString(16), ";");
            // Increase by 1 if we have a surrogate pair
            lastIdx = exports.xmlReplacer.lastIndex += Number((char & 0xfc00) === 0xd800);
        }
    }
    return ret + str.substr(lastIdx);
}
exports.encodeXML = encodeXML;
/**
 * Encodes all non-ASCII characters, as well as characters not valid in XML
 * documents using numeric hexadecimal reference (eg. `&#xfc;`).
 *
 * Have a look at `escapeUTF8` if you want a more concise output at the expense
 * of reduced transportability.
 *
 * @param data String to escape.
 */
exports.escape = encodeXML;
/**
 * Creates a function that escapes all characters matched by the given regular
 * expression using the given map of characters to escape to their entities.
 *
 * @param regex Regular expression to match characters to escape.
 * @param map Map of characters to escape to their entities.
 *
 * @returns Function that escapes all characters matched by the given regular
 * expression using the given map of characters to escape to their entities.
 */
function getEscaper(regex, map) {
    return function escape(data) {
        var match;
        var lastIdx = 0;
        var result = "";
        while ((match = regex.exec(data))) {
            if (lastIdx !== match.index) {
                result += data.substring(lastIdx, match.index);
            }
            // We know that this character will be in the map.
            result += map.get(match[0].charCodeAt(0));
            // Every match will be of length 1
            lastIdx = match.index + 1;
        }
        return result + data.substring(lastIdx);
    };
}
/**
 * Encodes all characters not valid in XML documents using XML entities.
 *
 * Note that the output will be character-set dependent.
 *
 * @param data String to escape.
 */
exports.escapeUTF8 = getEscaper(/[&<>'"]/g, xmlCodeMap);
/**
 * Encodes all characters that have to be escaped in HTML attributes,
 * following {@link https://html.spec.whatwg.org/multipage/parsing.html#escapingString}.
 *
 * @param data String to escape.
 */
exports.escapeAttribute = getEscaper(/["&\u00A0]/g, new Map([
    [34, "&quot;"],
    [38, "&amp;"],
    [160, "&nbsp;"],
]));
/**
 * Encodes all characters that have to be escaped in HTML text,
 * following {@link https://html.spec.whatwg.org/multipage/parsing.html#escapingString}.
 *
 * @param data String to escape.
 */
exports.escapeText = getEscaper(/[&<>\u00A0]/g, new Map([
    [38, "&amp;"],
    [60, "&lt;"],
    [62, "&gt;"],
    [160, "&nbsp;"],
]));

}(_escape));

var __importDefault$1 = (commonjsGlobal && commonjsGlobal.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(encode, "__esModule", { value: true });
encode.encodeNonAsciiHTML = encode.encodeHTML = void 0;
var encode_html_js_1 = __importDefault$1(encodeHtml);
var escape_js_1 = _escape;
var htmlReplacer = /[\t\n!-,./:-@[-`\f{-}$\x80-\uFFFF]/g;
/**
 * Encodes all characters in the input using HTML entities. This includes
 * characters that are valid ASCII characters in HTML documents, such as `#`.
 *
 * To get a more compact output, consider using the `encodeNonAsciiHTML`
 * function, which will only encode characters that are not valid in HTML
 * documents, as well as non-ASCII characters.
 *
 * If a character has no equivalent entity, a numeric hexadecimal reference
 * (eg. `&#xfc;`) will be used.
 */
function encodeHTML(data) {
    return encodeHTMLTrieRe(htmlReplacer, data);
}
encode.encodeHTML = encodeHTML;
/**
 * Encodes all non-ASCII characters, as well as characters not valid in HTML
 * documents using HTML entities. This function will not encode characters that
 * are valid in HTML documents, such as `#`.
 *
 * If a character has no equivalent entity, a numeric hexadecimal reference
 * (eg. `&#xfc;`) will be used.
 */
function encodeNonAsciiHTML(data) {
    return encodeHTMLTrieRe(escape_js_1.xmlReplacer, data);
}
encode.encodeNonAsciiHTML = encodeNonAsciiHTML;
function encodeHTMLTrieRe(regExp, str) {
    var ret = "";
    var lastIdx = 0;
    var match;
    while ((match = regExp.exec(str)) !== null) {
        var i = match.index;
        ret += str.substring(lastIdx, i);
        var char = str.charCodeAt(i);
        var next = encode_html_js_1.default.get(char);
        if (typeof next === "object") {
            // We are in a branch. Try to match the next char.
            if (i + 1 < str.length) {
                var nextChar = str.charCodeAt(i + 1);
                var value = typeof next.n === "number"
                    ? next.n === nextChar
                        ? next.o
                        : undefined
                    : next.n.get(nextChar);
                if (value !== undefined) {
                    ret += value;
                    lastIdx = regExp.lastIndex += 1;
                    continue;
                }
            }
            next = next.v;
        }
        // We might have a tree node without a value; skip and use a numeric entity.
        if (next !== undefined) {
            ret += next;
            lastIdx = i + 1;
        }
        else {
            var cp = (0, escape_js_1.getCodePoint)(str, i);
            ret += "&#x".concat(cp.toString(16), ";");
            // Increase by 1 if we have a surrogate pair
            lastIdx = regExp.lastIndex += Number(cp !== char);
        }
    }
    return ret + str.substr(lastIdx);
}

(function (exports) {
Object.defineProperty(exports, "__esModule", { value: true });
exports.decodeXMLStrict = exports.decodeHTML5Strict = exports.decodeHTML4Strict = exports.decodeHTML5 = exports.decodeHTML4 = exports.decodeHTMLAttribute = exports.decodeHTMLStrict = exports.decodeHTML = exports.decodeXML = exports.DecodingMode = exports.EntityDecoder = exports.encodeHTML5 = exports.encodeHTML4 = exports.encodeNonAsciiHTML = exports.encodeHTML = exports.escapeText = exports.escapeAttribute = exports.escapeUTF8 = exports.escape = exports.encodeXML = exports.encode = exports.decodeStrict = exports.decode = exports.EncodingMode = exports.EntityLevel = void 0;
var decode_js_1 = decode;
var encode_js_1 = encode;
var escape_js_1 = _escape;
/** The level of entities to support. */
var EntityLevel;
(function (EntityLevel) {
    /** Support only XML entities. */
    EntityLevel[EntityLevel["XML"] = 0] = "XML";
    /** Support HTML entities, which are a superset of XML entities. */
    EntityLevel[EntityLevel["HTML"] = 1] = "HTML";
})(EntityLevel = exports.EntityLevel || (exports.EntityLevel = {}));
var EncodingMode;
(function (EncodingMode) {
    /**
     * The output is UTF-8 encoded. Only characters that need escaping within
     * XML will be escaped.
     */
    EncodingMode[EncodingMode["UTF8"] = 0] = "UTF8";
    /**
     * The output consists only of ASCII characters. Characters that need
     * escaping within HTML, and characters that aren't ASCII characters will
     * be escaped.
     */
    EncodingMode[EncodingMode["ASCII"] = 1] = "ASCII";
    /**
     * Encode all characters that have an equivalent entity, as well as all
     * characters that are not ASCII characters.
     */
    EncodingMode[EncodingMode["Extensive"] = 2] = "Extensive";
    /**
     * Encode all characters that have to be escaped in HTML attributes,
     * following {@link https://html.spec.whatwg.org/multipage/parsing.html#escapingString}.
     */
    EncodingMode[EncodingMode["Attribute"] = 3] = "Attribute";
    /**
     * Encode all characters that have to be escaped in HTML text,
     * following {@link https://html.spec.whatwg.org/multipage/parsing.html#escapingString}.
     */
    EncodingMode[EncodingMode["Text"] = 4] = "Text";
})(EncodingMode = exports.EncodingMode || (exports.EncodingMode = {}));
/**
 * Decodes a string with entities.
 *
 * @param data String to decode.
 * @param options Decoding options.
 */
function decode$1(data, options) {
    if (options === void 0) { options = EntityLevel.XML; }
    var level = typeof options === "number" ? options : options.level;
    if (level === EntityLevel.HTML) {
        var mode = typeof options === "object" ? options.mode : undefined;
        return (0, decode_js_1.decodeHTML)(data, mode);
    }
    return (0, decode_js_1.decodeXML)(data);
}
exports.decode = decode$1;
/**
 * Decodes a string with entities. Does not allow missing trailing semicolons for entities.
 *
 * @param data String to decode.
 * @param options Decoding options.
 * @deprecated Use `decode` with the `mode` set to `Strict`.
 */
function decodeStrict(data, options) {
    var _a;
    if (options === void 0) { options = EntityLevel.XML; }
    var opts = typeof options === "number" ? { level: options } : options;
    (_a = opts.mode) !== null && _a !== void 0 ? _a : (opts.mode = decode_js_1.DecodingMode.Strict);
    return decode$1(data, opts);
}
exports.decodeStrict = decodeStrict;
/**
 * Encodes a string with entities.
 *
 * @param data String to encode.
 * @param options Encoding options.
 */
function encode$1(data, options) {
    if (options === void 0) { options = EntityLevel.XML; }
    var opts = typeof options === "number" ? { level: options } : options;
    // Mode `UTF8` just escapes XML entities
    if (opts.mode === EncodingMode.UTF8)
        return (0, escape_js_1.escapeUTF8)(data);
    if (opts.mode === EncodingMode.Attribute)
        return (0, escape_js_1.escapeAttribute)(data);
    if (opts.mode === EncodingMode.Text)
        return (0, escape_js_1.escapeText)(data);
    if (opts.level === EntityLevel.HTML) {
        if (opts.mode === EncodingMode.ASCII) {
            return (0, encode_js_1.encodeNonAsciiHTML)(data);
        }
        return (0, encode_js_1.encodeHTML)(data);
    }
    // ASCII and Extensive are equivalent
    return (0, escape_js_1.encodeXML)(data);
}
exports.encode = encode$1;
var escape_js_2 = _escape;
Object.defineProperty(exports, "encodeXML", { enumerable: true, get: function () { return escape_js_2.encodeXML; } });
Object.defineProperty(exports, "escape", { enumerable: true, get: function () { return escape_js_2.escape; } });
Object.defineProperty(exports, "escapeUTF8", { enumerable: true, get: function () { return escape_js_2.escapeUTF8; } });
Object.defineProperty(exports, "escapeAttribute", { enumerable: true, get: function () { return escape_js_2.escapeAttribute; } });
Object.defineProperty(exports, "escapeText", { enumerable: true, get: function () { return escape_js_2.escapeText; } });
var encode_js_2 = encode;
Object.defineProperty(exports, "encodeHTML", { enumerable: true, get: function () { return encode_js_2.encodeHTML; } });
Object.defineProperty(exports, "encodeNonAsciiHTML", { enumerable: true, get: function () { return encode_js_2.encodeNonAsciiHTML; } });
// Legacy aliases (deprecated)
Object.defineProperty(exports, "encodeHTML4", { enumerable: true, get: function () { return encode_js_2.encodeHTML; } });
Object.defineProperty(exports, "encodeHTML5", { enumerable: true, get: function () { return encode_js_2.encodeHTML; } });
var decode_js_2 = decode;
Object.defineProperty(exports, "EntityDecoder", { enumerable: true, get: function () { return decode_js_2.EntityDecoder; } });
Object.defineProperty(exports, "DecodingMode", { enumerable: true, get: function () { return decode_js_2.DecodingMode; } });
Object.defineProperty(exports, "decodeXML", { enumerable: true, get: function () { return decode_js_2.decodeXML; } });
Object.defineProperty(exports, "decodeHTML", { enumerable: true, get: function () { return decode_js_2.decodeHTML; } });
Object.defineProperty(exports, "decodeHTMLStrict", { enumerable: true, get: function () { return decode_js_2.decodeHTMLStrict; } });
Object.defineProperty(exports, "decodeHTMLAttribute", { enumerable: true, get: function () { return decode_js_2.decodeHTMLAttribute; } });
// Legacy aliases (deprecated)
Object.defineProperty(exports, "decodeHTML4", { enumerable: true, get: function () { return decode_js_2.decodeHTML; } });
Object.defineProperty(exports, "decodeHTML5", { enumerable: true, get: function () { return decode_js_2.decodeHTML; } });
Object.defineProperty(exports, "decodeHTML4Strict", { enumerable: true, get: function () { return decode_js_2.decodeHTMLStrict; } });
Object.defineProperty(exports, "decodeHTML5Strict", { enumerable: true, get: function () { return decode_js_2.decodeHTMLStrict; } });
Object.defineProperty(exports, "decodeXMLStrict", { enumerable: true, get: function () { return decode_js_2.decodeXML; } });

}(lib$1));

var foreignNames = {};

Object.defineProperty(foreignNames, "__esModule", { value: true });
foreignNames.attributeNames = foreignNames.elementNames = void 0;
foreignNames.elementNames = new Map([
    "altGlyph",
    "altGlyphDef",
    "altGlyphItem",
    "animateColor",
    "animateMotion",
    "animateTransform",
    "clipPath",
    "feBlend",
    "feColorMatrix",
    "feComponentTransfer",
    "feComposite",
    "feConvolveMatrix",
    "feDiffuseLighting",
    "feDisplacementMap",
    "feDistantLight",
    "feDropShadow",
    "feFlood",
    "feFuncA",
    "feFuncB",
    "feFuncG",
    "feFuncR",
    "feGaussianBlur",
    "feImage",
    "feMerge",
    "feMergeNode",
    "feMorphology",
    "feOffset",
    "fePointLight",
    "feSpecularLighting",
    "feSpotLight",
    "feTile",
    "feTurbulence",
    "foreignObject",
    "glyphRef",
    "linearGradient",
    "radialGradient",
    "textPath",
].map(function (val) { return [val.toLowerCase(), val]; }));
foreignNames.attributeNames = new Map([
    "definitionURL",
    "attributeName",
    "attributeType",
    "baseFrequency",
    "baseProfile",
    "calcMode",
    "clipPathUnits",
    "diffuseConstant",
    "edgeMode",
    "filterUnits",
    "glyphRef",
    "gradientTransform",
    "gradientUnits",
    "kernelMatrix",
    "kernelUnitLength",
    "keyPoints",
    "keySplines",
    "keyTimes",
    "lengthAdjust",
    "limitingConeAngle",
    "markerHeight",
    "markerUnits",
    "markerWidth",
    "maskContentUnits",
    "maskUnits",
    "numOctaves",
    "pathLength",
    "patternContentUnits",
    "patternTransform",
    "patternUnits",
    "pointsAtX",
    "pointsAtY",
    "pointsAtZ",
    "preserveAlpha",
    "preserveAspectRatio",
    "primitiveUnits",
    "refX",
    "refY",
    "repeatCount",
    "repeatDur",
    "requiredExtensions",
    "requiredFeatures",
    "specularConstant",
    "specularExponent",
    "spreadMethod",
    "startOffset",
    "stdDeviation",
    "stitchTiles",
    "surfaceScale",
    "systemLanguage",
    "tableValues",
    "targetX",
    "targetY",
    "textLength",
    "viewBox",
    "viewTarget",
    "xChannelSelector",
    "yChannelSelector",
    "zoomAndPan",
].map(function (val) { return [val.toLowerCase(), val]; }));

var __assign = (commonjsGlobal && commonjsGlobal.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (commonjsGlobal && commonjsGlobal.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (commonjsGlobal && commonjsGlobal.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (commonjsGlobal && commonjsGlobal.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(lib$2, "__esModule", { value: true });
lib$2.render = void 0;
/*
 * Module dependencies
 */
var ElementType = __importStar(lib$5);
var entities_1 = lib$1;
/**
 * Mixed-case SVG and MathML tags & attributes
 * recognized by the HTML parser.
 *
 * @see https://html.spec.whatwg.org/multipage/parsing.html#parsing-main-inforeign
 */
var foreignNames_js_1 = foreignNames;
var unencodedElements = new Set([
    "style",
    "script",
    "xmp",
    "iframe",
    "noembed",
    "noframes",
    "plaintext",
    "noscript",
]);
function replaceQuotes(value) {
    return value.replace(/"/g, "&quot;");
}
/**
 * Format attributes
 */
function formatAttributes(attributes, opts) {
    var _a;
    if (!attributes)
        return;
    var encode = ((_a = opts.encodeEntities) !== null && _a !== void 0 ? _a : opts.decodeEntities) === false
        ? replaceQuotes
        : opts.xmlMode || opts.encodeEntities !== "utf8"
            ? entities_1.encodeXML
            : entities_1.escapeAttribute;
    return Object.keys(attributes)
        .map(function (key) {
        var _a, _b;
        var value = (_a = attributes[key]) !== null && _a !== void 0 ? _a : "";
        if (opts.xmlMode === "foreign") {
            /* Fix up mixed-case attribute names */
            key = (_b = foreignNames_js_1.attributeNames.get(key)) !== null && _b !== void 0 ? _b : key;
        }
        if (!opts.emptyAttrs && !opts.xmlMode && value === "") {
            return key;
        }
        return "".concat(key, "=\"").concat(encode(value), "\"");
    })
        .join(" ");
}
/**
 * Self-enclosing tags
 */
var singleTag = new Set([
    "area",
    "base",
    "basefont",
    "br",
    "col",
    "command",
    "embed",
    "frame",
    "hr",
    "img",
    "input",
    "isindex",
    "keygen",
    "link",
    "meta",
    "param",
    "source",
    "track",
    "wbr",
]);
/**
 * Renders a DOM node or an array of DOM nodes to a string.
 *
 * Can be thought of as the equivalent of the `outerHTML` of the passed node(s).
 *
 * @param node Node to be rendered.
 * @param options Changes serialization behavior
 */
function render(node, options) {
    if (options === void 0) { options = {}; }
    var nodes = "length" in node ? node : [node];
    var output = "";
    for (var i = 0; i < nodes.length; i++) {
        output += renderNode(nodes[i], options);
    }
    return output;
}
lib$2.render = render;
lib$2.default = render;
function renderNode(node, options) {
    switch (node.type) {
        case ElementType.Root:
            return render(node.children, options);
        // @ts-expect-error We don't use `Doctype` yet
        case ElementType.Doctype:
        case ElementType.Directive:
            return renderDirective(node);
        case ElementType.Comment:
            return renderComment(node);
        case ElementType.CDATA:
            return renderCdata(node);
        case ElementType.Script:
        case ElementType.Style:
        case ElementType.Tag:
            return renderTag(node, options);
        case ElementType.Text:
            return renderText(node, options);
    }
}
var foreignModeIntegrationPoints = new Set([
    "mi",
    "mo",
    "mn",
    "ms",
    "mtext",
    "annotation-xml",
    "foreignObject",
    "desc",
    "title",
]);
var foreignElements = new Set(["svg", "math"]);
function renderTag(elem, opts) {
    var _a;
    // Handle SVG / MathML in HTML
    if (opts.xmlMode === "foreign") {
        /* Fix up mixed-case element names */
        elem.name = (_a = foreignNames_js_1.elementNames.get(elem.name)) !== null && _a !== void 0 ? _a : elem.name;
        /* Exit foreign mode at integration points */
        if (elem.parent &&
            foreignModeIntegrationPoints.has(elem.parent.name)) {
            opts = __assign(__assign({}, opts), { xmlMode: false });
        }
    }
    if (!opts.xmlMode && foreignElements.has(elem.name)) {
        opts = __assign(__assign({}, opts), { xmlMode: "foreign" });
    }
    var tag = "<".concat(elem.name);
    var attribs = formatAttributes(elem.attribs, opts);
    if (attribs) {
        tag += " ".concat(attribs);
    }
    if (elem.children.length === 0 &&
        (opts.xmlMode
            ? // In XML mode or foreign mode, and user hasn't explicitly turned off self-closing tags
                opts.selfClosingTags !== false
            : // User explicitly asked for self-closing tags, even in HTML mode
                opts.selfClosingTags && singleTag.has(elem.name))) {
        if (!opts.xmlMode)
            tag += " ";
        tag += "/>";
    }
    else {
        tag += ">";
        if (elem.children.length > 0) {
            tag += render(elem.children, opts);
        }
        if (opts.xmlMode || !singleTag.has(elem.name)) {
            tag += "</".concat(elem.name, ">");
        }
    }
    return tag;
}
function renderDirective(elem) {
    return "<".concat(elem.data, ">");
}
function renderText(elem, opts) {
    var _a;
    var data = elem.data || "";
    // If entities weren't decoded, no need to encode them back
    if (((_a = opts.encodeEntities) !== null && _a !== void 0 ? _a : opts.decodeEntities) !== false &&
        !(!opts.xmlMode &&
            elem.parent &&
            unencodedElements.has(elem.parent.name))) {
        data =
            opts.xmlMode || opts.encodeEntities !== "utf8"
                ? (0, entities_1.encodeXML)(data)
                : (0, entities_1.escapeText)(data);
    }
    return data;
}
function renderCdata(elem) {
    return "<![CDATA[".concat(elem.children[0].data, "]]>");
}
function renderComment(elem) {
    return "<!--".concat(elem.data, "-->");
}

var __importDefault = (commonjsGlobal && commonjsGlobal.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(stringify, "__esModule", { value: true });
stringify.innerText = stringify.textContent = stringify.getText = stringify.getInnerHTML = stringify.getOuterHTML = void 0;
var domhandler_1$3 = lib$6;
var dom_serializer_1 = __importDefault(lib$2);
var domelementtype_1 = lib$5;
/**
 * @category Stringify
 * @deprecated Use the `dom-serializer` module directly.
 * @param node Node to get the outer HTML of.
 * @param options Options for serialization.
 * @returns `node`'s outer HTML.
 */
function getOuterHTML(node, options) {
    return (0, dom_serializer_1.default)(node, options);
}
stringify.getOuterHTML = getOuterHTML;
/**
 * @category Stringify
 * @deprecated Use the `dom-serializer` module directly.
 * @param node Node to get the inner HTML of.
 * @param options Options for serialization.
 * @returns `node`'s inner HTML.
 */
function getInnerHTML(node, options) {
    return (0, domhandler_1$3.hasChildren)(node)
        ? node.children.map(function (node) { return getOuterHTML(node, options); }).join("")
        : "";
}
stringify.getInnerHTML = getInnerHTML;
/**
 * Get a node's inner text. Same as `textContent`, but inserts newlines for `<br>` tags. Ignores comments.
 *
 * @category Stringify
 * @deprecated Use `textContent` instead.
 * @param node Node to get the inner text of.
 * @returns `node`'s inner text.
 */
function getText(node) {
    if (Array.isArray(node))
        return node.map(getText).join("");
    if ((0, domhandler_1$3.isTag)(node))
        return node.name === "br" ? "\n" : getText(node.children);
    if ((0, domhandler_1$3.isCDATA)(node))
        return getText(node.children);
    if ((0, domhandler_1$3.isText)(node))
        return node.data;
    return "";
}
stringify.getText = getText;
/**
 * Get a node's text content. Ignores comments.
 *
 * @category Stringify
 * @param node Node to get the text content of.
 * @returns `node`'s text content.
 * @see {@link https://developer.mozilla.org/en-US/docs/Web/API/Node/textContent}
 */
function textContent(node) {
    if (Array.isArray(node))
        return node.map(textContent).join("");
    if ((0, domhandler_1$3.hasChildren)(node) && !(0, domhandler_1$3.isComment)(node)) {
        return textContent(node.children);
    }
    if ((0, domhandler_1$3.isText)(node))
        return node.data;
    return "";
}
stringify.textContent = textContent;
/**
 * Get a node's inner text, ignoring `<script>` and `<style>` tags. Ignores comments.
 *
 * @category Stringify
 * @param node Node to get the inner text of.
 * @returns `node`'s inner text.
 * @see {@link https://developer.mozilla.org/en-US/docs/Web/API/Node/innerText}
 */
function innerText(node) {
    if (Array.isArray(node))
        return node.map(innerText).join("");
    if ((0, domhandler_1$3.hasChildren)(node) && (node.type === domelementtype_1.ElementType.Tag || (0, domhandler_1$3.isCDATA)(node))) {
        return innerText(node.children);
    }
    if ((0, domhandler_1$3.isText)(node))
        return node.data;
    return "";
}
stringify.innerText = innerText;

var traversal = {};

Object.defineProperty(traversal, "__esModule", { value: true });
traversal.prevElementSibling = traversal.nextElementSibling = traversal.getName = traversal.hasAttrib = traversal.getAttributeValue = traversal.getSiblings = traversal.getParent = traversal.getChildren = void 0;
var domhandler_1$2 = lib$6;
/**
 * Get a node's children.
 *
 * @category Traversal
 * @param elem Node to get the children of.
 * @returns `elem`'s children, or an empty array.
 */
function getChildren(elem) {
    return (0, domhandler_1$2.hasChildren)(elem) ? elem.children : [];
}
traversal.getChildren = getChildren;
/**
 * Get a node's parent.
 *
 * @category Traversal
 * @param elem Node to get the parent of.
 * @returns `elem`'s parent node, or `null` if `elem` is a root node.
 */
function getParent(elem) {
    return elem.parent || null;
}
traversal.getParent = getParent;
/**
 * Gets an elements siblings, including the element itself.
 *
 * Attempts to get the children through the element's parent first. If we don't
 * have a parent (the element is a root node), we walk the element's `prev` &
 * `next` to get all remaining nodes.
 *
 * @category Traversal
 * @param elem Element to get the siblings of.
 * @returns `elem`'s siblings, including `elem`.
 */
function getSiblings(elem) {
    var _a, _b;
    var parent = getParent(elem);
    if (parent != null)
        return getChildren(parent);
    var siblings = [elem];
    var prev = elem.prev, next = elem.next;
    while (prev != null) {
        siblings.unshift(prev);
        (_a = prev, prev = _a.prev);
    }
    while (next != null) {
        siblings.push(next);
        (_b = next, next = _b.next);
    }
    return siblings;
}
traversal.getSiblings = getSiblings;
/**
 * Gets an attribute from an element.
 *
 * @category Traversal
 * @param elem Element to check.
 * @param name Attribute name to retrieve.
 * @returns The element's attribute value, or `undefined`.
 */
function getAttributeValue(elem, name) {
    var _a;
    return (_a = elem.attribs) === null || _a === void 0 ? void 0 : _a[name];
}
traversal.getAttributeValue = getAttributeValue;
/**
 * Checks whether an element has an attribute.
 *
 * @category Traversal
 * @param elem Element to check.
 * @param name Attribute name to look for.
 * @returns Returns whether `elem` has the attribute `name`.
 */
function hasAttrib(elem, name) {
    return (elem.attribs != null &&
        Object.prototype.hasOwnProperty.call(elem.attribs, name) &&
        elem.attribs[name] != null);
}
traversal.hasAttrib = hasAttrib;
/**
 * Get the tag name of an element.
 *
 * @category Traversal
 * @param elem The element to get the name for.
 * @returns The tag name of `elem`.
 */
function getName(elem) {
    return elem.name;
}
traversal.getName = getName;
/**
 * Returns the next element sibling of a node.
 *
 * @category Traversal
 * @param elem The element to get the next sibling of.
 * @returns `elem`'s next sibling that is a tag, or `null` if there is no next
 * sibling.
 */
function nextElementSibling(elem) {
    var _a;
    var next = elem.next;
    while (next !== null && !(0, domhandler_1$2.isTag)(next))
        (_a = next, next = _a.next);
    return next;
}
traversal.nextElementSibling = nextElementSibling;
/**
 * Returns the previous element sibling of a node.
 *
 * @category Traversal
 * @param elem The element to get the previous sibling of.
 * @returns `elem`'s previous sibling that is a tag, or `null` if there is no
 * previous sibling.
 */
function prevElementSibling(elem) {
    var _a;
    var prev = elem.prev;
    while (prev !== null && !(0, domhandler_1$2.isTag)(prev))
        (_a = prev, prev = _a.prev);
    return prev;
}
traversal.prevElementSibling = prevElementSibling;

var manipulation = {};

Object.defineProperty(manipulation, "__esModule", { value: true });
manipulation.prepend = manipulation.prependChild = manipulation.append = manipulation.appendChild = manipulation.replaceElement = manipulation.removeElement = void 0;
/**
 * Remove an element from the dom
 *
 * @category Manipulation
 * @param elem The element to be removed
 */
function removeElement(elem) {
    if (elem.prev)
        elem.prev.next = elem.next;
    if (elem.next)
        elem.next.prev = elem.prev;
    if (elem.parent) {
        var childs = elem.parent.children;
        var childsIndex = childs.lastIndexOf(elem);
        if (childsIndex >= 0) {
            childs.splice(childsIndex, 1);
        }
    }
    elem.next = null;
    elem.prev = null;
    elem.parent = null;
}
manipulation.removeElement = removeElement;
/**
 * Replace an element in the dom
 *
 * @category Manipulation
 * @param elem The element to be replaced
 * @param replacement The element to be added
 */
function replaceElement(elem, replacement) {
    var prev = (replacement.prev = elem.prev);
    if (prev) {
        prev.next = replacement;
    }
    var next = (replacement.next = elem.next);
    if (next) {
        next.prev = replacement;
    }
    var parent = (replacement.parent = elem.parent);
    if (parent) {
        var childs = parent.children;
        childs[childs.lastIndexOf(elem)] = replacement;
        elem.parent = null;
    }
}
manipulation.replaceElement = replaceElement;
/**
 * Append a child to an element.
 *
 * @category Manipulation
 * @param parent The element to append to.
 * @param child The element to be added as a child.
 */
function appendChild(parent, child) {
    removeElement(child);
    child.next = null;
    child.parent = parent;
    if (parent.children.push(child) > 1) {
        var sibling = parent.children[parent.children.length - 2];
        sibling.next = child;
        child.prev = sibling;
    }
    else {
        child.prev = null;
    }
}
manipulation.appendChild = appendChild;
/**
 * Append an element after another.
 *
 * @category Manipulation
 * @param elem The element to append after.
 * @param next The element be added.
 */
function append(elem, next) {
    removeElement(next);
    var parent = elem.parent;
    var currNext = elem.next;
    next.next = currNext;
    next.prev = elem;
    elem.next = next;
    next.parent = parent;
    if (currNext) {
        currNext.prev = next;
        if (parent) {
            var childs = parent.children;
            childs.splice(childs.lastIndexOf(currNext), 0, next);
        }
    }
    else if (parent) {
        parent.children.push(next);
    }
}
manipulation.append = append;
/**
 * Prepend a child to an element.
 *
 * @category Manipulation
 * @param parent The element to prepend before.
 * @param child The element to be added as a child.
 */
function prependChild(parent, child) {
    removeElement(child);
    child.parent = parent;
    child.prev = null;
    if (parent.children.unshift(child) !== 1) {
        var sibling = parent.children[1];
        sibling.prev = child;
        child.next = sibling;
    }
    else {
        child.next = null;
    }
}
manipulation.prependChild = prependChild;
/**
 * Prepend an element before another.
 *
 * @category Manipulation
 * @param elem The element to prepend before.
 * @param prev The element be added.
 */
function prepend(elem, prev) {
    removeElement(prev);
    var parent = elem.parent;
    if (parent) {
        var childs = parent.children;
        childs.splice(childs.indexOf(elem), 0, prev);
    }
    if (elem.prev) {
        elem.prev.next = prev;
    }
    prev.parent = parent;
    prev.prev = elem.prev;
    prev.next = elem;
    elem.prev = prev;
}
manipulation.prepend = prepend;

var querying = {};

Object.defineProperty(querying, "__esModule", { value: true });
querying.findAll = querying.existsOne = querying.findOne = querying.findOneChild = querying.find = querying.filter = void 0;
var domhandler_1$1 = lib$6;
/**
 * Search a node and its children for nodes passing a test function. If `node` is not an array, it will be wrapped in one.
 *
 * @category Querying
 * @param test Function to test nodes on.
 * @param node Node to search. Will be included in the result set if it matches.
 * @param recurse Also consider child nodes.
 * @param limit Maximum number of nodes to return.
 * @returns All nodes passing `test`.
 */
function filter(test, node, recurse, limit) {
    if (recurse === void 0) { recurse = true; }
    if (limit === void 0) { limit = Infinity; }
    return find(test, Array.isArray(node) ? node : [node], recurse, limit);
}
querying.filter = filter;
/**
 * Search an array of nodes and their children for nodes passing a test function.
 *
 * @category Querying
 * @param test Function to test nodes on.
 * @param nodes Array of nodes to search.
 * @param recurse Also consider child nodes.
 * @param limit Maximum number of nodes to return.
 * @returns All nodes passing `test`.
 */
function find(test, nodes, recurse, limit) {
    var result = [];
    /** Stack of the arrays we are looking at. */
    var nodeStack = [nodes];
    /** Stack of the indices within the arrays. */
    var indexStack = [0];
    for (;;) {
        // First, check if the current array has any more elements to look at.
        if (indexStack[0] >= nodeStack[0].length) {
            // If we have no more arrays to look at, we are done.
            if (indexStack.length === 1) {
                return result;
            }
            // Otherwise, remove the current array from the stack.
            nodeStack.shift();
            indexStack.shift();
            // Loop back to the start to continue with the next array.
            continue;
        }
        var elem = nodeStack[0][indexStack[0]++];
        if (test(elem)) {
            result.push(elem);
            if (--limit <= 0)
                return result;
        }
        if (recurse && (0, domhandler_1$1.hasChildren)(elem) && elem.children.length > 0) {
            /*
             * Add the children to the stack. We are depth-first, so this is
             * the next array we look at.
             */
            indexStack.unshift(0);
            nodeStack.unshift(elem.children);
        }
    }
}
querying.find = find;
/**
 * Finds the first element inside of an array that matches a test function. This is an alias for `Array.prototype.find`.
 *
 * @category Querying
 * @param test Function to test nodes on.
 * @param nodes Array of nodes to search.
 * @returns The first node in the array that passes `test`.
 * @deprecated Use `Array.prototype.find` directly.
 */
function findOneChild(test, nodes) {
    return nodes.find(test);
}
querying.findOneChild = findOneChild;
/**
 * Finds one element in a tree that passes a test.
 *
 * @category Querying
 * @param test Function to test nodes on.
 * @param nodes Node or array of nodes to search.
 * @param recurse Also consider child nodes.
 * @returns The first node that passes `test`.
 */
function findOne(test, nodes, recurse) {
    if (recurse === void 0) { recurse = true; }
    var elem = null;
    for (var i = 0; i < nodes.length && !elem; i++) {
        var node = nodes[i];
        if (!(0, domhandler_1$1.isTag)(node)) {
            continue;
        }
        else if (test(node)) {
            elem = node;
        }
        else if (recurse && node.children.length > 0) {
            elem = findOne(test, node.children, true);
        }
    }
    return elem;
}
querying.findOne = findOne;
/**
 * Checks if a tree of nodes contains at least one node passing a test.
 *
 * @category Querying
 * @param test Function to test nodes on.
 * @param nodes Array of nodes to search.
 * @returns Whether a tree of nodes contains at least one node passing the test.
 */
function existsOne(test, nodes) {
    return nodes.some(function (checked) {
        return (0, domhandler_1$1.isTag)(checked) &&
            (test(checked) || existsOne(test, checked.children));
    });
}
querying.existsOne = existsOne;
/**
 * Search an array of nodes and their children for elements passing a test function.
 *
 * Same as `find`, but limited to elements and with less options, leading to reduced complexity.
 *
 * @category Querying
 * @param test Function to test nodes on.
 * @param nodes Array of nodes to search.
 * @returns All nodes passing `test`.
 */
function findAll(test, nodes) {
    var result = [];
    var nodeStack = [nodes];
    var indexStack = [0];
    for (;;) {
        if (indexStack[0] >= nodeStack[0].length) {
            if (nodeStack.length === 1) {
                return result;
            }
            // Otherwise, remove the current array from the stack.
            nodeStack.shift();
            indexStack.shift();
            // Loop back to the start to continue with the next array.
            continue;
        }
        var elem = nodeStack[0][indexStack[0]++];
        if (!(0, domhandler_1$1.isTag)(elem))
            continue;
        if (test(elem))
            result.push(elem);
        if (elem.children.length > 0) {
            indexStack.unshift(0);
            nodeStack.unshift(elem.children);
        }
    }
}
querying.findAll = findAll;

var legacy = {};

Object.defineProperty(legacy, "__esModule", { value: true });
legacy.getElementsByTagType = legacy.getElementsByTagName = legacy.getElementById = legacy.getElements = legacy.testElement = void 0;
var domhandler_1 = lib$6;
var querying_js_1 = querying;
/**
 * A map of functions to check nodes against.
 */
var Checks = {
    tag_name: function (name) {
        if (typeof name === "function") {
            return function (elem) { return (0, domhandler_1.isTag)(elem) && name(elem.name); };
        }
        else if (name === "*") {
            return domhandler_1.isTag;
        }
        return function (elem) { return (0, domhandler_1.isTag)(elem) && elem.name === name; };
    },
    tag_type: function (type) {
        if (typeof type === "function") {
            return function (elem) { return type(elem.type); };
        }
        return function (elem) { return elem.type === type; };
    },
    tag_contains: function (data) {
        if (typeof data === "function") {
            return function (elem) { return (0, domhandler_1.isText)(elem) && data(elem.data); };
        }
        return function (elem) { return (0, domhandler_1.isText)(elem) && elem.data === data; };
    },
};
/**
 * Returns a function to check whether a node has an attribute with a particular
 * value.
 *
 * @param attrib Attribute to check.
 * @param value Attribute value to look for.
 * @returns A function to check whether the a node has an attribute with a
 *   particular value.
 */
function getAttribCheck(attrib, value) {
    if (typeof value === "function") {
        return function (elem) { return (0, domhandler_1.isTag)(elem) && value(elem.attribs[attrib]); };
    }
    return function (elem) { return (0, domhandler_1.isTag)(elem) && elem.attribs[attrib] === value; };
}
/**
 * Returns a function that returns `true` if either of the input functions
 * returns `true` for a node.
 *
 * @param a First function to combine.
 * @param b Second function to combine.
 * @returns A function taking a node and returning `true` if either of the input
 *   functions returns `true` for the node.
 */
function combineFuncs(a, b) {
    return function (elem) { return a(elem) || b(elem); };
}
/**
 * Returns a function that executes all checks in `options` and returns `true`
 * if any of them match a node.
 *
 * @param options An object describing nodes to look for.
 * @returns A function that executes all checks in `options` and returns `true`
 *   if any of them match a node.
 */
function compileTest(options) {
    var funcs = Object.keys(options).map(function (key) {
        var value = options[key];
        return Object.prototype.hasOwnProperty.call(Checks, key)
            ? Checks[key](value)
            : getAttribCheck(key, value);
    });
    return funcs.length === 0 ? null : funcs.reduce(combineFuncs);
}
/**
 * Checks whether a node matches the description in `options`.
 *
 * @category Legacy Query Functions
 * @param options An object describing nodes to look for.
 * @param node The element to test.
 * @returns Whether the element matches the description in `options`.
 */
function testElement(options, node) {
    var test = compileTest(options);
    return test ? test(node) : true;
}
legacy.testElement = testElement;
/**
 * Returns all nodes that match `options`.
 *
 * @category Legacy Query Functions
 * @param options An object describing nodes to look for.
 * @param nodes Nodes to search through.
 * @param recurse Also consider child nodes.
 * @param limit Maximum number of nodes to return.
 * @returns All nodes that match `options`.
 */
function getElements(options, nodes, recurse, limit) {
    if (limit === void 0) { limit = Infinity; }
    var test = compileTest(options);
    return test ? (0, querying_js_1.filter)(test, nodes, recurse, limit) : [];
}
legacy.getElements = getElements;
/**
 * Returns the node with the supplied ID.
 *
 * @category Legacy Query Functions
 * @param id The unique ID attribute value to look for.
 * @param nodes Nodes to search through.
 * @param recurse Also consider child nodes.
 * @returns The node with the supplied ID.
 */
function getElementById(id, nodes, recurse) {
    if (recurse === void 0) { recurse = true; }
    if (!Array.isArray(nodes))
        nodes = [nodes];
    return (0, querying_js_1.findOne)(getAttribCheck("id", id), nodes, recurse);
}
legacy.getElementById = getElementById;
/**
 * Returns all nodes with the supplied `tagName`.
 *
 * @category Legacy Query Functions
 * @param tagName Tag name to search for.
 * @param nodes Nodes to search through.
 * @param recurse Also consider child nodes.
 * @param limit Maximum number of nodes to return.
 * @returns All nodes with the supplied `tagName`.
 */
function getElementsByTagName(tagName, nodes, recurse, limit) {
    if (recurse === void 0) { recurse = true; }
    if (limit === void 0) { limit = Infinity; }
    return (0, querying_js_1.filter)(Checks["tag_name"](tagName), nodes, recurse, limit);
}
legacy.getElementsByTagName = getElementsByTagName;
/**
 * Returns all nodes with the supplied `type`.
 *
 * @category Legacy Query Functions
 * @param type Element type to look for.
 * @param nodes Nodes to search through.
 * @param recurse Also consider child nodes.
 * @param limit Maximum number of nodes to return.
 * @returns All nodes with the supplied `type`.
 */
function getElementsByTagType(type, nodes, recurse, limit) {
    if (recurse === void 0) { recurse = true; }
    if (limit === void 0) { limit = Infinity; }
    return (0, querying_js_1.filter)(Checks["tag_type"](type), nodes, recurse, limit);
}
legacy.getElementsByTagType = getElementsByTagType;

var helpers = {};

(function (exports) {
Object.defineProperty(exports, "__esModule", { value: true });
exports.uniqueSort = exports.compareDocumentPosition = exports.DocumentPosition = exports.removeSubsets = void 0;
var domhandler_1 = lib$6;
/**
 * Given an array of nodes, remove any member that is contained by another
 * member.
 *
 * @category Helpers
 * @param nodes Nodes to filter.
 * @returns Remaining nodes that aren't contained by other nodes.
 */
function removeSubsets(nodes) {
    var idx = nodes.length;
    /*
     * Check if each node (or one of its ancestors) is already contained in the
     * array.
     */
    while (--idx >= 0) {
        var node = nodes[idx];
        /*
         * Remove the node if it is not unique.
         * We are going through the array from the end, so we only
         * have to check nodes that preceed the node under consideration in the array.
         */
        if (idx > 0 && nodes.lastIndexOf(node, idx - 1) >= 0) {
            nodes.splice(idx, 1);
            continue;
        }
        for (var ancestor = node.parent; ancestor; ancestor = ancestor.parent) {
            if (nodes.includes(ancestor)) {
                nodes.splice(idx, 1);
                break;
            }
        }
    }
    return nodes;
}
exports.removeSubsets = removeSubsets;
/**
 * @category Helpers
 * @see {@link http://dom.spec.whatwg.org/#dom-node-comparedocumentposition}
 */
var DocumentPosition;
(function (DocumentPosition) {
    DocumentPosition[DocumentPosition["DISCONNECTED"] = 1] = "DISCONNECTED";
    DocumentPosition[DocumentPosition["PRECEDING"] = 2] = "PRECEDING";
    DocumentPosition[DocumentPosition["FOLLOWING"] = 4] = "FOLLOWING";
    DocumentPosition[DocumentPosition["CONTAINS"] = 8] = "CONTAINS";
    DocumentPosition[DocumentPosition["CONTAINED_BY"] = 16] = "CONTAINED_BY";
})(DocumentPosition = exports.DocumentPosition || (exports.DocumentPosition = {}));
/**
 * Compare the position of one node against another node in any other document,
 * returning a bitmask with the values from {@link DocumentPosition}.
 *
 * Document order:
 * > There is an ordering, document order, defined on all the nodes in the
 * > document corresponding to the order in which the first character of the
 * > XML representation of each node occurs in the XML representation of the
 * > document after expansion of general entities. Thus, the document element
 * > node will be the first node. Element nodes occur before their children.
 * > Thus, document order orders element nodes in order of the occurrence of
 * > their start-tag in the XML (after expansion of entities). The attribute
 * > nodes of an element occur after the element and before its children. The
 * > relative order of attribute nodes is implementation-dependent.
 *
 * Source:
 * http://www.w3.org/TR/DOM-Level-3-Core/glossary.html#dt-document-order
 *
 * @category Helpers
 * @param nodeA The first node to use in the comparison
 * @param nodeB The second node to use in the comparison
 * @returns A bitmask describing the input nodes' relative position.
 *
 * See http://dom.spec.whatwg.org/#dom-node-comparedocumentposition for
 * a description of these values.
 */
function compareDocumentPosition(nodeA, nodeB) {
    var aParents = [];
    var bParents = [];
    if (nodeA === nodeB) {
        return 0;
    }
    var current = (0, domhandler_1.hasChildren)(nodeA) ? nodeA : nodeA.parent;
    while (current) {
        aParents.unshift(current);
        current = current.parent;
    }
    current = (0, domhandler_1.hasChildren)(nodeB) ? nodeB : nodeB.parent;
    while (current) {
        bParents.unshift(current);
        current = current.parent;
    }
    var maxIdx = Math.min(aParents.length, bParents.length);
    var idx = 0;
    while (idx < maxIdx && aParents[idx] === bParents[idx]) {
        idx++;
    }
    if (idx === 0) {
        return DocumentPosition.DISCONNECTED;
    }
    var sharedParent = aParents[idx - 1];
    var siblings = sharedParent.children;
    var aSibling = aParents[idx];
    var bSibling = bParents[idx];
    if (siblings.indexOf(aSibling) > siblings.indexOf(bSibling)) {
        if (sharedParent === nodeB) {
            return DocumentPosition.FOLLOWING | DocumentPosition.CONTAINED_BY;
        }
        return DocumentPosition.FOLLOWING;
    }
    if (sharedParent === nodeA) {
        return DocumentPosition.PRECEDING | DocumentPosition.CONTAINS;
    }
    return DocumentPosition.PRECEDING;
}
exports.compareDocumentPosition = compareDocumentPosition;
/**
 * Sort an array of nodes based on their relative position in the document,
 * removing any duplicate nodes. If the array contains nodes that do not belong
 * to the same document, sort order is unspecified.
 *
 * @category Helpers
 * @param nodes Array of DOM nodes.
 * @returns Collection of unique nodes, sorted in document order.
 */
function uniqueSort(nodes) {
    nodes = nodes.filter(function (node, i, arr) { return !arr.includes(node, i + 1); });
    nodes.sort(function (a, b) {
        var relative = compareDocumentPosition(a, b);
        if (relative & DocumentPosition.PRECEDING) {
            return -1;
        }
        else if (relative & DocumentPosition.FOLLOWING) {
            return 1;
        }
        return 0;
    });
    return nodes;
}
exports.uniqueSort = uniqueSort;

}(helpers));

var feeds = {};

Object.defineProperty(feeds, "__esModule", { value: true });
feeds.getFeed = void 0;
var stringify_js_1 = stringify;
var legacy_js_1 = legacy;
/**
 * Get the feed object from the root of a DOM tree.
 *
 * @category Feeds
 * @param doc - The DOM to to extract the feed from.
 * @returns The feed.
 */
function getFeed(doc) {
    var feedRoot = getOneElement(isValidFeed, doc);
    return !feedRoot
        ? null
        : feedRoot.name === "feed"
            ? getAtomFeed(feedRoot)
            : getRssFeed(feedRoot);
}
feeds.getFeed = getFeed;
/**
 * Parse an Atom feed.
 *
 * @param feedRoot The root of the feed.
 * @returns The parsed feed.
 */
function getAtomFeed(feedRoot) {
    var _a;
    var childs = feedRoot.children;
    var feed = {
        type: "atom",
        items: (0, legacy_js_1.getElementsByTagName)("entry", childs).map(function (item) {
            var _a;
            var children = item.children;
            var entry = { media: getMediaElements(children) };
            addConditionally(entry, "id", "id", children);
            addConditionally(entry, "title", "title", children);
            var href = (_a = getOneElement("link", children)) === null || _a === void 0 ? void 0 : _a.attribs["href"];
            if (href) {
                entry.link = href;
            }
            var description = fetch$1("summary", children) || fetch$1("content", children);
            if (description) {
                entry.description = description;
            }
            var pubDate = fetch$1("updated", children);
            if (pubDate) {
                entry.pubDate = new Date(pubDate);
            }
            return entry;
        }),
    };
    addConditionally(feed, "id", "id", childs);
    addConditionally(feed, "title", "title", childs);
    var href = (_a = getOneElement("link", childs)) === null || _a === void 0 ? void 0 : _a.attribs["href"];
    if (href) {
        feed.link = href;
    }
    addConditionally(feed, "description", "subtitle", childs);
    var updated = fetch$1("updated", childs);
    if (updated) {
        feed.updated = new Date(updated);
    }
    addConditionally(feed, "author", "email", childs, true);
    return feed;
}
/**
 * Parse a RSS feed.
 *
 * @param feedRoot The root of the feed.
 * @returns The parsed feed.
 */
function getRssFeed(feedRoot) {
    var _a, _b;
    var childs = (_b = (_a = getOneElement("channel", feedRoot.children)) === null || _a === void 0 ? void 0 : _a.children) !== null && _b !== void 0 ? _b : [];
    var feed = {
        type: feedRoot.name.substr(0, 3),
        id: "",
        items: (0, legacy_js_1.getElementsByTagName)("item", feedRoot.children).map(function (item) {
            var children = item.children;
            var entry = { media: getMediaElements(children) };
            addConditionally(entry, "id", "guid", children);
            addConditionally(entry, "title", "title", children);
            addConditionally(entry, "link", "link", children);
            addConditionally(entry, "description", "description", children);
            var pubDate = fetch$1("pubDate", children) || fetch$1("dc:date", children);
            if (pubDate)
                entry.pubDate = new Date(pubDate);
            return entry;
        }),
    };
    addConditionally(feed, "title", "title", childs);
    addConditionally(feed, "link", "link", childs);
    addConditionally(feed, "description", "description", childs);
    var updated = fetch$1("lastBuildDate", childs);
    if (updated) {
        feed.updated = new Date(updated);
    }
    addConditionally(feed, "author", "managingEditor", childs, true);
    return feed;
}
var MEDIA_KEYS_STRING = ["url", "type", "lang"];
var MEDIA_KEYS_INT = [
    "fileSize",
    "bitrate",
    "framerate",
    "samplingrate",
    "channels",
    "duration",
    "height",
    "width",
];
/**
 * Get all media elements of a feed item.
 *
 * @param where Nodes to search in.
 * @returns Media elements.
 */
function getMediaElements(where) {
    return (0, legacy_js_1.getElementsByTagName)("media:content", where).map(function (elem) {
        var attribs = elem.attribs;
        var media = {
            medium: attribs["medium"],
            isDefault: !!attribs["isDefault"],
        };
        for (var _i = 0, MEDIA_KEYS_STRING_1 = MEDIA_KEYS_STRING; _i < MEDIA_KEYS_STRING_1.length; _i++) {
            var attrib = MEDIA_KEYS_STRING_1[_i];
            if (attribs[attrib]) {
                media[attrib] = attribs[attrib];
            }
        }
        for (var _a = 0, MEDIA_KEYS_INT_1 = MEDIA_KEYS_INT; _a < MEDIA_KEYS_INT_1.length; _a++) {
            var attrib = MEDIA_KEYS_INT_1[_a];
            if (attribs[attrib]) {
                media[attrib] = parseInt(attribs[attrib], 10);
            }
        }
        if (attribs["expression"]) {
            media.expression = attribs["expression"];
        }
        return media;
    });
}
/**
 * Get one element by tag name.
 *
 * @param tagName Tag name to look for
 * @param node Node to search in
 * @returns The element or null
 */
function getOneElement(tagName, node) {
    return (0, legacy_js_1.getElementsByTagName)(tagName, node, true, 1)[0];
}
/**
 * Get the text content of an element with a certain tag name.
 *
 * @param tagName Tag name to look for.
 * @param where Node to search in.
 * @param recurse Whether to recurse into child nodes.
 * @returns The text content of the element.
 */
function fetch$1(tagName, where, recurse) {
    if (recurse === void 0) { recurse = false; }
    return (0, stringify_js_1.textContent)((0, legacy_js_1.getElementsByTagName)(tagName, where, recurse, 1)).trim();
}
/**
 * Adds a property to an object if it has a value.
 *
 * @param obj Object to be extended
 * @param prop Property name
 * @param tagName Tag name that contains the conditionally added property
 * @param where Element to search for the property
 * @param recurse Whether to recurse into child nodes.
 */
function addConditionally(obj, prop, tagName, where, recurse) {
    if (recurse === void 0) { recurse = false; }
    var val = fetch$1(tagName, where, recurse);
    if (val)
        obj[prop] = val;
}
/**
 * Checks if an element is a feed root node.
 *
 * @param value The name of the element to check.
 * @returns Whether an element is a feed root node.
 */
function isValidFeed(value) {
    return value === "rss" || value === "feed" || value === "rdf:RDF";
}

(function (exports) {
var __createBinding = (commonjsGlobal && commonjsGlobal.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (commonjsGlobal && commonjsGlobal.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.hasChildren = exports.isDocument = exports.isComment = exports.isText = exports.isCDATA = exports.isTag = void 0;
__exportStar(stringify, exports);
__exportStar(traversal, exports);
__exportStar(manipulation, exports);
__exportStar(querying, exports);
__exportStar(legacy, exports);
__exportStar(helpers, exports);
__exportStar(feeds, exports);
/** @deprecated Use these methods from `domhandler` directly. */
var domhandler_1 = lib$6;
Object.defineProperty(exports, "isTag", { enumerable: true, get: function () { return domhandler_1.isTag; } });
Object.defineProperty(exports, "isCDATA", { enumerable: true, get: function () { return domhandler_1.isCDATA; } });
Object.defineProperty(exports, "isText", { enumerable: true, get: function () { return domhandler_1.isText; } });
Object.defineProperty(exports, "isComment", { enumerable: true, get: function () { return domhandler_1.isComment; } });
Object.defineProperty(exports, "isDocument", { enumerable: true, get: function () { return domhandler_1.isDocument; } });
Object.defineProperty(exports, "hasChildren", { enumerable: true, get: function () { return domhandler_1.hasChildren; } });

}(lib$3));

(function (exports) {
var __createBinding = (commonjsGlobal && commonjsGlobal.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (commonjsGlobal && commonjsGlobal.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (commonjsGlobal && commonjsGlobal.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (commonjsGlobal && commonjsGlobal.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DomUtils = exports.parseFeed = exports.getFeed = exports.ElementType = exports.Tokenizer = exports.createDomStream = exports.createDocumentStream = exports.parseDOM = exports.parseDocument = exports.DefaultHandler = exports.DomHandler = exports.Parser = void 0;
var Parser_js_1 = Parser$2;
var Parser_js_2 = Parser$2;
Object.defineProperty(exports, "Parser", { enumerable: true, get: function () { return Parser_js_2.Parser; } });
var domhandler_1 = lib$6;
var domhandler_2 = lib$6;
Object.defineProperty(exports, "DomHandler", { enumerable: true, get: function () { return domhandler_2.DomHandler; } });
// Old name for DomHandler
Object.defineProperty(exports, "DefaultHandler", { enumerable: true, get: function () { return domhandler_2.DomHandler; } });
// Helper methods
/**
 * Parses the data, returns the resulting document.
 *
 * @param data The data that should be parsed.
 * @param options Optional options for the parser and DOM handler.
 */
function parseDocument(data, options) {
    var handler = new domhandler_1.DomHandler(undefined, options);
    new Parser_js_1.Parser(handler, options).end(data);
    return handler.root;
}
exports.parseDocument = parseDocument;
/**
 * Parses data, returns an array of the root nodes.
 *
 * Note that the root nodes still have a `Document` node as their parent.
 * Use `parseDocument` to get the `Document` node instead.
 *
 * @param data The data that should be parsed.
 * @param options Optional options for the parser and DOM handler.
 * @deprecated Use `parseDocument` instead.
 */
function parseDOM(data, options) {
    return parseDocument(data, options).children;
}
exports.parseDOM = parseDOM;
/**
 * Creates a parser instance, with an attached DOM handler.
 *
 * @param callback A callback that will be called once parsing has been completed, with the resulting document.
 * @param options Optional options for the parser and DOM handler.
 * @param elementCallback An optional callback that will be called every time a tag has been completed inside of the DOM.
 */
function createDocumentStream(callback, options, elementCallback) {
    var handler = new domhandler_1.DomHandler(function (error) { return callback(error, handler.root); }, options, elementCallback);
    return new Parser_js_1.Parser(handler, options);
}
exports.createDocumentStream = createDocumentStream;
/**
 * Creates a parser instance, with an attached DOM handler.
 *
 * @param callback A callback that will be called once parsing has been completed, with an array of root nodes.
 * @param options Optional options for the parser and DOM handler.
 * @param elementCallback An optional callback that will be called every time a tag has been completed inside of the DOM.
 * @deprecated Use `createDocumentStream` instead.
 */
function createDomStream(callback, options, elementCallback) {
    var handler = new domhandler_1.DomHandler(callback, options, elementCallback);
    return new Parser_js_1.Parser(handler, options);
}
exports.createDomStream = createDomStream;
var Tokenizer_js_1 = Tokenizer;
Object.defineProperty(exports, "Tokenizer", { enumerable: true, get: function () { return __importDefault(Tokenizer_js_1).default; } });
/*
 * All of the following exports exist for backwards-compatibility.
 * They should probably be removed eventually.
 */
exports.ElementType = __importStar(lib$5);
var domutils_1 = lib$3;
var domutils_2 = lib$3;
Object.defineProperty(exports, "getFeed", { enumerable: true, get: function () { return domutils_2.getFeed; } });
var parseFeedDefaultOptions = { xmlMode: true };
/**
 * Parse a feed.
 *
 * @param feed The feed that should be parsed, as a string.
 * @param options Optionally, options for parsing. When using this, you should set `xmlMode` to `true`.
 */
function parseFeed(feed, options) {
    if (options === void 0) { options = parseFeedDefaultOptions; }
    return (0, domutils_1.getFeed)(parseDOM(feed, options));
}
exports.parseFeed = parseFeed;
exports.DomUtils = __importStar(lib$3);

}(lib$4));

/**
 * Sets root parent to null.
 *
 * @param  {Array<Comment|Element|ProcessingInstruction|Text>} nodes
 * @return {Array<Comment|Element|ProcessingInstruction|Text>}
 */

function unsetRootParent$1(nodes) {
  for (var index = 0, len = nodes.length; index < len; index++) {
    var node = nodes[index];
    node.parent = null;
  }
  return nodes;
}

var utilities$4 = {
  unsetRootParent: unsetRootParent$1
};

var Parser = lib$4.Parser;
var DomHandler = lib$6.DomHandler;

var unsetRootParent = utilities$4.unsetRootParent;

/**
 * Parses HTML string to DOM nodes in Node.js.
 *
 * This is the same method as `require('htmlparser2').parseDOM`
 * https://github.com/fb55/htmlparser2/blob/v6.0.0/src/index.ts#L29-L41
 *
 * @param  {string}            html      - HTML markup.
 * @param  {DomHandlerOptions} [options] - Parser options (https://github.com/fb55/domhandler/tree/v4.0.0#readme).
 * @return {Array<Comment|Element|ProcessingInstruction|Text>} - DOM nodes.
 */
function HTMLDOMParser$1(html, options) {
  if (typeof html !== 'string') {
    throw new TypeError('First argument must be a string.');
  }

  if (html === '') {
    return [];
  }

  var handler = new DomHandler(undefined, options);
  new Parser(handler, options).end(html);
  return unsetRootParent(handler.dom);
}

var htmlToDom = HTMLDOMParser$1;

/**
 * When running on Node.js, use the server parser.
 * When bundling for the browser, use the client parser.
 *
 * @see {@link https://github.com/substack/node-browserify#browser-field}
 */

var HTMLDOMParser = htmlToDom;

htmlDomParser.exports = HTMLDOMParser;
htmlDomParser.exports.default = HTMLDOMParser;

var lib = {};

var possibleStandardNamesOptimized$1 = {};

// An attribute in which the DOM/SVG standard name is the same as the React prop name (e.g., 'accept').
var SAME$1 = 0;
possibleStandardNamesOptimized$1.SAME = SAME$1;

// An attribute in which the React prop name is the camelcased version of the DOM/SVG standard name (e.g., 'acceptCharset').
var CAMELCASE$1 = 1;
possibleStandardNamesOptimized$1.CAMELCASE = CAMELCASE$1;

possibleStandardNamesOptimized$1.possibleStandardNames = {
  accept: 0,
  acceptCharset: 1,
  'accept-charset': 'acceptCharset',
  accessKey: 1,
  action: 0,
  allowFullScreen: 1,
  alt: 0,
  as: 0,
  async: 0,
  autoCapitalize: 1,
  autoComplete: 1,
  autoCorrect: 1,
  autoFocus: 1,
  autoPlay: 1,
  autoSave: 1,
  capture: 0,
  cellPadding: 1,
  cellSpacing: 1,
  challenge: 0,
  charSet: 1,
  checked: 0,
  children: 0,
  cite: 0,
  class: 'className',
  classID: 1,
  className: 1,
  cols: 0,
  colSpan: 1,
  content: 0,
  contentEditable: 1,
  contextMenu: 1,
  controls: 0,
  controlsList: 1,
  coords: 0,
  crossOrigin: 1,
  dangerouslySetInnerHTML: 1,
  data: 0,
  dateTime: 1,
  default: 0,
  defaultChecked: 1,
  defaultValue: 1,
  defer: 0,
  dir: 0,
  disabled: 0,
  disablePictureInPicture: 1,
  disableRemotePlayback: 1,
  download: 0,
  draggable: 0,
  encType: 1,
  enterKeyHint: 1,
  for: 'htmlFor',
  form: 0,
  formMethod: 1,
  formAction: 1,
  formEncType: 1,
  formNoValidate: 1,
  formTarget: 1,
  frameBorder: 1,
  headers: 0,
  height: 0,
  hidden: 0,
  high: 0,
  href: 0,
  hrefLang: 1,
  htmlFor: 1,
  httpEquiv: 1,
  'http-equiv': 'httpEquiv',
  icon: 0,
  id: 0,
  innerHTML: 1,
  inputMode: 1,
  integrity: 0,
  is: 0,
  itemID: 1,
  itemProp: 1,
  itemRef: 1,
  itemScope: 1,
  itemType: 1,
  keyParams: 1,
  keyType: 1,
  kind: 0,
  label: 0,
  lang: 0,
  list: 0,
  loop: 0,
  low: 0,
  manifest: 0,
  marginWidth: 1,
  marginHeight: 1,
  max: 0,
  maxLength: 1,
  media: 0,
  mediaGroup: 1,
  method: 0,
  min: 0,
  minLength: 1,
  multiple: 0,
  muted: 0,
  name: 0,
  noModule: 1,
  nonce: 0,
  noValidate: 1,
  open: 0,
  optimum: 0,
  pattern: 0,
  placeholder: 0,
  playsInline: 1,
  poster: 0,
  preload: 0,
  profile: 0,
  radioGroup: 1,
  readOnly: 1,
  referrerPolicy: 1,
  rel: 0,
  required: 0,
  reversed: 0,
  role: 0,
  rows: 0,
  rowSpan: 1,
  sandbox: 0,
  scope: 0,
  scoped: 0,
  scrolling: 0,
  seamless: 0,
  selected: 0,
  shape: 0,
  size: 0,
  sizes: 0,
  span: 0,
  spellCheck: 1,
  src: 0,
  srcDoc: 1,
  srcLang: 1,
  srcSet: 1,
  start: 0,
  step: 0,
  style: 0,
  summary: 0,
  tabIndex: 1,
  target: 0,
  title: 0,
  type: 0,
  useMap: 1,
  value: 0,
  width: 0,
  wmode: 0,
  wrap: 0,
  about: 0,
  accentHeight: 1,
  'accent-height': 'accentHeight',
  accumulate: 0,
  additive: 0,
  alignmentBaseline: 1,
  'alignment-baseline': 'alignmentBaseline',
  allowReorder: 1,
  alphabetic: 0,
  amplitude: 0,
  arabicForm: 1,
  'arabic-form': 'arabicForm',
  ascent: 0,
  attributeName: 1,
  attributeType: 1,
  autoReverse: 1,
  azimuth: 0,
  baseFrequency: 1,
  baselineShift: 1,
  'baseline-shift': 'baselineShift',
  baseProfile: 1,
  bbox: 0,
  begin: 0,
  bias: 0,
  by: 0,
  calcMode: 1,
  capHeight: 1,
  'cap-height': 'capHeight',
  clip: 0,
  clipPath: 1,
  'clip-path': 'clipPath',
  clipPathUnits: 1,
  clipRule: 1,
  'clip-rule': 'clipRule',
  color: 0,
  colorInterpolation: 1,
  'color-interpolation': 'colorInterpolation',
  colorInterpolationFilters: 1,
  'color-interpolation-filters': 'colorInterpolationFilters',
  colorProfile: 1,
  'color-profile': 'colorProfile',
  colorRendering: 1,
  'color-rendering': 'colorRendering',
  contentScriptType: 1,
  contentStyleType: 1,
  cursor: 0,
  cx: 0,
  cy: 0,
  d: 0,
  datatype: 0,
  decelerate: 0,
  descent: 0,
  diffuseConstant: 1,
  direction: 0,
  display: 0,
  divisor: 0,
  dominantBaseline: 1,
  'dominant-baseline': 'dominantBaseline',
  dur: 0,
  dx: 0,
  dy: 0,
  edgeMode: 1,
  elevation: 0,
  enableBackground: 1,
  'enable-background': 'enableBackground',
  end: 0,
  exponent: 0,
  externalResourcesRequired: 1,
  fill: 0,
  fillOpacity: 1,
  'fill-opacity': 'fillOpacity',
  fillRule: 1,
  'fill-rule': 'fillRule',
  filter: 0,
  filterRes: 1,
  filterUnits: 1,
  floodOpacity: 1,
  'flood-opacity': 'floodOpacity',
  floodColor: 1,
  'flood-color': 'floodColor',
  focusable: 0,
  fontFamily: 1,
  'font-family': 'fontFamily',
  fontSize: 1,
  'font-size': 'fontSize',
  fontSizeAdjust: 1,
  'font-size-adjust': 'fontSizeAdjust',
  fontStretch: 1,
  'font-stretch': 'fontStretch',
  fontStyle: 1,
  'font-style': 'fontStyle',
  fontVariant: 1,
  'font-variant': 'fontVariant',
  fontWeight: 1,
  'font-weight': 'fontWeight',
  format: 0,
  from: 0,
  fx: 0,
  fy: 0,
  g1: 0,
  g2: 0,
  glyphName: 1,
  'glyph-name': 'glyphName',
  glyphOrientationHorizontal: 1,
  'glyph-orientation-horizontal': 'glyphOrientationHorizontal',
  glyphOrientationVertical: 1,
  'glyph-orientation-vertical': 'glyphOrientationVertical',
  glyphRef: 1,
  gradientTransform: 1,
  gradientUnits: 1,
  hanging: 0,
  horizAdvX: 1,
  'horiz-adv-x': 'horizAdvX',
  horizOriginX: 1,
  'horiz-origin-x': 'horizOriginX',
  ideographic: 0,
  imageRendering: 1,
  'image-rendering': 'imageRendering',
  in2: 0,
  in: 0,
  inlist: 0,
  intercept: 0,
  k1: 0,
  k2: 0,
  k3: 0,
  k4: 0,
  k: 0,
  kernelMatrix: 1,
  kernelUnitLength: 1,
  kerning: 0,
  keyPoints: 1,
  keySplines: 1,
  keyTimes: 1,
  lengthAdjust: 1,
  letterSpacing: 1,
  'letter-spacing': 'letterSpacing',
  lightingColor: 1,
  'lighting-color': 'lightingColor',
  limitingConeAngle: 1,
  local: 0,
  markerEnd: 1,
  'marker-end': 'markerEnd',
  markerHeight: 1,
  markerMid: 1,
  'marker-mid': 'markerMid',
  markerStart: 1,
  'marker-start': 'markerStart',
  markerUnits: 1,
  markerWidth: 1,
  mask: 0,
  maskContentUnits: 1,
  maskUnits: 1,
  mathematical: 0,
  mode: 0,
  numOctaves: 1,
  offset: 0,
  opacity: 0,
  operator: 0,
  order: 0,
  orient: 0,
  orientation: 0,
  origin: 0,
  overflow: 0,
  overlinePosition: 1,
  'overline-position': 'overlinePosition',
  overlineThickness: 1,
  'overline-thickness': 'overlineThickness',
  paintOrder: 1,
  'paint-order': 'paintOrder',
  panose1: 0,
  'panose-1': 'panose1',
  pathLength: 1,
  patternContentUnits: 1,
  patternTransform: 1,
  patternUnits: 1,
  pointerEvents: 1,
  'pointer-events': 'pointerEvents',
  points: 0,
  pointsAtX: 1,
  pointsAtY: 1,
  pointsAtZ: 1,
  prefix: 0,
  preserveAlpha: 1,
  preserveAspectRatio: 1,
  primitiveUnits: 1,
  property: 0,
  r: 0,
  radius: 0,
  refX: 1,
  refY: 1,
  renderingIntent: 1,
  'rendering-intent': 'renderingIntent',
  repeatCount: 1,
  repeatDur: 1,
  requiredExtensions: 1,
  requiredFeatures: 1,
  resource: 0,
  restart: 0,
  result: 0,
  results: 0,
  rotate: 0,
  rx: 0,
  ry: 0,
  scale: 0,
  security: 0,
  seed: 0,
  shapeRendering: 1,
  'shape-rendering': 'shapeRendering',
  slope: 0,
  spacing: 0,
  specularConstant: 1,
  specularExponent: 1,
  speed: 0,
  spreadMethod: 1,
  startOffset: 1,
  stdDeviation: 1,
  stemh: 0,
  stemv: 0,
  stitchTiles: 1,
  stopColor: 1,
  'stop-color': 'stopColor',
  stopOpacity: 1,
  'stop-opacity': 'stopOpacity',
  strikethroughPosition: 1,
  'strikethrough-position': 'strikethroughPosition',
  strikethroughThickness: 1,
  'strikethrough-thickness': 'strikethroughThickness',
  string: 0,
  stroke: 0,
  strokeDasharray: 1,
  'stroke-dasharray': 'strokeDasharray',
  strokeDashoffset: 1,
  'stroke-dashoffset': 'strokeDashoffset',
  strokeLinecap: 1,
  'stroke-linecap': 'strokeLinecap',
  strokeLinejoin: 1,
  'stroke-linejoin': 'strokeLinejoin',
  strokeMiterlimit: 1,
  'stroke-miterlimit': 'strokeMiterlimit',
  strokeWidth: 1,
  'stroke-width': 'strokeWidth',
  strokeOpacity: 1,
  'stroke-opacity': 'strokeOpacity',
  suppressContentEditableWarning: 1,
  suppressHydrationWarning: 1,
  surfaceScale: 1,
  systemLanguage: 1,
  tableValues: 1,
  targetX: 1,
  targetY: 1,
  textAnchor: 1,
  'text-anchor': 'textAnchor',
  textDecoration: 1,
  'text-decoration': 'textDecoration',
  textLength: 1,
  textRendering: 1,
  'text-rendering': 'textRendering',
  to: 0,
  transform: 0,
  typeof: 0,
  u1: 0,
  u2: 0,
  underlinePosition: 1,
  'underline-position': 'underlinePosition',
  underlineThickness: 1,
  'underline-thickness': 'underlineThickness',
  unicode: 0,
  unicodeBidi: 1,
  'unicode-bidi': 'unicodeBidi',
  unicodeRange: 1,
  'unicode-range': 'unicodeRange',
  unitsPerEm: 1,
  'units-per-em': 'unitsPerEm',
  unselectable: 0,
  vAlphabetic: 1,
  'v-alphabetic': 'vAlphabetic',
  values: 0,
  vectorEffect: 1,
  'vector-effect': 'vectorEffect',
  version: 0,
  vertAdvY: 1,
  'vert-adv-y': 'vertAdvY',
  vertOriginX: 1,
  'vert-origin-x': 'vertOriginX',
  vertOriginY: 1,
  'vert-origin-y': 'vertOriginY',
  vHanging: 1,
  'v-hanging': 'vHanging',
  vIdeographic: 1,
  'v-ideographic': 'vIdeographic',
  viewBox: 1,
  viewTarget: 1,
  visibility: 0,
  vMathematical: 1,
  'v-mathematical': 'vMathematical',
  vocab: 0,
  widths: 0,
  wordSpacing: 1,
  'word-spacing': 'wordSpacing',
  writingMode: 1,
  'writing-mode': 'writingMode',
  x1: 0,
  x2: 0,
  x: 0,
  xChannelSelector: 1,
  xHeight: 1,
  'x-height': 'xHeight',
  xlinkActuate: 1,
  'xlink:actuate': 'xlinkActuate',
  xlinkArcrole: 1,
  'xlink:arcrole': 'xlinkArcrole',
  xlinkHref: 1,
  'xlink:href': 'xlinkHref',
  xlinkRole: 1,
  'xlink:role': 'xlinkRole',
  xlinkShow: 1,
  'xlink:show': 'xlinkShow',
  xlinkTitle: 1,
  'xlink:title': 'xlinkTitle',
  xlinkType: 1,
  'xlink:type': 'xlinkType',
  xmlBase: 1,
  'xml:base': 'xmlBase',
  xmlLang: 1,
  'xml:lang': 'xmlLang',
  xmlns: 0,
  'xml:space': 'xmlSpace',
  xmlnsXlink: 1,
  'xmlns:xlink': 'xmlnsXlink',
  xmlSpace: 1,
  y1: 0,
  y2: 0,
  y: 0,
  yChannelSelector: 1,
  z: 0,
  zoomAndPan: 1
};

Object.defineProperty(lib, '__esModule', { value: true });

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"];

  if (_i == null) return;
  var _arr = [];
  var _n = true;
  var _d = false;

  var _s, _e;

  try {
    for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

// A reserved attribute.
// It is handled by React separately and shouldn't be written to the DOM.
var RESERVED = 0; // A simple string attribute.
// Attributes that aren't in the filter are presumed to have this type.

var STRING = 1; // A string attribute that accepts booleans in React. In HTML, these are called
// "enumerated" attributes with "true" and "false" as possible values.
// When true, it should be set to a "true" string.
// When false, it should be set to a "false" string.

var BOOLEANISH_STRING = 2; // A real boolean attribute.
// When true, it should be present (set either to an empty string or its name).
// When false, it should be omitted.

var BOOLEAN = 3; // An attribute that can be used as a flag as well as with a value.
// When true, it should be present (set either to an empty string or its name).
// When false, it should be omitted.
// For any other value, should be present with that value.

var OVERLOADED_BOOLEAN = 4; // An attribute that must be numeric or parse as a numeric.
// When falsy, it should be removed.

var NUMERIC = 5; // An attribute that must be positive numeric or parse as a positive numeric.
// When falsy, it should be removed.

var POSITIVE_NUMERIC = 6;
function getPropertyInfo(name) {
  return properties.hasOwnProperty(name) ? properties[name] : null;
}

function PropertyInfoRecord(name, type, mustUseProperty, attributeName, attributeNamespace, sanitizeURL, removeEmptyString) {
  this.acceptsBooleans = type === BOOLEANISH_STRING || type === BOOLEAN || type === OVERLOADED_BOOLEAN;
  this.attributeName = attributeName;
  this.attributeNamespace = attributeNamespace;
  this.mustUseProperty = mustUseProperty;
  this.propertyName = name;
  this.type = type;
  this.sanitizeURL = sanitizeURL;
  this.removeEmptyString = removeEmptyString;
} // When adding attributes to this list, be sure to also add them to
// the `possibleStandardNames` module to ensure casing and incorrect
// name warnings.


var properties = {}; // These props are reserved by React. They shouldn't be written to the DOM.

var reservedProps = ['children', 'dangerouslySetInnerHTML', // TODO: This prevents the assignment of defaultValue to regular
// elements (not just inputs). Now that ReactDOMInput assigns to the
// defaultValue property -- do we need this?
'defaultValue', 'defaultChecked', 'innerHTML', 'suppressContentEditableWarning', 'suppressHydrationWarning', 'style'];
reservedProps.forEach(function (name) {
  properties[name] = new PropertyInfoRecord(name, RESERVED, false, // mustUseProperty
  name, // attributeName
  null, // attributeNamespace
  false, // sanitizeURL
  false);
}); // A few React string attributes have a different name.
// This is a mapping from React prop names to the attribute names.

[['acceptCharset', 'accept-charset'], ['className', 'class'], ['htmlFor', 'for'], ['httpEquiv', 'http-equiv']].forEach(function (_ref) {
  var _ref2 = _slicedToArray(_ref, 2),
      name = _ref2[0],
      attributeName = _ref2[1];

  properties[name] = new PropertyInfoRecord(name, STRING, false, // mustUseProperty
  attributeName, // attributeName
  null, // attributeNamespace
  false, // sanitizeURL
  false);
}); // These are "enumerated" HTML attributes that accept "true" and "false".
// In React, we let users pass `true` and `false` even though technically
// these aren't boolean attributes (they are coerced to strings).

['contentEditable', 'draggable', 'spellCheck', 'value'].forEach(function (name) {
  properties[name] = new PropertyInfoRecord(name, BOOLEANISH_STRING, false, // mustUseProperty
  name.toLowerCase(), // attributeName
  null, // attributeNamespace
  false, // sanitizeURL
  false);
}); // These are "enumerated" SVG attributes that accept "true" and "false".
// In React, we let users pass `true` and `false` even though technically
// these aren't boolean attributes (they are coerced to strings).
// Since these are SVG attributes, their attribute names are case-sensitive.

['autoReverse', 'externalResourcesRequired', 'focusable', 'preserveAlpha'].forEach(function (name) {
  properties[name] = new PropertyInfoRecord(name, BOOLEANISH_STRING, false, // mustUseProperty
  name, // attributeName
  null, // attributeNamespace
  false, // sanitizeURL
  false);
}); // These are HTML boolean attributes.

['allowFullScreen', 'async', // Note: there is a special case that prevents it from being written to the DOM
// on the client side because the browsers are inconsistent. Instead we call focus().
'autoFocus', 'autoPlay', 'controls', 'default', 'defer', 'disabled', 'disablePictureInPicture', 'disableRemotePlayback', 'formNoValidate', 'hidden', 'loop', 'noModule', 'noValidate', 'open', 'playsInline', 'readOnly', 'required', 'reversed', 'scoped', 'seamless', // Microdata
'itemScope'].forEach(function (name) {
  properties[name] = new PropertyInfoRecord(name, BOOLEAN, false, // mustUseProperty
  name.toLowerCase(), // attributeName
  null, // attributeNamespace
  false, // sanitizeURL
  false);
}); // These are the few React props that we set as DOM properties
// rather than attributes. These are all booleans.

['checked', // Note: `option.selected` is not updated if `select.multiple` is
// disabled with `removeAttribute`. We have special logic for handling this.
'multiple', 'muted', 'selected' // NOTE: if you add a camelCased prop to this list,
// you'll need to set attributeName to name.toLowerCase()
// instead in the assignment below.
].forEach(function (name) {
  properties[name] = new PropertyInfoRecord(name, BOOLEAN, true, // mustUseProperty
  name, // attributeName
  null, // attributeNamespace
  false, // sanitizeURL
  false);
}); // These are HTML attributes that are "overloaded booleans": they behave like
// booleans, but can also accept a string value.

['capture', 'download' // NOTE: if you add a camelCased prop to this list,
// you'll need to set attributeName to name.toLowerCase()
// instead in the assignment below.
].forEach(function (name) {
  properties[name] = new PropertyInfoRecord(name, OVERLOADED_BOOLEAN, false, // mustUseProperty
  name, // attributeName
  null, // attributeNamespace
  false, // sanitizeURL
  false);
}); // These are HTML attributes that must be positive numbers.

['cols', 'rows', 'size', 'span' // NOTE: if you add a camelCased prop to this list,
// you'll need to set attributeName to name.toLowerCase()
// instead in the assignment below.
].forEach(function (name) {
  properties[name] = new PropertyInfoRecord(name, POSITIVE_NUMERIC, false, // mustUseProperty
  name, // attributeName
  null, // attributeNamespace
  false, // sanitizeURL
  false);
}); // These are HTML attributes that must be numbers.

['rowSpan', 'start'].forEach(function (name) {
  properties[name] = new PropertyInfoRecord(name, NUMERIC, false, // mustUseProperty
  name.toLowerCase(), // attributeName
  null, // attributeNamespace
  false, // sanitizeURL
  false);
});
var CAMELIZE = /[\-\:]([a-z])/g;

var capitalize$1 = function capitalize(token) {
  return token[1].toUpperCase();
}; // This is a list of all SVG attributes that need special casing, namespacing,
// or boolean value assignment. Regular attributes that just accept strings
// and have the same names are omitted, just like in the HTML attribute filter.
// Some of these attributes can be hard to find. This list was created by
// scraping the MDN documentation.


['accent-height', 'alignment-baseline', 'arabic-form', 'baseline-shift', 'cap-height', 'clip-path', 'clip-rule', 'color-interpolation', 'color-interpolation-filters', 'color-profile', 'color-rendering', 'dominant-baseline', 'enable-background', 'fill-opacity', 'fill-rule', 'flood-color', 'flood-opacity', 'font-family', 'font-size', 'font-size-adjust', 'font-stretch', 'font-style', 'font-variant', 'font-weight', 'glyph-name', 'glyph-orientation-horizontal', 'glyph-orientation-vertical', 'horiz-adv-x', 'horiz-origin-x', 'image-rendering', 'letter-spacing', 'lighting-color', 'marker-end', 'marker-mid', 'marker-start', 'overline-position', 'overline-thickness', 'paint-order', 'panose-1', 'pointer-events', 'rendering-intent', 'shape-rendering', 'stop-color', 'stop-opacity', 'strikethrough-position', 'strikethrough-thickness', 'stroke-dasharray', 'stroke-dashoffset', 'stroke-linecap', 'stroke-linejoin', 'stroke-miterlimit', 'stroke-opacity', 'stroke-width', 'text-anchor', 'text-decoration', 'text-rendering', 'underline-position', 'underline-thickness', 'unicode-bidi', 'unicode-range', 'units-per-em', 'v-alphabetic', 'v-hanging', 'v-ideographic', 'v-mathematical', 'vector-effect', 'vert-adv-y', 'vert-origin-x', 'vert-origin-y', 'word-spacing', 'writing-mode', 'xmlns:xlink', 'x-height' // NOTE: if you add a camelCased prop to this list,
// you'll need to set attributeName to name.toLowerCase()
// instead in the assignment below.
].forEach(function (attributeName) {
  var name = attributeName.replace(CAMELIZE, capitalize$1);
  properties[name] = new PropertyInfoRecord(name, STRING, false, // mustUseProperty
  attributeName, null, // attributeNamespace
  false, // sanitizeURL
  false);
}); // String SVG attributes with the xlink namespace.

['xlink:actuate', 'xlink:arcrole', 'xlink:role', 'xlink:show', 'xlink:title', 'xlink:type' // NOTE: if you add a camelCased prop to this list,
// you'll need to set attributeName to name.toLowerCase()
// instead in the assignment below.
].forEach(function (attributeName) {
  var name = attributeName.replace(CAMELIZE, capitalize$1);
  properties[name] = new PropertyInfoRecord(name, STRING, false, // mustUseProperty
  attributeName, 'http://www.w3.org/1999/xlink', false, // sanitizeURL
  false);
}); // String SVG attributes with the xml namespace.

['xml:base', 'xml:lang', 'xml:space' // NOTE: if you add a camelCased prop to this list,
// you'll need to set attributeName to name.toLowerCase()
// instead in the assignment below.
].forEach(function (attributeName) {
  var name = attributeName.replace(CAMELIZE, capitalize$1);
  properties[name] = new PropertyInfoRecord(name, STRING, false, // mustUseProperty
  attributeName, 'http://www.w3.org/XML/1998/namespace', false, // sanitizeURL
  false);
}); // These attribute exists both in HTML and SVG.
// The attribute name is case-sensitive in SVG so we can't just use
// the React name like we do for attributes that exist only in HTML.

['tabIndex', 'crossOrigin'].forEach(function (attributeName) {
  properties[attributeName] = new PropertyInfoRecord(attributeName, STRING, false, // mustUseProperty
  attributeName.toLowerCase(), // attributeName
  null, // attributeNamespace
  false, // sanitizeURL
  false);
}); // These attributes accept URLs. These must not allow javascript: URLS.
// These will also need to accept Trusted Types object in the future.

var xlinkHref = 'xlinkHref';
properties[xlinkHref] = new PropertyInfoRecord('xlinkHref', STRING, false, // mustUseProperty
'xlink:href', 'http://www.w3.org/1999/xlink', true, // sanitizeURL
false);
['src', 'href', 'action', 'formAction'].forEach(function (attributeName) {
  properties[attributeName] = new PropertyInfoRecord(attributeName, STRING, false, // mustUseProperty
  attributeName.toLowerCase(), // attributeName
  null, // attributeNamespace
  true, // sanitizeURL
  true);
});

var _require = possibleStandardNamesOptimized$1,
    CAMELCASE = _require.CAMELCASE,
    SAME = _require.SAME,
    possibleStandardNamesOptimized = _require.possibleStandardNames;

var ATTRIBUTE_NAME_START_CHAR = ":A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD";
var ATTRIBUTE_NAME_CHAR = ATTRIBUTE_NAME_START_CHAR + "\\-.0-9\\u00B7\\u0300-\\u036F\\u203F-\\u2040";
/**
 * Checks whether a property name is a custom attribute.
 *
 * @see {@link https://github.com/facebook/react/blob/15-stable/src/renderers/dom/shared/HTMLDOMPropertyConfig.js#L23-L25}
 *
 * @param {string}
 * @return {boolean}
 */

var isCustomAttribute = RegExp.prototype.test.bind( // eslint-disable-next-line no-misleading-character-class
new RegExp('^(data|aria)-[' + ATTRIBUTE_NAME_CHAR + ']*$'));
var possibleStandardNames = Object.keys(possibleStandardNamesOptimized).reduce(function (accumulator, standardName) {
  var propName = possibleStandardNamesOptimized[standardName];

  if (propName === SAME) {
    accumulator[standardName] = standardName;
  } else if (propName === CAMELCASE) {
    accumulator[standardName.toLowerCase()] = standardName;
  } else {
    accumulator[standardName] = propName;
  }

  return accumulator;
}, {});

lib.BOOLEAN = BOOLEAN;
lib.BOOLEANISH_STRING = BOOLEANISH_STRING;
lib.NUMERIC = NUMERIC;
lib.OVERLOADED_BOOLEAN = OVERLOADED_BOOLEAN;
lib.POSITIVE_NUMERIC = POSITIVE_NUMERIC;
lib.RESERVED = RESERVED;
lib.STRING = STRING;
lib.getPropertyInfo = getPropertyInfo;
lib.isCustomAttribute = isCustomAttribute;
lib.possibleStandardNames = possibleStandardNames;

var cjs = {};

var styleToObject = {exports: {}};

// http://www.w3.org/TR/CSS21/grammar.html
// https://github.com/visionmedia/css-parse/pull/49#issuecomment-30088027
var COMMENT_REGEX = /\/\*[^*]*\*+([^/*][^*]*\*+)*\//g;

var NEWLINE_REGEX = /\n/g;
var WHITESPACE_REGEX = /^\s*/;

// declaration
var PROPERTY_REGEX = /^(\*?[-#/*\\\w]+(\[[0-9a-z_-]+\])?)\s*/;
var COLON_REGEX = /^:\s*/;
var VALUE_REGEX = /^((?:'(?:\\'|.)*?'|"(?:\\"|.)*?"|\([^)]*?\)|[^};])+)/;
var SEMICOLON_REGEX = /^[;\s]*/;

// https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String/Trim#Polyfill
var TRIM_REGEX = /^\s+|\s+$/g;

// strings
var NEWLINE = '\n';
var FORWARD_SLASH = '/';
var ASTERISK = '*';
var EMPTY_STRING = '';

// types
var TYPE_COMMENT = 'comment';
var TYPE_DECLARATION = 'declaration';

/**
 * @param {String} style
 * @param {Object} [options]
 * @return {Object[]}
 * @throws {TypeError}
 * @throws {Error}
 */
var inlineStyleParser = function(style, options) {
  if (typeof style !== 'string') {
    throw new TypeError('First argument must be a string');
  }

  if (!style) return [];

  options = options || {};

  /**
   * Positional.
   */
  var lineno = 1;
  var column = 1;

  /**
   * Update lineno and column based on `str`.
   *
   * @param {String} str
   */
  function updatePosition(str) {
    var lines = str.match(NEWLINE_REGEX);
    if (lines) lineno += lines.length;
    var i = str.lastIndexOf(NEWLINE);
    column = ~i ? str.length - i : column + str.length;
  }

  /**
   * Mark position and patch `node.position`.
   *
   * @return {Function}
   */
  function position() {
    var start = { line: lineno, column: column };
    return function(node) {
      node.position = new Position(start);
      whitespace();
      return node;
    };
  }

  /**
   * Store position information for a node.
   *
   * @constructor
   * @property {Object} start
   * @property {Object} end
   * @property {undefined|String} source
   */
  function Position(start) {
    this.start = start;
    this.end = { line: lineno, column: column };
    this.source = options.source;
  }

  /**
   * Non-enumerable source string.
   */
  Position.prototype.content = style;

  /**
   * Error `msg`.
   *
   * @param {String} msg
   * @throws {Error}
   */
  function error(msg) {
    var err = new Error(
      options.source + ':' + lineno + ':' + column + ': ' + msg
    );
    err.reason = msg;
    err.filename = options.source;
    err.line = lineno;
    err.column = column;
    err.source = style;

    if (options.silent) ; else {
      throw err;
    }
  }

  /**
   * Match `re` and return captures.
   *
   * @param {RegExp} re
   * @return {undefined|Array}
   */
  function match(re) {
    var m = re.exec(style);
    if (!m) return;
    var str = m[0];
    updatePosition(str);
    style = style.slice(str.length);
    return m;
  }

  /**
   * Parse whitespace.
   */
  function whitespace() {
    match(WHITESPACE_REGEX);
  }

  /**
   * Parse comments.
   *
   * @param {Object[]} [rules]
   * @return {Object[]}
   */
  function comments(rules) {
    var c;
    rules = rules || [];
    while ((c = comment())) {
      if (c !== false) {
        rules.push(c);
      }
    }
    return rules;
  }

  /**
   * Parse comment.
   *
   * @return {Object}
   * @throws {Error}
   */
  function comment() {
    var pos = position();
    if (FORWARD_SLASH != style.charAt(0) || ASTERISK != style.charAt(1)) return;

    var i = 2;
    while (
      EMPTY_STRING != style.charAt(i) &&
      (ASTERISK != style.charAt(i) || FORWARD_SLASH != style.charAt(i + 1))
    ) {
      ++i;
    }
    i += 2;

    if (EMPTY_STRING === style.charAt(i - 1)) {
      return error('End of comment missing');
    }

    var str = style.slice(2, i - 2);
    column += 2;
    updatePosition(str);
    style = style.slice(i);
    column += 2;

    return pos({
      type: TYPE_COMMENT,
      comment: str
    });
  }

  /**
   * Parse declaration.
   *
   * @return {Object}
   * @throws {Error}
   */
  function declaration() {
    var pos = position();

    // prop
    var prop = match(PROPERTY_REGEX);
    if (!prop) return;
    comment();

    // :
    if (!match(COLON_REGEX)) return error("property missing ':'");

    // val
    var val = match(VALUE_REGEX);

    var ret = pos({
      type: TYPE_DECLARATION,
      property: trim(prop[0].replace(COMMENT_REGEX, EMPTY_STRING)),
      value: val
        ? trim(val[0].replace(COMMENT_REGEX, EMPTY_STRING))
        : EMPTY_STRING
    });

    // ;
    match(SEMICOLON_REGEX);

    return ret;
  }

  /**
   * Parse declarations.
   *
   * @return {Object[]}
   */
  function declarations() {
    var decls = [];

    comments(decls);

    // declarations
    var decl;
    while ((decl = declaration())) {
      if (decl !== false) {
        decls.push(decl);
        comments(decls);
      }
    }

    return decls;
  }

  whitespace();
  return declarations();
};

/**
 * Trim `str`.
 *
 * @param {String} str
 * @return {String}
 */
function trim(str) {
  return str ? str.replace(TRIM_REGEX, EMPTY_STRING) : EMPTY_STRING;
}

var parse = inlineStyleParser;

/**
 * Parses inline style to object.
 *
 * @example
 * // returns { 'line-height': '42' }
 * StyleToObject('line-height: 42;');
 *
 * @param  {String}      style      - The inline style.
 * @param  {Function}    [iterator] - The iterator function.
 * @return {null|Object}
 */
function StyleToObject(style, iterator) {
  var output = null;
  if (!style || typeof style !== 'string') {
    return output;
  }

  var declaration;
  var declarations = parse(style);
  var hasIterator = typeof iterator === 'function';
  var property;
  var value;

  for (var i = 0, len = declarations.length; i < len; i++) {
    declaration = declarations[i];
    property = declaration.property;
    value = declaration.value;

    if (hasIterator) {
      iterator(property, value, declaration);
    } else if (value) {
      output || (output = {});
      output[property] = value;
    }
  }

  return output;
}

styleToObject.exports = StyleToObject;
styleToObject.exports.default = StyleToObject; // ESM support

var utilities$3 = {};

utilities$3.__esModule = true;
utilities$3.camelCase = void 0;
var CUSTOM_PROPERTY_REGEX = /^--[a-zA-Z0-9-]+$/;
var HYPHEN_REGEX = /-([a-z])/g;
var NO_HYPHEN_REGEX = /^[^-]+$/;
var VENDOR_PREFIX_REGEX = /^-(webkit|moz|ms|o|khtml)-/;
var MS_VENDOR_PREFIX_REGEX = /^-(ms)-/;
var skipCamelCase = function (property) {
    return !property ||
        NO_HYPHEN_REGEX.test(property) ||
        CUSTOM_PROPERTY_REGEX.test(property);
};
var capitalize = function (match, character) {
    return character.toUpperCase();
};
var trimHyphen = function (match, prefix) { return "".concat(prefix, "-"); };
var camelCase = function (property, options) {
    if (options === void 0) { options = {}; }
    if (skipCamelCase(property)) {
        return property;
    }
    property = property.toLowerCase();
    if (options.reactCompat) {
        property = property.replace(MS_VENDOR_PREFIX_REGEX, trimHyphen);
    }
    else {
        property = property.replace(VENDOR_PREFIX_REGEX, trimHyphen);
    }
    return property.replace(HYPHEN_REGEX, capitalize);
};
utilities$3.camelCase = camelCase;

(function (exports) {
var __importDefault = (commonjsGlobal && commonjsGlobal.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var style_to_object_1 = __importDefault(styleToObject.exports);
var utilities_1 = utilities$3;
function StyleToJS(style, options) {
    var output = {};
    if (!style || typeof style !== 'string') {
        return output;
    }
    (0, style_to_object_1["default"])(style, function (property, value) {
        if (property && value) {
            output[(0, utilities_1.camelCase)(property, options)] = value;
        }
    });
    return output;
}
exports["default"] = StyleToJS;
}(cjs));

var React$1 = React$2;
var styleToJS = cjs.default;

/**
 * Swap key with value in an object.
 *
 * @param {object} obj - The object.
 * @param {Function} [override] - The override method.
 * @returns - The inverted object.
 */
function invertObject(obj, override) {
  if (!obj || typeof obj !== 'object') {
    throw new TypeError('First argument must be an object');
  }

  var isOverridePresent = typeof override === 'function';
  var overrides = {};
  var result = {};

  for (var key in obj) {
    var value = obj[key];

    if (isOverridePresent) {
      overrides = override(key, value);
      if (overrides && overrides.length === 2) {
        result[overrides[0]] = overrides[1];
        continue;
      }
    }

    if (typeof value === 'string') {
      result[value] = key;
    }
  }

  return result;
}

/**
 * Check if a given tag is a custom component.
 *
 * @see {@link https://github.com/facebook/react/blob/v16.6.3/packages/react-dom/src/shared/isCustomComponent.js}
 *
 * @param {string} tagName - The name of the html tag.
 * @param {object} props - The props being passed to the element.
 * @returns - Whether tag is custom component.
 */

var RESERVED_SVG_MATHML_ELEMENTS = new Set([
  'annotation-xml',
  'color-profile',
  'font-face',
  'font-face-src',
  'font-face-uri',
  'font-face-format',
  'font-face-name',
  'missing-glyph'
]);

function isCustomComponent(tagName, props) {
  if (tagName.indexOf('-') === -1) {
    return props && typeof props.is === 'string';
  }
  // These are reserved SVG and MathML elements.
  // We don't mind this whitelist too much because we expect it to never grow.
  // The alternative is to track the namespace in a few places which is convoluted.
  // https://w3c.github.io/webcomponents/spec/custom/#custom-elements-core-concepts
  if (RESERVED_SVG_MATHML_ELEMENTS.has(tagName)) {
    return false;
  }
  return true;
}

// styleToJSOptions
var STYLE_TO_JS_OPTIONS = { reactCompat: true };

/**
 * Sets style prop.
 *
 * @param {null|undefined|string} style
 * @param {object} props
 */
function setStyleProp$1(style, props) {
  if (style === null || style === undefined) {
    return;
  }
  try {
    props.style = styleToJS(style, STYLE_TO_JS_OPTIONS);
  } catch (err) {
    props.style = {};
  }
}

/**
 * @constant {boolean}
 * @see {@link https://reactjs.org/blog/2017/09/08/dom-attributes-in-react-16.html}
 */
var PRESERVE_CUSTOM_ATTRIBUTES = React$1.version.split('.')[0] >= 16;

// Taken from
// https://github.com/facebook/react/blob/cae635054e17a6f107a39d328649137b83f25972/packages/react-dom/src/client/validateDOMNesting.js#L213
var ELEMENTS_WITH_NO_TEXT_CHILDREN = new Set([
  'tr',
  'tbody',
  'thead',
  'tfoot',
  'colgroup',
  'table',
  'head',
  'html',
  'frameset'
]);

/**
 * Checks if the given node can contain text nodes
 *
 * @param {DomElement} node - Node.
 * @returns - Whether node can contain text nodes.
 */
function canTextBeChildOfNode$1(node) {
  return !ELEMENTS_WITH_NO_TEXT_CHILDREN.has(node.name);
}

function returnFirstArg(arg) {
  return arg;
}

var utilities$2 = {
  PRESERVE_CUSTOM_ATTRIBUTES: PRESERVE_CUSTOM_ATTRIBUTES,
  ELEMENTS_WITH_NO_TEXT_CHILDREN: ELEMENTS_WITH_NO_TEXT_CHILDREN,
  invertObject: invertObject,
  isCustomComponent: isCustomComponent,
  setStyleProp: setStyleProp$1,
  canTextBeChildOfNode: canTextBeChildOfNode$1,
  returnFirstArg: returnFirstArg
};

var reactProperty = lib;
var utilities$1 = utilities$2;

// https://reactjs.org/docs/uncontrolled-components.html
// https://developer.mozilla.org/docs/Web/HTML/Attributes
var UNCONTROLLED_COMPONENT_ATTRIBUTES = ['checked', 'value'];
var UNCONTROLLED_COMPONENT_NAMES = ['input', 'select', 'textarea'];

var VALUE_ONLY_INPUTS = {
  reset: true,
  submit: true
};

/**
 * Converts HTML/SVG DOM attributes to React props.
 *
 * @param {object} [attributes={}] - HTML/SVG DOM attributes.
 * @param {string} [nodeName] - DOM node name.
 * @returns - React props.
 */
var attributesToProps$2 = function attributesToProps(attributes, nodeName) {
  attributes = attributes || {};

  var attributeName;
  var attributeNameLowerCased;
  var attributeValue;
  var propName;
  var propertyInfo;
  var props = {};
  var inputIsValueOnly = attributes.type && VALUE_ONLY_INPUTS[attributes.type];

  for (attributeName in attributes) {
    attributeValue = attributes[attributeName];

    // ARIA (aria-*) or custom data (data-*) attribute
    if (reactProperty.isCustomAttribute(attributeName)) {
      props[attributeName] = attributeValue;
      continue;
    }

    // convert HTML/SVG attribute to React prop
    attributeNameLowerCased = attributeName.toLowerCase();
    propName = getPropName(attributeNameLowerCased);

    if (propName) {
      propertyInfo = reactProperty.getPropertyInfo(propName);

      // convert attribute to uncontrolled component prop (e.g., `value` to `defaultValue`)
      if (
        UNCONTROLLED_COMPONENT_ATTRIBUTES.indexOf(propName) !== -1 &&
        UNCONTROLLED_COMPONENT_NAMES.indexOf(nodeName) !== -1 &&
        !inputIsValueOnly
      ) {
        propName = getPropName('default' + attributeNameLowerCased);
      }

      props[propName] = attributeValue;

      switch (propertyInfo && propertyInfo.type) {
        case reactProperty.BOOLEAN:
          props[propName] = true;
          break;
        case reactProperty.OVERLOADED_BOOLEAN:
          if (attributeValue === '') {
            props[propName] = true;
          }
          break;
      }
      continue;
    }

    // preserve custom attribute if React >=16
    if (utilities$1.PRESERVE_CUSTOM_ATTRIBUTES) {
      props[attributeName] = attributeValue;
    }
  }

  // transform inline style to object
  utilities$1.setStyleProp(attributes.style, props);

  return props;
};

/**
 * Gets prop name from lowercased attribute name.
 *
 * @param {string} attributeName - Lowercased attribute name.
 * @returns - Prop name.
 */
function getPropName(attributeName) {
  return reactProperty.possibleStandardNames[attributeName];
}

var React = React$2;
var attributesToProps$1 = attributesToProps$2;
var utilities = utilities$2;

var setStyleProp = utilities.setStyleProp;
var canTextBeChildOfNode = utilities.canTextBeChildOfNode;

/**
 * Converts DOM nodes to JSX element(s).
 *
 * @param {DomElement[]} nodes - DOM nodes.
 * @param {object} [options={}] - Options.
 * @param {Function} [options.replace] - Replacer.
 * @param {Function} [options.transform] - Transform.
 * @param {object} [options.library] - Library (React, Preact, etc.).
 * @returns - String or JSX element(s).
 */
function domToReact$1(nodes, options) {
  options = options || {};

  var library = options.library || React;
  var cloneElement = library.cloneElement;
  var createElement = library.createElement;
  var isValidElement = library.isValidElement;

  var result = [];
  var node;
  var isWhitespace;
  var hasReplace = typeof options.replace === 'function';
  var transform = options.transform || utilities.returnFirstArg;
  var replaceElement;
  var props;
  var children;
  var trim = options.trim;

  for (var i = 0, len = nodes.length; i < len; i++) {
    node = nodes[i];

    // replace with custom React element (if present)
    if (hasReplace) {
      replaceElement = options.replace(node);

      if (isValidElement(replaceElement)) {
        // set "key" prop for sibling elements
        // https://fb.me/react-warning-keys
        if (len > 1) {
          replaceElement = cloneElement(replaceElement, {
            key: replaceElement.key || i
          });
        }
        result.push(transform(replaceElement, node, i));
        continue;
      }
    }

    if (node.type === 'text') {
      isWhitespace = !node.data.trim().length;

      if (isWhitespace && node.parent && !canTextBeChildOfNode(node.parent)) {
        // We have a whitespace node that can't be nested in its parent
        // so skip it
        continue;
      }

      if (trim && isWhitespace) {
        // Trim is enabled and we have a whitespace node
        // so skip it
        continue;
      }

      // We have a text node that's not whitespace and it can be nested
      // in its parent so add it to the results
      result.push(transform(node.data, node, i));
      continue;
    }

    props = node.attribs;
    if (skipAttributesToProps(node)) {
      setStyleProp(props.style, props);
    } else if (props) {
      props = attributesToProps$1(props, node.name);
    }

    children = null;

    switch (node.type) {
      case 'script':
      case 'style':
        // prevent text in <script> or <style> from being escaped
        // https://reactjs.org/docs/dom-elements.html#dangerouslysetinnerhtml
        if (node.children[0]) {
          props.dangerouslySetInnerHTML = {
            __html: node.children[0].data
          };
        }
        break;

      case 'tag':
        // setting textarea value in children is an antipattern in React
        // https://reactjs.org/docs/forms.html#the-textarea-tag
        if (node.name === 'textarea' && node.children[0]) {
          props.defaultValue = node.children[0].data;
        } else if (node.children && node.children.length) {
          // continue recursion of creating React elements (if applicable)
          children = domToReact$1(node.children, options);
        }
        break;

      // skip all other cases (e.g., comment)
      default:
        continue;
    }

    // set "key" prop for sibling elements
    // https://fb.me/react-warning-keys
    if (len > 1) {
      props.key = i;
    }

    result.push(transform(createElement(node.name, props, children), node, i));
  }

  return result.length === 1 ? result[0] : result;
}

/**
 * Determines whether DOM element attributes should be transformed to props.
 * Web Components should not have their attributes transformed except for `style`.
 *
 * @param {DomElement} node
 * @returns - Whether node attributes should be converted to props.
 */
function skipAttributesToProps(node) {
  return (
    utilities.PRESERVE_CUSTOM_ATTRIBUTES &&
    node.type === 'tag' &&
    utilities.isCustomComponent(node.name, node.attribs)
  );
}

var domToReact_1 = domToReact$1;

var domhandler = lib$6;
var htmlToDOM = htmlDomParser.exports;

var attributesToProps = attributesToProps$2;
var domToReact = domToReact_1;

// support backwards compatibility for ES Module
htmlToDOM =
  /* istanbul ignore next */
  typeof htmlToDOM.default === 'function' ? htmlToDOM.default : htmlToDOM;

var domParserOptions = { lowerCaseAttributeNames: false };

/**
 * Converts HTML string to React elements.
 *
 * @param {string} html - HTML string.
 * @param {object} [options] - Parser options.
 * @param {object} [options.htmlparser2] - htmlparser2 options.
 * @param {object} [options.library] - Library for React, Preact, etc.
 * @param {Function} [options.replace] - Replace method.
 * @returns {JSX.Element|JSX.Element[]|string} - React element(s), empty array, or string.
 */
function HTMLReactParser(html, options) {
  if (typeof html !== 'string') {
    throw new TypeError('First argument must be a string');
  }
  if (html === '') {
    return [];
  }
  options = options || {};
  return domToReact(
    htmlToDOM(html, options.htmlparser2 || domParserOptions),
    options
  );
}

HTMLReactParser.domToReact = domToReact;
HTMLReactParser.htmlToDOM = htmlToDOM;
HTMLReactParser.attributesToProps = attributesToProps;

// domhandler
HTMLReactParser.Comment = domhandler.Comment;
HTMLReactParser.Element = domhandler.Element;
HTMLReactParser.ProcessingInstruction = domhandler.ProcessingInstruction;
HTMLReactParser.Text = domhandler.Text;

// support CommonJS and ES Modules
var htmlReactParser = HTMLReactParser;
HTMLReactParser.default = HTMLReactParser;

htmlReactParser.domToReact;
htmlReactParser.htmlToDOM;
htmlReactParser.attributesToProps;

// domhandler
htmlReactParser.Comment;
htmlReactParser.Element;
htmlReactParser.ProcessingInstruction;
htmlReactParser.Text;

class ComponentDemo$2 extends Component$1 {
  constructor(props) {
    super(props);
    this.state = {
      runJavascript: this.runJavascript.bind(this)
    };
  }
  componentDidMount() {
    const {
      pageData = {}
    } = this.props;
    this.setState({
      title: pageData.title
    });
    this.runJavascript();
  }
  runJavascript() {
    const {
      pageData = {}
    } = this.props;
    if (pageData.setUpJavascript) {
      eval(pageData.setUpJavascript);
    }
    eval(pageData.javascript);
  }
  static getDerivedStateFromProps(props, state) {
    const {
      pageData = {}
    } = props;
    if (pageData.title !== state.title) {
      state.runJavascript();
      return {
        title: pageData.title,
        runJavascript: state.runJavascript
      };
    }
  }
  render() {
    const {
      metaData = {},
      pageData = {},
      transitionClass
    } = this.props;
    const {
      className = ''
    } = pageData;
    const {
      Header,
      LinkButton,
      NavigationList,
      CodeHighlight
    } = components$1.get();
    return jsx("div", {
      className: `sq-content-toc sq-content-page__body ${className}`,
      children: jsx("div", {
        className: "container-fluid",
        children: jsxs("div", {
          className: "row",
          children: [jsx("div", {
            className: "col-xs-12 col-md-3",
            children: jsx(NavigationList, {
              links: metaData.siblingPages
            })
          }), jsxs("div", {
            className: `col-xs-12 col-md-9 ${transitionClass}`,
            children: [jsx(Header, {
              header: pageData.title,
              subHeader: pageData.description
            }), pageData.jsFiddle && jsx(LinkButton, {
              href: pageData.jsFiddle,
              target: '_blank',
              iconName: "arrow-right",
              iconDirection: "right",
              buttonText: "Edit in fiddle",
              className: 'mb-wide'
            }), jsx("h5", {
              className: "mb-wide",
              children: "Example"
            }), jsx("div", {
              className: "mb-wide",
              children: htmlReactParser(pageData.html)
            }), jsx("h5", {
              className: "mb-wide",
              children: "HTML"
            }), jsx(CodeHighlight, {
              language: "html",
              className: "mb-wide",
              code: pageData.html
            }), jsx("h5", {
              className: "mb-wide",
              children: "JAVASCRIPT"
            }), jsx(CodeHighlight, {
              language: "javascript",
              className: "mb-wide",
              code: pageData.javascript
            })]
          })]
        })
      })
    });
  }
}
ComponentDemo$2.propTypes = {};

class ComponentDemo$1 extends Component$1 {
  constructor(props) {
    super(props);
    this.state = {};
  }
  componentDidMount() {
    const _a = this.props,
      {
        metaData = {},
        pageData = {}
      } = _a;
      __rest(_a, ["metaData", "pageData"]);
    if (pageData.javascript) {
      eval(pageData.javascript);
    }
  }
  render() {
    const _a = this.props,
      {
        metaData = {},
        pageData = {},
        transitionClass
      } = _a,
      rest = __rest(_a, ["metaData", "pageData", "transitionClass"]);
    const {
      className = ''
    } = pageData;
    const {
      NavigationList
    } = components$1.get();
    return jsx("div", {
      className: `sq-content-doc sq-content-page__body ${className}`,
      children: jsx("div", {
        className: "container-fluid",
        children: jsxs("div", {
          className: "row",
          children: [jsx("div", {
            className: "col-xs-12 col-md-3",
            children: jsx(NavigationList, {
              links: metaData.siblingPages
            })
          }), jsx("div", {
            className: `col-xs-12 col-md-9 ${transitionClass}`,
            children: jsx(Content, Object.assign({
              pageData: pageData
            }, rest))
          })]
        })
      })
    });
  }
}
ComponentDemo$1.propTypes = {
  commonStore: PropTypes.object,
  contentStore: PropTypes.object
};

class ContentWithSearch extends Component$1 {
  constructor(props) {
    super(props);
    this.state = {};
  }
  componentDidMount() {
    const {
      metaData = {},
      pageData = {}
    } = this.props;
    const grouped = groupBy(pageData.items, 'group');
    this.setState({
      sortedGroups: Object.keys(grouped).sort((a, b) => {
        return a < b ? -1 : 1;
      }),
      grouped
    });
  }
  getPageData() {
    const {
      pageData = {}
    } = this.props;
    return pageData;
  }
  handleLeftClick(i, idx) {
    const {
      scrollContainer
    } = this.getPageData();
    const offset = document.getElementById(`${i}${idx}`) && document.getElementById(`${i}${idx}`).offsetTop;
    if (document.querySelector(scrollContainer) && offset !== undefined) {
      animateScrollTo(offset, {
        elementToScroll: document.querySelector(scrollContainer)
      });
    } else if (offset !== undefined) {
      animateScrollTo(offset);
    }
  }
  render() {
    const _a = this.props,
      {
        metaData = {},
        pageData = {}
      } = _a,
      rest = __rest(_a, ["metaData", "pageData"]);
    const {
      className = '',
      pageLayout = {},
      leftNavigation
    } = pageData;
    const {
      grouped = {},
      sortedGroups = []
    } = this.state;
    const {
      Header
    } = components$1.get();
    return jsxs("div", {
      className: `sq-search-content sq-search-content__body ${className}`,
      children: [Object.keys(pageLayout).map(layoutKey => {
        const layoutObj = pageLayout[layoutKey];
        const {
            className = ''
          } = layoutObj,
          restLayout = __rest(layoutObj, ["className"]);
        return jsx("div", {
          className: `sq-layout-content__block ${className}`,
          children: jsx(Content, Object.assign({
            flat: true
          }, rest, {
            pageData: Object.assign({}, restLayout)
          }))
        });
      }), jsx("div", {
        className: "sq-search-content__root",
        children: jsx("div", {
          className: "container sq-search-content__container",
          children: jsxs("div", {
            className: "row",
            children: [jsxs("div", {
              className: "col-xs-12 col-md-4 sq-search-content__left-nav",
              children: [jsx("h3", {
                className: "sq-search-content__left-nav-header",
                children: leftNavigation === null || leftNavigation === void 0 ? void 0 : leftNavigation.title
              }), sortedGroups.map((i, idx) => {
                return jsx("div", {
                  className: "sq-search-content__nav-block",
                  onClick: () => this.handleLeftClick(i, idx),
                  children: i
                });
              })]
            }), jsx("div", {
              className: "col-xs-12 col-md-8 sq-search-content__right_content",
              children: sortedGroups.map((i, idx) => {
                return jsx(Fragment, {
                  children: jsxs("div", {
                    className: "sq-search-content__block",
                    children: [jsx("h2", {
                      id: `${i}${idx}`,
                      className: "sq-search-content__content-header",
                      children: i
                    }), grouped[i].map(restData => {
                      return jsxs(Fragment, {
                        children: [jsx(Header, {
                          className: "sq-search-content__block-header",
                          headerTag: `h5`,
                          header: restData.title,
                          subHeader: restData.description
                        }), jsx(Content, Object.assign({
                          flat: true
                        }, rest, {
                          pageData: Object.assign({}, restData)
                        }))]
                      });
                    })]
                  })
                });
              })
            })]
          })
        })
      })]
    });
  }
}
ContentWithSearch.propTypes = {};

const Option = ({
  item
}) => {
  return /*#__PURE__*/jsxs(Fragment, {
    children: [/*#__PURE__*/jsx("h3", {
      children: item.name
    }), /*#__PURE__*/jsxs("div", {
      className: "sq-content-doc__info",
      children: [/*#__PURE__*/jsxs("div", {
        className: "sq-content-doc__info-row",
        children: [/*#__PURE__*/jsx("div", {
          className: "sq-content-doc__info-row-label",
          children: "Type:"
        }), /*#__PURE__*/jsx("div", {
          className: "sq-content-doc__info-row-text",
          children: item.type
        })]
      }), /*#__PURE__*/jsxs("div", {
        className: "sq-content-doc__info-row mb-xtrawide",
        children: [/*#__PURE__*/jsx("div", {
          className: "sq-content-doc__info-row-label",
          children: "Default:"
        }), /*#__PURE__*/jsx("code", {
          className: "sq-content-doc__info-row-code",
          children: item.defaultValue && item.defaultValue.toString()
        })]
      }), /*#__PURE__*/jsx("div", {
        className: "sq-content-doc__info-row ",
        children: /*#__PURE__*/jsx("div", {
          className: "sq-content-doc__info-row-text",
          children: htmlReactParser(item.description)
        })
      }), item.note && /*#__PURE__*/jsx("div", {
        className: "sq-content-doc__info-row",
        children: /*#__PURE__*/jsxs("div", {
          className: "sq-content-doc__info-row-text",
          children: [/*#__PURE__*/jsx("span", {
            className: "sq-content-doc__info-row-label",
            children: "Note: "
          }), item.note]
        })
      })]
    })]
  });
};
Option.propTypes = {
  item: PropTypes.object
};

const Method = ({
  item
}) => {
  const params = item.inputs && item.inputs.map(i => i.name).join(',') || '';
  return /*#__PURE__*/jsxs(Fragment, {
    children: [/*#__PURE__*/jsxs("div", {
      className: "sq-content-doc__name",
      children: [/*#__PURE__*/jsxs("h3", {
        className: "sq-content-doc__name-text",
        children: [item.name, params && ['(', params, ')'].join('')]
      }), item.returns && /*#__PURE__*/jsxs("div", {
        className: "sq-content-doc__info-row mb-xtrawide",
        children: [/*#__PURE__*/jsx("div", {
          className: "sq-content-doc__info-row-label",
          children: "Returns:"
        }), /*#__PURE__*/jsx("div", {
          className: "sq-content-doc__info-row-text",
          children: item.returns
        })]
      })]
    }), item.inputs && /*#__PURE__*/jsx("ul", {
      className: "sq-content-doc__params",
      children: item.inputs.map((inputKey, idx) => {
        return /*#__PURE__*/jsxs("li", {
          children: [/*#__PURE__*/jsx("div", {
            className: "sq-content-doc__params-name",
            children: inputKey.name
          }), /*#__PURE__*/jsxs("div", {
            children: ["Type: ", inputKey.type]
          }), /*#__PURE__*/jsx("div", {
            children: htmlReactParser(inputKey.description)
          })]
        }, idx);
      })
    }), /*#__PURE__*/jsxs("div", {
      className: "sq-content-doc__info",
      children: [/*#__PURE__*/jsx("div", {
        className: "sq-content-doc__info-row ",
        children: /*#__PURE__*/jsx("div", {
          className: "sq-content-doc__info-row-text",
          children: item.description
        })
      }), item.note && /*#__PURE__*/jsx("div", {
        className: "sq-content-doc__info-row",
        children: /*#__PURE__*/jsxs("div", {
          className: "sq-content-doc__info-row-text",
          children: [/*#__PURE__*/jsx("span", {
            className: "sq-content-doc__info-row-label",
            children: "Note: "
          }), item.note]
        })
      })]
    })]
  });
};
Method.propTypes = {
  item: PropTypes.object
};

const Event = ({
  item
}) => {
  const params = item.inputs && item.inputs.map(i => i.name).join(', ') || '';
  return /*#__PURE__*/jsxs(Fragment, {
    children: [/*#__PURE__*/jsx("div", {
      className: "sq-content-doc__name",
      children: /*#__PURE__*/jsxs("h3", {
        className: "sq-content-doc__name-text",
        children: [item.name, params && ['(', params, ')'].join('')]
      })
    }), item.inputs && /*#__PURE__*/jsx("ul", {
      children: item.inputs.map(inputKey => {
        return /*#__PURE__*/jsxs("li", {
          children: [/*#__PURE__*/jsx("div", {
            className: "sq-content-doc__params-name",
            children: inputKey.name
          }), /*#__PURE__*/jsxs("div", {
            children: ["Type: ", inputKey.type]
          }), inputKey.description && /*#__PURE__*/jsx("div", {
            children: inputKey.description
          })]
        });
      })
    }), /*#__PURE__*/jsxs("div", {
      className: "sq-content-doc__info",
      children: [/*#__PURE__*/jsx("div", {
        className: "sq-content-doc__info-row ",
        children: /*#__PURE__*/jsx("div", {
          className: "sq-content-doc__info-row-text",
          children: item.description
        })
      }), item.note && /*#__PURE__*/jsx("div", {
        className: "sq-content-doc__info-row",
        children: /*#__PURE__*/jsxs("div", {
          className: "sq-content-doc__info-row-text",
          children: [/*#__PURE__*/jsx("span", {
            className: "sq-content-doc__info-row-label",
            children: "Note: "
          }), item.note]
        })
      })]
    })]
  });
};
Event.propTypes = {
  item: PropTypes.object
};

const _tempaltes = {
  Option,
  Method,
  Event
};
class ComponentDemo extends Component$1 {
  constructor(props) {
    super(props);
    this.state = {};
  }
  componentDidMount() {
    const _a = this.props,
      {
        metaData = {},
        pageData = {}
      } = _a;
      __rest(_a, ["metaData", "pageData"]);
    eval(pageData.javascript);
  }
  render() {
    const _a = this.props,
      {
        metaData = {},
        pageData = {},
        transitionClass
      } = _a;
      __rest(_a, ["metaData", "pageData", "transitionClass"]);
    const {
      className = '',
      itemTemplate
    } = pageData;
    const {
      CodeHighlight,
      Header,
      NavigationList
    } = components$1.get();
    return jsx("div", {
      className: `sq-content-doc sq-content-page__body ${className}`,
      children: jsx("div", {
        className: "container-fluid",
        children: jsxs("div", {
          className: "row",
          children: [jsx("div", {
            className: "col-xs-12 col-md-3",
            children: jsx(NavigationList, {
              links: metaData.siblingPages
            })
          }), jsxs("div", {
            className: `col-xs-12 col-md-9 ${transitionClass}`,
            children: [jsx(Header, {
              header: pageData.title,
              subHeader: pageData.description
            }), jsx("div", {
              className: "sq-content-doc__navigation mb-xtrawide",
              children: pageData.items && pageData.items.map(item => {
                return jsx("div", {
                  children: jsxs("a", {
                    href: `#${item.name}`,
                    children: [item.name, " "]
                  })
                });
              })
            }), pageData.items && pageData.items.map(item => {
              const TemplateToRender = _tempaltes[itemTemplate] || _tempaltes.Option;
              return jsxs("div", {
                className: "sq-content-doc__block mb-xtrawide",
                children: [jsx(TemplateToRender, {
                  item: item
                }), jsx("h4", {
                  children: "Code Example"
                }), jsx(CodeHighlight, {
                  language: "javascript",
                  code: item.example
                })]
              });
            })]
          })]
        })
      })
    });
  }
}
ComponentDemo.propTypes = {
  commonStore: PropTypes.object,
  contentStore: PropTypes.object
};

var _a$2;
const {
  translate: translate$4
} = translate$6;
const defaultPageSize$3 = (_a$2 = GLOBAL_OPTIONS.noOfResultsDropdown.toArray()[0]) === null || _a$2 === void 0 ? void 0 : _a$2.value;
class EmailTemplateList extends React$2.Component {
  constructor(props) {
    super(props);
    this.state = {
      currentTab: GLOBAL_OPTIONS.userTabs.keys.ACTIVE_USERS,
      showFilters: false,
      currentFilter: {},
      currentSort: {
        sortColumn: 'subject',
        sortOrder: 'asc',
        showEditColumns: false,
        gridColumnUI: [],
        initialColumns: [],
        selectedColumns: [],
        editColumnData: [{
          name: 'name',
          headerText: 'name',
          className: 'col-lg-fixed'
        }, {
          name: 'emailId',
          className: 'col-lg-fixed',
          headerText: 'Email ID'
        }, {
          name: 'roleName',
          className: 'col-lg-fixed',
          headerText: 'Role'
        }, {
          name: 'contactNumber',
          className: 'col-lg-fixed',
          headerText: 'Primary Phone Number'
        }, {
          name: 'loginDate',
          className: 'col-lg-fixed',
          headerText: 'Login Date'
        }, {
          name: 'tickets',
          className: 'col-lg-fixed',
          headerText: translate$4('Service Requests')
        }]
      }
    };
    this.tabs = GLOBAL_OPTIONS.userTabs.toArray();
    this.handlePageChange = this.handlePageChange.bind(this);
    this.onFilterChange = this.onFilterChange.bind(this);
    this.onFilterToggle = this.onFilterToggle.bind(this);
    this.onFilterAction = this.onFilterAction.bind(this);
    this.onGridAction = this.onGridAction.bind(this);
    this.users_OnSort = this.users_OnSort.bind(this);
    this.onEditColumnToggle = this.onEditColumnToggle.bind(this);
    this.onColFilterSelection = this.onColFilterSelection.bind(this);
  }
  componentDidMount() {
    return __awaiter$1(this, void 0, void 0, function* () {
      const params = query$1.get();
      this.props;
      if (params.status) {
        yield this.setState({
          currentFilter: Object.assign(Object.assign({}, this.state.currentFilter), {
            status: params.status
          })
        });
      }
      yield this.refreshTemplates({
        filter: {
          active: true
        }
      });
    });
  }
  refreshTemplates({
    pageNo,
    pageSize,
    sort,
    filter,
    source
  } = {}) {
    var _a, _b, _c, _d;
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        pageData
      } = this.props;
      pageSize = pageSize || defaultPageSize$3 || ((_a = this.props.store.emailtemplate) === null || _a === void 0 ? void 0 : _a.templatesPagination.pageSize);
      pageNo = pageNo || ((_c = (_b = this.props.store.emailtemplate) === null || _b === void 0 ? void 0 : _b.templatesPagination) === null || _c === void 0 ? void 0 : _c.currentPage) || 1;
      const sortBy = (sort || this.state.currentSort).sortColumn;
      const sortDir = (sort || this.state.currentSort).sortOrder;
      const extra = {};
      this.props.commonActions.startLoading();
      yield this.props.emailActions.loadTemplates({
        body: Object.assign(Object.assign({
          active: true
        }, filter || this.state.currentFilter), extra),
        source,
        query: {
          sortBy: sortBy,
          sortDir: sortDir === null || sortDir === void 0 ? void 0 : sortDir.toUpperCase(),
          pageSize: pageSize,
          pageNo: pageNo
        }
      }, (_d = pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig) === null || _d === void 0 ? void 0 : _d.getTemplates);
      this.props.commonActions.stopLoading();
    });
  }
  onColFilterSelection(data) {
    this.setState({
      selectedColumns: data.value,
      showFilter: false,
      showEditColumns: !this.state.showEditColumns
    });
  }
  updateColumnUI() {
    return __awaiter$1(this, void 0, void 0, function* () {
      const columnsUI = (this.state.editColumnData || []).map(col => {
        if (col.name !== 'projectCode') {
          col.hide = !this.state.currentEditColumn[col.name];
        }
        return col;
      });
      const finalColumns = columnsUI.filter(i => !i.hide).map(i => i.name);
      this.setState({
        showFilter: false,
        showEditColumns: false,
        gridColumnUI: columnsUI,
        selectedColumns: finalColumns
      });
    });
  }
  onEditColumnToggle(data, field) {
    this.setState({
      showFilter: false,
      showEditColumns: !this.state.showEditColumns
    });
  }
  onFilterChange(data) {
    Object.keys(data.value).forEach(key => (data.value[key] == null || data.value[key] == '') && delete data.value[key]);
    this.setState({
      currentFilter: data.value
    });
  }
  handlePageChange(data) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const pageNo = data.value.currentPage;
      yield this.refreshTemplates({
        pageNo: pageNo
      });
    });
  }
  onGridAction(row, value, column) {
    var _a;
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        currentPage,
        pageSize
      } = this.props.store.emailtemplate.templatesPagination;
      const {
        pageData
      } = this.props;
      switch (value.action) {
        case 'edit':
          redirectTo('editEmailTemplate', {
            uid: row.uid /* replace with data.uid */
          });

          break;
        case 'delete':
          yield this.props.emailActions.deleteUser(row, (_a = pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig) === null || _a === void 0 ? void 0 : _a.deleteUser);
          yield this.refreshTemplates({
            filter: Object.assign({
              active: true
            }, this.state.currentFilter),
            pageSize: pageSize
          });
          break;
      }
    });
  }
  onFilterToggle() {
    this.setState({
      showFilter: !this.state.showFilter
    });
  }
  onFilterAction(data, action) {
    return __awaiter$1(this, void 0, void 0, function* () {
      this.setState({
        showFilter: false
      });
      if (data.value) {
        return;
      }
      const {
        currentPage
      } = this.props.store.emailtemplate.templatesPagination;
      switch (action.actionType) {
        case 'resetfilter':
          this.setState({
            currentFilter: {}
          });
          yield this.refreshTemplates({
            filter: {},
            currentPage
          });
          break;
        case 'applyfilter':
          yield this.refreshTemplates({
            filter: Object.assign({}, this.state.currentFilter),
            currentPage: currentPage
          });
          break;
      }
    });
  }
  users_OnSort(data, column) {
    return __awaiter$1(this, void 0, void 0, function* () {
      yield this.setState({
        currentSort: data
      });
      yield this.refreshTemplates({
        filter: Object.assign({}, this.state.currentFilter),
        sort: data
      });
    });
  }
  render() {
    var _a, _b, _c, _d, _e;
    const {
      store,
      pageData
    } = this.props;
    const {
      isLoading
    } = this.state;
    const createUserPerm = hasPermission('emailTemplate', store);
    const editUserPerm = hasPermission('emailTemplate', store);
    const deleteUserPerm = hasPermission('emailTemplate', store);
    const {
      Grid,
      Form,
      Dialog,
      Actions,
      Skeleton
    } = components$1.get();
    const {
      fieldMapping = {}
    } = pageData;
    // console.log('@@@@userstore', store);
    return jsx("div", {
      className: "sq-email-template-list sq-v-screen sq-v-screen--fixed",
      children: jsxs("div", {
        className: "sq-v-screen__container",
        children: [jsx("div", {
          className: "container-fluid",
          children: jsxs("div", {
            className: "sq-v-screen__top-header mt-wide",
            children: [jsx("div", {
              children: jsxs("h2", {
                className: "mt-3",
                children: ['Email Templates', !isLoading && ((_a = store.emailtemplate) === null || _a === void 0 ? void 0 : _a.templatesPagination) && `(${store.emailtemplate.templatesPagination.total})`]
              })
            }), jsx(Actions, {
              actions: [createUserPerm && {
                type: 'Button',
                iconName: 'add',
                size: 'medium',
                to: 'createNewEmailTemplate',
                buttonText: 'Add New'
              }, {
                type: 'Button',
                variant: 'outlined',
                onClick: this.onEditColumnToggle,
                iconName: 'edit',
                buttonText: 'Edit Columns',
                color: this.state.showEditColumns ? 'secondary' : 'primary'
              }, {
                type: 'Button',
                iconName: 'filter-list',
                variant: 'outlined',
                size: 'medium',
                onClick: this.onFilterToggle,
                buttonText: 'Filters',
                color: this.state.showFilter ? 'secondary' : 'primary'
              }].filter(i => i)
            })]
          })
        }), jsxs("div", {
          className: `sq-v-screen__body-container ${this.state.showFilter ? 'sq-email-template-list__container-filter' : ''}`,
          children: [jsx(Dialog, {
            open: this.state.showFilter,
            classes: {
              dialog: {
                root: 'sq-dialog--fixed-right'
              }
            },
            onClose: this.onFilterAction,
            onAction: this.onFilterAction,
            actions: [{
              actionType: 'applyfilter',
              buttonText: 'Apply'
            }, {
              actionType: 'resetfilter',
              buttonText: 'Reset',
              variant: 'outlined'
            }],
            title: 'Filters',
            children: jsx(Form, {
              onChange: this.onFilterChange,
              value: this.state.currentFilter,
              onAction: this.onFilterAction,
              name: 'userFilter',
              fields: [Object.assign(Object.assign({
                name: 'from',
                label: translate$4('From')
              }, fieldMapping.from), {
                cmpType: 'Input'
              }), Object.assign(Object.assign({
                name: 'name',
                label: translate$4('Name')
              }, fieldMapping.name), {
                cmpType: 'Input'
              }), Object.assign(Object.assign({
                name: 'status',
                cmpType: 'Autocomplete',
                label: translate$4('Status')
              }, fieldMapping.roleCode), {
                options: GLOBAL_OPTIONS.emailStatus.toArray()
              })]
            })
          }), jsx("div", {
            className: "sq-v-screen-grow",
            children: jsx(Grid, {
              paginationProps: {
                className: 'j-content-fl-end ml-3',
                enablePageSize: true,
                count: (_c = (_b = store.emailtemplate) === null || _b === void 0 ? void 0 : _b.templatesPagination) === null || _c === void 0 ? void 0 : _c.totalPages,
                value: (_d = store.emailtemplate) === null || _d === void 0 ? void 0 : _d.templatesPagination,
                disabled: isLoading,
                pageSizeOptions: GLOBAL_OPTIONS.noOfResultsDropdown.toArray(),
                onChange: data => {
                  const pageNo = data.value.currentPage;
                  const pageSize = data.value.pageSize;
                  if (!isLoading) {
                    this.refreshTemplates({
                      pageNo,
                      pageSize
                    });
                  }
                }
              },
              onColFilterChange: this.onColFilterSelection,
              className: "sq-basic-grid sq-grid--fixed sq-basic-grid--rounded",
              sortColumn: this.state.currentSort.sortColumn,
              sortOrder: this.state.currentSort.sortOrder,
              enableSort: true,
              onSort: this.users_OnSort,
              loader: jsx(Skeleton, {
                styleName: `grid-tran`,
                rows: 8
              }),
              onAction: this.onGridAction,
              selectedColumns: this.state.selectedColumns,
              showColSelection: this.state.showEditColumns,
              columns: [Object.assign({
                name: 'icon',
                cmpType: 'Icon',
                className: 'col-icon',
                sort: false,
                component: {
                  textIcon: row => `${string.getTwoChars(row.name)}`,
                  iconClass: 'sq-icon--square sq-icon--text-icon-accent',
                  variant: row => {
                    var _a;
                    return row.active ? accent.getColorByChar((_a = row.name) === null || _a === void 0 ? void 0 : _a.substr(0, 1)) : 'default';
                  }
                }
              }, fieldMapping.icon), Object.assign({
                name: 'name',
                cmpType: 'TextFields',
                headerText: 'Template Name',
                className: 'sq-email-template-list-screen--col-default',
                component: {
                  fields: [{
                    name: 'name',
                    className: ''
                  }, {
                    name: 'type',
                    className: 'sub-header'
                  }]
                }
              }, fieldMapping.firstName), Object.assign({
                name: 'subject',
                className: 'sq-email-template-list-screen--col-small',
                headerText: 'Subject'
              }, fieldMapping.subject), Object.assign({
                name: 'from',
                className: 'sq-email-template-list-screen--col-title',
                headerText: 'From Email ID',
                tooltip: {
                  title: row => row.from,
                  arrow: true
                }
              }, fieldMapping.from), {
                name: 'actions',
                headerText: 'Actions',
                className: 'col-actions',
                cmpType: 'MoreActions',
                sort: false,
                component: {
                  actions: [{
                    cmpType: 'LinkButton',
                    action: 'edit',
                    disabled: !editUserPerm,
                    iconName: 'edit',
                    buttonText: translate$4('Edit')
                  }, {
                    cmpType: 'LinkButton',
                    iconName: 'Delete',
                    confirm: {
                      title: translate$4('Confirm'),
                      content: translate$4('Are you sure you want to delete this template?')
                    },
                    iconColor: 'error',
                    action: 'delete',
                    disabled: !deleteUserPerm,
                    buttonText: translate$4('Delete'),
                    render: row => {
                      return row.active && deleteUserPerm;
                    }
                  }, {
                    cmpType: 'LinkButton',
                    iconName: 'active',
                    iconColor: 'success',
                    action: 'activate',
                    disabled: !createUserPerm,
                    buttonText: translate$4('Re-Activate'),
                    render: row => {
                      return !row.active;
                    }
                  }]
                }
              }],
              showHeader: true,
              data: ((_e = store === null || store === void 0 ? void 0 : store.emailtemplate) === null || _e === void 0 ? void 0 : _e.templates) || []
            })
          })]
        })]
      })
    });
  }
}
const mapStateToProps$4 = state => {
  return {
    store: Object.assign({}, state)
  };
};
const mapDispatchToProps$4 = dispatch => {
  return {
    emailActions: {
      loadTemplates: (data, config) => dispatch(loadTemplates(data, config))
    }
  };
};
EmailTemplateList.propTypes = {
  raiseAction: PropTypes.func,
  store: PropTypes.object,
  commonActions: PropTypes.object,
  appActions: PropTypes.object,
  location: PropTypes.object,
  history: PropTypes.object
};
var index$4 = connect(mapStateToProps$4, mapDispatchToProps$4)(EmailTemplateList);

const {
  preference: preference$1
} = storage;
const defaultPageSize$2 = GLOBAL_OPTIONS.noOfResultsDropdown.toArray()[0].value;
class GenericListing extends Component$1 {
  constructor(props) {
    super(props);
    this.state = {
      isLoading: false,
      showEditColumns: false
    };
    this.handleAction = this.handleAction.bind(this);
    this.onFilterChange = this.onFilterChange.bind(this);
    this.onQuickFilterChange = this.onQuickFilterChange.bind(this);
    this.handlePageChange = this.handlePageChange.bind(this);
    this.handleOnSortChange = this.handleOnSortChange.bind(this);
    this.onGridAction = this.onGridAction.bind(this);
    this.onEditColumnChange = this.onEditColumnChange.bind(this);
    this.onFilterAction = this.onFilterAction.bind(this);
    this.onGridChange = this.onGridChange.bind(this);
    this.onTopFilterChange = this.onTopFilterChange.bind(this);
    this.onPopupCloseAction = this.onPopupCloseAction.bind(this);
    this.filterActions = [{
      actionType: 'edit-filter',
      iconName: 'filter-list',
      variant: 'outlined',
      cmpType: 'Button',
      buttonText: translate$5('Filters')
    }];
    events.subscribe('onPopupScreenCloseAction', this.onPopupCloseAction);
  }
  onPopupCloseAction(action) {
    if (action.refreshAfter) {
      this.refreshData({});
    }
  }
  getKey(name) {
    const {
      pageData
    } = this.props;
    return `${pageData.dataKey || window.location.pathname}_${name}`;
  }
  componentDidUpdate(preProps) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        pageData: prevPageData
      } = preProps;
      const {
        pageData
      } = this.props;
      if (prevPageData.dataKey !== pageData.dataKey) {
        this.componentDidMount();
      }
    });
  }
  processField(field) {
    const {
      userData,
      store
    } = this.props;
    return Object.assign(Object.assign({}, field), {
      label: processEachParam(userData, field.label, undefined, store),
      beforeRender: (field, val, row) => {
        return Object.assign({}, field.componentProps ? processParams(Object.assign(Object.assign({}, userData), row), field.componentProps, undefined, store) : {});
      },
      fields: field.fields ? field.fields.map(field => {
        return this.processField(field);
      }) : undefined
    });
  }
  componentDidMount() {
    var _a, _b, _c, _d;
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        pageData,
        userData,
        store
      } = this.props;
      const overrideParams = {
        filterParams: {},
        quickFilterParams: {},
        topFilterParams: {}
      };
      const queryParams = Object.assign(Object.assign({}, processParams(userData, pageData.initialValues || {})), query$1.get());
      this.filterFields = (_a = pageData.filterFields) === null || _a === void 0 ? void 0 : _a.map(field => {
        if (queryParams[field.name]) {
          overrideParams.filterParams[field.name] = queryParams[field.name];
        }
        return this.processField(field);
      });
      this.topFilterFields = (_b = pageData.topFilterFields) === null || _b === void 0 ? void 0 : _b.map(field => {
        if (queryParams[field.name]) {
          overrideParams.topFilterParams[field.name] = queryParams[field.name];
        }
        return this.processField(field);
      });
      this.quickFilterFields = (_c = pageData.quickFilterFields) === null || _c === void 0 ? void 0 : _c.map(field => {
        if (queryParams[field.name]) {
          overrideParams.quickFilterParams[field.name] = queryParams[field.name];
        }
        return this.processField(field);
      });
      this.topActions = (_d = pageData.topActions) === null || _d === void 0 ? void 0 : _d.map(action => {
        return Object.assign(Object.assign({}, action), {
          buttonText: processEachParam(userData, action.buttonText, undefined, store),
          beforeRender: (col, val, row) => {
            let match = true;
            if (action === null || action === void 0 ? void 0 : action.renderMatch) {
              const valid = new Validator$1(action === null || action === void 0 ? void 0 : action.renderMatch);
              valid.setValues(userData);
              match = valid.validateAll();
            }
            if (match === false) {
              return false;
            }
            return Object.assign({}, action.componentProps ? processParams(Object.assign(Object.assign({}, userData), row), action.componentProps, undefined, store) : {});
          }
        });
      });
      this.pageFields = this.columns = pageData.columns.map(item => {
        return Object.assign(Object.assign({}, item), {
          headerText: processEachParam(userData, item.headerText, undefined, store),
          beforeRender: (col, val, row) => {
            return {
              component: Object.assign(Object.assign({}, item.componentProps ? processParams(Object.assign(Object.assign({}, userData), row), item.componentProps, undefined, store) : {}), {
                fields: item.fields ? item.fields.map(field => {
                  return Object.assign(Object.assign({}, field), {
                    beforeRender: (col, val, row) => {
                      return field.componentProps ? {
                        component: processParams(Object.assign(Object.assign({}, userData), row), field.componentProps, undefined, store)
                      } : {};
                    }
                  });
                }) : undefined,
                actions: item.actions ? item.actions.map(action => {
                  return Object.assign(Object.assign(Object.assign({}, action), processParams(Object.assign(Object.assign({}, userData), row), action, undefined, store)), {
                    render: row => {
                      if (action.renderMatch) {
                        const vald = new Validator$1(action.renderMatch);
                        vald.setValues(row);
                        if (!vald.validateAll()) {
                          return false;
                        }
                      }
                      return true;
                    }
                  });
                }) : undefined
              })
            };
          }
        });
      });
      const objToSave = {
        lastQuery: query$1.get(),
        currentSort: Object.assign(Object.assign({}, pageData.currentSort || {}), preference$1.read('currentSort')),
        currentFilter: Object.assign(Object.assign({}, queryParams.savedFilter === undefined ? preference$1.read('currentFilter') : {}), overrideParams.filterParams),
        currentQuickFilter: Object.assign(Object.assign({}, queryParams.savedFilter === undefined ? preference$1.read('quickFilter') : {}), overrideParams.topFilterParams),
        topFilter: Object.assign(Object.assign({}, queryParams.savedFilter === undefined ? preference$1.read('topFilter') : {}), overrideParams.topFilterParams),
        selectedColumns: preference$1.read('selectedColumns', true) || pageData.defaultColumns || undefined,
        columnsOrder: preference$1.read('columnsOrder', true)
      };
      yield this.props.raiseAction(updateUserData({
        lastQuery: objToSave.lastQuery,
        [`${this.getKey('currentSort')}`]: objToSave.currentSort,
        [`${this.getKey('currentFilter')}`]: objToSave.currentFilter,
        [`${this.getKey('currentQuickFilter')}`]: objToSave.currentQuickFilter,
        [`${this.getKey('topFilter')}`]: objToSave.topFilter,
        [`${this.getKey('selectedColumns')}`]: objToSave.selectedColumns,
        [`${this.getKey('columnsOrder')}`]: objToSave.columnsOrder
      }));
      this.refreshData();
    });
  }
  onFilterChange(data, field) {
    return __awaiter$1(this, void 0, void 0, function* () {
      this.setState({
        __currentFilter: data.value
      });
      yield this.props.raiseAction(updateUserData({
        [`${this.getKey('tempCurrentFilter')}`]: data.value
      }));
    });
  }
  onQuickFilterChange(data, action) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        onAction
      } = this.props;
      this.props.raiseAction(startLoading());
      if (action.actionType) {
        action.currentData = data.value;
        onAction && onAction(data.value, action);
      }
      yield this.props.raiseAction(updateUserData({
        [`${this.getKey('currentQuickFilter')}`]: data.value
      }));
      yield this.refreshData({
        pageNo: 1
      });
      preference$1.write('quickFilter', data.value);
      this.props.raiseAction(stopLoading());
    });
  }
  onTopFilterChange(data, action) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        onAction
      } = this.props;
      this.props.raiseAction(startLoading());
      yield this.props.raiseAction(updateUserData({
        [`${this.getKey('topFilter')}`]: data.value
      }));
      if (action.actionType) {
        action.currentData = data.value;
        (yield onAction) && onAction(data.value, action);
      }
      yield this.refreshData({
        pageNo: 1
      });
      preference$1.write('topFilter', data.value);
      this.props.raiseAction(stopLoading());
    });
  }
  getBookmarkableFields(keys, data) {
    if (!keys) {
      return data;
    }
    const obj = {};
    Object.keys(data).forEach(itemKey => {
      if ((keys === null || keys === void 0 ? void 0 : keys.indexOf(itemKey)) > -1) {
        obj[itemKey] = data[itemKey];
      }
    });
    return obj;
  }
  refreshData({
    filter,
    sort,
    pageSize,
    pageNo
  } = {}) {
    var _a, _b, _c, _d, _e;
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        pageData,
        data,
        userData,
        store,
        raiseAction,
        onAction
      } = this.props;
      if ((_a = pageData.apiConfig) === null || _a === void 0 ? void 0 : _a.search) {
        this.setState({
          isLoading: true
        });
        const pagination = userData[this.getKey('pagination')] || {};
        const currentSort = userData[this.getKey('currentSort')] || {};
        pageSize = pageSize || pagination.pageSize || pageData.defaultPageSize || defaultPageSize$2;
        pageNo = pageNo || (pagination === null || pagination === void 0 ? void 0 : pagination.currentPage) || 1;
        const sortBy = (sort || currentSort).sortColumn;
        const sortDir = (sort || currentSort).sortOrder;
        if (pageData.bookmarkable === true) {
          const fields = this.getBookmarkableFields(pageData.bookmarkableFields, Object.assign(Object.assign(Object.assign({}, filter || userData[this.getKey('currentFilter')] || {}), userData[this.getKey('currentQuickFilter')] || {}), userData[this.getKey('topFilter')] || {}));
          onAction && onAction({}, {
            actionType: 'self-redirect',
            urlParams: fields
          });
        }
        yield raiseAction(postApi(Object.assign(Object.assign({}, (_b = pageData.apiConfig) === null || _b === void 0 ? void 0 : _b.search), {
          params: object$1.extendData(processParams(userData, (_c = pageData.apiConfig) === null || _c === void 0 ? void 0 : _c.search.params, undefined, store), Object.assign(Object.assign(Object.assign({}, filter || userData[this.getKey('currentFilter')] || {}), userData[this.getKey('currentQuickFilter')] || {}), userData[this.getKey('topFilter')] || {})),
          query: ((_d = pageData.apiConfig) === null || _d === void 0 ? void 0 : _d.search.query) ? processParams(Object.assign(Object.assign({}, userData), {
            sortBy: `${sortBy}|${sortDir}`,
            pageSize: pageSize,
            pageNo: pageNo
          }), (_e = pageData.apiConfig) === null || _e === void 0 ? void 0 : _e.search.query) : {
            sortBy: sortBy && sortDir ? `${sortBy}|${sortDir}` : undefined,
            pageSize: pageSize,
            pageNo: pageNo
          },
          dataKey: this.getKey('results')
        }), data));
        this.setState({
          isLoading: false
        });
      }
    });
  }
  handlePageChange(data) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const pageNo = data.value.currentPage,
        pageSize = data.value.pageSize;
      this.props.raiseAction(startLoading());
      yield this.refreshData({
        pageNo,
        pageSize
      });
      this.props.raiseAction(stopLoading());
    });
  }
  handleAction(data, action) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        onAction,
        userData
      } = this.props;
      switch (action.actionType) {
        case 'edit-cols':
          this.setState({
            showEditColumns: true
          });
          break;
        case 'edit-filter':
          this.setState({
            showFilter: true
          });
          this.setState({
            __currentFilter: userData[this.getKey('currentFilter')]
          });
          break;
        default:
          if (action.actionType) {
            action.currentData = data;
            onAction && onAction(data, action);
          }
      }
    });
  }
  onGridAction(row, action, _) {
    return __awaiter$1(this, void 0, void 0, function* () {
      switch (action.actionType) {
        default:
          this.props.raiseAction(startLoading());
          action.currentData = row;
          this.props.onAction && (yield this.props.onAction(row, action));
          if (action.refreshAfter) {
            yield this.refreshData({});
          }
          this.props.raiseAction(stopLoading());
      }
    });
  }
  onGridChange(data, action, row) {
    return __awaiter$1(this, void 0, void 0, function* () {
      if (action.actionType) {
        this.props.raiseAction(startLoading());
        action.currentData = Object.assign(Object.assign({}, row), data);
        this.props.onAction && (yield this.props.onAction(Object.assign(Object.assign({}, row), data), action));
        if (action.refreshAfter) {
          yield this.refreshData({});
        }
        this.props.raiseAction(stopLoading());
      }
    });
  }
  onFilterAction(data, action) {
    return __awaiter$1(this, void 0, void 0, function* () {
      switch (action.actionType) {
        case 'resetFilter':
          yield this.setState({
            showFilter: !this.state.showFilter
          });
          this.props.raiseAction(startLoading());
          yield this.props.raiseAction(updateUserData({
            [`${this.getKey('currentFilter')}`]: {},
            [`${this.getKey('tempCurrentFilter')}`]: {}
          }));
          preference$1.write('currentFilter', {});
          yield this.setState({
            __currentFilter: undefined
          });
          yield this.refreshData({
            filter: {},
            pageNo: 1
          });
          this.props.raiseAction(stopLoading());
          break;
        case 'applyFilter':
          yield this.props.raiseAction(updateUserData({
            [`${this.getKey('currentFilter')}`]: this.state.__currentFilter,
            [`${this.getKey('tempCurrentFilter')}`]: this.state.__currentFilter
          }));
          preference$1.write('currentFilter', this.state.__currentFilter);
          yield this.setState({
            showFilter: !this.state.showFilter,
            __currentFilter: undefined
          });
          this.props.raiseAction(startLoading());
          yield this.refreshData({});
          this.props.raiseAction(stopLoading());
          break;
      }
    });
  }
  handleOnSortChange(data) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        currentPage
      } = this.props.userData[this.getKey('results')] || {};
      preference$1.write('currentSort', data);
      yield this.props.raiseAction(updateUserData({
        [`${this.getKey('currentSort')}`]: data
      }));
      this.props.raiseAction(startLoading());
      yield this.refreshData({
        pageNo: currentPage,
        sort: data
      });
      this.props.raiseAction(stopLoading());
    });
  }
  onEditColumnChange(data) {
    const {
      userData
    } = this.props;
    if (!data.cancel) {
      this.props.raiseAction(updateUserData({
        [`${this.getKey('selectedColumns')}`]: data.value,
        [`${this.getKey('columnsOrder')}`]: data.columnsOrder ? data.columnsOrder : userData[this.getKey('columnsOrder')]
      }));
      const objToSave = {
        selectedColumns: data.value
      };
      if (data.columnsOrder) {
        objToSave.columnsOrder = data.columnsOrder;
      }
      preference$1.writeAll(objToSave);
    }
    this.setState({
      showEditColumns: !this.state.showEditColumns
    });
  }
  render() {
    var _a, _b;
    const {
      pageData = {},
      userData
    } = this.props;
    const {
      className = ''
    } = pageData;
    const {
      Actions,
      Dialog,
      Form,
      Grid,
      Skeleton
    } = components$1.get();
    const currentSort = userData[this.getKey('currentSort')];
    const currentFilter = userData[this.getKey('currentFilter')];
    const currentQuickFilter = userData[this.getKey('currentQuickFilter')];
    const topFilter = userData[this.getKey('topFilter')];
    const selectedColumns = userData[this.getKey('selectedColumns')];
    const columnsOrder = userData[this.getKey('columnsOrder')];
    return jsxs("div", {
      className: `sq-generic-listing sq-v-screen sq-v-screen--fixed ${className}`,
      children: [jsxs("div", {
        className: "sq-v-screen__container",
        children: [jsxs("div", {
          className: "sq-v-screen__sub-header",
          children: [pageData.showPageTitle && jsx("div", {
            className: 'sq-generic-listing__header',
            children: pageData.title
          }), this.topFilterFields && jsx("div", {
            className: 'sq-generic-listing__quick',
            children: jsx(Form, {
              disabled: this.state.isLoading,
              userData: userData,
              onChange: this.onTopFilterChange,
              className: "sq-form--inline-auto p-0",
              value: topFilter,
              fields: this.topFilterFields
            })
          }), jsx(Actions, {
            className: "w-auto",
            userData: userData,
            disabled: this.state.isLoading,
            onAction: this.handleAction,
            actions: [...(this.topActions || []), {
              buttonText: translate$5('Edit Columns'),
              iconName: 'edit',
              variant: 'outlined',
              cmpType: 'Button',
              actionType: 'edit-cols'
            }, ...(this.filterFields ? this.filterActions : [])]
          })]
        }), jsxs("div", {
          className: `sq-v-screen__container`,
          children: [jsx("div", {
            className: "container-fluid",
            children: jsx("div", {
              className: "sq-v-screen__pagination-bar d-flex fl-a-items-center justify-content-end mb-2",
              children: this.quickFilterFields && jsx("div", {
                className: 'sq-generic-listing__quick mt-2',
                children: jsx(Form, {
                  disabled: this.state.isLoading,
                  userData: userData,
                  onChange: this.onQuickFilterChange,
                  className: "sq-form--inline-auto p-0",
                  value: currentQuickFilter,
                  fields: this.quickFilterFields
                })
              })
            })
          }), jsx("div", {
            className: "sq-generic-listing__grid-wrapper sq-v-screen-grow",
            children: jsx(Grid, {
              paginationProps: {
                onChange: this.handlePageChange,
                count: (_a = userData[this.getKey('results')]) === null || _a === void 0 ? void 0 : _a.totalPages,
                value: userData[this.getKey('results')],
                enablePageSize: true,
                defaultPageSize: pageData.defaultPageSize || defaultPageSize$2,
                pageSizeOptions: GLOBAL_OPTIONS.noOfResultsDropdown.toArray()
              },
              disabled: this.state.isLoading,
              className: "sq-basic-grid sq-grid--fixed",
              loader: jsx(Skeleton, {
                styleName: `grid-tran`,
                rows: 4
              }),
              onChange: this.onGridChange,
              onColFilterChange: this.onEditColumnChange,
              selectedColumns: selectedColumns,
              columnsOrder: columnsOrder,
              showColSelection: this.state.showEditColumns,
              onAction: this.onGridAction,
              sortColumn: currentSort === null || currentSort === void 0 ? void 0 : currentSort.sortColumn,
              sortOrder: currentSort === null || currentSort === void 0 ? void 0 : currentSort.sortOrder,
              onSort: this.handleOnSortChange,
              columns: this.columns,
              enableSort: pageData.enableSort === undefined ? true : pageData.enableSort,
              showHeader: pageData.showHeader === undefined ? true : pageData.showHeader,
              data: (_b = userData[this.getKey('results')]) === null || _b === void 0 ? void 0 : _b.data
            })
          })]
        })]
      }), jsx(Fragment, {
        children: jsx(Dialog, {
          open: this.state.showFilter,
          transitionDir: "left",
          classes: {
            dialog: {
              root: 'sq-dialog--fixed-right'
            }
          },
          onClose: () => this.setState({
            showFilter: false
          }),
          onAction: this.onFilterAction,
          actions: [{
            actionType: 'applyFilter',
            buttonText: translate$5('Apply')
          }, {
            actionType: 'resetFilter',
            buttonText: translate$5('Reset'),
            variant: 'outlined'
          }],
          title: 'Filters',
          children: jsx(Form, {
            onChange: this.onFilterChange,
            userData: userData,
            className: "mt-wide",
            value: this.state.__currentFilter || currentFilter,
            fields: this.filterFields
          })
        })
      })]
    });
  }
}
GenericListing.propTypes = {};

class LayoutContent extends Component$1 {
  constructor(props) {
    super(props);
    this.state = {};
  }
  componentDidMount() {
    const {
      metaData = {},
      pageData = {}
    } = this.props;
    eval(pageData.javascript);
  }
  render() {
    const _a = this.props,
      {
        metaData = {},
        pageData = {}
      } = _a,
      rest = __rest(_a, ["metaData", "pageData"]);
    const {
      className = '',
      rows = []
    } = pageData;
    return jsx("div", {
      className: `sq-layout-content sq-content-page__body ${className}`,
      children: jsx("div", {
        className: "container-fluid",
        children: rows.map((rowObj, idx) => {
          const {
            pageLayout = {},
            className = 'row'
          } = rowObj;
          return jsx("div", {
            className: `${className}`,
            children: Object.keys(pageLayout).map(layoutKey => {
              const layoutObj = pageLayout[layoutKey];
              const {
                  className = ''
                } = layoutObj,
                restLayout = __rest(layoutObj, ["className"]);
              return jsx("div", {
                className: `sq-layout-content__block ${layoutKey} ${className}`,
                children: jsx(Content, Object.assign({
                  flat: true
                }, rest, {
                  pageData: Object.assign({}, restLayout)
                }))
              }, layoutKey);
            })
          }, idx);
        })
      })
    });
  }
}
LayoutContent.propTypes = {};

const MediaGallery = ({
  pageData,
  className: _className = '',
  appStore,
  raiseAction,
  onAction
}) => {
  var _a, _b, _c, _d, _e, _f;
  const {
    allowedFileTypes = ['JPG', 'JPEG', 'PNG', 'mp4', 'gif']
  } = pageData;
  useEffect(() => {
    loadImages();
  }, []);
  const [uploadDialog, setUploadDialog] = useState(false);
  const [status, setStatus] = useState(undefined);
  const loadImages = ({
    pageNo: _pageNo = 1,
    filter: _filter = {}
  } = {}) => __awaiter$1(void 0, void 0, void 0, function* () {
    var _g;
    const {
      pageSize
    } = ((_g = appStore.admin) === null || _g === void 0 ? void 0 : _g.mediaPage) || {};
    raiseAction(startLoading());
    yield raiseAction(loadMedia({
      body: Object.assign({
        status
      }, _filter),
      query: {
        pageSize: pageSize,
        pageNo: _pageNo
      }
    }, pageData.searchMedia));
    raiseAction(stopLoading());
  });
  const handlePageChange = data => {
    loadImages({
      pageNo: data.value.currentPage
    });
  };
  const handleAction = (item, action) => __awaiter$1(void 0, void 0, void 0, function* () {
    var _h, _j, _k;
    switch (action.actionType) {
      case 'copy-link':
        raiseAction(copyMediaLink(item.filePath));
        break;
      case 'remove':
        raiseAction(startLoading());
        yield raiseAction(deleteLink(item, pageData.deleteMedia));
        loadImages({
          pageNo: (_h = appStore.admin) === null || _h === void 0 ? void 0 : _h.mediaPage.currentPage
        });
        raiseAction(stopLoading());
        break;
      case 'publish':
        raiseAction(startLoading());
        yield raiseAction(updateMedia(Object.assign(Object.assign({}, item), {
          status: 'PUBLISHED'
        }), pageData.updateMedia));
        loadImages({
          pageNo: (_j = appStore.admin) === null || _j === void 0 ? void 0 : _j.mediaPage.currentPage
        });
        raiseAction(stopLoading());
        break;
      case 'draft':
        raiseAction(startLoading());
        yield raiseAction(updateMedia(Object.assign(Object.assign({}, item), {
          status: 'DRAFT'
        }), pageData.updateMedia));
        loadImages({
          pageNo: (_k = appStore.admin) === null || _k === void 0 ? void 0 : _k.mediaPage.currentPage
        });
        raiseAction(stopLoading());
        break;
      default:
        onAction && onAction(item, action);
    }
  });
  const handleFormChange = data => {};
  const handleButtonSelectionChange = data => {
    var _a;
    setStatus(data.value);
    loadImages({
      pageNo: (_a = appStore.admin) === null || _a === void 0 ? void 0 : _a.mediaPage.currentPage,
      filter: {
        status: data.value
      }
    });
  };
  const handleFormAction = (data, action) => __awaiter$1(void 0, void 0, void 0, function* () {
    raiseAction(startLoading());
    yield raiseAction(uploadMedia(action, pageData.uploadMedia));
    setUploadDialog(false);
    raiseAction(stopLoading());
  });
  const handleAddNewClick = d => {
    setUploadDialog(true);
  };
  const {
    Pagination,
    ImageCardList,
    ButtonSelection,
    Skeleton,
    Button,
    Form,
    Dialog
  } = components$1.get();
  const {
    totalPages
  } = ((_a = appStore.admin) === null || _a === void 0 ? void 0 : _a.mediaPage) || {};
  return jsx("div", {
    className: "sq-media-gallery sq-v-screen",
    children: jsxs("div", {
      className: `sq-v-screen__container`,
      children: [jsxs("div", {
        className: "container-fluid d-flex j-content-fl-end sq-media-gallery__top-actions",
        children: [jsx("div", {
          className: "fl-grow j-content-fl-start",
          children: jsx(ButtonSelection, {
            options: GLOBAL_OPTIONS.mediaStatus.toArray(),
            value: status,
            onChange: handleButtonSelectionChange
          })
        }), jsx(Button, {
          iconName: "add",
          buttonText: pageData.addMediaText || 'Add Media',
          onClick: handleAddNewClick
        }), ((_b = appStore.admin) === null || _b === void 0 ? void 0 : _b.mediaPage) && jsx(Pagination, {
          onChange: handlePageChange,
          count: totalPages,
          value: (_c = appStore.admin) === null || _c === void 0 ? void 0 : _c.mediaPage
        })]
      }), jsx("div", {
        className: "sq-media-gallery__list",
        children: jsx("div", {
          className: "sq-v-screen-grow container-fluid",
          children: jsx(ImageCardList, {
            imageUrlField: ((_d = pageData.fields) === null || _d === void 0 ? void 0 : _d.imageUrlField) || 'url',
            titleField: ((_e = pageData.fields) === null || _e === void 0 ? void 0 : _e.imageTitleField) || 'fileName',
            data: (_f = appStore.admin) === null || _f === void 0 ? void 0 : _f.media,
            tagColor: 'info',
            tagFieldName: 'status',
            loader: jsx(Skeleton, {
              styleName: "projects",
              rows: 5
            }),
            onAction: handleAction,
            actions: [{
              cmpType: 'Button',
              iconSize: 'small',
              className: 'sq-button--block',
              buttonText: 'Copy Link',
              actionType: 'copy-link'
            }, {
              cmpType: 'Button',
              iconSize: 'small',
              className: 'sq-button--block',
              beforeRender: (action, col, row) => {
                return !row.status || row.status === 'DRAFT';
              },
              buttonText: 'Publish',
              color: 'info',
              actionType: 'publish'
            }, {
              cmpType: 'Button',
              iconSize: 'small',
              className: 'sq-button--block',
              buttonText: 'Move to Draft',
              actionType: 'draft',
              beforeRender: (action, col, row) => row.status === 'PUBLISHED'
            }, {
              cmpType: 'Button',
              iconSize: 'small',
              className: 'sq-button--block',
              buttonText: 'Remove',
              color: 'error',
              actionType: 'remove',
              confirm: {
                title: 'Confirmation',
                content: 'Are you sure to remove this image?'
              }
            }, ...(pageData.imageActions || [])]
          })
        })
      }), jsx(Dialog, {
        title: 'Upload media',
        classes: {
          body: 'sq-dialog__content-body--auto'
        },
        closeButton: true,
        open: uploadDialog,
        onClose: () => setUploadDialog(false),
        children: jsx(Form, {
          className: 'pt-4',
          fields: [Object.assign({
            cmpType: 'FileUploader',
            uploadOnChange: false,
            actionType: 'module',
            name: 'files',
            multiple: false,
            fileTypes: allowedFileTypes
          }, pageData.uploadConfig), {
            cmpType: 'Alert',
            message: "Please ensure that the image name doesn't have any spaces."
          }],
          onAction: handleFormAction,
          onChange: handleFormChange
        })
      })]
    })
  });
};
MediaGallery.propTypes = {
  className: PropTypes.string
};

class MultiView extends Component$1 {
  constructor(props) {
    super(props);
    this.state = {};
  }
  render() {
    const {
      BottomNavigation
    } = components$1.get();
    const _a = this.props,
      {
        className = '',
        views = [],
        defaultView
      } = _a,
      rest = __rest(_a, ["className", "views", "defaultView"]);
    defaultView || (views.length > 0 ? views[0].name : '');
    const {
      routeName,
      location = {}
    } = rest;
    const {
      pathname
    } = location;
    return jsxs("div", {
      className: `multi-view screen ${className}`,
      children: [jsx("div", {
        className: "screen__content",
        children: jsx(Routes, {
          children: views.map(view => {
            const {
              container: Container,
              name
            } = view;
            return jsx(Route, {
              path: `${routeName}/${name}`,
              element: jsx(Container, Object.assign({}, rest))
            }, name);
          })
        })
      }), jsx("div", {
        className: "screen__footer",
        children: jsx(BottomNavigation, {
          onChange: value => {
            redirectTo(value);
          },
          iconOnly: browser.breakpoints.down('sm'),
          value: pathname,
          links: views.map(view => {
            return {
              label: view.label,
              icon: view.iconName,
              value: `${routeName}/${view.name}`
            };
          })
        })
      })]
    });
  }
}
MultiView.propTypes = {
  commonStore: PropTypes.object,
  authStore: PropTypes.object,
  store: PropTypes.object
};

class BaseContainer extends Component$1 {
  trackPageView() {
    const {
      load
    } = this.analytics || {};
    const {
      onAnalytics
    } = this.props;
    if (load) {
      onAnalytics && onAnalytics(Object.assign({
        type: 'pageview'
      }, load));
    }
  }
  trackEvent({
    eventName,
    category,
    section,
    action,
    label
  }) {
    const {
      onAnalytics
    } = this.props;
    eventName && onAnalytics && onAnalytics({
      type: 'event',
      eventName,
      category,
      section,
      action,
      label
    });
  }
  render() {
    return jsx("div", {
      className: "container-fluid",
      children: jsx("h1", {
        children: "Not Implemented"
      })
    });
  }
}
BaseContainer.propTypes = {
  onAnalytics: PropTypes.func,
  userStore: PropTypes.object,
  commonStore: PropTypes.object
};

var clonePageConfig = ((_a = {}) => {
  var _b, _c, _d;
  var {
      formData = {}
    } = _a,
    config = __rest(_a, ["formData"]);
  return {
    pageData: {
      init: {
        formData: Object.assign(Object.assign({
          to: '/content/',
          name: formData.name || ((_b = formData.pageData) === null || _b === void 0 ? void 0 : _b.title)
        }, formData), config.init)
      },
      items: [{
        component: 'Form',
        name: 'formData',
        fields: [{
          cmpType: 'Input',
          name: 'name',
          label: 'Name'
        }, {
          cmpType: 'DataField',
          name: 'path',
          label: 'Clone From'
        }, {
          cmpType: 'Input',
          name: 'to',
          label: 'Target path',
          validators: [Object.assign({
            type: 'required',
            defaultValue: '/content/'
          }, (_c = config.targetPath) === null || _c === void 0 ? void 0 : _c.required), {
            type: 'path'
          }, Object.assign({
            type: 'startsWith',
            startsWith: '/content/'
          }, (_d = config.targetPath) === null || _d === void 0 ? void 0 : _d.starts)]
        }]
      }, {
        component: 'Button',
        actionType: 'submit-event',
        buttonText: 'Clone',
        params: {
          '...': '.formData',
          name: '.formData.name',
          from: '.formData.path',
          to: '.formData.to',
          type: '.formData.type'
        }
      }]
    }
  };
});

function PathTree({
  value,
  data = [],
  onChange
}) {
  const handleChange = (data, val) => {
    if (val !== value) {
      onChange && onChange({
        value: val
      });
    }
  };
  const renderTree = nodes => jsx(TreeItem, {
    nodeId: nodes.path,
    label: nodes.name,
    children: Array.isArray(nodes.children) ? nodes.children.map(node => renderTree(node)) : null
  }, nodes.path);
  return jsx(TreeView, {
    className: "sq-path-tree",
    "aria-label": "rich object",
    defaultCollapseIcon: jsx(ExpandMoreIcon, {}),
    defaultExpanded: ['root'],
    defaultExpandIcon: jsx(ChevronRightIcon, {}),
    onNodeSelect: handleChange,
    sx: {
      height: 110,
      flexGrow: 1,
      maxWidth: 400,
      overflowY: 'auto'
    },
    children: renderTree(data)
  });
}

const {
  translate: translate$3
} = translate$6;
class PageListing extends BaseContainer {
  constructor(props) {
    super(props);
    this.onQuickFilterChange = (data, action) => __awaiter$1(this, void 0, void 0, function* () {
      yield this.setState({
        __currentFilter: data.value
      });
    });
    this.state = {
      showFilter: false,
      currentQuickFilter: {}
    };
    this.onGridAction = this.onGridAction.bind(this);
    this.toggleEditForm = this.toggleEditForm.bind(this);
    this.onCloneFormSubmit = this.onCloneFormSubmit.bind(this);
    this.handleAction = this.handleAction.bind(this);
    this.onTreeSelect = this.onTreeSelect.bind(this);
    this.onFilterAction = this.onFilterAction.bind(this);
  }
  processField(field) {
    const {
      userData,
      store
    } = this.props;
    return Object.assign(Object.assign({}, field), {
      beforeRender: (field, val, row) => {
        return Object.assign({}, field.componentProps ? processParams(Object.assign(Object.assign({}, userData), row), field.componentProps, undefined, store) : {});
      },
      fields: field.fields ? field.fields.map(field => {
        return this.processField(field);
      }) : undefined
    });
  }
  componentDidMount() {
    var _a;
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        pageData,
        store
      } = this.props;
      this.props.commonActions.startLoading();
      if (pageData.enableTree !== false) {
        yield this.props.raiseAction(loadPageTree({}, pageData.getPageTreeConfig));
        yield this.setState({
          parentPath: (_a = store.admin.contentTree) === null || _a === void 0 ? void 0 : _a.path
        });
      }
      yield this.setState({
        currentFilter: preference$2.read('currentFilter')
      });
      this.filterFields = [{
        name: 'pageName',
        label: 'Name'
      }, {
        name: 'path',
        label: 'Path'
      }, {
        name: 'type',
        cmpType: 'ButtonSelection',
        label: 'Page Type',
        options: GLOBAL_OPTIONS.contentType.toArray()
      }].concat((pageData.filterFields || []).map(field => {
        return this.processField(field);
      }));
      this.refreshPages();
    });
  }
  refreshPages(options = {}) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        parentPath = this.state.parentPath,
        pageNo = 1,
        pageSize = GLOBAL_OPTIONS.noOfResultsDropdown.toArray()[0].value
      } = options;
      const {
        pageData
      } = this.props;
      this.props.commonActions.startLoading();
      yield this.props.raiseAction(loadPagesByPath(Object.assign(Object.assign({}, this.state.currentFilter), {
        parentPath
      }), pageData.getPagesConfig, {
        pageNo,
        pageSize
      }));
      this.props.commonActions.stopLoading();
    });
  }
  componentDidUpdate(prevProps) {
    const {
      pageData
    } = this.props;
    const {
      pageData: prevPageData
    } = prevProps;
    if (pageData.id !== prevPageData.id) {
      this.refreshPages();
    }
  }
  getCurrentPage() {
    var _a, _b;
    const {
      store
    } = this.props;
    return {
      pageNo: (_a = store.admin) === null || _a === void 0 ? void 0 : _a.contentPagesPagination.currentPage,
      pageSize: (_b = store.admin) === null || _b === void 0 ? void 0 : _b.contentPagesPagination.pageSize
    };
  }
  onGridAction(row, value, column) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        pageData,
        store
      } = this.props;
      let currentPage;
      switch (value.action) {
        case 'clone':
          this.setState({
            openClone: true,
            cloneFrom: row
          });
          break;
        case 'delete':
          this.props.commonActions.startLoading();
          yield this.props.raiseAction(deletePage(row, pageData.contentPageConfig));
          currentPage = this.getCurrentPage();
          this.refreshPages(Object.assign({}, currentPage));
          break;
        case 'edit':
          redirectTo(row.type === 'SITE_MAP' ? pageData.editSiteMap || 'editSiteMap' : pageData.editPage || 'editPage', {
            path: row.path,
            pageId: row.pageId /* replace with data.uid */
          }, {
            target: '_blank'
          });
          break;
        case 'preview':
          redirectTo(row.path, {}, {
            target: '_blank'
          });
          break;
      }
    });
  }
  onCloneFormSubmit(data) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        pageData
      } = this.props;
      this.props.commonActions.startLoading();
      yield this.props.raiseAction(clonePage(data, pageData.clonePageConfig, {
        success: () => {
          const currentPage = this.getCurrentPage();
          this.refreshPages(Object.assign({}, currentPage));
          this.setState({
            openClone: false
          });
        },
        error: resp => {
          var _a;
          this.props.commonActions.stopLoading();
          if ((_a = resp.error) === null || _a === void 0 ? void 0 : _a.errors) {
            this.props.raiseAction(updateUserData({
              formData_errors: resp.error.errors
            }));
          }
        }
      }));
    });
  }
  toggleEditForm() {
    return __awaiter$1(this, void 0, void 0, function* () {
      this.setState({
        openClone: false
      });
    });
  }
  onTreeSelect(data) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        pageData
      } = this.props;
      this.props.commonActions.startLoading();
      this.setState({
        parentPath: data.value
      });
      yield this.props.raiseAction(loadPagesByPath({
        parentPath: data.value
      }, pageData.getPagesConfig));
      this.props.commonActions.stopLoading();
    });
  }
  onFilterAction(data, action) {
    return __awaiter$1(this, void 0, void 0, function* () {
      switch (action.actionType) {
        case 'resetFilter':
          yield this.setState({
            showFilter: !this.state.showFilter
          });
          this.setState({
            currentFilter: {}
          });
          yield this.setState({
            __currentFilter: {}
          });
          yield this.refreshPages();
          break;
        case 'applyFilter':
          yield this.setState({
            currentFilter: this.state.__currentFilter
          });
          preference$2.write('currentFilter', this.state.__currentFilter);
          yield this.setState({
            showFilter: !this.state.showFilter,
            __currentFilter: undefined
          });
          yield this.refreshPages();
          break;
      }
    });
  }
  handleAction(data, action) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        onAction
      } = this.props;
      switch (action.actionType) {
        case 'edit-filter':
          this.setState({
            showFilter: true
          });
          break;
        default:
          if (action.actionType) {
            action.currentData = data;
            onAction && onAction(data, action);
          }
      }
    });
  }
  render() {
    var _a, _b, _c, _d;
    const {
      store,
      pageData = {},
      userData
    } = this.props;
    const {
      isLoading
    } = this.state;
    const {
      Form,
      Dialog,
      Grid,
      Actions
    } = components$1.get();
    return jsxs("div", {
      className: "sq-page-listing sq-v-screen sq-v-screen--fixed",
      children: [jsx(Fragment, {
        children: jsx(Dialog, {
          open: this.state.showFilter,
          transitionDir: "left",
          classes: {
            dialog: {
              root: 'sq-dialog--fixed-right'
            }
          },
          onClose: () => this.setState({
            showFilter: false
          }),
          onAction: this.onFilterAction,
          actions: [{
            actionType: 'applyFilter',
            buttonText: 'Apply'
          }, {
            actionType: 'resetFilter',
            buttonText: 'Reset',
            variant: 'outlined'
          }],
          title: 'Filters',
          children: jsx(Form, {
            onChange: this.onQuickFilterChange,
            userData: userData,
            className: "mt-wide",
            value: this.state.__currentFilter || this.state.currentFilter,
            fields: this.filterFields
          })
        })
      }), jsx("div", {
        className: `container-fluid ${pageData.actionInBreadCrumb === true ? 'sq-v-screen__sub-header' : 'mt-wide'}`,
        children: jsx(Actions, {
          onAction: this.handleAction,
          actions: [{
            type: 'Button',
            iconName: 'add',
            to: pageData.addNewPage || 'addNewPage',
            buttonText: 'Add New'
          }, {
            actionType: 'edit-filter',
            iconName: 'filter-list',
            variant: 'outlined',
            cmpType: 'Button',
            buttonText: 'Filters'
          }].filter(i => i)
        })
      }), jsxs("div", {
        className: "sq-v-screen__container mt-wide",
        children: [jsx(Dialog, {
          open: this.state.openClone,
          onClose: this.toggleEditForm,
          title: `Clone page`,
          children: jsx("div", {
            className: "mt-wide",
            children: jsx(DynamicContentRoot, {
              className: `sq-page-listing__clone`,
              onSubmit: this.onCloneFormSubmit,
              pageConfig: clonePageConfig(Object.assign({
                formData: this.state.cloneFrom
              }, pageData.cloneConfig))
            })
          })
        }), jsx("div", {
          className: `container-fluid  sq-v-screen__body-container`,
          children: jsxs("div", {
            className: "sq-v-screen-grow mb-wide sq-page-listing__container",
            children: [pageData.enableTree !== false && jsx("div", {
              className: "sq-page-listing__left",
              children: jsx(PathTree, {
                data: store.admin.contentTree,
                onChange: this.onTreeSelect,
                value: this.state.parentPath
              })
            }), jsx(Grid, {
              className: "sq-basic-grid sq-page-listing__right sq-basic-grid--rounded sq-grid--fixed",
              paginationProps: {
                disabled: isLoading,
                count: (_b = (_a = store.admin) === null || _a === void 0 ? void 0 : _a.contentPagesPagination) === null || _b === void 0 ? void 0 : _b.totalPages,
                value: (_c = store.admin) === null || _c === void 0 ? void 0 : _c.contentPagesPagination,
                onChange: data => {
                  var _a, _b;
                  return this.refreshPages({
                    pageNo: (_a = data === null || data === void 0 ? void 0 : data.value) === null || _a === void 0 ? void 0 : _a.currentPage,
                    pageSize: (_b = data === null || data === void 0 ? void 0 : data.value) === null || _b === void 0 ? void 0 : _b.pageSize
                  });
                },
                enablePageSize: true,
                pageSizeOptions: GLOBAL_OPTIONS.noOfResultsDropdown.toArray()
              },
              // loader={<Skeleton styleName={`grid-tran`} rows={8} />}
              onAction: this.onGridAction,
              columns: [{
                name: 'icon',
                cmpType: 'Icon',
                className: 'col-icon',
                sort: false,
                component: {
                  name: row => {
                    return row.type === 'SITE_MAP' ? 'AccountTree' : 'Description';
                  }
                }
              }, {
                name: 'title',
                headerText: 'Page Title',
                className: 'col-large',
                render: (val, col, row) => {
                  var _a, _b, _c;
                  return row.pageName || (row.type === 'SITE_MAP' ? `${(_b = (_a = row.pageData) === null || _a === void 0 ? void 0 : _a.siteMap) === null || _b === void 0 ? void 0 : _b.title}` : `${(_c = row.pageData) === null || _c === void 0 ? void 0 : _c.title}`);
                }
              }, {
                name: 'path',
                headerText: 'Path',
                className: 'col-large-grow'
              }, {
                name: 'actions',
                headerText: 'Actions',
                className: 'col-actions',
                cmpType: 'MoreActions',
                sort: false,
                component: {
                  actions: [{
                    cmpType: 'LinkButton',
                    action: 'edit',
                    iconName: 'edit',
                    buttonText: translate$3('Edit')
                  }, {
                    cmpType: 'LinkButton',
                    action: 'preview',
                    iconName: 'RestartAlt',
                    buttonText: translate$3('Preview'),
                    render: row => row.type === 'PAGE'
                  }, {
                    cmpType: 'LinkButton',
                    iconName: 'deactivate',
                    iconColor: 'error',
                    action: 'unpublish',
                    buttonText: translate$3('UnPublish'),
                    render: row => {
                      return row.status === 'PUBLISHED';
                    }
                  }, {
                    cmpType: 'LinkButton',
                    iconName: 'active',
                    iconColor: 'success',
                    action: 'publish',
                    buttonText: translate$3('Publish'),
                    render: row => {
                      return row.status === 'DRAFT';
                    }
                  }, {
                    cmpType: 'LinkButton',
                    iconName: 'ContentCopy',
                    iconColor: 'info',
                    action: 'clone',
                    buttonText: translate$3('Clone')
                  }, {
                    cmpType: 'LinkButton',
                    iconName: 'Delete',
                    iconColor: 'error',
                    action: 'delete',
                    buttonText: translate$3('Delete'),
                    confirm: {
                      title: 'Confirm?',
                      content: 'Are you sure you want to delete this page?'
                    }
                  }]
                }
              }],
              showHeader: true,
              data: (_d = store.admin) === null || _d === void 0 ? void 0 : _d.contentPages
            })]
          })
        })]
      })]
    });
  }
}
PageListing.propTypes = {
  raiseAction: PropTypes.func,
  store: PropTypes.object,
  commonActions: PropTypes.object,
  appActions: PropTypes.object,
  location: PropTypes.object,
  history: PropTypes.object
};

const {
  translate: translate$2
} = translate$6;
class RoleDetails extends BaseContainer {
  constructor(props) {
    super(props);
    this.state = {
      currentTab: GLOBAL_OPTIONS.permissionTypes.keys.PERMISSIONS,
      currentSelected: {},
      currentSort: 'priority_ASC'
    };
    this.tabs = GLOBAL_OPTIONS.permissionTypes.toArray();
    this.permissionTabs = [GLOBAL_OPTIONS.permissionTypes.keys.PERMISSIONS, GLOBAL_OPTIONS.permissionTypes.keys.LEFT_NAVIGATION];
    this.handleTabChange = this.handleTabChange.bind(this);
    this.handleOnSaveClick = this.handleOnSaveClick.bind(this);
    this.handleOnCheckChange = this.handleOnCheckChange.bind(this);
    this.onUserChange = this.onUserChange.bind(this);
    this.onButtonAdduserClick = this.onButtonAdduserClick.bind(this);
    this.onGridAction = this.onGridAction.bind(this);
  }
  componentDidMount() {
    var _a, _b, _c, _d;
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        pageData
      } = this.props;
      this.props.commonActions.startLoading();
      const actions = [];
      actions.push(this.props.userActions.getPermissionsByRole(query$1.get().code, (_a = pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig) === null || _a === void 0 ? void 0 : _a.getPermissionsByRole));
      this.setState({
        roleCode: query$1.get().code
      });
      actions.push(this.props.userActions.loadUsers({
        body: {
          isActive: true
        },
        source: 'ACTIVE_USERS',
        query: {
          pageSize: 10000
        }
      }, (_b = pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig) === null || _b === void 0 ? void 0 : _b.getUsers));
      actions.push(this.props.userActions.loadPermissions({}, (_c = pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig) === null || _c === void 0 ? void 0 : _c.getPermissions));
      actions.push(this.props.userActions.loadUsersByRole({
        roleCode: query$1.get().code
      }, (_d = pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig) === null || _d === void 0 ? void 0 : _d.roleMapping));
      yield Promise.all(actions);
      this.props.commonActions.stopLoading();
    });
  }
  handleTabChange(data) {
    this.setState({
      currentTab: data.value
    });
  }
  handleOnSaveClick() {
    var _a, _b;
    const {
      pageData
    } = this.props;
    let allPerms = [];
    const currentRole = this.props.store.authentication.currentRole;
    Object.keys(currentRole.allPermissions).forEach(key => {
      if (currentRole.allPermissions[key]) {
        allPerms = allPerms.concat(currentRole.allPermissions[key]);
      }
    });
    this.props.userActions.savePermissionMapping({
      roleCode: this.state.roleCode,
      permissionCode: uniq(allPerms)
    }, Object.assign(Object.assign({}, (_a = pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig) === null || _a === void 0 ? void 0 : _a.permissionMapping), {
      userProfile: (_b = pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig) === null || _b === void 0 ? void 0 : _b.userProfile
    }));
  }
  handleOnCheckChange(key, data) {
    var _a;
    const {
      pageData
    } = this.props;
    this.props.userActions.updateCurrentRolePermissions({
      key,
      data: data.value
    }, (_a = pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig) === null || _a === void 0 ? void 0 : _a.permissionMapping);
  }
  onUserChange(data) {
    this.setState({
      selectedUser: data.value
    });
    this.props.userActions.clearAddUserError();
  }
  onGridAction(row, value, column) {
    var _a, _b;
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        pageData
      } = this.props;
      switch (value.action) {
        case 'remove':
          yield this.props.userActions.removeUserToRole({
            roleCode: this.state.roleCode,
            userId: row.userId
          }, (_a = pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig) === null || _a === void 0 ? void 0 : _a.roleMapping);
          this.props.userActions.loadUsersByRole({
            roleCode: query$1.get().code
          }, (_b = pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig) === null || _b === void 0 ? void 0 : _b.roleMapping);
          break;
      }
    });
  }
  onButtonAdduserClick() {
    var _a;
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        pageData
      } = this.props;
      if (this.state.selectedUser) {
        yield this.props.userActions.addUserToRole({
          roleCode: this.state.roleCode,
          userId: this.state.selectedUser
        }, pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig.roleMapping);
        this.setState({
          selectedUser: ''
        });
        this.props.userActions.loadUsersByRole({
          roleCode: this.state.roleCode
        }, (_a = pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig) === null || _a === void 0 ? void 0 : _a.roleMapping);
      }
    });
  }
  render() {
    const {
      store,
      pageData
    } = this.props;
    const data = store.authentication.permissions ? store.authentication.permissions[this.state.currentTab] : undefined;
    const {
      currentRole
    } = this.props.store.authentication;
    const viewOnly = !!(pageData === null || pageData === void 0 ? void 0 : pageData.viewOnly);
    const {
      Grid,
      Tabs,
      LinkButton,
      CheckboxList,
      Autocomplete,
      Button,
      Skeleton
    } = components$1.get();
    return jsx("div", {
      className: "sq-role-details container-med",
      children: jsxs("div", {
        className: "sq-role-details__container mt-wide",
        children: [jsx("div", {
          className: "sq-role-details__header",
          children: jsx("div", {
            className: "sq-role-details__header-text",
            children: jsx("h2", {
              children: (currentRole === null || currentRole === void 0 ? void 0 : currentRole.roleName) || 'No Name'
            })
          })
        }), jsx(Tabs, {
          options: this.tabs,
          onChange: this.handleTabChange,
          value: this.state.currentTab
        }), !currentRole && jsx(Skeleton, {
          styleName: "permissions"
        }), this.permissionTabs.indexOf(this.state.currentTab) > -1 && jsxs("div", {
          className: "sq-role-details__permissions-wrapper",
          children: [false , jsx("div", {
            className: "sq-role-details__permissions pt-wide",
            children: data && Object.keys(data).map(key => {
              const innerData = data[key];
              const uniQueKey = `${this.state.currentTab}_${key}`;
              return jsxs("div", {
                className: "sq-role-details__group-wrapper",
                children: [jsx("div", {
                  className: "sq-role-details__group-header",
                  children: jsx("h3", {
                    children: key
                  })
                }), jsx(CheckboxList, {
                  disabled: viewOnly,
                  className: "sq-role-details__permission-check",
                  options: innerData,
                  valueField: "code",
                  textField: "name",
                  onChange: (data, config) => this.handleOnCheckChange(uniQueKey, data),
                  value: currentRole === null || currentRole === void 0 ? void 0 : currentRole.allPermissions[uniQueKey]
                })]
              }, uniQueKey);
            })
          })]
        }), this.state.currentTab === GLOBAL_OPTIONS.permissionTypes.keys.ASSIGN_USERS && jsxs("div", {
          className: "sq-role-details__users-wrapper",
          children: [!viewOnly && jsxs("div", {
            className: "sq-role-details__user-add",
            children: [jsx(Autocomplete, Object.assign({
              label: "User"
            }, store.authentication.currentUserError, {
              onChange: this.onUserChange,
              value: this.state.selectedUser,
              textField: "displayName",
              valueField: "emailId",
              options: store.authentication.users,
              className: "sq-role-details__user-list"
            })), jsx(Button, {
              className: "mt-1",
              disabled: !this.state.selectedUser,
              buttonText: 'Add',
              onClick: this.onButtonAdduserClick
            })]
          }), jsx(Grid, {
            className: "sq-basic-grid sq-basic-grid--default sq-basic-grid--rounded pt-wide",
            columns: [{
              name: 'icon',
              cmpType: 'Icon',
              className: 'col-icon',
              component: {
                textIcon: row => `${string.getTwoChars(row.firstName)}`,
                iconClass: 'sq-icon--square sq-icon--text-icon-accent',
                variant: row => accent.getColorByChar(string.getTwoChars(row.firstName))
              }
            }, {
              name: 'fullName',
              headerText: 'Name',
              className: 'col-initial'
            }, {
              name: 'userId',
              headerText: 'Email ID',
              className: 'col-initial'
            }, {
              name: 'actions',
              className: 'col-actions',
              cmpType: 'MoreActions',
              component: {
                actions: [{
                  cmpType: 'LinkButton',
                  iconName: 'deactivate',
                  disabled: viewOnly,
                  confirm: {
                    title: translate$2('Confirm'),
                    content: translate$2('Are you sure you want to remove this user?')
                  },
                  iconColor: 'error',
                  action: 'remove',
                  buttonText: translate$2('Remove')
                }]
              }
            }],
            onAction: this.onGridAction,
            data: store.authentication.currentRoleUsers
          })]
        }), !viewOnly && this.state.currentTab !== GLOBAL_OPTIONS.permissionTypes.keys.ASSIGN_USERS && jsxs("div", {
          className: "sq-role-details__actions mt-wide",
          children: [jsx(Button, {
            size: "large",
            buttonText: 'Save',
            onClick: this.handleOnSaveClick
          }), jsx(LinkButton, {
            buttonText: 'Cancel',
            size: "large",
            type: "Button",
            variant: "outlined",
            to: "roles"
          })]
        })]
      })
    });
  }
}
const mapStateToProps$3 = state => {
  return {};
};
const mapDispatchToProps$3 = dispatch => {
  return {
    userActions: {
      updateCurrentRolePermissions: (data, config) => dispatch(updateCurrentRolePermissions(data, config)),
      removeUserToRole: (data, config) => dispatch(removeUserToRole(data, config)),
      clearAddUserError: (data, config) => dispatch(clearAddUserError(data, config)),
      addUserToRole: (data, config) => dispatch(addUserToRole(data, config)),
      loadUsers: (data, config) => dispatch(loadUsers$1(data, config)),
      loadPermissions: (data, config) => dispatch(loadPermissions(data, config)),
      loadUsersByRole: (data, config) => dispatch(loadUsersByRole(data, config)),
      savePermissionMapping: (data, config) => dispatch(savePermissionMapping(data, config)),
      getPermissionsByRole: (roleCode, config) => dispatch(getPermissionsByRole(roleCode, config))
    }
  };
};
RoleDetails.propTypes = {
  raiseAction: PropTypes.func,
  store: PropTypes.object,
  commonActions: PropTypes.object,
  appActions: PropTypes.object,
  location: PropTypes.object,
  history: PropTypes.object
};
var index$3 = connect(mapStateToProps$3, mapDispatchToProps$3)(RoleDetails);

var _a$1;
const {
  translate: translate$1
} = translate$6;
const {
  Validator
} = validator;
const defaultPageSize$1 = +((_a$1 = GLOBAL_OPTIONS.noOfResultsDropdown.toArray()[0]) === null || _a$1 === void 0 ? void 0 : _a$1.value);
class Roles extends BaseContainer {
  constructor(props) {
    super(props);
    this.state = {
      currentTab: GLOBAL_OPTIONS.roleTabs.keys.ALL_ROLES,
      currentSort: {
        sortColumn: 'id',
        sortOrder: 'desc'
      },
      showEditColumns: false,
      showFilter: false,
      currentFilter: {},
      singleData: false
    };
    this.tabs = GLOBAL_OPTIONS.roleTabs.toArray();
    this.handleOnSort = this.handleOnSort.bind(this);
    this.handleTabChange = this.handleTabChange.bind(this);
    this.onRowClick = this.onRowClick.bind(this);
    this.onGridAction = this.onGridAction.bind(this);
    this.onEditColumnToggle = this.onEditColumnToggle.bind(this);
    this.onEditColumnChange = this.onEditColumnChange.bind(this);
    this.handlePageChange = this.handlePageChange.bind(this);
    this.onFilterToggle = this.onFilterToggle.bind(this);
    this.onFilterAction = this.onFilterAction.bind(this);
    this.onFilterChange = this.onFilterChange.bind(this);
    this.validator = new Validator({
      contactNumber: {
        validators: [{
          type: 'phone'
        }]
      }
    });
  }
  componentDidMount() {
    var _a;
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        pageData
      } = this.props;
      this.loadForms();
      this.props.raiseAction(loadRoles({}, (_a = pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig) === null || _a === void 0 ? void 0 : _a.getRoles));
    });
  }
  handleOnSort(data, column) {
    var _a, _b;
    return __awaiter$1(this, void 0, void 0, function* () {
      this.setState({
        currentSort: data
      });
      const currentPage = (_b = (_a = this.props.store.authentication) === null || _a === void 0 ? void 0 : _a.allRolesPage) === null || _b === void 0 ? void 0 : _b.currentPage;
      yield this.loadForms({
        pageNo: currentPage,
        sort: data
      });
    });
  }
  onFilterChange(data, field) {
    return __awaiter$1(this, void 0, void 0, function* () {
      this.validator.setValue(field.name, data.value[field.name]);
      this.validator.validate(field.name);
      yield this.setState({
        errors: this.validator.errors
      });
      Object.keys(data.value).forEach(key => (data.value[key] == null || data.value[key] == '') && delete data.value[key]);
      this.setState({
        currentFilter: data.value,
        singleData: false
      });
    });
  }
  onFilterToggle() {
    return __awaiter$1(this, void 0, void 0, function* () {
      this.setState({
        showFilter: !this.state.showFilter,
        showEditColumns: false
      });
    });
  }
  onFilterAction(data, action) {
    return __awaiter$1(this, void 0, void 0, function* () {
      switch (action.actionType) {
        case 'resetfilter':
          this.setState({
            showFilter: !this.state.showFilter
          });
          this.setState({
            currentFilter: {},
            singleData: false
          });
          yield this.loadForms({
            filter: {}
          });
          break;
        case 'applyfilter':
          this.setState({
            showFilter: !this.state.showFilter
          });
          this.setState({
            singleData: true
          });
          yield this.loadForms({});
          break;
      }
    });
  }
  loadForms({
    pageNo,
    sort,
    filter,
    pageSize
  } = {}) {
    var _a;
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        pageData
      } = this.props;
      pageSize = pageSize || defaultPageSize$1;
      pageNo = pageNo || 1;
      const sortBy = (sort || this.state.currentSort).sortColumn;
      const sortDir = (sort || this.state.currentSort).sortOrder;
      this.props.commonActions.startLoading();
      yield this.props.raiseAction(showAllRoles({
        body: Object.assign({}, filter || this.state.currentFilter),
        query: {
          sortBy: `${sortBy}|${sortDir}`,
          pageSize: pageSize,
          pageNo: pageNo
        }
      }, (_a = pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig) === null || _a === void 0 ? void 0 : _a.searchRoles));
      this.props.commonActions.stopLoading();
    });
  }
  handlePageChange(data) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const pageNo = data.value.currentPage;
      const pageSize = data.value.pageSize;
      yield this.loadForms({
        pageNo: pageNo,
        pageSize
      });
    });
  }
  onEditColumnToggle() {
    this.setState({
      showEditColumns: !this.state.showEditColumns
    });
  }
  onEditColumnChange(data) {
    this.setState({
      selectedColumns: data.value,
      showEditColumns: !this.state.showEditColumns
    });
  }
  handleTabChange(data) {
    this.setState({
      currentTab: data.value
    });
  }
  onRowClick(row) {
    if (hasPermission('viewRole', this.props.store)) redirectTo('viewRole', {
      code: row.code
    });
  }
  onGridAction(row, value, column) {
    var _a, _b;
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        pageData
      } = this.props;
      switch (value.action) {
        case 'edit':
          redirectTo('editRole', {
            code: row.code,
            userType: row.userType
          });
          break;
        case 'assign':
          redirectTo('assignRole', {
            code: row.code
          });
          break;
        case 'delete':
          yield this.props.raiseAction(removeRole(row, (_a = pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig) === null || _a === void 0 ? void 0 : _a.userRole));
          this.props.raiseAction(loadRoles({}, (_b = pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig) === null || _b === void 0 ? void 0 : _b.getRoles));
      }
    });
  }
  render() {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
    const {
      store
    } = this.props;
    const {
      isLoading
    } = this.state;
    const {
      Grid,
      Tabs,
      Actions,
      Pagination,
      Dialog,
      Form,
      Skeleton
    } = components$1.get();
    const actions = hasPermission('createRole', store) ? [{
      type: 'Button',
      iconName: 'add',
      className: '',
      to: 'createNewRole',
      buttonText: 'Add New'
    }, {
      buttonText: 'Edit Columns',
      iconName: 'edit',
      variant: 'outlined',
      cmpType: 'Button',
      onClick: this.onEditColumnToggle
    }, {
      buttonText: 'Filters',
      iconName: 'filter-list',
      variant: 'outlined',
      cmpType: 'Button',
      onClick: this.onFilterToggle
    }] : [{
      buttonText: 'Edit Columns',
      iconName: 'edit',
      variant: 'outlined',
      cmpType: 'Button',
      onClick: this.onEditColumnToggle
    }, {
      buttonText: 'Filters',
      iconName: 'filter-list',
      variant: 'outlined',
      cmpType: 'Button',
      onClick: this.onFilterToggle
    }];
    return jsx("div", {
      className: "sq-user-roles sq-v-screen sq-v-screen--fixed",
      children: jsxs("div", {
        className: "sq-v-screen__container ",
        children: [jsxs("div", {
          className: "container-fluid",
          children: [jsxs("div", {
            className: "sq-v-screen__top-header mt-wide",
            children: [jsxs("h2", {
              children: ['Manage Roles', !isLoading && ((_b = (_a = store.authentication) === null || _a === void 0 ? void 0 : _a.roles) === null || _b === void 0 ? void 0 : _b.ALL_ROLES) && `(${store.authentication.roles.ALL_ROLES.length})`]
            }), jsx(Actions, {
              actions: actions
            })]
          }), jsx("div", {
            className: "mt-wide",
            children: ((_d = (_c = store.authentication) === null || _c === void 0 ? void 0 : _c.allRoles) === null || _d === void 0 ? void 0 : _d.ALL_ROLES) && jsx(Pagination, {
              className: "j-content-fl-end",
              onChange: this.handlePageChange,
              count: (_f = (_e = store.authentication) === null || _e === void 0 ? void 0 : _e.allRolesPage) === null || _f === void 0 ? void 0 : _f.totalPages,
              value: (_g = store.authentication) === null || _g === void 0 ? void 0 : _g.allRolesPage,
              enablePageSize: true,
              pageSizeOptions: GLOBAL_OPTIONS.noOfResultsDropdown.toArray()
            })
          })]
        }), jsx(Tabs, {
          options: this.tabs,
          onChange: this.handleTabChange,
          value: this.state.currentTab
        }), jsxs("div", {
          className: 'sq-v-screen__body-container',
          children: [jsx(Dialog, {
            open: this.state.showFilter,
            transitionDir: "left",
            classes: {
              dialog: {
                root: 'sq-dialog--fixed-right'
              }
            },
            onClose: this.onFilterToggle,
            onAction: this.onFilterAction,
            actions: [{
              actionType: 'applyfilter',
              buttonText: 'Apply'
            }, {
              actionType: 'resetfilter',
              buttonText: 'Reset',
              variant: 'outlined'
            }],
            title: 'Filters',
            children: jsx(Form, {
              onChange: this.onFilterChange,
              className: "mt-wide",
              value: this.state.currentFilter,
              onAction: this.onFilterAction,
              name: 'ticketFilter',
              fields: [{
                name: 'code',
                label: translate$1('Name'),
                cmpType: 'Autocomplete',
                options: (_h = store === null || store === void 0 ? void 0 : store.authentication.roles) === null || _h === void 0 ? void 0 : _h.ALL_ROLES,
                textField: 'name',
                valueField: 'code'
              }]
            })
          }), jsx("div", {
            className: "sq-v-screen-grow",
            children: this.state.singleData ? jsx(Grid, {
              sortColumn: this.state.currentSort.sortColumn,
              sortOrder: this.state.currentSort.sortOrder,
              onColFilterChange: this.onEditColumnChange,
              selectedColumns: this.state.selectedColumns,
              showColSelection: this.state.showEditColumns,
              enableSort: true,
              onSort: this.handleOnSort,
              onRowClick: this.onRowClick,
              className: "sq-basic-grid sq-grid--fixed sq-basic-grid--rounded sq-basic-grid",
              loader: jsx(Skeleton, {
                styleName: `grid-tran`,
                rows: 5
              }),
              onAction: this.onGridAction,
              columns: [{
                name: 'icon',
                sort: false,
                cmpType: 'Icon',
                className: 'col-icon',
                customize: false,
                component: {
                  textIcon: row => `${string.getTwoChars(row.name)}`,
                  iconClass: 'sq-icon--square sq-icon--text-icon-accent',
                  variant: row => {
                    var _a;
                    return `${accent.getColorByChar((_a = row.name) === null || _a === void 0 ? void 0 : _a.substr(0, 1))}`;
                  },
                  className: 'sq-icon--square'
                }
              }, {
                name: 'category',
                cmpType: 'TextFields',
                headerText: 'Role',
                className: 'col-large-grow',
                component: {
                  fields: [{
                    name: 'name',
                    className: 'header'
                  }]
                }
              }, {
                name: 'description',
                className: 'col-large',
                headerText: 'Description'
              }, {
                name: 'userType',
                className: 'col-medium',
                headerText: 'Type',
                tooltip: {
                  title: row => row.userType,
                  arrow: false
                }
              }, {
                name: 'actions',
                className: 'col-actions',
                cmpType: 'MoreActions',
                sort: false,
                component: {
                  actions: [{
                    cmpType: 'LinkButton',
                    action: 'edit',
                    disabled: !hasPermission('editRole', store),
                    iconName: 'default',
                    buttonText: translate$1('Edit')
                  }, {
                    cmpType: 'LinkButton',
                    disabled: !hasPermission('editRole', store),
                    action: 'assign',
                    iconName: 'default',
                    buttonText: translate$1('Assign Permissions')
                  }, {
                    cmpType: 'LinkButton',
                    action: 'delete',
                    disabled: !hasPermission('editRole', store),
                    iconName: 'default',
                    buttonText: translate$1('Delete'),
                    confirm: {
                      title: 'Confirm?',
                      content: 'Are you sure you want to delete this page?'
                    }
                  }]
                }
              }],
              showHeader: true,
              data: ((_k = (_j = store.authentication) === null || _j === void 0 ? void 0 : _j.allRoles) === null || _k === void 0 ? void 0 : _k.ALL_ROLES.data) || []
            }) : jsx(Grid, {
              sortColumn: this.state.currentSort.sortColumn,
              sortOrder: this.state.currentSort.sortOrder,
              onColFilterChange: this.onEditColumnChange,
              selectedColumns: this.state.selectedColumns,
              showColSelection: this.state.showEditColumns,
              enableSort: true,
              onSort: this.handleOnSort,
              onRowClick: this.onRowClick,
              className: "sq-basic-grid sq-basic-grid--default sq-basic-grid--rounded sq-grid--fixed pt-wide",
              loader: jsx(Skeleton, {
                styleName: `grid-tran`,
                rows: 5
              }),
              onAction: this.onGridAction,
              columns: [{
                name: 'icon',
                sort: false,
                cmpType: 'Icon',
                className: 'col-icon',
                customize: false,
                component: {
                  textIcon: row => `${string.getTwoChars(row.name)}`,
                  variant: row => {
                    var _a;
                    return `${accent.getColorByChar((_a = row.name) === null || _a === void 0 ? void 0 : _a.substr(0, 1))}`;
                  },
                  className: 'sq-icon--square'
                }
              }, {
                name: 'category',
                cmpType: 'TextFields',
                headerText: 'Role',
                className: 'col-large-grow',
                component: {
                  fields: [{
                    name: 'name',
                    className: 'header'
                  }]
                }
              }, {
                name: 'description',
                className: 'col-large',
                headerText: 'Description'
              }, {
                name: 'userType',
                className: 'col-medium',
                headerText: 'Type',
                tooltip: {
                  title: row => row.userType,
                  arrow: false
                }
              }, {
                name: 'actions',
                className: 'col-actions',
                cmpType: 'MoreActions',
                sort: false,
                component: {
                  actions: [{
                    cmpType: 'LinkButton',
                    action: 'edit',
                    disabled: !hasPermission('editRole', store),
                    iconName: 'default',
                    buttonText: translate$1('Edit')
                  }, {
                    cmpType: 'LinkButton',
                    disabled: !hasPermission('editRole', store),
                    action: 'assign',
                    iconName: 'default',
                    buttonText: translate$1('Assign Permissions')
                  }, {
                    cmpType: 'LinkButton',
                    action: 'delete',
                    disabled: !hasPermission('editRole', store),
                    iconName: 'default',
                    buttonText: translate$1('Delete'),
                    confirm: {
                      title: 'Confirm?',
                      content: 'Are you sure you want to delete this page?'
                    }
                  }]
                }
              }],
              showHeader: true,
              data: ((_m = (_l = store.authentication) === null || _l === void 0 ? void 0 : _l.roles) === null || _m === void 0 ? void 0 : _m.ALL_ROLES) || []
            })
          })]
        })]
      })
    });
  }
}
Roles.propTypes = {
  raiseAction: PropTypes.func,
  store: PropTypes.object,
  commonActions: PropTypes.object,
  appActions: PropTypes.object,
  location: PropTypes.object,
  history: PropTypes.object
};

const DefaulSearch = ({
  iconName,
  header,
  description,
  date,
  link,
  dateFormat
}) => {
  const {
    Icon,
    LinkButton
  } = components$1.get();
  const {
    to,
    urlParams
  } = link || {};
  const options = {};
  if (dateFormat) {
    options.format = dateFormat;
  }
  const {
    dateFullTime
  } = getFormatters$1();
  const handleClick = () => {
    if (to) {
      redirectTo(to, urlParams);
    }
  };
  return /*#__PURE__*/jsxs("div", {
    className: `sq-search-result-template default ${to ? 'has-link' : ''}`,
    onClick: handleClick,
    children: [/*#__PURE__*/jsxs("div", {
      className: "top-header",
      children: [iconName && /*#__PURE__*/jsx(Icon, {
        size: "large",
        name: iconName
      }), /*#__PURE__*/jsx("h2", {
        className: "title",
        children: header
      })]
    }), /*#__PURE__*/jsx("div", {
      className: "description",
      children: description
    }), date && /*#__PURE__*/jsx("div", {
      className: "date-field",
      children: dateFullTime(date, options)
    }), to && /*#__PURE__*/jsx(LinkButton, {
      to: to,
      urlParams: urlParams,
      target: "_blank",
      className: "open-link",
      iconName: "ArrowOutward"
    })]
  });
};

const {
  preference
} = storage;
const templates$1 = {
  default: DefaulSearch
};
class SearchWithListing extends Component$1 {
  constructor(props) {
    super(props);
    this.onSearchChange = data => {
      this.setState({
        searchTerm: data.value
      });
    };
    this.onSearchClick = () => __awaiter$1(this, void 0, void 0, function* () {
      yield this.props.raiseAction(updateUserData({
        [`${this.getKey('searchTerm')}`]: this.state.searchTerm
      }));
      yield this.refreshData({
        pageNo: 1
      });
    });
    this.state = {
      defaultPageSize: 10,
      isLoading: false,
      showEditColumns: false
    };
    this.handleAction = this.handleAction.bind(this);
    this.onFilterChange = this.onFilterChange.bind(this);
    this.onQuickFilterChange = this.onQuickFilterChange.bind(this);
    this.handlePageChange = this.handlePageChange.bind(this);
    this.handleOnSortChange = this.handleOnSortChange.bind(this);
    this.onGridAction = this.onGridAction.bind(this);
    this.onEditColumnChange = this.onEditColumnChange.bind(this);
    this.onFilterAction = this.onFilterAction.bind(this);
    this.onGridChange = this.onGridChange.bind(this);
    this.onTopFilterChange = this.onTopFilterChange.bind(this);
    this.onPopupCloseAction = this.onPopupCloseAction.bind(this);
    this.filterActions = [{
      actionType: 'edit-filter',
      iconName: 'filter-list',
      variant: 'outlined',
      cmpType: 'Button',
      buttonText: 'Filters'
    }];
    events.subscribe('onPopupScreenCloseAction', this.onPopupCloseAction);
  }
  onPopupCloseAction(action) {
    if (action.refreshAfter) {
      this.refreshData({});
    }
  }
  getKey(name) {
    const {
      pageData
    } = this.props;
    return `${pageData.dataKey || window.location.pathname}_${name}`;
  }
  componentDidUpdate(preProps) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        pageData: prevPageData
      } = preProps;
      const {
        pageData
      } = this.props;
      if (prevPageData.dataKey !== pageData.dataKey) {
        this.componentDidMount();
      }
    });
  }
  processField(field) {
    const {
      userData,
      store
    } = this.props;
    return Object.assign(Object.assign({}, field), {
      beforeRender: (field, val, row) => {
        return Object.assign({}, field.componentProps ? processParams(Object.assign(Object.assign({}, userData), row), field.componentProps, undefined, store) : {});
      },
      fields: field.fields ? field.fields.map(field => {
        return this.processField(field);
      }) : undefined
    });
  }
  componentDidMount() {
    var _a, _b, _c, _d, _e;
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        pageData,
        userData,
        store
      } = this.props;
      const overrideParams = {
        filterParams: {},
        quickFilterParams: {},
        topFilterParams: {}
      };
      const queryParams = Object.assign(Object.assign({}, processParams(userData, pageData.initialValues || {})), query$1.get());
      this.filterFields = (_a = pageData.filterFields) === null || _a === void 0 ? void 0 : _a.map(field => {
        if (queryParams[field.name]) {
          overrideParams.filterParams[field.name] = queryParams[field.name];
        }
        return this.processField(field);
      });
      this.topFilterFields = (_b = pageData.topFilterFields) === null || _b === void 0 ? void 0 : _b.map(field => {
        if (queryParams[field.name]) {
          overrideParams.topFilterParams[field.name] = queryParams[field.name];
        }
        return this.processField(field);
      });
      if (pageData.searchField) {
        this.searchField = this.processField(pageData.searchField);
      }
      this.quickFilterFields = (_c = pageData.quickFilterFields) === null || _c === void 0 ? void 0 : _c.map(field => {
        if (queryParams[field.name]) {
          overrideParams.quickFilterParams[field.name] = queryParams[field.name];
        }
        return this.processField(field);
      });
      this.topActions = (_d = pageData.topActions) === null || _d === void 0 ? void 0 : _d.map(action => {
        return Object.assign(Object.assign({}, action), {
          beforeRender: (col, val, row) => {
            if (action === null || action === void 0 ? void 0 : action.renderMatch) {
              const valid = new Validator$1(action === null || action === void 0 ? void 0 : action.renderMatch);
              valid.setValues(userData);
              return valid.validateAll();
            }
            return true;
          }
        });
      });
      this.pageFields = this.columns = (_e = pageData.columns) === null || _e === void 0 ? void 0 : _e.map(item => {
        return Object.assign(Object.assign({}, item), {
          beforeRender: (col, val, row) => {
            return {
              component: Object.assign(Object.assign({}, item.componentProps ? processParams(Object.assign(Object.assign({}, userData), row), item.componentProps, undefined, store) : {}), {
                fields: item.fields ? item.fields.map(field => {
                  return Object.assign(Object.assign({}, field), {
                    beforeRender: (col, val, row) => {
                      return field.componentProps ? {
                        component: processParams(Object.assign(Object.assign({}, userData), row), field.componentProps, undefined, store)
                      } : {};
                    }
                  });
                }) : undefined,
                actions: item.actions ? item.actions.map(action => {
                  return Object.assign(Object.assign(Object.assign({}, action), processParams(Object.assign(Object.assign({}, userData), row), action, undefined, store)), {
                    render: row => {
                      if (action.renderMatch) {
                        const vald = new Validator$1(action.renderMatch);
                        vald.setValues(row);
                        if (!vald.validateAll()) {
                          return false;
                        }
                      }
                      return true;
                    }
                  });
                }) : undefined
              })
            };
          }
        });
      });
      const objToSave = {
        lastQuery: query$1.get(),
        currentSort: Object.assign(Object.assign({}, pageData.currentSort || {}), preference.read('currentSort')),
        currentFilter: Object.assign(Object.assign({}, queryParams.savedFilter === undefined ? preference.read('currentFilter') : {}), overrideParams.filterParams),
        currentQuickFilter: Object.assign(Object.assign({}, queryParams.savedFilter === undefined ? preference.read('quickFilter') : {}), overrideParams.topFilterParams),
        topFilter: Object.assign(Object.assign({}, queryParams.savedFilter === undefined ? preference.read('topFilter') : {}), overrideParams.topFilterParams),
        selectedColumns: preference.read('selectedColumns', true) || pageData.defaultColumns || undefined,
        columnsOrder: preference.read('columnsOrder', true)
      };
      const pagination = preference.read('pagination') || {};
      this.setState({
        defaultPageSize: pagination.pageSize || GLOBAL_OPTIONS.noOfResultsDropdown.toArray()[0].value
      });
      yield this.props.raiseAction(updateUserData({
        lastQuery: objToSave.lastQuery,
        [`${this.getKey('currentSort')}`]: objToSave.currentSort,
        [`${this.getKey('currentFilter')}`]: objToSave.currentFilter,
        [`${this.getKey('currentQuickFilter')}`]: objToSave.currentQuickFilter,
        [`${this.getKey('topFilter')}`]: objToSave.topFilter,
        [`${this.getKey('selectedColumns')}`]: objToSave.selectedColumns,
        [`${this.getKey('columnsOrder')}`]: objToSave.columnsOrder
      }));
      this.refreshData();
    });
  }
  onFilterChange(data, field) {
    return __awaiter$1(this, void 0, void 0, function* () {
      this.setState({
        __currentFilter: data.value
      });
      yield this.props.raiseAction(updateUserData({
        [`${this.getKey('tempCurrentFilter')}`]: data.value
      }));
    });
  }
  onQuickFilterChange(data, action) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        onAction
      } = this.props;
      this.props.raiseAction(startLoading());
      if (action.actionType) {
        action.currentData = data.value;
        onAction && onAction(data.value, action);
      }
      yield this.props.raiseAction(updateUserData({
        [`${this.getKey('currentQuickFilter')}`]: data.value
      }));
      yield this.refreshData({
        pageNo: 1
      });
      preference.write('quickFilter', data.value);
      this.props.raiseAction(stopLoading());
    });
  }
  onTopFilterChange(data, action) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        onAction
      } = this.props;
      this.props.raiseAction(startLoading());
      yield this.props.raiseAction(updateUserData({
        [`${this.getKey('topFilter')}`]: data.value
      }));
      if (action.actionType) {
        action.currentData = data.value;
        (yield onAction) && onAction(data.value, action);
      }
      yield this.refreshData({
        pageNo: 1
      });
      preference.write('topFilter', data.value);
      this.props.raiseAction(stopLoading());
    });
  }
  getBookmarkableFields(keys, data) {
    if (!keys) {
      return data;
    }
    const obj = {};
    Object.keys(data).forEach(itemKey => {
      if ((keys === null || keys === void 0 ? void 0 : keys.indexOf(itemKey)) > -1) {
        obj[itemKey] = data[itemKey];
      }
    });
    return obj;
  }
  getUserData() {
    const {
      userData
    } = this.props;
    return Object.assign(Object.assign({}, userData), {
      searchTerm: userData[this.getKey('searchTerm')] || undefined
    });
  }
  refreshData({
    filter,
    sort,
    pageSize,
    pageNo
  } = {}) {
    var _a, _b, _c, _d, _e;
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        pageData,
        data,
        store,
        raiseAction,
        onAction
      } = this.props;
      const finalUserData = this.getUserData();
      if ((_a = pageData.apiConfig) === null || _a === void 0 ? void 0 : _a.search) {
        this.setState({
          isLoading: true
        });
        const pagination = finalUserData[this.getKey('pagination')] || {};
        const currentSort = finalUserData[this.getKey('currentSort')] || {};
        pageSize = pageSize || pagination.pageSize || pageData.defaultPageSize || this.state.defaultPageSize;
        pageNo = pageNo || (pagination === null || pagination === void 0 ? void 0 : pagination.currentPage) || 1;
        const sortBy = (sort || currentSort).sortColumn;
        const sortDir = (sort || currentSort).sortOrder;
        if (pageData.bookmarkable === true) {
          const fields = this.getBookmarkableFields(pageData.bookmarkableFields, Object.assign(Object.assign(Object.assign({}, filter || finalUserData[this.getKey('currentFilter')] || {}), finalUserData[this.getKey('currentQuickFilter')] || {}), finalUserData[this.getKey('topFilter')] || {}));
          onAction && onAction({}, {
            actionType: 'self-redirect',
            urlParams: fields
          });
        }
        yield raiseAction(postApi(Object.assign(Object.assign({}, (_b = pageData.apiConfig) === null || _b === void 0 ? void 0 : _b.search), {
          params: object$1.extendData(processParams(finalUserData, (_c = pageData.apiConfig) === null || _c === void 0 ? void 0 : _c.search.params, undefined, store), Object.assign(Object.assign(Object.assign({}, filter || finalUserData[this.getKey('currentFilter')] || {}), finalUserData[this.getKey('currentQuickFilter')] || {}), finalUserData[this.getKey('topFilter')] || {})),
          query: ((_d = pageData.apiConfig) === null || _d === void 0 ? void 0 : _d.search.query) ? processParams(Object.assign(Object.assign({}, finalUserData), {
            sortBy: `${sortBy}|${sortDir}`,
            pageSize: pageSize,
            pageNo: pageNo
          }), (_e = pageData.apiConfig) === null || _e === void 0 ? void 0 : _e.search.query) : {
            sortBy: sortBy && sortDir ? `${sortBy}|${sortDir}` : undefined,
            pageSize: pageSize,
            pageNo: pageNo
          },
          dataKey: this.getKey('results')
        }), data));
        this.setState({
          isLoading: false
        });
      }
    });
  }
  handlePageChange(data) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const pageNo = data.value.currentPage,
        pageSize = data.value.pageSize;
      this.props.raiseAction(startLoading());
      yield this.refreshData({
        pageNo,
        pageSize
      });
      this.props.raiseAction(stopLoading());
      preference.write('pagination', data.value);
    });
  }
  handleAction(data, action) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        onAction
      } = this.props;
      switch (action.actionType) {
        case 'edit-cols':
          this.setState({
            showEditColumns: true
          });
          break;
        case 'edit-filter':
          this.setState({
            showFilter: true
          });
          break;
        default:
          if (action.actionType) {
            action.currentData = data;
            onAction && onAction(data, action);
          }
      }
    });
  }
  onGridAction(row, action, _) {
    return __awaiter$1(this, void 0, void 0, function* () {
      switch (action.actionType) {
        default:
          this.props.raiseAction(startLoading());
          action.currentData = row;
          this.props.onAction && (yield this.props.onAction(row, action));
          if (action.refreshAfter) {
            yield this.refreshData({});
          }
          this.props.raiseAction(stopLoading());
      }
    });
  }
  onGridChange(data, action, row) {
    return __awaiter$1(this, void 0, void 0, function* () {
      if (action.actionType) {
        this.props.raiseAction(startLoading());
        action.currentData = Object.assign(Object.assign({}, row), data);
        this.props.onAction && (yield this.props.onAction(Object.assign(Object.assign({}, row), data), action));
        if (action.refreshAfter) {
          yield this.refreshData({});
        }
        this.props.raiseAction(stopLoading());
      }
    });
  }
  onFilterAction(data, action) {
    return __awaiter$1(this, void 0, void 0, function* () {
      switch (action.actionType) {
        case 'resetFilter':
          yield this.setState({
            showFilter: !this.state.showFilter
          });
          this.props.raiseAction(startLoading());
          yield this.props.raiseAction(updateUserData({
            [`${this.getKey('currentFilter')}`]: {},
            [`${this.getKey('tempCurrentFilter')}`]: {}
          }));
          preference.write('currentFilter', {});
          yield this.setState({
            __currentFilter: undefined
          });
          yield this.refreshData({
            filter: {},
            pageNo: 1
          });
          this.props.raiseAction(stopLoading());
          break;
        case 'applyFilter':
          yield this.props.raiseAction(updateUserData({
            [`${this.getKey('currentFilter')}`]: this.state.__currentFilter,
            [`${this.getKey('tempCurrentFilter')}`]: this.state.__currentFilter
          }));
          preference.write('currentFilter', this.state.__currentFilter);
          yield this.setState({
            showFilter: !this.state.showFilter,
            __currentFilter: undefined
          });
          this.props.raiseAction(startLoading());
          yield this.refreshData({});
          this.props.raiseAction(stopLoading());
          break;
      }
    });
  }
  handleOnSortChange(data) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        currentPage
      } = this.props.userData[this.getKey('results')] || {};
      preference.write('currentSort', data);
      yield this.props.raiseAction(updateUserData({
        [`${this.getKey('currentSort')}`]: data
      }));
      this.props.raiseAction(startLoading());
      yield this.refreshData({
        pageNo: currentPage,
        sort: data
      });
      this.props.raiseAction(stopLoading());
    });
  }
  onEditColumnChange(data) {
    const {
      userData
    } = this.props;
    if (!data.cancel) {
      this.props.raiseAction(updateUserData({
        [`${this.getKey('selectedColumns')}`]: data.value,
        [`${this.getKey('columnsOrder')}`]: data.columnsOrder ? data.columnsOrder : userData[this.getKey('columnsOrder')]
      }));
      const objToSave = {
        selectedColumns: data.value
      };
      if (data.columnsOrder) {
        objToSave.columnsOrder = data.columnsOrder;
      }
      preference.writeAll(objToSave);
    }
    this.setState({
      showEditColumns: !this.state.showEditColumns
    });
  }
  render() {
    var _a, _b, _c, _d, _e;
    const {
      pageData = {},
      store,
      userData
    } = this.props;
    const {
      className = '',
      actionInBreadCrumb = true
    } = pageData;
    const {
      Actions,
      Dialog,
      Form,
      Input,
      IconButton,
      HeroContent,
      Repeater,
      Card,
      Pagination,
      Skeleton
    } = components$1.get();
    userData[this.getKey('currentSort')];
    const currentFilter = userData[this.getKey('currentFilter')];
    const currentQuickFilter = userData[this.getKey('currentQuickFilter')];
    const topFilter = userData[this.getKey('topFilter')];
    userData[this.getKey('selectedColumns')];
    userData[this.getKey('columnsOrder')];
    const SearchTemplate = templates$1[pageData.searchTemplate] || templates$1.default;
    return jsxs("div", {
      className: `sq-search-with-listing sq-v-screen ${className}`,
      children: [jsxs("div", {
        className: "sq-v-screen__container",
        children: [jsxs("div", {
          className: `mt-wide`,
          children: [pageData.showPageTitle && jsx("div", {
            className: "sq-search-with-listing__header",
            children: pageData.title
          }), this.topFilterFields && jsx("div", {
            className: 'sq-search-with-listing__quick',
            children: jsx(Form, {
              disabled: this.state.isLoading,
              userData: userData,
              onChange: this.onTopFilterChange,
              className: "sq-form--inline-auto p-0",
              value: topFilter,
              fields: this.topFilterFields
            })
          }), jsx(Actions, {
            className: "w-auto",
            userData: userData,
            disabled: this.state.isLoading,
            onAction: this.handleAction,
            actions: [...(this.topActions || []), ...(this.filterFields ? this.filterActions : [])]
          })]
        }), jsxs("div", {
          className: `sq-v-screen__container`,
          children: [pageData.headerContent && jsx(HeroContent, Object.assign({}, pageData.headerContent)), pageData.items && jsx(Content, Object.assign({}, this.props, {
            pageData: pageData,
            metaData: {},
            userData: userData
          })), jsxs("div", {
            className: "sq-search-with-listing__search-bar",
            children: [this.searchField && jsx(Input, Object.assign({}, this.searchField, {
              value: this.state.searchTerm,
              onChange: this.onSearchChange
            })), jsx(IconButton, {
              iconName: "search",
              onClick: this.onSearchClick
            })]
          }), jsx("div", {
            className: "container-fluid",
            children: jsx("div", {
              className: "sq-v-screen__pagination-bar d-flex fl-a-items-center justify-content-center mb-2",
              children: this.quickFilterFields && jsx("div", {
                className: 'sq-search-with-listing__quick mt-2',
                children: jsx(Form, {
                  disabled: this.state.isLoading,
                  userData: userData,
                  onChange: this.onQuickFilterChange,
                  className: "sq-form--inline-auto p-0",
                  value: currentQuickFilter,
                  fields: this.quickFilterFields
                })
              })
            })
          }), jsx("div", {
            className: "container-fluid d-flex fl-a-items-center justify-content-end sq-search-with-listing__pagination mb-wide",
            children: ((_b = (_a = userData[this.getKey('results')]) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.length) > 0 && jsx(Pagination, {
              onChange: this.handlePageChange,
              count: (_c = userData[this.getKey('results')]) === null || _c === void 0 ? void 0 : _c.totalPages,
              value: userData[this.getKey('results')],
              enablePageSize: true,
              defaultPageSize: pageData.defaultPageSize || this.state.defaultPageSize,
              pageSizeOptions: GLOBAL_OPTIONS.noOfResultsDropdown.toArray()
            })
          }), jsx("div", {
            className: "sq-search-with-listing__grid-wrapper sq-v-screen-grow",
            children: jsx(Repeater, {
              className: `sq-search-with-listing__results ${((_d = pageData.classes) === null || _d === void 0 ? void 0 : _d.searchResults) || ''}`,
              loader: jsx(Skeleton, {
                rows: 5,
                styleName: "tiles"
              }),
              data: (_e = userData[this.getKey('results')]) === null || _e === void 0 ? void 0 : _e.data,
              template: ({
                data
              }) => {
                var _a, _b, _c;
                const _data = {
                  header: object$1.getDataFromKey(data, (_a = pageData.dataKeys) === null || _a === void 0 ? void 0 : _a.header),
                  description: object$1.getDataFromKey(data, (_b = pageData.dataKeys) === null || _b === void 0 ? void 0 : _b.description),
                  date: object$1.getDataFromKey(data, (_c = pageData.dataKeys) === null || _c === void 0 ? void 0 : _c.date),
                  link: processParams(Object.assign(Object.assign({}, userData), data), pageData.dataKeys.link, undefined, store),
                  dateFormat: pageData.dataKeys.dateFormat,
                  iconName: pageData.dataKeys.iconName
                };
                return jsx(SearchTemplate, Object.assign({}, _data));
              }
            })
          })]
        })]
      }), jsx(Fragment, {
        children: jsx(Dialog, {
          open: this.state.showFilter,
          transitionDir: "left",
          classes: {
            dialog: {
              root: 'sq-dialog--fixed-right'
            }
          },
          onClose: () => this.setState({
            showFilter: false
          }),
          onAction: this.onFilterAction,
          actions: [{
            actionType: 'applyFilter',
            buttonText: 'Apply'
          }, {
            actionType: 'resetFilter',
            buttonText: 'Reset',
            variant: 'outlined'
          }],
          title: 'Filters',
          children: jsx(Form, {
            onChange: this.onFilterChange,
            userData: userData,
            className: "mt-wide",
            value: this.state.__currentFilter || currentFilter,
            fields: this.filterFields
          })
        })
      })]
    });
  }
}
SearchWithListing.propTypes = {};

const ItemTypes = {
  FORM_ELEMENT: 'formElement',
  COMPONENT: 'component',
  FORM: 'form',
  CUSTOM: 'custom',
  WRAPPER: 'wrapper',
  PLACEHOLDER: 'placeholder'
};

const Component = ({
  type,
  name,
  displayText,
  metaData,
  onSuccess
}) => {
  const [{
    opacity
  }, drag] = useDrag(() => ({
    type: type || ItemTypes.COMPONENT,
    item: {
      name,
      metaData
    },
    end(item, monitor) {
      const dropResult = monitor.getDropResult();
      if (item && dropResult) {
        const isDropAllowed = dropResult.allowedDropEffect === 'any' || dropResult.allowedDropEffect === dropResult.dropEffect;
        if (isDropAllowed) {
          onSuccess && onSuccess(item, dropResult);
        }
      }
    },
    collect: monitor => ({
      opacity: monitor.isDragging() ? 'dragged' : 'default'
    })
  }), [name]);
  return jsx("div", {
    className: `sq-component-list__item sq-component-list__item--${opacity}`,
    ref: drag,
    children: displayText
  });
};
const ComponentList = ({
  onDrop,
  compList: _compList = {},
  filter
}) => {
  const grouped = groupBy(Object.keys(_compList).map(key => {
    return Object.assign({
      name: key,
      displayText: key
    }, _compList[key]);
  }).sort((a, b) => a.name > b.name ? 1 : a.name < b.name ? -1 : 0).filter(item => {
    if (filter) {
      return filter.indexOf(item.group) > -1;
    }
    return true;
  }), 'group');
  return jsx("div", {
    className: "sq-component-list",
    children: Object.keys(grouped).sort((a, b) => a > b ? 1 : a < b ? -1 : 0).map((group, idx) => {
      return jsxs("div", {
        className: "sq-component-list__group-wrapper",
        children: [jsx("div", {
          className: "sq-component-list__group",
          children: group
        }), grouped[group].map((comp, idx) => {
          return jsx(Component, {
            type: comp.type,
            name: comp.name,
            displayText: comp.displayText,
            metaData: comp,
            onSuccess: item => {
              onDrop && onDrop(item);
            }
          }, idx);
        })]
      }, group);
    })
  });
};
ComponentList.propTypes = {
  onDrop: PropTypes.func,
  compList: PropTypes.array,
  filter: PropTypes.array
};
Component.propTypes = {
  type: PropTypes.string,
  displayText: PropTypes.string,
  name: PropTypes.string,
  metaData: PropTypes.object,
  onSuccess: PropTypes.func
};

const Panel = ({
  className,
  theme: _theme = 'default',
  header,
  children,
  enableFullScreen: _enableFullScreen = false,
  onClose
}) => {
  const [fullscreen, setFullScreen] = React$2.useState(false);
  const {
    IconButton
  } = components$1.get();
  const toggleFullscreen = () => {
    setFullScreen(!fullscreen);
  };
  return jsx("div", {
    className: `sq-c-panel sq-c-panel--${_theme} ${fullscreen ? 'sq-c-panel--full-screen' : ''} ${className} `,
    children: jsxs("div", {
      className: "sq-c-panel__container",
      children: [header && jsxs("div", {
        className: "sq-c-panel__header",
        children: [_enableFullScreen && jsx(IconButton, {
          className: "sq-c-panel__full-screen",
          iconSize: "small",
          color: "black",
          iconName: fullscreen ? 'FullscreenExit' : 'Fullscreen',
          onClick: toggleFullscreen
        }), jsx("h5", {
          className: "mb-none",
          children: header
        }), jsx(IconButton, {
          className: "sq-c-panel__close",
          iconSize: "normal",
          color: "black",
          iconName: 'close',
          onClick: onClose
        })]
      }), jsx("div", {
        className: "sq-c-panel__body",
        children: children
      })]
    })
  });
};
Panel.propTypes = {
  className: PropTypes.string,
  theme: PropTypes.string,
  header: PropTypes.string,
  children: PropTypes.any,
  enableFullScreen: PropTypes.bool,
  onClose: PropTypes.func
};

const allowedValidators = ['required', 'equals', 'email', 'or', 'phone', 'internationalphone', 'fieldName', 'path', 'requiredArray', 'exists', 'notExists', 'digits', 'strongPassword', 'length', 'compareField', 'number', 'emailphone', 'emailinternationalphone', 'date', 'password', 'postalCN', 'regex'];
const validatorExport = inner => ({
  name: 'validators',
  cmpType: 'FormList',
  label: 'Validators',
  formClassName: 'sq-form--2-cols',
  fields: [{
    cmpType: 'Select',
    label: 'Type',
    name: 'type',
    options: () => sortBy(filter$1(Object.keys(getValidators()), item => allowedValidators.indexOf(item) > -1).map(item => ({
      text: item,
      value: item
    })), 'text')
  }, Object.assign({}, !inner ? validatorExport(true) : {}, {
    name: 'validations',
    match: {
      type: {
        validators: [{
          type: 'equals',
          matchValue: 'or'
        }]
      }
    }
  }), {
    cmpType: 'Input',
    label: 'Match Value',
    name: 'matchValue',
    match: {
      type: {
        validators: [{
          type: 'equals',
          matchValue: 'equals'
        }]
      }
    }
  }, {
    cmpType: 'Input',
    label: 'Default Value',
    name: 'defaultValue',
    match: {
      type: {
        validators: [{
          type: 'equals',
          matchValue: 'required'
        }]
      }
    }
  }, {
    cmpType: 'Input',
    label: 'fieldName',
    name: 'fieldName',
    match: {
      type: {
        validators: [{
          type: 'oneOf',
          options: ['equals', 'compareField']
        }]
      }
    }
  }, {
    cmpType: 'Radio',
    display: 'Inline',
    label: 'subType',
    name: 'subType',
    options: GLOBAL_OPTIONS.compareOperators.toArray(),
    match: {
      type: {
        validators: [{
          type: 'oneOf',
          options: ['equals']
        }]
      }
    }
  }, {
    cmpType: 'Radio',
    display: 'Inline',
    label: 'compare',
    name: 'compare',
    options: GLOBAL_OPTIONS.compareOperators.toArray(),
    match: {
      type: {
        validators: [{
          type: 'oneOf',
          options: ['compareField']
        }]
      }
    }
  }, {
    cmpType: 'Input',
    label: 'exact',
    name: 'exact',
    match: {
      type: {
        validators: [{
          type: 'equals',
          matchValue: 'length'
        }]
      }
    }
  }, {
    cmpType: 'Input',
    label: 'min',
    name: 'min',
    match: {
      type: {
        validators: [{
          type: 'oneOf',
          options: ['date', 'length']
        }]
      }
    }
  }, {
    cmpType: 'Input',
    label: 'max',
    name: 'max',
    match: {
      type: {
        validators: [{
          type: 'oneOf',
          options: ['date', 'length']
        }]
      }
    }
  }, {
    cmpType: 'Input',
    label: 'digits',
    name: 'digits',
    match: {
      type: {
        validators: [{
          type: 'equals',
          matchValue: 'digits'
        }]
      }
    }
  }, {
    cmpType: 'Input',
    label: 'regex',
    name: 'regex',
    match: {
      type: {
        validators: [{
          type: 'equals',
          matchValue: 'regex'
        }]
      }
    }
  }, {
    cmpType: 'Input',
    label: 'Error Message',
    name: 'message'
  }, {
    cmpType: 'Input',
    label: 'Error Key',
    name: 'key'
  }]
});

const tabs$1 = {
  general: {
    text: 'General',
    value: 'general'
  },
  validations: {
    text: 'Validations',
    value: 'validations'
  },
  actions: {
    text: 'Actions',
    value: 'actions'
  },
  match: {
    text: 'Match',
    value: 'match'
  }
};
const defaultGeneral = ({
  classNames: _classNames = []
} = {}) => [{
  name: 'name',
  cmpType: 'Input',
  label: 'Field Name',
  validators: [{
    type: 'fieldName'
  }]
}, {
  name: 'inject',
  cmpType: 'FormObject',
  label: 'Inject'
}, {
  name: 'className',
  cmpType: 'InputWithOptions',
  optionsLabel: 'Pre-defined Classes',
  optionCmpType: 'Autocomplete',
  label: 'className',
  options: [...GLOBAL_OPTIONS.genericStyles.toArray(), ..._classNames]
}];
const defaultValidations = [validatorExport()];
const defaultParams = {
  '...main': '.main'
};
const defaultMatch = [{
  name: 'match',
  cmpType: 'FormObject',
  label: 'match',
  formClassName: 'mb-wide',
  fields: [{
    cmpType: 'Input',
    label: 'Field Name',
    name: 'key'
  }, {
    cmpType: 'Form',
    name: 'value',
    fields: [validatorExport()]
  }]
}];
const getIconListMap = () => {
  return Object.keys(icons.getAll()).map(name => ({
    text: name,
    value: name
  }));
};
const withEditTabsConfig = cb => {
  return withEditTabs(cb({
    defaultParams,
    iconList: getIconListMap(),
    getIcons: getIconListMap,
    colorTypes: GLOBAL_OPTIONS.colorTypes.toArray(),
    iconColorTypes: GLOBAL_OPTIONS.iconColorTypes.toArray()
  }));
};
const withEditTabs = ({
  classNames: _classNames2 = [],
  pageData: _pageData = {},
  general: _general = [],
  validations: _validations = [],
  saveParams: _saveParams = {},
  enableValidations: _enableValidations = true,
  actions: _actions = []
}) => {
  const finalGeneral = [...defaultGeneral({
    classNames: _classNames2
  }), ..._general];
  const finalValidations = _enableValidations ? [...defaultValidations, ..._validations] : _validations;
  const isGeneralTab = finalGeneral && finalGeneral.length !== 0;
  const isValidationsTab = finalValidations && finalValidations.length !== 0;
  const isActionsTab = _actions && _actions.length !== 0;
  let firstSelectedTab = '';
  if (isGeneralTab) {
    firstSelectedTab = tabs$1.general.value;
  } else if (isValidationsTab) {
    firstSelectedTab = tabs$1.validations.value;
  } else if (isActionsTab) {
    firstSelectedTab = tabs$1.actions.value;
  }
  const createTabItems = (data, key) => {
    return data && key && data.length !== 0 ? data.map((item = {}) => Object.assign({}, item, {
      match: Object.assign({}, item.match, {
        editTab: {
          validators: [{
            type: 'equals',
            matchValue: key
          }]
        }
      })
    })) : [];
  };
  return {
    pageData: Object.assign({}, _pageData, {
      init: Object.assign({}, _pageData.init, {
        advanced: false,
        main: {
          editTab: firstSelectedTab
        }
      }),
      items: [{
        component: 'Wrapper',
        bodyClassName: 'text-right',
        items: [{
          component: 'Switch',
          name: 'advanced',
          label: 'Advanced Mode'
        }]
      }, {
        component: 'FormObject',
        name: 'main',
        match: {
          advanced: {
            validators: [{
              type: 'equals',
              matchValue: true
            }]
          }
        }
      }, {
        component: 'Form',
        name: 'main',
        fields: [{
          name: 'editTab',
          cmpType: 'Tabs',
          className: 'mb-wide',
          options: [...(isGeneralTab ? [tabs$1.general] : []), ...(isValidationsTab ? [tabs$1.validations] : []), ...(isActionsTab ? [tabs$1.actions] : []), tabs$1.match],
          match: {
            advanced: {
              validators: [{
                type: 'equals',
                matchValue: false
              }]
            }
          }
        }, ...createTabItems(finalGeneral, tabs$1.general.value), ...createTabItems(finalValidations, tabs$1.validations.value), ...createTabItems(_actions, tabs$1.actions.value), ...createTabItems(defaultMatch, tabs$1.match.value)],
        match: {
          advanced: {
            validators: [{
              type: 'equals',
              matchValue: false
            }]
          }
        }
      }, ...(_pageData.items ? _pageData.items : []), {
        component: 'Wrapper',
        bodyClassName: '',
        items: [{
          component: 'Button',
          actionType: 'submit-event',
          buttonText: 'Save',
          size: 'medium',
          params: Object.assign({
            '...main': '.main'
          }, _saveParams)
        }, {
          component: 'Button',
          actionType: 'cancel-event',
          buttonText: 'Cancel',
          variant: 'outlined',
          size: 'medium'
        }]
      }]
    })
  };
};

function selectBackgroundColor(isActive, canDrop) {
  if (isActive) {
    return 'sq-placeholder-drop--active';
  } else if (canDrop) {
    return 'sq-placeholder-drop--can-drop';
  } else {
    return 'sq-placeholder-drop--default';
  }
}
const Placeholder = ({
  component,
  name,
  allowedDropEffect,
  onDrop,
  accept,
  plaecHolderStyle: _plaecHolderStyle = 'box',
  hoverText
}) => {
  const [{
    canDrop,
    isOver
  }, drop] = useDrop(() => ({
    accept: accept || ItemTypes.COMPONENT,
    drop: (item, params) => {
      onDrop && onDrop(item, params);
      return {
        name: `${allowedDropEffect}placeholder`,
        allowedDropEffect
      };
    },
    collect: monitor => ({
      isOver: monitor.isOver(),
      canDrop: monitor.canDrop()
    })
  }), [allowedDropEffect]);
  const isActive = canDrop && isOver;
  const {
    Icon
  } = components$1.get();
  const backgroundCSS = selectBackgroundColor(isActive, canDrop);
  const finalName = component && name ? `for ${component}#${name}` : 'for root';
  return jsxs("div", {
    className: `sq-placeholder-drop ${backgroundCSS} sq-placeholder-drop-style-${_plaecHolderStyle}`,
    ref: drop,
    style: {},
    children: [jsx("div", {
      className: "sq-placeholder-drop__hover-text",
      children: _plaecHolderStyle === 'line' ? hoverText : ''
    }), jsxs("div", {
      className: "sq-placeholder-drop__content",
      children: [jsx(Icon, {
        name: "Move",
        variant: 'extra1'
      }), jsx("br", {}), isActive ? `Release to drop ${finalName}` : `Drag component here ${finalName}`]
    })]
  });
};
Placeholder.propTypes = {
  component: PropTypes.string,
  name: PropTypes.string,
  allowedDropEffect: PropTypes.string,
  onDrop: PropTypes.func,
  accept: PropTypes.array,
  plaecHolderStyle: PropTypes.string,
  hoverText: PropTypes.string
};

class ComponentEditor extends Component$1 {
  constructor(props) {
    super(props);
    this.state = {
      openSettings: false,
      active: false
    };
    this.toggleEditForm = this.toggleEditForm.bind(this);
    this.saveFormData = this.saveFormData.bind(this);
    this.cancelChanges = this.cancelChanges.bind(this);
    this.saveFormElData = this.saveFormElData.bind(this);
    this.onComponentDrop = this.onComponentDrop.bind(this);
    this.deleteComponentByIdx = this.deleteComponentByIdx.bind(this);
    this.deleteComponent = this.deleteComponent.bind(this);
    this.moveUp = this.moveUp.bind(this);
    this.moveDown = this.moveDown.bind(this);
    this.moveElemDown = this.moveElemDown.bind(this);
    this.moveElemUp = this.moveElemUp.bind(this);
  }
  toggleEditForm() {
    this.setState({
      openSettings: !this.state.openSettings
    });
  }
  cancelChanges() {
    this.toggleEditForm();
  }
  saveFormData(data) {
    const {
      onChange
    } = this.props;
    onChange && onChange(data);
    this.toggleEditForm();
  }
  saveFormElData(data, idx) {
    const {
      onChange,
      itemsPropName,
      value = {}
    } = this.props;
    const finalData = Object.assign(Object.assign({}, value), {
      [itemsPropName]: [...(value && value[itemsPropName] || [])]
    });
    finalData[itemsPropName][idx] = Object.assign(Object.assign({}, finalData[itemsPropName][idx]), data);
    onChange && onChange(finalData);
  }
  deleteComponentByIdx(idx) {
    const {
      onChange,
      itemsPropName,
      value = {}
    } = this.props;
    const finalData = Object.assign(Object.assign({}, value), {
      [itemsPropName]: [...(value && value[itemsPropName] || [])]
    });
    finalData[itemsPropName].splice(idx, 1);
    onChange && onChange(finalData);
  }
  deleteComponent() {
    const {
      onDelete
    } = this.props;
    onDelete && onDelete();
  }
  getName(name, items) {
    let idx = 1;
    let loop = 0;
    let fname = name.toLowerCase();
    while (items && loop < items.length) {
      if (items.filter(i => i.name === `${fname}${idx}`).length > 0) {
        idx++;
      } else {
        break;
      }
    }
    return `${fname}${idx}`;
  }
  onComponentDrop(dataComp, evt, idx) {
    const {
        metaData
      } = dataComp,
      data = __rest(dataComp, ["metaData"]);
    const {
      onChange,
      itemsPropName,
      compTypeProp,
      value
    } = this.props;
    let finalData;
    if (idx !== undefined) {
      const itemArra = [...(value && value[itemsPropName] || [])];
      itemArra.splice(idx, 0, Object.assign(Object.assign(Object.assign({
        [compTypeProp]: data.name
      }, metaData.sampleData), data), {
        name: this.getName(data.name, value[itemsPropName])
      }));
      finalData = Object.assign(Object.assign({}, value), {
        [itemsPropName]: itemArra
      });
    } else {
      finalData = Object.assign(Object.assign({}, value), {
        [itemsPropName]: [...(value && value[itemsPropName] || []), Object.assign(Object.assign(Object.assign({
          [compTypeProp]: data.name
        }, metaData.sampleData), data), {
          name: this.getName(data.name, value[itemsPropName])
        })]
      });
    }
    onChange && onChange(finalData);
  }
  moveUp() {
    const {
      onMoveUp
    } = this.props;
    onMoveUp && onMoveUp();
  }
  moveDown() {
    const {
      onMoveDown
    } = this.props;
    onMoveDown && onMoveDown();
  }
  moveElemUp(index) {
    const {
      onChange,
      itemsPropName,
      compTypeProp,
      value
    } = this.props;
    const items = value && value[itemsPropName] || [];
    let updatedCols = update(items, {
      $splice: [[index, 1], [index - 1, 0, items[index]]]
    });
    const finalData = Object.assign(Object.assign({}, value), {
      [itemsPropName]: updatedCols
    });
    onChange && onChange(finalData);
  }
  moveElemDown(index) {
    const {
      onChange,
      itemsPropName,
      compTypeProp,
      value
    } = this.props;
    const items = value && value[itemsPropName] || [];
    let updatedCols = update(items, {
      $splice: [[index, 1], [index + 1, 0, items[index]]]
    });
    const finalData = Object.assign(Object.assign({}, value), {
      [itemsPropName]: updatedCols
    });
    onChange && onChange(finalData);
  }
  // TODO:
  // onMouseEnter(e) {
  //   e.stopPropagation();
  //   e.preventDefault();
  //   this.setState({ active: true });
  // }
  // onMouseLeave(e) {
  //   e.preventDefault();
  //   e.stopPropagation();
  //   this.setState({ active: false });
  // }
  render() {
    const {
      pageData = {},
      Component: RenderedComponent,
      fieldsMeta,
      idx,
      itemsPropName,
      name,
      isStart,
      isEnd,
      value,
      compTypeProp,
      component,
      editData
    } = this.props;
    const {
      hasItems
    } = this.props;
    const {
      [itemsPropName]: items
    } = value || {};
    const {
      className = ''
    } = pageData;
    const {
      hasPlaceholder,
      accept,
      compList,
      defaultComp
    } = this.props;
    const identifier = `${component}${value.className ? `.${value.className}` : ''}${value.name ? `#${value.name}` : ''}`;
    const {
      Dialog,
      Actions
    } = components$1.get();
    const Component = components$1.get()[RenderedComponent];
    return jsxs("div", {
      className: `sq-component-editor ${className} ${this.state.active ? 'active' : ''} ${value.className || ''}`,
      children: [jsx(Dialog, {
        classes: {
          dialog: {
            root: 'sq-dialog--fixed-right'
          }
        },
        backDropClick: false,
        disableEscapeKeyDown: true,
        onClose: this.toggleEditForm,
        title: `${component} Configuration`,
        open: this.state.openSettings,
        children: jsx("div", {
          className: "mt-wide",
          children: editData && jsx(DynamicContentRoot, {
            pageConfig: editData,
            keyPressChange: false,
            initialData: {
              main: value,
              fieldsMeta
            },
            onSubmit: this.saveFormData,
            onCancel: this.cancelChanges
          })
        })
      }), jsx(Tooltip, {
        title: identifier,
        children: jsxs("div", {
          className: "sq-component-editor__name",
          children: [component, value.name ? `#${value.name}` : '', value.className ? `.${value.className}` : '']
        })
      }), jsx("div", {
        className: "sq-component-editor__actions",
        children: jsx(Actions, {
          actions: [{
            cmpType: 'IconButton',
            size: 'small',
            iconSize: 'small',
            iconName: 'arrow-up',
            onClick: this.moveUp,
            beforeRender: () => !isStart
          }, {
            cmpType: 'IconButton',
            size: 'small',
            iconSize: 'small',
            iconName: 'arrow-down',
            onClick: this.moveDown,
            beforeRender: () => !isEnd
          }, {
            cmpType: 'IconButton',
            size: 'small',
            iconSize: 'small',
            iconName: 'Settings',
            onClick: this.toggleEditForm
          }, {
            cmpType: 'IconButton',
            size: 'small',
            iconSize: 'small',
            iconName: 'Delete',
            onClick: this.deleteComponent,
            confirm: {
              title: 'Delete?',
              content: 'Are you sure you want to delete?'
            }
          }]
        })
      }), jsx("div", {
        className: `sq-component-editor__container ${value.bodyClassName}`,
        children: jsxs(ErrorBoundry$1, {
          children: [!hasItems && jsxs(ErrorBoundry$1, {
            children: [Component && jsx(Component, Object.assign({}, value)), !Component && jsx("div", {
              className: "text-center",
              children: "Preview not available"
            })]
          }), hasItems && items && items.map((item, idx) => {
            const Component = compList[item[compTypeProp]] || (!item[compTypeProp] && defaultComp ? compList[defaultComp] : compList.Custom);
            const _a = item,
              _b = compTypeProp;
              _a[_b];
              __rest(_a, [typeof _b === "symbol" ? _b : _b + ""]);
            return jsxs(ErrorBoundry$1, {
              children: [hasPlaceholder && jsx(Placeholder, {
                hoverText: identifier,
                plaecHolderStyle: "line",
                name: value.name,
                component: component,
                accept: accept,
                onDrop: (d, evt) => this.onComponentDrop(d, evt, idx)
              }), jsx(ComponentEditor, Object.assign({
                idx: idx,
                parentName: item.name,
                fieldsMeta: fieldsMeta,
                accept: accept,
                component: item[compTypeProp] || defaultComp,
                isStart: idx === 0,
                isEnd: idx === items.length - 1,
                index: idx,
                itemsPropName: itemsPropName,
                compList: compList
              }, Component, {
                value: item,
                onDelete: () => this.deleteComponentByIdx(idx),
                onChange: data => this.saveFormElData(data, idx),
                onMoveUp: () => this.moveElemUp(idx),
                onMoveDown: () => this.moveElemDown(idx)
              }))]
            }, idx);
          })]
        })
      }), hasPlaceholder && jsx(Placeholder, {
        hoverText: identifier,
        plaecHolderStyle: !items || (items === null || items === void 0 ? void 0 : items.length) === 0 ? undefined : 'line',
        name: value.name,
        component: component,
        accept: accept,
        onDrop: this.onComponentDrop
      })]
    });
  }
}
ComponentEditor.propTypes = {
  data: PropTypes.object
};

class ContentEditor extends Component$1 {
  constructor(props) {
    super(props);
    this.state = {};
    this.componentOnChange = this.componentOnChange.bind(this);
    this.componentOnDelete = this.componentOnDelete.bind(this);
    this.handleMoveUp = this.handleMoveUp.bind(this);
    this.handleMoveDown = this.handleMoveDown.bind(this);
    this.componentOnDrop = this.componentOnDrop.bind(this);
  }
  componentOnChange(data, idx) {
    const {
      onChange
    } = this.props;
    onChange && onChange(data, idx);
  }
  componentOnDelete(idx) {
    const {
      onDelete
    } = this.props;
    onDelete && onDelete(idx);
  }
  handleMoveDown(idx) {
    const {
      onMoveItemDown
    } = this.props;
    onMoveItemDown && onMoveItemDown(idx);
  }
  componentOnDrop(item, evt, atIdx) {
    const {
      onDrop
    } = this.props;
    onDrop && onDrop(item, evt, atIdx);
  }
  handleMoveUp(idx) {
    const {
      onMoveItemUp
    } = this.props;
    onMoveItemUp && onMoveItemUp(idx);
  }
  render() {
    var _a;
    const {
      pageData = {},
      compList = {},
      fieldsMeta
    } = this.props;
    const {
      className = ''
    } = pageData;
    return jsxs("div", {
      className: `sq-content-editor ${className}`,
      children: [pageData.items && pageData.items.map((block, idx) => {
        const Component = compList[block.component] || compList.Custom;
        return jsxs(ErrorBoundry$1, {
          children: [jsx(Placeholder, {
            allowedDropEffect: 'any',
            hoverText: 'root',
            plaecHolderStyle: "line",
            accept: [ItemTypes.COMPONENT, ItemTypes.FORM],
            onDrop: (item, e) => this.componentOnDrop(item, e, idx)
          }), jsx(ComponentEditor, Object.assign({
            component: block.component,
            fieldsMeta: fieldsMeta
          }, block.metaData, {
            isStart: idx === 0,
            isEnd: idx === pageData.items.length - 1
          }, Component, {
            value: block,
            compList: compList,
            onDelete: () => this.componentOnDelete(idx),
            onChange: data => this.componentOnChange(data, idx),
            onMoveUp: () => this.handleMoveUp(idx),
            onMoveDown: () => this.handleMoveDown(idx)
          }))]
        }, idx);
      }), jsx(Placeholder, {
        allowedDropEffect: 'any',
        hoverText: 'root',
        plaecHolderStyle: ((_a = pageData.items) === null || _a === void 0 ? void 0 : _a.length) > 0 ? 'line' : undefined,
        accept: [ItemTypes.COMPONENT, ItemTypes.FORM],
        onDrop: this.componentOnDrop
      })]
    });
  }
}
ContentEditor.propTypes = {
  pageData: PropTypes.object,
  compList: PropTypes.object
};

var HeaderEdit = withEditTabsConfig(({
  defaultParams
}) => ({
  enableValidations: false,
  classNames: GLOBAL_OPTIONS.headerStyles.toArray(),
  pageData: {
    items: []
  },
  general: [{
    name: 'header',
    cmpType: 'Input',
    label: 'Header'
  }, {
    name: 'headerTag',
    cmpType: 'Select',
    label: 'Header Tag',
    options: GLOBAL_OPTIONS.headerTags.toArray()
  }, {
    name: 'subHeader',
    cmpType: 'RichText',
    label: 'Sub Header',
    editorStyle: 'full'
  }]
}));

var HeroContentEdit = withEditTabsConfig(({
  defaultParams,
  getIcons
}) => ({
  classNames: GLOBAL_OPTIONS.heroStyles.toArray(),
  enableValidations: false,
  pageData: {
    items: []
  },
  general: [{
    name: 'header',
    cmpType: 'Input',
    label: 'Header'
  }, {
    name: 'headerTag',
    cmpType: 'Select',
    label: 'Header Tag',
    options: GLOBAL_OPTIONS.headerTags.toArray()
  }, {
    name: 'templateClassName',
    cmpType: 'Input',
    label: 'templateClassName'
  }, {
    name: 'imageUrl',
    cmpType: 'Input',
    label: 'imageUrl'
  }, {
    name: 'background',
    cmpType: 'Input',
    label: 'background'
  }, {
    name: 'template',
    cmpType: 'Autocomplete',
    label: 'template',
    options: GLOBAL_OPTIONS.heroTemplates.toArray()
  }, {
    name: 'subHeader',
    cmpType: 'RichText',
    label: 'Sub Header',
    editorStyle: 'full'
  }, {
    name: 'links',
    cmpType: 'FormList',
    label: 'Links',
    formClassName: 'sq-form--2-cols',
    fields: [{
      cmpType: 'Input',
      label: 'Button Text',
      name: 'buttonText'
    }, {
      cmpType: 'Autocomplete',
      label: 'Icon Name',
      name: 'iconName',
      options: getIcons
    }, {
      cmpType: 'Autocomplete',
      label: 'Type',
      name: 'type',
      options: GLOBAL_OPTIONS.linkTypes.toArray()
    }, {
      cmpType: 'Autocomplete',
      label: 'variant',
      name: 'variant',
      options: GLOBAL_OPTIONS.variants.toArray()
    }, {
      cmpType: 'Autocomplete',
      label: 'size',
      name: 'size',
      options: GLOBAL_OPTIONS.buttonSize.toArray()
    }, {
      cmpType: 'Input',
      label: 'Icon Direction',
      name: 'iconDirection'
    }, {
      cmpType: 'Input',
      label: 'To Path',
      name: 'to'
    }]
  }]
}));

var IframeEdit = withEditTabsConfig(({
  defaultParams
}) => ({
  enableValidations: false,
  pageData: {
    items: []
  },
  general: [{
    name: 'url',
    cmpType: 'Input',
    label: 'Url'
  }, {
    name: 'iframeParams',
    cmpType: 'FormObject',
    label: 'iframeParams'
  }]
}));

var LaunchTimerEdit = withEditTabs({
  enableValidations: false,
  pageData: {
    items: []
  },
  general: [{
    name: 'value',
    cmpType: 'DateSelector',
    instanceType: 'DateTimePicker',
    inputFormat: 'MM/dd/yyyy HH:mm',
    mask: '__/__/____ __:__',
    label: 'value'
  }]
});

var CodeHighlightEdit = withEditTabs({
  pageData: {
    items: []
  },
  general: [{
    name: 'code',
    cmpType: 'Textarea',
    label: 'code',
    minRows: 15
  }, {
    name: 'language',
    cmpType: 'Input',
    label: 'language'
  }]
});

var PricingEdit = withEditTabsConfig(({
  defaultParams,
  getIcons,
  iconColorTypes,
  colorTypes
}) => ({
  enableValidations: false,
  pageData: {
    items: []
  },
  general: [{
    name: 'icon',
    cmpType: 'Autocomplete',
    label: 'icon',
    options: getIcons
  }, {
    name: 'eyebrow',
    label: 'eyebrow'
  }, {
    name: 'template',
    cmpType: 'Autocomplete',
    label: 'template',
    options: GLOBAL_OPTIONS.pricingTemplates.toArray()
  }, {
    name: 'header',
    label: 'header'
  }, {
    name: 'subHeader',
    label: 'subHeader'
  }, {
    name: 'features',
    label: 'feature',
    cmpType: 'FormObject'
  }, {
    name: 'items',
    label: 'items',
    cmpType: 'FormList',
    fields: [{
      name: 'header',
      label: 'header'
    }, {
      name: 'price',
      label: 'price'
    }, {
      name: 'color',
      cmpType: 'Autocomplete',
      label: 'color',
      options: colorTypes
    }, {
      cmpType: 'Autocomplete',
      name: 'icon',
      label: 'icon',
      options: getIcons
    }, {
      label: 'features',
      name: 'features',
      cmpType: 'FormObject'
    }, {
      label: 'actions',
      name: 'actions',
      cmpType: 'FormObject'
    }]
  }]
}));

var ContactUsInfoEdit = withEditTabsConfig(({
  defaultParams,
  getIcons,
  iconColorTypes
}) => ({
  enableValidations: false,
  pageData: {
    items: []
  },
  general: [{
    name: 'style',
    cmpType: 'Input',
    label: 'style'
  }, {
    name: 'template',
    cmpType: 'Autocomplete',
    label: 'template',
    options: GLOBAL_OPTIONS.featureContentTemplates.toArray()
  }, {
    name: 'classes',
    cmpType: 'FormObject',
    label: 'classes'
  }, {
    name: 'items',
    cmpType: 'FormList',
    label: 'items',
    formClassName: 'sq-form--2-cols',
    fields: [{
      cmpType: 'Autocomplete',
      label: 'Icon',
      options: getIcons,
      name: 'iconName'
    }, {
      cmpType: 'Autocomplete',
      label: 'Icon color',
      options: iconColorTypes,
      name: 'iconColor'
    }, {
      cmpType: 'Input',
      label: 'Icon size',
      name: 'iconSize'
    }, {
      cmpType: 'Input',
      label: 'header',
      name: 'header'
    }, {
      cmpType: 'Input',
      label: 'subHeader',
      name: 'subHeader'
    }, {
      cmpType: 'FormObject',
      label: 'links',
      name: 'links'
    }]
  }]
}));

var ImageWithSpotlightEdit = withEditTabsConfig(({
  defaultParams,
  getIcons,
  colorTypes
}) => ({
  enableValidations: false,
  pageData: {
    items: []
  },
  general: [{
    name: 'header',
    cmpType: 'Input',
    label: 'header'
  }, {
    name: 'bgImageUrl',
    cmpType: 'Input',
    label: 'bgImageUrl'
  }, {
    name: 'actions',
    cmpType: 'FormList',
    fields: [{
      name: 'buttonText',
      label: 'buttonText'
    }, {
      name: 'iconName',
      cmpType: 'Autocomplete',
      options: getIcons,
      label: 'iconName'
    }, {
      name: 'actionType',
      label: 'actionType'
    }, {
      name: 'to',
      label: 'to'
    }, {
      name: 'color',
      cmpType: 'Autocomplete',
      options: colorTypes,
      label: 'color'
    }]
  }]
}));

var ImageBlockWithTextEdit = withEditTabsConfig(({
  defaultParams,
  iconList
}) => ({
  classNames: GLOBAL_OPTIONS.imageBlockWithTextCss.toArray(),
  enableValidations: false,
  pageData: {
    items: []
  },
  general: [{
    name: 'styleName',
    cmpType: 'Autocomplete',
    options: GLOBAL_OPTIONS.imageBlockWithTextStyles.toArray(),
    label: 'styleName'
  }, {
    name: 'header',
    cmpType: 'Input',
    label: 'header'
  }, {
    name: 'imageUrl',
    cmpType: 'Input',
    label: 'imageUrl'
  }, {
    name: 'subHeader',
    cmpType: 'RichText',
    label: 'subHeader'
  }, {
    cmpType: 'FormObject',
    label: 'links',
    name: 'links'
  }]
}));

var ImageWithHeaderBodyEdit = withEditTabsConfig(({
  defaultParams,
  iconList
}) => ({
  enableValidations: false,
  pageData: {
    items: []
  },
  general: [{
    name: 'styleName',
    cmpType: 'Autocomplete',
    options: GLOBAL_OPTIONS.imageWithHeaderBody.toArray(),
    label: 'styleName'
  }, {
    name: 'imageUrl',
    cmpType: 'Input',
    label: 'imageUrl'
  }, {
    name: 'header',
    cmpType: 'Input',
    label: 'header'
  }, {
    name: 'subHeader',
    cmpType: 'RichText',
    label: 'subHeader'
  }, {
    cmpType: 'FormObject',
    label: 'links',
    name: 'links'
  }]
}));

var TeamEdit = withEditTabsConfig(({
  defaultParams,
  iconList
}) => ({
  enableValidations: false,
  pageData: {
    items: []
  },
  general: [{
    name: 'style',
    cmpType: 'Input',
    label: 'style'
  }, {
    name: 'header',
    cmpType: 'Input',
    label: 'header'
  }, {
    name: 'classes',
    cmpType: 'FormObject',
    label: 'classes'
  }, {
    name: 'items',
    cmpType: 'FormList',
    label: 'items',
    formClassName: 'sq-form--2-cols',
    fields: [{
      cmpType: 'Input',
      label: 'Picture',
      name: 'profilePic'
    }, {
      cmpType: 'Input',
      label: 'header',
      name: 'header'
    }, {
      cmpType: 'Input',
      label: 'designation',
      name: 'designation'
    }, {
      cmpType: 'Input',
      label: 'subHeader',
      name: 'subHeader'
    }, {
      cmpType: 'FormObject',
      label: 'links',
      name: 'links'
    }]
  }]
}));

var ImageWithSlideEdit = withEditTabsConfig(({
  defaultParams,
  iconList
}) => ({
  enableValidations: false,
  pageData: {
    items: []
  },
  general: [{
    name: 'image',
    cmpType: 'FormObject',
    label: 'image'
  }, {
    name: 'align',
    cmpType: 'Autocomplete',
    label: 'align',
    options: GLOBAL_OPTIONS.imageAlign.toArray()
  }, {
    name: 'classes',
    cmpType: 'FormObject',
    label: 'classes'
  }, {
    name: 'eyebrow',
    cmpType: 'Input',
    label: 'eyebrow'
  }, {
    name: 'header',
    cmpType: 'Input',
    label: 'header'
  }, {
    name: 'headerTag',
    cmpType: 'Autocomplete',
    label: 'headerTag',
    options: GLOBAL_OPTIONS.headerTags.toArray()
  }, {
    name: 'bgColor',
    cmpType: 'ColorPicker',
    label: 'bgColor'
  }]
}));

var UsefulLinksEdit = withEditTabsConfig(({
  defaultParams,
  getIcons,
  iconColorTypes
}) => ({
  classNames: [{
    text: 're-useful-links--gray-background',
    value: 're-useful-links--gray-background'
  }, {
    text: 're-useful-links--cols-only',
    value: 're-useful-links--cols-only'
  }],
  pageData: {
    items: []
  },
  general: [{
    name: 'header',
    label: 'Header'
  }, {
    name: 'links',
    cmpType: 'FormList',
    label: 'Links',
    fields: [{
      name: 'text',
      label: 'Text'
    }, {
      name: 'iconName',
      cmpType: 'Autocomplete',
      options: getIcons,
      label: 'iconName'
    }, {
      name: 'iconColor',
      cmpType: 'Autocomplete',
      options: iconColorTypes,
      label: 'iconColor'
    }, {
      name: 'to',
      label: 'Url'
    }]
  }]
}));

var DisclaimerEdit = withEditTabsConfig(({
  defaultParams
}) => ({
  classNames: GLOBAL_OPTIONS.dislcaimerStyles.toArray(),
  pageData: {
    items: []
  },
  general: [{
    name: 'text',
    cmpType: 'RichText',
    label: 'Disclaimer'
  }, {
    name: 'innerClassName',
    cmpType: 'Input',
    label: 'innerClassName'
  }]
}));

var FeaturedContentEdit = withEditTabsConfig(({
  defaultParams,
  getIcons,
  iconColorTypes
}) => ({
  enableValidations: false,
  pageData: {
    items: []
  },
  general: [{
    name: 'itemClassName',
    cmpType: 'Input',
    label: 'itemClassName'
  }, {
    name: 'template',
    cmpType: 'Autocomplete',
    label: 'template',
    options: GLOBAL_OPTIONS.featureContentTemplates.toArray()
  }, {
    name: 'items',
    cmpType: 'FormList',
    label: 'items',
    formClassName: 'sq-form--2-cols',
    fields: [{
      cmpType: 'Autocomplete',
      label: 'Icon',
      options: getIcons,
      name: 'icon'
    }, {
      cmpType: 'Autocomplete',
      label: 'Icon color',
      options: iconColorTypes,
      name: 'iconColor'
    }, {
      cmpType: 'Input',
      label: 'Icon size',
      name: 'iconSize'
    }, {
      cmpType: 'Input',
      label: 'title',
      name: 'title'
    }, {
      cmpType: 'Input',
      label: 'description',
      name: 'description'
    }, {
      cmpType: 'FormObject',
      label: 'links',
      name: 'links'
    }]
  }]
}));

var TextColumnWithLinksEdit = withEditTabsConfig(({
  defaultParams,
  getIcons,
  iconColorTypes
}) => ({
  enableValidations: false,
  pageData: {
    items: []
  },
  general: [{
    name: 'style',
    cmpType: 'Input',
    label: 'style'
  }, {
    name: 'template',
    cmpType: 'Autocomplete',
    label: 'template',
    options: GLOBAL_OPTIONS.featureContentTemplates.toArray()
  }, {
    name: 'classes',
    cmpType: 'FormObject',
    label: 'classes'
  }, {
    name: 'items',
    cmpType: 'FormList',
    label: 'items',
    formClassName: 'sq-form--2-cols',
    fields: [{
      cmpType: 'Autocomplete',
      label: 'Icon',
      options: getIcons,
      name: 'iconName'
    }, {
      cmpType: 'Autocomplete',
      label: 'Icon color',
      options: iconColorTypes,
      name: 'iconColor'
    }, {
      cmpType: 'Input',
      label: 'Icon size',
      name: 'iconSize'
    }, {
      cmpType: 'Input',
      label: 'header',
      name: 'header'
    }, {
      cmpType: 'Input',
      label: 'subHeader',
      name: 'subHeader'
    }]
  }]
}));

var GetInTouchEdit = withEditTabsConfig(({
  defaultParams,
  getIcons,
  iconColorTypes
}) => ({
  pageData: {
    items: []
  },
  general: [{
    name: 'header',
    label: 'Header'
  }, {
    name: 'bgColor',
    label: 'Background Color',
    helperText: 'Use only dark colors'
  }, {
    name: 'footerText',
    cmpType: 'RichText',
    label: 'Footer Text'
  }, {
    name: 'info',
    cmpType: 'FormList',
    label: 'Info',
    fields: [{
      name: 'iconName',
      cmpType: 'Autocomplete',
      options: getIcons,
      label: 'Icon'
    }, {
      name: 'text',
      cmpType: 'Input',
      label: 'Text'
    }, {
      name: 'iconColor',
      cmpType: 'Autocomplete',
      options: iconColorTypes,
      label: 'iconColor'
    }, {
      name: 'href',
      label: 'Href'
    }, {
      name: 'target',
      cmpType: 'CheckboxField',
      selectedValue: '_blank',
      text: 'Open in new window'
    }]
  }]
}));

var RichTextEdit = withEditTabs({
  pageData: {
    items: []
  },
  general: [{
    name: 'label',
    cmpType: 'Input',
    label: 'Label'
  }, {
    name: 'className',
    cmpType: 'Input',
    label: 'className'
  }, {
    name: 'editorStyle',
    cmpType: 'Autocomplete',
    label: 'editorStyle',
    options: GLOBAL_OPTIONS.editorStyle.toArray()
  }]
});

var InputEdit = withEditTabs({
  pageData: {
    items: []
  },
  general: [{
    name: 'label',
    cmpType: 'Input',
    label: 'Label'
  }, {
    name: 'helperText',
    cmpType: 'Input',
    label: 'helperText'
  }, {
    name: 'variant',
    cmpType: 'Radio',
    label: 'variant',
    options: GLOBAL_OPTIONS.inputVariant.toArray()
  }, {
    name: 'type',
    cmpType: 'CheckboxField',
    text: 'Password',
    selectedValue: 'password'
  }, {
    name: 'mask',
    cmpType: 'Form',
    fields: [{
      cmpType: 'Autocomplete',
      name: 'type',
      label: 'Mask Type',
      options: GLOBAL_OPTIONS.maskTypes.toArray()
    }]
  }]
});

var ReCaptchaEdit = withEditTabs({
  pageData: {
    items: []
  },
  general: [{
    name: 'sitekey',
    cmpType: 'Input',
    label: 'sitekey'
  }]
});

var SelectEdit = withEditTabs({
  pageData: {
    items: []
  },
  general: [{
    name: 'label',
    cmpType: 'Input',
    label: 'Label'
  }, {
    name: 'inputVariant',
    cmpType: 'Radio',
    label: 'Input Variant',
    options: GLOBAL_OPTIONS.inputVariant.toArray()
  }, {
    name: 'options',
    cmpType: 'FormList',
    label: 'Options',
    formClassName: 'sq-form--2-cols',
    fields: [{
      cmpType: 'Input',
      label: 'Value',
      name: 'value'
    }, {
      cmpType: 'Input',
      label: 'Text',
      name: 'text'
    }]
  }]
});

var RadioEdit = withEditTabs({
  pageData: {
    items: []
  },
  general: [{
    name: 'label',
    cmpType: 'Input',
    label: 'Label'
  }, {
    name: 'className',
    cmpType: 'Input',
    label: 'className'
  }, {
    name: 'display',
    cmpType: 'Radio',
    options: GLOBAL_OPTIONS.radioDisplay.toArray()
  }, {
    name: 'options',
    cmpType: 'FormList',
    label: 'Options',
    formClassName: 'sq-form--2-cols',
    fields: [{
      cmpType: 'Input',
      label: 'Key',
      name: 'value'
    }, {
      cmpType: 'Input',
      label: 'Value',
      name: 'text'
    }]
  }]
});

var FormEdit = withEditTabs({
  enableValidations: false,
  pageData: {
    items: []
  },
  general: [{
    name: 'group',
    label: 'Validation Group'
  }, {
    name: 'forceValidate',
    label: 'forceValidate',
    cmpType: 'CheckboxField',
    selectedValue: true,
    defaultValue: false
  }]
});

var CheckboxEdit = withEditTabs({
  pageData: {
    items: []
  },
  general: [{
    name: 'className',
    cmpType: 'Input',
    label: 'className'
  }, {
    name: 'label',
    cmpType: 'Input',
    label: 'Label'
  }, {
    name: 'text',
    cmpType: 'Input',
    label: 'Text'
  }, {
    name: 'selectedValue',
    cmpType: 'Input',
    label: 'selectedValue'
  }, {
    name: 'defaultValue',
    cmpType: 'Input',
    label: 'defaultValue'
  }]
});

var CheckboxListEdit = withEditTabs({
  pageData: {
    items: []
  },
  general: [{
    name: 'className',
    cmpType: 'Input',
    label: 'className'
  }, {
    name: 'label',
    cmpType: 'Input',
    label: 'Label'
  }, {
    name: 'textField',
    cmpType: 'Input',
    label: 'textField'
  }, {
    name: 'valueField',
    cmpType: 'Input',
    label: 'valueField'
  }, {
    name: 'options',
    cmpType: 'FormList',
    label: 'Options',
    formClassName: 'sq-form--2-cols',
    fields: [{
      cmpType: 'Input',
      label: 'Value',
      name: 'value'
    }, {
      cmpType: 'Input',
      label: 'Text',
      name: 'text'
    }]
  }]
});

var WrapperEdit = withEditTabsConfig(({
  defaultParams: _defaultParams = {}
}) => ({
  enableValidations: false,
  pageData: {
    items: []
  },
  general: [{
    name: 'bodyClassName',
    cmpType: 'Input',
    label: 'bodyClassName'
  }, {
    name: 'bodyContainerClassName',
    cmpType: 'Input',
    label: 'bodyContainerClassName'
  }]
}));

var ButtonEdit = withEditTabs({
  enableValidations: false,
  enableActions: true,
  classNames: GLOBAL_OPTIONS.formStyles.toArray(),
  saveParams: {},
  pageData: {
    merge: {
      liveFields: '.fieldsMeta.liveFields',
      formFields: '.fieldsMeta.formFields'
    },
    items: []
  },
  general: [{
    name: 'buttonText',
    cmpType: 'Input',
    label: 'buttonText'
  }, {
    name: 'size',
    cmpType: 'Select',
    label: 'size',
    options: GLOBAL_OPTIONS.buttonSize.toArray()
  }, {
    name: 'actionType',
    cmpType: 'Select',
    label: 'Action Type',
    options: GLOBAL_OPTIONS.actionTypes.toArray()
  }, {
    name: 'url',
    cmpType: 'Input',
    label: 'url'
  }, {
    name: 'to',
    cmpType: 'Input',
    label: 'to'
  }, {
    name: 'method',
    cmpType: 'Radio',
    display: 'Inline',
    options: GLOBAL_OPTIONS.methodTypes.toArray(),
    label: 'method'
  }, {
    name: 'variant',
    cmpType: 'Autocomplete',
    options: GLOBAL_OPTIONS.variants.toArray(),
    label: 'variant'
  }, {
    name: 'advanced',
    cmpType: 'Switch',
    label: 'Advanced params'
  }, {
    name: 'params',
    cmpType: 'FormObject',
    output: 'object',
    label: 'params',
    match: {
      fieldsMeta: {
        validators: [{
          type: 'or',
          validations: [{
            type: 'notExists'
          }, {
            type: 'equals',
            fieldName: 'advanced',
            matchValue: true
          }]
        }]
      }
    }
  }, {
    name: 'params',
    cmpType: 'FormObject',
    output: 'object',
    label: 'params',
    fields: [{
      cmpType: 'Autocomplete',
      name: 'key',
      label: 'Field Name',
      inject: {
        options: 'fieldsMeta.formFields'
      },
      textField: 'name',
      valueField: 'code',
      match: {
        formFields: {
          validators: [{
            type: 'exists'
          }]
        }
      }
    }, {
      cmpType: 'Input',
      name: 'key',
      label: 'Field Name',
      match: {
        formFields: {
          validators: [{
            type: 'notExists'
          }]
        }
      }
    }, {
      label: 'Value Field',
      cmpType: 'Input',
      name: 'value',
      match: {
        liveFields: {
          validators: [{
            type: 'notExists'
          }]
        }
      }
    }, {
      label: 'Value Field',
      cmpType: 'Autocomplete',
      inject: {
        options: 'fieldsMeta.liveFields'
      },
      name: 'value',
      match: {
        liveFields: {
          validators: [{
            type: 'exists'
          }]
        }
      }
    }],
    match: {
      fieldsMeta: {
        validators: [{
          type: 'exists'
        }]
      },
      advanced: {
        validators: [{
          type: 'notExists'
        }]
      }
    }
  }, {
    name: 'postHook',
    cmpType: 'Input',
    label: 'postHook'
  }, {
    name: 'validateGroup',
    cmpType: 'Input',
    label: 'validateGroup'
  }, {
    name: 'defaultResponse',
    cmpType: 'Form',
    fields: [{
      cmpType: 'FormObject',
      name: 'success',
      label: 'success'
    }, {
      cmpType: 'FormObject',
      nameL: 'success',
      label: 'error'
    }]
  }, {
    name: 'successAfterScript',
    cmpType: 'Textarea',
    label: 'successAfterScript'
  }, {
    name: 'errorAfterScript',
    cmpType: 'Textarea',
    label: 'errorAfterScript'
  }]
});

var TextEdit = withEditTabs({
  enableValidations: false,
  pageData: {
    items: []
  },
  general: [{
    name: 'text',
    cmpType: 'Input',
    label: 'text'
  }, {
    name: 'tag',
    cmpType: 'Autocomplete',
    options: GLOBAL_OPTIONS.htmlTags.toArray(),
    label: 'Tag'
  }, {
    name: 'parentTag',
    cmpType: 'Autocomplete',
    options: GLOBAL_OPTIONS.htmlTags.toArray(),
    label: 'Wrapper Tag'
  }]
});

var TabsEdit = withEditTabs({
  enableValidations: false,
  pageData: {
    items: []
  },
  general: [{
    name: 'textField',
    cmpType: 'Input',
    label: 'textField'
  }, {
    name: 'valueField',
    cmpType: 'Input',
    label: 'valueField'
  }, {
    name: 'options',
    cmpType: 'FormList',
    label: 'Options',
    formClassName: 'sq-form--2-cols',
    fields: [{
      cmpType: 'Input',
      label: 'Value',
      name: 'value'
    }, {
      cmpType: 'Input',
      label: 'Text',
      name: 'text'
    }]
  }]
});

var TextareaEdit = withEditTabs({
  pageData: {
    items: []
  },
  general: [{
    name: 'label',
    cmpType: 'Input',
    label: 'Label'
  }, {
    name: 'minRows',
    cmpType: 'Input',
    label: 'minRows'
  }, {
    name: 'maxRows',
    cmpType: 'Input',
    label: 'maxRows'
  }]
});

var ProgressEdit = withEditTabs({
  enableValidations: false,
  pageData: {
    items: []
  },
  general: [{
    name: 'type',
    cmpType: 'Autocomplete',
    options: GLOBAL_OPTIONS.progressTypes.toArray(),
    label: 'type'
  }, {
    name: 'style',
    cmpType: 'Autocomplete',
    options: GLOBAL_OPTIONS.progressStyles.toArray(),
    label: 'style'
  }, {
    name: 'color',
    cmpType: 'Autocomplete',
    options: GLOBAL_OPTIONS.colorTypes.toArray(),
    label: 'color'
  }, {
    name: 'overlay',
    cmpType: 'Switch',
    label: 'overlay'
  }, {
    name: 'overlayStyle',
    cmpType: 'Input',
    label: 'overlayStyle'
  }]
});

// import { GLOBAL_OPTIONS } from '../../../globals';
// import { withEditTabs } from './Common';
var CustomEdit = {
  pageData: {
    init: {},
    items: [{
      component: 'FormObject',
      name: 'main'
    }, {
      component: 'Button',
      actionType: 'submit-event',
      buttonText: 'Save',
      size: 'medium',
      params: {
        '...main': '.main'
      }
    }]
  }
};

var HTMLEdit = withEditTabsConfig(({
  defaultParams
}) => ({
  enableValidations: false,
  pageData: {
    items: []
  },
  general: [{
    name: 'html',
    cmpType: 'RichText',
    label: 'Html',
    editorStyle: 'full'
  }]
}));

var ImageEdit = withEditTabsConfig(({
  defaultParams
}) => ({
  classNames: GLOBAL_OPTIONS.imageStyles.toArray(),
  enableValidations: false,
  pageData: {
    items: []
  },
  general: [{
    name: 'imageUrl',
    cmpType: 'Input',
    label: 'imageUrl'
  }, {
    name: 'size',
    cmpType: 'Input',
    label: 'size'
  }, {
    name: 'style',
    cmpType: 'FormObject',
    label: 'style'
  }]
}));

var DateSelectorEdit = withEditTabs({
  pageData: {
    items: []
  },
  general: [{
    name: 'label',
    cmpType: 'Input',
    label: 'label'
  }, {
    name: 'inputFormat',
    cmpType: 'Input',
    label: 'inputFormat'
  }, {
    name: 'outputFormat',
    cmpType: 'Input',
    label: 'outputFormat'
  }, {
    name: 'instanceType',
    cmpType: 'Autocomplete',
    label: 'instanceType',
    options: GLOBAL_OPTIONS.datePickerInstanceType.toArray()
  }, {
    name: 'mask',
    cmpType: 'Input',
    label: 'mask'
  }]
});

var AutocompleteEdit = withEditTabs({
  pageData: {
    items: []
  },
  general: [{
    name: 'label',
    cmpType: 'Input',
    label: 'Label'
  }, {
    name: 'options',
    cmpType: 'FormList',
    label: 'Options',
    formClassName: 'sq-form--2-cols',
    fields: [{
      cmpType: 'Input',
      label: 'Value',
      name: 'value'
    }, {
      cmpType: 'Input',
      label: 'Text',
      name: 'text'
    }]
  }]
});

var AlertEdit = withEditTabs({
  pageData: {
    items: []
  },
  general: [{
    name: 'message',
    cmpType: 'Input',
    label: 'Message'
  }, {
    name: 'header',
    cmpType: 'Input',
    label: 'Header'
  }, {
    name: 'type',
    cmpType: 'Autocomplete',
    label: 'Type',
    options: GLOBAL_OPTIONS.alerTypes.toArray()
  }, {
    name: 'variant',
    cmpType: 'Autocomplete',
    label: 'Variant',
    options: GLOBAL_OPTIONS.inputVariant.toArray()
  }, {
    name: 'color',
    cmpType: 'Autocomplete',
    label: 'Color',
    options: GLOBAL_OPTIONS.alerTypes.toArray()
  }]
});

// import Input from '../../components/Input';
// import RichText from '../../components/RichText';
// import Text from '../../components/Text';
// import Header from '../../components/Header';
// import Tabs from '../../components/Tabs';
// import DateSelector from '../../components/DateSelector';
// import Image from '../../components/Image';
// import Button from '../../components/Button';
// import Link from '../../components/Link';
// import LinkButton from '../../components/LinkButton';
// import Textarea from '../../components/Textarea';
// import Radio from '../../components/Radio';
// import ReCaptcha from '../../components/ReCaptcha';
// import Select from '../../components/Select';
// import Autocomplete from '../../components/Autocomplete';
// import CheckboxField from '../../components/Checkbox';
// import Alert from '../../components/Alert';
// import CheckboxList from '../../components/CheckboxList';
// import HeroContent from '../../components/HeroContent';
// import Pricing from '../../components/Pricing';
// import ImageWithSlide from '../../components/ImageWithSlide';
// import UsefulLinks from '../../components/UsefulLinks';
// import Disclaimer from '../../components/Disclaimer';
// import FeaturedContent from '../../components/FeaturedContent';
// import ContactUsInfo from '../../components/ContactUsInfo';
// import GetInTouch from '../../components/GetInTouch';
// import CodeHighlight from '../../components/CodeHighlight';
// import Team from '../../components/Team';
// import ImageBlockWithText from '../../components/ImageBlockWithText';
// import ImageWithHeaderBody from '../../components/ImageWithHeaderBody';
// import HTML from '../../components/HTML';
const compList = {
  Form: {
    Component: 'Form',
    group: 'Form',
    hasPlaceholder: true,
    type: ItemTypes.FORM,
    defaultComp: 'Input',
    itemsPropName: 'fields',
    accept: [ItemTypes.FORM_ELEMENT, ItemTypes.FORM],
    compTypeProp: 'cmpType',
    hasItems: true,
    editData: FormEdit,
    sampleData: {}
  },
  Address: {
    Component: 'Form',
    name: 'Form',
    group: 'Pre-defined Elements',
    hasPlaceholder: true,
    type: ItemTypes.FORM,
    itemsPropName: 'fields',
    accept: [ItemTypes.FORM_ELEMENT, ItemTypes.FORM],
    compTypeProp: 'cmpType',
    hasItems: false,
    editData: FormEdit,
    sampleData: {
      className: 'sq-form--2-cols',
      fields: [{
        cmpType: 'Input',
        label: 'Address Line 1',
        name: 'addressLine1'
      }, {
        cmpType: 'Input',
        label: 'Address Line 2',
        name: 'addressLine2'
      }, {
        cmpType: 'Input',
        label: 'City',
        name: 'city'
      }, {
        cmpType: 'Input',
        label: 'State',
        name: 'state'
      }, {
        cmpType: 'Input',
        label: 'Postal Code',
        name: 'postalCode'
      }]
    }
  },
  Name: {
    Component: 'Form',
    name: 'Form',
    group: 'Pre-defined Elements',
    hasPlaceholder: true,
    type: ItemTypes.FORM,
    itemsPropName: 'fields',
    accept: [ItemTypes.FORM_ELEMENT, ItemTypes.FORM],
    compTypeProp: 'cmpType',
    hasItems: false,
    editData: FormEdit,
    sampleData: {
      className: 'sq-form--3-cols',
      fields: [{
        cmpType: 'Select',
        label: 'Salutation',
        name: 'salutation',
        options: GLOBAL_OPTIONS.salutation.toArray()
      }, {
        cmpType: 'Input',
        label: 'First Name',
        name: 'firstName'
      }, {
        cmpType: 'Input',
        label: 'Last Name',
        name: 'lastName'
      }]
    }
  },
  Input: {
    Component: 'Input',
    group: 'Form Elements',
    type: ItemTypes.FORM_ELEMENT,
    editData: InputEdit,
    sampleData: {
      label: 'Label'
    }
  },
  RichText: {
    Component: 'RichText',
    group: 'Form Elements',
    type: ItemTypes.FORM_ELEMENT,
    editData: RichTextEdit,
    sampleData: {
      label: 'Rich Label'
    }
  },
  Text: {
    Component: 'Text',
    group: 'Content',
    type: ItemTypes.COMPONENT,
    editData: TextEdit,
    sampleData: {
      text: 'text goes here'
    }
  },
  LaunchTimer: {
    Component: 'LaunchTimer',
    group: 'Content',
    sampleData: {},
    type: ItemTypes.COMPONENT,
    editData: LaunchTimerEdit
  },
  FormText: {
    Component: 'Text',
    name: 'Text',
    group: 'Form Elements',
    type: ItemTypes.FORM_ELEMENT,
    editData: TextEdit,
    sampleData: {
      text: 'text goes here'
    }
  },
  FormHeader: {
    Component: 'Header',
    name: 'Header',
    group: 'Form Elements',
    type: ItemTypes.FORM_ELEMENT,
    editData: TextEdit,
    sampleData: {
      text: 'text goes here',
      tag: 'h1'
    }
  },
  Tabs: {
    Component: 'Tabs',
    group: 'Form Elements',
    type: ItemTypes.FORM_ELEMENT,
    editData: TabsEdit,
    sampleData: {}
  },
  DateSelector: {
    Component: 'DateSelector',
    group: 'Form Elements',
    type: ItemTypes.FORM_ELEMENT,
    editData: DateSelectorEdit,
    sampleData: {
      inputFormat: 'yyyy/MM/dd',
      outputFormat: 'YYYY/MM/DD',
      mask: '____/__/__'
    }
  },
  Progress: {
    Component: 'Progress',
    group: 'Content',
    type: ItemTypes.COMPONENT,
    editData: ProgressEdit,
    sampleData: {
      className: 'tp-progress--active',
      overlay: true
    }
  },
  Header: {
    Component: 'Header',
    group: 'Content',
    sampleData: {
      header: 'Header',
      subHeader: 'Sub header'
    },
    editData: HeaderEdit
  },
  Image: {
    Component: 'Image',
    group: 'Content',
    type: ItemTypes.COMPONENT,
    editData: ImageEdit,
    sampleData: {}
  },
  Button: {
    Component: 'Button',
    group: 'Actions',
    type: ItemTypes.COMPONENT,
    editData: ButtonEdit,
    sampleData: {
      buttonText: 'Action'
    }
  },
  Link: {
    Component: 'Link',
    group: 'Actions',
    type: ItemTypes.COMPONENT,
    editData: ButtonEdit,
    sampleData: {
      buttonText: 'Action'
    }
  },
  LinkButton: {
    Component: 'LinkButton',
    group: 'Actions',
    type: ItemTypes.COMPONENT,
    editData: ButtonEdit,
    sampleData: {
      buttonText: 'Action'
    }
  },
  Textarea: {
    Component: 'Textarea',
    group: 'Form Elements',
    type: ItemTypes.FORM_ELEMENT,
    editData: TextareaEdit,
    sampleData: {
      label: 'Label',
      minRows: 3
    }
  },
  Radio: {
    Component: 'Radio',
    group: 'Form Elements',
    type: ItemTypes.FORM_ELEMENT,
    editData: RadioEdit,
    sampleData: {
      label: 'Radio Label'
    }
  },
  ReCaptcha: {
    Component: 'ReCaptcha',
    group: 'Form Elements',
    type: ItemTypes.FORM_ELEMENT,
    editData: ReCaptchaEdit,
    sampleData: {}
  },
  Select: {
    Component: 'Select',
    group: 'Form Elements',
    type: ItemTypes.FORM_ELEMENT,
    editData: SelectEdit,
    sampleData: {
      label: 'Select'
    }
  },
  Autocomplete: {
    Component: 'Autocomplete',
    group: 'Form Elements',
    type: ItemTypes.FORM_ELEMENT,
    editData: AutocompleteEdit,
    sampleData: {
      label: 'Autocomplete'
    }
  },
  CheckboxField: {
    Component: 'CheckboxField',
    group: 'Form Elements',
    type: ItemTypes.FORM_ELEMENT,
    editData: CheckboxEdit,
    sampleData: {
      label: 'Label'
    }
  },
  AlertElement: {
    Component: 'Alert',
    name: 'Alert',
    group: 'Form Elements',
    type: ItemTypes.FORM_ELEMENT,
    editData: AlertEdit,
    sampleData: {
      message: 'Sample message'
    }
  },
  Alert: {
    Component: 'Alert',
    group: 'Content',
    type: ItemTypes.COMPONENT,
    editData: AlertEdit,
    sampleData: {
      message: 'Sample message'
    }
  },
  CheckboxList: {
    Component: 'CheckboxList',
    group: 'Form Elements',
    type: ItemTypes.FORM_ELEMENT,
    editData: CheckboxListEdit,
    sampleData: {
      label: 'Label'
    }
  },
  Wrapper: {
    Component: 'Wrapper',
    group: 'Layout',
    accept: [ItemTypes.COMPONENT, ItemTypes.FORM],
    hasPlaceholder: true,
    compTypeProp: 'component',
    editData: WrapperEdit,
    itemsPropName: 'items',
    hasItems: true,
    sampleData: {
      bodyClassName: 'row'
    }
  },
  'Center-Layout': {
    Component: 'Wrapper',
    name: 'Wrapper',
    group: 'Layout',
    accept: [ItemTypes.COMPONENT, ItemTypes.FORM],
    hasPlaceholder: true,
    compTypeProp: 'component',
    editData: WrapperEdit,
    itemsPropName: 'items',
    hasItems: true,
    sampleData: {
      bodyClassName: 'text-center',
      bodyContainerClassName: ''
    }
  },
  '2-ColumnLayout': {
    Component: 'Wrapper',
    name: 'Wrapper',
    group: 'Layout',
    accept: [ItemTypes.COMPONENT, ItemTypes.FORM],
    hasPlaceholder: true,
    compTypeProp: 'component',
    editData: WrapperEdit,
    itemsPropName: 'items',
    hasItems: true,
    sampleData: {
      className: '',
      bodyClassName: 'row',
      items: [{
        component: 'Wrapper',
        name: 'wrapper1',
        className: 'col-xs-12 col-md-6',
        bodyContainerClassName: '',
        bodyClassName: ''
      }, {
        component: 'Wrapper',
        bodyContainerClassName: '',
        name: 'wrapper2',
        bodyClassName: '',
        className: 'col-xs-12 col-md-6'
      }]
    }
  },
  '3-ColumnLayout': {
    Component: 'Wrapper',
    name: 'Wrapper',
    group: 'Layout',
    accept: [ItemTypes.COMPONENT, ItemTypes.FORM],
    hasPlaceholder: true,
    compTypeProp: 'component',
    editData: WrapperEdit,
    itemsPropName: 'items',
    hasItems: true,
    sampleData: {
      className: '',
      bodyClassName: 'row',
      items: [{
        component: 'Wrapper',
        name: 'wrapper1',
        className: 'col-xs-12 col-md-4',
        bodyContainerClassName: '',
        bodyClassName: ''
      }, {
        component: 'Wrapper',
        bodyContainerClassName: '',
        name: 'wrapper2',
        bodyClassName: '',
        className: 'col-xs-12 col-md-4'
      }, {
        component: 'Wrapper',
        bodyContainerClassName: '',
        name: 'wrapper3',
        bodyClassName: '',
        className: 'col-xs-12 col-md-4'
      }]
    }
  },
  Custom: {
    group: 'Content',
    type: ItemTypes.FORM,
    editData: CustomEdit,
    sampleData: {
      component: 'Custom'
    }
  },
  CustomElement: {
    group: 'Form Elements',
    type: ItemTypes.FORM_ELEMENT,
    editData: CustomEdit,
    sampleData: {
      cmpType: 'CustomElement'
    }
  },
  HeroContent: {
    group: 'Content',
    Component: 'HeroContent',
    sampleData: {
      header: 'Awesome header content',
      subHeader: 'Reprehenderit laborum in ut enim Lorem minim ea ex dolore eiusmod id consequat pariatur amet. Id Lorem id et elit.'
    },
    editData: HeroContentEdit
  },
  Pricing: {
    group: 'Content',
    Component: 'Pricing',
    sampleData: {
      header: 'Pricing Plan',
      subHeader: 'Choose your own plan',
      features: [{
        icon: 'person',
        text: 'Users'
      }, {
        icon: 'Computer',
        text: 'Websites '
      }, {
        icon: 'CheckCircleOutline',
        text: 'Dynamic Editing'
      }, {
        icon: 'Cloud',
        text: 'Taken changes'
      }, {
        icon: 'Donwload',
        text: 'Future updates'
      }, {
        icon: 'CheckCircleOutline',
        text: 'Parking'
      }],
      items: [{
        header: 'Basic',
        price: 'USD $25',
        icon: 'calendar',
        lastPrice: 'USD $49',
        color: 'primary',
        features: [{
          icon: 'CheckCircleOutline',
          text: 'Ut pariatur'
        }, {
          icon: 'CheckCircleOutline',
          text: 'aliqua commodo elit mollit ullamco. '
        }, {
          icon: 'CheckCircleOutline',
          text: 'Eiusmod ut dolor reprehenderit reprehenderit'
        }, {
          icon: 'CheckCircleOutline',
          text: ' tempor mollit et deserunt labore. Occaecat '
        }, {
          icon: 'CheckCircleOutline',
          text: 'elit ea consectetur minim do officia'
        }, {
          icon: 'CheckCircleOutline',
          text: 'qui cillum minim veniam ea nostrud irure sit'
        }],
        actions: [{
          buttonText: 'Contact us',
          type: 'Button',
          variant: 'outlined'
        }]
      }, {
        header: 'Professional',
        price: 'USD $50',
        icon: 'calendar',
        color: 'success',
        features: [{
          icon: 'CheckCircleOutline',
          text: 'Ut pariatur'
        }, {
          icon: 'CheckCircleOutline',
          text: 'aliqua commodo elit mollit ullamco. '
        }, {
          icon: 'CheckCircleOutline',
          text: 'Eiusmod ut dolor reprehenderit reprehenderit'
        }, {
          icon: 'CheckCircleOutline',
          text: ' tempor mollit et deserunt labore. Occaecat '
        }, {
          icon: 'CheckCircleOutline',
          text: 'elit ea consectetur minim do officia'
        }, {
          icon: 'CheckCircleOutline',
          text: 'qui cillum minim veniam ea nostrud irure sit'
        }],
        actions: [{
          buttonText: 'Contact us',
          type: 'Button',
          variant: 'outlined'
        }]
      }, {
        header: 'Enterprise',
        price: 'USD $599',
        color: 'info',
        icon: 'calendar',
        features: [{
          icon: 'CheckCircleOutline',
          text: 'Ut pariatur'
        }, {
          icon: 'CheckCircleOutline',
          text: 'aliqua commodo elit mollit ullamco. '
        }, {
          icon: 'CheckCircleOutline',
          text: 'Eiusmod ut dolor reprehenderit reprehenderit'
        }, {
          icon: 'CheckCircleOutline',
          text: ' tempor mollit et deserunt labore. Occaecat '
        }, {
          icon: 'CheckCircleOutline',
          text: 'elit ea consectetur minim do officia'
        }, {
          icon: 'CheckCircleOutline',
          text: 'qui cillum minim veniam ea nostrud irure sit'
        }],
        actions: [{
          buttonText: 'Contact us',
          type: 'Button',
          variant: 'outlined'
        }]
      }]
    },
    editData: PricingEdit
  },
  ImageWithSlide: {
    group: 'Content',
    Component: 'ImageWithSlide',
    sampleData: {
      eyebrow: 'eybrow',
      header: 'Header',
      image: {
        src: 'https://st.depositphotos.com/2309453/3447/i/600/depositphotos_34479387-stock-photo-group-of-young-business-people.jpg'
      }
    },
    editData: ImageWithSlideEdit
  },
  UsefulLinks: {
    group: 'Content',
    Component: 'UsefulLinks',
    sampleData: {
      header: 'Header goes here',
      links: [{
        iconName: 'star',
        text: 'Learn more about',
        to: '/content/new'
      }]
    },
    editData: UsefulLinksEdit
  },
  Disclaimer: {
    group: 'Content',
    Component: 'Disclaimer',
    sampleData: {
      text: 'Reprehenderit minim non id dolor fugiat amet. Esse ea aliquip eu ex duis id et reprehenderit sint non elit cillum fugiat. Quis cupidatat adipisicing magna tempor occaecat minim. Culpa duis deserunt laboris ad voluptate laboris est laboris duis nostrud. Mollit incididunt esse Lorem pariatur dolor nulla ea aute sit sunt occaecat.'
    },
    editData: DisclaimerEdit
  },
  FeaturedContent: {
    group: 'Content',
    Component: 'FeaturedContent',
    sampleData: {
      className: 'text-center',
      items: [{
        icon: 'call',
        title: 'ALL BRANDS',
        description: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ultrices dui sapien eget mi proin sed. Eget nulla facilisi etiam dignissim diam. Eu nisl nunc mi ipsum faucibus. Risus sed vulputate odio ut. Orci a scelerisque purus semper eget duis. Mattis ullamcorper velit sed ullamcorper morbi tincidunt. Ut faucibus pulvinar'
      }, {
        icon: 'message',
        title: 'FREE SUPPORT',
        description: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ultrices dui sapien eget mi proin sed. Eget nulla facilisi etiam dignissim diam. Eu nisl nunc mi ipsum faucibus. Risus sed vulputate odio ut. Orci a scelerisque purus semper eget duis. Mattis ullamcorper velit sed ullamcorper morbi tincidunt. Ut faucibus pulvinar'
      }, {
        icon: 'creditcard',
        title: 'AFFORDABLE',
        description: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ultrices dui sapien eget mi proin sed. Eget nulla facilisi etiam dignissim diam. Eu nisl nunc mi ipsum faucibus. Risus sed vulputate odio ut. Orci a scelerisque purus semper eget duis. Mattis ullamcorper velit sed ullamcorper morbi tincidunt. Ut faucibus pulvinar'
      }]
    },
    editData: FeaturedContentEdit
  },
  TextColumnWithLinks: {
    group: 'Content',
    Component: 'TextColumnWithLinks',
    sampleData: {
      items: [{
        iconName: 'call',
        header: 'Header 1',
        subHeader: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ultrices dui sapien eget mi proin sed. Eget nulla facilisi etiam dignissim diam. Eu nisl nunc mi ipsum faucibus. Risus sed vulputate odio ut. Orci a scelerisque purus semper eget duis. Mattis ullamcorper velit sed ullamcorper morbi tincidunt. Ut faucibus pulvinar'
      }, {
        iconName: 'location',
        header: 'Header 2',
        subHeader: 'Sub header 2'
      }, {
        iconName: 'home',
        header: 'Header 3',
        subHeader: 'Sub header 3'
      }]
    },
    editData: TextColumnWithLinksEdit
  },
  ContactUsInfo: {
    group: 'Content',
    Component: 'ContactUsInfo',
    sampleData: {
      items: [{
        iconName: 'call',
        iconColor: 'error',
        header: 'Call us',
        subHeader: '+91 21292 19222',
        iconSize: 'xxl'
      }, {
        iconName: 'email-outline',
        iconColor: 'error',
        iconSize: 'xxl',
        header: 'Mail us',
        subHeader: 'info@example.com'
      }, {
        iconName: 'share',
        iconColor: 'info',
        iconSize: 'xxl',
        header: 'Connect with us',
        subHeader: '',
        links: [{
          iconName: 'facebook',
          to: 'http://example.com',
          iconColor: 'info',
          target: '_blank'
        }, {
          iconName: 'instagram',
          to: 'http://insta.com',
          iconColor: 'red',
          target: '_blank'
        }]
      }]
    },
    editData: ContactUsInfoEdit
  },
  GetInTouch: {
    group: 'Content',
    Component: 'GetInTouch',
    sampleData: {
      header: 'Get in touch',
      info: [{
        iconName: 'message',
        text: 'email@example.com',
        href: 'mailto:email@example.com'
      }, {
        iconName: 'call',
        href: 'call:101111111',
        text: '101.111.111'
      }, {
        iconName: 'Computer',
        text: 'www.domain.com',
        href: 'www.domain.com',
        target: '_blank'
      }],
      footerText: 'Laborum ex duis ut labore eiusmod amet fugiat. Qui ullamco exercitation eu sint laborum. Magna aliquip adipisicing laborum anim anim id dolore ea tempor. Do cupidatat velit exercitation cupidatat magna anim. Nisi nulla est ad duis consectetur officia incididunt nostrud enim veniam incididunt veniam labore eu. Non minim deserunt laborum ullamco esse fugiat mollit deserunt consectetur velit dolore exercitation pariatur. Ex elit sunt occaecat cupidatat cillum elit aliqua veniam cupidatat nulla adipisicing.'
    },
    editData: GetInTouchEdit
  },
  CodeHighlight: {
    group: 'Content',
    Component: 'CodeHighlight',
    sampleData: {
      code: 'funciton(a, b) {\n return a + b; \n }',
      language: 'javscript'
    },
    editData: CodeHighlightEdit
  },
  Team: {
    group: 'Content',
    Component: 'Team',
    sampleData: {
      header: 'Meet the team',
      items: [{
        profilePic: 'https://images.pexels.com/photos/220453/pexels-photo-220453.jpeg?auto=compress&cs=tinysrgb&w=1260&h=750&dpr=2',
        header: 'John Peter',
        designation: 'Software Engineer',
        subHeader: 'Officia nulla elit do eu pariatur in esse amet. Ex est fugiat pariatur veniam laboris occaecat ad reprehenderit consequat nisi tempor do do ea. Proident non nostrud elit irure incididunt ea in eu qui incididunt pariatur reprehenderit eiusmod. Adipisicing elit voluptate adipisicing voluptate aliqua nisi sit.'
      }, {
        profilePic: 'https://images.pexels.com/photos/220453/pexels-photo-220453.jpeg?auto=compress&cs=tinysrgb&w=1260&h=750&dpr=2',
        header: 'John Peter',
        designation: 'Software Engineer',
        subHeader: 'Officia nulla elit do eu pariatur in esse amet. Ex est fugiat pariatur veniam laboris occaecat ad reprehenderit consequat nisi tempor do do ea. Proident non nostrud elit irure incididunt ea in eu qui incididunt pariatur reprehenderit eiusmod. Adipisicing elit voluptate adipisicing voluptate aliqua nisi sit.'
      }]
    },
    editData: TeamEdit
  },
  ImageBlockWithText: {
    group: 'Content',
    Component: 'ImageBlockWithText',
    sampleData: {
      header: 'Meet the team',
      subHeader: 'Ipsum eu consequat nostrud nisi ut commodo anim labore labore exercitation dolore qui amet consequat. Anim amet laborum occaecat est aute occaecat. Velit anim adipisicing nostrud dolore quis. Laborum labore eu ad nisi est duis Lorem nulla aliquip tempor nisi.',
      imageUrl: 'https://st.depositphotos.com/2309453/3447/i/600/depositphotos_34479387-stock-photo-group-of-young-business-people.jpg'
    },
    editData: ImageBlockWithTextEdit
  },
  ImageWithHeaderBody: {
    group: 'Content',
    Component: 'ImageWithHeaderBody',
    sampleData: {
      header: 'Mission',
      styleName: 'shadow',
      subHeader: 'Ipsum eu consequat nostrud nisi ut commodo anim labore labore exercitation dolore qui amet consequat. Anim amet laborum occaecat est aute occaecat. Velit anim adipisicing nostrud dolore quis. Laborum labore eu ad nisi est duis Lorem nulla aliquip tempor nisi.',
      imageUrl: 'https://w7.pngwing.com/pngs/691/585/png-transparent-computer-icons-business-mission-company-text-people-thumbnail.png'
    },
    editData: ImageWithHeaderBodyEdit
  },
  Iframe: {
    group: 'Content',
    Component: 'Iframe',
    sampleData: {
      url: 'https://www.google.com'
    },
    editData: IframeEdit
  },
  ImageWithSpotlight: {
    group: 'Content',
    Component: 'ImageWithSpotlight',
    sampleData: {
      header: 'Grow with us',
      bgImageUrl: 'https://img.freepik.com/free-photo/new-york-city-central-park-panorama_649448-89.jpg?w=2000&t=st=1683292030~exp=1683292630~hmac=b87b9d4a48fea6a2b3ed84f236b7f9213fc24948ed32023d554ff3e9c31d4315',
      actions: [{
        buttonText: 'Blogs',
        iconName: 'list',
        color: 'success'
      }, {
        buttonText: 'Contact us',
        color: 'warning',
        iconName: 'call'
      }, {
        buttonText: 'Support',
        iconName: 'person',
        color: 'error'
      }]
    },
    editData: ImageWithSpotlightEdit
  },
  HTML: {
    group: 'Content',
    Component: 'HTML',
    sampleData: {
      html: '<p>Here pu<span style="background-color: rgb(236, 202, 250);">t your ht</span>ml</p>'
    },
    editData: HTMLEdit
  }
};
dynamicComponents.set(compList);

pageBuilder.setHelpers({
  withEditTabsConfig
});
pageBuilder.set({
  templates: [{
    text: 'apps/core/templates/page',
    value: 'apps/core/templates/page'
  }],
  layouts: [{
    text: 'apps/core/layouts/spa',
    value: 'apps/core/layouts/spa'
  }],
  containers: [{
    text: 'Default',
    value: 'Default'
  }],
  containerTemplates: [{
    text: 'Default',
    value: 'Default'
  }],
  themes: [{
    text: 'Main',
    value: 'main'
  }]
});
class PageBuilder extends Component$1 {
  constructor(props) {
    super(props);
    this.state = {
      enableMenu: true,
      advancedPageConfig: false,
      autoSave: !!query$1.get().path,
      enableProps: false,
      pageFetched: false,
      contentData: {
        pageData: {
          updatePageTitle: false,
          items: []
        }
      }
    };
    this.componentOnDrop = this.componentOnDrop.bind(this);
    this.toggleProps = this.toggleProps.bind(this);
    this.toggleElements = this.toggleElements.bind(this);
    this.toggleQuickPreview = this.toggleQuickPreview.bind(this);
    this.toggleAutoSave = this.toggleAutoSave.bind(this);
    this.onContentChange = this.onContentChange.bind(this);
    this.onContentDelete = this.onContentDelete.bind(this);
    this.savePageAsDraft = this.savePageAsDraft.bind(this);
    this.toggleAdvPageConfig = this.toggleAdvPageConfig.bind(this);
    this.formOnChange = this.formOnChange.bind(this);
    this.onMoveItemDown = this.onMoveItemDown.bind(this);
    this.onMoveItemUp = this.onMoveItemUp.bind(this);
    this.savePageAsNew = this.savePageAsNew.bind(this);
    this.advFormOnChange = this.advFormOnChange.bind(this);
  }
  componentDidUpdate() {
    return __awaiter$1(this, void 0, void 0, function* () {
      if (!this.state.pageFetched && query$1.get().path) {
        this.checkPageLoad();
      }
    });
  }
  checkPageLoad() {
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        pageData,
        store
      } = this.props;
      this.props.commonActions.startLoading();
      if (query$1.get().path) {
        yield this.props.raiseAction(getPage({
          path: query$1.get().path
        }, pageData.getPageConfig));
        this.setState({
          pageFetched: true,
          autoSave: true
        });
      } else {
        yield this.props.raiseAction(createPage());
      }
      pageData.fieldsMetaConfig && (yield this.props.raiseAction(getFieldsMeta({}, pageData.fieldsMetaConfig)));
      this.setState({
        contentData: this.props.store.admin.contentData
      });
      this.props.commonActions.stopLoading();
    });
  }
  componentDidMount() {
    return __awaiter$1(this, void 0, void 0, function* () {
      this.checkPageLoad();
    });
  }
  savePageAsNew() {
    var _a;
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        pageData
      } = this.props;
      pageData.createPopupScreen && this.props.commonActions.showPopupScreen(Object.assign(Object.assign({}, pageData.createPopupScreen), {
        initialData: {
          current: Object.assign(Object.assign({}, this.state.contentData), (_a = pageData.createPopupScreen) === null || _a === void 0 ? void 0 : _a.initialData)
        }
      }));
    });
  }
  savePageAsDraft(autoSave) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        pageData
      } = this.props;
      !autoSave && this.props.commonActions.startLoading();
      yield this.props.raiseAction(savePageDraft(this.state.contentData, Object.assign({
        autoSave: autoSave === true
      }, pageData.savePageConfig)));
      !autoSave && this.props.commonActions.stopLoading();
    });
  }
  componentOnDrop(item, e, idx) {
    var _a;
    const newItems = [...(this.state.contentData.pageData.items || []), Object.assign({
      component: item.name,
      name: item.name.toLowerCase() + (((_a = this.state.contentData.pageData.items) === null || _a === void 0 ? void 0 : _a.length) || 1)
    }, item.metaData.sampleData)];
    const dragIndex = newItems.length - 1;
    this.setState({
      contentData: Object.assign(Object.assign({}, this.state.contentData), {
        pageData: Object.assign(Object.assign({}, this.state.contentData.pageData), {
          items: idx !== undefined ? update(newItems, {
            $splice: [[dragIndex, 1], [idx, 0, newItems[dragIndex]]]
          }) : newItems
        })
      })
    });
    this.checkAutoSave();
  }
  toggleProps() {
    this.setState({
      enableProps: !this.state.enableProps
    });
  }
  toggleElements() {
    this.setState({
      enableMenu: !this.state.enableMenu
    });
  }
  advFormOnChange(data) {
    this.setState({
      contentData: Object.assign(Object.assign({}, this.state.contentData), data.value)
    });
    this.checkAutoSave();
  }
  formOnChange(data) {
    const finalData = {
      pageData: Object.assign(Object.assign({}, this.state.contentData.pageData), data.value)
    };
    this.setState({
      contentData: Object.assign(Object.assign({}, this.state.contentData), {
        pageData: finalData.pageData
      })
    });
    this.checkAutoSave();
  }
  onContentChange(data, idx) {
    const finalData = {
      pageData: Object.assign(Object.assign({}, this.state.contentData.pageData), {
        items: [...this.state.contentData.pageData.items]
      })
    };
    finalData.pageData.items[idx] = Object.assign(Object.assign({}, finalData.pageData.items[idx]), data);
    this.setState({
      contentData: Object.assign(Object.assign({}, this.state.contentData), {
        pageData: finalData.pageData
      })
    });
    this.checkAutoSave();
  }
  onContentDelete(idx) {
    const finalData = {
      pageData: Object.assign(Object.assign({}, this.state.contentData.pageData), {
        items: [...this.state.contentData.pageData.items]
      })
    };
    finalData.pageData.items.splice(idx, 1);
    this.setState({
      contentData: Object.assign(Object.assign({}, this.state.contentData), {
        pageData: finalData.pageData
      })
    });
    this.checkAutoSave();
  }
  toggleQuickPreview() {
    this.setState({
      preview: !this.state.preview
    });
  }
  toggleAutoSave() {
    this.setState({
      autoSave: !this.state.autoSave
    });
  }
  toggleAdvPageConfig() {
    this.setState({
      advancedPageConfig: !this.state.advancedPageConfig
    });
  }
  showPreview() {
    redirectTo(query$1.get().path, {}, {
      target: '_blank'
    });
  }
  onMoveItemDown(index) {
    const finalData = {
      pageData: Object.assign(Object.assign({}, this.state.contentData.pageData), {
        items: [...this.state.contentData.pageData.items]
      })
    };
    finalData.pageData.items = update(finalData.pageData.items, {
      $splice: [[index, 1], [index + 1, 0, finalData.pageData.items[index]]]
    });
    this.setState({
      contentData: Object.assign(Object.assign({}, this.state.contentData), {
        pageData: finalData.pageData
      })
    });
    this.checkAutoSave();
  }
  checkAutoSave() {
    setTimeout(() => {
      this.state.autoSave && this.savePageAsDraft(true);
    }, 200);
  }
  onMoveItemUp(index) {
    const finalData = {
      pageData: Object.assign(Object.assign({}, this.state.contentData.pageData), {
        items: [...this.state.contentData.pageData.items]
      })
    };
    finalData.pageData.items = update(finalData.pageData.items, {
      $splice: [[index, 1], [index - 1, 0, finalData.pageData.items[index]]]
    });
    this.setState({
      contentData: Object.assign(Object.assign({}, this.state.contentData), {
        pageData: finalData.pageData
      })
    });
    this.checkAutoSave();
  }
  getLiveFields(items = []) {
    let arryData = [];
    items.forEach(item => {
      var _a;
      if (item.component === 'Form') {
        arryData.push({
          text: `.${item.name}`,
          value: `.${item.name}`
        });
        (_a = item.fields) === null || _a === void 0 ? void 0 : _a.forEach(field => {
          arryData.push({
            text: `.${item.name}.${field.name}`,
            value: `.${item.name}.${field.name}`
          });
        });
      }
      if (item.items) {
        arryData = arryData.concat(this.getLiveFields(item.items));
      }
    });
    return arryData;
  }
  getDrivedProps() {
    var _a, _b;
    return {
      liveFields: this.getLiveFields((_b = (_a = this.state.contentData) === null || _a === void 0 ? void 0 : _a.pageData) === null || _b === void 0 ? void 0 : _b.items)
    };
  }
  render() {
    const {
      className = '',
      pageData,
      store
    } = this.props;
    const compList = Object.assign({}, dynamicComponents.get());
    const {
      Form,
      IconButton,
      Button,
      Switch
    } = components$1.get();
    return jsx("div", {
      className: `sq-page-builder sq-v-screen sq-v-screen--fixed ${className}`,
      children: jsxs("div", {
        className: "sq-v-screen__container",
        children: [jsxs("div", {
          className: "sq-page-builder__top-actions mb-wide",
          children: [query$1.get().path && jsx(Switch, {
            label: "Autosave",
            value: this.state.autoSave,
            onChange: this.toggleAutoSave
          }), jsx(Switch, {
            label: "Quick Preview",
            value: this.state.preview,
            onChange: this.toggleQuickPreview
          }), !query$1.get().path && jsx(Button, {
            iconName: 'Save',
            variant: "outlined",
            disabled: !pageData.createPopupScreen,
            buttonText: "Save",
            onClick: () => this.savePageAsNew()
          }), query$1.get().path && jsx(Button, {
            iconName: 'Save',
            variant: "outlined",
            buttonText: "Save",
            onClick: () => this.savePageAsDraft()
          }), jsx(Button, {
            iconName: 'Preview',
            variant: "outlined",
            disabled: !query$1.get().path,
            buttonText: "Full Preview",
            onClick: this.showPreview
          })]
        }), jsx("div", {
          className: "sq-page-builder__content sq-v-screen-grow",
          children: jsxs(DndProvider, {
            backend: HTML5Backend,
            children: [jsxs("div", {
              className: "sq-page-builder__l-options",
              children: [jsx(IconButton, {
                title: "Elements",
                iconName: "code",
                variant: !this.state.enableMenu ? 'default' : 'primary',
                onClick: this.toggleElements
              }), jsx(IconButton, {
                title: "Page Properties",
                iconName: "default",
                variant: !this.state.enableProps ? 'default' : 'primary',
                onClick: this.toggleProps
              })]
            }), jsxs("div", {
              className: "sq-page-builder__container",
              children: [jsx("div", {
                className: "sq-page-builder__left",
                children: this.state.enableMenu && jsx(Panel, {
                  header: "Elements",
                  theme: "dark",
                  onClose: this.toggleElements,
                  children: jsx("div", {
                    className: "row",
                    children: jsx(ComponentList, {
                      compList: compList,
                      filter: pageData.compListFilter
                    })
                  })
                })
              }), jsxs("div", {
                className: "sq-page-builder__center",
                children: [this.state.preview && jsx(ErrorBoundry$1, {
                  children: jsx(DynamicContentRoot, {
                    mode: "preview",
                    pageConfig: this.state.contentData
                  })
                }), !this.state.preview && jsx(Fragment, {
                  children: jsx(ContentEditor, {
                    fieldsMeta: Object.assign(Object.assign({}, this.getDrivedProps()), store.admin.fieldsMeta),
                    pageData: this.state.contentData.pageData,
                    compList: compList,
                    onDelete: this.onContentDelete,
                    onChange: this.onContentChange,
                    onMoveItemUp: this.onMoveItemUp,
                    onMoveItemDown: this.onMoveItemDown,
                    onDrop: this.componentOnDrop
                  })
                })]
              }), jsx("div", {
                className: "sq-page-builder__right",
                children: this.state.enableProps && jsxs(Panel, {
                  header: "Page Properties",
                  enableFullScreen: true,
                  onClose: this.toggleProps,
                  children: [jsx("div", {
                    className: "d-flex j-content-fl-end",
                    children: jsx(Switch, {
                      label: "Advanced",
                      value: this.state.advancedPageConfig,
                      onChange: this.toggleAdvPageConfig
                    })
                  }), this.state.advancedPageConfig && jsx(Form, {
                    fields: [{
                      name: 'pageData',
                      cmpType: 'FormObject',
                      label: 'Page Data'
                    }],
                    onChange: this.advFormOnChange,
                    value: this.state.contentData
                  }), !this.state.advancedPageConfig && jsx(Form, {
                    value: this.state.contentData.pageData,
                    onChange: this.formOnChange,
                    fields: [{
                      name: 'title',
                      cmpType: 'Input',
                      label: 'Title'
                    }, {
                      name: 'headScript',
                      cmpType: 'Textarea',
                      label: 'Dynamic Script'
                    },
                    // {
                    //   name: 'pageBackground',
                    //   cmpType: 'ColorPicker',
                    //   label: 'Page background',
                    // },
                    {
                      name: 'updatePageTitle',
                      cmpType: 'Switch',
                      label: 'updatePageTitle'
                    }, {
                      name: 'init',
                      cmpType: 'FormObject',
                      label: 'init'
                    }, {
                      name: 'hook',
                      cmpType: 'FormObject',
                      label: 'hook'
                    }, {
                      name: 'merge',
                      cmpType: 'FormObject',
                      label: 'merge'
                    }, {
                      name: 'template',
                      cmpType: 'Autocomplete',
                      label: 'Template',
                      options: pageData.templates || pageBuilder.get().templates
                    }, {
                      name: 'layout',
                      cmpType: 'Autocomplete',
                      label: 'Layout',
                      options: pageData.layouts || pageBuilder.get().layouts
                    }, {
                      name: 'theme',
                      cmpType: 'Autocomplete',
                      label: 'Theme',
                      options: pageData.themes || pageBuilder.get().themes
                    }, {
                      name: 'className',
                      cmpType: 'InputWithOptions',
                      label: 'className',
                      options: GLOBAL_OPTIONS.bodyContainers.toArray()
                    }, {
                      name: 'wrapperClassName',
                      cmpType: 'InputWithOptions',
                      label: 'wrapperClassName',
                      options: GLOBAL_OPTIONS.pageWrapperClasses.toArray()
                    }, {
                      cmpType: 'Autocomplete',
                      name: 'container',
                      label: 'container',
                      options: pageData.containers || pageBuilder.get().containers
                    }, {
                      cmpType: 'Input',
                      name: 'bodyClassName',
                      label: 'bodyClassName'
                    }, {
                      cmpType: 'Autocomplete',
                      name: 'containerTemplate',
                      label: 'containerTemplate',
                      options: pageData.containerTemplates || pageBuilder.get().containerTemplates
                    }, {
                      cmpType: 'FormObject',
                      name: 'custom',
                      label: 'custom'
                    }, ...(pageData.customProps || [])]
                  })]
                })
              })]
            })]
          })
        }), jsx("div", {
          className: "sq-page-builder__footer"
        })]
      })
    });
  }
}
PageBuilder.propTypes = {};
const mapStateToProps$2 = state => {
  return {
    store: Object.assign({}, state)
  };
};
const mapDispatchToProps$2 = dispatch => {
  return {
    contentActions: {
      postApi: data => dispatch(postApi(data)),
      downloadApi: data => dispatch(downloadApi(data)),
      updateMetaData: data => dispatch(updateMetaData(data)),
      executeHook: data => dispatch(executeHook(data)),
      fetchContentPage: data => dispatch(fetchContentPage(data)),
      resetUserData: data => dispatch(resetUserData(data)),
      updateUserData: data => dispatch(updateUserData(data)),
      mergeUserData: data => dispatch(mergeUserData(data)),
      sendContact: data => dispatch(sendContact(data)),
      updateErrorData: data => dispatch(updateErrorData(data))
    },
    commonActions: {
      showNotificationMessage: data => dispatch(showNotificationMessage(data)),
      closeNotification: () => dispatch(closeNotification()),
      startLoading: data => dispatch(startLoading(data)),
      showPopup: data => dispatch(showPopup(data)),
      stopLoading: data => dispatch(stopLoading(data)),
      setError: data => dispatch(setError(data)),
      clearError: data => dispatch(clearError(data)),
      showPopupScreen: data => dispatch(showPopupScreen(data))
    },
    raiseAction: dispatch
  };
};
var index$2 = connect(mapStateToProps$2, mapDispatchToProps$2)(PageBuilder);

const tabs = {
  general: {
    text: 'General',
    value: 'general'
  },
  validations: {
    text: 'Validations',
    value: 'validations'
  },
  actions: {
    text: 'Actions',
    value: 'actions'
  },
  match: {
    text: 'Match',
    value: 'match'
  }
};
const createTabItems = (data, key, extraMatch = {}) => {
  return data && key && data.length !== 0 ? data.map((item = {}) => Object.assign(Object.assign({}, item), {
    match: Object.assign(Object.assign(Object.assign({}, item.match), {
      editTab: {
        validators: [{
          type: 'equals',
          matchValue: key
        }]
      }
    }), extraMatch)
  })) : [];
};
var editData = (({
  themes
} = {}) => ({
  pageData: {
    init: {
      editTab: tabs.general.value,
      advanceParams: 'N'
    },
    params: {
      '...main': '.main'
    },
    items: [{
      component: 'Wrapper',
      bodyClassName: 'd-flex j-content-fl-end',
      items: [{
        component: 'Switch',
        name: 'advanceParams',
        label: 'Advanced',
        selectedValue: 'Y',
        defaultValue: 'N'
      }]
    }, {
      component: 'FormObject',
      name: 'main',
      match: {
        advanceParams: {
          validators: [{
            type: 'equals',
            matchValue: 'Y'
          }]
        }
      }
    }, {
      component: 'Form',
      name: 'main',
      fields: [{
        name: 'editTab',
        cmpType: 'Tabs',
        options: [tabs.general],
        match: {
          advanceParams: {
            validators: [{
              type: 'equals',
              matchValue: 'N'
            }]
          }
        }
      }, ...createTabItems([{
        name: 'analytics',
        label: 'analytics',
        cmpType: 'FormObject'
      }, {
        name: 'theme',
        label: 'theme',
        cmpType: 'Autocomplete',
        options: themes
      }, {
        name: 'appVersion',
        label: 'appVersion',
        cmpType: 'Input'
      }, {
        name: 'launchConfig',
        label: 'launchConfig',
        cmpType: 'FormObject'
      }, {
        name: 'siteMap',
        cmpType: 'Form',
        fields: [{
          name: 'maxNavigationLevel',
          label: 'maxNavigationLevel'
        }, {
          name: 'title',
          label: 'title'
        }, {
          name: 'errorRedirects',
          cmpType: 'FormObject',
          label: 'errorRedirects'
        }, {
          name: 'defaultRedirect',
          cmpType: 'FormObject',
          label: 'defaultRedirect'
        }, {
          name: 'globalNavigation',
          cmpType: 'FormObject',
          label: 'globalNavigation'
        }, {
          name: 'globalNavigationLoggedIn',
          cmpType: 'FormObject',
          label: 'globalNavigationLoggedIn'
        }, {
          name: 'customNavigationLoggedIn',
          cmpType: 'FormObject',
          label: 'customNavigationLoggedIn'
        }, {
          name: 'logo',
          cmpType: 'FormObject',
          label: 'logo'
        }, {
          name: 'globalFooter',
          cmpType: 'FormObject',
          label: 'globalFooter'
        }, {
          name: 'children',
          label: 'children',
          cmpType: 'FormObject'
        }, {
          name: 'i18n',
          label: 'i18n',
          cmpType: 'FormObject'
        }]
      }], tabs.general.value, {
        advanceParams: {
          validators: [{
            type: 'equals',
            matchValue: 'N'
          }]
        }
      })]
    }]
  }
}));

class SiteMapBuilder extends Component$1 {
  constructor(props) {
    super(props);
    this.state = {
      enableMenu: true,
      autoSave: false,
      enableProps: false,
      contentData: {}
    };
    this.savePageAsDraft = this.savePageAsDraft.bind(this);
    this.onContentChange = this.onContentChange.bind(this);
    this.toggleAutoSave = this.toggleAutoSave.bind(this);
  }
  toggleAutoSave() {
    this.setState({
      autoSave: !this.state.autoSave
    });
  }
  componentDidMount() {
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        pageData,
        store
      } = this.props;
      this.props.commonActions.startLoading();
      yield this.props.raiseAction(getPage({
        path: query$1.get().path
      }, pageData.getPageConfig));
      this.setState({
        contentData: this.props.store.admin.contentData
      });
      this.props.commonActions.stopLoading();
    });
  }
  savePageAsDraft(data, autoSave) {
    return __awaiter$1(this, void 0, void 0, function* () {
      const parms = {};
      if (data) {
        parms.pageData = Object.assign({}, data);
      }
      const {
        pageData,
        store
      } = this.props;
      !autoSave && this.props.commonActions.startLoading();
      yield this.props.raiseAction(savePageDraft(Object.assign(Object.assign({}, this.state.contentData), parms), Object.assign({
        autoSave
      }, pageData.savePageConfig)));
      !autoSave && this.props.commonActions.stopLoading();
    });
  }
  onContentChange(data) {
    return __awaiter$1(this, void 0, void 0, function* () {
      yield this.setState({
        contentData: Object.assign(Object.assign({}, this.state.contentData), {
          pageData: Object.assign({}, data)
        })
      });
      this.state.autoSave && this.savePageAsDraft(data, this.state.autoSave);
    });
  }
  render() {
    var _a;
    const {
      className = ''
    } = this.props;
    const editfinalData = editData(pageBuilder.get());
    const {
      Button,
      Switch
    } = components$1.get();
    return jsx("div", {
      className: `sq-sitemap-builder sq-v-screen sq-v-screen--fixed ${className}`,
      children: jsx(DndProvider, {
        backend: HTML5Backend,
        children: jsxs("div", {
          className: "sq-v-screen__container",
          children: [jsx("div", {
            className: "sq-sitemap-builder__top-actions text-right mb-wide",
            children: jsxs("div", {
              className: "container-fluid",
              children: [jsx(Switch, {
                label: "Autosave",
                value: this.state.autoSave,
                onChange: this.toggleAutoSave
              }), jsx(Button, {
                iconName: 'Save',
                variant: "outlined",
                buttonText: "Save",
                onClick: () => this.savePageAsDraft()
              })]
            })
          }), jsx("div", {
            className: "sq-sitemap-builder__content sq-v-screen-grow auto",
            children: jsx("div", {
              className: "container-fluid",
              children: ((_a = this.state.contentData) === null || _a === void 0 ? void 0 : _a.pageData) && jsx(DynamicContentRoot, {
                pageConfig: editfinalData,
                rootClass: '',
                initialData: {
                  main: this.state.contentData.pageData
                },
                contentParams: editfinalData.pageData.params,
                onContentChange: this.onContentChange,
                onSubmit: this.savePageAsDraft
              })
            })
          })]
        })
      })
    });
  }
}
SiteMapBuilder.propTypes = {};
const mapStateToProps$1 = state => {
  return {
    store: Object.assign({}, state)
  };
};
const mapDispatchToProps$1 = dispatch => {
  return {
    contentActions: {
      postApi: data => dispatch(postApi(data)),
      downloadApi: data => dispatch(downloadApi(data)),
      updateMetaData: data => dispatch(updateMetaData(data)),
      executeHook: data => dispatch(executeHook(data)),
      fetchContentPage: data => dispatch(fetchContentPage(data)),
      resetUserData: data => dispatch(resetUserData(data)),
      updateUserData: data => dispatch(updateUserData(data)),
      mergeUserData: data => dispatch(mergeUserData(data)),
      sendContact: data => dispatch(sendContact(data)),
      updateErrorData: data => dispatch(updateErrorData(data))
    },
    commonActions: {
      showNotificationMessage: data => dispatch(showNotificationMessage(data)),
      closeNotification: () => dispatch(closeNotification()),
      startLoading: data => dispatch(startLoading(data)),
      showPopup: data => dispatch(showPopup(data)),
      stopLoading: data => dispatch(stopLoading(data)),
      setError: data => dispatch(setError(data)),
      clearError: data => dispatch(clearError(data)),
      showPopupScreen: data => dispatch(showPopupScreen(data))
    },
    raiseAction: dispatch
  };
};
var index$1 = connect(mapStateToProps$1, mapDispatchToProps$1)(SiteMapBuilder);

class TableOfContent extends Component$1 {
  constructor(props) {
    super(props);
    this.state = {};
  }
  extractCatName(name) {
    if (name.indexOf('::') > -1) {
      return name.substr(name.indexOf('::') + 2);
    }
    return name;
  }
  render() {
    const _a = this.props,
      {
        metaData = {},
        pageData = {}
      } = _a;
      __rest(_a, ["metaData", "pageData"]);
    const {
      className = ''
    } = pageData;
    const {
      Header,
      LinkButton
    } = components$1.get();
    const categories = Object.keys(metaData.siblingPages || {});
    return jsx("div", {
      className: `sq-content-toc sq-content-page__body ${className}`,
      children: jsxs("div", {
        className: "container-fluid",
        children: [jsx(Header, {
          header: pageData.title,
          className: "text-center"
        }), jsx("div", {
          className: "sq-content-toc__container",
          children: categories && categories.map((cat, idx) => {
            const category = metaData.siblingPages[cat];
            return jsxs("div", {
              className: "sq-content-toc__item",
              children: [jsxs("h4", {
                className: "sq-content-toc__header ",
                children: [this.extractCatName(cat), ' ']
              }), category.map((data, cIdx) => {
                return jsx("div", {
                  children: jsx(LinkButton, {
                    to: data.path,
                    buttonText: data.title
                  })
                }, cIdx);
              })]
            }, idx);
          })
        })]
      })
    });
  }
}
TableOfContent.propTypes = {
  commonStore: PropTypes.object,
  contentStore: PropTypes.object
};

var _a;
const {
  formatters
} = format;
const {
  translate
} = translate$6;
const defaultPageSize = (_a = GLOBAL_OPTIONS.noOfResultsDropdown.toArray()[0]) === null || _a === void 0 ? void 0 : _a.value;
class Users extends React$2.Component {
  constructor(props) {
    super(props);
    this.state = {
      currentTab: GLOBAL_OPTIONS.userTabs.keys.ACTIVE_USERS,
      showFilters: false,
      currentFilter: {},
      currentSort: {
        sortColumn: 'firstName',
        sortOrder: 'asc',
        showEditColumns: false,
        gridColumnUI: [],
        initialColumns: [],
        selectedColumns: [],
        editColumnData: [{
          name: 'firstName',
          headerText: 'User',
          className: 'col-lg-fixed'
        }, {
          name: 'emailId',
          className: 'col-lg-fixed',
          headerText: 'Email ID'
        }, {
          name: 'roleName',
          className: 'col-lg-fixed',
          headerText: 'Role'
        }, {
          name: 'contactNumber',
          className: 'col-lg-fixed',
          headerText: 'Primary Phone Number'
        }, {
          name: 'loginDate',
          className: 'col-lg-fixed',
          headerText: 'Login Date'
        }, {
          name: 'tickets',
          className: 'col-lg-fixed',
          headerText: translate('Service Requests')
        }]
      }
    };
    this.tabs = GLOBAL_OPTIONS.userTabs.toArray();
    this.handleTabChange = this.handleTabChange.bind(this);
    this.handlePageChange = this.handlePageChange.bind(this);
    this.onFilterChange = this.onFilterChange.bind(this);
    this.onFilterToggle = this.onFilterToggle.bind(this);
    this.onFilterAction = this.onFilterAction.bind(this);
    this.onGridAction = this.onGridAction.bind(this);
    this.users_OnSort = this.users_OnSort.bind(this);
    this.onEditColumnToggle = this.onEditColumnToggle.bind(this);
    this.onColFilterSelection = this.onColFilterSelection.bind(this);
  }
  componentDidMount() {
    var _a, _b;
    return __awaiter$1(this, void 0, void 0, function* () {
      const params = query$1.get();
      const {
        pageData
      } = this.props;
      if (params.status) {
        yield this.setState({
          currentFilter: Object.assign(Object.assign({}, this.state.currentFilter), {
            status: params.status
          })
        });
      }
      yield this.refreshUsers({
        filter: {
          isActive: true
        }
      });
      if ((_a = pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig) === null || _a === void 0 ? void 0 : _a.getRoles) {
        yield this.props.userActions.loadRoles({
          userType: 'INTERNAL'
        }, (_b = pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig) === null || _b === void 0 ? void 0 : _b.getRoles);
      }
    });
  }
  refreshUsers({
    pageNo,
    pageSize,
    sort,
    filter,
    source
  } = {}) {
    var _a, _b, _c, _d;
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        pageData
      } = this.props;
      pageSize = pageSize || defaultPageSize || ((_a = this.props.store.authentication) === null || _a === void 0 ? void 0 : _a.usersPagination.pageSize);
      pageNo = pageNo || ((_c = (_b = this.props.store.authentication) === null || _b === void 0 ? void 0 : _b.usersPagination) === null || _c === void 0 ? void 0 : _c.currentPage) || 1;
      const sortBy = (sort || this.state.currentSort).sortColumn;
      const sortDir = (sort || this.state.currentSort).sortOrder;
      const extra = {};
      if (!source) {
        source = this.state.currentTab;
      }
      if (source === GLOBAL_OPTIONS.userTabs.keys.ALL_USERS) {
        extra.isActive = undefined;
      }
      this.props.commonActions.startLoading();
      yield this.props.userActions.loadUsers({
        body: Object.assign(Object.assign({}, filter || this.state.currentFilter), extra),
        source,
        query: {
          sortBy: sortBy,
          sortDir: sortDir === null || sortDir === void 0 ? void 0 : sortDir.toUpperCase(),
          pageSize: pageSize,
          pageNo: pageNo
        }
      }, (_d = pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig) === null || _d === void 0 ? void 0 : _d.getUsers);
      this.props.commonActions.stopLoading();
    });
  }
  onColFilterSelection(data) {
    this.setState({
      selectedColumns: data.value,
      showFilter: false,
      showEditColumns: !this.state.showEditColumns
    });
  }
  resetEditColumns() {
    var _a;
    return __awaiter$1(this, void 0, void 0, function* () {
      const columnList = {};
      (((_a = this.state) === null || _a === void 0 ? void 0 : _a.editColumnData) || []).forEach(l => {
        columnList[l.name] = 'on';
      });
      yield this.setState({
        currentEditColumn: columnList
      }, () => __awaiter$1(this, void 0, void 0, function* () {
        this.updateColumnUI();
      }));
    });
  }
  updateColumnUI() {
    return __awaiter$1(this, void 0, void 0, function* () {
      const columnsUI = (this.state.editColumnData || []).map(col => {
        if (col.name !== 'projectCode') {
          col.hide = !this.state.currentEditColumn[col.name];
        }
        return col;
      });
      const finalColumns = columnsUI.filter(i => !i.hide).map(i => i.name);
      this.setState({
        showFilter: false,
        showEditColumns: false,
        gridColumnUI: columnsUI,
        selectedColumns: finalColumns
      });
    });
  }
  onEditColumnToggle(data, field) {
    this.setState({
      showFilter: false,
      showEditColumns: !this.state.showEditColumns
    });
  }
  handleTabChange(data) {
    this.setState({
      currentTab: data.value
    });
    const source = data.value;
    if (data.value === 'ACTIVE_USERS') {
      this.refreshUsers({
        filter: Object.assign({
          isActive: true
        }, this.state.currentFilter),
        source: source,
        pageNo: 1
      });
    } else if (data.value === 'INACTIVE_USERS') {
      this.refreshUsers({
        filter: Object.assign({
          isActive: false
        }, this.state.currentFilter),
        source: source,
        pageNo: 1
      });
    } else {
      this.refreshUsers({
        source: source,
        pageNo: 1
      });
    }
  }
  onFilterChange(data) {
    Object.keys(data.value).forEach(key => (data.value[key] == null || data.value[key] == '') && delete data.value[key]);
    this.setState({
      currentFilter: data.value
    });
  }
  handlePageChange(data) {
    return __awaiter$1(this, void 0, void 0, function* () {
      this.props.store.authentication.usersPagination;
      const pageNo = data.value.currentPage;
      if (this.state.currentTab === 'ACTIVE_USERS') {
        yield this.refreshUsers({
          pageNo: pageNo
        });
      } else if (this.state.currentTab === 'INACTIVE_USERS') {
        yield this.refreshUsers({
          pageNo: pageNo
        });
      } else {
        yield this.refreshUsers({
          pageNo: pageNo
        });
      }
    });
  }
  onGridAction(row, value, column) {
    var _a, _b, _c, _d;
    return __awaiter$1(this, void 0, void 0, function* () {
      const {
        currentPage,
        pageSize
      } = this.props.store.authentication.usersPagination;
      const {
        pageData
      } = this.props;
      switch (value.action) {
        case 'edit':
          redirectTo('editUser', {
            userId: row.email || row.emailId /* replace with data.uid */,
            uid: row.uid /* replace with data.uid */
          });

          break;
        case 'activate':
          yield this.props.userActions.reactivateUser(row, (_a = pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig) === null || _a === void 0 ? void 0 : _a.activateUser);
          yield this.refreshUsers({
            filter: Object.assign({
              isActive: false
            }, this.state.currentFilter),
            pageSize: pageSize
          });
          break;
        case 'deactivate':
          yield this.props.userActions.deactivateUser(row, (_b = pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig) === null || _b === void 0 ? void 0 : _b.deActivateUser);
          yield this.refreshUsers({
            filter: Object.assign({
              isActive: true
            }, this.state.currentFilter),
            pageSize: pageSize
          });
          break;
        case 'delete':
          yield this.props.userActions.deleteUser(row, (_c = pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig) === null || _c === void 0 ? void 0 : _c.deleteUser);
          yield this.refreshUsers({
            filter: Object.assign({
              isActive: true
            }, this.state.currentFilter),
            pageSize: pageSize
          });
          break;
        case 'reset-password':
          this.props.userActions.resetPassword(row, (_d = pageData === null || pageData === void 0 ? void 0 : pageData.apiConfig) === null || _d === void 0 ? void 0 : _d.resetPassword);
          break;
      }
    });
  }
  onFilterToggle() {
    this.setState({
      showFilter: !this.state.showFilter
    });
  }
  onFilterAction(data, action) {
    return __awaiter$1(this, void 0, void 0, function* () {
      this.setState({
        showFilter: false
      });
      if (data.value) {
        return;
      }
      const {
        currentPage
      } = this.props.store.authentication.usersPagination;
      switch (action.actionType) {
        case 'resetfilter':
          this.setState({
            currentFilter: {}
          });
          yield this.refreshUsers({
            filter: {},
            currentPage: currentPage
          });
          break;
        case 'applyfilter':
          if (this.state.currentTab === 'ACTIVE_USERS') {
            yield this.refreshUsers({
              filter: Object.assign(Object.assign({}, this.state.currentFilter), {
                isActive: true
              }),
              currentPage: currentPage
            });
          } else if (this.state.currentTab === 'INACTIVE_USERS') {
            yield this.refreshUsers({
              filter: Object.assign(Object.assign({}, this.state.currentFilter), {
                isActive: false
              }),
              currentPage: currentPage
            });
          } else {
            yield this.refreshUsers({});
          }
          break;
      }
    });
  }
  users_OnSort(data, column) {
    return __awaiter$1(this, void 0, void 0, function* () {
      yield this.setState({
        currentSort: data
      });
      if (this.state.currentTab === 'ACTIVE_USERS') {
        yield this.refreshUsers({
          filter: Object.assign(Object.assign({}, this.state.currentFilter), {
            isActive: true
          }),
          sort: data
        });
      } else if (this.state.currentTab === 'INACTIVE_USERS') {
        yield this.refreshUsers({
          filter: Object.assign(Object.assign({}, this.state.currentFilter), {
            isActive: false
          }),
          sort: data
        });
      } else {
        yield this.refreshUsers({
          sort: data
        });
      }
    });
  }
  render() {
    var _a, _b, _c, _d, _e, _f;
    const {
      store,
      pageData
    } = this.props;
    const {
      isLoading
    } = this.state;
    const createUserPerm = hasPermission('createUser', store);
    const editUserPerm = hasPermission('editUser', store);
    const deleteUserPerm = hasPermission('deleteUser', store);
    const {
      Grid,
      Tabs,
      Form,
      Pagination,
      Dialog,
      Actions,
      Skeleton
    } = components$1.get();
    const {
      fieldMapping = {}
    } = pageData;
    // console.log('@@@@userstore', store);
    return jsx("div", {
      className: "sq-auth-users sq-v-screen sq-v-screen--fixed",
      children: jsxs("div", {
        className: "sq-v-screen__container",
        children: [jsx("div", {
          className: "container-fluid",
          children: jsxs("div", {
            className: "sq-v-screen__top-header mt-wide",
            children: [jsx("div", {
              children: jsxs("h2", {
                className: "mt-3",
                children: ['Users', !isLoading && ((_a = store.authentication) === null || _a === void 0 ? void 0 : _a.usersPagination) && `(${store.authentication.usersPagination.total})`]
              })
            }), jsx(Actions, {
              actions: [createUserPerm && {
                type: 'Button',
                iconName: 'add',
                size: 'medium',
                to: 'createNewUser',
                buttonText: 'Add New'
              }, {
                type: 'Button',
                variant: 'outlined',
                onClick: this.onEditColumnToggle,
                iconName: 'edit',
                buttonText: 'Edit Columns',
                color: this.state.showEditColumns ? 'secondary' : 'primary'
              }, {
                type: 'Button',
                iconName: 'filter-list',
                variant: 'outlined',
                size: 'medium',
                onClick: this.onFilterToggle,
                buttonText: 'Filters',
                color: this.state.showFilter ? 'secondary' : 'primary'
              }].filter(i => i)
            })]
          })
        }), jsxs("div", {
          className: `sq-v-screen__body-container ${this.state.showFilter ? 'sq-auth-users__container-filter' : ''}`,
          children: [jsx(Dialog, {
            open: this.state.showFilter,
            classes: {
              dialog: {
                root: 'sq-dialog--fixed-right'
              }
            },
            onClose: this.onFilterAction,
            onAction: this.onFilterAction,
            actions: [{
              actionType: 'applyfilter',
              buttonText: 'Apply'
            }, {
              actionType: 'resetfilter',
              buttonText: 'Reset',
              variant: 'outlined'
            }],
            title: 'Filters',
            children: jsx(Form, {
              onChange: this.onFilterChange,
              value: this.state.currentFilter,
              onAction: this.onFilterAction,
              name: 'userFilter',
              fields: [Object.assign(Object.assign({
                name: 'emailId',
                label: translate('Email ID')
              }, fieldMapping.emailId), {
                cmpType: 'Input'
              }), Object.assign(Object.assign({
                name: 'firstName',
                label: translate('First Name')
              }, fieldMapping.firstName), {
                cmpType: 'Input'
              }), Object.assign(Object.assign({
                name: 'lastName',
                label: translate('Last Name')
              }, fieldMapping.lastName), {
                cmpType: 'Input'
              }), Object.assign(Object.assign({
                name: 'roleCode',
                cmpType: 'Autocomplete',
                label: translate('Roles')
              }, fieldMapping.roleCode), {
                options: (_b = this.props.store.authentication.roles) === null || _b === void 0 ? void 0 : _b.ALL_ROLES,
                valueField: 'code',
                textField: 'name'
              })]
            })
          }), jsx(Tabs, {
            options: this.tabs,
            onChange: this.handleTabChange,
            value: this.state.currentTab
          }), jsx("div", {
            className: "mb-wide",
            children: ((_c = store.authentication.usersPagination) === null || _c === void 0 ? void 0 : _c.total) > 0 && jsx(Pagination, {
              className: "j-content-fl-end ml-3",
              enablePageSize: true,
              count: (_d = store.authentication) === null || _d === void 0 ? void 0 : _d.usersPagination.totalPages,
              disabled: isLoading,
              pageSizeOptions: GLOBAL_OPTIONS.noOfResultsDropdown.toArray(),
              value: (_e = store.authentication) === null || _e === void 0 ? void 0 : _e.usersPagination,
              onChange: data => {
                const pageNo = data.value.currentPage;
                const pageSize = data.value.pageSize;
                if (!isLoading) {
                  this.refreshUsers({
                    pageNo,
                    pageSize
                  });
                }
              }
            })
          }), jsx("div", {
            className: "sq-v-screen-grow",
            children: jsx(Grid, {
              onColFilterChange: this.onColFilterSelection,
              className: "sq-basic-grid sq-grid--fixed sq-basic-grid--rounded",
              sortColumn: this.state.currentSort.sortColumn,
              sortOrder: this.state.currentSort.sortOrder,
              enableSort: true,
              onSort: this.users_OnSort,
              loader: jsx(Skeleton, {
                styleName: `grid-tran`,
                rows: 8
              }),
              onAction: this.onGridAction,
              selectedColumns: this.state.selectedColumns,
              showColSelection: this.state.showEditColumns,
              columns: [Object.assign({
                name: 'icon',
                cmpType: 'Icon',
                className: 'col-icon',
                sort: false,
                component: {
                  textIcon: row => `${string.getTwoChars(row.firstName)}`,
                  iconClass: 'sq-icon--square sq-icon--text-icon-accent',
                  variant: row => {
                    var _a;
                    return row.active ? accent.getColorByChar((_a = row.firstName) === null || _a === void 0 ? void 0 : _a.substr(0, 1)) : 'default';
                  }
                }
              }, fieldMapping.icon), Object.assign({
                name: 'firstName',
                cmpType: 'TextFields',
                headerText: 'User',
                className: 'sq-auth-users-screen--col-default',
                component: {
                  fields: [{
                    name: 'firstName',
                    className: 'header'
                  }, {
                    name: 'lastName',
                    className: 'sub-header'
                  }]
                }
              }, fieldMapping.firstName), Object.assign({
                name: 'emailId',
                className: 'sq-auth-users-screen--col-title',
                headerText: 'Email ID',
                tooltip: {
                  title: row => row.emailId,
                  arrow: true
                }
              }, fieldMapping.emailId), Object.assign({
                name: 'roleName',
                className: 'sq-auth-users-screen--col-small',
                headerText: 'Role'
              }, fieldMapping.roleName), Object.assign({
                name: 'contactNumber',
                className: 'sq-auth-users-screen--col-title',
                headerText: 'Primary Phone Number',
                formatter: {
                  type: 'phoneNumber'
                }
              }, fieldMapping.contactNumber), Object.assign({
                name: 'loginDate',
                className: 'sq-auth-users-screen--col-default',
                headerText: 'Login Date',
                render: value => {
                  return formatters.dateFullTime(value);
                }
              }, fieldMapping.loginDate), {
                name: 'actions',
                headerText: 'Actions',
                className: 'col-actions',
                cmpType: 'MoreActions',
                sort: false,
                component: {
                  actions: [{
                    cmpType: 'LinkButton',
                    action: 'edit',
                    disabled: !editUserPerm,
                    iconName: 'edit',
                    buttonText: translate('Edit')
                  }, {
                    cmpType: 'LinkButton',
                    action: 'reset-password',
                    disabled: !editUserPerm,
                    iconName: 'RestartAlt',
                    buttonText: translate('Reset Password')
                  }, {
                    cmpType: 'LinkButton',
                    iconName: 'deactivate',
                    confirm: {
                      title: translate('Confirm'),
                      content: translate('Are you sure you want to deactivate this user?')
                    },
                    iconColor: 'error',
                    action: 'deactivate',
                    disabled: !editUserPerm,
                    buttonText: translate('Deactivate'),
                    render: row => {
                      return row.active;
                    }
                  }, {
                    cmpType: 'LinkButton',
                    iconName: 'Delete',
                    confirm: {
                      title: translate('Confirm'),
                      content: translate('Are you sure you want to delete this user?')
                    },
                    iconColor: 'error',
                    action: 'delete',
                    disabled: !deleteUserPerm,
                    buttonText: translate('Delete'),
                    render: row => {
                      return row.active && deleteUserPerm;
                    }
                  }, {
                    cmpType: 'LinkButton',
                    iconName: 'active',
                    iconColor: 'success',
                    action: 'activate',
                    disabled: !createUserPerm,
                    buttonText: translate('Re-Activate'),
                    render: row => {
                      return !row.active;
                    }
                  }]
                }
              }],
              showHeader: true,
              data: ((_f = store === null || store === void 0 ? void 0 : store.authentication) === null || _f === void 0 ? void 0 : _f.users) || []
            })
          })]
        })]
      })
    });
  }
}
const mapStateToProps = state => {
  return {
    store: Object.assign({}, state)
  };
};
const mapDispatchToProps = dispatch => {
  return {
    userActions: {
      loadUsers: (data, config) => dispatch(loadUsers$1(data, config)),
      deactivateUser: (data, config) => dispatch(deactivateUser(data, config)),
      deleteUser: (data, config) => dispatch(deleteUser(data, config)),
      reactivateUser: (data, config) => dispatch(reactivateUser(data, config)),
      resetPassword: (data, config) => dispatch(resetPassword(data, config)),
      loadRoles: (data, config) => dispatch(loadRoles(data, config))
    }
  };
};
Users.propTypes = {
  raiseAction: PropTypes.func,
  store: PropTypes.object,
  commonActions: PropTypes.object,
  appActions: PropTypes.object,
  location: PropTypes.object,
  history: PropTypes.object
};
var index = connect(mapStateToProps, mapDispatchToProps)(Users);

var containers = /*#__PURE__*/Object.freeze({
  __proto__: null,
  Application: index$5,
  ComponentDemo: ComponentDemo$2,
  Content: Content,
  ContentWithLeftNavigation: ComponentDemo$1,
  ContentWithSearch: ContentWithSearch,
  Documentation: ComponentDemo,
  DynamicContent: DynamicContentRoot,
  EmailTemplateList: index$4,
  GenericListing: GenericListing,
  LayoutContent: LayoutContent,
  MediaGallery: MediaGallery,
  MultiView: MultiView,
  PageBuilder: index$2,
  PageListing: PageListing,
  RoleDetails: index$3,
  RolesList: Roles,
  SearchWithListing: SearchWithListing,
  SiteMapBuilder: index$1,
  TocIndex: TableOfContent,
  UsersList: index
});

class Homepage extends BaseContainer {
  constructor(props) {
    super(props);
  }
  render() {
    var _a;
    const {
      children,
      pageData = {},
      data = {},
      store
    } = this.props;
    const {
      metaData = {},
      siteMap = {}
    } = data;
    const {
      GlobalNavigation,
      Footer
    } = components$1.get();
    let props = {};
    if (!((_a = store.authentication) === null || _a === void 0 ? void 0 : _a.currentUser)) {
      props = Object.assign({}, siteMap.siteMap.globalNavigation);
    } else {
      props = Object.assign({}, siteMap.siteMap.globalNavigationLoggedIn);
    }
    return jsxs("div", {
      className: `sq-content-page sq-content-page--header-footer-body  ${pageData.className || ''}`,
      children: [jsx("header", {
        children: jsx(GlobalNavigation, Object.assign({
          logo: siteMap.logo,
          items: metaData.navigation
        }, props))
      }), children, jsx("footer", {
        children: jsx(Footer, Object.assign({
          logo: siteMap.logo,
          items: metaData.navigation
        }, siteMap.siteMap.globalFooter))
      })]
    });
  }
}
Homepage.propTypes = {
  children: PropTypes.any,
  pageData: PropTypes.object,
  userStore: PropTypes.object,
  data: PropTypes.object
};

class AdminPlainPage extends BaseContainer {
  constructor(props) {
    super(props);
  }
  render() {
    const {
      children,
      pageData = {},
      data = {},
      store
    } = this.props;
    const {
      Icon
    } = components$1.get();
    return jsxs("div", {
      className: `sq-admin-plain ${pageData.className || ''}`,
      children: [jsx("div", {
        className: "sq-admin-plain__left",
        style: {
          backgroundImage: `url('/client/assets/imgs/building.png')`
        }
      }), jsx("div", {
        className: "sq-admin-plain__right",
        children: jsxs("div", {
          className: "sq-admin-plain__body",
          children: [jsx("header", {
            className: "sq-admin-plain__header",
            children: jsx("div", {
              className: "sq-admin-plain__logo",
              children: jsx(Icon, {
                name: "logo-wide",
                size: "auto"
              })
            })
          }), children]
        })
      })]
    });
  }
}
AdminPlainPage.propTypes = {
  children: PropTypes.any,
  pageData: PropTypes.object,
  userStore: PropTypes.object,
  data: PropTypes.object
};

setUrlMapping(config.urlMapping);
addValidator('hasAdminMatchingUrl', (value, {
  setName
}) => {
  var _a;
  const currentUrl = window.location.pathname;
  let isAllowed = false;
  (_a = config.urls[setName]) === null || _a === void 0 ? void 0 : _a.forEach(item => {
    if (new RegExp(item).test(currentUrl)) {
      isAllowed = true;
    }
  });
  return isAllowed;
});
class AdminContainer extends BaseContainer {
  render() {
    return jsx("div", {
      className: "container-fluid",
      children: jsx("h1", {
        children: "Not Implemented"
      })
    });
  }
}
AdminContainer.propTypes = {
  onAnalytics: PropTypes.func,
  userStore: PropTypes.object,
  commonStore: PropTypes.object
};

class AdminDashboard extends AdminContainer {
  constructor(props) {
    super(props);
    this.state = {
      openDrawer: false
    };
    this.onMenuItemClick = this.onMenuItemClick.bind(this);
    this.onUserMenuAction = this.onUserMenuAction.bind(this);
    this.toggleMenu = this.toggleMenu.bind(this);
  }
  onMenuItemClick(item) {
    const {
      onAnalytics
    } = this.props;
    if (item.type === 'popup') {
      this.props.commonActions.openScreen(item.path, item.text, {
        close: () => {}
      });
    }
    onAnalytics && onAnalytics({
      type: 'event',
      eventName: 'link_click',
      action: 'navigate',
      category: 'Navigation',
      viewType: item.type,
      label: item.text,
      section: 'Global'
    });
  }
  handleNavClick(item) {
    redirectTo(item.href);
  }
  onUserMenuAction(action) {
    switch (action.actionType) {
      case 'logout':
        redirectTo('login');
        this.props.raiseAction(clearUser());
        break;
      case 'redirect':
        redirectTo(action.to, action.params, action.options);
        break;
    }
  }
  toggleMenu() {
    this.setState({
      openDrawer: !this.state.openDrawer
    });
  }
  render() {
    var _a, _b, _c;
    const {
      children,
      appBarColor = 'default',
      location,
      userData
    } = this.props;
    const {
      Icon,
      LeftNavigation,
      UserMenu,
      BreadCrumb
    } = components$1.get();
    const {
      data = {},
      store
    } = this.props;
    const {
      pageData = {},
      metaData = {},
      siteMap = {}
    } = data;
    const {
      templateClasses = {}
    } = pageData;
    const listOfActions = [{
      iconName: 'person',
      iconSize: 'small',
      actionType: 'redirect',
      to: 'editProfile',
      className: 'sq-icon--bg-icon',
      buttonText: 'Profile'
    }, {
      iconName: 'login',
      actionType: 'redirect',
      to: 'profileChangePassword',
      iconSize: 'small',
      className: 'sq-icon--bg-icon',
      buttonText: 'Change password'
    }, 'divider',
    // {
    //   iconName: 'money',
    //   actionType: 'billing',
    //   iconSize: 'small',
    //   buttonText: 'Billing',
    // },
    {
      iconName: 'logout',
      actionType: 'logout',
      iconSize: 'small',
      buttonText: 'Logout'
    }].filter(i => !!i);
    const {
      currentUser
    } = this.props.store.authentication;
    const logo = siteMap.siteMap.logo;
    return jsx(Fragment, {
      children: jsxs("div", {
        className: `admin-app-template admin-app-template--dashboard ${templateClasses.root || ''}`,
        children: [jsxs("header", {
          className: `sq-template__header ${appBarColor}`,
          children: [jsxs("div", {
            className: "sq-template__header-text",
            children: [jsx("div", {
              className: "sq-template-menu-trigger",
              children: jsx(Icon, {
                name: "Menu",
                size: "medium",
                variant: "primary",
                onClick: this.toggleMenu
              })
            }), logo.topName && jsx(Icon, {
              name: logo.topName,
              className: "sq-template-menu-logo",
              size: "wide"
            }), logo.topImage && jsx("img", {
              className: "sq-template-menu-logo-img",
              src: logo.topImage,
              alt: logo.topImageAlt
            }), jsx("div", {
              className: "sq-template-menu-logo"
            })]
          }), store.authentication.currentUser && jsx(UserMenu, {
            user: store.authentication.currentUser,
            onAction: this.onUserMenuAction,
            listOfActions: listOfActions
          })]
        }), jsxs("div", {
          className: "admin-app-template__main",
          children: [jsx("div", {
            className: "admin-app-template__left-nav",
            children: jsx(LeftNavigation, {
              logo: logo,
              onCloseDrawer: this.toggleMenu,
              openDrawer: this.state.openDrawer,
              items: currentUser ? siteMap.siteMap.globalNavigationLoggedIn.navigation : siteMap.siteMap.globalNavigation.navigation,
              roles: [],
              permissions: (_a = store.authentication.currentUser) === null || _a === void 0 ? void 0 : _a.permissions,
              className: 'sq-left-navigation--compact',
              onClick: this.handleNavClick
            })
          }), jsx("div", {
            className: "admin-app-template__right-body",
            children: jsxs("div", {
              className: `sq-content-page sq-template sq-template--100-h sq-template--flex-page sq-template--dashobard sq-content-page--top-0`,
              children: [jsx("section", {
                className: "sq-template__sub-header",
                children: jsx(BreadCrumb, {
                  navigation: siteMap.siteMap.globalNavigationLoggedIn.navigation,
                  userData: userData,
                  appStore: store,
                  breadcrumb: store.content.breadcrumb,
                  roles: (_b = store.authentication.currentUser) === null || _b === void 0 ? void 0 : _b.roles,
                  permissions: (_c = store.authentication.currentUser) === null || _c === void 0 ? void 0 : _c.permissions,
                  currentPath: location.pathname
                })
              }), jsx("div", {
                className: `sq-template__content ${templateClasses.content || ''}`,
                children: jsx(ErrorBoundry$1, {
                  children: children
                })
              }), jsx("footer", {
                className: "sq-template__footer",
                children: jsx("div", {
                  className: "sq-template__footer-text"
                })
              })]
            })
          })]
        })]
      })
    });
  }
}
AdminDashboard.propTypes = {
  children: PropTypes.any,
  appStore: PropTypes.object,
  commonActions: PropTypes.object,
  forecastStore: PropTypes.object,
  location: PropTypes.object,
  onAnalytics: PropTypes.func,
  raiseAction: PropTypes.func,
  appBarColor: PropTypes.string,
  color: PropTypes.object,
  data: PropTypes.object
};

var templates = {
  AdminHomepage: Homepage,
  AdminPlainPage,
  AdminDashboard
};

components$1.set(components);
containers$1.set(Object.assign(Object.assign({}, containers), templates));
// export { storage, components };

const cordova = getWindow().cordova;
function resolveFile(filePath) {
  if (cordova) {
    return cordova.file.applicationDirectory + 'www/' + filePath.substr(2);
  } else {
    return filePath;
  }
}
function resolveImageUrl(filePath) {
  if (cordova) {
    return getWindow().API_SERVER + filePath;
  } else {
    return filePath;
  }
}
function getPathName() {
  if (cordova) {
    return getWindow().location.hash && getWindow().location.hash.substr(1);
  } else {
    return getWindow().location.pathname;
  }
}
function isiOS() {
  return !!getWindow().cordova && getWindow().cordova.platformId === 'ios';
}
function isAndroid() {
  return !!getWindow().cordova && getWindow().cordova.platformId === 'android';
}
function isApp() {
  return !!getWindow().cordova;
}

var cordova$1 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  cordova: cordova,
  resolveFile: resolveFile,
  resolveImageUrl: resolveImageUrl,
  getPathName: getPathName,
  isAndroid: isAndroid,
  isiOS: isiOS,
  isApp: isApp
});

export { components, containers, cordova$1 as cordova, index$n as domain, globals, index$6 as hooks, plugins, index$8 as reducers, storage, templates, utils };
